@@ -1,3 +1,5 @@
+#include <stdlib.h>
+#include <string.h>
 #include <gtk/gtk.h>
 #include "guiutils.h"
 #include "animicon.h"
@@ -15,6 +17,32 @@ typedef enum {
 
 
 typedef struct _AnimIcon		AnimIcon;
+#define ANIM_ICON(p)			((AnimIcon *)(p))
+#define ANIM_ICON_DATA_KEY		"ANIM_ICON"
+
+
+static void AnimIconDataDestroyCB(gpointer data);
+static gint AnimIconEventCB(
+	GtkWidget *widget, GdkEvent *event, gpointer data
+);
+static void AnimIconRealizeCB(GtkWidget *widget, gpointer data);
+static gint AnimIconDrawTOCB(gpointer data);
+
+static void AnimIconDraw(AnimIcon *a);
+
+GtkWidget *AnimIconNew(const gint width, const gint height);
+void AnimIconSetFromXPMDataList(
+	GtkWidget *w, GList *data_list, const gulong interval
+);
+void AnimIconSetColorBG(GtkWidget *w, const GdkColor *c);
+void AnimIconSetRepeating(GtkWidget *w, const gboolean repeating);
+
+gint AnimIconFrames(GtkWidget *w);
+gint AnimIconFrame(GtkWidget *w);
+gboolean AnimIconIsPlaying(GtkWidget *w);
+void AnimIconSeek(GtkWidget *w, const gint frame);
+void AnimIconPause(GtkWidget *w);
+void AnimIconPlay(GtkWidget *w);
 
 
 /*
@@ -24,9 +52,16 @@ struct _AnimIcon {
 
 	AnimIconFlags	flags;
 
-	GdkPixmap	**pixmaps_list;
-	GdkBitmap	**masks_list;
+	gint		freeze_count;
 
+	GdkColormap	*colormap;
+	GdkGC		*gc;
+
+	GList		*pixmaps_list;	/* GList of GdkPixmap * */
+	GList		*masks_list;	/* GList of GdkBitmap * */
+	GdkColor	*color_bg;
+
+	GtkWidget	*drawing_area;
 	GdkPixmap	*pixmap;	/* Back buffer */
 
 	gint		width,
@@ -39,42 +74,18 @@ struct _AnimIcon {
 
 	guint		toid;
 };
-#define ANIM_ICON(p)		((AnimIcon *)(p))
-#define ANIM_ICON_DATA_KEY	"anim_icon"
 
 
-static void AnimIconDataDestroyCB(gpointer data);
-static gint AnimIconEventCB(
-	GtkWidget *widget, GdkEvent *event, gpointer data
-);
-static gint AnimIconDrawTOCB(gpointer data);
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-static void AnimIconDraw(GtkWidget *w, AnimIcon *a);
-
-GtkWidget *AnimIconNew(gint width, gint height);
-void AnimIconSetFromXPMDataList(
-	GtkWidget *w, GList *data_list, gulong interval
-);
-void AnimIconSetRepeating(GtkWidget *w, gboolean repeating);
-
-gint AnimIconFrames(GtkWidget *w);
-gint AnimIconFrame(GtkWidget *w);
-gboolean AnimIconIsPlaying(GtkWidget *w);
-void AnimIconSeek(GtkWidget *w, gint frame);
-void AnimIconPause(GtkWidget *w);
-void AnimIconPlay(GtkWidget *w);
-
-
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -82,23 +93,53 @@ void AnimIconPlay(GtkWidget *w);
  */
 static void AnimIconDataDestroyCB(gpointer data)
 {
-	gint i;
 	AnimIcon *a = ANIM_ICON(data);
 	if(a == NULL)
 	    return;
 
-	GTK_TIMEOUT_REMOVE(a->toid);
-	a->toid = 0;
+	a->freeze_count++;
 
-	for(i = 0; i < a->nframes; i++)
+	if(a->toid != 0)
+	    gtk_timeout_remove(a->toid);
+
+	if(a->pixmaps_list != NULL)
 	{
-	    GDK_PIXMAP_UNREF(a->pixmaps_list[i]);
-	    GDK_BITMAP_UNREF(a->masks_list[i]);
+	    GList *glist;
+	    for(glist = a->pixmaps_list;
+	        glist != NULL;
+	        glist = g_list_next(glist)
+	    )
+	    {
+		if(glist->data != NULL)
+		    gdk_pixmap_unref((GdkPixmap *)glist->data);
+	    }
+	    g_list_free(a->pixmaps_list);
+	}
+	if(a->masks_list != NULL)
+	{
+	    GList *glist;
+	    for(glist = a->masks_list;
+	        glist != NULL;
+	        glist = g_list_next(glist)
+	    )
+	    {
+		if(glist->data != NULL)
+		    gdk_bitmap_unref((GdkBitmap *)glist->data);
+	    }
+	    g_list_free(a->masks_list);
 	}
-	g_free(a->pixmaps_list);
-	g_free(a->masks_list);
 
+	if(a->color_bg != NULL)
+	{
+	    GDK_COLORMAP_FREE_COLOR(a->colormap, a->color_bg);
+	    g_free(a->color_bg);
+	}
+
+	GDK_GC_UNREF(a->gc);
 	GDK_PIXMAP_UNREF(a->pixmap);
+	GDK_COLORMAP_UNREF(a->colormap);
+
+	a->freeze_count--;
 
 	g_free(a);
 }
@@ -117,37 +158,85 @@ static gint AnimIconEventCB(
 	if((widget == NULL) || (event == NULL) || (a == NULL))
 	    return(status);
 
+	if(a->freeze_count > 0)
+	    return(status);
+
 	switch((gint)event->type)
 	{
 	  case GDK_CONFIGURE:
 	    configure = (GdkEventConfigure *)event;
 	    GDK_PIXMAP_UNREF(a->pixmap);
-	    a->pixmap = GDK_PIXMAP_NEW(
-		configure->width, configure->height
+	    a->pixmap = gdk_pixmap_new(
+		configure->window,
+		configure->width, configure->height, -1
 	    );
 	    break;
 
 	  case GDK_EXPOSE:
 	    expose = (GdkEventExpose *)event;
-	    AnimIconDraw(widget, a);
+	    AnimIconDraw(a);
 	    break;
 	}
 
 	return(status);
 }
 
+/*
+ *	GtkDrawingArea "realize" signal callback.
+ */
+static void AnimIconRealizeCB(GtkWidget *widget, gpointer data)
+{
+	gboolean need_alloc_colors = FALSE;
+	GdkColormap *colormap;
+	GdkGC *gc;
+	GdkWindow *window;
+	AnimIcon *a = ANIM_ICON(data);
+	if((widget == NULL) || (a == NULL))
+	    return;
+
+	if(a->freeze_count > 0)
+	    return;
+
+	window = widget->window;
+	if(window == NULL)
+	    return;
+
+	colormap = a->colormap;
+	if(colormap == NULL)
+	{
+	    a->colormap = colormap = gdk_window_get_colormap(window);
+	    if(colormap != NULL)
+	    {
+		gdk_colormap_ref(colormap);
+		need_alloc_colors = TRUE;
+	    }
+	}
+
+	gc = a->gc;
+	if(gc == NULL)
+	    a->gc = gc = gdk_gc_new(window);
+
+	if(need_alloc_colors)
+	    GDK_COLORMAP_ALLOC_COLOR(colormap, a->color_bg);
+}
+
 /*
  *	Animated Icon draw timeout callback.
  */
 static gint AnimIconDrawTOCB(gpointer data)
 {
-	GtkWidget *w = (GtkWidget *)data;
-	AnimIcon *a = ANIM_ICON((w != NULL) ?
-	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
-	);
+	AnimIcon *a = ANIM_ICON(data);
 	if(a == NULL)
 	    return(FALSE);
 
+	gdk_threads_enter();
+
+	if(a->freeze_count > 0)
+	{
+	    gdk_threads_leave();
+	    return(TRUE);
+	}
+
 	/* Go to next frame */
 	a->frame++;
 	if(a->frame >= a->nframes)
@@ -160,12 +249,16 @@ static gint AnimIconDrawTOCB(gpointer data)
 	    {
 		a->frame = MAX(a->nframes - 1, 0);
 		a->toid = 0;
+		gdk_threads_leave();
 		return(FALSE);
 	    }
 	}
 
 	/* Queue draw */
-	gtk_widget_queue_draw(w);
+	if(a->drawing_area != NULL)
+	    gtk_widget_queue_draw(a->drawing_area);
+
+	gdk_threads_leave();
 
 	return(TRUE);
 }
@@ -174,25 +267,34 @@ static gint AnimIconDrawTOCB(gpointer data)
 /*
  *	Draws the Animated Icon.
  */
-static void AnimIconDraw(GtkWidget *w, AnimIcon *a)
+static void AnimIconDraw(AnimIcon *a)
 {
 	gint width, height;
 	GdkWindow *window;
 	GdkDrawable *drawable;
+	GdkGC *gc;
 	GtkStateType state;
 	GtkStyle *style;
+	GtkWidget *w;
 
+	if(a == NULL)
+	    return;
 
-	if((w == NULL) || (a == NULL))
+	if(a->freeze_count > 0)
+	    return;
+
+	w = a->drawing_area;
+	if(w == NULL)
 	    return;
 
 	if(!GTK_WIDGET_VISIBLE(w))
 	    return;
 
 	window = w->window;
+	gc = a->gc;
 	state = GTK_WIDGET_STATE(w);
 	style = gtk_widget_get_style(w);
-	if((window == NULL) || (style == NULL))
+	if((window == NULL) || (gc == NULL) || (style == NULL))
 	    return;
 
 	gdk_window_get_size(window, &width, &height);
@@ -204,24 +306,35 @@ static void AnimIconDraw(GtkWidget *w, AnimIcon *a)
 	    drawable = (GdkDrawable *)window;
 
 	/* Draw background */
-	gtk_style_apply_default_background(
-	    style, drawable, FALSE, state,
-	    NULL,
-	    0, 0, width, height
-	);
-
-	/* Draw animated icon */
-	if(a->nframes > 0)
+	if(a->color_bg != NULL)
 	{
-	    const gint	frame = a->frame,
-			nframes = a->nframes;
-	    GdkGC *gc = style->fg_gc[state];
+	    gdk_gc_set_foreground(gc, a->color_bg);
+	    gdk_draw_rectangle(
+		drawable, gc, TRUE,
+		0, 0, width, height
+	    );
+	}
+	else
+	{
+	    gtk_style_apply_default_background(
+		style, drawable, FALSE, state,
+		NULL,
+		0, 0, width, height
+	    );
+	}
 
-	    if((frame >= 0) && (frame < nframes) && (gc != NULL))
+	/* Draw the animated icon */
+	if(a->pixmaps_list != NULL)
+	{
+	    const guint frame = (guint)a->frame;
+	    GdkPixmap *pixmap = (GdkPixmap *)g_list_nth_data(
+		a->pixmaps_list, frame
+	    );
+	    if(pixmap != NULL)
 	    {
-		GdkPixmap *pixmap = a->pixmaps_list[frame];
-		GdkBitmap *mask = a->masks_list[frame];
-
+		GdkBitmap *mask = (GdkBitmap *)g_list_nth_data(
+		    a->masks_list, frame
+		);
 		gdk_gc_set_clip_mask(gc, mask);
 		gdk_gc_set_clip_origin(gc, 0, 0);
 		gdk_draw_pixmap(
@@ -242,7 +355,7 @@ static void AnimIconDraw(GtkWidget *w, AnimIcon *a)
 	/* Send drawable to window if drawable is not the window */
 	if(drawable != window)
 	    gdk_draw_pixmap(
-		window, style->fg_gc[state], drawable,
+		window, gc, drawable,
 		0, 0, 0, 0, width, height
 	    );
 }
@@ -251,16 +364,37 @@ static void AnimIconDraw(GtkWidget *w, AnimIcon *a)
 /*
  *	Creates a new Animated Icon.
  */
-GtkWidget *AnimIconNew(gint width, gint height)
+GtkWidget *AnimIconNew(const gint width, const gint height)
 {
-	GtkWidget *w = gtk_drawing_area_new();
-	AnimIcon *a = ANIM_ICON(g_malloc0(sizeof(AnimIcon)));
+	GtkWidget *w;
+	AnimIcon *a;
 
+	a = ANIM_ICON(g_malloc0(sizeof(AnimIcon)));
+	if(a == NULL)
+	    return(NULL);
+
+	a->flags = 0;
+	a->freeze_count = 0;
+	a->colormap = NULL;
+	a->gc = NULL;
+	a->pixmaps_list = NULL;
+	a->masks_list = NULL;
+	a->color_bg = NULL;
+	a->drawing_area = NULL;
+	a->pixmap = NULL;
+	a->nframes = 0;
+	a->frame = 0;
+	a->width = width;
+	a->height = height;
+	a->toid = 0;
+
+	a->freeze_count++;
+
+	a->drawing_area = w = gtk_drawing_area_new();
 	gtk_object_set_data_full(
 	    GTK_OBJECT(w), ANIM_ICON_DATA_KEY,
 	    a, AnimIconDataDestroyCB
 	);
-
 	gtk_widget_set_usize(w, width, height);
 	gtk_widget_add_events(
 	    w,
@@ -274,16 +408,12 @@ GtkWidget *AnimIconNew(gint width, gint height)
 	    GTK_OBJECT(w), "expose_event",
 	    GTK_SIGNAL_FUNC(AnimIconEventCB), a 
 	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "realize",
+	    GTK_SIGNAL_FUNC(AnimIconRealizeCB), a
+	);
 
-	a->flags = 0;
-	a->pixmaps_list = NULL;
-	a->masks_list = NULL;
-	a->pixmap = NULL;
-	a->frame = 0;
-	a->nframes = 0;
-	a->width = width;
-	a->height = height;
-	a->toid = 0;
+	a->freeze_count--;
 
 	return(w);
 }
@@ -295,92 +425,127 @@ GtkWidget *AnimIconNew(gint width, gint height)
  *	and unloaded.
  */
 void AnimIconSetFromXPMDataList(
-	GtkWidget *w, GList *data_list, gulong interval
+	GtkWidget *w, GList *data_list, const gulong interval
 )
 {
-	gint i;
 	AnimIcon *a = ANIM_ICON((w != NULL) ?
 	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
 	);
 	if(a == NULL)
 	    return;
 
-	/* Remove current draw timeout if any */
-	if(a->toid > 0)
+	a->freeze_count++;
+
+	/* Remove the current draw timeout if any */
+	if(a->toid != 0)
 	{
 	    gtk_timeout_remove(a->toid);
 	    a->toid = 0;
 	}
 
-	/* Delete existing pixmaps and masks lists */
-	for(i = 0; i < a->nframes; i++)
+	/* Delete the pixmaps and masks lists */
+	if(a->pixmaps_list != NULL)
 	{
-	    GDK_PIXMAP_UNREF(a->pixmaps_list[i]);
-	    GDK_BITMAP_UNREF(a->masks_list[i]);
+	    GList *glist;
+	    for(glist = a->pixmaps_list;
+	        glist != NULL;
+	        glist = g_list_next(glist)
+	    )
+	    {
+		if(glist->data != NULL)
+		    gdk_pixmap_unref((GdkPixmap *)glist->data);
+	    }
+	    g_list_free(a->pixmaps_list);
+	    a->pixmaps_list = NULL;
+	}
+	if(a->masks_list != NULL)
+	{
+	    GList *glist;
+	    for(glist = a->masks_list;
+	        glist != NULL;
+	        glist = g_list_next(glist)
+	    )
+	    {
+		if(glist->data != NULL)
+		    gdk_bitmap_unref((GdkBitmap *)glist->data);
+	    }
+	    g_list_free(a->masks_list);
+	    a->masks_list = NULL;
 	}
-	g_free(a->pixmaps_list);
-	a->pixmaps_list = NULL;
-	g_free(a->masks_list);
-	a->masks_list = NULL;
 	a->nframes = 0;
 	a->frame = 0;
 	a->interval = (gulong)AnimIconIntervalDefault;
 
 
-	/* Load XPM data */
+	/* Load the XPM data */
 	if(data_list != NULL)
 	{
 	    guint8 **data;
 	    GList *glist;
-
-	    /* Get number of frames */
-	    a->nframes = g_list_length(data_list);
-
-	    /* Allocate pixmaps and masks lists */
-	    a->pixmaps_list = (GdkPixmap **)g_malloc0(
-		a->nframes * sizeof(GdkPixmap *)
-	    );
-	    a->masks_list = (GdkBitmap **)g_malloc0(
-		a->nframes * sizeof(GdkBitmap *)
-	    );
-	    if((a->pixmaps_list == NULL) || (a->masks_list == NULL))
-	    {
-		g_free(a->pixmaps_list);
-		a->pixmaps_list = NULL;
-		g_free(a->masks_list);
-		a->masks_list = NULL;
-		a->nframes = 0;
-		return;
-	    }
+	    GdkBitmap *mask;
+	    GdkPixmap *pixmap;
 
 	    /* Load each pixmap and mask */
-	    for(glist = data_list, i = 0;
+	    for(glist = data_list;
 		glist != NULL;
-		glist = g_list_next(glist), i++
+		glist = g_list_next(glist)
 	    )
 	    {
 		data = (guint8 **)glist->data;
 		if(data == NULL)
 		    continue;
 
-		a->pixmaps_list[i] = GDK_PIXMAP_NEW_FROM_XPM_DATA(
-		    &a->masks_list[i], (guint8 **)data
+		pixmap = GDK_PIXMAP_NEW_FROM_XPM_DATA(
+		    &mask, (guint8 **)data
 		);
+		if(pixmap == NULL)
+		    continue;
+
+		a->pixmaps_list = g_list_append(a->pixmaps_list, pixmap);
+		a->masks_list = g_list_append(a->masks_list, mask);
 	    }
 
-	    /* Set interval */
+	    /* Set the total number of frames */
+	    a->nframes = g_list_length(a->pixmaps_list);
+
+	    /* Set the interval */
 	    a->interval = interval;
 	}
+
+	a->freeze_count--;
+}
+
+/*
+ *	Sets the Animated Icon's background color.
+ */
+void AnimIconSetColorBG(GtkWidget *w, const GdkColor *c)
+{
+	AnimIcon *a = ANIM_ICON((w != NULL) ?
+	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
+	);
+	if(a == NULL)
+	    return;
+
+	GDK_COLORMAP_FREE_COLOR(a->colormap, a->color_bg);
+	g_free(a->color_bg);
+	a->color_bg = NULL;
+
+	if(c == NULL)
+	    return;
+
+	a->color_bg = (GdkColor *)g_malloc(sizeof(GdkColor));
+	memcpy(a->color_bg, c, sizeof(GdkColor));
+	GDK_COLORMAP_ALLOC_COLOR(a->colormap, a->color_bg);
 }
 
 /*
  *	Sets the Animated Icon to be repeating or not.
  */
-void AnimIconSetRepeating(GtkWidget *w, gboolean repeating)
+void AnimIconSetRepeating(GtkWidget *w, const gboolean repeating)
 {
-        AnimIcon *a = ANIM_ICON((w != NULL) ?
-            gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
-        );
+	AnimIcon *a = ANIM_ICON((w != NULL) ?
+	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
+	);
 	if(a == NULL)
 	    return;
 
@@ -421,34 +586,35 @@ gboolean AnimIconIsPlaying(GtkWidget *w)
 	AnimIcon *a = ANIM_ICON((w != NULL) ?
 	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
 	);
-	return((a != NULL) ? (a->toid > 0) : FALSE);
+	return((a != NULL) ? (a->toid != 0) : FALSE);
 }
 
 /*
  *	Seek to the Animated Icon's specified frame.
  */
-void AnimIconSeek(GtkWidget *w, gint frame)
+void AnimIconSeek(GtkWidget *w, const gint frame)
 {
+	gint _frame = frame;
 	AnimIcon *a = ANIM_ICON((w != NULL) ?
 	    gtk_object_get_data(GTK_OBJECT(w), ANIM_ICON_DATA_KEY) : NULL
 	);
 	if(a == NULL)
 	    return;
 
-	if((frame >= 0) && (frame < a->nframes))
+	if((_frame >= 0) && (_frame < a->nframes))
 	{
-	    if(a->frame != frame)
+	    if(a->frame != _frame)
 	    {
-		a->frame = frame;
+		a->frame = _frame;
 		gtk_widget_queue_draw(w);
 	    }
 	}
 	else
 	{
-	    frame = 0;
-	    if(a->frame != frame)
+	    _frame = 0;
+	    if(a->frame != _frame)
 	    {
-		a->frame = frame;
+		a->frame = _frame;
 		gtk_widget_queue_draw(w);
 	    }
 	}
@@ -465,11 +631,11 @@ void AnimIconPause(GtkWidget *w)
 	if(a == NULL)
 	    return;
 
-	if(a->toid > 0)
-	{
-	    gtk_timeout_remove(a->toid);
-	    a->toid = 0;
-	}
+	if(a->toid == 0)
+	    return;
+
+	gtk_timeout_remove(a->toid);
+	a->toid = 0;
 }
 
 
@@ -485,7 +651,7 @@ void AnimIconPlay(GtkWidget *w)
 	    return;
 
 	/* Already playing? */
-	if(a->toid > 0)
+	if(a->toid != 0)
 	    return;
 
 	gtk_widget_queue_draw(w);
@@ -494,7 +660,6 @@ void AnimIconPlay(GtkWidget *w)
 	if(a->interval > 0l)
 	    a->toid = gtk_timeout_add(
 		a->interval,
-		AnimIconDrawTOCB, w
+		AnimIconDrawTOCB, a
 	    );
 }
-
