@@ -17,13 +17,16 @@ struct _edv_archive_info_dlg_struct {
 
 	GtkWidget	*toplevel;
 	GtkAccelGroup	*accelgrp;
-	gboolean	processing;
-	gint		busy_count,
-			freeze_count;
+	gboolean	has_changes;
+	gint		freeze_count,
+			busy_count;
 	edv_core_struct *core;
 
-	GdkCursor	*text_cur,
-			*busy_cur;
+	gchar		*arch_path;	/* Path to the archive that we are
+					 * displaying the statistics and
+					 * comments of */
+
+	GdkCursor	*busy_cur;
 
 	GtkWidget	*main_vbox,
 			*icon_pm,
@@ -33,28 +36,26 @@ struct _edv_archive_info_dlg_struct {
 			*comment_set_btn,
 			*comment_paste_btn,
 			*comment_insert_file_btn,
-			*ok_btn;
+			*ok_btn,
+			*cancel_btn;
 
 	pie_chart_struct	*piechart;
 
-	gchar		*arch_obj;
-	gboolean	has_changes;
-
 };
 
 
 extern edv_archive_info_dlg_struct *EDVArchiveInfoNew(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *ref_toplevel
 );
-extern void EDVArchiveInfoResetHasChanges(
-        edv_archive_info_dlg_struct *d, gboolean has_changes
+extern void EDVArchiveInfoSetHasChanges(
+        edv_archive_info_dlg_struct *d, const gboolean has_changes
 );
 extern void EDVArchiveInfoUpdateMenus(edv_archive_info_dlg_struct *d);
 extern void EDVArchiveInfoSetBusy(
-	edv_archive_info_dlg_struct *d, gboolean is_busy
+	edv_archive_info_dlg_struct *d, const gboolean is_busy
 );
 extern gboolean EDVArchiveInfoIsMapped(edv_archive_info_dlg_struct *d);
 extern void EDVArchiveInfoMap(edv_archive_info_dlg_struct *d);
