@@ -33,6 +33,7 @@
 typedef struct {
 	GtkWidget	*toplevel;
 	GtkAccelGroup	*accelgrp;
+	gboolean	got_response;
 	gint		freeze_count;
 	edv_core_struct	*core;
 	GtkWidget	*main_vbox,
@@ -40,7 +41,6 @@ typedef struct {
 			*ok_btn,
 			*cancel_btn;
 	gchar           *path;		/* Path to archive */
-	gboolean	got_response;
 } Dlg;
 #define DLG(p)			((Dlg *)(p))
 
@@ -49,7 +49,8 @@ typedef struct {
  */
 typedef struct {
 	Dlg		d;
-	GtkWidget	*recursive_check,
+	GtkWidget	*password_entry,
+			*recursive_check,
 			*compression_scale,
 			*compression_label,
 			*dereference_links_check;
@@ -63,7 +64,8 @@ typedef struct {
  */
 typedef struct {
 	Dlg		d;
-	GtkWidget	*preserve_directories_check,
+	GtkWidget	*password_entry,
+			*preserve_directories_check,
 			*preserve_timestamps_check;
 } ExtractDlg;
 #define EXTRACT_DLG(p)		((ExtractDlg *)(p))
@@ -100,7 +102,8 @@ static void EDVDlgDelete(Dlg *d);
 gboolean EDVArchAddOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *recursive,
 	gint *compression,		/* 0 to 100 */
 	gboolean *dereference_links
@@ -110,7 +113,8 @@ gboolean EDVArchAddOptsGetResponse(
 gboolean EDVArchExtractOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *preserve_directories,
 	gboolean *preserve_timestamps
 );
@@ -276,21 +280,33 @@ static Dlg *EDVDlgNew(
 	const gint	border_major = 5;
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
-	GtkWidget *w, *parent, *parent2;
-	Dlg *d = (size > 0) ? DLG(g_malloc0(size)) : NULL;
+	GtkWidget	*w,
+			*parent, *parent2,
+			*toplevel;
+	Dlg *d;
+
+	if(size <= 0)
+	    return(NULL);
+
+	d = DLG(g_malloc0(size));
 	if(d == NULL)
 	    return(NULL);
 
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
+	d->got_response = FALSE;
 	d->freeze_count = 0;
 	d->core = core;
 	d->path = STRDUP(path);
-	d->got_response = FALSE;
 
-	/* Toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	d->freeze_count++;
+
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, width, height);
-	gtk_window_set_title(GTK_WINDOW(w), title);
+	if(title != NULL)
+	    gtk_window_set_title(GTK_WINDOW(w), title);
 	gtk_window_set_wmclass(
 	    GTK_WINDOW(w), "dialog", PROG_NAME
 	);
@@ -300,12 +316,11 @@ static Dlg *EDVDlgNew(
 	{
 	    gdk_window_set_decorations(
 		window,
-		GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU |
-		GDK_DECOR_MINIMIZE
+		GDK_DECOR_BORDER | GDK_DECOR_TITLE
 	    );
 	    gdk_window_set_functions(
 		window,
-		GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE
+		GDK_FUNC_MOVE | GDK_FUNC_CLOSE
 	    );
 	    GUISetWMIcon(window, icon_data);
 	}
@@ -329,13 +344,13 @@ static Dlg *EDVDlgNew(
 	}
 	parent = w;
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	d->main_vbox = w = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	gtk_widget_show(w);
 	parent = w;
 
-	/* Client vbox */
+	/* Client GtkVBox */
 	d->client_vbox = w = gtk_vbox_new(FALSE, border_major);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
@@ -345,7 +360,7 @@ static Dlg *EDVDlgNew(
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 
-	/* Buttons hbox */
+	/* Buttons GtkHBox */
 	w = gtk_hbox_new(TRUE, 0);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
@@ -393,6 +408,8 @@ static Dlg *EDVDlgNew(
 	gtk_widget_grab_default(w);
 	gtk_widget_grab_focus(w);
 
+	d->freeze_count--;
+
 	return(d);
 }
 
@@ -407,13 +424,15 @@ static void EDVDlgDelete(Dlg *d)
 	gtk_widget_hide(d->toplevel);
 	gtk_window_set_modal(GTK_WINDOW(d->toplevel), FALSE);
 	gtk_window_set_transient_for(GTK_WINDOW(d->toplevel), NULL);
-	GTK_WIDGET_DESTROY(d->ok_btn);
-	GTK_WIDGET_DESTROY(d->cancel_btn);
-	GTK_WIDGET_DESTROY(d->client_vbox);
-	GTK_WIDGET_DESTROY(d->main_vbox);
-	GTK_WIDGET_DESTROY(d->toplevel);
-	GTK_ACCEL_GROUP_UNREF(d->accelgrp);
+
+	d->freeze_count++;
+
+	gtk_widget_destroy(d->toplevel);
+	gtk_accel_group_unref(d->accelgrp);
 	g_free(d->path);
+
+	d->freeze_count--;
+
 	g_free(d);
 }
 
@@ -427,7 +446,8 @@ static void EDVDlgDelete(Dlg *d)
 gboolean EDVArchAddOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *recursive,
 	gint *compression,		/* 0 to 100 */
 	gboolean *dereference_links
@@ -521,7 +541,7 @@ gboolean EDVArchAddOptsGetResponse(
 #endif
 	    , (guint8 **)icon_cancel_20x20_xpm,
 	    ref_toplevel,
-	    core, arch_obj
+	    core, arch_path
 	);
 	if(d == NULL)
 	    return(got_response);
@@ -560,7 +580,7 @@ gboolean EDVArchAddOptsGetResponse(
 	    GUIButtonLabelUnderline(w, GDK_c);
 
 	    /* Archive object */
-	    if(!STRISEMPTY(arch_obj))
+	    if(!STRISEMPTY(arch_path))
 	    {
 		w = gtk_frame_new(NULL);
 		gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
@@ -615,7 +635,7 @@ gboolean EDVArchAddOptsGetResponse(
 "-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-iso8859-1"
 #endif
 		);
-		w = gtk_label_new(g_basename(arch_obj));
+		w = gtk_label_new(g_basename(arch_path));
 		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 		gtk_widget_modify_style(w, rcstyle);
@@ -646,18 +666,70 @@ archive formats and therefore do not apply."
 		gtk_widget_show(w);
 	    }
 
-	    /* Vbox for option widgets */
+	    /* GtkVBox for the option widgets */
 	    w = gtk_vbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent2 = w;
 
-	    /* Recursive check */
+
+	    /* Password GtkHBox */
 	    w = gtk_hbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent3 = w;
-	    /* Check button */
+
+	    /* Password GtkLabel */
+	    w = gtk_label_new(
+#if defined(PROG_LANGUAGE_SPANISH)
+"Contrasea:"
+#elif defined(PROG_LANGUAGE_FRENCH)
+"Mot De Passe:"
+#elif defined(PROG_LANGUAGE_GERMAN)
+"Kennwort:"
+#elif defined(PROG_LANGUAGE_ITALIAN)
+"Parola D'Ordine:"
+#elif defined(PROG_LANGUAGE_DUTCH)
+"Wachtwoord:"
+#elif defined(PROG_LANGUAGE_PORTUGUESE)
+"Senha:"
+#elif defined(PROG_LANGUAGE_NORWEGIAN)
+"Passord:"
+#elif defined(PROG_LANGUAGE_POLISH) 
+"Haso:"
+#else                 
+"Password:"
+#endif
+	    );
+	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
+	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+	    gtk_widget_show(w);
+
+	    /* Password GtkEntry */
+	    d->password_entry = w = gtk_entry_new();
+	    gtk_entry_set_visibility(GTK_ENTRY(w), FALSE);
+	    gtk_entry_set_editable(GTK_ENTRY(w), TRUE);
+	    gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
+	    if(password != NULL)
+		gtk_entry_set_text(
+		    GTK_ENTRY(w),
+		    (*password != NULL) ? *password : ""
+		);
+	    GUIEditableEndowPopupMenu(w, 0);
+	    GUISetWidgetTip(
+		w,
+"Enter the password to encrypt the archive with or leave this\
+ blank for no encryption"
+	    );
+	    gtk_widget_show(w);
+
+
+	    /* Recursive GtkHBox */
+	    w = gtk_hbox_new(FALSE, border_minor);
+	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	    gtk_widget_show(w);
+	    parent3 = w;
+	    /* Recursive GtkCheckButton */
 	    d->recursive_check = w =
 		gtk_check_button_new_with_label(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -683,7 +755,8 @@ archive formats and therefore do not apply."
 	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	    if((recursive != NULL) ? *recursive : FALSE)
 		GTK_TOGGLE_BUTTON(w)->active = TRUE;
-	    GUISetWidgetTip(w,
+	    GUISetWidgetTip(
+		w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto incluir el agregar de objetos encontr en todas guas\
  de nio del sendero(s) escogido"
@@ -715,12 +788,12 @@ archive formats and therefore do not apply."
 	    );
 	    gtk_widget_show(w);
 
-	    /* Compression scale */
+	    /* Compression GtkHBox */
 	    w = gtk_hbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent3 = w;
-	    /* Label */
+	    /* Compression GtkLabel */
 	    w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Compresin"
@@ -745,7 +818,7 @@ archive formats and therefore do not apply."
 	    );
 	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
-	    /* Adjustment and scale */
+	    /* Compression GtkScale */
 	    d->compression_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
 		(compression != NULL) ? (gfloat)*compression : 0.0f,
 		0.0f, 100.0f,
@@ -759,7 +832,8 @@ archive formats and therefore do not apply."
 		GTK_OBJECT(adj), "value_changed",
 		GTK_SIGNAL_FUNC(EDVArchOptsDlgAddCompressionValueChangedCB), d
 	    );
-	    GUISetWidgetTip(w,
+	    GUISetWidgetTip(
+		w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Ponga la cantidad de la compresin; 0% para ninguna compresin,\
  50% para la compresin uniforme, y 100% para la compresin mxima"
@@ -792,7 +866,7 @@ archive formats and therefore do not apply."
 	    );
 	    gtk_widget_show(w);
 
-	    /* Compression label */
+	    /* Compression GtkLabel */
 	    d->compression_label = w = gtk_label_new("");
 	    gtk_widget_set_usize(w, 30, -1);
 	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
@@ -804,6 +878,8 @@ archive formats and therefore do not apply."
 		90, -1
 	    );
 	    pulist = PUListBoxGetPUList(pulistbox);
+	    if(core->run_flags & EDV_RUN_SAFE_MODE)
+		PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
 	    PUListBoxSetChangedCB(
 		pulistbox,
 		EDVArchOptsDlgAddCompressionPUListChangedCB, d
@@ -926,8 +1002,8 @@ archive formats and therefore do not apply."
 "Zaznacz to pole by do archwium zostay dodane obiekty wskazywane\
  przez odnoniki zamiast ich samych"
 #else
-"Check this to add the destination of link objects instead of the\
- link objects themselves"
+"Check this to add the destination of links instead of the\
+ links themselves"
 #endif
 	    );
 	    gtk_widget_show(w);
@@ -935,7 +1011,7 @@ archive formats and therefore do not apply."
 
 	}
 
-	/* Map toplevel */
+	/* Map the dialog */
 	gtk_widget_show(toplevel);
 
 	/* Wait for user response */
@@ -948,6 +1024,15 @@ archive formats and therefore do not apply."
 	/* Update returns if the user responded with ok */
 	if(got_response)
 	{
+	    /* Password */
+	    if(password != NULL)
+	    {
+		g_free(*password);
+		*password = STRDUP(gtk_entry_get_text(
+		    GTK_ENTRY(d->password_entry)
+		));
+	    }
+
 	    /* Recursive */
 	    if(recursive != NULL)
 	    {
@@ -1000,7 +1085,8 @@ archive formats and therefore do not apply."
 gboolean EDVArchExtractOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *preserve_directories,
 	gboolean *preserve_timestamps
 )
@@ -1095,7 +1181,7 @@ gboolean EDVArchExtractOptsGetResponse(
 #endif
 	    , (guint8 **)icon_cancel_20x20_xpm,
 	    ref_toplevel,
-	    core, arch_obj
+	    core, arch_path
 	);
 	if(d == NULL)
 	    return(got_response);
@@ -1130,7 +1216,7 @@ gboolean EDVArchExtractOptsGetResponse(
 	    GUIButtonLabelUnderline(w, GDK_c);
 
 	    /* Archive object */
-	    if(!STRISEMPTY(arch_obj))
+	    if(!STRISEMPTY(arch_path))
 	    {
 		w = gtk_frame_new(NULL);
 		gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
@@ -1185,7 +1271,7 @@ gboolean EDVArchExtractOptsGetResponse(
 "-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-iso8859-1"
 #endif
 		);
-		w = gtk_label_new(g_basename(arch_obj));
+		w = gtk_label_new(g_basename(arch_path));
 		gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 		gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 		gtk_widget_modify_style(w, rcstyle);
@@ -1217,17 +1303,71 @@ archive formats and therefore do not apply."
 		gtk_widget_show(w);
 	    }
 
+	    /* GtkVBox for the option widgets */
 	    w = gtk_vbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent2 = w;
 
-	    /* Preserve Directories Check */
+
+	    /* Password GtkHBox */
 	    w = gtk_hbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent3 = w;
-	    /* Check button */
+
+	    /* Password GtkLabel */
+	    w = gtk_label_new(
+#if defined(PROG_LANGUAGE_SPANISH)
+"Contrasea:"
+#elif defined(PROG_LANGUAGE_FRENCH)
+"Mot De Passe:"
+#elif defined(PROG_LANGUAGE_GERMAN)
+"Kennwort:"
+#elif defined(PROG_LANGUAGE_ITALIAN)
+"Parola D'Ordine:"
+#elif defined(PROG_LANGUAGE_DUTCH)
+"Wachtwoord:"
+#elif defined(PROG_LANGUAGE_PORTUGUESE)
+"Senha:"
+#elif defined(PROG_LANGUAGE_NORWEGIAN)
+"Passord:"
+#elif defined(PROG_LANGUAGE_POLISH) 
+"Haso:"
+#else                 
+"Password:"
+#endif
+	    );
+	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
+	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+	    gtk_widget_show(w);
+
+	    /* Password GtkEntry */
+	    d->password_entry = w = gtk_entry_new();
+	    gtk_entry_set_visibility(GTK_ENTRY(w), FALSE);
+	    gtk_entry_set_editable(GTK_ENTRY(w), TRUE);
+	    gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
+	    if(password != NULL)
+		gtk_entry_set_text(
+		    GTK_ENTRY(w),
+		    (*password != NULL) ? *password : ""
+		);
+	    GUIEditableEndowPopupMenu(w, 0);
+	    GUISetWidgetTip(
+		w,
+"Enter the password to decrypt the archive with or leave this\
+ blank for no decryption"
+	    );
+	    gtk_widget_show(w);
+
+
+	    /* Preserve Directories GtkHBox */
+	    w = gtk_hbox_new(FALSE, border_minor);
+	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	    gtk_widget_show(w);
+	    parent3 = w;
+
+	    /* Preserve Directories GtkCheckButton */
 	    d->preserve_directories_check = w =
 		gtk_check_button_new_with_label(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1253,7 +1393,8 @@ archive formats and therefore do not apply."
 	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	    if((preserve_directories != NULL) ? *preserve_directories : FALSE)
 		GTK_TOGGLE_BUTTON(w)->active = TRUE;
-	    GUISetWidgetTip(w,
+	    GUISetWidgetTip(
+		w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto mantener los prefijos originales de sendero de\
  cada objeto extrado del archivo"
@@ -1285,12 +1426,14 @@ archive formats and therefore do not apply."
 	    );
 	    gtk_widget_show(w);
 
-	    /* Preserve Time Stamps Check */
+
+	    /* Preserve Time Stamps GtkHBox */
 	    w = gtk_hbox_new(FALSE, border_minor);
 	    gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	    gtk_widget_show(w);
 	    parent3 = w;
-	    /* Check Button */
+
+	    /* Preserve Time Stamps GtkCheckButton */
 	    d->preserve_timestamps_check = w =
 		gtk_check_button_new_with_label(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1316,7 +1459,8 @@ archive formats and therefore do not apply."
 	    gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	    if((preserve_timestamps != NULL) ? *preserve_timestamps : FALSE)
 		GTK_TOGGLE_BUTTON(w)->active = TRUE;
-	    GUISetWidgetTip(w,
+	    GUISetWidgetTip(
+		w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto mantener los sellos originales de tiempo de cada\
  objeto extrado del archivo"
@@ -1349,7 +1493,7 @@ archive formats and therefore do not apply."
 	    gtk_widget_show(w);
 	}
 
-	/* Map toplevel */
+	/* Map the dialog */
 	gtk_widget_show(toplevel);
 
 	/* Wait for user response */
@@ -1361,6 +1505,15 @@ archive formats and therefore do not apply."
 	/* Update returns if the user responded with ok */
 	if(got_response)
 	{
+	    /* Password */
+	    if(password != NULL)
+	    {
+		g_free(*password);
+		*password = STRDUP(gtk_entry_get_text(
+		    GTK_ENTRY(d->password_entry)
+		));
+	    }
+
 	    /* Preserve Directories */
 	    if(preserve_directories != NULL)
 	    {
