@@ -12,7 +12,8 @@
 extern gboolean EDVArchAddOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *recursive,
 	gint *compression,		/* 0 to 100 */
 	gboolean *dereference_links
@@ -20,7 +21,8 @@ extern gboolean EDVArchAddOptsGetResponse(
 extern gboolean EDVArchExtractOptsGetResponse(
 	edv_core_struct *core,
 	GtkWidget *ref_toplevel,
-	const gchar *arch_obj,
+	const gchar *arch_path,
+	gchar **password,
 	gboolean *preserve_directories,
 	gboolean *preserve_timestamps
 );
