@@ -4,8 +4,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "../include/disk.h"
-
 #include "guiutils.h"
 #include "toolbar.h"
 #include "cdialog.h"
@@ -32,7 +30,7 @@
 #include "images/icon_save_20x20.xpm"
 #include "images/icon_trace_20x20.xpm"
 #include "images/icon_run_20x20.xpm"
-#include "images/icon_terminal_20x20.xpm"
+#include "images/icon_terminal2_20x20.xpm"
 #include "images/icon_close_20x20.xpm"
 #include "images/icon_exit_20x20.xpm"
 #include "images/icon_archive_add_20x20.xpm"
@@ -82,7 +80,7 @@ void EDVArchiverSetTitle(
 gchar *EDVArchiverCurrentLocation(edv_archiver_struct *archiver);
 void EDVArchiverSetLocation(
 	edv_archiver_struct *archiver,
-	const gchar *path, gboolean record_history
+	const gchar *path, const gboolean record_history
 );
 void EDVArchiverUpdateLocationIcon(
 	edv_archiver_struct *archiver, const gchar *path
@@ -257,7 +255,7 @@ gchar *EDVArchiverCurrentLocation(edv_archiver_struct *archiver)
  */
 void EDVArchiverSetLocation(
 	edv_archiver_struct *archiver,
-	const gchar *path, gboolean record_history
+	const gchar *path, const gboolean record_history
 )
 {
 	const gchar *s;
@@ -274,8 +272,8 @@ void EDVArchiverSetLocation(
 	/* Get new path */
 	new_path = STRDUP(path);
 
-	/* Simplify new path */
-	SimplifyPath(new_path);
+	/* Simplify the new path */
+	EDVSimplifyPath(new_path);
 
 	/* No change? */
 	s = gtk_entry_get_text(entry);
@@ -691,16 +689,15 @@ void EDVArchiverAccelkeysRegenerate(edv_archiver_struct *archiver)
  */
 void EDVArchiverToolBarRegenerate(edv_archiver_struct *archiver)
 {
-	gint i, total_items;
-	GList *glist;
+	GList *glist, *items_list;
+	GtkWidget *parent;
 	toolbar_display tb_display;
 	toolbar_relief tb_relief;
-	GtkWidget *w, *parent;
+	toolbar_item_struct *item;
+	toolbar_struct *toolbar;
 	const cfg_intlist_struct *intlist;
 	const cfg_item_struct *cfg_list;
 	edv_archiver_opid_struct *opid;
-	toolbar_struct *toolbar;
-	toolbar_item_struct **item, *item_ptr;
 	edv_core_struct *core;
 
 	if(archiver == NULL)
@@ -708,12 +705,9 @@ void EDVArchiverToolBarRegenerate(edv_archiver_struct *archiver)
 
 	parent = archiver->tool_bar_handle;
 	core = archiver->core;
-	if((parent == NULL) || (core == NULL))
-	    return;
-
 	cfg_list = core->cfg_list;
 
-	/* Get Tool Bar configuration */
+	/* Get the Tool Bar configuration */
 	tb_display = (toolbar_display)EDV_GET_I(
 	    EDV_CFG_PARM_ARCHIVER_TOOL_BAR_DISPLAY
 	);
@@ -724,47 +718,34 @@ void EDVArchiverToolBarRegenerate(edv_archiver_struct *archiver)
 	if(intlist == NULL)
 	    return;
 
-	/* Delete existing Tool Bar if any */
+	/* Delete the existing Tool Bar */
 	ToolBarDelete(archiver->tool_bar);
 	archiver->tool_bar = NULL;
 
-	/* Create Tool Bar items list */
-	glist = intlist->list;
-	total_items = g_list_length(glist);
-	if(total_items > 0)
+	/* Create the Tool Bar items list */
+	items_list = NULL;
+	for(glist = intlist->list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
 	{
-	    item = (toolbar_item_struct **)g_malloc0(
-		total_items * sizeof(toolbar_item_struct *)
-	    );
-	    if(item == NULL)
-		total_items = 0;
-	}
-	else
-	{
-	    item = NULL;
-	}
-	for(i = 0; i < total_items; i++)
-	{
-	    /* Get Operation ID */
+	    /* Get the Operation ID */
 	    opid = EDVArchiverMatchOPID(
 		archiver,
-		(edv_archiver_op)((glist != NULL) ?
-		    glist->data : NULL
-		)
+		(edv_archiver_op)glist->data
 	    );
-	    glist = g_list_next(glist);
 	    if(opid == NULL)
 		continue;
 
 	    switch(opid->op)
 	    {
 	      case EDV_ARCHIVER_OP_NONE:
-		item[i] = item_ptr = NULL;
 		break;
 
 	      default:
-		item[i] = item_ptr = ToolBarItemNew(
+		item = ToolBarItemNew(
 		    opid->tb_item_type,
+		    NULL,
 		    opid->button_name,
 		    opid->button_icon_data,
 		    opid->tooltip,
@@ -773,26 +754,25 @@ void EDVArchiverToolBarRegenerate(edv_archiver_struct *archiver)
 		    EDVArchiverOPEnterCB, opid,
 		    EDVArchiverOPLeaveCB, opid
 		);
+		items_list = g_list_append(items_list, item);
 		break;
 	    }
 	}
 
-	/* Create new Tool Bar */
+	/* Create the Tool Bar */
 	archiver->tool_bar = toolbar = ToolBarNew(
-	    item, total_items, parent,
+	    parent,
+	    items_list,
 	    tb_display, tb_relief, FALSE
 	);
 	ToolBarMap(toolbar);
 
-	/* Delete Tool Bar items list */
-	ToolBarItemListDeleteAll(&item, &total_items);
-
-	/* Need to queue resize of toplevel since the Tool Bar size
-	 * may have changed
-	 */
-	w = archiver->toplevel;
-	if(w != NULL)
-	    gtk_widget_queue_resize(w);
+	/* Delete the Tool Bar items list */
+	if(items_list != NULL)
+	{
+	    g_list_foreach(items_list, (GFunc)ToolBarItemDelete, NULL);
+	    g_list_free(items_list);
+	}
 }
 
 /*
@@ -907,20 +887,17 @@ void EDVArchiverSyncConfiguration(edv_archiver_struct *archiver)
 	/* Location history */
 	w = archiver->location_combo;
 	if(w != NULL)
-	{
 	    EDVSaveFileGList(
 		EDV_GET_S(EDV_CFG_PARM_FILE_ARCHIVER_LOCATION_HISTORY),
 		GUIComboGetList(w)
 	    );
-	}
 
 	/* Toplevel geometry */
 	w = archiver->toplevel;
 	if(w != NULL)
 	{
-	    GdkWindow *window = w->window;
 	    gint x = 0, y = 0;
-
+	    GdkWindow *window = w->window;
 	    if(window != NULL)
 		gdk_window_get_root_origin(window, &x, &y);
 
@@ -1052,14 +1029,14 @@ void EDVArchiverOpenArchive(
 
 	if(TRUE)
 	{
-	    gchar *s = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 "Archive \"%s\" opened (%s %s)",
 		g_basename(darch_obj),
-		EDVGetObjectSizeStr(core, size),
+		EDVSizeStrDelim(size),
 		(size == 1l) ? "byte" : "bytes"
 	    );
-	    EDVStatusBarMessage(archiver->status_bar, s, FALSE);
-	    g_free(s);
+	    EDVStatusBarMessage(archiver->status_bar, msg, FALSE);
+	    g_free(msg);
 	}
 
 	/* Did an error occure while getting the listing? */
@@ -1067,7 +1044,7 @@ void EDVArchiverOpenArchive(
 	   (size > 0l)
 	)
 	{
-	    gchar *s = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 "%s.",
 		get_listing_error_msg
 	    );
@@ -1075,14 +1052,14 @@ void EDVArchiverOpenArchive(
 	    CDialogSetTransientFor(archiver->toplevel);
 	    CDialogGetResponse(
 		"Open Archive Warning",
-		s,
+		msg,
 		NULL,
 		CDIALOG_ICON_WARNING,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
+	    g_free(msg);
 	    CDialogSetTransientFor(NULL);
-	    g_free(s);
 	}
 
 	g_free(darch_obj);
@@ -1316,7 +1293,7 @@ GList *EDVArchiverOPIDListNew(edv_archiver_struct *archiver)
 	op = EDV_ARCHIVER_OP_RUN_TERMINAL;
 	tb_item_type = TOOLBAR_ITEM_BUTTON;
 	flags = 0;
-	button_icon_data = (guint8 **)icon_terminal_20x20_xpm;
+	button_icon_data = (guint8 **)icon_terminal2_20x20_xpm;
 	menu_icon_data = button_icon_data;
 #if defined(PROG_LANGUAGE_SPANISH)
 	button_name = "La Terminal";
@@ -2802,43 +2779,40 @@ static void EDVArchiverCreateMenuBar(
 edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 {
 	const gint	border_major = 5;
-	gint	toplevel_x, toplevel_y,
-		toplevel_width, toplevel_height;
 	gboolean	tool_bar_map_state,
 			location_bar_map_state,
 			find_bar_map_state,
 			status_bar_map_state;
-	const gchar *location_history_file;
-	const gchar *wm_name, *wm_class;
-	gint i;
+	gint	toplevel_x, toplevel_y,
+		toplevel_width, toplevel_height;
 	gpointer entry_rtn, combo_rtn;
+	const gchar	*wm_name, *wm_class,
+			*location_history_file;
 	GList *glist;
 	GdkCursor *cursor;
 	GdkWindow *window;
 	GtkRcStyle *standard_rcstyle, *lists_rcstyle;
 	GtkStyle *style;
-	GtkWidget *w, *parent, *parent2, *parent3, *parent4;
+	GtkWidget	*w,
+			*parent, *parent2, *parent3, *parent4,
+			*toplevel;
 	GtkEntry *entry;
 	GtkCombo *combo;
 	GtkCList *clist;
 	const GtkTargetEntry dnd_tar_types[] = {
-{"text/plain",				0,	EDV_DND_INFO_TEXT_PLAIN},
-{"text/uri-list",			0,	EDV_DND_INFO_TEXT_URI_LIST},
-{"STRING",				0,	EDV_DND_INFO_STRING}
+{GUI_TARGET_NAME_TEXT_PLAIN,	0,	EDV_DND_INFO_TEXT_PLAIN},
+{GUI_TARGET_NAME_TEXT_URI_LIST,	0,	EDV_DND_INFO_TEXT_URI_LIST},
+{GUI_TARGET_NAME_STRING,	0,	EDV_DND_INFO_STRING}
 	};
 	const GtkTargetEntry dnd_src_types[] = {
-{EDV_DND_TARGET_ARCHIVE_OBJECT,		0,	EDV_DND_INFO_ARCHIVE_OBJECT}
+{EDV_DND_TARGET_ARCHIVE_OBJECT,	0,	EDV_DND_INFO_ARCHIVE_OBJECT}
 	};
 	const cfg_item_struct *cfg_list;
 	edv_status_bar_struct *status_bar;
-	edv_archiver_struct *archiver = EDV_ARCHIVER(
-	    g_malloc0(sizeof(edv_archiver_struct))
-	);
-	if((archiver == NULL) || (core == NULL))
-	{
-	    g_free(archiver);
+	edv_archiver_struct *archiver;
+
+	if(core == NULL)
 	    return(NULL);
-	}
 
 	cfg_list = core->cfg_list;
 
@@ -2881,9 +2855,16 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	    EDV_CFG_PARM_ARCHIVER_SHOW_STATUS_BAR
 	);
 
+
+	/* Create the Archiver */
+	archiver = EDV_ARCHIVER(g_malloc0(sizeof(edv_archiver_struct)));
+	if(archiver == NULL)
+	    return(NULL);
+
+	archiver->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	archiver->processing = FALSE;
-	archiver->busy_count = 0;
 	archiver->freeze_count = 0;
+	archiver->busy_count = 0;
 	archiver->core = core;
 
 	archiver->opid_list = EDVArchiverOPIDListNew(archiver);
@@ -2901,11 +2882,12 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	archiver->find_bar = NULL;
 	archiver->status_bar = NULL;
 
+	archiver->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	archiver->toplevel = w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_widget_set_name(w, EDV_ARCHIVER_TOPLEVEL_WIDGET_NAME);
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_uposition(
 	    w, toplevel_x, toplevel_y
 	);
@@ -2918,10 +2900,6 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	    gtk_window_set_wmclass(
 		GTK_WINDOW(w), "archiver", PROG_NAME
 	    );
-	gtk_window_set_policy(
-	    GTK_WINDOW(w),
-	    TRUE, TRUE, TRUE
-	);
 	gtk_widget_realize(w);
 	window = w->window;
 	if(window != NULL)
@@ -2963,21 +2941,21 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	parent = w;
 
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	archiver->main_vbox = w = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	gtk_widget_show(w);
 	parent = w;
 
 
-	/* Vbox for tool bars */
+	/* Tool Bars GtkVBox */
 	archiver->bars_vbox = w = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Handle for menu bar */
+	/* Menu Bar GtkHandleBox */
 	archiver->menu_bar_handle = w = gtk_handle_box_new();
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_signal_connect_after(
@@ -2993,7 +2971,7 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	EDVArchiverCreateMenuBar(archiver, w);
 
 
-	/* Handle for tool bar */
+	/* Tool Bar GtkHandleBox */
 	archiver->tool_bar_handle = w = gtk_handle_box_new();
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_signal_connect_after(
@@ -3010,7 +2988,7 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	archiver->tool_bar = NULL;
 
 
-	/* Location bar handle box */
+	/* Location Bar GtkHandleBox */
 	archiver->location_bar_handle = w = gtk_handle_box_new();
 	gtk_signal_connect_after(
 	    GTK_OBJECT(w), "child_attached",
@@ -3093,7 +3071,7 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	/* Reset location icon pixmap */
 	archiver->location_icon_pm = NULL;
 
-	/* Location combo */
+	/* Location GtkCombo */
 	glist = EDVOpenFileGList(location_history_file, 0);
 	w = GUIComboCreate(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -3139,9 +3117,9 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	{
 	    /* Need to set different dnd_src_types for the drag source */
 	    const GtkTargetEntry dnd_src_types[] = {
-{"text/plain",				0,	EDV_DND_INFO_TEXT_PLAIN},
-{"text/uri-list",			0,	EDV_DND_INFO_TEXT_URI_LIST},
-{"STRING",				0,	EDV_DND_INFO_STRING}
+{GUI_TARGET_NAME_TEXT_PLAIN,	0,	EDV_DND_INFO_TEXT_PLAIN},
+{GUI_TARGET_NAME_TEXT_URI_LIST,	0,	EDV_DND_INFO_TEXT_URI_LIST},
+{GUI_TARGET_NAME_STRING,	0,	EDV_DND_INFO_STRING}
 	    };
 	    GUIDNDSetSrc(
 		archiver->location_icon_fixed,
@@ -3157,10 +3135,12 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 		combo
 	    );
 	}
-	EDVEntrySetDND(core, combo->entry);
-	EDVEntrySetCompletePath(core, combo->entry);
+	w = combo->entry;
+	EDVEntrySetDND(core, w);
+	EDVEntrySetCompletePath(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
 
-	/* VBox for password prompt */
+	/* Password GtkVBox */
 	w = gtk_vbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, border_major);
 	gtk_widget_show(w);
@@ -3200,7 +3180,9 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	    GTK_OBJECT(w), "activate",
 	    GTK_SIGNAL_FUNC(EDVArchiverComboActivateCB), archiver
 	);
-	GUISetWidgetTip(w,
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Entre la contrasea (si necesit) para el decrypting/codificado\
  el archivo"
@@ -3257,7 +3239,7 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	EDVFindBarMap(archiver->find_bar);
 
 
-	/* Scrolled window for contents clist */
+	/* GtkScrolledWindow for the GtkCList */
 	w = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(
 	    GTK_SCROLLED_WINDOW(w),
@@ -3268,10 +3250,11 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Contents clist */
+	/* Contents GtkCList */
 	archiver->contents_clist = w = gtk_clist_new(
 	    EDV_ARCHIVER_CONTENTS_CLIST_COLUMNS_MAX
 	);
+	gtk_widget_set_name(w, EDV_ARCHIVER_CLIST_WIDGET_NAME);
 	clist = GTK_CLIST(w);
 	gtk_widget_add_events(
 	    w,
@@ -3313,12 +3296,9 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	    GTK_SIGNAL_FUNC(EDVArchiverButtonPressEventCB), archiver
 	);
 	gtk_container_add(GTK_CONTAINER(parent2), w);
-	gtk_widget_realize(w);
-	gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
-	for(i = 0; i < clist->columns; i++)
-	    gtk_clist_set_column_width(clist, i, 10);
-	gtk_clist_set_row_height(clist, EDV_LIST_ROW_SPACING);
 	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);
+	gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
+	gtk_clist_set_row_height(clist, EDV_LIST_ROW_SPACING);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "resize_column",
 	    GTK_SIGNAL_FUNC(EDVArchiverResizeColumnCB), archiver
@@ -3335,7 +3315,6 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	    GTK_OBJECT(w), "unselect_row",
 	    GTK_SIGNAL_FUNC(EDVArchiverUnselectRowCB), archiver
 	);
-	/* Set up DND for the contents list */
 	GUIDNDSetSrc(
 	    w,
 	    dnd_src_types,
@@ -3365,11 +3344,10 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 	gtk_widget_show(w);
 
 
-
 	/* Create the Tool Bar */
 	EDVArchiverToolBarRegenerate(archiver);
 
-	/* Create the Contents List Menu */
+	/* Create the Contents List popup menu */
 	EDVArchiverContentsMenuRegenerate(archiver);
 
 
@@ -3387,10 +3365,9 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 
 
 	/* The Archive Info Dialog is created later when needed */
-	archiver->arch_info = NULL;
+	archiver->arch_info_dlg = NULL;
 
-
-	/* Set initial RC styles */
+	/* Set the initial RC styles */
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(
 		archiver->toplevel, standard_rcstyle
@@ -3404,12 +3381,13 @@ edv_archiver_struct *EDVArchiverNew(edv_core_struct *core)
 		archiver->contents_clist_menu, standard_rcstyle
 	    );
 
-
-	/* Reset values */
+	/* Set the initial values */
 	EDVArchiverSetTitle(archiver, NULL);
 	EDVArchiverAccelkeysRegenerate(archiver);
 	EDVArchiverUpdateMenus(archiver);
 
+	archiver->freeze_count--;
+
 	return(archiver);
 }
 
@@ -3448,21 +3426,20 @@ void EDVArchiverUpdateMenus(edv_archiver_struct *archiver)
 	if(archiver == NULL)
 	    return;
 
-	core = archiver->core;
-	if(core == NULL)
-	    return;
-
-	cfg_list = core->cfg_list;
 	tb = archiver->tool_bar;
+	core = archiver->core;
+	cfg_list = core->cfg_list;
 
-#define DO_SHOW			\
+	archiver->freeze_count++;
+
+#define DO_SHOW				\
 { if(w != NULL) gtk_widget_show(w); }
-#define DO_HIDE			\
+#define DO_HIDE				\
 { if(w != NULL) gtk_widget_hide(w); }
-#define DO_SET_CHECK_STATE	\
+#define DO_SET_CHECK_STATE		\
 { GUIMenuItemSetCheck(w, state, FALSE); }
 
-	/* Get global write protect state */
+	/* Get the global write protect state */
 	write_protect = EDV_GET_B(EDV_CFG_PARM_WRITE_PROTECT);
 	if(archiver->last_write_protect_state < 0)
 	{
@@ -3483,7 +3460,7 @@ void EDVArchiverUpdateMenus(edv_archiver_struct *archiver)
 	);
 
 	/* Get last selected row on the contents clist */
-	clist = (GtkCList *)archiver->contents_clist;
+	clist = GTK_CLIST(archiver->contents_clist);
 	selected_obj_row = EDVCListGetSelectedLast(clist, NULL);
 
 	/* Update recycle bin icon data, setting it to not NULL if it
@@ -3512,41 +3489,45 @@ void EDVArchiverUpdateMenus(edv_archiver_struct *archiver)
 	archiver->last_recbin_items = core->last_recbin_items;
 	recbin_icon_data = icon_data;
 
-	/* Get current location */
+	/* Get the current location */
 	location = EDVArchiverCurrentLocation(archiver);
 
 
+        /* Update the find bar */
+        EDVFindBarUpdateMenus(archiver->find_bar);
+
+
 
 	/* Begin updating menu items and tool bar items */
 
 	/* Edit */
 	/* Add */
 	sensitive = !STRISEMPTY(location);
-	GTK_WIDGET_SET_SENSITIVE(archiver->edit_add_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(archiver->edit_add_mi, sensitive);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_ADD, sensitive
 	);
 	/* Extract all */
-	GTK_WIDGET_SET_SENSITIVE(archiver->edit_extract_all_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(archiver->edit_extract_all_mi, sensitive);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_EXTRACT_ALL, sensitive
 	);
 
 	/* Extract */
 	sensitive = (selected_obj_row < 0) ? FALSE : TRUE;
-	GTK_WIDGET_SET_SENSITIVE(archiver->edit_extract_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(archiver->edit_extract_mi, sensitive);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_EXTRACT, sensitive
 	);
 	/* Delete */
-	GTK_WIDGET_SET_SENSITIVE(archiver->edit_delete_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(archiver->edit_delete_mi, sensitive);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_DELETE, sensitive
 	);
 
 	/* Properties */
 	sensitive = !STRISEMPTY(location);
-	GTK_WIDGET_SET_SENSITIVE(archiver->properties_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(archiver->properties_mi, sensitive);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_PROPERTIES, sensitive
 	);
@@ -3557,7 +3538,7 @@ void EDVArchiverUpdateMenus(edv_archiver_struct *archiver)
 	sensitive = !STRISEMPTY(location);
 	GTK_WIDGET_SET_SENSITIVE(
 	    archiver->comment_and_statistics_mi, sensitive
-	)
+	);
 	ToolBarItemSetSensitiveID(
 	    tb, EDV_ARCHIVER_OP_COMMENT_AND_STATISTICS, sensitive
 	);
@@ -3647,35 +3628,33 @@ void EDVArchiverUpdateMenus(edv_archiver_struct *archiver)
 	}
 
 
-	/* Update contents clist menu */
+	/* Contents List popup menu */
 	sensitive = (selected_obj_row < 0) ?
 	    FALSE : TRUE;
 	GTK_WIDGET_SET_SENSITIVE(
 	    archiver->contents_clist_extract_mi, sensitive
-	)
+	);
 	GTK_WIDGET_SET_SENSITIVE(
 	    archiver->contents_clist_delete_mi, sensitive
-	)
+	);
 
 	sensitive = !STRISEMPTY(location);
 	GTK_WIDGET_SET_SENSITIVE(
 	    archiver->contents_clist_comment_and_statistics_mi, sensitive
-	)
+	);
 	GTK_WIDGET_SET_SENSITIVE(
 	    archiver->contents_clist_properties_mi, sensitive
-	)
+	);
 
 
-
-
-	/* Update status bar */
+	/* Status Bar */
 	EDVStatusBarUpdate(archiver->status_bar);
 
-
-
 #undef DO_SHOW
 #undef DO_HIDE
 #undef DO_SET_CHECK_STATE
+
+	archiver->freeze_count--;
 }
 
 /*
@@ -3692,8 +3671,6 @@ void EDVArchiverSetBusy(edv_archiver_struct *archiver, gboolean busy)
 
 	w = archiver->toplevel;
 	core = archiver->core;
-	if((w == NULL) || (core == NULL))
-	    return;
 
 	if(busy)
 	{
@@ -3726,8 +3703,10 @@ void EDVArchiverSetBusy(edv_archiver_struct *archiver, gboolean busy)
  */
 gboolean EDVArchiverIsMapped(edv_archiver_struct *archiver)
 {
-	GtkWidget *w = (archiver != NULL) ? archiver->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(archiver == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(archiver->toplevel));
 }
 
 /*
@@ -3735,11 +3714,10 @@ gboolean EDVArchiverIsMapped(edv_archiver_struct *archiver)
  */
 void EDVArchiverMap(edv_archiver_struct *archiver)
 {
-	GtkWidget *w = (archiver != NULL) ? archiver->toplevel : NULL;
-	if(w == NULL)
+	if(archiver == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(archiver->toplevel);
 }
 
 /*
@@ -3747,11 +3725,10 @@ void EDVArchiverMap(edv_archiver_struct *archiver)
  */
 void EDVArchiverUnmap(edv_archiver_struct *archiver)
 {
-	GtkWidget *w = (archiver != NULL) ? archiver->toplevel : NULL;
-	if(w == NULL)
-	    return;
+        if(archiver == NULL)
+            return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(archiver->toplevel);
 }
 
 /*
@@ -3762,42 +3739,32 @@ void EDVArchiverDelete(edv_archiver_struct *archiver)
 	if(archiver == NULL)
 	    return;
 
-
-	/* Delete all rows in the Contents List */
+	/* Clear the Contents List */
 	EDVArchiverContentsClear(archiver);
 
-	/* Reset last selected item references */
+	/* Reset the last selected item references */
 	archiver->contents_clist_selected_row = -1;
 
+	/* Delete the Archive Info Dialog */
+	EDVArchiveInfoDelete(archiver->arch_info_dlg);
+	archiver->arch_info_dlg = NULL;
 
-	EDVArchiveInfoDelete(archiver->arch_info);
+	EDVArchiverUnmap(archiver);
+
+	archiver->freeze_count++;
 
 	GTK_WIDGET_DESTROY(archiver->contents_clist_menu);
-
 	EDVStatusBarDelete(archiver->status_bar);
-
 	EDVFindBarDelete(archiver->find_bar);
-
-	GTK_WIDGET_DESTROY(archiver->location_password_entry);
-	GTK_WIDGET_DESTROY(archiver->location_combo);
-	GTK_WIDGET_DESTROY(archiver->location_icon_pm);
-	GTK_WIDGET_DESTROY(archiver->location_icon_fixed);
-
 	ToolBarDelete(archiver->tool_bar);
-
-	GTK_WIDGET_DESTROY(archiver->menu_bar);
-
-	GTK_WIDGET_DESTROY(archiver->menu_bar_handle);
-	GTK_WIDGET_DESTROY(archiver->tool_bar_handle);
-	GTK_WIDGET_DESTROY(archiver->bars_vbox);
-	GTK_WIDGET_DESTROY(archiver->contents_clist);
-	GTK_WIDGET_DESTROY(archiver->main_vbox);
 	GTK_WIDGET_DESTROY(archiver->toplevel);
 
-	/* Delete all Operation IDs */
+	/* Delete the Operation IDs */
 	EDVArchiverOPIDListDelete(archiver->opid_list);
 
 	g_free(archiver->contents_list_filter);
 
+	archiver->freeze_count--;
+
 	g_free(archiver);
 }
