@@ -305,10 +305,10 @@ gint EDVArchiverDeleteEventCB(
 {
 	edv_archiver_struct *archiver = EDV_ARCHIVER(data);
 	if(archiver == NULL)
-	    return(TRUE);
+	    return(FALSE);
 
-	if(archiver->processing)
-	    return(TRUE);
+	if(archiver->processing || (archiver->freeze_count > 0))
+	    return(FALSE);
 
 	EDVArchiverOPClose(archiver);
 
@@ -341,17 +341,18 @@ gint EDVArchiverKeyEventCB(
 	    return(status);
 
 	w = archiver->toplevel;
-	focus_widget = (w != NULL) ? GTK_WINDOW(w)->focus_widget : NULL;
+	focus_widget = GTK_WINDOW(w)->focus_widget;
 	cfg_list = core->cfg_list;
+
 	etype = key->type;
 	is_press = (etype == GDK_KEY_PRESS) ? TRUE : FALSE;
 	keyval = key->keyval;
 	state = key->state;
 
 /* Stop emit of signal */
-#define DO_STOP_KEY_SIGNAL_EMIT	{		\
+#define STOP_SIGNAL_EMISSION(_w_)	{	\
  gtk_signal_emit_stop_by_name(			\
-  GTK_OBJECT(widget),				\
+  GTK_OBJECT(_w_),				\
   is_press ?					\
    "key_press_event" : "key_release_event"	\
  );						\
@@ -378,7 +379,7 @@ gint EDVArchiverKeyEventCB(
 		    if(opid != NULL)
 			EDVArchiverOPCB(NULL, -1, opid);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		return(status);
 	    }
@@ -421,7 +422,7 @@ gint EDVArchiverKeyEventCB(
 			gtk_clist_select_row(clist, row, 0);
 		    gtk_clist_thaw(clist);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		break;
 
@@ -431,44 +432,38 @@ gint EDVArchiverKeyEventCB(
 		 * item who's name starts with the letter of the key
 		 * that was pressed
 		 */
-		if(isalnum((int)keyval) && is_press &&
-		   !(state & GDK_CONTROL_MASK) &&
-		   !(state & GDK_SHIFT_MASK)    
-		)
+		if(isalnum((int)keyval) && is_press)
 		{
-		    /* Get the column that is displaying the object
-		     * name as column_type_name
-		     */
-		    gint column_type_name = -1;
-		    const cfg_intlist_struct *intlist = EDV_GET_INTLIST(
+		    const gboolean backwards = (state & GDK_MOD1_MASK) ? TRUE : FALSE;
+		    const gint start_row = (backwards) ?
+			(clist->rows - 1) : 0;
+
+		    /* Find the column that is set to display the name */
+		    cfg_intlist_struct *intlist = EDV_GET_INTLIST(
 			EDV_CFG_PARM_ARCHIVER_CONTENTS_COLUMN
 		    );
-		    if(intlist != NULL)
-		    {
-			/* Iterate through column type intlist */
-			gint i = 0;
-			GList *glist = intlist->list;
-			while(glist != NULL)
-			{
-			    if((edv_archiver_column_type)glist->data ==
-				EDV_ARCHIVER_COLUMN_TYPE_NAME
-			    )
-			    {
-				column_type_name = i;
-				break;
-			    }
-			    i++;
-			    glist = g_list_next(glist);
-			}
-		    }
+		    const gint column_type_name = (intlist != NULL) ?
+			g_list_index(
+			    intlist->list,
+			    (gpointer)EDV_ARCHIVER_COLUMN_TYPE_NAME
+			) : 0;
 
+
+		    /* Seek to the row who's text matches this
+		     * key event's character
+		     */
 		    gtk_clist_freeze(clist);
 		    EDVCListSeekCharacter(
-			clist, column_type_name, 0, keyval
+			clist,
+			column_type_name,	/* Column */
+			start_row,
+			backwards,
+			0,			/* Text index */
+			keyval			/* Character */
 		    );
 		    gtk_clist_thaw(clist);
 
-		    DO_STOP_KEY_SIGNAL_EMIT
+		    STOP_SIGNAL_EMISSION(widget);
 		    status = TRUE;
 		}
 		break;
@@ -476,7 +471,7 @@ gint EDVArchiverKeyEventCB(
 	}
 
 	return(status);
-#undef DO_STOP_KEY_SIGNAL_EMIT
+#undef STOP_SIGNAL_EMISSION
 }
 
 /*
@@ -867,8 +862,6 @@ void EDVArchiverSelectRowCB(
 	/* Check which clist this signal is for */
 	if(GTK_WIDGET(clist) == archiver->contents_clist)
 	{
-	    edv_core_struct *core = archiver->core;
-
 	    /* Get total number of objects selected */
 	    const gint nselected = g_list_length(clist->selection);
 
@@ -902,10 +895,7 @@ void EDVArchiverSelectRowCB(
 			type_str = "File";
 			size_str = g_strdup_printf(
 			    " (%s %s)",
-			    EDVGetObjectSizeStr(
-				archiver->core,
-				obj->size
-			    ),
+			    EDVSizeStrDelim(obj->size),
 			    (obj->size == 1l) ? "byte" : "bytes"
 			);
 			break;
@@ -943,7 +933,7 @@ void EDVArchiverSelectRowCB(
 		    if(nselected > 1)
 			s = g_strdup_printf(
 			    "%s objects selected",
-			    EDVGetObjectSizeStr(core, nselected)
+			    EDVSizeStrDelim(nselected)
 			);
 		    else if(!strcmp(obj->name, ".."))
 			s = g_strdup_printf(
@@ -1006,7 +996,6 @@ void EDVArchiverUnselectRowCB(
 	if(GTK_WIDGET(clist) == archiver->contents_clist)
 	{
 	    gchar *s;
-	    edv_core_struct *core = archiver->core;
 
 	    /* Get total number of objects selected */
 	    const gint nselected = g_list_length(clist->selection);
@@ -1014,9 +1003,9 @@ void EDVArchiverUnselectRowCB(
 	    /* Update status bar message */
 	    if(nselected > 0)
 		s = g_strdup_printf(
-		    "%s object%s selected",
-		    EDVGetObjectSizeStr(core, nselected),
-		    (nselected == 1) ? "" : "s"
+		    "%s %s selected",
+		    EDVSizeStrDelim(nselected),
+		    (nselected == 1) ? "object" : "objects"
 		);
 	    else
 		s = STRDUP("No objects selected");
@@ -1236,7 +1225,7 @@ void EDVArchiverObjectAddedNotifyCB(
 	const struct stat *lstat_buf
 )
 {
-	if(archiver == NULL)
+	if((archiver == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(archiver->processing)
@@ -1257,37 +1246,34 @@ void EDVArchiverObjectModifiedNotifyCB(
 	const gchar *new_path, const struct stat *lstat_buf
 )
 {
-	if(archiver == NULL)
+	gchar *cur_arch_path;
+
+	if((archiver == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(archiver->processing)
 	    return;
 
-	/* Check current location first, if the old path matches the
-	 * current location then the current location needs to be updated
-	 * to reflect the new path before notifying the contents clist
-	 */
-	if(path != NULL)
+	if(new_path == NULL)
+	    new_path = path;
+
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	/* Is the modified path the current archive? */
+	if(!strcmp((const char *)cur_arch_path, (const char *)path))
 	{
-	    const gchar *s;
-
-	    if(new_path == NULL)
-		new_path = path;
-
-	    /* Check if the old path matches the current location, if it
-	     * does then the current location needs to be updated to the
-	     * new path
-	     */
-	    s = EDVArchiverCurrentLocation(archiver);
-	    if((s != NULL) ? !strcmp(s, path) : FALSE)
+	    /* Path changed? */
+	    if(path != new_path)
 	    {
-		/* Old path matches current location, change values
-		 * reflecting the current location to the value of
-		 * new_path
-		 */
-		EDVArchiverSetTitle(archiver, new_path);
-		EDVArchiverSetLocation(archiver, new_path, FALSE);
-		EDVArchiverUpdateLocationIcon(archiver, new_path);
+		if(strcmp((const char *)path, (const char *)new_path))
+		{
+		    /* Update the current archive path */
+		    EDVArchiverSetTitle(archiver, new_path);
+		    EDVArchiverSetLocation(archiver, new_path, FALSE);
+		    EDVArchiverUpdateLocationIcon(archiver, new_path);
+		}
 	    }
 	}
 
@@ -1296,6 +1282,8 @@ void EDVArchiverObjectModifiedNotifyCB(
 	);
 
 /*	EDVArchiverUpdateMenus(archiver); */
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1305,39 +1293,29 @@ void EDVArchiverObjectRemovedNotifyCB(
 	edv_archiver_struct *archiver, const gchar *path
 )
 {
-	if(archiver == NULL)
+	gchar *cur_arch_path;
+
+	if((archiver == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(archiver->processing)
 	    return;
 
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
 	EDVArchiverContentsObjectRemovedNotify(archiver, path);
 
-	/* Check if the path that was removed is the current location
-	 * (after letting contents clist and directory ctree check on it
-	 * first), if the path that was removed is the location then
-	 * set the current location to be the parent of the path that
-	 * was removed.
-	 */
-	if(path != NULL)
+	/* Is the removed object the current archive? */
+	if(!strcmp((const char *)cur_arch_path, (const char *)path))
 	{
-	    const gchar *s;
-
-	    /* Check if the removed object path matches the current
-	     * location. If it matches then the current location needs to
-	     * be updated to reflect that.
-	     */
-	    s = EDVArchiverCurrentLocation(archiver);
-	    if((s != NULL) ? !strcmp(s, path) : FALSE)
-	    {
-		/* Removed object path matches the current location */
-
-/* Do not need to update title or location because of this */
-
-	    }
+	    /* No need to update the title or the location */
 	}
 
 /*	EDVArchiverUpdateMenus(archiver); */
+
+	g_free(cur_arch_path);
 }
 
 
@@ -1349,30 +1327,32 @@ void EDVArchiverArchiveObjectAddedNotifyCB(
 	const gchar *path, edv_archive_object_struct *obj
 )
 {
-	const gchar *cur_loc;
+	gchar *cur_arch_path;
 
-	if(archiver == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path) || (obj == NULL)
+	)
 	    return;
 
 	if(archiver->processing)
 	    return;
 
-	/* Compare the archive path with the current archive selected
-	 * by this archiver
-	 */
-	cur_loc = EDVArchiverCurrentLocation(archiver);
-	if((cur_loc != NULL) && (arch_path != NULL))
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	/* Is the added object in our archive? */
+	if(!strcmp((const char *)cur_arch_path, (const char *)arch_path))
 	{
-	    if(!strcmp(cur_loc, arch_path))
-	    {
-		/* Notify contents list */
-		EDVArchiverContentsArchiveObjectAddedNotify(
-		    archiver, arch_path, path, obj
-		);
-	    }
+	    /* Notify the contents list */
+	    EDVArchiverContentsArchiveObjectAddedNotify(
+		archiver, arch_path, path, obj
+	    );
 	}
 
 /*	EDVArchiverUpdateMenus(archiver); */
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1384,30 +1364,35 @@ void EDVArchiverArchiveObjectModifiedNotifyCB(
 	edv_archive_object_struct *obj
 )
 {
-	const gchar *cur_loc;
+	gchar *cur_arch_path;
 
-	if(archiver == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path) || (obj == NULL)
+	)
 	    return;
 
 	if(archiver->processing)
 	    return;
 
-	/* Compare the archive path with the current archive selected
-	 * by this archiver
-	 */
-	cur_loc = EDVArchiverCurrentLocation(archiver);
-	if((cur_loc != NULL) && (arch_path != NULL))
+	if(new_path == NULL)
+	    new_path = path;
+
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	/* Is the modified object in our archive? */
+	if(!strcmp((const char *)cur_arch_path, (const char *)arch_path))
 	{
-	    if(!strcmp(cur_loc, arch_path))
-	    {
-		/* Notify contents list */
-		EDVArchiverContentsArchiveObjectModifiedNotify(
-		    archiver, arch_path, path, new_path, obj
-		);
-	    }
+	    /* Notify the contents list */
+	    EDVArchiverContentsArchiveObjectModifiedNotify(
+		archiver, arch_path, path, new_path, obj
+	    );
 	}
 
 /*	EDVArchiverUpdateMenus(archiver); */
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1418,30 +1403,32 @@ void EDVArchiverArchiveObjectRemovedNotifyCB(
 	const gchar *path
 )
 {
-	const gchar *cur_loc;
+	gchar *cur_arch_path;
 
-	if(archiver == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path)
+	)
 	    return;
 
 	if(archiver->processing)
 	    return;
 
-	/* Compare the archive path with the current archive selected
-	 * by this archiver
-	 */
-	cur_loc = EDVArchiverCurrentLocation(archiver);
-	if((cur_loc != NULL) && (arch_path != NULL))
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	/* Is the removed object in our archive? */
+	if(!strcmp((const char *)cur_arch_path, (const char *)arch_path))
 	{
-	    if(!strcmp(cur_loc, arch_path))
-	    {
-		/* Notify contents list */
-		EDVArchiverContentsArchiveObjectRemovedNotify(
-		    archiver, arch_path, path
-		);
-	    }
+	    /* Notify the contents list */
+	    EDVArchiverContentsArchiveObjectRemovedNotify(
+		archiver, arch_path, path
+	    );
 	}
 
 /*	EDVArchiverUpdateMenus(archiver); */
+
+	g_free(cur_arch_path);
 }
 
 /*
