@@ -23,6 +23,7 @@
 #include "archiver_contents_list.h"
 #include "endeavour2.h"
 #include "edv_cb.h"
+#include "edv_op.h"
 #include "edv_utils.h"
 #include "edv_utils_gtk.h"
 
@@ -30,8 +31,8 @@
 #include "config.h"
 
 
-typedef struct _stat_list_data_struct	stat_list_data_struct;
-#define STAT_LIST_DATA(p)		((stat_list_data_struct *)(p))
+typedef struct _EDVArchiverNextObjectReturnData	EDVArchiverNextObjectReturnData;
+#define EDV_ARCHIVER_NEXT_OBJECT_RETURN_DATA(p)		((EDVArchiverNextObjectReturnData *)(p))
 
 
 #if 0
@@ -40,7 +41,7 @@ static edv_archive_object_struct *EDVArchiverNewErrorObject(
 );
 #endif
 
-static gint EDVArchiverContentsStatProgressCB(
+static gint EDVArchiverContentsStatNextObjectReturnCB(
 	const gchar *arch_path,
 	edv_archive_object_struct *obj,
 	const gulong i, const gulong m,
@@ -61,13 +62,17 @@ static void EDVArchiverContentsSetCellSize(
 	edv_core_struct *core, edv_archiver_struct *archiver,
 	GtkCList *clist, edv_archive_object_struct *obj,
 	const gint row, const gint column,
-	const gboolean hide_dir_size, const gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 );
 static void EDVArchiverContentsSetCellCompressedSize(
 	edv_core_struct *core, edv_archiver_struct *archiver,
 	GtkCList *clist, edv_archive_object_struct *obj,
 	const gint row, const gint column,
-	const gboolean hide_dir_size, const gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 );
 static void EDVArchiverContentsSetCellType(
 	edv_core_struct *core, edv_archiver_struct *archiver,
@@ -208,7 +213,13 @@ void EDVArchiverContentsArchiveObjectRemovedNotify(
 );
 
 
-static const gchar *get_listing_last_error;
+struct _EDVArchiverNextObjectReturnData {
+	edv_core_struct	*core;
+	edv_archiver_struct	*archiver;
+	GtkCList	*clist;
+	gint		last_progress_percent;
+
+};
 
 
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
@@ -223,14 +234,6 @@ static const gchar *get_listing_last_error;
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
-struct _stat_list_data_struct {
-
-	edv_archiver_struct	*archiver;
-	gint	last_progress_percent;
-
-};
-
-
 #if 0
 /* 
  *	Creates a new error object.
@@ -252,7 +255,7 @@ static edv_archive_object_struct *EDVArchiverNewErrorObject(
 
 	    obj->full_path = STRDUP(path);
 
-	    s = strrchr(path, G_DIR_SEPARATOR);
+	    s = (const gchar *)strrchr((const char *)path, G_DIR_SEPARATOR);
 	    if(s != NULL)
 		s++;
 	    else
@@ -268,22 +271,26 @@ static edv_archive_object_struct *EDVArchiverNewErrorObject(
 /*
  *	Stat list progress callback.
  */
-static gint EDVArchiverContentsStatProgressCB(
+static gint EDVArchiverContentsStatNextObjectReturnCB(
 	const gchar *arch_path,
 	edv_archive_object_struct *obj,
 	const gulong i, const gulong m,
 	gpointer data
 )
 {
-	edv_archiver_struct *archiver;
-	stat_list_data_struct *d = STAT_LIST_DATA(data);
-	if(d == NULL)
-	    return(-1);
+	EDVArchiverNextObjectReturnData *d = EDV_ARCHIVER_NEXT_OBJECT_RETURN_DATA(data);
+	edv_archiver_struct *archiver = d->archiver;
 
-	archiver = d->archiver;
-	if(archiver == NULL)
-	    return(-1);
+	/* Got object to append to the Contents List? */
+	if(obj != NULL)
+	    EDVArchiverContentsAppendObject(
+		d->core,
+		archiver,
+		d->clist,
+		obj
+	    );
 
+	/* Report progress */
 	if(m > 0l)
 	{
 	    const gint progress_percent = (gint)(i * 100l / m);
@@ -339,6 +346,8 @@ static void EDVArchiverContentsResetColumns(
 	if(column_width_intlist == NULL)
 	    return;
 
+	gtk_clist_freeze(clist);
+
 	/* Update clist column settings */
 	gtk_clist_column_titles_active(clist);
 	gtk_clist_column_titles_show(clist);
@@ -457,6 +466,8 @@ static void EDVArchiverContentsResetColumns(
 	    gtk_widget_modify_style_recursive(
 		GTK_WIDGET(clist), lists_rcstyle
 	    );
+
+	gtk_clist_thaw(clist);
 }
 
 
@@ -505,6 +516,7 @@ static void EDVArchiverContentsSetCellName(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	if(pixmap != NULL)
 	    gtk_clist_set_pixtext(
 		clist, row, column,
@@ -521,6 +533,7 @@ static void EDVArchiverContentsSetCellName(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -531,7 +544,9 @@ static void EDVArchiverContentsSetCellSize(
 	edv_core_struct *core, edv_archiver_struct *archiver,
 	GtkCList *clist, edv_archive_object_struct *obj,
 	const gint row, const gint column,
-	const gboolean hide_dir_size, const gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 )
 {
 	const gint border_minor = 2;
@@ -548,16 +563,20 @@ static void EDVArchiverContentsSetCellSize(
 	)
 	    s = "";
 	else
-	    s = EDVGetObjectSizeStr(core, obj->size);
+	    s = EDVSizeStrFormat(
+		obj->size, size_format, block_size, ',', TRUE
+	    );
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, s
+	    clist, row, column, (s != NULL) ? s : ""
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -568,7 +587,9 @@ static void EDVArchiverContentsSetCellCompressedSize(
 	edv_core_struct *core, edv_archiver_struct *archiver,
 	GtkCList *clist, edv_archive_object_struct *obj,
 	const gint row, const gint column,
-	const gboolean hide_dir_size, const gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 )
 {
 	const gint border_minor = 2;
@@ -585,16 +606,20 @@ static void EDVArchiverContentsSetCellCompressedSize(
 	)
 	    s = "";
 	else
-	    s = EDVGetObjectSizeStr(core, obj->compressed_size);
+	    s = EDVSizeStrFormat(
+		obj->compressed_size, size_format, block_size, ',', TRUE
+	    );
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, s
+	    clist, row, column, (s != NULL) ? s : ""
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -619,6 +644,7 @@ static void EDVArchiverContentsSetCellType(
 	);
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, (type_str != NULL) ? type_str : ""
 	);
@@ -626,6 +652,7 @@ static void EDVArchiverContentsSetCellType(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -639,7 +666,7 @@ static void EDVArchiverContentsSetCellPermissions(
 	const gboolean hide_link_permissions
 )
 {
-	edv_permission_flags permissions = obj->permissions;
+	const edv_permission_flags permissions = obj->permissions;
 	gchar s[80];
 
 	/* Format permissions string */
@@ -667,6 +694,7 @@ static void EDVArchiverContentsSetCellPermissions(
 	    );
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, s
 	);
@@ -674,6 +702,7 @@ static void EDVArchiverContentsSetCellPermissions(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -692,12 +721,14 @@ static void EDVArchiverContentsSetCellOwner(
 	    owner_name = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, owner_name
 	);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -716,6 +747,7 @@ static void EDVArchiverContentsSetCellGroup(
 	    group_name = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, group_name
 	);
@@ -723,6 +755,7 @@ static void EDVArchiverContentsSetCellGroup(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -749,6 +782,7 @@ static void EDVArchiverContentsSetCellDateAccess(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -756,6 +790,7 @@ static void EDVArchiverContentsSetCellDateAccess(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -782,6 +817,7 @@ static void EDVArchiverContentsSetCellDateModified(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -789,6 +825,7 @@ static void EDVArchiverContentsSetCellDateModified(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -815,6 +852,7 @@ static void EDVArchiverContentsSetCellDateChanged(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -822,6 +860,7 @@ static void EDVArchiverContentsSetCellDateChanged(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -872,6 +911,7 @@ static void EDVArchiverContentsSetCellLocation(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	if(pixmap != NULL)
 	    gtk_clist_set_pixtext(
 		clist, row, column,
@@ -887,6 +927,7 @@ static void EDVArchiverContentsSetCellLocation(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 
 	g_free(path);
 	g_free(location);
@@ -902,18 +943,20 @@ static void EDVArchiverContentsSetCellLinkedTo(
 	const gint row, const gint column
 )
 {
-	/* Get link destination */
-	const gchar *link_dest = (obj->link_value != NULL) ?
-	    obj->link_value : "";
+	/* Get the link's target value */
+	const gchar *target = (obj->link_target != NULL) ?
+	    obj->link_target : "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, link_dest
+	    clist, row, column, target
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -952,6 +995,7 @@ static void EDVArchiverContentsSetCellDeviceType(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, num_str
 	);
@@ -959,6 +1003,7 @@ static void EDVArchiverContentsSetCellDeviceType(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -1012,11 +1057,14 @@ static void EDVArchiverContentsSetCellCompression(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	if(pixmap != NULL)
 	    gtk_clist_set_pixmap(clist, row, column, pixmap, mask);
 	else
 	    gtk_clist_set_text(clist, row, column, "");
 	gtk_clist_set_cell_style(clist, row, column, NULL);
+	gtk_clist_thaw(clist);
+
 	GDK_PIXMAP_UNREF(pixmap);
 	GDK_BITMAP_UNREF(mask);
 #else
@@ -1046,11 +1094,13 @@ static void EDVArchiverContentsSetCellCompression(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(clist, row, column, s);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 #endif
 }
 
@@ -1079,11 +1129,13 @@ static void EDVArchiverContentsSetCellMethod(
 	    method = STRDUP("");
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(clist, row, column, method);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 
 	g_free(method);
 }
@@ -1103,11 +1155,13 @@ static void EDVArchiverContentsSetCellCRC(
 	    crc = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(clist, row, column, crc);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -1128,10 +1182,12 @@ static void EDVArchiverContentsSetRow(
 	const gint row
 )
 {
-	gint i;
 	gboolean hide_dir_size, hide_link_size, hide_link_permissions;
+	gint i;
+	gulong block_size;
 	const gchar *date_format;
 	GList *glist;
+	edv_size_format size_format;
 	edv_date_relativity date_relativity;
 	edv_archiver_column_type column_type;
 	cfg_intlist_struct *column_types_intlist;
@@ -1145,6 +1201,10 @@ static void EDVArchiverContentsSetRow(
 	    return;
 
 	/* Get additional display options */
+	size_format = (edv_size_format)EDV_GET_I(
+	    EDV_CFG_PARM_SIZE_FORMAT
+	);
+	block_size = EDV_GET_UL(EDV_CFG_PARM_BLOCK_SIZE);
 	date_relativity = (edv_date_relativity)EDV_GET_I(
 	    EDV_CFG_PARM_DATE_RELATIVITY
 	);
@@ -1153,6 +1213,8 @@ static void EDVArchiverContentsSetRow(
 	hide_link_size = EDV_GET_B(EDV_CFG_PARM_ARCHIVER_CONTENTS_HIDE_LINK_SIZE);
 	hide_link_permissions = EDV_GET_B(EDV_CFG_PARM_ARCHIVER_CONTENTS_HIDE_LINK_PERMISSIONS);
 
+	gtk_clist_freeze(clist);
+
 	/* Iterate through each column */
 	for(i = 0, glist = column_types_intlist->list;
 	    glist != NULL;
@@ -1172,14 +1234,16 @@ static void EDVArchiverContentsSetRow(
 		EDVArchiverContentsSetCellSize(
 		    core, archiver, clist, obj,
 		    row, i,
-		    hide_dir_size, hide_link_size
+		    hide_dir_size, hide_link_size,
+		    size_format, block_size
 		);
 		break;
 	      case EDV_ARCHIVER_COLUMN_TYPE_COMPRESSED_SIZE:
 		EDVArchiverContentsSetCellCompressedSize(
 		    core, archiver, clist, obj,
 		    row, i,
-		    hide_dir_size, hide_link_size
+		    hide_dir_size, hide_link_size,
+		    size_format, block_size
 		);
 		break;
 	      case EDV_ARCHIVER_COLUMN_TYPE_TYPE:
@@ -1263,6 +1327,8 @@ static void EDVArchiverContentsSetRow(
 		break;
 	    }
 	}
+
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -1285,13 +1351,15 @@ static gint EDVArchiverContentsAppendObject(
 	edv_archive_object_struct *obj
 )
 {
-	const gint columns = MAX(clist->columns, 1);
+	const gint ncolumns = MAX(clist->columns, 1);
 	gint i, new_row;
 	gchar **strv;
 
+	gtk_clist_freeze(clist);
+
 	/* Allocate the row cell values */
-	strv = (gchar **)g_malloc0(columns * sizeof(gchar *));
-	for(i = 0; i < columns; i++)
+	strv = (gchar **)g_malloc(ncolumns * sizeof(gchar *));
+	for(i = 0; i < ncolumns; i++)
 	    strv[i] = "";
 
 	/* Append the new row */
@@ -1303,11 +1371,12 @@ static gint EDVArchiverContentsAppendObject(
 	/* Failed to appended row? */
 	if(new_row < 0)
 	{
+	    gtk_clist_thaw(clist);
 	    EDVArchObjectDelete(obj);
 	    return(-1);
 	}
 
-	/* Update the new row's values */
+	/* Set the new row's values */
 	EDVArchiverContentsSetRow(
 	    core, archiver, clist,
 	    obj,
@@ -1320,6 +1389,8 @@ static gint EDVArchiverContentsAppendObject(
 	    obj, EDVArchiverContentsItemDestroyCB
 	);
 
+	gtk_clist_thaw(clist);
+
 	return(new_row);
 }
 
@@ -1336,9 +1407,6 @@ static gint EDVArchiverContentsAppendObject(
  *	If update_status_bar is TRUE then the status bar will be
  *	updated during this operation.
  *
- *	If an error occures during this operation then
- *	get_listing_last_error will be updated with the error message.
- *
  *	All inputs assumed valid.
  */
 static void EDVArchiverContentsAppendListing(
@@ -1350,86 +1418,64 @@ static void EDVArchiverContentsAppendListing(
 	const gboolean update_status_bar
 )
 {
-	gint new_row;
-	GList *glist, *obj_list;
-	edv_archive_object_struct *obj;
-	stat_list_data_struct *d = STAT_LIST_DATA(g_malloc0(
-	    sizeof(stat_list_data_struct)
+	EDVArchiverNextObjectReturnData *d = EDV_ARCHIVER_NEXT_OBJECT_RETURN_DATA(g_malloc0(
+	    sizeof(EDVArchiverNextObjectReturnData)
 	));
-	if(d != NULL)
-	{
-	    d->archiver = archiver;
-	    d->last_progress_percent = -1;
-	}
+	if(d == NULL)
+	    return;
+
+	d->core = core;
+	d->archiver = archiver;
+	d->clist = clist;
+	d->last_progress_percent = -1;
 
 	/* Get the statistics of all the objects in the archive */
-	obj_list = EDVArchStatList(
+	gtk_clist_freeze(clist);
+	EDVArchStatListSequential(
 	    core,
 	    arch_path,
 	    NULL,			/* No paths list */
 	    filter,
 	    password,
-	    update_status_bar ?
-		EDVArchiverContentsStatProgressCB : NULL,
-	    d
+	    EDVArchiverContentsStatNextObjectReturnCB, d
 	);
+	gtk_clist_thaw(clist);
 
 	g_free(d);
-
-	/* Record any warnings when obtaining the list of objects */
-	get_listing_last_error = EDVArchStatGetError();
-
-	/* Iterate the list of objects in the archive */
-	for(glist = obj_list; glist != NULL; glist = g_list_next(glist))
-	{
-	    obj = EDV_ARCHIVE_OBJECT(glist->data);
-	    if(obj == NULL)
-		continue;
-
-	    /* Append the row and transfer the object to the new row,
-	     * the object should not be referenced again after this
-	     * point
-	     */
-	    new_row = EDVArchiverContentsAppendObject(
-		core, archiver, clist,
-		obj
-	    );
-	}
-
-	g_list_free(obj_list);
 }
 
 /*
- *	Returns the row index of the contents clist that contains an
- *	archive object structure who's full_path matches the given path.
+ *	Finds the row by path.
  *
- *	Can return -1 on failed match.
+ *	The path specifies the string describing the path to match
+ *	with a row's object path.
+ *
+ *	Returns the matched row or negative on error.
  */
 gint EDVArchiverContentsFindRowByPath(
 	edv_archiver_struct *archiver, const gchar *path
 )
 {
-	gint i;
+	gint i, n;
 	GtkCList *clist;
 	edv_archive_object_struct *obj;
 
 	if((archiver == NULL) || (path == NULL))
-	    return(-1);
+	    return(-2);
 
-	clist = (GtkCList *)archiver->contents_clist;
-	if(clist == NULL)
-	    return(-1);
+	clist = GTK_CLIST(archiver->contents_clist);
 
-	/* Iterate through all rows */
-	for(i = 0; i < clist->rows; i++)
+	n = clist->rows;
+	for(i = 0; i < n; i++)
 	{
-	    obj = EDV_ARCHIVE_OBJECT(
-		gtk_clist_get_row_data(clist, i)
-	    );
-	    if((obj != NULL) ? (obj->full_path == NULL) : TRUE)
+	    obj = EDV_ARCHIVE_OBJECT(gtk_clist_get_row_data(clist, i));
+	    if(obj == NULL)
 		continue;
 
-	    if(!strcmp(obj->full_path, path))
+	    if(obj->full_path == NULL)
+		continue;
+
+	    if(!strcmp((const char *)obj->full_path, (const char *)path))
 		return(i);
 	}
 
@@ -1449,8 +1495,7 @@ gint EDVArchiverContentsFindRowByPath(
  */
 void EDVArchiverContentsRealizeListing(edv_archiver_struct *archiver)
 {
-	gint i;
-	GtkWidget *w;
+	gint i, n;
 	GtkCList *clist;
 	edv_core_struct *core;
 	edv_archive_object_struct *obj;
@@ -1458,12 +1503,8 @@ void EDVArchiverContentsRealizeListing(edv_archiver_struct *archiver)
 	if(archiver == NULL)
 	    return;
 
-	w = archiver->contents_clist;
+	clist = GTK_CLIST(archiver->contents_clist);
 	core = archiver->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
-	clist = GTK_CLIST(w);
 
 	gtk_clist_freeze(clist);
 
@@ -1471,11 +1512,10 @@ void EDVArchiverContentsRealizeListing(edv_archiver_struct *archiver)
 	EDVArchiverContentsResetColumns(core, archiver, clist);
 
 	/* Update all the rows */
-	for(i = 0; i < clist->rows; i++)
+	n = clist->rows;
+	for(i = 0; i < n; i++)
 	{
-	    obj = EDV_ARCHIVE_OBJECT(
-		gtk_clist_get_row_data(clist, i)
-	    );
+	    obj = EDV_ARCHIVE_OBJECT(gtk_clist_get_row_data(clist, i));
 	    if(obj == NULL)
 		continue;
 
@@ -1492,13 +1532,17 @@ void EDVArchiverContentsRealizeListing(edv_archiver_struct *archiver)
 
 
 /*
- *	Returns a statically allocated string describing the last
- *	warning that occured while calling
- *	EDVArchiverContentsGetListing() or NULL if there was none.
+ *	Returns the last archive error message.
  */
 const gchar *EDVArchiverContentsGetError(edv_archiver_struct *archiver)
 {
-	return((archiver != NULL) ? get_listing_last_error : NULL);
+	edv_core_struct *core;
+
+	if(archiver == NULL)
+	    return(NULL);
+
+	core = archiver->core;
+	return(core->archive_last_error);
 }
 
 
@@ -1508,33 +1552,37 @@ const gchar *EDVArchiverContentsGetError(edv_archiver_struct *archiver)
  *
  *	If update_status_bar is TRUE then the status bar will be updated
  *	during this operation.
+ *
+ *	Returns 0 on success or nonzero if there was an error.
  */
 void EDVArchiverContentsGetListing(
 	edv_archiver_struct *archiver,
 	const gboolean update_status_bar
 )
 {
-	const gchar *cur_path, *filter, *password;
-	GtkWidget *w;
+	gchar *cur_path, *filter, *password;
 	GtkCList *clist;
 	edv_core_struct *core;
 
-	get_listing_last_error = NULL;
-
 	if(archiver == NULL)
 	    return;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	filter = archiver->contents_list_filter;
-	password = EDVArchiverCurrentPassword(archiver);
-	w = archiver->contents_clist;
+	clist = GTK_CLIST(archiver->contents_clist);
 	core = archiver->core;
-	if((cur_path == NULL) || (password == NULL) ||
-	   (w == NULL) || (core == NULL)
-	)
-	    return;
 
-	clist = GTK_CLIST(w);
+	/* Clear all the error messages on the core */
+	EDVClearErrorMessages(core);
+
+	cur_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	filter = STRDUP(archiver->contents_list_filter);
+	password = STRDUP(EDVArchiverCurrentPassword(archiver));
+	if((cur_path == NULL) || (password == NULL))
+	{
+	    g_free(cur_path);
+	    g_free(filter);
+	    g_free(password);
+	    return;
+	}
 
 	/* Report the initial progress? */
 	if(update_status_bar)
@@ -1581,6 +1629,10 @@ void EDVArchiverContentsGetListing(
 		archiver->status_bar, 0.0f, TRUE
 	    );
 	}
+
+	g_free(cur_path);
+	g_free(filter);
+	g_free(password);
 }
 
 /*
@@ -1593,10 +1645,7 @@ void EDVArchiverContentsClear(edv_archiver_struct *archiver)
 	if(archiver == NULL)
 	    return;
 
-	clist = (GtkCList *)archiver->contents_clist;
-	if(clist == NULL)
-	    return;
-
+	clist = GTK_CLIST(archiver->contents_clist);
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 	gtk_clist_thaw(clist);
@@ -1611,24 +1660,22 @@ void EDVArchiverContentsObjectAddedNotify(
 	const struct stat *lstat_buf
 )
 {
-	const gchar *cur_path;
-	GtkCList *clist;
+	gchar *cur_arch_path;
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
 
-	if((archiver == NULL) || (path == NULL) || (lstat_buf == NULL))
+	if((archiver == NULL) || STRISEMPTY(path) || (lstat_buf == NULL))
+	    return;
+
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
 	    return;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	clist = (GtkCList *)archiver->contents_clist;
 	core = archiver->core;
-	if((clist == NULL) || (core == NULL))
-	    return;
-
 	cfg_list = core->cfg_list;
 
-	/* Is the added path the same as the current location? */
-	if(!strcmp(path, cur_path))
+	/* Is the added path the same as the current archive? */
+	if(!strcmp((const char *)path, (const char *)cur_arch_path))
 	{
 	    /* Reget the listing */
 	    EDVArchiverContentsGetListing(
@@ -1636,6 +1683,8 @@ void EDVArchiverContentsObjectAddedNotify(
 		EDV_GET_B(EDV_CFG_PARM_LISTS_ANIMATE_UPDATES)
 	    );
 	}
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1646,27 +1695,27 @@ void EDVArchiverContentsObjectModifiedNotify(
 	const gchar *new_path, const struct stat *lstat_buf
 )
 {
-	const gchar *cur_path;
+	gchar *cur_arch_path;
 	GtkCList *clist;
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
 
-	if((archiver == NULL) || (path == NULL) || (lstat_buf == NULL))
+	if((archiver == NULL) || STRISEMPTY(path) || (lstat_buf == NULL))
 	    return;
 
 	if(new_path == NULL)
 	    new_path = path;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	clist = (GtkCList *)archiver->contents_clist;
-	core = archiver->core;
-	if((clist == NULL) || (core == NULL))
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
 	    return;
 
+	clist = GTK_CLIST(archiver->contents_clist);
+	core = archiver->core;
 	cfg_list = core->cfg_list;
 
-	/* Is the modified path the same as the current location? */
-	if(!strcmp(new_path, cur_path))
+	/* Is the modified path the current archive? */
+	if(!strcmp((const char *)new_path, (const char *)cur_arch_path))
 	{
 	    /* Reget the listing */
 	    EDVArchiverContentsGetListing(
@@ -1674,6 +1723,8 @@ void EDVArchiverContentsObjectModifiedNotify(
 		EDV_GET_B(EDV_CFG_PARM_LISTS_ANIMATE_UPDATES)
 	    );
 	}
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1683,28 +1734,23 @@ void EDVArchiverContentsObjectRemovedNotify(
 	edv_archiver_struct *archiver, const gchar *path
 )
 {
-	const gchar *cur_path;
-	GtkWidget *w;
-	GtkCList *clist;
-	edv_core_struct *core;
+	gchar *cur_arch_path;
 
 	if(archiver == NULL)
 	    return;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	w = archiver->contents_clist;
-	core = archiver->core;
-	if((cur_path == NULL) || (w == NULL) || (core == NULL))
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
 	    return;
 
-	clist = GTK_CLIST(w);
-
-	/* Is the removed path the same as the current location? */
-	if(!strcmp(path, cur_path))
+	/* Is the removed path the same as the current archive? */
+	if(!strcmp((const char *)path, (const char *)cur_arch_path))
 	{
 	    /* Clear the listing */
 	    EDVArchiverContentsClear(archiver);
 	}
+
+	g_free(cur_arch_path);
 }
 
 
@@ -1727,38 +1773,39 @@ void EDVArchiverContentsArchiveObjectAddedNotify(
 )
 {
 	gint row;
-	const gchar *cur_path;
-	GtkWidget *w;
+	gchar *cur_arch_path;
 	GtkCList *clist;
 	edv_core_struct *core;
 
-	if((archiver == NULL) || (arch_path == NULL) ||
-	   (path == NULL) || (obj == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path) || (obj == NULL)
 	)
 	    return;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	w = archiver->contents_clist;
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	clist = GTK_CLIST(archiver->contents_clist);
 	core = archiver->core;
-	if((cur_path == NULL) || (w == NULL) || (core == NULL))
-	    return;
 
-	clist = GTK_CLIST(w);
-
-	/* Skip if the archives are different */
-	if(strcmp(cur_path, arch_path))
+	/* Not our archive? */
+	if(strcmp((const char *)arch_path, (const char *)cur_arch_path))
+	{
+	    g_free(cur_arch_path);
 	    return;
+	}
 
 	/* Check if the path of the added archive object describes an
 	 * archive object that is already in the list
 	 */
 	row = EDVArchiverContentsFindRowByPath(archiver, path);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
 	{
-	    /* Update the existing object in the list */
-	    EDVArchObjectDelete(EDV_ARCHIVE_OBJECT(
-		gtk_clist_get_row_data(clist, row)
-	    ));
+	    /* Copy the object and set it as the matched row's
+	     * data then update the matched row's values
+	     */
+	    gtk_clist_freeze(clist);
 	    gtk_clist_set_row_data_full(
 		clist, row,
 		EDVArchObjectCopy(obj), EDVArchiverContentsItemDestroyCB
@@ -1768,6 +1815,7 @@ void EDVArchiverContentsArchiveObjectAddedNotify(
 		obj,
 		row
 	    );
+	    gtk_clist_thaw(clist);
 	}
 	else
 	{
@@ -1779,6 +1827,8 @@ void EDVArchiverContentsArchiveObjectAddedNotify(
 		    new_obj
 		);
 	}
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1802,41 +1852,42 @@ void EDVArchiverContentsArchiveObjectModifiedNotify(
 )
 {
 	gint row;
-	const gchar *cur_path;
-	GtkWidget *w;
+	gchar *cur_arch_path;
 	GtkCList *clist;
 	edv_core_struct *core;
 
-	if((archiver == NULL) || (arch_path == NULL) ||
-	   (path == NULL) || (obj == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path) || (obj == NULL)
 	)
 	    return;
 
 	if(new_path == NULL)
 	    new_path = path;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	w = archiver->contents_clist;
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	clist = GTK_CLIST(archiver->contents_clist);
 	core = archiver->core;
-	if((cur_path == NULL) || (w == NULL) || (core == NULL))
-	    return;
 
-	clist = GTK_CLIST(w);
-
-	/* Skip if the archives are different */
-	if(strcmp(cur_path, arch_path))
+	/* Not our archive? */
+	if(strcmp((const char *)arch_path, (const char *)cur_arch_path))
+	{
+	    g_free(cur_arch_path);
 	    return;
+	}
 
 	/* Check if the old path of the modified archive object is in
 	 * the listing
 	 */
 	row = EDVArchiverContentsFindRowByPath(archiver, path);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
 	{
-	    /* Update the existing object in the list */
-	    EDVArchObjectDelete(EDV_ARCHIVE_OBJECT(
-		gtk_clist_get_row_data(clist, row)
-	    ));
+	    /* Copy the object and set it as the matched row's
+	     * data then update the matched row's values
+	     */
+	    gtk_clist_freeze(clist);
 	    gtk_clist_set_row_data_full(
 		clist, row,
 		EDVArchObjectCopy(obj), EDVArchiverContentsItemDestroyCB
@@ -1846,7 +1897,10 @@ void EDVArchiverContentsArchiveObjectModifiedNotify(
 		obj,
 		row
 	    );
+	    gtk_clist_thaw(clist);
 	}
+
+	g_free(cur_arch_path);
 }
 
 /*
@@ -1863,35 +1917,42 @@ void EDVArchiverContentsArchiveObjectRemovedNotify(
 )
 {
 	gint row;
-	const gchar *cur_path;
-	GtkWidget *w;
+	gchar *cur_arch_path;
 	GtkCList *clist;
 	edv_core_struct *core;
 
-	if((archiver == NULL) || (arch_path == NULL) ||
-	   (path == NULL)
+	if((archiver == NULL) || STRISEMPTY(arch_path) ||
+	   STRISEMPTY(path)
 	)
 	    return;
 
-	cur_path = EDVArchiverCurrentLocation(archiver);
-	w = archiver->contents_clist;
+	cur_arch_path = STRDUP(EDVArchiverCurrentLocation(archiver));
+	if(cur_arch_path == NULL)
+	    return;
+
+	clist = GTK_CLIST(archiver->contents_clist);
 	core = archiver->core;
-	if((cur_path == NULL) || (w == NULL) || (core == NULL))
+
+	/* Not our archive? */
+	if(strcmp((const char *)arch_path, (const char *)cur_arch_path))
+	{
+	    g_free(cur_arch_path);
 	    return;
+	}
 
-	clist = GTK_CLIST(w);
-
-	/* Skip if the archives are different */
-	if(strcmp(cur_path, arch_path))
-	    return;
-
-	/* Check if the removed path of the modified archive object is
-	 * in the listing
+	/* Remove all the rows who's object's path matches the removed
+	 * path
 	 */
 	row = EDVArchiverContentsFindRowByPath(archiver, path);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
 	{
-	    /* Remove this object */
-	    gtk_clist_remove(clist, row);
+	    gtk_clist_freeze(clist);
+	    do {
+		gtk_clist_remove(clist, row);
+		row = EDVArchiverContentsFindRowByPath(archiver, path);
+	    } while(row > -1);
+	    gtk_clist_thaw(clist);
 	}
+
+	g_free(cur_arch_path);
 }
