@@ -13,11 +13,13 @@
 #include "edv_mount_bar.h"
 #include "edv_find_bar.h"
 #include "edv_status_bar.h"
-#include "obj_op_dlg.h"
 
 
 typedef struct _edv_browser_struct	edv_browser_struct;
 #define EDV_BROWSER(p)			((edv_browser_struct *)(p))
+typedef struct _edv_browser_cmd_struct	edv_browser_cmd_struct;
+#define EDV_BROWSER_CMD(p)		((edv_browser_cmd_struct *)(p))
+#define EDV_BROWSER_CMD_DATA_KEY	"EDV/BROWSER/CMD"
 typedef struct _edv_browser_opid_struct	edv_browser_opid_struct;
 #define EDV_BROWSER_OPID(p)		((edv_browser_opid_struct *)(p))
 
@@ -66,6 +68,19 @@ typedef enum {
 #define EDV_BROWSER_TOTAL_CELL_STYLES		7
 
 
+/*
+ *	File Browser Command:
+ */
+struct _edv_browser_cmd_struct {
+
+	edv_browser_struct	*browser;
+	gchar		*label,
+			*command,
+			*description,
+			*ext_data;
+};
+
+
 /*
  *	File Browser Operations:
  */
@@ -217,8 +232,8 @@ struct _edv_browser_struct {
 
 	GtkWidget	*toplevel;
 	gboolean	processing;
-	gint		busy_count,
-			freeze_count;
+	gint		freeze_count,
+			busy_count;
 	edv_core_struct	*core;
 
 	GList		*opid_list;
@@ -260,7 +275,8 @@ struct _edv_browser_struct {
 	edv_findbar_struct	*find_bar;
 
 	/* Menus and Menu Items */
-	GtkWidget	*open_mi,
+	GtkWidget	*file_menu,
+			*open_mi,
 			*open_to_mi,
 			*open_to_menu,
 			*open_with_mi,
@@ -278,6 +294,7 @@ struct _edv_browser_struct {
 			*run_mi,
 			*close_mi,
 
+			*edit_menu,
 			*move_mi,
 			*copy_mi,
 			*link_mi,
@@ -295,13 +312,7 @@ struct _edv_browser_struct {
 			*find_mi,
 			*properties_mi,
 
-			*mount_mi,
-			*eject_mi,
-			*device_check_mi,
-			*device_tools_mi,
-			*device_format_mi,
-			*devices_mi,
-
+			*view_menu,
 			*refresh_mi,
 			*refresh_all_mi,
 			*show_tool_bar_micheck,
@@ -315,17 +326,28 @@ struct _edv_browser_struct {
 			*directory_tree_origin_mi,
 			*mime_types_mi,
 
+			*device_menu,
+			*mount_mi,
+			*eject_mi,
+			*device_check_mi,
+			*device_tools_mi,
+			*device_format_mi,
+			*devices_mi,
+
+			*settings_menu,
 			*write_protect_mi,
 			*delete_method_recycle_micheck,
 			*delete_method_purge_micheck,
 			*options_mi,
 			*customize_mi,
 
+			*windows_menu,
 			*window_new_browser_mi,
 			*window_new_imbr_mi,
 			*window_new_archiver_mi,
 			*window_recycle_bin_mi,
 
+			*help_menu,
 			*help_contents_mi,
 			*help_file_browser_mi,
 			*help_keys_list_mi,
@@ -384,8 +406,6 @@ struct _edv_browser_struct {
 	/* Special icons */
 	GdkPixmap	*folder_parent_pixmap;
 	GdkBitmap	*folder_parent_mask;
-	GdkPixmap	*folder_noaccess_pixmap;
-	GdkBitmap	*folder_noaccess_mask;
 
 	/* Directory toplevel origin directory */
 	gchar		*directory_ctree_origin_path;
@@ -429,10 +449,6 @@ struct _edv_browser_struct {
 	/* Status Bar */
 	edv_status_bar_struct	*status_bar;
 
-
-	/* Object Operations Dialog */
-	edv_obj_op_dlg_struct	*obj_op_dlg;
-
 };
 
 
@@ -454,17 +470,24 @@ extern void EDVBrowserSetTitle(
 extern gchar *EDVBrowserCurrentLocation(edv_browser_struct *browser);
 extern void EDVBrowserSetLocation(
 	edv_browser_struct *browser,
-	const gchar *path, gboolean record_history
+	const gchar *path, const gboolean record_history
 );
 extern void EDVBrowserUpdateLocationIcon(
 	edv_browser_struct *browser, const gchar *path
 );
 
+/* Selected Objects & Paths */
+extern GList *EDVBrowserGetSelectedObjectsList(
+	edv_browser_struct *browser, const gboolean copy
+);
+extern GList *EDVBrowserGetSelectedPathsList(edv_browser_struct *browser);
+
 /* Regenerating Menus, Bars, etc */
 extern void EDVBrowserAccelkeysRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserListStylesRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserToolBarRegenerate(edv_browser_struct *browser);
-extern void EDVBrowserNewObjectMenuRegenerate(edv_browser_struct *browser);
+extern void EDVBrowserFileMenuItemsRegenerate(edv_browser_struct *browser);
+extern void EDVBrowserNewObjectMenuItemsRegenerate(edv_browser_struct *browser);
 extern void EDVBrowserOpenToMenuRegenerate(
 	edv_browser_struct *browser, edv_mime_type_struct *m
 );
