@@ -9,6 +9,7 @@
 
 #include "guiutils.h"
 #include "cdialog.h"
+#include "progressdialog.h"
 
 #include "edv_types.h"
 #include "cfg.h"
@@ -26,6 +27,8 @@
 #include "browser_contents_list.h"
 #include "browser_dnd.h"
 #include "endeavour2.h"
+#include "edv_obj_create.h"
+#include "edv_open.h"
 #include "edv_op.h"
 #include "edv_cb.h"
 #include "edv_utils.h"
@@ -40,6 +43,7 @@ typedef struct _browser_dir_tree_cb_data_struct	browser_dir_tree_cb_data_struct;
 
 
 /* GTK+ Signal Callbacks */
+void EDVBrowserCmdDestroyCB(gpointer data);
 void EDVBrowserContentsItemDestroyCB(gpointer data);
 void EDVBrowserDirTreeItemDestroyCB(gpointer data);
 
@@ -75,7 +79,7 @@ void EDVBrowserHandleChildDetachedCB(
 );
 
 static gint EDVBrowserTreeSelectRowIdleCB(gpointer data);
-static gint EDVBrowserTreeExpandOptimizeIdleCB(gpointer data);
+static gint EDVBrowserTreeExpandScrollOptimizeIdleCB(gpointer data);
 void EDVBrowserTreeSelectRowCB(
 	GtkCTree *ctree, GtkCTreeNode *node, gint column,
 	gpointer data
@@ -108,6 +112,7 @@ void EDVBrowserUnselectRowCB(
 
 void EDVBrowserComboActivateCB(GtkWidget *widget, gpointer data);
 
+
 /* Menu Item Callbacks */
 void EDVBrowserMenuItemCB(GtkWidget *widget, gpointer data);
 gint EDVBrowserMenuItemEnterCB(
@@ -116,10 +121,20 @@ gint EDVBrowserMenuItemEnterCB(
 gint EDVBrowserMenuItemLeaveCB(
 	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
 );
+void EDVBrowserMenuItemCmdObjectOpCB(GtkWidget *widget, gpointer data);
+void EDVBrowserMenuItemCmdNewObjectCB(GtkWidget *widget, gpointer data);
+gint EDVBrowserMenuItemCmdEnterCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+);
+gint EDVBrowserMenuItemCmdLeaveCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+);
+
 
 /* Paths List Get Callback */
 GList *EDVBrowserGetSelectedPathsListCB(gpointer data);
 
+
 /* Goto Directory Callback */
 void EDVBrowserGotoDirectoryCB(gpointer data, const gchar *path);
 
@@ -237,6 +252,22 @@ struct _browser_dir_tree_cb_data_struct {
 };
 
 
+/*
+ *	Browser command "destroy" signal callback.
+ */
+void EDVBrowserCmdDestroyCB(gpointer data)
+{
+	edv_browser_cmd_struct *d = EDV_BROWSER_CMD(data);
+	if(d == NULL)
+	    return;
+
+	g_free(d->label);
+	g_free(d->command);
+	g_free(d->description);
+	g_free(d->ext_data);
+	g_free(d);
+}
+
 /*
  *	Browser contents GtkCList item "destroy" signal callback.
  */
@@ -346,10 +377,10 @@ gint EDVBrowserDeleteEventCB(
 {
 	edv_browser_struct *browser = EDV_BROWSER(data);
 	if(browser == NULL)
-	    return(TRUE);
+	    return(FALSE);
 
-	if(browser->processing)
-	    return(TRUE);
+	if(browser->processing || (browser->freeze_count > 0))
+	    return(FALSE);
 
 	EDVBrowserOPClose(browser);
 
@@ -378,22 +409,20 @@ gint EDVBrowserKeyEventCB(
 	if(browser->processing)
 	    return(status);
 
-	core = browser->core;
-	if(core == NULL)
-	    return(status);
-
 	w = browser->toplevel;
-	focus_widget = (w != NULL) ? GTK_WINDOW(w)->focus_widget : NULL;
+	focus_widget = GTK_WINDOW(w)->focus_widget;
+	core = browser->core;
 	cfg_list = core->cfg_list;
+
 	etype = key->type;
 	is_press = (etype == GDK_KEY_PRESS) ? TRUE : FALSE;
 	keyval = key->keyval;
 	state = key->state;
 
 /* Stop emit of signal */
-#define DO_STOP_KEY_SIGNAL_EMIT	{		\
+#define STOP_SIGNAL_EMISSION(_w_) {		\
  gtk_signal_emit_stop_by_name(			\
-  GTK_OBJECT(widget),				\
+  GTK_OBJECT(_w_),				\
   is_press ?					\
    "key_press_event" : "key_release_event"	\
  );						\
@@ -420,7 +449,7 @@ gint EDVBrowserKeyEventCB(
 		    if(opid != NULL)
 			EDVBrowserOPCB(NULL, -1, opid);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		return(status);
 	    }
@@ -441,28 +470,31 @@ gint EDVBrowserKeyEventCB(
 		node = gtk_ctree_node_nth(ctree, clist->focus_row);
 		if((node != NULL) && is_press)
 		{
-		    gboolean already_selected = FALSE;
+		    gboolean is_selected;
+		    GList *glist;
 
 		    /* Check if this node is already selected */
-		    GList *glist = clist->selection;
-		    while(glist != NULL)
+		    is_selected = FALSE;
+		    for(glist = clist->selection;
+			glist != NULL;
+			glist = g_list_next(glist)
+		    )
 		    {
 			if(node == (GtkCTreeNode *)glist->data)
 			{
-			    already_selected = TRUE;
+			    is_selected = TRUE;
 			    break;
 			}
-			glist = g_list_next(glist);
 		    }
 
 		    gtk_clist_freeze(clist);
-		    if(already_selected)
+		    if(is_selected)
 			gtk_ctree_unselect(ctree, node);
 		    else
 			gtk_ctree_select(ctree, node);
 		    gtk_clist_thaw(clist);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		break;
 	    }
@@ -480,28 +512,31 @@ gint EDVBrowserKeyEventCB(
 		row = clist->focus_row;
 		if((row >= 0) && (row < clist->rows) && is_press)
 		{
-		    gboolean already_selected = FALSE;
+		    gboolean is_selected;
+		    GList *glist;
 
 		    /* Check if this row is already selected */
-		    GList *glist = clist->selection;
-		    while(glist != NULL)
+		    is_selected = FALSE;
+		    for(glist = clist->selection;
+			glist != NULL;
+			glist = g_list_next(glist)
+		    )
 		    {
 			if(row == (gint)glist->data)
 			{
-			    already_selected = TRUE;
+			    is_selected = TRUE;
 			    break;
 			}
-			glist = g_list_next(glist);
 		    }
 
 		    gtk_clist_freeze(clist);
-		    if(already_selected)
+		    if(is_selected)
 			gtk_clist_unselect_row(clist, row, 0);
 		    else
 			gtk_clist_select_row(clist, row, 0);
 		    gtk_clist_thaw(clist);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		break;
 
@@ -511,44 +546,37 @@ gint EDVBrowserKeyEventCB(
 		 * item who's name starts with the letter of the key
 		 * that was pressed
 		 */
-		if(isalnum((int)keyval) && is_press &&
-		   !(state & GDK_CONTROL_MASK) &&
-		   !(state & GDK_SHIFT_MASK)
-		)
+		if(isalnum((int)keyval) && is_press)
 		{
-		    /* Get the column that is displaying the object
-		     * name as column_type_name
-		     */
-		    gint column_type_name = -1;
-		    const cfg_intlist_struct *intlist = EDV_GET_INTLIST(
+                    const gboolean backwards = (state & GDK_MOD1_MASK) ? TRUE : FALSE;
+                    const gint start_row = (backwards) ?
+                        (clist->rows - 1) : 0;
+
+		    /* Find the column that is set to display the name */
+		    cfg_intlist_struct *intlist = EDV_GET_INTLIST(
 			EDV_CFG_PARM_BROWSER_CONTENTS_COLUMN
 		    );
-		    if(intlist != NULL)
-		    {
-			/* Iterate through column type intlist */
-			gint i = 0;
-			GList *glist = intlist->list;
-			while(glist != NULL)
-			{
-			    if((edv_browser_column_type)glist->data ==
-				EDV_BROWSER_COLUMN_TYPE_NAME
-			    )
-			    {
-				column_type_name = i;
-				break;
-			    }
-			    i++;
-			    glist = g_list_next(glist);
-			}
-		    }
+		    const gint column_type_name = (intlist != NULL) ?
+			g_list_index(
+			    intlist->list,
+			    (gpointer)EDV_BROWSER_COLUMN_TYPE_NAME
+			) : 0;
 
+		    /* Seek to the row who's text matches this
+		     * key event's character
+		     */
 		    gtk_clist_freeze(clist);
 		    EDVCListSeekCharacter(
-			clist, column_type_name, 0, keyval
+			clist,
+			column_type_name,	/* Column */
+			start_row,
+			backwards,
+			0,			/* Text index */
+			keyval			/* Character */
 		    );
 		    gtk_clist_thaw(clist);
 
-		    DO_STOP_KEY_SIGNAL_EMIT
+		    STOP_SIGNAL_EMISSION(widget);
 		    status = TRUE;
 		}
 		break;
@@ -556,7 +584,7 @@ gint EDVBrowserKeyEventCB(
 	}
 
 	return(status);
-#undef DO_STOP_KEY_SIGNAL_EMIT
+#undef STOP_SIGNAL_EMISSION
 }
 
 /*
@@ -579,14 +607,8 @@ gint EDVBrowserButtonPressEventCB(
 	    return(status);
 
 	core = browser->core;
-	if(core == NULL)
-	    return(status);
-
 	cfg_list = core->cfg_list;
-
-	/* Get event type */
 	etype = button->type;
-
 	state = button->state;
 
 	/* Check which widget this signal is for */
@@ -664,7 +686,7 @@ gint EDVBrowserButtonPressEventCB(
 		if(etype == GDK_BUTTON_PRESS)
 		{
 		    if(node != NULL)
-			EDVBrowserDirTreeDoFPromptRename(browser, node);
+			EDVBrowserDirTreePromptRename(browser, node);
 		}
 		status = TRUE;
 		break;
@@ -744,7 +766,7 @@ gint EDVBrowserButtonPressEventCB(
 		if(etype == GDK_BUTTON_PRESS)
 		{
 		    if((row >= 0) && (row < clist->rows))
-			EDVBrowserContentsDoFPromptRename(
+			EDVBrowserContentsPromptRename(
 			    browser, row, column
 			);
 		}
@@ -826,14 +848,8 @@ static gint EDVBrowserTreeSelectRowIdleCB(gpointer data)
 	    return(FALSE);
 
 	browser = d->browser;
-	if(browser == NULL)
-	{
-	    g_free(d);
-	    return(FALSE);
-	}
-
 	toplevel = browser->toplevel;
-	ctree = (GtkCTree *)browser->directory_ctree;
+	ctree = GTK_CTREE(browser->directory_ctree);
 	clist = GTK_CLIST(ctree);
 	core = browser->core;
 	cfg_list = core->cfg_list;
@@ -850,82 +866,14 @@ static gint EDVBrowserTreeSelectRowIdleCB(gpointer data)
 	if(gtk_ctree_node_is_visible(ctree, node)
 	    != GTK_VISIBILITY_FULL
 	)
-	{
-#if 0
-	    GtkCTreeRow *row_ptr = GTK_CTREE_ROW(node);
-	    GtkAdjustment *adj = clist->hadjustment;
-	    GtkStyle *style = gtk_widget_get_style(GTK_WIDGET(clist));
-	    GdkFont *font = (style != NULL) ? style->font : NULL;
-#endif
-
 	    gtk_ctree_node_moveto(
 		ctree,
 		node, -1,
-		0.5f, 0.0f		/* Row align, column align */
+		0.5f, 0.0f		/* Row, column */
 	    );
 
-#if 0
-	    if((row_ptr != NULL) && (adj != NULL))
-	    {
-		gint x_min, x_max;
-		const gint level = MAX(row_ptr->level - 1, 0);
-		guint8 spacing;
-		gchar *text;
-		GdkBitmap *mask, *mask_opened;
-		GdkPixmap *pixmap, *pixmap_opened;
-		gboolean is_leaf, expanded;
-
-		gtk_ctree_get_node_info(
-		    ctree, node,
-		    &text, &spacing,
-		    &pixmap, &mask,
-		    &pixmap_opened, &mask_opened,
-		    &is_leaf, &expanded
-		);
-
-		if(font == NULL)
-		{
-		    style = gtk_ctree_node_get_row_style(ctree, node);
-		    font = (style != NULL) ? style->font : NULL;
-		}
-
-		if(pixmap != NULL)
-		{
-		    gint width, height;
-		    gdk_window_get_size(pixmap, &width, &height);
-		    x_min = level * width;
-		    x_max = x_min + width;
-		}
-		else
-		{
-		    x_min = 0;
-		    x_max = x_min;
-		}
-		x_max += spacing;
-		if((font != NULL) && !STRISEMPTY(text))
-		{
-		    GdkTextBounds b;
-		    gdk_string_bounds(font, text, &b);
-		    x_max += b.width;
-		}
-
-#if 0
-g_print("%i  %i %i\n", level, x_min, x_max);
-		if(x_min < adj->value)
-		{
-g_print("%i %i\n", x_min, (gint)adj->value);
-		}
-		else if(x_max > (adj->value + adj->page_size))
-		{
-g_print("%i %i\n", x_max, (gint)(adj->value + adj->page_size));
-		}
-#endif
-	    }
-#endif
-	}
-
-	/* Update the current selected node and reset the last selected
-	 * object on the contents list
+	/* Update the current selected node and reset the last
+	 * selected object on the contents list
 	 */
 	browser->directory_ctree_selected_node = node;
 	browser->contents_clist_selected_row = -1;
@@ -1027,7 +975,7 @@ g_print("%i %i\n", x_max, (gint)(adj->value + adj->page_size));
 /*
  *	Browser GtkCTree expand optimize idle callback.
  */
-static gint EDVBrowserTreeExpandOptimizeIdleCB(gpointer data)
+static gint EDVBrowserTreeExpandScrollOptimizeIdleCB(gpointer data)
 {
 	GtkCTreeNode *node;
 	GtkCTree *ctree;
@@ -1037,13 +985,7 @@ static gint EDVBrowserTreeExpandOptimizeIdleCB(gpointer data)
 	    return(FALSE);
 
 	browser = d->browser;
-	if(browser == NULL)
-	{
-	    g_free(d);
-	    return(FALSE);
-	}
-
-	ctree = (GtkCTree *)browser->directory_ctree;
+	ctree = GTK_CTREE(browser->directory_ctree);
 	node = d->node;
 
 	/* Scroll to the optimul viewing position */
@@ -1076,9 +1018,6 @@ void EDVBrowserTreeSelectRowCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Check which widget this signal is for */
@@ -1128,7 +1067,6 @@ void EDVBrowserTreeUnselectRowCB(
 	{
 	    gchar *s;
 	    GtkCList *clist = GTK_CLIST(ctree);
-	    edv_core_struct *core = browser->core;
 
 	    /* Get the total number of objects selected */
 	    const gint nselected = g_list_length(clist->selection);
@@ -1151,7 +1089,7 @@ void EDVBrowserTreeUnselectRowCB(
 	    if(nselected > 0)
 		s = g_strdup_printf(
 		    "%s %s selected",
-		    EDVGetObjectSizeStr(core, nselected),
+		    EDVSizeStrDelim(nselected),
 		    (nselected == 1) ? "object" : "objects"
 		);
 	    else
@@ -1182,9 +1120,6 @@ void EDVBrowserTreeExpandCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Check which widget this signal is for */
@@ -1195,8 +1130,14 @@ void EDVBrowserTreeExpandCB(
 	    GUIBlockInput(browser->toplevel, TRUE);
 	    browser->processing = TRUE;
 
-	    /* Expand node and get child nodes */
-	    EDVBrowserDirTreeExpand(browser, node, TRUE);
+	    /* Expand this node and get a listing of child nodes that
+	     * recurse no more than 2 levels
+	     */
+	    EDVBrowserDirTreeExpand(
+		browser,
+		node,
+		TRUE				/* Update progress */
+	    );
 
 	    /* Scroll to optimize viewing of expand nodes? */
 	    if(EDV_GET_B(EDV_CFG_PARM_TREE_EXPAND_OPTIMIZE_POS))
@@ -1213,7 +1154,7 @@ void EDVBrowserTreeExpandCB(
 		    d->node = node;
 		    browser->directory_ctree_expand_optimize_idleid = gtk_idle_add_priority(
 			G_PRIORITY_LOW,
-			EDVBrowserTreeExpandOptimizeIdleCB, d
+			EDVBrowserTreeExpandScrollOptimizeIdleCB, d
 		    );
 		}
 	    }
@@ -1244,23 +1185,54 @@ void EDVBrowserTreeCollapseCB(
 	/* Directory Tree */
 	if(GTK_WIDGET(ctree) == browser->directory_ctree)
 	{
-#if 0
+	    GtkCList *clist = GTK_CLIST(ctree);
+	    edv_object_struct *obj;
+
 	    EDVBrowserSetBusy(browser, TRUE);
 	    GUIBlockInput(browser->toplevel, TRUE);
 	    browser->processing = TRUE;
 
-	    gtk_clist_freeze(GTK_CLIST(ctree));
-
-
-
-	    gtk_clist_thaw(GTK_CLIST(ctree));
+	    /* Remove all the grandchild nodes of this node in order
+	     * to save memory
+	     */
+	    EDVBrowserDirTreeRemoveGrandChildrenNodes(browser, node);
 
 	    browser->processing = FALSE;
 	    GUIBlockInput(browser->toplevel, FALSE);
 	    EDVBrowserSetBusy(browser, FALSE);
-#endif
 
 	    EDVBrowserUpdateMenus(browser);
+
+	    /* If the current location is a child of this collapsed
+	     * node then change the current location to this node
+	     */
+	    obj = EDV_OBJECT(gtk_ctree_node_get_row_data(ctree, node));
+	    if(obj != NULL)
+	    {
+		if(EDVIsParentPath(
+		    obj->full_path,
+		    EDVBrowserCurrentLocation(browser)
+		))
+		{
+		    GList *glist;
+
+		    /* If this node is not selected then select it */
+		    for(glist = clist->selection;
+			glist != NULL;
+			glist = g_list_next(glist)
+		    )
+		    {
+			if((GtkCTreeNode *)glist->data == node)
+			    break;
+		    }
+		    if(glist == NULL)
+		    {
+			gtk_clist_freeze(clist);
+			gtk_ctree_select(ctree, node);
+			gtk_clist_thaw(clist);
+		    }
+		}
+	    }
 	}
 }
 
@@ -1282,9 +1254,6 @@ void EDVBrowserResizeColumnCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Check which widget this signal is for */
@@ -1374,9 +1343,6 @@ void EDVBrowserClickColumnCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Check which widget this signal is for */
@@ -1558,10 +1524,8 @@ void EDVBrowserSelectRowCB(
 			type_str = "File";
 			size_str = g_strdup_printf(
 			    " (%s %s)",
-			    EDVGetObjectSizeStr(
-				core, obj->size
-			    ),
-			    (obj->size == 1) ? "byte" : "bytes"
+			    EDVSizeStrDelim(obj->size),
+			    (obj->size == 1l) ? "byte" : "bytes"
 			);
 			break;
 		      case EDV_OBJECT_TYPE_DIRECTORY:
@@ -1598,7 +1562,7 @@ void EDVBrowserSelectRowCB(
 		    if(nselected > 1)
 			s = g_strdup_printf(
 			    "%s objects selected",
-			    EDVGetObjectSizeStr(core, nselected)
+			    EDVSizeStrDelim(nselected)
 			);
 		    else if(!strcmp(obj->name, ".."))
 			s = g_strdup_printf(
@@ -1692,9 +1656,9 @@ void EDVBrowserUnselectRowCB(
 	    /* Update the status bar message */
 	    if(nselected > 0)
 		s = g_strdup_printf(
-		    "%s object%s selected",
-		    EDVGetObjectSizeStr(core, nselected),
-		    (nselected == 1) ? "" : "s"
+		    "%s %s selected",
+		    EDVSizeStrDelim(nselected),
+		    (nselected == 1) ? "object" : "objects"
 		);
 	    else
 		s = STRDUP("No objects selected");
@@ -1730,9 +1694,8 @@ void EDVBrowserComboActivateCB(GtkWidget *widget, gpointer data)
 	    );
 	    if(path != NULL)
 	    {
+		gboolean set_origin_path;
 		GtkWidget *toplevel = browser->toplevel;
-		GtkCTree *ctree = GTK_CTREE(browser->directory_ctree);
-		GtkCList *clist = GTK_CLIST(ctree);
 
 		EDVBrowserSetBusy(browser, TRUE);
 		GUIBlockInput(toplevel, TRUE);
@@ -1747,11 +1710,18 @@ void EDVBrowserComboActivateCB(GtkWidget *widget, gpointer data)
 		 * the current Directory Tree Origin then the Directory
 		 * Tree Origin needs to be updated first
 		 */
-		if(!EDVBrowserIsPathFromDirTreeOrigin(browser, path))
+		if(EDVBrowserIsPathFromDirTreeOrigin(browser, path))
+		{
+		    set_origin_path = FALSE;
+		}
+		else
+		{
 		    EDVBrowserDirTreeSetOriginPath(browser, path);
+		    set_origin_path = TRUE;
+		}
 
-		/* Update location combo to record the new location in
-		 * the location history
+		/* Update the location combo to record the new location
+		 * in the locations history
 		 *
 		 * The combo will be updated again (without recording
 		 * history) if the new path is valid and selected in
@@ -1760,23 +1730,19 @@ void EDVBrowserComboActivateCB(GtkWidget *widget, gpointer data)
 		 */
 		EDVBrowserSetLocation(browser, path, TRUE);
 
-		/* Update selected node on directory ctree */
-		if(clist != NULL)
-		{
-		    /* Attempt to select the node who's object's path
-		     * matches path (recurse and expand as needed)
-		     *
-		     * The ctree will be frozen and thawed by the
-		     * callbacks each time it is expanded so we do not
-		     * need to freeze or thaw it here
-		     *
-		     * Also, the location prompt will be updated again
-		     * (but its history will not be re-recorded) when the
-		     * the node is selected
-		     */
+		/* Attempt to select the node who's object's path
+		 * matches path (recurse and expand as needed)
+		 *
+		 * The ctree will be frozen and thawed by the
+		 * callbacks each time it is expanded so we do not
+		 * need to freeze or thaw it here
+		 *
+		 * Also, the location prompt will be updated again
+		 * (but its history will not be re-recorded) when the
+		 * the node is selected
+		 */
+		if(!set_origin_path)
 		    EDVBrowserDirTreeSelectPath(browser, path);
-		}
-
 
 		GUIBlockInput(toplevel, FALSE);
 		EDVBrowserSetBusy(browser, FALSE);
@@ -1788,7 +1754,7 @@ void EDVBrowserComboActivateCB(GtkWidget *widget, gpointer data)
 
 
 /*
- *	Menu item activate callback.
+ *	Menu item "activate" callback.
  *
  *	The data must be a edv_browser_opid_struct *.
  */
@@ -1823,61 +1789,303 @@ gint EDVBrowserMenuItemLeaveCB(
 	return(TRUE);
 }
 
+/*
+ *	Menu item command object op "activate" signal callback.
+ */
+void EDVBrowserMenuItemCmdObjectOpCB(GtkWidget *widget, gpointer data)
+{
+	const gchar *command;
+	GtkWidget *toplevel;
+	GList *paths_list;
+	edv_browser_struct *browser;
+	edv_core_struct *core;
+	edv_browser_cmd_struct *d = EDV_BROWSER_CMD(data);
+	if(d == NULL)
+	    return;
+
+	browser = d->browser;
+	toplevel = browser->toplevel;
+	core = browser->core;
+
+	if(browser->processing)
+	    return;
+
+	EDVBrowserSetBusy(browser, TRUE);
+
+	/* Get the command for this menu item */
+	command = d->command;
+	if(STRISEMPTY(command))
+	{
+	    gchar *msg = g_strdup_printf(
+"The custom menu item:\n\
+\n\
+    %s\n\
+\n\
+Does not have a defined command.",
+		d->label
+	    );
+	    EDVPlaySoundWarning(core);
+	    EDVMessageWarning(
+		"Open Failed",
+		msg,
+"To set the command for this custom menu item,\n\
+go to Settings->Customize...->Global->Object Ops Menu.",
+		toplevel
+	    );
+	    g_free(msg);
+	    EDVBrowserSetBusy(browser, FALSE);
+	    return;
+	}
+
+	/* Create the selected paths list */
+	paths_list = EDVBrowserGetSelectedPathsList(browser);
+	if(paths_list != NULL)
+	{
+	    /* Open the selected objects */
+	    gchar *stdout_path, *stderr_path;
+
+	    EDVOpenCommand(
+		core,
+		paths_list,
+		command,
+		toplevel,
+		TRUE,
+		&stdout_path, &stderr_path
+	    );
+
+	    g_free(stdout_path);
+	    g_free(stderr_path);
+
+	    /* Delete the selected paths list */
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
+
+	}
+
+	EDVBrowserSetBusy(browser, FALSE);
+}
+
+/*
+ *	Menu item command new object "activate" signal callback.
+ */
+void EDVBrowserMenuItemCmdNewObjectCB(GtkWidget *widget, gpointer data)
+{
+	gboolean yes_to_all = FALSE;
+	gint status;
+	gchar *new_name, *new_path;
+	const gchar *command, *error_msg;
+	GList *new_paths_list;
+	GtkWidget *toplevel;
+	edv_browser_struct *browser;
+	edv_core_struct *core;
+	edv_browser_cmd_struct *d = EDV_BROWSER_CMD(data);
+	if(d == NULL)
+	    return;
+
+	browser = d->browser;
+	toplevel = browser->toplevel;
+	core = browser->core;
+
+	if(browser->processing)
+	    return;
+
+	EDVBrowserSetBusy(browser, TRUE);
+
+	/* Get the command for this menu item */
+	command = d->command;
+	if(STRISEMPTY(command))
+	{
+	    gchar *msg = g_strdup_printf(
+"The custom menu item:\n\
+\n\
+    %s\n\
+\n\
+Does not have a defined command.",
+		d->label
+	    );
+	    EDVPlaySoundWarning(core);
+	    EDVMessageWarning(
+		"Open Failed",
+		msg,
+"To set the command for this custom menu item,\n\
+go to Settings->Customize...->Global->New Objects Menu.",
+		toplevel
+	    );
+	    g_free(msg);
+	    EDVBrowserSetBusy(browser, FALSE);
+	    return;
+	}
+
+	/* Format the suggested name for the new object */
+	if(!STRISEMPTY(d->ext_data))
+	{
+	    const gchar *ext = d->ext_data;
+	    gchar *s, *new_ext;
+
+	    if(*ext != '.')
+		new_ext = g_strconcat(".", ext, NULL);
+	    else
+		new_ext = STRDUP(ext);
+	    s = (gchar *)strpbrk((char *)new_ext, " \t,;");
+	    if(s != NULL)
+		*s = '\0';
+
+	    new_name = g_strconcat(
+		"new",
+		new_ext,
+		NULL
+	    );
+
+	    g_free(new_ext);
+	}
+	else
+	{
+	    new_name = STRDUP("new");
+	}
+
+	/* Prompt the user for the name of the new file and create the
+	 * new file
+	 */
+	status = EDVObjCreateFileQueryName(
+	    core,
+	    EDVBrowserCurrentLocation(browser),	/* Parent path */
+	    new_name,				/* Suggested name */
+	    &new_path,
+	    toplevel,
+	    TRUE,				/* Show progress */
+	    &yes_to_all
+	);
+
+	g_free(new_name);
+
+	/* Check for errors */
+	error_msg = EDVObjCreateGetError(core);
+	if(!STRISEMPTY(error_msg))
+	{
+	    /* Report the error */
+	    EDVPlaySoundError(core);
+	    EDVMessageError(
+		"Create New Object Failed",
+		error_msg,   
+		NULL,
+		toplevel
+	    );
+	}
+
+	/* Report the new object being added and create the new paths
+	 * list
+	 */
+	new_paths_list = NULL;
+	if(new_path != NULL)
+	{
+	    struct stat lstat_buf;
+	    if(!lstat((const char *)new_path, &lstat_buf))
+		EDVObjectAddedEmit(
+		    core,
+		    new_path,
+		    &lstat_buf
+		);
+
+	    new_paths_list = g_list_append(
+		new_paths_list,
+		STRDUP(new_path)
+	    );
+	}
+
+	/* Unmap the progress dialog */
+	ProgressDialogBreakQuery(TRUE);
+	ProgressDialogSetTransientFor(NULL);
+
+	/* Open the new object */
+	if(new_paths_list != NULL)
+	{
+	    /* Open the selected objects */
+	    gchar *stdout_path, *stderr_path;
+
+	    EDVOpenCommand(
+		core,
+		new_paths_list,
+		command,
+		toplevel,
+		TRUE,
+		&stdout_path, &stderr_path
+	    );
+
+	    g_free(stdout_path);
+	    g_free(stderr_path);
+
+	    /* Delete the new paths list */
+	    g_list_foreach(new_paths_list, (GFunc)g_free, NULL);
+	    g_list_free(new_paths_list);
+	}
+
+	g_free(new_path);
+
+	/* Play the "completed" sound on success */
+	if(status == 0)
+	    EDVPlaySoundCompleted(core);
+
+	EDVBrowserSetBusy(browser, FALSE);
+}
+
+/*
+ *	Menu item command "leave_enter_event" signal callback.
+ */
+gint EDVBrowserMenuItemCmdEnterCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+)
+{
+	edv_browser_struct *browser;
+	edv_browser_cmd_struct *d = EDV_BROWSER_CMD(data);
+	if(d == NULL)
+	    return(FALSE);
+
+	browser = d->browser;
+
+	EDVStatusBarMessage(
+	    browser->status_bar,
+	    STRISEMPTY(d->description) ?
+		d->command : d->description,
+	    FALSE
+	);
+
+	return(TRUE);
+}
+
+/*
+ *	Menu item command "leave_notify_event" signal callback.
+ */
+gint EDVBrowserMenuItemCmdLeaveCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+)
+{
+	edv_browser_struct *browser;
+	edv_browser_cmd_struct *d = EDV_BROWSER_CMD(data);
+	if(d == NULL)
+	    return(FALSE);
+
+	browser = d->browser;
+
+	EDVStatusBarMessage(
+	    browser->status_bar,
+	    NULL,
+	    FALSE
+	);
+
+	return(TRUE);
+}
+
 
 /*
  *	Paths list get callback.
  */
 GList *EDVBrowserGetSelectedPathsListCB(gpointer data)
 {
-	GList *paths_list = NULL;
 	edv_browser_struct *browser = EDV_BROWSER(data);
 	if(browser == NULL)
-	    return(paths_list);
+	    return(NULL);
 
-	/* Contents clist was last selected? */
-	if(browser->contents_clist_selected_row > -1)
-	{
-	    GtkCList *clist = (GtkCList *)browser->contents_clist;
-	    if(clist != NULL)
-	    {
-		gint row;
-		GList *glist = clist->selection;
-		edv_object_struct *obj;
-		while(glist != NULL)
-		{
-		    row = (gint)glist->data;
-		    obj = EDV_OBJECT(gtk_clist_get_row_data(clist, row));
-		    if(obj != NULL)
-			paths_list = g_list_append(
-			    paths_list, STRDUP(obj->full_path)
-			);
-		    glist = g_list_next(glist);
-		}
-	    }
-	}
-	else if(browser->directory_ctree_selected_node != NULL)
-	{
-	    GtkCTree *ctree = (GtkCTree *)browser->directory_ctree;
-	    GtkCList *clist = (GtkCList *)ctree;
-	    if(clist != NULL)
-	    {
-		GList *glist = clist->selection;
-		GtkCTreeNode *node;
-		edv_object_struct *obj;
-		while(glist != NULL)
-		{
-		    node = (GtkCTreeNode *)glist->data;
-		    obj = EDV_OBJECT(gtk_ctree_node_get_row_data(ctree, node));
-		    if(obj != NULL)
-			paths_list = g_list_append(
-			    paths_list, STRDUP(obj->full_path)
-			);
-		    glist = g_list_next(glist);
-		}
-	    }
-	}
-
-	return(paths_list);
+	return(EDVBrowserGetSelectedPathsList(browser));
 }
 
 /*
@@ -1933,8 +2141,6 @@ void EDVBrowserMountBarMountCB(
 
 	toplevel = browser->toplevel;
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
 	EDVBrowserSetBusy(browser, TRUE);
 
@@ -1966,9 +2172,9 @@ void EDVBrowserMountBarMountCB(
 	EDVBrowserSetBusy(browser, FALSE);
 
 	/* Mount error? */
-	if(status)
+	if(status != 0)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -2009,8 +2215,6 @@ void EDVBrowserMountBarEjectCB(
 
 	toplevel = browser->toplevel;
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
 	EDVBrowserSetBusy(browser, TRUE);
 
@@ -2042,9 +2246,9 @@ void EDVBrowserMountBarEjectCB(
 	EDVBrowserSetBusy(browser, FALSE);
 
 	/* Mount error? */
-	if(status)
+	if(status != 0)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -2127,9 +2331,7 @@ void EDVBrowserFindBarStartCB(edv_findbar_struct *fb, gpointer data)
 	if(browser == NULL)
 	    return;
 
-	clist = (GtkCList *)browser->contents_clist;
-	if(clist == NULL)
-	    return;
+	clist = GTK_CLIST(browser->contents_clist);
 
 	EDVBrowserSetBusy(browser, TRUE);
 
@@ -2169,13 +2371,15 @@ void EDVBrowserFindBarMatchCB(
 	if((path == NULL) || (lstat_buf == NULL) || (browser == NULL))
 	    return;
 
-	clist = (GtkCList *)browser->contents_clist;
-	if(clist == NULL)
-	    return;
+	clist = GTK_CLIST(browser->contents_clist);
 
 	row = EDVBrowserContentsFindRowByPath(browser, path);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
+	{
+	    gtk_clist_freeze(clist);
 	    gtk_clist_select_row(clist, row, 0);
+	    gtk_clist_thaw(clist);
+	}
 }
 
 
@@ -2228,7 +2432,7 @@ void EDVBrowserObjectAddedNotifyCB(
 	const struct stat *lstat_buf
 )
 {
-	if(browser == NULL)
+	if((browser == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(browser->processing)
@@ -2259,39 +2463,34 @@ void EDVBrowserObjectModifiedNotifyCB(
 	const struct stat *lstat_buf
 )
 {
-	if(browser == NULL)
+	gchar *cur_path;
+
+	if((browser == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(browser->processing)
 	    return;
 
-	/* Check current location first, if the old path matches the
-	 * current location then the current location needs to be updated
-	 * to reflect the new path before notifying the contents clist
-	 * and directory ctree
-	 */
-	if(!STRISEMPTY(path))
+	if(new_path == NULL)
+	    new_path = path;
+
+	cur_path = STRDUP(EDVBrowserCurrentLocation(browser));
+	if(cur_path == NULL)
+	    return;
+
+	/* Is the modified path the current path? */
+	if(!strcmp((const char *)cur_path, (const char *)path))
 	{
-	    const gchar *cur_location = EDVBrowserCurrentLocation(browser);
-
-	    if(new_path == NULL)
-		new_path = path;
-
-	    /* Check if the old path matches the current location, if
-	     * it does then the current location needs to be updated
-	     * to the new path
-	     */
-	    if((cur_location != NULL) ?
-		!strcmp(cur_location, path) : FALSE
-	    )
+	    /* Path changed? */
+	    if(path != new_path)
 	    {
-		/* Old path matches current location, change values
-		 * reflecting the current location to the value of
-		 * new_path
-		 */
-		EDVBrowserSetTitle(browser, new_path);
-		EDVBrowserSetLocation(browser, new_path, FALSE);
-		EDVBrowserUpdateLocationIcon(browser, new_path);
+		if(strcmp((const char *)path, (const char *)new_path))
+		{
+		    /* Update the current location */
+		    EDVBrowserSetTitle(browser, new_path);
+		    EDVBrowserSetLocation(browser, new_path, FALSE);
+		    EDVBrowserUpdateLocationIcon(browser, new_path);
+		}
 	    }
 	}
 
@@ -2309,6 +2508,8 @@ void EDVBrowserObjectModifiedNotifyCB(
 	EDVMountBarUpdateMenus(browser->mount_bar);
 
 /*	EDVBrowserUpdateMenus(browser); */
+
+	g_free(cur_path);
 }
 
 /*
@@ -2318,12 +2519,18 @@ void EDVBrowserObjectRemovedNotifyCB(
 	edv_browser_struct *browser, const gchar *path
 )
 {
-	if(browser == NULL)
+	gchar *cur_path;
+
+	if((browser == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(browser->processing)
 	    return;
 
+	cur_path = STRDUP(EDVBrowserCurrentLocation(browser));
+	if(cur_path == NULL)
+	    return;
+
 	/* Contents List */
 	EDVBrowserContentsObjectRemovedNotify(browser, path);
 
@@ -2333,42 +2540,27 @@ void EDVBrowserObjectRemovedNotifyCB(
 	/* Mount Bar */
 	EDVMountBarUpdateMenus(browser->mount_bar);
 
-	/* Check if the path that was removed is the current location
-	 * (after letting contents clist and directory ctree check on it
-	 * first), if the path that was removed is the location then
-	 * set the current location to be the parent of the path that
-	 * was removed
+	/* Is the removed path the current location or the parent
+	 * of the current location?
 	 */
-	if(!STRISEMPTY(path))
+	if(strpfx((const char *)cur_path, (const char *)path))
 	{
-	    const gchar *cur_location = EDVBrowserCurrentLocation(browser);
-
-	    /* Check if the removed object path matches the current
-	     * location, if it matches then the current location needs
-	     * to be updated to reflect that
+	    /* Change the current location to the parent of the
+	     * removed path
 	     */
-	    if((cur_location != NULL) ?
-		!strcmp(cur_location, path) : FALSE
-	    )
+	    gchar *parent_path = g_dirname(path);
+	    if(parent_path != NULL)
 	    {
-		/* Removed object path matches the current location, so
-		 * technically the current location needs to be cleared
-		 * however we'll instead just change the current location
-		 * value to reflect the value of the parent path of the
-		 * removed object path
-		 */
-		gchar *parent_path = g_dirname(path);
-		if(parent_path != NULL)
-		{
-		    EDVBrowserSetTitle(browser, parent_path);
-		    EDVBrowserSetLocation(browser, parent_path, FALSE);
-		    EDVBrowserUpdateLocationIcon(browser, parent_path);
-		    g_free(parent_path);
-		}
+		EDVBrowserSetTitle(browser, parent_path);
+		EDVBrowserSetLocation(browser, parent_path, FALSE);
+		EDVBrowserUpdateLocationIcon(browser, parent_path);
+		g_free(parent_path);
 	    }
 	}
 
 /*	EDVBrowserUpdateMenus(browser); */
+
+	g_free(cur_path);
 }
 
 
@@ -2410,8 +2602,6 @@ void EDVBrowserRecycledObjectAddedNotifyCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
 	/* There is not much interest when a recycled object has been
 	 * added or removed. Only the menus need to be updated if there
@@ -2434,8 +2624,6 @@ void EDVBrowserRecycledObjectRemovedNotifyCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
 	/* There is not much interest when a recycled object has been
 	 * added or removed. Only the menus need to be updated if there
@@ -2465,9 +2653,6 @@ void EDVBrowserReconfiguredNotifyCB(edv_browser_struct *browser)
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 	standard_rcstyle = core->standard_rcstyle;
 	lists_rcstyle = core->lists_rcstyle;
@@ -2484,15 +2669,14 @@ void EDVBrowserReconfiguredNotifyCB(edv_browser_struct *browser)
 	cur_loc = STRDUP(EDVBrowserCurrentLocation(browser));
 
 
-	/* Update Title */
+	/* Update the title */
 	EDVBrowserSetTitle(browser, cur_loc);
 
-	/* Recreate Accelkeys labels */
-	EDVBrowserAccelkeysRegenerate(browser);
-
-	/* Recreate Tool Bar */
+	/* Recreate the Tool Bar */
 	EDVBrowserToolBarRegenerate(browser);
 
+	/* Recreate the File menu items */
+	EDVBrowserFileMenuItemsRegenerate(browser);
 
 	/* Show/hide Tool Bar */
 	w = browser->tool_bar_handle;
@@ -2559,9 +2743,39 @@ void EDVBrowserReconfiguredNotifyCB(edv_browser_struct *browser)
 		EDVStatusBarUnmap(status_bar);
 	}
 
-	/* Recreate the New Object submenu */
-	EDVBrowserNewObjectMenuRegenerate(browser);
+	/* Recreate the New Object submenu items */
+	EDVBrowserNewObjectMenuItemsRegenerate(browser);
 
+	/* Recreate the Open To menus */
+	if(browser->contents_clist_selected_row > -1)
+	{
+	    const gint row = browser->contents_clist_selected_row;
+	    edv_object_struct *obj = EDV_OBJECT(gtk_clist_get_row_data(
+		GTK_CLIST(browser->contents_clist), row
+	    ));
+	    const gchar *path = (obj != NULL) ? obj->full_path : NULL;
+	    edv_mime_type_struct *m = EDVMimeTypesListMatchPath(
+		core->mimetype, core->total_mimetypes,
+		path
+	    );
+	    EDVBrowserOpenToMenuRegenerate(browser, m);
+	}
+	else if(browser->directory_ctree_selected_node != NULL)
+	{
+	    GtkCTreeNode *node = browser->directory_ctree_selected_node;
+	    edv_object_struct *obj = EDV_OBJECT(gtk_ctree_node_get_row_data(
+		GTK_CTREE(browser->directory_ctree), node
+	    ));
+	    const gchar *path = (obj != NULL) ? obj->full_path : NULL;
+	    edv_mime_type_struct *m = EDVMimeTypesListMatchPath(
+		core->mimetype, core->total_mimetypes,
+		path
+	    );
+	    EDVBrowserOpenToMenuRegenerate(browser, m);
+	}
+
+	/* Recreate the Accelkey labels */
+	EDVBrowserAccelkeysRegenerate(browser);
 
 	/* Update RC styles */
 	w = browser->toplevel;
@@ -2583,7 +2797,7 @@ void EDVBrowserReconfiguredNotifyCB(edv_browser_struct *browser)
 	EDVBrowserListStylesRegenerate(browser);
 
 
-	/* Realize listings */
+	/* Realize the listings */
 	EDVBrowserDirTreeRealizeListing(browser, NULL);
 	EDVBrowserContentsRealizeListing(browser);
 
@@ -2631,10 +2845,8 @@ void EDVBrowserMimeTypeModifiedCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
-	/* Get current location */
+	/* Get the current location */
 	cur_loc = STRDUP(EDVBrowserCurrentLocation(browser));
 
 	/* Realize listings */
@@ -2663,10 +2875,8 @@ void EDVBrowserMimeTypeRemovedCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
-	/* Get current location */
+	/* Get the current location */
 	cur_loc = STRDUP(EDVBrowserCurrentLocation(browser));
 
 	/* Realize listings */
@@ -2713,10 +2923,8 @@ void EDVBrowserDeviceModifiedCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
-	/* Get current location */
+	/* Get the current location */
 	cur_loc = STRDUP(EDVBrowserCurrentLocation(browser));
 
 	/* Realize listings */
@@ -2747,10 +2955,8 @@ void EDVBrowserDeviceRemovedCB(
 	    return;
 
 	core = browser->core;
-	if(core == NULL)
-	    return;
 
-	/* Get current location */
+	/* Get the current location */
 	cur_loc = STRDUP(EDVBrowserCurrentLocation(browser));
 
 	/* Check if mount bar is currently referencing this device, if it
