@@ -43,7 +43,7 @@ extern void EDVBrowserContentsOpenWith(
 );
 
 /* Renaming */
-extern void EDVBrowserContentsDoFPromptRename(
+extern void EDVBrowserContentsPromptRename(
 	edv_browser_struct *browser,
 	const gint row, const gint column
 );
