@@ -1,5 +1,5 @@
 /*
-                      Endeavour Browser Directory Tree
+		      Endeavour Browser Directory Tree
  */
 
 #ifndef BROWSER_DIR_TREE_H
@@ -14,10 +14,10 @@
 #include "browser.h"
 
 
-/* Getting */
+/* Node Utilities */
 extern GtkCTreeNode *EDVBrowserDirTreeGetToplevel(edv_browser_struct *browser);
 
-/* Set Origin Path */
+/* Origin Path */
 extern void EDVBrowserDirTreeSetOriginPath(
 	edv_browser_struct *browser, const gchar *path
 );
@@ -26,14 +26,15 @@ extern void EDVBrowserDirTreeSetOriginPath(
 extern void EDVBrowserDirTreeCreateToplevels(edv_browser_struct *browser);
 extern void EDVBrowserDirTreeGetListing(
 	edv_browser_struct *browser, GtkCTreeNode *node,
-	gboolean update_progress
+	const gboolean update_progress
 );
 extern void EDVBrowserDirTreeClear(edv_browser_struct *browser);
 
 /* Expanding */
 extern void EDVBrowserDirTreeExpand(
-	edv_browser_struct *browser, GtkCTreeNode *node,
-	gboolean update_progress
+	edv_browser_struct *browser,
+	GtkCTreeNode *node,
+	const gboolean update_progress
 );
 
 /* Realize Listing */
@@ -48,20 +49,25 @@ extern void EDVBrowserDirTreeSelectPath(
 
 /* Finding */
 extern GtkCTreeNode *EDVBrowserDirTreeFindNodeByIndex(
-        edv_browser_struct *browser,
-        const gulong device_index, const gulong index
+	edv_browser_struct *browser,
+	const gulong device_index, const gulong index
 );
 extern GtkCTreeNode *EDVBrowserDirTreeFindNodeByPath(
 	edv_browser_struct *browser, const gchar *path
 );
 
+/* Removing */
+extern void EDVBrowserDirTreeRemoveGrandChildrenNodes(
+	edv_browser_struct *browser, GtkCTreeNode *node
+);
+
 /* Opening */
-extern void EDVBrowserDirTreeDoOpenWithObject(
+extern void EDVBrowserDirTreeOpenWith(
 	edv_browser_struct *browser, GtkCTreeNode *node
 );
 
 /* Renaming */
-extern void EDVBrowserDirTreeDoFPromptRename(
+extern void EDVBrowserDirTreePromptRename(
 	edv_browser_struct *browser, GtkCTreeNode *node
 );
 
