@@ -260,7 +260,7 @@ static void EDVBrowserDragDataReceivedNexus(
 	GtkWidget *toplevel = browser->toplevel;
 	const url_struct *url;
 
-#define DO_FREE_LOCALS	{		\
+#define CLEANUP_RETURN	{		\
  g_free(parent_path);			\
 					\
  if(url_list != NULL) {			\
@@ -269,6 +269,8 @@ static void EDVBrowserDragDataReceivedNexus(
   );					\
   g_list_free(url_list);		\
  }					\
+					\
+ return;				\
 }
 
 	/* Get copy of parent_path as the object's full path if the
@@ -321,8 +323,7 @@ static void EDVBrowserDragDataReceivedNexus(
 		);
 
 		/* Do not continue with standard dnd handling */
-		DO_FREE_LOCALS
-		return;
+		CLEANUP_RETURN;
 	    }
 	}
 
@@ -354,7 +355,7 @@ static void EDVBrowserDragDataReceivedNexus(
 	    EDVBrowserSetBusy(browser, TRUE);
 	    browser->processing = TRUE;
 
-	    /* Iterate through URL list */
+	    /* Iterate through the URL list */
 	    for(glist = url_list;
 		glist != NULL;
 		glist = g_list_next(glist)
@@ -407,22 +408,23 @@ static void EDVBrowserDragDataReceivedNexus(
 		    break;
 		}
 
-		/* Check for and report any errors */
+		/* Check for errors */
 		if(error_msg == NULL)
-		    error_msg = EDVObjectOPGetError();
+		    error_msg = EDVObjectOPGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
+		    EDVMessageError(
 			error_title,
 			error_msg,
-			path,
+			NULL,
 			toplevel
 		    );
 		}
 
 		/* Get the source object's statistics for reporting */
-		if(lstat(path, &lstat_buf))
+		if(lstat((const char *)path, &lstat_buf))
 		{
 #ifdef ENOENT
 		    /* Report the source object being removed */
@@ -439,7 +441,7 @@ static void EDVBrowserDragDataReceivedNexus(
 		/* Report the new object being added? */
 		if(new_path != NULL)
 		{
-		    if(!lstat(new_path, &lstat_buf))
+		    if(!lstat((const char *)new_path, &lstat_buf))
 		    {
 			objects_processed++;
 
@@ -466,13 +468,11 @@ static void EDVBrowserDragDataReceivedNexus(
 		nurls, objects_processed, status
 	    );
 
-	    /* Unmap progress dialog, it may have been mapped if any
-	     * operations occured in the above loop
-	     */
+	    /* Unmap the progress dialog */
 	    ProgressDialogBreakQuery(TRUE);
 	    ProgressDialogSetTransientFor(NULL);
 
-	    /* Play completed sound on success */
+	    /* Play the "completed" sound on success */
 	    if(status == 0)
 		EDVPlaySoundCompleted(core);
 
@@ -534,16 +534,17 @@ static void EDVBrowserDragDataReceivedNexus(
 		    break;
 		}
 
-		/* Check for and report any errors */
+		/* Check for errors */
 		if(error_msg == NULL)
-		    error_msg = EDVRecBinRecoverGetError();
+		    error_msg = EDVRecBinRecoverGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
+		    EDVMessageError(
 			"Recover Recycled Object Error",
 			error_msg,
-			url->path_arg,
+			NULL,
 			toplevel
 		    ); 
 		}
@@ -579,13 +580,11 @@ static void EDVBrowserDragDataReceivedNexus(
 		nurls, objects_recovered, status
 	    );
 
-	    /* Unmap progress dialog, it may have been mapped if any
-	     * operations occured in the above loop
-	     */
+	    /* Unmap the progress dialog */
 	    ProgressDialogBreakQuery(TRUE);
 	    ProgressDialogSetTransientFor(NULL);
 
-	    /* Play completed sound on success */
+	    /* Play the "completed" sound on success */
 	    if(status == 0)
 		EDVPlaySoundCompleted(core);
 
@@ -601,8 +600,9 @@ static void EDVBrowserDragDataReceivedNexus(
 	    gboolean	preserve_directories = TRUE,
 			preserve_timestamps = TRUE;
 	    gint status, objects_extracted = 0;
-	    const gchar *arch_obj = NULL, *error_msg;
-	    GList *obj_list = NULL, *new_paths_list;
+	    const gchar *arch_path = NULL, *error_msg;
+	    gchar *password = NULL;
+	    GList *obj_list, *new_paths_list;
 
 	    EDVBrowserSetBusy(browser, TRUE);
 	    browser->processing = TRUE;
@@ -613,12 +613,14 @@ static void EDVBrowserDragDataReceivedNexus(
 	    if(nurls >= 1)
 	    {
 		url = URL(url_list->data);
-		arch_obj = (url != NULL) ? url->path : NULL;
+		arch_path = (url != NULL) ? url->path : NULL;
 	    }
 
 	    /* Query user for extract from archive options */
 	    if(!EDVArchExtractOptsGetResponse(
-		core, toplevel, arch_obj,
+		core, toplevel,
+		arch_path,
+		&password,
 		&preserve_directories,
 		&preserve_timestamps
 	    ))
@@ -626,11 +628,12 @@ static void EDVBrowserDragDataReceivedNexus(
 		/* User canceled */
 		browser->processing = FALSE;
 		EDVBrowserSetBusy(browser, FALSE);
-		DO_FREE_LOCALS
-		return;
+		g_free(password);
+		CLEANUP_RETURN;
 	    }
 
 	    /* Get the stats for all the objects found in the archive */
+	    obj_list = NULL;
 	    if(nurls >= 2)
 	    {
 		GList *glist, *paths_list = NULL;
@@ -642,48 +645,55 @@ static void EDVBrowserDragDataReceivedNexus(
 		)
 		{
 		    url = URL(glist->data);
-		    if(url != NULL)
-			paths_list = g_list_append(
-			    paths_list, url->path
-			);
+		    if(url == NULL)
+			continue;
+
+		    if(url->path == NULL)
+			continue;
+
+		    paths_list = g_list_append(
+			paths_list,
+			STRDUP(url->path)
+		    );
+		}
+		if(paths_list != NULL)
+		{
+		    obj_list = EDVArchStatList(
+			core,
+			arch_path,
+			paths_list,
+			NULL,			/* No filter */
+			password,
+			NULL, NULL		/* No progress callback */
+		    );
+		    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+		    g_list_free(paths_list);
 		}
-
-		obj_list = EDVArchStatList(
-		    core,
-		    arch_obj,
-		    paths_list,
-		    NULL,			/* No filter */
-		    NULL,			/* No password */
-		    NULL, NULL			/* No progress callback */
-		);
-
-		g_list_free(paths_list);
 	    }
 
 	    /* Extract the object(s) from the archive */
 	    new_paths_list = NULL;
 	    status = EDVArchExtract(
-		core, arch_obj,
+		core, arch_path,
 		obj_list,
+		FALSE,				/* Not extract all */
 		parent_path, &new_paths_list,
-		NULL,			/* No password */
-/* TODO: Prompt for password? */
+		password,
 		toplevel, TRUE, TRUE, &yes_to_all,
 		preserve_directories,
 		preserve_timestamps
 	    );
 
-	    /* Get error message (if any) describing the error that
-	     * might have occured in the above operation
-	     */
-	    error_msg = EDVArchExtractGetError();
+	    /* Check for errors */
+	    error_msg = EDVArchExtractGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
-		EDVMessageObjectOPError(
+		EDVMessageError(
 		    "Extract Object Error",
 		    error_msg,
-		    arch_obj,
+		    NULL,
 		    toplevel
 		);
 	    }
@@ -703,14 +713,13 @@ static void EDVBrowserDragDataReceivedNexus(
 		)
 		{
 		    path = (const gchar *)glist->data;
-		    if(!STRISEMPTY(path) ? lstat(path, &lstat_buf) : TRUE)
+		    if(STRISEMPTY(path))
 			continue;
 
-		    EDVObjectAddedEmit(
-			core,
-			path,
-			&lstat_buf
-		    );
+		    if(lstat((const char *)path, &lstat_buf))
+			continue;
+
+		    EDVObjectAddedEmit(core, path, &lstat_buf);
 		    objects_extracted++;
 		}
 
@@ -727,26 +736,29 @@ static void EDVBrowserDragDataReceivedNexus(
 		g_list_length(obj_list), objects_extracted, status
 	    );
 
-	    /* Unmap progress dialog, it may have been mapped if any
-	     * operations occured in the above loop
-	     */
+	    /* Unmap the progress dialog */
 	    ProgressDialogBreakQuery(TRUE);
 	    ProgressDialogSetTransientFor(NULL);
 
-	    /* Play completed sound on success */
+	    g_free(password);
+
+	    /* Delete the list of archive object stats */
+	    if(obj_list != NULL)
+	    {
+		g_list_foreach(obj_list, (GFunc)EDVArchObjectDelete, NULL);
+		g_list_free(obj_list);
+	    }
+
+	    /* Play the "completed" sound on success */
 	    if(status == 0)
 		EDVPlaySoundCompleted(core);
 
 	    browser->processing = FALSE;
 	    EDVBrowserSetBusy(browser, FALSE);
-
-	    /* Delete the list of archive object stats */
-	    g_list_foreach(obj_list, (GFunc)EDVArchObjectDelete, NULL);
-	    g_list_free(obj_list);
 	}
 
-	DO_FREE_LOCALS
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN;
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -768,8 +780,6 @@ gint EDVBrowserLocBarIconCrossingCB(
 
 	w = browser->location_icon_pm;
 	sb = browser->status_bar;
-	if((w == NULL) || (sb == NULL))
-	    return(status);
 
 	switch((gint)crossing->type)
 	{
@@ -831,10 +841,7 @@ void EDVBrowserDirTreeDNDSetIcon(
 	if((browser == NULL) || (node == NULL))
 	    return;
 
-	ctree = (GtkCTree *)browser->directory_ctree;
-	if(ctree == NULL)
-	    return;
-
+	ctree = GTK_CTREE(browser->directory_ctree);
 	obj = EDV_OBJECT(gtk_ctree_node_get_row_data(ctree, node));
 	if(obj != NULL)
 	{
@@ -909,12 +916,9 @@ void EDVBrowserDirTreeDragDataGetCB(
 	    return;
 
 	w = browser->directory_ctree;
-	core = browser->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
 	ctree = GTK_CTREE(w);
 	clist = GTK_CLIST(w);
+	core = browser->core;
 
 	EDVBrowserSyncData(browser);
 
@@ -956,9 +960,12 @@ void EDVBrowserDirTreeDragDataGetCB(
 	    }
 	}
 
-	/* Delete URL list */
-	g_list_foreach(url_list, (GFunc)URLDelete, NULL);
-	g_list_free(url_list);
+	/* Delete the URL list */
+	if(url_list != NULL)
+	{
+	    g_list_foreach(url_list, (GFunc)URLDelete, NULL);
+	    g_list_free(url_list);
+	}
 
 	/* If failed to send out data then respond with error */
 	if(!data_sent)
@@ -997,12 +1004,9 @@ void EDVBrowserDirTreeDragDataReceivedCB(
 	    return;
 
 	w = browser->directory_ctree;
-	core = browser->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
 	ctree = GTK_CTREE(w);
 	clist = GTK_CLIST(w);
+	core = browser->core;
 
 	/* Check if received data is not empty */
 	if((selection_data != NULL) ? (selection_data->length <= 0) : TRUE)
@@ -1072,9 +1076,7 @@ void EDVBrowserContentsDNDSetIcon(
 	if(browser == NULL)
 	    return;
 
-	clist = (GtkCList *)browser->contents_clist;
-	if(clist == NULL)
-	    return;
+	clist = GTK_CLIST(browser->contents_clist);
 
 	obj = EDV_OBJECT(gtk_clist_get_row_data(clist, row));
 	if(obj != NULL)
@@ -1152,11 +1154,8 @@ void EDVBrowserContentsDragDataGetCB(
 	    return;
 
 	w = browser->contents_clist;
-	core = browser->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
 	clist = GTK_CLIST(w);
+	core = browser->core;
 
 	EDVBrowserSyncData(browser);
 
@@ -1196,9 +1195,12 @@ void EDVBrowserContentsDragDataGetCB(
 	    }
 	}
 
-	/* Delete URL list */
-	g_list_foreach(url_list, (GFunc)URLDelete, NULL);
-	g_list_free(url_list);
+	/* Delete the URL list */
+	if(url_list != NULL)
+	{
+	    g_list_foreach(url_list, (GFunc)URLDelete, NULL);
+	    g_list_free(url_list);
+	}
 
 	/* If failed to send out data then respond with error */ 
 	if(!data_sent)
@@ -1211,7 +1213,6 @@ void EDVBrowserContentsDragDataGetCB(
 		s,			/* Data */
 		STRLEN(s)		/* Length */
 	    );
-	    data_sent = TRUE;
 	}
 }
 
@@ -1235,11 +1236,8 @@ void EDVBrowserContentsDragDataReceivedCB(
 	    return;
 
 	w = browser->contents_clist;
-	core = browser->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
 	clist = GTK_CLIST(w);
+	core = browser->core;
 
 	/* Check if received data is not empty */
 	if((selection_data != NULL) ? (selection_data->length <= 0) : TRUE)
