@@ -15,11 +15,13 @@
 #include "config.h"
 
 
+static gchar *CFGStripQuotes(gchar *s);
+
 gint CFGFileOpen(
-	const gchar *filename, cfg_item_struct *cfg_list
+	const gchar *path, cfg_item_struct *cfg_list
 );
 gint CFGFileSave(
-	const gchar *filename, const cfg_item_struct *cfg_list
+	const gchar *path, const cfg_item_struct *cfg_list
 );
 
 
@@ -28,7 +30,7 @@ gint CFGFileSave(
 #endif
 
 #ifndef CFG_DELIMINATOR_CHAR
-# define CFG_DELIMINATOR_CHAR	'='
+# define CFG_DELIMINATOR_CHAR		'='
 #endif
 
 #define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
@@ -43,6 +45,31 @@ gint CFGFileSave(
 #define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
+static gchar *CFGStripQuotes(gchar *s)
+{
+	if(s == NULL)
+	    return(NULL);
+
+	if(*s == '"')
+	{
+	    gchar *s2 = s;
+	    while(*s2 != '\0')
+	    {
+		*s2 = *(s2 + 1);
+		s2++;
+	    }
+	    s2 -= 2;
+	    if(s2 >= s)
+	    {
+		if(*s2 == '"')
+		    *s2 = '\0';
+	    }
+	}
+
+	return(s);
+}
+
+
 /*
  *	Loads the cfg list from the specified file and stores the
  *	loaded values in to the given cfg list.
@@ -51,30 +78,40 @@ gint CFGFileSave(
  *
  *	0	Success.
  *	-1	General error (including no such file).
- *	-2	Bad value.
- *	-3	System error.
+ *	-2	Invalid value.
+ *	-3	System error or memory allocation error.
  */
 gint CFGFileOpen(
-	const gchar *filename, cfg_item_struct *cfg_list
+	const gchar *path, cfg_item_struct *cfg_list
 )
 {
-	gboolean ignore_all_errors = FALSE;
-	gint i;
 	FILE *fp;
 	struct stat stat_buf;
+	gboolean ignore_all_errors = FALSE;
+	gint i;
 	gchar *parm = NULL;
 
-	if(STRISEMPTY(filename) || (cfg_list == NULL))
+	if(STRISEMPTY(path) || (cfg_list == NULL))
 	    return(-2);
 
-	if(stat(filename, &stat_buf))
+	if(stat((const char *)path, &stat_buf))
 	{
-#if 0
-	    g_printerr(
-		"%s: Warning: No such file.\n",
-		filename
-	    );
+	    const gint error_code = (gint)errno;
+#ifdef ENOENT
+	    if(error_code == ENOENT)
+#else
+	    if(FALSE)
 #endif
+	    {
+
+	    }
+	    else
+	    {
+		g_printerr(
+		    "%s: %s.\n",
+		    path, g_strerror(error_code)
+		);
+	    }
 	    return(-1);
 	}
 #ifdef S_ISREG
@@ -82,24 +119,25 @@ gint CFGFileOpen(
 	{
 	    g_printerr(
 		"%s: Error: Not a file.\n",
-		filename
+		path
 	    );
 	    return(-2);
 	}
 #endif
 
 	/* Open the configuration file for reading */
-	fp = FOpen((const char *)filename, "rb");
+	fp = fopen((const char *)path, "rb");
 	if(fp == NULL)
 	{
+	    const gint error_code = (gint)errno;
 	    g_printerr(
-		"%s: Error: Unable to open file for reading.\n",
-		filename
+		"%s: %s.\n",
+		path, g_strerror(error_code)
 	    );
 	    return(-1);
 	}
 
-	/* Begin reading configuration file */
+	/* Begin reading the configuration file */
 	while(TRUE)
 	{
 	    parm = (gchar *)FSeekNextParm(
@@ -117,7 +155,7 @@ gint CFGFileOpen(
 		if(!ignore_all_errors)
 		    g_printerr(
 "%s: Warning: Unsupported parameter \"%s\".\n",
-			filename,
+			path,
 			parm
 		    );
 		FSeekNextLine(fp);
@@ -233,10 +271,10 @@ gint CFGFileOpen(
 				s2++;
 			}
 
-			/* Set value to cfg item */
+			/* Set the value to the cfg item */
 			ci->value = intlist;
 
-			/* Delete string */
+			/* Delete the string */
 			g_free(s);
 		    }
 		    break;
@@ -333,10 +371,10 @@ gint CFGFileOpen(
 #undef SEEK_NEXT
 			}
 
-			/* Set value to cfg item */
+			/* Set the value to the cfg item */
 			ci->value = akey_list;
 
-			/* Delete string */
+			/* Delete the string */
 			g_free(s);
 		    }
 		    break;
@@ -413,22 +451,121 @@ gint CFGFileOpen(
 			    SEEK_NEXT
 			}
 
-			/* Set value to cfg item */
+			/* Set the value to the cfg item */
 			ci->value = style;
 
-			/* Delete string */
+			/* Delete the string */
 			g_free(s);
 #undef GET_COLOR
 #undef SEEK_NEXT
 		    }
 		    break;
 
+		  case CFG_ITEM_TYPE_MENU:
+		    s = (gchar *)FGetString(fp);
+		    if(s != NULL)
+		    {
+			gchar **strv;
+			cfg_menu_struct
+			    *menu_src = CFG_MENU(ci->value),
+			    *menu = CFGMenuNew(NULL);
+
+			/* Delete the existing Cfg Item value (if any) */
+			CFGItemResetValue(ci);
+			menu_src = NULL;
+
+			/* Unable to create a new Menu? */
+			if(menu == NULL)
+			{
+			    g_free(s);
+			    break;
+			}
+
+			/* Format:
+			 *
+			 * <label>;<command>;<icon_file>;<description>;<ext_data>;...
+			 *
+			 * Each value is deliminated by a ';'
+			 * character, each menu item occures every
+			 * 5 values
+			 */
+			strv = g_strsplit(s, ";", -1);
+			if(strv != NULL)
+			{
+			    gint i = 0;
+			    const gchar *label,
+					*command,
+					*icon_file,
+					*description,
+					*ext_data;
+
+			    while(strv[i] != NULL)
+			    {
+				label = CFGStripQuotes(strv[i]);
+				i++;
+
+				if(strv[i] != NULL)
+				{
+				    command = CFGStripQuotes(strv[i]);
+				    i++;
+				}
+				else
+				    command = NULL;
+
+				if(strv[i] != NULL)
+				{
+				    icon_file = CFGStripQuotes(strv[i]);
+				    i++;
+				}
+				else
+				    icon_file = NULL;
+
+				if(strv[i] != NULL)
+				{
+				    description = CFGStripQuotes(strv[i]);
+				    i++;
+				}
+				else
+				    description = NULL;
+
+				if(strv[i] != NULL)
+				{
+				    ext_data = CFGStripQuotes(strv[i]);
+				    i++;
+				}
+				else
+				    ext_data = NULL;
+
+				menu->list = g_list_append(
+				    menu->list,
+				    CFGMenuItemNew(
+					label,
+					command,
+					icon_file,
+					description,
+					ext_data
+				    )
+				);
+			    }
+
+			    /* Delete the exploded values */
+			    g_strfreev(strv);
+			}
+
+			/* Set the value to the cfg item */
+			ci->value = menu;
+
+			/* Delete the string */
+			g_free(s);
+		    }
+		    break;
+
 		  default:
 		    /* Unsupported value type */
 		    if(!ignore_all_errors)
 			g_printerr(
 "%s: Warning: Unsupported value type \"%i\" for parameter \"%s\".\n",
-			    filename,
+			    path,
 			    ci->type,
 			    parm
 			);
@@ -440,8 +577,8 @@ gint CFGFileOpen(
 
 	g_free(parm);
 
-	/* Close configuration file */
-	FClose(fp);
+	/* Close the configuration file */
+	fclose(fp);
 
 	return(0);
 }
@@ -458,20 +595,20 @@ gint CFGFileOpen(
  *	-3	System error
  */
 gint CFGFileSave(
-	const gchar *filename, const cfg_item_struct *cfg_list
+	const gchar *path, const cfg_item_struct *cfg_list
 )
 {
-	gint i;
-	gchar *parent;
 	FILE *fp;
 	struct stat stat_buf;
+	gint i;
+	gchar *parent;
 	const cfg_item_struct *ci;
 
-	if(STRISEMPTY(filename) || (cfg_list == NULL))
+	if(STRISEMPTY(path) || (cfg_list == NULL))
 	    return(-2);
 
 	/* Check if the file already exists */
-	if(!stat((const char *)filename, &stat_buf))
+	if(!stat((const char *)path, &stat_buf))
 	{
 #ifdef S_ISDIR
 	    /* Check if it is a directory */
@@ -479,7 +616,7 @@ gint CFGFileSave(
 	    {
 		g_printerr(
 "%s: Is a directory.\n",
-		    filename
+		    path
 		);
 		return(-2);
 	    }
@@ -487,11 +624,11 @@ gint CFGFileSave(
 	}
 
 	/* Get the parent directory and create it as needed */
-	parent = g_dirname(filename);
+	parent = g_dirname(path);
 	if(parent != NULL)
 	{
 	    const mode_t m = umask(0);
-	    gint status;
+	    gint status, error_code;
 
 	    umask(m);
 
@@ -502,38 +639,33 @@ gint CFGFileSave(
 		     S_IRGRP | S_IWGRP | S_IXGRP |
 		     S_IROTH | S_IWOTH | S_IXOTH)
 	    );
-
-	    g_free(parent);
-
-	    if(status)
+	    error_code = (gint)errno;
+	    if(status != 0)
 	    {
 		g_printerr(
-"%s: Unable to create the parent directory.\n",
-		    filename
+		    "%s: %s.\n",
+		    parent, g_strerror(error_code)
 		);
+		g_free(parent);
 		return(-1);
 	    }
+
+	    g_free(parent);
 	}
 
-	/* Open configuration file for writing */
-	fp = fopen((const char *)filename, "wb");
+	/* Open the configuration file for writing */
+	fp = fopen((const char *)path, "wb");
 	if(fp == NULL)
 	{
 	    const gint error_code = (gint)errno;
-	    gchar *error_msg = STRDUP(g_strerror(error_code));
-	    if(error_msg != NULL)
-	    {
-		*error_msg = toupper((int)*error_msg);
-		g_printerr(
-"%s: %s.\n",
-		    filename, error_msg
-		);
-		g_free(error_msg);
-	    }
+	    g_printerr(
+		"%s: %s.\n",
+		path, g_strerror(error_code)
+	    );
 	    return(-1);
 	}
 
-	/* Begin writing configuration file */
+	/* Begin writing the configuration file */
 	i = 0;
 	ci = &cfg_list[i];
 	while((ci->type != 0) &&
@@ -548,6 +680,7 @@ gint CFGFileSave(
 		const cfg_color_struct *color;
 		const cfg_accelkey_list_struct *accelkey_list;
 		const cfg_style_struct *style;
+		const cfg_menu_struct *menu;
 
 		switch(ci->type)
 		{
@@ -722,21 +855,20 @@ gint CFGFileSave(
 		    );
 		    if(accelkey_list != NULL)
 		    {
+			GList *glist;
 			cfg_accelkey_struct *akey;
-			GList *glist = accelkey_list->list;
-			while(glist != NULL)
+
+			for(glist = accelkey_list->list;
+			    glist != NULL;
+			    glist = g_list_next(glist)
+			)
 			{
 			    akey = CFG_ACCELKEY(glist->data);
 			    if(akey == NULL)
-			    {
-				glist = g_list_next(glist);
 				continue;
-			    }
+
 			    if(akey->opid <= 0)
-			    {
-				glist = g_list_next(glist);
 				continue;
-			    }
 
 			    fprintf(
 				fp, "%i %i %i",
@@ -744,8 +876,8 @@ gint CFGFileSave(
 				(int)akey->key,
 				(int)akey->modifiers
 			    );
-			    glist = g_list_next(glist);
-			    if(glist != NULL)
+
+			    if(g_list_next(glist) != NULL)
 				fprintf(fp, "\\\n ");
 			}
 		    }
@@ -818,6 +950,70 @@ gint CFGFileSave(
 		    fprintf(fp, "\n");
 		    break;
 
+		  case CFG_ITEM_TYPE_MENU:
+		    /* Format:
+		     *
+		     * <label>;<command>;<icon_file>;<description>;<ext_data>;...
+		     *
+		     * Each value is deliminated by a ';' character,
+		     * each menu item occures every 5 values
+		     */
+		    menu = CFG_MENU(value);
+		    fprintf(
+			fp,
+			"%s %c ",
+			parm, CFG_DELIMINATOR_CHAR
+		    );
+		    if(menu != NULL)
+		    {
+			gchar *s = NULL, *sv;
+			GList *glist;
+			cfg_menu_item_struct *mi;
+
+			for(glist = menu->list;
+			    glist != NULL;
+			    glist = g_list_next(glist)
+			)
+			{
+			    mi = CFG_MENU_ITEM(glist->data);
+			    if(mi == NULL)
+				continue;
+
+			    sv = g_strdup_printf(
+				"\"%s\";\"%s\";\"%s\";\"%s\";\"%s\"",
+				(mi->label != NULL) ? mi->label : "",
+				(mi->command != NULL) ? mi->command : "",
+				(mi->icon_file != NULL) ? mi->icon_file : "",
+				(mi->description != NULL) ? mi->description : "",
+				(mi->ext_data != NULL) ? mi->ext_data : ""
+			    );
+			    if(s != NULL)
+			    {
+				gchar *s_tmp = g_strconcat(
+				    s,
+				    ";\\\n",
+				    sv,
+				    NULL
+				);
+				g_free(sv);
+				g_free(s);
+				s = s_tmp;
+			    }
+			    else
+			    {
+				s = STRDUP(sv);
+				g_free(sv);
+			    }
+			}
+
+			if(s != NULL)
+			{
+			    fprintf(fp, "%s", s);
+			    g_free(s);
+			}
+		    }
+		    fprintf(fp, "\n");  
+		    break;
 
 		}
 	    }
@@ -826,8 +1022,8 @@ gint CFGFileSave(
 	    ci = &cfg_list[i];
 	}
 
-	/* Close configuration file */
-	FClose(fp);
+	/* Close the configuration file */
+	fclose(fp);
 
 	return(0);
 }
