@@ -8,10 +8,10 @@
 #include "cfg.h"
 
 extern gint CFGFileOpen(
-	const gchar *filename, cfg_item_struct *parm_list
+	const gchar *path, cfg_item_struct *parm_list
 );
 extern gint CFGFileSave(
-	const gchar *filename, const cfg_item_struct *parm_list
+	const gchar *path, const cfg_item_struct *parm_list
 );
 
 #endif	/* CFGFIO_H */
