@@ -27,11 +27,11 @@
  */
 #define PROG_NAME			"Endeavour"
 #define PROG_NAME_FULL			"Endeavour Mark II"
-#define PROG_VERSION			"2.7.5"
+#define PROG_VERSION			"2.8.2"
 
 #define PROG_VERSION_MAJOR		2
-#define PROG_VERSION_MINOR		7
-#define PROG_VERSION_RELEASE		5
+#define PROG_VERSION_MINOR		8
+#define PROG_VERSION_RELEASE		2
 
 /*
  *      Copyright message:
@@ -139,6 +139,7 @@ to the fullest extent of the law.\n"
 #define ENV_VAR_NAME_PRINTCMD			"PRINTCMD"
 #define ENV_VAR_NAME_PRINTCOMMAND		"PRINTCOMMAND"
 #define ENV_VAR_NAME_PRINTER			"PRINTER"
+#define ENV_VAR_NAME_SHELL			"SHELL"
 #define ENV_VAR_NAME_TMPDIR			"TMPDIR"
 #define ENV_VAR_NAME_USER			"USER"
 #define ENV_VAR_NAME_USERNAME			"USERNAME"
@@ -276,7 +277,18 @@ to the fullest extent of the law.\n"
 
 
 /*
- *	Default configuration files (local data dir prefixed):
+ *	Default subdirectories:
+ *
+ *	All values must be relative paths.
+ */
+#define EDV_DEF_ICONS_SUBDIR		"icons"
+#define EDV_DEF_IMAGES_SUBDIR		"images"
+
+/*
+ *	Default configuration files:
+ *
+ *	The local data directory will be prefixed, so all values must
+ *	be relative paths.
  */
 #define EDV_DEF_CONFIG_FILE		"endeavour2.ini"
 #define EDV_DEF_DEVICES_FILE		"devices.ini"
@@ -315,7 +327,8 @@ to the fullest extent of the law.\n"
 /*
  *	Default about image file:
  */
-#define EDV_DEF_ABOUT_IMAGE_FILE	"images/endeavour_about.xpm"
+#define EDV_DEF_ABOUT_IMAGE_FILE	EDV_DEF_IMAGES_SUBDIR \
+ "/endeavour_about.xpm"
 
 
 /*
@@ -324,11 +337,11 @@ to the fullest extent of the law.\n"
  *	These paths are initially set to all new device entries.
  */
 #define EDV_DEF_DEVICE_ICON_SMALL_FILE	EDV_DEF_GLOBAL_DATA_DIR	\
- "/icons/icon_drive_fixed_20x20.xpm"
+ "/" EDV_DEF_ICONS_SUBDIR "/icon_drive_fixed_20x20.xpm"
 #define EDV_DEF_DEVICE_ICON_MEDIUM_FILE	EDV_DEF_GLOBAL_DATA_DIR	\
- "/icons/icon_drive_fixed_32x32.xpm"
+ "/" EDV_DEF_ICONS_SUBDIR "/icon_drive_fixed_32x32.xpm"
 #define EDV_DEF_DEVICE_ICON_LARGE_FILE	EDV_DEF_GLOBAL_DATA_DIR	\
- "/icons/icon_drive_fixed_48x48.xpm"
+ "/" EDV_DEF_ICONS_SUBDIR "/icon_drive_fixed_48x48.xpm"
 
 /*
  *	Default programs:
@@ -630,6 +643,24 @@ to the fullest extent of the law.\n"
 #define EDV_GDKCOLOR_EXECUTABLE_FG_INSENSITIVE	EDV_GDKCOLOR_LIGHT_GREY
 
 
+/*
+ *	Widget Names:
+ */
+#define EDV_BROWSER_TOPLEVEL_WIDGET_NAME	"edv-file-browser"
+#define EDV_BROWSER_CTREE_WIDGET_NAME	"edv-file-browser-ctree"
+#define EDV_BROWSER_CLIST_WIDGET_NAME	"edv-file-browser-clist"
+
+#define EDV_IMBR_TOPLEVEL_WIDGET_NAME	"edv-image-browser"
+#define EDV_IMBR_TLIST_WIDGET_NAME	"edv-image-browser-thumbs-list"
+#define EDV_IMBR_IMGVIEW_WIDGET_NAME	"edv-image-browser-imgview"
+
+#define EDV_ARCHIVER_TOPLEVEL_WIDGET_NAME	"edv-archiver"
+#define EDV_ARCHIVER_CLIST_WIDGET_NAME	"edv-archiver-clist"
+
+#define EDV_RECBIN_TOPLEVEL_WIDGET_NAME	"edv-recbin"
+#define EDV_RECBIN_CLIST_WIDGET_NAME	"edv-recbin-clist"
+
+
 /*
  *	Drag & Drop Target Type Names & Info Values:
  */
@@ -637,13 +668,21 @@ to the fullest extent of the law.\n"
 #define EDV_DND_INFO_TEXT_URI_LIST	1	/* "text/uri-list" */
 #define EDV_DND_INFO_STRING		2	/* "STRING" */
 
-#define EDV_DND_INFO_RECYCLED_OBJECT	3
+#define EDV_DND_INFO_MIME_TYPE		3
+#define EDV_DND_TARGET_MIME_TYPE	"Endeavour2/MimeType"
+#define EDV_DND_INFO_MIME_TYPE_COMMAND	4
+#define EDV_DND_TARGET_MIME_TYPE_COMMAND "Endeavour2/MimeType/Command"
+#define EDV_DND_INFO_DEVICE		5
+#define EDV_DND_TARGET_DEVICE		"Endeavour2/Device"
+
+#define EDV_DND_INFO_RECYCLED_OBJECT	6
 #define EDV_DND_TARGET_RECYCLED_OBJECT	"Endeavour2/RecycledObject"
 
-#define EDV_DND_INFO_ARCHIVE_OBJECT	4
+#define EDV_DND_INFO_ARCHIVE_OBJECT	7
 #define EDV_DND_TARGET_ARCHIVE_OBJECT	"Endeavour2/ArchiveObject"
 
 
+
 /*
  *	List of disk object paths (always absolute path) that may never
  *	be deleted. Each entry consists of three pointers, the first
@@ -762,7 +801,7 @@ Usage: endeavour2 [path] [options] [GUI_options]\n\
      votre GUI, consulter le manuel de votre GUI pour les options\n\
     disponibles.\n\
 \n"
-#else
+#else	/* PROG_LANGUAGE_ENGLISH */
 #define PROG_HELP_MESG  "\
 Usage: endeavour2 [path] [extra] [options] [GUI_options]\n\
 \n\
@@ -773,7 +812,7 @@ Usage: endeavour2 [path] [extra] [options] [GUI_options]\n\
 \n\
     The [options] can be any of the following:\n\
 \n\
-        --config <file>         Load configuration from <file>.\n\
+        --config <file>         Open the configuration from <file>.\n\
         --rcfile                Same as --config.\n\
         -f                      Same as --config.\n\
 \n\
@@ -794,8 +833,11 @@ Usage: endeavour2 [path] [extra] [options] [GUI_options]\n\
         --find                  Start up with the Find Window.\n\
         --about                 Start up with the About Dialog.\n\
 \n\
-        --help                  Prints (this) help screen and exits.\n\
-        --version               Prints version information and exits.\n\
+        --safe-mode             Run in \"safe mode\".\n\
+        --safe                  Same as --safe-mode.\n\
+\n\
+        --help                  Prints this help screen and exits.\n\
+        --version               Prints the version information and exits.\n\
 \n\
     The [GUI_options] can be any of the following\n\
 \n\
