@@ -20,55 +20,38 @@
 #include "images/icon_clear_20x20.xpm"
 
 
-/*
- *	Message Dialog:
- */
-typedef struct {
-
-	GtkWidget	*toplevel;
-	GtkAccelGroup	*accelgrp;
-	gint		freeze_count;
-	GdkFont		*font;
-
-	GtkWidget	*text,
-			*ok_btn,
-			*no_more_messages_btn,
-			*clear_btn;
-} ConMsgDlg;
-#define CON_MSG_DLG(p)		((ConMsgDlg *)(p))
-
-
-static gchar		*prog_name = NULL,
-			*out_dlg_title = NULL,
-			*err_dlg_title = NULL;
-static gint		terminal_columns = 80,
-			terminal_lines = 20;
-static gboolean		out_show = TRUE,
-			err_show = TRUE;
-static gint		out_rfd = -1,
-			out_wfd = -1,
-			err_rfd = -1,
-			err_wfd = -1;
-static guint		msg_check_toid = 0;
-static ConMsgDlg	*out_dlg = NULL,
-			*err_dlg = NULL;
+typedef struct _ConMsgDlg		ConMsgDlg;
+#define CON_MSG_DLG(p)			((ConMsgDlg *)(p))
+typedef struct _ConMsgCore		ConMsgCore;
+#define CON_MSG_CORE(p)			((ConMsgCore *)(p))
 
 
+/* Callbacks */
 static gint ConMsgDlgEventCB(
 	GtkWidget *widget, GdkEvent *event, gpointer data
 );
+static gint ConMsgDlgTextEventCB(
+	GtkWidget *widget, GdkEvent *event, gpointer data
+);
 static void ConMsgDlgOKCB(GtkWidget *widget, gpointer data);
 static void ConMsgDlgNoMoreMessagesCB(GtkWidget *widget, gpointer data);
 static void ConMsgDlgClearCB(GtkWidget *widget, gpointer data);
+
+/* Console Message Dialog */
 static ConMsgDlg *ConMsgDlgNew(
+	ConMsgCore *core,
 	const gchar *title, guint8 **icon_data,
-	gint columns, gint lines
+	const gchar *font_name,
+	const gint columns,
+	const gint lines
 );
 static void ConMsgDlgMsgAppend(ConMsgDlg *d, const gchar *s);
+static void ConMsgDlgUpdateWidgets(ConMsgDlg *d);
 static void ConMsgDlgDelete(ConMsgDlg *d);
 
 static gboolean ConMsgGetMsg(
-	gint fd, ConMsgDlg **d_ptr,
+	ConMsgCore *core,
+	const gint fd, ConMsgDlg **d_ptr,
 	const gchar *title, guint8 **icon_data
 );
 static gint ConMsgCheckTimeoutCB(gpointer data);
@@ -76,17 +59,19 @@ static gint ConMsgBuffersReset(void);
 
 gint ConMsgInit(
 	const gchar *name,	/* Program name */
-	gint columns,		/* Can be 0 to indicate use env var $COLUMNS */
-	gint lines,		/* Can be 0 to indicate use env var $LINES */
-	gboolean show_stdout,
-	gboolean show_stderr
+	const gchar *font_name,
+	const gint columns,	/* Can be 0 to indicate use env var $COLUMNS */
+	const gint lines,	/* Can be 0 to indicate use env var $LINES */
+	const gboolean show_stdout,
+	const gboolean show_stderr
 );
 gint ConMsgReset(
 	const gchar *name,	/* Program name */
-	gint columns,		/* Can be 0 to indicate use env var $COLUMNS */
-	gint lines,		/* Can be 0 to indicate use env var $LINES */
-	gboolean show_stdout,
-	gboolean show_stderr
+	const gchar *font_name,
+	const gint columns,	/* Can be 0 to indicate use env var $COLUMNS */
+	const gint lines,	/* Can be 0 to indicate use env var $LINES */
+	const gboolean show_stdout,
+	const gboolean show_stderr
 );
 void ConMsgShutdown(void);
 
@@ -94,6 +79,50 @@ void ConMsgStdOutDlgMap(void);
 void ConMsgStdErrDlgMap(void);
 
 
+/*
+ *	Message Dialog:
+ */
+struct _ConMsgDlg {
+
+	GtkWidget	*toplevel;
+	GtkAccelGroup	*accelgrp;
+	gint		freeze_count;
+	GdkFont		*font;
+	ConMsgCore	*core;
+
+	GtkWidget	*text,
+			*ok_btn,
+			*no_more_messages_btn,
+			*clear_btn;
+
+};
+
+/*
+ *	Core:
+ */
+struct _ConMsgCore {
+
+	gchar		*prog_name,
+			*out_dlg_title,
+			*err_dlg_title,
+			*font_name;
+	gint		terminal_columns,
+			terminal_lines;
+	gboolean	out_show,
+			err_show;
+	gint		out_rfd,
+			out_wfd,
+			err_rfd,
+			err_wfd;
+	guint		msg_check_toid;
+	ConMsgDlg	*out_dlg,
+			*err_dlg;
+
+};
+
+static ConMsgCore	*con_msg_core = NULL;
+
+
 #define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
@@ -110,7 +139,7 @@ void ConMsgStdErrDlgMap(void);
 
 
 /*
- *	Message Dialog toplevel GtkWindow event signal callback.
+ *	Toplevel GtkWindow event signal callback.
  */
 static gint ConMsgDlgEventCB(
 	GtkWidget *widget, GdkEvent *event, gpointer data
@@ -121,11 +150,13 @@ static gint ConMsgDlgEventCB(
 	if((widget == NULL) || (event == NULL) || (d == NULL))
 	    return(status);
 
+	if(d->freeze_count > 0)
+	    return(status);
+
 	switch((gint)event->type)
 	{
 	  case GDK_DELETE:
-	    ConMsgDlgOKCB(d->ok_btn, d);
-	    d = NULL;
+	    ConMsgDlgOKCB(widget, data);
 	    status = TRUE;
 	    break;
 	}
@@ -134,53 +165,179 @@ static gint ConMsgDlgEventCB(
 }
 
 /*
- *	Message Dialog OK callback.
+ *	GtkText event signal callback.
+ */
+static gint ConMsgDlgTextEventCB(
+	GtkWidget *widget, GdkEvent *event, gpointer data
+)
+{
+	gint status = FALSE;
+	GdkEventButton *button;
+	GtkText *text;
+	ConMsgDlg *d = CON_MSG_DLG(data);
+	if((widget == NULL) || (event == NULL) || (d == NULL))
+	    return(status);
+
+	if(d->freeze_count > 0)
+	    return(status);
+
+	text = GTK_TEXT(widget);
+
+	switch((gint)event->type)
+	{
+	  case GDK_BUTTON_PRESS:
+	    button = (GdkEventButton *)event;
+	    switch(button->button)
+	    {
+	      case GDK_BUTTON4:
+		/* Scroll up */
+		if(text->vadj != NULL)
+		{
+		    GtkAdjustment *adj = text->vadj;
+		    const gfloat inc = MAX(
+			(0.25f * adj->page_size),
+			adj->step_increment  
+		    );
+		    gfloat v = adj->value - inc;
+		    if(v > (adj->upper - adj->page_size))
+			v = adj->upper - adj->page_size;
+		    if(v < adj->lower)
+			v = adj->lower;
+		    gtk_adjustment_set_value(adj, v);
+		}     
+		/* Need to mark the GtkText button as 0 or else it will
+		 * keep marking
+		 */
+		text->button = 0;
+		gtk_grab_remove(widget);
+		gtk_signal_emit_stop_by_name(
+		    GTK_OBJECT(widget), "button_press_event"
+		);
+		status = TRUE;
+		break;
+
+	      case GDK_BUTTON5:
+		/* Scroll down */
+		if(text->vadj != NULL)
+		{
+		    GtkAdjustment *adj = text->vadj;
+		    const gfloat inc = MAX(
+			(0.25f * adj->page_size),
+			adj->step_increment
+		    );
+		    gfloat v = adj->value + inc;
+		    if(v > (adj->upper - adj->page_size))
+			v = adj->upper - adj->page_size;
+		    if(v < adj->lower)
+			v = adj->lower;
+		    gtk_adjustment_set_value(adj, v);
+		}
+		/* Need to mark the GtkText button as 0 or else it will
+		 * keep marking
+		 */
+		text->button = 0;
+		gtk_grab_remove(widget);
+		gtk_signal_emit_stop_by_name(
+		    GTK_OBJECT(widget), "button_press_event"
+		);
+		status = TRUE;
+		break;
+	    }
+	    break;
+
+	  case GDK_BUTTON_RELEASE:
+	    button = (GdkEventButton *)event;
+	    switch(button->button)
+	    {
+	      case GDK_BUTTON4:
+		/* Need to mark the GtkText button as 0 or else it will
+		 * keep marking
+		 */
+		text->button = 0;
+		gtk_grab_remove(widget);
+		gtk_signal_emit_stop_by_name(
+		    GTK_OBJECT(widget), "button_release_event"
+		);
+		status = TRUE;
+		break;
+	      case GDK_BUTTON5:
+		/* Need to mark the GtkText button as 0 or else it will
+		 * keep marking
+		 */
+		text->button = 0;
+		gtk_grab_remove(widget);
+		gtk_signal_emit_stop_by_name(
+		    GTK_OBJECT(widget), "button_release_event"
+		);
+		status = TRUE;
+		break;
+	    }
+	    break;
+	}
+
+	return(status);
+}
+
+/*
+ *	OK callback.
  */
 static void ConMsgDlgOKCB(GtkWidget *widget, gpointer data)
 {
+	ConMsgCore *core;
 	ConMsgDlg *d = CON_MSG_DLG(data);
 	if(d == NULL)
 	    return;
 
-	/* Reset the global variable that references this Message Dialog
-	 * and delete this Message Dialog
+	if(d->freeze_count > 0)
+	    return;
+
+	core = d->core;
+
+	/* Reset the global variable that references this Message
+	 * Dialog and delete this Message Dialog
 	 */
-	if(d == out_dlg)
-	    out_dlg = NULL;
-	else if(d == err_dlg)
-	    err_dlg = NULL;
+	if(d == core->out_dlg)
+	    core->out_dlg = NULL;
+	else if(d == core->err_dlg)
+	    core->err_dlg = NULL;
 
 	ConMsgDlgDelete(d);
 }
 
 /*
- *	Message Dialog No More Messages callback.
+ *	No More Messages callback.
  */
 static void ConMsgDlgNoMoreMessagesCB(GtkWidget *widget, gpointer data)
 {                          
+	ConMsgCore *core;
 	ConMsgDlg *d = CON_MSG_DLG(data);
 	if(d == NULL)
 	    return;
 
+	if(d->freeze_count > 0)
+	    return;
+
+	core = d->core;
+
 	/* Reset the global variable that references this Message Dialog
 	 * and delete this Message Dialog
 	 */
-	if(d == out_dlg)
+	if(d == core->out_dlg)
 	{
-	    out_show = FALSE;
-	    out_dlg = NULL;
+	    core->out_show = FALSE;
+	    core->out_dlg = NULL;
 	}
-	else if(d == err_dlg)
+	else if(d == core->err_dlg)
 	{
-	    err_show = FALSE;
-	    err_dlg = NULL;
+	    core->err_show = FALSE;
+	    core->err_dlg = NULL;
 	}
 
 	ConMsgDlgDelete(d);
 }
 
 /*
- *	Message Dialog Clear callback.
+ *	Clear callback.
  */
 static void ConMsgDlgClearCB(GtkWidget *widget, gpointer data)
 {
@@ -189,7 +346,16 @@ static void ConMsgDlgClearCB(GtkWidget *widget, gpointer data)
 	if(d == NULL)
 	    return;
 
+	if(d->freeze_count > 0)
+	    return;
+
 	text = GTK_TEXT(d->text);
+
+	d->freeze_count++;
+
+	while(gtk_events_pending() > 0)
+	    gtk_main_iteration();
+
 	gtk_text_freeze(text);
 	gtk_text_set_point(text, 0);
 	gtk_text_forward_delete(
@@ -197,6 +363,13 @@ static void ConMsgDlgClearCB(GtkWidget *widget, gpointer data)
 	    gtk_text_get_length(text)
 	);
 	gtk_text_thaw(text);
+
+	while(gtk_events_pending() > 0)
+	    gtk_main_iteration();
+
+	ConMsgDlgUpdateWidgets(d);
+
+	d->freeze_count--;
 }
 
 
@@ -204,32 +377,42 @@ static void ConMsgDlgClearCB(GtkWidget *widget, gpointer data)
  *	Creates a new Message Dialog.
  */
 static ConMsgDlg *ConMsgDlgNew(
+	ConMsgCore *core,
 	const gchar *title, guint8 **icon_data,
-	gint columns, gint lines
+	const gchar *font_name,
+	const gint columns,
+	const gint lines
 )
 {
 	const gint	border_major = 5,
 			border_minor = 2;
-	gint font_width, font_height;
+	gint		_columns = columns,
+			_lines = lines,
+			font_width, font_height;
 	GdkFont *font;
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
 	GtkWidget *w, *parent, *parent2, *toplevel, *sb;
-	ConMsgDlg *d = CON_MSG_DLG(g_malloc0(sizeof(ConMsgDlg)));
-	if(d == NULL)
+	GtkText *text;
+	ConMsgDlg *d;
+
+	if(core == NULL)
 	    return(NULL);
 
-	if(columns <= 0)
-	    columns = 80;
-	if(lines <= 0)
-	    lines = 20;
+	if(_columns <= 0)
+	    _columns = 80;
+	if(_lines <= 0)
+	    _lines = 20;
 
-	/* Load default font (must be a fixed-width font) and get its
-	 * size
+	/* Get the default font (must be a fixed-width font) and
+	 * get its size
 	 */
-	font = gdk_font_load(
+	if(STRISEMPTY(font_name))
+	    font = gdk_font_load(
 "-adobe-courier-medium-r-normal-*-12-*-*-*-m-*-iso8859-1"
-	);
+	    );
+	else
+	    font = gdk_font_load(font_name);
 	if(font == NULL)
 	    font = gdk_font_load("6x12");
 	if(font != NULL)
@@ -245,24 +428,30 @@ static ConMsgDlg *ConMsgDlgNew(
 	    font_height = 12;
 	}
 
-	/* Reset values */
-	d->accelgrp = accelgrp = gtk_accel_group_new();
-	d->freeze_count = 0;  
-	d->font = font;
+	/* Create the Message Dialog */
+	d = CON_MSG_DLG(g_malloc0(sizeof(ConMsgDlg))); 
+	if(d == NULL)  
+	    return(NULL);
 
-	/* Toplevel */
-	d->toplevel = toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
+	d->accelgrp = accelgrp = gtk_accel_group_new();
+	d->freeze_count = 0;
+	d->font = font;
+	d->core = core;
+
+	d->freeze_count++;
+
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_widget_set_name(w, CON_MSG_TOPLEVEL_WIDGET_NAME);
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_MOUSE);
-	gtk_window_set_policy(
-	    GTK_WINDOW(w), TRUE, TRUE, TRUE
-	);
 	if(title != NULL)
 	    gtk_window_set_title(GTK_WINDOW(w), title);
-	if(!STRISEMPTY(prog_name))
+	if(!STRISEMPTY(core->prog_name))
 	    gtk_window_set_wmclass(
-		GTK_WINDOW(w), "dialog", prog_name
+		GTK_WINDOW(w), "dialog", core->prog_name
 	    );
-
 	gtk_widget_realize(w);
 	window = w->window;
 	if(window != NULL)
@@ -285,13 +474,13 @@ static ConMsgDlg *ConMsgDlgNew(
 	gtk_window_add_accel_group(GTK_WINDOW(w), accelgrp);
 	parent = w;
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	w = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	gtk_widget_show(w);
 	parent = w;
 
-	/* Text table */
+	/* Text GtkTable */
 	w = gtk_table_new(2, 2, FALSE);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_table_set_row_spacing(GTK_TABLE(w), 0, border_minor);
@@ -301,19 +490,29 @@ static ConMsgDlg *ConMsgDlgNew(
 	parent2 = w;
 	/* Text */
 	d->text = w = gtk_text_new(NULL, NULL);
+	text = GTK_TEXT(w);
+	gtk_widget_set_name(w, CON_MSG_TEXT_WIDGET_NAME);
 	gtk_widget_set_usize(
 	    w,
 	    MAX(
-		(font_width * columns) + (2 * border_major),
+		(font_width * _columns) + (2 * border_major),
 		300
 	    ),
 	    MAX(
-		(font_height * lines) + (2 * border_major),
+		(font_height * _lines) + (2 * border_major),
 		100
 	    )
-	);	    
-	gtk_text_set_editable(GTK_TEXT(w), FALSE);
-	gtk_text_set_word_wrap(GTK_TEXT(w), TRUE);
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "button_press_event",
+	    GTK_SIGNAL_FUNC(ConMsgDlgTextEventCB), d
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "button_release_event",
+	    GTK_SIGNAL_FUNC(ConMsgDlgTextEventCB), d
+	);
+	gtk_text_set_editable(text, FALSE);
+	gtk_text_set_word_wrap(text, TRUE);
 	gtk_table_attach(
 	    GTK_TABLE(parent2), w,
 	    0, 1, 0, 1,
@@ -321,8 +520,9 @@ static ConMsgDlg *ConMsgDlgNew(
 	    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 	    0, 0
 	);
+	GUIEditableEndowPopupMenu(w, GUI_EDITABLE_POPUP_MENU_READ_ONLY);
 	gtk_widget_show(w);
-	/* Vertical scroll bar */
+	/* Vertical GtkScrollBar */
 	sb = gtk_vscrollbar_new(GTK_TEXT(w)->vadj);
 	gtk_table_attach(   
 	    GTK_TABLE(parent2), sb,
@@ -334,13 +534,13 @@ static ConMsgDlg *ConMsgDlgNew(
 	gtk_widget_show(sb);
 
 
-	/* Separator */
+	/* GtkSeparator */
 	w = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);                                     
+	gtk_widget_show(w);
 
 
-	/* Hbox for buttons */
+	/* Buttons GtkHBox */
 	w = gtk_hbox_new(TRUE, 0);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
@@ -412,16 +612,17 @@ static ConMsgDlg *ConMsgDlgNew(
 	    GTK_OBJECT(w), "clicked"
 	);
 	GUIButtonLabelUnderline(w, GDK_l);
-	gtk_widget_show(w);  
+	gtk_widget_show(w);
 
 
-
-	/* Map Message Dialog */
+	/* Map the dialog */
 	gtk_widget_show_raise(d->toplevel);
 
 	/* Have the OK button grab focus */
 	gtk_widget_grab_focus(d->ok_btn);
-	gtk_widget_grab_default(d->ok_btn);  
+	gtk_widget_grab_default(d->ok_btn);
+
+	d->freeze_count--;
 
 	return(d);
 }
@@ -432,12 +633,14 @@ static ConMsgDlg *ConMsgDlgNew(
 static void ConMsgDlgMsgAppend(ConMsgDlg *d, const gchar *s)
 {
 	GtkAdjustment *adj;
+	GtkWidget *w;
 	GtkEditable *editable;
 	GtkText *text;
-	GtkWidget *w = (d != NULL) ? d->text : NULL;
-	if((w == NULL) || (s == NULL))
+
+	if((d == NULL) || (s == NULL))
 	    return;
 
+	w = d->text;
 	editable = GTK_EDITABLE(w);
 	text = GTK_TEXT(w);
 
@@ -455,6 +658,22 @@ static void ConMsgDlgMsgAppend(ConMsgDlg *d, const gchar *s)
 	    );
 }
 
+/*
+ *	Updates the Message Dialog's widget values.
+ */
+static void ConMsgDlgUpdateWidgets(ConMsgDlg *d)
+{
+	GtkEditable *editable;
+	GtkText *text;
+
+	if(d == NULL)
+	    return;
+
+	editable = GTK_EDITABLE(d->text);
+	text = GTK_TEXT(editable);
+
+}
+
 /*
  *	Deletes the Message Dialog.
  */
@@ -463,13 +682,16 @@ static void ConMsgDlgDelete(ConMsgDlg *d)
 	if(d == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(d->text);
-	GTK_WIDGET_DESTROY(d->ok_btn);
-	GTK_WIDGET_DESTROY(d->no_more_messages_btn);
-	GTK_WIDGET_DESTROY(d->clear_btn);
+	gtk_widget_hide(d->toplevel);
+
+	d->freeze_count++;
+
 	GTK_WIDGET_DESTROY(d->toplevel);
 	GTK_ACCEL_GROUP_UNREF(d->accelgrp);
 	GDK_FONT_UNREF(d->font);
+
+	d->freeze_count--;
+
 	g_free(d);
 }
 
@@ -484,12 +706,13 @@ static void ConMsgDlgDelete(ConMsgDlg *d)
  *	Returns TRUE if data was processed.
  */
 static gboolean ConMsgGetMsg(
-	gint fd, ConMsgDlg **d_ptr,
+	ConMsgCore *core,
+	const gint fd, ConMsgDlg **d_ptr,
 	const gchar *title, guint8 **icon_data
 )
 {
-	gint buf_len, bytes_read, total_bytes_read = 0;
 	struct stat stat_buf;
+	gint buf_len, bytes_read, total_bytes_read = 0;
 	gchar *buf;
  
 	/* Check if the specified descriptor is valid and get its
@@ -501,7 +724,11 @@ static gboolean ConMsgGetMsg(
 	/* Get block size for optimul reading and allocate the read
 	 * buffer
 	 */
+#if defined(_WIN32)
+	buf_len = 1024;
+#else
 	buf_len = (gint)stat_buf.st_blksize;
+#endif
 	if(buf_len <= 0)
 	    buf_len = 1024;
 	buf = (gchar *)g_malloc(buf_len * sizeof(gchar));
@@ -531,8 +758,10 @@ static gboolean ConMsgGetMsg(
 		ConMsgDlg *d = *d_ptr;
 		if(d == NULL)
 		    *d_ptr = d = ConMsgDlgNew(
+			core,
 			title, icon_data,
-			terminal_columns, terminal_lines
+			core->font_name,
+			core->terminal_columns, core->terminal_lines
 		    );
 		ConMsgDlgMsgAppend(d, buf);
 	    }
@@ -558,19 +787,25 @@ static gboolean ConMsgGetMsg(
  */
 static gint ConMsgCheckTimeoutCB(gpointer data)
 {
+	ConMsgCore *core = CON_MSG_CORE(data);
+	if(core == NULL)
+	    return(FALSE);
+
 	/* Check for data on stdout and stderr, if there is data then
 	 * read it and display it on the Message Dialogs
 	 */
 	ConMsgGetMsg(
-	    out_rfd,
-	    out_show ? &out_dlg : NULL,
-	    out_dlg_title,
+	    core,
+	    core->out_rfd,
+	    core->out_show ? &core->out_dlg : NULL,
+	    core->out_dlg_title,
 	    (guint8 **)icon_info_32x32_xpm
 	);
 	ConMsgGetMsg(
-	    err_rfd,
-	    err_show ? &err_dlg : NULL,
-	    err_dlg_title,
+	    core,
+	    core->err_rfd,
+	    core->err_show ? &core->err_dlg : NULL,
+	    core->err_dlg_title,
 	    (guint8 **)icon_warning_32x32_xpm
 	);
 
@@ -582,37 +817,41 @@ static gint ConMsgCheckTimeoutCB(gpointer data)
  */
 static gint ConMsgBuffersReset(void)
 {
+	ConMsgCore *core = con_msg_core;
+	if(core == NULL)
+	    return(-1);
+
 	/* Close existing message buffers */
-	CLOSE(out_rfd);
-	out_rfd = -1;
-	CLOSE(out_wfd);
-	out_wfd = -1;
-	CLOSE(err_rfd);
-	err_rfd = -1;
-	CLOSE(err_wfd);
-	err_wfd = -1;  
+	CLOSE(core->out_rfd);
+	core->out_rfd = -1;
+	CLOSE(core->out_wfd);
+	core->out_wfd = -1;
+	CLOSE(core->err_rfd);
+	core->err_rfd = -1;
+	CLOSE(core->err_wfd);
+	core->err_wfd = -1;
 
 	/* Create new message buffers */
-	if(out_show)
+	if(core->out_show)
 	{
 	    gint fdpair[2];
 	    if(!pipe(fdpair))
 	    {
-		out_rfd = fdpair[0];
-		out_wfd = fdpair[1];
-		dup2(out_wfd, fileno(stdout));
-		fcntl(out_rfd, F_SETFL, O_NONBLOCK);
+		core->out_rfd = fdpair[0];
+		core->out_wfd = fdpair[1];
+		dup2(core->out_wfd, fileno(stdout));
+		fcntl(core->out_rfd, F_SETFL, O_NONBLOCK);
 	    }
 	}
-	if(err_show)
+	if(core->err_show)
 	{
 	    gint fdpair[2];
 	    if(!pipe(fdpair))
 	    {
-		err_rfd = fdpair[0];
-		err_wfd = fdpair[1];
-		dup2(err_wfd, fileno(stderr));
-		fcntl(err_rfd, F_SETFL, O_NONBLOCK);
+		core->err_rfd = fdpair[0];
+		core->err_wfd = fdpair[1];
+		dup2(core->err_wfd, fileno(stderr));
+		fcntl(core->err_rfd, F_SETFL, O_NONBLOCK);
 	    }
 	}
 
@@ -625,77 +864,90 @@ static gint ConMsgBuffersReset(void)
  */
 gint ConMsgInit(
 	const gchar *name,	/* Program name */
-	gint columns,		/* Can be 0 to indicate use env var $COLUMNS */ 
-	gint lines,		/* Can be 0 to indicate use env var $LINES */
-	gboolean show_stdout,
-	gboolean show_stderr
+	const gchar *font_name,
+	const gint columns,	/* Can be 0 to indicate use env var $COLUMNS */
+	const gint lines,	/* Can be 0 to indicate use env var $LINES */
+	const gboolean show_stdout,
+	const gboolean show_stderr
 )
 {
-	/* Set program name */
-	g_free(prog_name);
-	prog_name = STRDUP(name);
+	ConMsgCore *core;
 
-	/* Set dialog titles */
-	g_free(out_dlg_title);
-	if(STRISEMPTY(prog_name))
-	    out_dlg_title = STRDUP("Message");
-	else
-	    out_dlg_title = g_strdup_printf("%s - Message", prog_name);
-	g_free(err_dlg_title);
-	if(STRISEMPTY(prog_name))
-	    err_dlg_title = STRDUP("Error Message");
-	else
-	    err_dlg_title = g_strdup_printf("%s - Error Message", prog_name);
+	/* Already initialized? */
+	if(con_msg_core != NULL)
+	    return(0);
 
-	/* Set terminal geometry */
+	/* Create a new Console Message Core */
+	con_msg_core = core = CON_MSG_CORE(g_malloc0(sizeof(ConMsgCore)));
+	if(core == NULL)
+	    return(-3);
+
+	core->prog_name = STRDUP(name);
+	core->font_name = STRDUP(font_name);
+
+	/* Set the dialog titles */
+	if(STRISEMPTY(core->prog_name))
+	    core->out_dlg_title = STRDUP("Message");
+	else
+	    core->out_dlg_title = g_strconcat(
+		core->prog_name,
+		" - Message",
+		NULL
+	    );
+
+	if(STRISEMPTY(core->prog_name))
+	    core->err_dlg_title = STRDUP("Error Message");
+	else
+	    core->err_dlg_title = g_strconcat(
+		core->prog_name,
+		" - Error Message",
+		NULL
+	    );
+
+	/* Set the terminal geometry */
 	if(columns > 0)
 	{
-	    terminal_columns = columns;
+	    core->terminal_columns = columns;
 	}
 	else
 	{
 	    const gchar *s = g_getenv("COLUMNS");
 	    if(s != NULL)
-		terminal_columns = ATOI(s);
+		core->terminal_columns = ATOI(s);
 	}
 	if(lines > 0)
 	{
-	    terminal_lines = lines;
+	    core->terminal_lines = lines;
 	}
 	else
 	{
 	    const gchar *s = g_getenv("LINES");
 	    if(s != NULL)
-		terminal_lines = ATOI(s);
+		core->terminal_lines = ATOI(s);
 	}
 
 	/* Set show stdout & stderr options */
-	out_show = show_stdout;
-	err_show = show_stderr;
+	core->out_show = show_stdout;
+	core->err_show = show_stderr;
 
-	/* Reset buffer descriptors */
-	out_rfd = -1;
-	out_wfd = -1;
-	err_rfd = -1;
-	err_wfd = -1;
+	/* Reset the buffer descriptors */
+	core->out_rfd = -1;
+	core->out_wfd = -1;
+	core->err_rfd = -1;
+	core->err_wfd = -1;
 
-	/* Reset Message Dialogs */
-	out_dlg = NULL;
-	err_dlg = NULL;
+	/* Reset the Message Dialogs */
+	core->out_dlg = NULL;
+	core->err_dlg = NULL;
 
 	/* Create message buffers for the first time */
 	ConMsgBuffersReset();
 
-	/* Set console message check timeout callback */
-	GTK_TIMEOUT_REMOVE(msg_check_toid);
-	msg_check_toid = gtk_timeout_add(
+	/* Set the console message check timeout callback */
+	core->msg_check_toid = gtk_timeout_add(
 	    1000l,
-	    ConMsgCheckTimeoutCB, NULL
+	    ConMsgCheckTimeoutCB, core
 	);
-#if 0
-g_printerr(PROG_USAGE_MESG);
-g_printerr(PROG_COPYRIGHT);
-#endif
 
 	return(0);
 }
@@ -705,70 +957,87 @@ g_printerr(PROG_COPYRIGHT);
  */
 gint ConMsgReset(
 	const gchar *name,	/* Program name */
-	gint columns,		/* Can be 0 to indicate use env var $COLUMNS */
-	gint lines,		/* Can be 0 to indicate use env var $LINES */
-	gboolean show_stdout,
-	gboolean show_stderr
+	const gchar *font_name,
+	const gint columns,	/* Can be 0 to indicate use env var $COLUMNS */
+	const gint lines,	/* Can be 0 to indicate use env var $LINES */
+	const gboolean show_stdout,
+	const gboolean show_stderr
 )
 {
-	/* Set program name */
-	g_free(prog_name);
-	prog_name = STRDUP(name);
+	ConMsgCore *core = con_msg_core;
+	if(core == NULL)
+	    return(-1);
 
-	/* Set dialog titles */
-	g_free(out_dlg_title);
-	if(STRISEMPTY(prog_name))
-	    out_dlg_title = STRDUP("Message");
+	/* Set the program name */
+	g_free(core->prog_name);
+	core->prog_name = STRDUP(name);
+
+	/* Set the font name */
+	g_free(core->font_name);
+	core->font_name = STRDUP(font_name);
+
+	/* Set the dialog titles */
+	g_free(core->out_dlg_title);
+	if(STRISEMPTY(core->prog_name))
+	    core->out_dlg_title = STRDUP("Message");
 	else
-	    out_dlg_title = g_strdup_printf("%s - Message", prog_name);
-	g_free(err_dlg_title);
-	if(STRISEMPTY(prog_name))
-	    err_dlg_title = STRDUP("Error Message");
+	    core->out_dlg_title = g_strconcat(
+		core->prog_name,
+		" - Message",
+		NULL
+	    );
+	g_free(core->err_dlg_title);
+	if(STRISEMPTY(core->prog_name))
+	    core->err_dlg_title = STRDUP("Error Message");
 	else
-	    err_dlg_title = g_strdup_printf("%s - Error Message", prog_name);
+	    core->err_dlg_title = g_strconcat(
+		core->prog_name,
+		" - Error Message",
+		NULL
+	    );
 
 	/* Set terminal geometry */
 	if(columns > 0)
 	{
-	    terminal_columns = columns;
+	    core->terminal_columns = columns;
 	}
 	else
 	{   
 	    const gchar *s = g_getenv("COLUMNS");
 	    if(s != NULL)
-		terminal_columns = ATOI(s);
+		core->terminal_columns = ATOI(s);
 	}
 	if(lines > 0)
 	{
-	    terminal_lines = lines;
+	    core->terminal_lines = lines;
 	}
 	else
 	{   
 	    const gchar *s = g_getenv("LINES");
 	    if(s != NULL)
-		terminal_lines = ATOI(s);
+		core->terminal_lines = ATOI(s);
 	}
 
-	/* Set show stdout & stderr options */
-	out_show = show_stdout;
-	err_show = show_stderr;
+	/* Set the show stdout & stderr options */
+	core->out_show = show_stdout;
+	core->err_show = show_stderr;
 
-	/* Delete Message Dialogs so that they are created next time
-	 * with the reset values
+	/* Delete the Message Dialogs so that they are created next
+	 * time with the reset values
 	 */
-	ConMsgDlgDelete(out_dlg);
-	out_dlg = NULL;
-	ConMsgDlgDelete(err_dlg);
-	err_dlg = NULL;
+	ConMsgDlgDelete(core->out_dlg);
+	core->out_dlg = NULL;
+	ConMsgDlgDelete(core->err_dlg);
+	core->err_dlg = NULL;
 
-	/* Recreate message buffers */
+	/* Recreate the message buffers */
 	ConMsgBuffersReset();
 
-	/* Reset console message check timeout callback */
-	GTK_TIMEOUT_REMOVE(msg_check_toid);
-	msg_check_toid = gtk_timeout_add(
+	/* Reset the console message check timeout callback */
+	GTK_TIMEOUT_REMOVE(core->msg_check_toid);
+	core->msg_check_toid = gtk_timeout_add(
 	    1000l,
-	    ConMsgCheckTimeoutCB, NULL
+	    ConMsgCheckTimeoutCB, core
 	);
 
 	return(0);
@@ -779,41 +1048,42 @@ gint ConMsgReset(
  */
 void ConMsgShutdown(void)
 {
-	/* Wait for all Message Dialogs to be unmapped */
+	ConMsgCore *core = con_msg_core;
+	if(core == NULL)
+	    return;
+
+	con_msg_core = NULL;
+
+	/* Wait for all the Message Dialogs to be unmapped */
 	while(gtk_events_pending() > 0)
 	    gtk_main_iteration();
-	while((out_dlg != NULL) || (err_dlg != NULL))
+	while((core->out_dlg != NULL) || (core->err_dlg != NULL))
 	{
 	    if(gtk_events_pending() > 0)
 		gtk_main_iteration();
 	}
 
-	/* Remove console message check timeout callback */
-	GTK_TIMEOUT_REMOVE(msg_check_toid);
-	msg_check_toid = 0;
+	/* Remove the console message check timeout callback */
+	GTK_TIMEOUT_REMOVE(core->msg_check_toid);
+	core->msg_check_toid = 0;
 
-	/* Delete Message Dialogs */
-	ConMsgDlgDelete(out_dlg);
-	out_dlg = NULL;
-	ConMsgDlgDelete(err_dlg);
-	err_dlg = NULL;
+	/* Delete the Message Dialogs */
+	ConMsgDlgDelete(core->out_dlg);
+	core->out_dlg = NULL;
+	ConMsgDlgDelete(core->err_dlg);
+	core->err_dlg = NULL;
 
-	/* Close existing message buffers */
-	CLOSE(out_rfd);
-	out_rfd = -1;
-	CLOSE(out_wfd);  
-	out_wfd = -1;
-	CLOSE(err_rfd);
-	err_rfd = -1;
-	CLOSE(err_wfd);
-	err_wfd = -1;
+	/* Close the message buffers */
+	CLOSE(core->out_rfd);
+	CLOSE(core->out_wfd);
+	CLOSE(core->err_rfd);
+	CLOSE(core->err_wfd);
 
-	g_free(prog_name);
-	prog_name = NULL;
-	g_free(out_dlg_title);
-	out_dlg_title = NULL;
-	g_free(err_dlg_title);
-	err_dlg_title = NULL;
+	g_free(core->prog_name);
+	g_free(core->font_name);
+	g_free(core->out_dlg_title);
+	g_free(core->err_dlg_title);
+	g_free(core);
 }
 
 
@@ -822,11 +1092,18 @@ void ConMsgShutdown(void)
  */
 void ConMsgStdOutDlgMap(void)
 {
-	ConMsgDlg *d = out_dlg;
+	ConMsgDlg *d;
+	ConMsgCore *core = con_msg_core;
+	if(core == NULL)
+	    return;
+
+	d = core->out_dlg;
 	if(d == NULL)
-	    out_dlg = d = ConMsgDlgNew(
-		out_dlg_title, (guint8 **)icon_info_32x32_xpm,
-		terminal_columns, terminal_lines
+	    core->out_dlg = d = ConMsgDlgNew(
+		core,
+		core->out_dlg_title, (guint8 **)icon_info_32x32_xpm,
+		core->font_name,
+		core->terminal_columns, core->terminal_lines
 	    );
 	if(d == NULL)
 	    return;
@@ -843,11 +1120,18 @@ void ConMsgStdOutDlgMap(void)
  */
 void ConMsgStdErrDlgMap(void)
 {
-	ConMsgDlg *d = err_dlg;
+	ConMsgDlg *d;
+	ConMsgCore *core = con_msg_core;
+	if(core == NULL)
+	    return;
+
+	d = core->err_dlg;
 	if(d == NULL)          
-	    err_dlg = d = ConMsgDlgNew(
-		err_dlg_title, (guint8 **)icon_warning_32x32_xpm,
-		terminal_columns, terminal_lines
+	    core->err_dlg = d = ConMsgDlgNew(
+		core,
+		core->err_dlg_title, (guint8 **)icon_warning_32x32_xpm,
+		core->font_name,
+		core->terminal_columns, core->terminal_lines
 	    );
 	if(d == NULL)
 	    return;
