@@ -6,9 +6,11 @@
 
 #include "guiutils.h"
 #include "style_edit.h"
+#include "menucfg_list.h"
 
 #include "cfg.h"
 #include "edv_types.h"
+#include "edv_utils.h"
 #include "edv_generic_options_win.h"
 #include "customize_win.h"
 #include "edv_utils_gtk.h"
@@ -28,12 +30,14 @@
 gchar *EDVCustomizeSimplifyOPIDName(const gchar *name);
 
 /* General */
-#if 0
+void EDVCustomizeGeneralCreateObjectOpsMenuPage(
+	edv_core_struct *core, edv_gen_opt_win_struct *optwin,
+	GtkWidget *parent
+);
 void EDVCustomizeGeneralCreateNewObjectsMenuPage(
 	edv_core_struct *core, edv_gen_opt_win_struct *optwin,
 	GtkWidget *parent
 );
-#endif
 void EDVCustomizeGeneralCreateStylesPage(
 	edv_core_struct *core, edv_gen_opt_win_struct *optwin,
 	GtkWidget *parent
@@ -44,7 +48,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 );
 
 /* Customize Window */
-void EDVCustomizeWindowSelectTab(
+gboolean EDVCustomizeWindowSelectTab(
 	edv_gen_opt_win_struct *optwin, const gchar *tab_name
 );
 edv_gen_opt_win_struct *EDVCustomizeWindowNew(edv_core_struct *core);
@@ -73,19 +77,85 @@ gchar *EDVCustomizeSimplifyOPIDName(const gchar *name)
 {
 	gchar *s, *name_rtn = STRDUP(name);
 
-	s = strsub(name_rtn, "_", "");
+	/* Remove all occurances of "_" */
+	s = EDVStrSub(name_rtn, "_", "");
 	g_free(name_rtn);
 	name_rtn = s;
 
-	s = strsub(name_rtn, "...", "");
+	/* Remove all occurances of "..." */
+	s = EDVStrSub(name_rtn, "...", "");
 	g_free(name_rtn);
 	name_rtn = s;
 
+	/* Strip spaces */
+	name_rtn = g_strstrip(name_rtn);
+
 	return(name_rtn);
 }
 
 
-#if 0
+/*
+ *	Creates the General New Objects Menu page.
+ */
+void EDVCustomizeGeneralCreateObjectOpsMenuPage(
+	edv_core_struct *core, edv_gen_opt_win_struct *optwin,
+	GtkWidget *parent
+)
+{
+	const gint	border_major = 5;
+	GtkWidget *w, *parent2, *parent3;
+	edv_gen_opt_wref_struct *wref;
+	menucfg_list_struct *menucfg_list;
+
+	/* Object Ops Menu Frame */
+	w = gtk_frame_new("Object Ops Menu");
+	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
+	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
+	gtk_widget_show(w);
+	parent2 = w;
+
+	w = gtk_vbox_new(FALSE, border_major);
+	gtk_container_add(GTK_CONTAINER(parent2), w);
+	gtk_container_border_width(GTK_CONTAINER(w), border_major);
+	gtk_widget_show(w);
+	parent2 = w;
+
+w = gtk_hbox_new(FALSE, border_major);
+gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+gtk_widget_show(w);
+parent3 = w;
+
+w = gtk_label_new(
+"This menu typically appears as either a submenu or a subsection under the File menu of most\n\
+windows. It allows you to create customized commands that are to be executed on the selected\n\
+objects.\n\
+\n\
+Each item's command must have the \"%paths\" or \"%p\" token to substitute for the selected\n\
+objects."
+);
+gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+gtk_widget_show(w);
+
+	/* MenuCfg List */
+	menucfg_list = MenuCfgListNew(
+	    parent2,
+	    GTK_ORIENTATION_HORIZONTAL
+	);
+	MenuCfgListSetListStyle(menucfg_list, MENUCFG_LIST_STYLE_DETAILS);
+	MenuCfgListSetEditable(menucfg_list, TRUE);
+	MenuCfgListMap(menucfg_list);
+
+	wref = EDVGenOptWinWRefAppend(
+	    optwin,
+	    EDV_GEN_OPT_WIDGET_MENUCFG_LIST, menucfg_list,
+	    NULL, EDV_CFG_PARM_OBJECT_OPS_MENU
+	);
+
+
+
+}
+
 /*
  *	Creates the General New Objects Menu page.
  */
@@ -94,13 +164,64 @@ void EDVCustomizeGeneralCreateNewObjectsMenuPage(
 	GtkWidget *parent
 )
 {
+	const gint	border_major = 5;
+	GList *glist;
+	GtkWidget *w, *parent2, *parent3;
+	edv_gen_opt_wref_struct *wref;
+	menucfg_list_struct *menucfg_list;
 
+	/* New Objects Menu Frame */
+	w = gtk_frame_new("New Objects Menu");
+	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
+	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
+	gtk_widget_show(w);
+	parent2 = w;
 
+	w = gtk_vbox_new(FALSE, border_major);
+	gtk_container_add(GTK_CONTAINER(parent2), w);
+	gtk_container_border_width(GTK_CONTAINER(w), border_major);
+	gtk_widget_show(w);
+	parent2 = w;
+
+w = gtk_hbox_new(FALSE, border_major);
+gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+gtk_widget_show(w);
+parent3 = w;
+
+w = gtk_label_new(
+"This menu typically appears as either a submenu or a subsection when you go to File->New on\n\
+most windows. It allows you to create customized commands that create a new object and then runs\n\
+a program to edit that object.\n\
+\n\
+Each item's command must have the \"%p\" token to substitute for the new object."
+);
+gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+gtk_widget_show(w);
+
+	/* MenuCfg List */
+	menucfg_list = MenuCfgListNew(
+	    parent2,
+	    GTK_ORIENTATION_HORIZONTAL
+	);
+	glist = NULL;
+	glist = g_list_append(glist, "Extension");
+	MenuCfgListSetCustomData(menucfg_list, glist);
+	g_list_foreach(glist, (GFunc)g_free, NULL);
+	g_list_free(glist);
+	MenuCfgListSetListStyle(menucfg_list, MENUCFG_LIST_STYLE_DETAILS);
+	MenuCfgListSetEditable(menucfg_list, TRUE);
+	MenuCfgListMap(menucfg_list);
+
+	wref = EDVGenOptWinWRefAppend(
+	    optwin,
+	    EDV_GEN_OPT_WIDGET_MENUCFG_LIST, menucfg_list,
+	    NULL, EDV_CFG_PARM_NEW_OBJECTS_MENU
+	);
 
 
 
 }
-#endif
 
 /*
  *	Creates the General Styles page.
@@ -258,7 +379,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	edv_gen_opt_wref_struct *wref;
 
 
-	/* Window Placement Frame */
+	/* Window Placement GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La Colocacin De La Ventana"
@@ -289,12 +410,12 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Hbox for Window Cascade Offset X */
+	/* Window Cascade Offset GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent3 = w;
-	/* Label */
+	/* GtkLabel */
 	w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La Desviacin De La Cascada"
@@ -318,7 +439,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 
-	/* Cascade Offset X Spin */
+	/* Cascade Offset X GtkSpinButton */
 	adj = (GtkAdjustment *)gtk_adjustment_new(
 	    0.0f, -100000.0f, 100000.0f,
 	    1.0f, 5.0f, 0.0f
@@ -326,6 +447,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	wref = EDVGenOptWinWRefAppend(
 	    optwin,
@@ -333,11 +455,11 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	    NULL, EDV_CFG_PARM_WINDOW_CASCADE_OFFSET_X
 	);
 
-	/* Label */
+	/* GtkLabel */
 	w = gtk_label_new("Y:");
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
-	/* Cascade Offset Y Spin */
+	/* Cascade Offset Y GtkSpinButton */
 	adj = (GtkAdjustment *)gtk_adjustment_new(
 	    0.0f, -100000.0f, 100000.0f,
 	    1.0f, 5.0f, 0.0f
@@ -345,6 +467,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	wref = EDVGenOptWinWRefAppend(
 	    optwin,
@@ -353,7 +476,7 @@ void EDVCustomizeGeneralCreateAppearancePage(
 	);
 
 
-	/* Tool Tips Frame */
+	/* Tool Tips GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Instrumento Inclina"
@@ -450,23 +573,21 @@ void EDVCustomizeGeneralCreateAppearancePage(
 }
 
 /*
- *	Selects the notebook tab specified by tab_name for the given
- *	customize window.
+ *	Selects the page by the tab name.
+ *
+ *	Returns TRUE if the page was switched to or FALSE on error.
  */
-void EDVCustomizeWindowSelectTab(
+gboolean EDVCustomizeWindowSelectTab(
 	edv_gen_opt_win_struct *optwin, const gchar *tab_name
 )
 {
 	gint page_num;
 	GtkNotebook *notebook;
 
-
 	if(optwin == NULL)
-	    return;
+	    return(FALSE);
 
-	notebook = (GtkNotebook *)optwin->notebook;
-	if(notebook == NULL)
-	    return;
+	notebook = GTK_NOTEBOOK(optwin->notebook);
 
 	/* Tab name given? */
 	if(!STRISEMPTY(tab_name))
@@ -485,14 +606,7 @@ void EDVCustomizeWindowSelectTab(
 	    else if(!g_strcasecmp(tab_name, "Recycle Bin"))
 		page_num = 4;
 	    else
-	    {
 		page_num = -1;
-		g_printerr(
-"EDVCustomizeWindowSelectTab(): Warning:\
- Invalid tab name \"%s\".\n",
-		    tab_name
-		);
-	    }
 	}
 	else
 	{
@@ -501,7 +615,14 @@ void EDVCustomizeWindowSelectTab(
 
 	/* Got valid page? */
 	if(page_num > -1)
+	{
 	    gtk_notebook_set_page(notebook, page_num);
+	    return(TRUE);
+	}
+	else
+	{
+	    return(FALSE);
+	}
 }
 
 /*
@@ -643,7 +764,19 @@ edv_gen_opt_win_struct *EDVCustomizeWindowNew(edv_core_struct *core)
 		core, optwin, w
 	    );
 
-#if 0
+	    /* Object Ops Menu */
+	    w = gtk_vbox_new(FALSE, border_major);
+	    gtk_container_border_width(GTK_CONTAINER(w), border_major);
+	    gtk_notebook_append_page(
+		GTK_NOTEBOOK(parent2), w, gtk_label_new(
+"Object Ops Menu"
+		)
+	    );
+	    gtk_widget_show(w);
+	    EDVCustomizeGeneralCreateObjectOpsMenuPage(
+		core, optwin, w
+	    );
+
 	    /* New Objects Menu */
 	    w = gtk_vbox_new(FALSE, border_major);
 	    gtk_container_border_width(GTK_CONTAINER(w), border_major);
@@ -656,7 +789,6 @@ edv_gen_opt_win_struct *EDVCustomizeWindowNew(edv_core_struct *core)
 	    EDVCustomizeGeneralCreateNewObjectsMenuPage(
 		core, optwin, w
 	    );
-#endif
 
 	}
 
