@@ -204,16 +204,16 @@ void EDVCustomizeArchiverCreateKeymapsPage(
 			gchar *name = EDVCustomizeSimplifyOPIDName(
 			    opid->menu_name
 			);
-			keymap_key_struct *km = KeymapKeyNew(
-			    ak->key, ak->modifiers, (gpointer)op
-			);
 			KeymapListAppend(
 			    kmlist,
 			    opid->button_icon_data,
 			    name,
-			    km			/* Shared */
+			    KeymapKeyNew(
+				ak->key,
+				ak->modifiers,
+				(gpointer)op
+			    )
 			);
-			KeymapKeyDelete(km);
 			g_free(name);
 			break;
 		    }
@@ -801,7 +801,7 @@ void EDVCustomizeArchiverCreateContentsListPage(
 "Forbind destinasjon",
 #else
 "Linked To",
-"Link destination",
+"Link target value",
 #endif
 	    NULL,               /* No icon data */
 	    NULL,               /* No client data */
@@ -960,14 +960,14 @@ void EDVCustomizeArchiverCreateToolBarPage(
 	edv_archiver_opid_struct *opid;
 
 
-	/* Hbox for Tool Bar Display and Relief frames */
+	/* GtkHBox for Tool Bar Display and Relief frames */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Tool Bar Display Frame */
+	/* Tool Bar Display GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue Abrocha Como"
@@ -987,7 +987,7 @@ void EDVCustomizeArchiverCreateToolBarPage(
 "Display Buttons As"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -1092,7 +1092,7 @@ void EDVCustomizeArchiverCreateToolBarPage(
 	wref->radio_value = TOOLBAR_DISPLAY_PICTURES_AND_TEXT;
 
 
-	/* Buttons Relief Frame */
+	/* Buttons Relief GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Alivio De Botones"
@@ -1112,7 +1112,7 @@ void EDVCustomizeArchiverCreateToolBarPage(
 "Buttons Relief"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -1217,7 +1217,7 @@ void EDVCustomizeArchiverCreateToolBarPage(
 	wref->radio_value = TOOLBAR_RELIEF_NORMAL;
 
 
-	/* Arrangement Frame */
+	/* Arrangement GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Arreglo"
@@ -1317,20 +1317,20 @@ void EDVCustomizeArchiverCreateAppearancePage(
 	edv_gen_opt_wref_struct *wref;
 
 
-	/* Hbox to separate two columns */
+	/* GtkHBox to separate two columns */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent = w;
 
 
-	/* First column vbox */
+	/* First column GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Show In Title Bar Frame */
+	/* Show In Title Bar GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Muestre En La Barra Del Ttulo"
@@ -1430,7 +1430,7 @@ void EDVCustomizeArchiverCreateAppearancePage(
 	);
 
 
-	/* Display Frame */
+	/* Display GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue"
@@ -1598,13 +1598,13 @@ void EDVCustomizeArchiverCreateAppearancePage(
 	);
 
 
-	/* Second column vbox */
+	/* Second column GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Archive Display Frame */
+	/* Archive Display GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue Del Archivo"
