@@ -205,16 +205,16 @@ void EDVCustomizeBrowserCreateKeymapsPage(
 			gchar *name = EDVCustomizeSimplifyOPIDName(
 			    opid->menu_name
 			);
-			keymap_key_struct *km = KeymapKeyNew(
-			    ak->key, ak->modifiers, (gpointer)op
-			);
 			KeymapListAppend(
 			    kmlist,
 			    opid->button_icon_data,
 			    name,
-			    km			/* Shared */
+			    KeymapKeyNew(
+				ak->key,
+				ak->modifiers,
+				(gpointer)op
+			    )
 			);
-			KeymapKeyDelete(km);
 			g_free(name);
 			break;
 		    }
@@ -747,7 +747,7 @@ void EDVCustomizeBrowserCreateContentsListPage(
 "Forbind destinasjon",
 #else
 "Linked To",
-"Link destination",
+"Link target value",
 #endif
 	    NULL,               /* No icon data */
 	    NULL,               /* No client data */
@@ -846,14 +846,14 @@ void EDVCustomizeBrowserCreateToolBarPage(
 	edv_browser_opid_struct *opid;
 
 
-	/* Hbox for Tool Bar Display and Relief frames */
+	/*GtkHBox for Tool Bar Display and Relief frames */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Display Buttons As Frame */
+	/* Display Buttons AsGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue Abrocha Como"
@@ -873,7 +873,7 @@ void EDVCustomizeBrowserCreateToolBarPage(
 "Display Buttons As"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -978,7 +978,7 @@ void EDVCustomizeBrowserCreateToolBarPage(
 	wref->radio_value = TOOLBAR_DISPLAY_PICTURES_AND_TEXT;
 
 
-	/* Buttons Relief Frame */
+	/* Buttons ReliefGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Alivio De Botones"
@@ -998,7 +998,7 @@ void EDVCustomizeBrowserCreateToolBarPage(
 "Buttons Relief"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -1104,7 +1104,7 @@ void EDVCustomizeBrowserCreateToolBarPage(
 
 
 
-	/* Arrangement Frame */
+	/* ArrangementGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Arreglo"
@@ -1208,20 +1208,20 @@ void EDVCustomizeBrowserCreateAppearancePage(
 	edv_gen_opt_wref_struct *wref;
 
 
-	/* Hbox to separate two columns */
+	/*GtkHBox to separate two columns */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent = w;
 
 
-	/* First column vbox */
+	/* First columnGtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Show In Title Bar Frame */
+	/* Show In Title BarGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Muestre En La Barra Del Ttulo"
@@ -1321,7 +1321,7 @@ void EDVCustomizeBrowserCreateAppearancePage(
 	);
 
 
-	/* Display Frame */
+	/* DisplayGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue"
@@ -1523,14 +1523,14 @@ void EDVCustomizeBrowserCreateAppearancePage(
 	);
 
 
-	/* Second column vbox */
+	/* Second columnGtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Filter Frame */
+	/* FilterGtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Filtro"
