@@ -2,6 +2,7 @@
 #include <gtk/gtk.h>
 
 #include "guiutils.h"
+#include "pulist.h"
 #include "csd.h"
 #include "fsd.h"
 #include "stacklist.h"
@@ -93,26 +94,29 @@ void EDVCustomizeImbrCreatePresentationPage(
 {
 	const gint      border_major = 5,
 			border_minor = 2;
+	gint i, label_width;
 	GtkAdjustment *adj;
 	GtkWidget *w, *parent2, *parent3, *parent4, *parent5, *parent6;
+        pulist_struct *pulist;           
+        pulistbox_struct *pulistbox;
 	edv_gen_opt_wref_struct *wref;
 
 
-	/* Hbox to separate two columns */
+	/* GtkHBox for two columns */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent = w;
 
 
-	/* First column vbox */
+	/* Left column GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Show Frame */
+	/* Show GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Exposicio'n"
@@ -168,7 +172,8 @@ void EDVCustomizeImbrCreatePresentationPage(
 #endif
 	);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto mostrar inicialmente la Barra del Control al cambiar\
  to el Modo de la Presentacio'n"
@@ -203,7 +208,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	    EDV_CFG_PARM_PRESENTATION_SHOW_NAVIGATOR
 	);
 
-	/* Fonts Frame */
+	/* Fonts GtkFrame */
 	w = gtk_frame_new("Fonts");
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
@@ -232,7 +237,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	);
 
 
-	/* Colors Frame */
+	/* Colors GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Los Colores"
@@ -262,6 +267,8 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_widget_show(w);
 	parent3 = w;       
 
+	label_width = 80;
+
 	/* Foreground */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
@@ -272,7 +279,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_widget_show(w);
 	parent5 = w;
 	w = gtk_alignment_new(1.0f, 0.5f, 0.0f, 0.0f);
-	gtk_widget_set_usize(w, 80, -1);
+	gtk_widget_set_usize(w, label_width, -1);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent6 = w;
@@ -314,7 +321,8 @@ void EDVCustomizeImbrCreatePresentationPage(
 	w = gtk_toggle_button_new_with_label("Invert");
 	gtk_widget_set_usize(w, 60, -1);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 "Check this to invert the foreground color, useful when colors\
  match the foreground color and make it indistinguishable"
 	);
@@ -336,7 +344,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_widget_show(w);                                      
 	parent5 = w;
 	w = gtk_alignment_new(1.0f, 0.5f, 0.0f, 0.0f);
-	gtk_widget_set_usize(w, 80, -1);
+	gtk_widget_set_usize(w, label_width, -1);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent6 = w;
@@ -376,7 +384,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	);
 
 
-	/* Automation Frame */
+	/* Automation GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Automatizacio'n"
@@ -407,17 +415,85 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_widget_show(w);
 	parent3 = w;
 
+	label_width = 130;
+
+	/* Transision */
+	w = gtk_hbox_new(FALSE, border_minor);
+	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
+	parent4 = w;
+
+	/* Label */
+	w = gtk_label_new("Transision:");
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
+
+	/* Transision Popup List Box */
+        pulistbox = PUListBoxNew(parent4, 200, -1);
+        pulist = PUListBoxGetPUList(pulistbox);
+	if(core->run_flags & EDV_RUN_SAFE_MODE)
+	    PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
+        i = PUListAddItem(pulist, "None");
+        i = PUListAddItem(pulist, "Fade To Black");
+        i = PUListAddItem(pulist, "Fade To White");
+        i = PUListAddItem(pulist, "Fade To Background");
+        i = PUListAddItem(pulist, "Move Horizontal");
+        i = PUListAddItem(pulist, "Move Vertical");
+        i = PUListAddItem(pulist, "Move Diagonal");
+        PUListBoxSetLinesVisible(pulistbox, MIN((i + 1), 10));
+        PUListBoxMap(pulistbox);
+        wref = EDVGenOptWinWRefAppend(
+            optwin,
+            EDV_GEN_OPT_WIDGET_POPUP_LIST_BOX, pulistbox,
+            NULL,
+            EDV_CFG_PARM_PRESENTATION_TRANSISION
+        );
+
+	/* Transision Interval GtkLabel */
+	w = gtk_label_new("Interval:");
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
+	/* Transision Interval GtkSpinButton */
+	adj = (GtkAdjustment *)gtk_adjustment_new(
+	    1.0f, 1.0f, (gfloat)((guint32)-1),
+	    10.0f, 100.0f, 0.0f
+	);
+	w = gtk_spin_button_new(adj, 1.0, 0);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
+"Set the transision interval (in milliseconds)"
+	);
+	gtk_widget_show(w);
+	wref = EDVGenOptWinWRefAppend(
+	    optwin,
+	    EDV_GEN_OPT_WIDGET_SPIN_BUTTON, w,
+	    NULL,
+	    EDV_CFG_PARM_PRESENTATION_TRANSISION_INT
+	);
+	/* Units GtkLabel */
+	w = gtk_label_new("milliseconds");
+	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
+
+
+
 	/* Hide Pointer Delay */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 	w = gtk_alignment_new(1.0f, 0.5f, 0.0f, 0.0f);
-	gtk_widget_set_usize(w, 130, -1);
+	gtk_widget_set_usize(w, label_width, -1);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
-	/* Label */
+	/* GtkLabel */
 	w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Esconda La Demora De La Flecha"
@@ -441,15 +517,17 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
 	gtk_container_add(GTK_CONTAINER(parent5), w);
 	gtk_widget_show(w);
-	/* Spin */
+	/* Hide Pointer Delay GtkSpinButton */
 	adj = (GtkAdjustment *)gtk_adjustment_new(
-	    0.0f, 0.0f, 100000.0f,
+	    0.0f, 0.0f, (gfloat)((guint32)-1),
 	    1.0f, 5.0f, 0.0f
 	);
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
-	GUISetWidgetTip(w,
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Ponga la demora en la que la flecha se escondera despues que una\
  falta del movimiento de la flecha o pondra la demora a 0 nunca\
@@ -478,8 +556,9 @@ void EDVCustomizeImbrCreatePresentationPage(
 "Sett utsettelsen som viseren gjemt etter et mangel av viserbevegelse\
  eller sett utsettelsen til 0 aldri i gjemme viseren"
 #else
-"Set the delay in which the pointer will be hidden after a lack of\
- pointer motion or set the delay to 0 to never hide the pointer"
+"Set the delay (in seconds) in which the pointer will be hidden\
+ after a lack of pointer motion or set the delay to 0 to never\
+ hide the pointer"
 #endif
 	);
 	gtk_widget_show(w);
@@ -489,7 +568,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	    NULL,
 	    EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY
 	);
-	/* Label */
+	/* Units Label */
 	w = gtk_label_new("seconds");
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -500,7 +579,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_widget_show(w);
 	parent4 = w;
 	w = gtk_alignment_new(1.0f, 0.5f, 0.0f, 0.0f);
-	gtk_widget_set_usize(w, 130, -1);
+	gtk_widget_set_usize(w, label_width, -1);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
@@ -508,10 +587,11 @@ void EDVCustomizeImbrCreatePresentationPage(
 	gtk_container_add(GTK_CONTAINER(parent5), w);
 	gtk_widget_show(w);
 	parent5 = w;
-	/* Check Button */
+	/* Slideshow GtkCheckButton */
 	w = gtk_check_button_new_with_label("Slideshow");
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto empezar exposicion de diapositivas en el inicio del\
  Modo de la Presentacion"
@@ -532,7 +612,8 @@ void EDVCustomizeImbrCreatePresentationPage(
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "Sjekk dette starte slideshow pa oppstartingen av framstillingsModus"
 #else
-"Check this to start slideshow at the startup of the Presentation Mode"
+"Check this to start the slideshow at the startup of the Presentation\
+ Mode"
 #endif
 	);
 	gtk_widget_show(w);
@@ -542,12 +623,12 @@ void EDVCustomizeImbrCreatePresentationPage(
 	    NULL,
 	    EDV_CFG_PARM_PRESENTATION_SLIDESHOW
 	);
-	/* Label */
+	/* Slideshow Delay GtkLabel */
 	w = gtk_label_new("Delay:");
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_RIGHT);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
-	/* Spin */
+	/* Slideshow Delay GtkSpinButton */
 	adj = (GtkAdjustment *)gtk_adjustment_new(
 	    0.0f, 1.0f, 100000.0f,
 	    1.0f, 5.0f, 0.0f
@@ -555,7 +636,9 @@ void EDVCustomizeImbrCreatePresentationPage(
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
-	GUISetWidgetTip(w,
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Ponga la demora a demostrar la proxima imagen en una exposicion de\
  diapositivas"
@@ -574,7 +657,8 @@ void EDVCustomizeImbrCreatePresentationPage(
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "Sett utsettelsen ved vise det neste avbildet i en slideshow"
 #else
-"Set the delay in displaying the next image in a slideshow"
+"Set the delay (in seconds) in displaying the next image during a\
+ slideshow"
 #endif
 	);
 	gtk_widget_show(w);
@@ -584,7 +668,7 @@ void EDVCustomizeImbrCreatePresentationPage(
 	    NULL,
 	    EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY
 	);
-	/* Label */
+	/* Units GtkLabel */
 	w = gtk_label_new("seconds");
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
@@ -748,16 +832,16 @@ void EDVCustomizeImbrCreateKeymapsPage(
 			gchar *name = EDVCustomizeSimplifyOPIDName(
 			    opid->menu_name
 			);
-			keymap_key_struct *km = KeymapKeyNew(
-			    ak->key, ak->modifiers, (gpointer)op
-			);
 			KeymapListAppend(
 			    kmlist,
 			    opid->button_icon_data,
 			    name,
-			    km		/* Shared */
+			    KeymapKeyNew(
+				ak->key,
+				ak->modifiers,
+				(gpointer)op
+			    )
 			);
-			KeymapKeyDelete(km);
 			g_free(name);
 			break;
 		    }
@@ -1435,20 +1519,19 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	parent3 = w;
 
 
-	/* Hbox for prompts */
+	/* Prompts GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 
 
-	/* Hbox for size prompts */
+	/* Size prompts GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
-	/* Width */
-	/* Label */
+	/* Width GtkLabel */
 	w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Width"
@@ -1471,13 +1554,14 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
-	/* Spin Button */
+	/* Width GtkSpinButton */
 	d->width_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
 	    20.0f, 20.0f, 320.0f, 1.0f, 5.0f, 0.0f
 	);
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	wref = EDVGenOptWinWRefAppend(
 	    optwin,
@@ -1485,8 +1569,7 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	    NULL,
 	    EDV_CFG_PARM_IMBR_THUMB_WIDTH
 	);
-	/* Height */
-	/* Label */
+	/* Height GtkLabel */
 	w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Height"
@@ -1509,13 +1592,14 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
-	/* Spin */
+	/* Height GtkSpinButton */
 	d->height_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
 	    20.0f, 20.0f, 320.0f, 1.0f, 5.0f, 0.0f
 	);
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	wref = EDVGenOptWinWRefAppend(
 	    optwin,
@@ -1524,7 +1608,7 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	    EDV_CFG_PARM_IMBR_THUMB_HEIGHT
 	);
 
-	/* Hbox for border */
+	/* Border GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1552,13 +1636,14 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
-	/* Spin Button */
+	/* Border GtkSpinButton */
 	d->border_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
 	    2.0f, 0.0f, 48.0f, 1.0f, 5.0f, 0.0f
 	);
 	w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 50, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	wref = EDVGenOptWinWRefAppend(
 	    optwin,
@@ -1568,7 +1653,7 @@ void EDVCustomizeImbrCreateThumbsListPage(
 	);
 
 
-	/* Hbox for sample thumb */
+	/* Sample thumb GtkHBox */
 	w = gtk_hbox_new(TRUE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
