@@ -204,16 +204,16 @@ void EDVCustomizeRecBinCreateKeymapsPage(
 			gchar *name = EDVCustomizeSimplifyOPIDName(
 			    opid->menu_name
 			);
-			keymap_key_struct *km = KeymapKeyNew(
-			    ak->key, ak->modifiers, (gpointer)op
-			);
 			KeymapListAppend(
 			    kmlist,
 			    opid->button_icon_data,
 			    name,
-			    km			/* Shared */
+			    KeymapKeyNew(
+				ak->key,
+				ak->modifiers,
+				(gpointer)op
+			    )
 			);
-			KeymapKeyDelete(km);
 			g_free(name);
 			break;
 		    }
@@ -744,7 +744,7 @@ void EDVCustomizeRecBinCreateContentsListPage(
 "Forbind destinasjon",
 #else
 "Linked To",
-"Link destination",
+"Link target value",
 #endif
 	    NULL,               /* No icon data */
 	    NULL,               /* No client data */
@@ -809,7 +809,7 @@ void EDVCustomizeRecBinCreateToolBarPage(
 	parent2 = w;
 
 
-	/* Display Buttons As Frame */
+	/* Display Buttons As GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue Abrocha Como"
@@ -829,7 +829,7 @@ void EDVCustomizeRecBinCreateToolBarPage(
 "Display Buttons As"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -934,7 +934,7 @@ void EDVCustomizeRecBinCreateToolBarPage(
 	wref->radio_value = TOOLBAR_DISPLAY_PICTURES_AND_TEXT;
 
 
-	/* Buttons Relief Frame */
+	/* Buttons Relief GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Alivio De Botones"
@@ -954,7 +954,7 @@ void EDVCustomizeRecBinCreateToolBarPage(
 "Buttons Relief"
 #endif
 	);
-	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_show(w);
 	parent3 = w;
@@ -1059,7 +1059,7 @@ void EDVCustomizeRecBinCreateToolBarPage(
 	wref->radio_value = TOOLBAR_RELIEF_NORMAL;
 
 
-	/* Arrangement Frame */
+	/* Arrangement GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Arreglo"
@@ -1173,7 +1173,7 @@ void EDVCustomizeRecBinCreateAppearancePage(
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Show In Title Bar Frame */
+	/* Show In Title Bar GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Muestre En La Barra Del Ttulo"
@@ -1240,7 +1240,7 @@ void EDVCustomizeRecBinCreateAppearancePage(
 	);
 
 
-	/* Display Frame */
+	/* Display GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Despliegue"
