@@ -10,7 +10,7 @@
 static void DeskIconFPromptApplyCB(
 	gpointer data, const gchar *s
 );
-static void DeskIconDoRename(desktop_icon_struct *di);
+static void DeskIconRenameQuery(desktop_icon_struct *di);
 
 static void DeskIconDraw(desktop_icon_struct *di);
 static void DeskIconQueueDraw(desktop_icon_struct *di);
@@ -20,7 +20,9 @@ static gint DeskIconEventCB(
 );
 
 static void DeskIconSyncMoveLabelToIcon(
-	desktop_icon_struct *di, gint x, gint y, gboolean need_resize
+	desktop_icon_struct *di,
+	const gint x, const gint y,
+	const gboolean need_resize
 );
 
 void DeskIconSetFlags(
@@ -34,7 +36,8 @@ void DeskIconGetPosition(
 	desktop_icon_struct *di, gint *x, gint *y
 );
 void DeskIconSetPosition(
-	desktop_icon_struct *di, gint x, gint y
+	desktop_icon_struct *di,
+	const gint x, const gint y
 );
 
 static gboolean DeskIconLoadIconPixmap(
@@ -45,7 +48,7 @@ static gboolean DeskIconLoadLabel(
 );
 
 desktop_icon_struct *DeskIconNew(
-	gint x, gint y,
+	const gint x, const gint y,
 	guint8 **icon_data, const gchar *label
 );
 
@@ -55,7 +58,9 @@ void DeskIconSet(
 );
 
 gboolean DeskIconIsSensitive(desktop_icon_struct *di);
-void DeskIconSetSensitive(desktop_icon_struct *di, gboolean sensitive);
+void DeskIconSetSensitive(
+	desktop_icon_struct *di, const gboolean sensitive
+);
 
 gboolean DeskIconIsSelected(desktop_icon_struct *di);
 void DeskIconSelect(desktop_icon_struct *di);
@@ -63,38 +68,38 @@ void DeskIconUnselect(desktop_icon_struct *di);
 
 void DeskIconSetMapRefCB(
 	desktop_icon_struct *di,
-	void (*map_ref_cb)(gpointer, gpointer),
-	gpointer client_data
+	void (*activate_cb)(gpointer, gpointer),
+	gpointer data
 );
 void DeskIconSetSelectCB(
 	desktop_icon_struct *di,
 	void (*select_cb)(gpointer, gpointer),
-	gpointer client_data
+	gpointer data
 );
 void DeskIconSetUnselectCB(
 	desktop_icon_struct *di,
 	void (*unselect_cb)(gpointer, gpointer),
-	gpointer client_data
+	gpointer data
 );
 void DeskIconSetRenameCB(
 	desktop_icon_struct *di,
 	void (*rename_cb)(gpointer, const gchar *, const gchar *, gpointer),
-	gpointer client_data
+	gpointer data
 );
 void DeskIconSetMoveBeginCB(
 	desktop_icon_struct *di,
 	void (*move_begin_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 );
 void DeskIconSetMovingCB(
 	desktop_icon_struct *di,
 	void (*moving_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 );
 void DeskIconSetMovedCB(
 	desktop_icon_struct *di,
 	void (*moved_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 );
 
 void DeskIconSetMenu(desktop_icon_struct *di, GtkWidget *w);
@@ -140,9 +145,9 @@ static void DeskIconFPromptApplyCB(
 }
 
 /*
- *	Renames the Desktop Icon.
+ *	Puts the Desktop Icon into rename query.
  */
-static void DeskIconDoRename(desktop_icon_struct *di)
+static void DeskIconRenameQuery(desktop_icon_struct *di)
 {
 	gint x, y, width;
 	GdkWindow *window;
@@ -156,17 +161,13 @@ static void DeskIconDoRename(desktop_icon_struct *di)
 	    return;
 
 	w = di->label_toplevel;
-	window = (w != NULL) ? w->window : NULL;
+	window = w->window;
 	if(window == NULL)
 	    return;
 
 	/* Get position of label window relative to the root window */
 	gdk_window_get_root_origin(window, &x, &y);
 
-	w = di->label_da;
-	if(w == NULL)
-	    return;
-
 	/* Calculate position of prompt to fit over the current position 
 	 * and size of the deskicon's label
 	 */
@@ -202,23 +203,21 @@ static void DeskIconDraw(desktop_icon_struct *di)
 	if(di == NULL)
 	    return;
 
-	if(!di->map_state)
+	if(!di->map_state || (di->freeze_count > 0))
 	    return;
 
 	gc = di->gc;
-	if(gc == NULL)
-	    return;
 
-	/* Begin drawing icon */
+	/* Begin drawing the icon */
 	w = di->icon_da;
 	pixmap = di->icon_pixmap;
 	mask = di->icon_mask;
 	if((w != NULL) && (pixmap != NULL))
 	{
 	    GdkWindow *window = w->window;
+	    const GtkStateType state = GTK_WIDGET_STATE(w);
 	    GtkStyle *style = gtk_widget_get_style(w);
-	    gint	state = GTK_WIDGET_STATE(w),
-			x = 0,
+	    gint	x = 0,
 			y = 0,
 			width = di->icon_width,
 			height = di->icon_height;
@@ -227,7 +226,7 @@ static void DeskIconDraw(desktop_icon_struct *di)
 	       (window != NULL) && (style != NULL)
 	    )
 	    {
-		/* Draw icon pixmap */
+		/* Draw the icon pixmap */
 		gdk_gc_set_function(gc, GDK_COPY);
 		gdk_gc_set_fill(gc, GDK_SOLID);
 		gdk_gc_set_clip_mask(gc, mask);
@@ -242,7 +241,7 @@ static void DeskIconDraw(desktop_icon_struct *di)
 		);
 		gdk_gc_set_clip_mask(gc, NULL);
 
-		/* Draw selection? */
+		/* Draw the selection? */
 		if(state == GTK_STATE_SELECTED)
 		{
 		    gdk_gc_set_function(gc, GDK_INVERT);
@@ -256,16 +255,16 @@ static void DeskIconDraw(desktop_icon_struct *di)
 	    }
 	}
 
-	/* Begin drawing label */
+	/* Begin drawing the label */
 	w = di->label_da;
 	s = di->label_str;
 	if(w != NULL)
 	{
 	    const gint	border_minor = 2;
 	    GdkWindow *window = w->window;
+	    const GtkStateType state = GTK_WIDGET_STATE(w);
 	    GtkStyle *style = gtk_widget_get_style(w);
-	    gint        state = GTK_WIDGET_STATE(w),
-			x = 0,
+	    gint	x = 0,
 			y = 0,
 			width = di->label_width,
 			height = di->label_height;
@@ -364,11 +363,10 @@ static void DeskIconDraw(desktop_icon_struct *di)
  */
 static void DeskIconQueueDraw(desktop_icon_struct *di)
 {
-	GtkWidget *w = (di != NULL) ? di->icon_toplevel : NULL;
-	if(w == NULL)
+	if(di == NULL)
 	    return;
 
-	gtk_widget_queue_draw(w);
+	gtk_widget_queue_draw(di->icon_toplevel);
 }
 
 
@@ -388,7 +386,9 @@ static gint DeskIconEventCB(
 	if((widget == NULL) || (event == NULL) || (di == NULL))
 	    return(status);
 
-	/* Handle by event type */
+	if(di->freeze_count > 0)
+	    return(status);
+
 	switch((gint)event->type)
 	{
 	  case GDK_EXPOSE:
@@ -454,7 +454,7 @@ static gint DeskIconEventCB(
 		break;
 	      case 2:
 		/* Rename */
-		DeskIconDoRename(di);
+		DeskIconRenameQuery(di);
 		break;
 	      case 3:
 		/* Map menu */
@@ -481,10 +481,10 @@ static gint DeskIconEventCB(
 		di->drag_last_x = 0;
 		di->drag_last_y = 0;
 		/* Call map reference callback? */
-		if(di->map_ref_cb != NULL)
-		    di->map_ref_cb(
+		if(di->activate_cb != NULL)
+		    di->activate_cb(
 			di,			/* Deskicon */
-			di->map_ref_client_data	/* Data */
+			di->activate_data	/* Data */
 		    );
 		break;
 	    }
@@ -517,7 +517,7 @@ static gint DeskIconEventCB(
 			di->moved_cb(
 			    di,				/* Deskicon */
 			    x, y,			/* Position */
-			    di->moved_client_data	/* Data */
+			    di->moved_data	/* Data */
 			);
 		    }
 		}
@@ -584,7 +584,7 @@ static gint DeskIconEventCB(
 			di->moving_cb(
 			    di,				/* Deskicon */
 			    icon_x, icon_y,		/* Position */
-			    di->moving_client_data	/* Data */
+			    di->moving_data	/* Data */
 			);
 		}
 		else
@@ -615,7 +615,7 @@ static gint DeskIconEventCB(
 				di->move_begin_cb(
 				    di,				/* Deskicon */
 				    icon_x, icon_y,		/* Position */
-				    di->move_begin_client_data	/* Data */
+				    di->move_begin_data	/* Data */
 				);
 			}
 
@@ -647,7 +647,7 @@ static gint DeskIconEventCB(
 			    di->moving_cb(
 				di,			/* Deskicon */
 				icon_x, icon_y,		/* Position */
-				di->moving_client_data	/* Data */
+				di->moving_data	/* Data */
 			    );
 		    }
 		}
@@ -669,7 +669,9 @@ static gint DeskIconEventCB(
  *	changed).
  */
 static void DeskIconSyncMoveLabelToIcon(
-	desktop_icon_struct *di, gint x, gint y, gboolean need_resize
+	desktop_icon_struct *di,
+	const gint x, const gint y,
+	const gboolean need_resize
 )
 {
 	gint width, height, nx, ny;
@@ -679,19 +681,19 @@ static void DeskIconSyncMoveLabelToIcon(
 	if(di == NULL)
 	    return;
 
-	/* Get icon toplevel window */
+	/* Icon toplevel */
 	w = di->icon_toplevel;
-	window = (w != NULL) ? w->window : NULL;
+	window = w->window;
 	if(window == NULL)
 	    return;
 
-	/* Get size of icon toplevel */
+	/* Size of the icon */
 	width = di->icon_width;
 	height = di->icon_height;
 
-	/* Get label toplevel */
+	/* Label toplevel */
 	w = di->label_toplevel;
-	window = (w != NULL) ? w->window : NULL;
+	window = w->window;
 	if(window == NULL)
 	    return;
 
@@ -711,7 +713,7 @@ static void DeskIconSyncMoveLabelToIcon(
 	if(need_resize)
 	{
 	    gtk_widget_queue_resize(di->label_da);
-	    gtk_widget_queue_resize(w);
+	    gtk_widget_queue_resize(di->label_toplevel);
 	}
 }
 
@@ -720,10 +722,10 @@ static void DeskIconSyncMoveLabelToIcon(
  *	Sets the Desktop Icon's flags.
  */
 void DeskIconSetFlags(
-	desktop_icon_struct *di, guint flags
+	desktop_icon_struct *di, const desktop_icon_flags flags
 )
 {
-	guint last_flags;
+	desktop_icon_flags last_flags;
 
 	if(di == NULL)
 	    return;
@@ -750,10 +752,10 @@ void DeskIconSetFlags(
  *	Unset the Desktop Icon's flags.
  */
 void DeskIconUnsetFlags(
-	desktop_icon_struct *di, guint flags
+	desktop_icon_struct *di, const desktop_icon_flags flags
 )
 {
-	guint last_flags;
+	desktop_icon_flags last_flags;
 
 	if(di == NULL)
 	    return;
@@ -797,7 +799,7 @@ void DeskIconGetPosition(
 	    return;
 
 	w = di->icon_toplevel;
-	window = (w != NULL) ? w->window : NULL;
+	window = w->window;
 	if(window == NULL)
 	    return;
 
@@ -813,7 +815,8 @@ void DeskIconGetPosition(
  *	Moves the Desktop Icon.
  */
 void DeskIconSetPosition(
-	desktop_icon_struct *di, gint x, gint y
+	desktop_icon_struct *di,
+	const gint x, const gint y
 )
 {
 	GdkWindow *window;
@@ -839,7 +842,7 @@ void DeskIconSetPosition(
 	    di->moved_cb(
 		di,			/* Desktop icon */
 		x, y,			/* Position */
-		di->moved_client_data	/* Data */
+		di->moved_data	/* Data */
 	    );
 }
 
@@ -865,8 +868,6 @@ static gboolean DeskIconLoadIconPixmap(
 	    return(FALSE);
 
 	w = di->icon_da;
-	if(w == NULL)
-	    return(FALSE);
 
 	/* Record old pixmap and mask pair that will be unref'ed later
 	 * on a successful load
@@ -900,23 +901,18 @@ static gboolean DeskIconLoadIconPixmap(
 
 	/* Update size and shape of toplevel widget and drawing area */
 	w = di->icon_toplevel;
-	if(w != NULL)
-	{
-	    gdk_window_resize(w->window, width, height);
-	    gtk_widget_set_usize(w, width, height);
-	    gtk_widget_shape_combine_mask(w, mask, 0, 0);
-	}
-	w = di->icon_da;
-	if(w != NULL)
-	{
-	    gdk_window_resize(w->window, width, height);
-	    gtk_widget_set_usize(w, width, height);
-	    gtk_widget_shape_combine_mask(w, mask, 0, 0);
-	}
+	gdk_window_resize(w->window, width, height);
+	gtk_widget_set_usize(w, width, height);
+	gtk_widget_shape_combine_mask(w, mask, 0, 0);
 
-	/* Unref old icon pixmap and mask */
-	GDK_PIXMAP_UNREF(old_pixmap)
-	GDK_BITMAP_UNREF(old_mask)
+	w = di->icon_da;
+	gdk_window_resize(w->window, width, height);
+	gtk_widget_set_usize(w, width, height);
+	gtk_widget_shape_combine_mask(w, mask, 0, 0);
+
+	/* Unref the old icon pixmap and mask */
+	GDK_PIXMAP_UNREF(old_pixmap);
+	GDK_BITMAP_UNREF(old_mask);
 
 	return(TRUE);
 }
@@ -942,20 +938,18 @@ static gboolean DeskIconLoadLabel(
 	    return(FALSE);
 	if((di->label_str != NULL) && (label != NULL))
 	{
-	    if(!strcmp(di->label_str, label))
+	    if(!strcmp((const char *)di->label_str, (const char *)label))
 		return(FALSE);
 	}
 
 	toplevel = di->label_toplevel;
 	w = di->label_da;
-	if((toplevel == NULL) || (w == NULL))
-	    return(FALSE);
 
 	style = gtk_widget_get_style(w);
 	if(style == NULL)
 	    return(FALSE);
 
-	/* Update label */
+	/* Update the label */
 	if(!STRISEMPTY(label))
 	{
 	    const gint	border_minor = 2;
@@ -1061,7 +1055,7 @@ static gboolean DeskIconLoadLabel(
  *	Creates a new Desktop Icon.
  */
 desktop_icon_struct *DeskIconNew(
-	gint x, gint y,
+	const gint x, const gint y,
 	guint8 **icon_data, const gchar *label
 )
 {
@@ -1073,9 +1067,9 @@ desktop_icon_struct *DeskIconNew(
 	if(di == NULL)
 	    return(di);
 
-	/* Reset values */
-	di->initialized = TRUE;
 	di->map_state = FALSE;
+	di->freeze_count = 0;
+	di->gc = NULL;
 	di->flags = 0;
 	di->last_icon_data = NULL;
 	di->icon_pixmap = NULL;
@@ -1085,22 +1079,24 @@ desktop_icon_struct *DeskIconNew(
 	di->button = 0;
 	di->drag_last_x = 0;
 	di->drag_last_y = 0;
-	di->map_ref_cb = NULL;
-	di->map_ref_client_data = NULL;
+	di->activate_cb = NULL;
+	di->activate_data = NULL;
 	di->select_cb = NULL;
-	di->select_client_data = NULL;
+	di->select_data = NULL;
 	di->unselect_cb = NULL;
-	di->unselect_client_data = NULL;
+	di->unselect_data = NULL;
 	di->rename_cb = NULL;
-	di->rename_client_data = NULL;
+	di->rename_data = NULL;
 	di->move_begin_cb = NULL;
-	di->move_begin_client_data = NULL;
+	di->move_begin_data = NULL;
 	di->moving_cb = NULL;
-	di->moving_client_data = NULL;
+	di->moving_data = NULL;
 	di->moved_cb = NULL;
-	di->moved_client_data = NULL;
+	di->moved_data = NULL;
 
-	/* Icon toplevel */
+	di->freeze_count++;
+
+	/* Toplevel GtkWindow */
 	di->icon_toplevel = w = gtk_window_new(GTK_WINDOW_POPUP);
 	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_FOCUS | GTK_CAN_DEFAULT);
 	gtk_widget_add_events(
@@ -1157,10 +1153,10 @@ desktop_icon_struct *DeskIconNew(
 	gtk_container_border_width(GTK_CONTAINER(w), 0);
 	parent = w;
 
-	/* Create graphics context */
-	di->gc = GDK_GC_NEW();
+	/* Graphics context */
+	di->gc = gdk_gc_new(window);
 
-	/* Icon drawing area */
+	/* Icon GtkDrawingArea */
 	di->icon_da = w = gtk_drawing_area_new();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(w), 48, 48);
 	gtk_widget_add_events(
@@ -1175,7 +1171,7 @@ desktop_icon_struct *DeskIconNew(
 	gtk_widget_show(w);
 
 
-	/* Label toplevel */
+	/* Label Toplevel GtkWindow */
 	di->label_toplevel = w = gtk_window_new(GTK_WINDOW_POPUP);
 	gtk_widget_add_events(
 	    w,
@@ -1216,7 +1212,7 @@ desktop_icon_struct *DeskIconNew(
 	gtk_container_border_width(GTK_CONTAINER(w), 0);
 	parent = w;
 
-	/* Label drawing area */
+	/* Label GtkDrawingArea */
 	di->label_da = w = gtk_drawing_area_new();
 	gtk_drawing_area_size(GTK_DRAWING_AREA(w), 48, 48);
 	gtk_widget_add_events(
@@ -1231,13 +1227,15 @@ desktop_icon_struct *DeskIconNew(
 	gtk_widget_show(w);
 
 
-	/* Set icon and label */
+	/* Set the initial icon and label */
 	DeskIconLoadIconPixmap(di, icon_data);
 	DeskIconLoadLabel(di, label);
 
-	/* Set position */
+	/* Set the initial position */
 	DeskIconSetPosition(di, x, y);
 
+	di->freeze_count--;
+
 	return(di);
 }
 
@@ -1255,9 +1253,6 @@ void DeskIconSet(
 	if(di == NULL)
 	    return;
 
-	if((di->icon_toplevel == NULL) || (di->label_toplevel == NULL))
-	    return;
-
 	/* Record previous label string */
 	old_label_str = STRDUP(di->label_str);
 
@@ -1287,7 +1282,7 @@ void DeskIconSet(
 		di,			/* Deskicon */
 		old_label_str,		/* Old name */
 		di->label_str,		/* New name */
-		di->rename_client_data	/* Data */
+		di->rename_data		/* Data */
 	    );
 
 	g_free(old_label_str);
@@ -1299,14 +1294,18 @@ void DeskIconSet(
  */
 gboolean DeskIconIsSensitive(desktop_icon_struct *di)
 {
-	GtkWidget *w = (di != NULL) ? di->icon_toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_SENSITIVE(w) : FALSE);
+	if(di == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_SENSITIVE(di->icon_toplevel));
 }
 
 /*
  *	Sets the Desktop Icon's sensitivity.
  */
-void DeskIconSetSensitive(desktop_icon_struct *di, gboolean sensitive)
+void DeskIconSetSensitive(
+	desktop_icon_struct *di, const gboolean sensitive
+)
 {
 	if(di == NULL)
 	    return;
@@ -1316,8 +1315,8 @@ void DeskIconSetSensitive(desktop_icon_struct *di, gboolean sensitive)
 	    if(DeskIconIsSensitive(di))
 		return;
 
-	    GTK_WIDGET_SET_SENSITIVE(di->icon_toplevel, TRUE)
-	    GTK_WIDGET_SET_SENSITIVE(di->label_toplevel, TRUE)
+	    gtk_widget_set_sensitive(di->icon_toplevel, TRUE);
+	    gtk_widget_set_sensitive(di->label_toplevel, TRUE);
 	    DeskIconQueueDraw(di);
 	}
 	else
@@ -1325,8 +1324,8 @@ void DeskIconSetSensitive(desktop_icon_struct *di, gboolean sensitive)
 	    if(!DeskIconIsSensitive(di))
 		return;
 
-	    GTK_WIDGET_SET_SENSITIVE(di->icon_toplevel, FALSE)
-	    GTK_WIDGET_SET_SENSITIVE(di->label_toplevel, FALSE)
+	    gtk_widget_set_sensitive(di->icon_toplevel, FALSE);
+	    gtk_widget_set_sensitive(di->label_toplevel, FALSE);
 	    DeskIconQueueDraw(di);
 	}
 }
@@ -1337,10 +1336,10 @@ void DeskIconSetSensitive(desktop_icon_struct *di, gboolean sensitive)
  */
 gboolean DeskIconIsSelected(desktop_icon_struct *di)
 {
-	GtkWidget *w = (di != NULL) ? di->icon_toplevel : NULL;
-	return((w != NULL) ?
-	    (GTK_WIDGET_STATE(w) == GTK_STATE_SELECTED) : FALSE
-	);
+	if(di == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_STATE(di->icon_toplevel) == GTK_STATE_SELECTED);
 }
 
 /*
@@ -1348,8 +1347,6 @@ gboolean DeskIconIsSelected(desktop_icon_struct *di)
  */
 void DeskIconSelect(desktop_icon_struct *di)
 {
-	GtkWidget *w;
-
 	if(di == NULL)
 	    return;
 
@@ -1359,20 +1356,16 @@ void DeskIconSelect(desktop_icon_struct *di)
 	if(DeskIconIsSelected(di) && DeskIconIsSensitive(di))
 	    return;
 
-	w = di->icon_toplevel;
-	if(w != NULL)
-	    gtk_widget_set_state(w, GTK_STATE_SELECTED);
-	w = di->label_toplevel;
-	if(w != NULL)
-	    gtk_widget_set_state(w, GTK_STATE_SELECTED);
+	gtk_widget_set_state(di->icon_toplevel, GTK_STATE_SELECTED);
+	gtk_widget_set_state(di->label_toplevel, GTK_STATE_SELECTED);
 
 	DeskIconQueueDraw(di);
 
-	/* Call select callback */
+	/* Call the select callback */
 	if(di->select_cb != NULL)
 	    di->select_cb(
 		di,			/* Deskicon */
-		di->select_client_data	/* Data */
+		di->select_data	/* Data */
 	    );
 }
 
@@ -1381,8 +1374,6 @@ void DeskIconSelect(desktop_icon_struct *di)
  */
 void DeskIconUnselect(desktop_icon_struct *di)
 {
-	GtkWidget *w;
-
 	if(di == NULL)
 	    return;
 
@@ -1392,20 +1383,16 @@ void DeskIconUnselect(desktop_icon_struct *di)
 	if(!DeskIconIsSelected(di) && DeskIconIsSensitive(di))
 	    return;
 
-	w = di->icon_toplevel;
-	if(w != NULL)
-	    gtk_widget_set_state(w, GTK_STATE_NORMAL);
-	w = di->label_toplevel;
-	if(w != NULL)
-	    gtk_widget_set_state(w, GTK_STATE_NORMAL);
+	gtk_widget_set_state(di->icon_toplevel, GTK_STATE_NORMAL);
+	gtk_widget_set_state(di->label_toplevel, GTK_STATE_NORMAL);
 
 	DeskIconQueueDraw(di);
 
-	/* Call unselect callback */
+	/* Call the unselect callback */
 	if(di->unselect_cb != NULL)
 	    di->unselect_cb(
 		di,				/* Deskicon */
-		di->unselect_client_data	/* Data */
+		di->unselect_data	/* Data */
 	    );
 }
 
@@ -1415,15 +1402,15 @@ void DeskIconUnselect(desktop_icon_struct *di)
  */
 void DeskIconSetMapRefCB(
 	desktop_icon_struct *di,
-	void (*map_ref_cb)(gpointer, gpointer),
-	gpointer client_data
+	void (*activate_cb)(gpointer, gpointer),
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
-	di->map_ref_cb = map_ref_cb;
-	di->map_ref_client_data = client_data;
+	di->activate_cb = activate_cb;
+	di->activate_data = data;
 }
 
 /*
@@ -1432,14 +1419,14 @@ void DeskIconSetMapRefCB(
 void DeskIconSetSelectCB(
 	desktop_icon_struct *di,
 	void (*select_cb)(gpointer, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->select_cb = select_cb;
-	di->select_client_data = client_data;
+	di->select_data = data;
 }
 
 /*
@@ -1448,14 +1435,14 @@ void DeskIconSetSelectCB(
 void DeskIconSetUnselectCB(
 	desktop_icon_struct *di,
 	void (*unselect_cb)(gpointer, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->unselect_cb = unselect_cb;
-	di->unselect_client_data = client_data;
+	di->unselect_data = data;
 }
 
 /*
@@ -1464,14 +1451,14 @@ void DeskIconSetUnselectCB(
 void DeskIconSetRenameCB(
 	desktop_icon_struct *di,
 	void (*rename_cb)(gpointer, const gchar *, const gchar *, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->rename_cb = rename_cb;
-	di->rename_client_data = client_data;
+	di->rename_data = data;
 }
 
 /*
@@ -1480,14 +1467,14 @@ void DeskIconSetRenameCB(
 void DeskIconSetMoveBeginCB(
 	desktop_icon_struct *di,
 	void (*move_begin_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->move_begin_cb = move_begin_cb;
-	di->move_begin_client_data = client_data;
+	di->move_begin_data = data;
 }
 
 /*
@@ -1496,14 +1483,14 @@ void DeskIconSetMoveBeginCB(
 void DeskIconSetMovingCB(
 	desktop_icon_struct *di,
 	void (*moving_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->moving_cb = moving_cb;
-	di->moving_client_data = client_data;
+	di->moving_data = data;
 }
 
 /*
@@ -1512,14 +1499,14 @@ void DeskIconSetMovingCB(
 void DeskIconSetMovedCB(
 	desktop_icon_struct *di,
 	void (*moved_cb)(gpointer, gint, gint, gpointer),
-	gpointer client_data
+	gpointer data
 )
 {
 	if(di == NULL)
 	    return;
 
 	di->moved_cb = moved_cb;
-	di->moved_client_data = client_data;
+	di->moved_data = data;
 }
 
 /*
@@ -1544,7 +1531,7 @@ void DeskIconSetMenu(desktop_icon_struct *di, GtkWidget *w)
 	if(di->menu != NULL)
 	    gtk_widget_destroy(di->menu);
 
-	/* Set new menu (which may be NULL) */
+	/* Set the new menu (which may be NULL) */
 	di->menu = w;
 }
 
@@ -1553,7 +1540,10 @@ void DeskIconSetMenu(desktop_icon_struct *di, GtkWidget *w)
  */
 GtkWidget *DeskIconGetMenu(desktop_icon_struct *di)
 {
-	return((di != NULL) ? di->menu : NULL);
+	if(di == NULL)
+	    return(NULL);
+
+	return(di->menu);
 }
 
 
@@ -1572,22 +1562,19 @@ void DeskIconMap(desktop_icon_struct *di)
 	    gint x, y;
 
 	    w = di->icon_toplevel;
-	    if(w != NULL)
+	    if(di->flags & DESKICON_ALWAYS_ON_TOP)
 	    {
-		if(di->flags & DESKICON_ALWAYS_ON_TOP)
-		{
-		    gtk_widget_show_raise(w);
-		}
-		else
-		{
-		    gtk_widget_show(w);
-		    if(w->window != NULL)
-			gdk_window_lower(w->window);
-		}
+		gtk_widget_show_raise(w);
+	    }
+	    else
+	    {
+		gtk_widget_show(w);
+		if(w->window != NULL)
+		    gdk_window_lower(w->window);
 	    }
 
 	    w = di->label_toplevel;
-	    if((w != NULL) && !STRISEMPTY(di->label_str))
+	    if(!STRISEMPTY(di->label_str))
 	    {
 		if(di->flags & DESKICON_ALWAYS_ON_TOP)
 		{
@@ -1613,21 +1600,13 @@ void DeskIconMap(desktop_icon_struct *di)
  */
 void DeskIconUnmap(desktop_icon_struct *di)
 {
-	GtkWidget *w;
-
-
 	if(di == NULL)
 	    return;
 
 	if(di->map_state)
 	{
-	    w = di->icon_toplevel;
-	    if(w != NULL)
-		gtk_widget_hide(w);
-
-	    w = di->label_toplevel;
-	    if(w != NULL)
-		gtk_widget_hide(w);
+	    gtk_widget_hide(di->icon_toplevel);
+	    gtk_widget_hide(di->label_toplevel);
 
 	    di->map_state = FALSE;
 	}
@@ -1641,18 +1620,24 @@ void DeskIconDelete(desktop_icon_struct *di)
 	if(di == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(di->menu)
-	GTK_WIDGET_DESTROY(di->label_da)
-	GTK_WIDGET_DESTROY(di->label_toplevel)
-	GTK_WIDGET_DESTROY(di->icon_da)
-	GTK_WIDGET_DESTROY(di->icon_toplevel)
+	DeskIconUnmap(di);
 
-	GDK_PIXMAP_UNREF(di->icon_pixmap)
-	GDK_BITMAP_UNREF(di->icon_mask)
+	di->freeze_count++;
 
-	GDK_GC_UNREF(di->gc)
+	GTK_WIDGET_DESTROY(di->menu);
+	GTK_WIDGET_DESTROY(di->label_da);
+	GTK_WIDGET_DESTROY(di->label_toplevel);
+	GTK_WIDGET_DESTROY(di->icon_da);
+	GTK_WIDGET_DESTROY(di->icon_toplevel);
+
+	GDK_PIXMAP_UNREF(di->icon_pixmap);
+	GDK_BITMAP_UNREF(di->icon_mask);
+
+	GDK_GC_UNREF(di->gc);
 
 	g_free(di->label_str);
 
+	di->freeze_count--;
+
 	g_free(di);
 }
