@@ -5,9 +5,9 @@
 
 #include "guiutils.h"
 #include "pulist.h"
+#include "fprompt.h"
 #include "cdialog.h"
 #include "fb.h"
-#include "fprompt.h"
 
 #include "edv_types.h"
 #include "edv_device.h"
@@ -27,6 +27,7 @@
 #include "images/icon_device_misc_48x48.xpm"
 
 
+/* Callbacks */
 static void EDVIconPickerUpdateDisplay(
 	edv_device_icon_picker_struct *ei, const gint state
 );
@@ -40,15 +41,22 @@ static void EDVIconPickerNextStateCB(
 	GtkWidget *widget, gpointer data
 );
 
-static void EDVDevicesEditWinBrowseProgramCB(
-	GtkWidget *widget, gpointer data
-);
 static gint EDVDevicesEditWinDeleteEventCB(
 	GtkWidget *widget, GdkEvent *event, gpointer data
 );
 static void EDVDevicesEditWinAnyChangedCB(
 	GtkWidget *widget, gpointer data
 );
+
+static void EDVDevicesEditWinBrowseDeviceCB(
+	GtkWidget *widget, gpointer data
+);
+static void EDVDevicesEditWinBrowseMountPathCB(
+	GtkWidget *widget, gpointer data
+);
+static void EDVDevicesEditWinBrowseProgramCB(
+	GtkWidget *widget, gpointer data
+);
 static void EDVDeviceEditDlgFSTypeChangedCB(
 	pulistbox_struct *pulistbox, gint i, gpointer data
 );
@@ -72,8 +80,8 @@ void EDVDevicesEditWinDeviceRemovedCB(
 static edv_device_icon_picker_struct *EDVDevicesEditIconNew(
 	edv_device_edit_dlg_struct *d,
 	GtkWidget *parent, const gchar *title,
-	gint icon_width, gint icon_height,
-	gint btn_width, gint btn_height
+	const gint icon_width, const gint icon_height,
+	const gint btn_width, const gint btn_height
 );
 static void EDVDevicesEditIconDelete(edv_device_icon_picker_struct *ei);
 
@@ -260,19 +268,14 @@ static void EDVIconPickerUpdateDisplay(
 
 #if 0
 	/* Update button sensitivities */
-	w = ei->left_btn;
-	if(w != NULL)
-	{
-	    gboolean sensitive = (state > 0) ? TRUE : FALSE;
-	    GTK_WIDGET_SET_SENSITIVE(w, sensitive)
-	}
-	w = ei->right_btn;
-	if(w != NULL)
-	{
-	    gboolean sensitive =
-		(state < (EDV_DEVICE_TOTAL_ICON_STATES - 1)) ? TRUE : FALSE;
-	    GTK_WIDGET_SET_SENSITIVE(w, sensitive)
-	}
+	gtk_widget_set_sensitive(
+	    ei->left_btn,
+	    (state > 0) ? TRUE : FALSE
+	);
+	gtk_widget_set_sensitive(
+	    ei->right_btn,
+	    (state < (EDV_DEVICE_TOTAL_ICON_STATES - 1)) ? TRUE : FALSE
+	);
 #endif
 }
 
@@ -283,14 +286,12 @@ static void EDVIconPickerChangeIconCB(
 	GtkWidget *widget, gpointer data
 )
 {
-	gint state, status;
+	gboolean response;
+	gint state, nftypes = 0, npaths = 0;
 	const gchar *old_path;
-	gchar *old_parent_path;
+	gchar *old_parent_path, **paths_list = NULL;
 	GtkWidget *toplevel;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	edv_core_struct *core;
 	edv_device_edit_dlg_struct *d;
 	edv_device_icon_picker_struct *ei =
@@ -304,11 +305,11 @@ static void EDVIconPickerChangeIconCB(
 	    return;
 
 	d = EDV_DEVICE_EDIT_DLG(ei->d);
-	core = d->core;
-	if((d == NULL) || (core == NULL))
-	    return;
-
 	toplevel = d->toplevel;
+	core = d->core;
+
+	if(d->freeze_count > 0)
+	    return;
 
 	/* Get current path to the icon file for this state */
 	old_path = ei->path[state];
@@ -317,23 +318,23 @@ static void EDVIconPickerChangeIconCB(
 	else
 	    old_parent_path = NULL;
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    ".xpm", "XPM Image"
+	    &ftypes_list, &nftypes,
+	    ".xpm", "X Pixmap"
 	);
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
 	/* Current old_path must be considered invalid at this point */
 	old_path = NULL;
 
-
-	/* Query user for new icon file */
+	/* Query the user for an image */
+	d->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Icono Selecto",
 "Selecto",
@@ -368,148 +369,64 @@ static void EDVIconPickerChangeIconCB(
 "Cancel",
 #endif
 	    old_parent_path,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	d->freeze_count--;
 
 	/* Got user response? */
-	if(status)
+	if(response)
 	{
-	    const gchar *new_path = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
-	    if(!STRISEMPTY(new_path))
+	    const gchar *new_path = (npaths > 0) ?
+		paths_list[0] : NULL;
+	    if(new_path != NULL)
 	    {
-		/* Load new pixmap and mask */
+		/* Open the new pixmap and mask */
 		GdkBitmap *mask;
 		GdkPixmap *pixmap = GDK_PIXMAP_NEW_FROM_XPM_FILE(
 		    &mask, new_path
 		);
 		if(pixmap != NULL)
 		{
-		    /* New pixmap and mask loaded successfully */
+		    /* New pixmap and mask opened successfully */
 
-		    /* Unref old and set new pixmap and mask */
-		    GDK_PIXMAP_UNREF(ei->pixmap[state])
-		    GDK_BITMAP_UNREF(ei->mask[state])
+		    /* Unref the old and set the new pixmap and mask */
+		    GDK_PIXMAP_UNREF(ei->pixmap[state]);
+		    GDK_BITMAP_UNREF(ei->mask[state]);
 		    EDVResizePixmap(
 			pixmap, mask,
 			ei->icon_width, ei->icon_height,
 			&ei->pixmap[state], &ei->mask[state]
 		    );
-		    GDK_PIXMAP_UNREF(pixmap)
-		    GDK_BITMAP_UNREF(mask)
+		    GDK_PIXMAP_UNREF(pixmap);
+		    GDK_BITMAP_UNREF(mask);
 
-		    /* Set new icon file path */
+		    /* Set the new icon file path */
 		    g_free(ei->path[state]);
 		    ei->path[state] = STRDUP(new_path);
-
-		    /* Update the displayed icon */
-		    EDVIconPickerUpdateDisplay(ei, state);
-
-		    /* Mark has changes as needed */
-		    EDVDevicesEditWinResetHasChanges(d, TRUE);
 		}
 		else
 		{
-		    /* Failed to load icon */
-		    gchar *buf;
+                    GDK_PIXMAP_UNREF(ei->pixmap[state]);
+		    ei->pixmap[state] = NULL;
 
-		    /* Update the displayed icon */
-		    EDVIconPickerUpdateDisplay(ei, state);
+                    GDK_BITMAP_UNREF(ei->mask[state]);
+		    ei->mask[state] = NULL;
 
-		    /* Notify user about failed load */
-		    buf = g_strdup_printf(
-#if defined(PROG_LANGUAGE_SPANISH)
-"Incapaz de cargar el archivo de icono:\n\
-\n\
-    %s\n\
-\n\
-Verifique por favor que el sendero especificado es correcto y que el\n\
-archivo es un archivo vlido de la imagen de XPM.\n"
-#elif defined(PROG_LANGUAGE_FRENCH)
-"Incapable de charger le fichier d'icne:\n\
-\n\
-    %s\n\
-\n\
-S'il vous plat vrifier que le sentier spcifi est exact et que le\n\
-fichier est un fichier d'image de XPM valide.\n"
-#elif defined(PROG_LANGUAGE_GERMAN)
-"Unfhig, abbild akte zu beladen:\n\
-\n\
-    %s\n\
-\n\
-Beglaubigen sie bitte, da der angegebene pfad richtig ist, und, da\n\
-die akte eine gltige XPM bildnis akte ist.\n"
-#elif defined(PROG_LANGUAGE_ITALIAN)
-"Incapace per caricare il file di icona:\n\
-\n\
-    %s\n\
-\n\
-Per favore di verificare che il sentiero specificato  corretto e\n\
-che il file  un file di immagine di XPM valido.\n"
-#elif defined(PROG_LANGUAGE_DUTCH)
-"Onbekwaam beeld dossier te laden:\n\
-\n\
-    %s\n\
-\n\
-Bevestiig alstublieft dat het gespecificeerd pad correcte is en dat\n\
-het dossier een geldig XPM beeld dossier is.\n"
-#elif defined(PROG_LANGUAGE_PORTUGUESE)
-"Incapaz de carregar arquivo de cone:\n\
-\n\
-    %s\n\
-\n\
-Por favor verifique-se que o caminho especificado  correto e que o\n\
-arquivo  um arquivo vlido de imagem de XPM.\n"
-#elif defined(PROG_LANGUAGE_NORWEGIAN)
-"Maktesls laste ikonarkiv:\n\
-\n\
-    %s\n\
-\n\
-Vr s snill og bekreft at den spesifiserte stien er riktig og at\n\
-arkivet er et gyldig XPM avbildearkiv\n"
-#else
-"Unable to load icon file:\n\
-\n\
-    %s\n\
-\n\
-Please verify that the specified path is correct and that the file\n\
-is a valid XPM image file.\n"
-#endif
-			, new_path
-		    );
-		    EDVPlaySoundWarning(core);
-		    EDVMessageWarning(
-#if defined(PROG_LANGUAGE_SPANISH)
-"Cargue Icono Fallado"
-#elif defined(PROG_LANGUAGE_FRENCH)
-"Charger L'Icne Echoue"
-#elif defined(PROG_LANGUAGE_GERMAN)
-"Beladen Sie Abbild, Versagt Wird, Das"
-#elif defined(PROG_LANGUAGE_ITALIAN)
-"Caricare L'Icona Fallita"
-#elif defined(PROG_LANGUAGE_DUTCH)
-"Laad Beeld Verzuimde"
-#elif defined(PROG_LANGUAGE_PORTUGUESE)
-"Carregue cone Fracassado"
-#elif defined(PROG_LANGUAGE_NORWEGIAN)
-"Last Sviktet Ikon"
-#else
-"Load Icon Failed"
-#endif
-			, buf, NULL,
-			toplevel
-		    );
-		    g_free(buf);
+		    g_free(ei->path[state]);
+		    ei->path[state] = NULL;
 		}
+
+		/* Update the displayed icon and mark has changes */
+		EDVIconPickerUpdateDisplay(ei, state);
+		EDVDevicesEditWinResetHasChanges(d, TRUE);
 	    }
 	}
 
-
-	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	/* Delete the file types list */
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	g_free(old_parent_path);
 }
@@ -567,60 +484,6 @@ static void EDVIconPickerNextStateCB(
 }
 
 
-/*
- *	Devices Edit Window browse program callback.
- */
-static void EDVDevicesEditWinBrowseProgramCB(
-	GtkWidget *widget, gpointer data
-)
-{
-	gboolean status;
-	GtkWidget *toplevel;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
-	GtkEntry *entry = (GtkEntry *)data;
-	if((entry == NULL) || FileBrowserIsQuery())
-	    return;
-
-	toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
-
-	/* Create file types list */
-	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
-	);
-
-	/* Query user for program */
-	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
-	    "Select Program",
-	    "Select", "Cancel",
-	    gtk_entry_get_text(entry),          /* Startup path */
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
-	    &ftype_rtn
-	);
-	FileBrowserSetTransientFor(NULL);
-
-	/* Got user response? */
-	if(status)
-	{
-	    const gchar *prog = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
-	    if(!STRISEMPTY(prog))
-	    {
-		gtk_entry_set_text(entry, prog);
-		gtk_entry_set_position(entry, -1);
-	    }
-	}
-
-	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
-}
-
-
 /*
  *	GtkWindow "delete_event" signal callback.
  */
@@ -632,17 +495,15 @@ static gint EDVDevicesEditWinDeleteEventCB(
 	if(d == NULL)
 	    return(TRUE);
 
-	EDVDevicesEditWinCancelCB(NULL, d);
+	EDVDevicesEditWinCancelCB(widget, d);
 
 	return(TRUE);
 }
 
 /*
- *	GtkWidget any changed signal callback.
+ *	GtkWidget any "changed" signal callback.
  */
-static void EDVDevicesEditWinAnyChangedCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVDevicesEditWinAnyChangedCB(GtkWidget *widget, gpointer data)
 {
 	edv_device_edit_dlg_struct *d = EDV_DEVICE_EDIT_DLG(data);
 	if(d == NULL)
@@ -659,6 +520,162 @@ static void EDVDevicesEditWinAnyChangedCB(
 	d->freeze_count--;
 }
 
+
+/*
+ *	Devices Edit Window browse device callback.
+ */
+static void EDVDevicesEditWinBrowseDeviceCB(GtkWidget *widget, gpointer data)
+{
+	gboolean response;
+	GtkWidget *toplevel;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
+	GtkEntry *entry = (GtkEntry *)data;
+	if((entry == NULL) || FileBrowserIsQuery())
+	    return;
+
+	toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
+
+	/* Create the file types list */
+	FileBrowserTypeListNew(
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
+	);
+
+	/* Query the user for the device */
+	FileBrowserSetTransientFor(toplevel);
+	response = FileBrowserGetResponse(
+	    "Select Device",
+	    "Select", "Cancel",
+	    gtk_entry_get_text(entry),		/* Startup path */
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
+	    &ftype_rtn
+	);
+	FileBrowserSetTransientFor(NULL);
+
+	/* Got user response? */
+	if(response)
+	{
+	    const gchar *path = (npaths > 0) ?
+		paths_list[0] : NULL;
+	    if(!STRISEMPTY(path))
+	    {
+		gtk_entry_set_text(entry, path);
+		gtk_entry_set_position(entry, -1);
+	    }
+	}
+
+	/* Delete the file types list */
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
+}
+
+/*
+ *	Devices Edit Window browse mount path callback.
+ */
+static void EDVDevicesEditWinBrowseMountPathCB(
+	GtkWidget *widget, gpointer data
+)
+{
+	gboolean response;
+	GtkWidget *toplevel;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
+	GtkEntry *entry = (GtkEntry *)data;
+	if((entry == NULL) || FileBrowserIsQuery())
+	    return;
+
+	toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
+
+	/* Create the file types list */
+	FileBrowserTypeListNew(
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
+	);
+
+	/* Query the user for the mount path */
+	FileBrowserSetTransientFor(toplevel);
+	response = FileBrowserGetResponse(
+	    "Select Mount Path",
+	    "Select", "Cancel",
+	    gtk_entry_get_text(entry),          /* Startup path */
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
+	    &ftype_rtn
+	);
+	FileBrowserSetTransientFor(NULL);
+
+	/* Got user response? */
+	if(response)
+	{
+	    const gchar *path = (npaths > 0) ?
+		paths_list[0] : NULL;
+	    if(!STRISEMPTY(path))
+	    {
+		gtk_entry_set_text(entry, path);
+		gtk_entry_set_position(entry, -1);
+	    }
+	}
+
+	/* Delete the file types list */
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
+}
+
+/*
+ *	Devices Edit Window browse program callback.
+ */
+static void EDVDevicesEditWinBrowseProgramCB(
+	GtkWidget *widget, gpointer data
+)
+{
+	gboolean response;
+	GtkWidget *toplevel;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
+	GtkEntry *entry = (GtkEntry *)data;
+	if((entry == NULL) || FileBrowserIsQuery())
+	    return;
+
+	toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
+
+	/* Create the file types list */
+	FileBrowserTypeListNew(
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
+	);
+
+	/* Query the user for the program */
+	FileBrowserSetTransientFor(toplevel);
+	response = FileBrowserGetResponse(
+	    "Select Program",
+	    "Select", "Cancel",
+	    gtk_entry_get_text(entry),          /* Startup path */
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
+	    &ftype_rtn
+	);
+	FileBrowserSetTransientFor(NULL);
+
+	/* Got user response? */
+	if(response)
+	{
+	    const gchar *path = (npaths > 0) ?
+		paths_list[0] : NULL;
+	    if(!STRISEMPTY(path))
+	    {
+		gtk_entry_set_text(entry, path);
+		gtk_entry_set_position(entry, -1);
+	    }
+	}
+
+	/* Delete the file types list */
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
+}
+
+
 /*
  *	Filesystem type popup list box changed signal callback.
  */
@@ -801,7 +818,7 @@ static void EDVDevicesEditWinApplyCB(
 	    dev->flags |= EDV_DEVICE_NO_UNMOUNT;
 	else
 	    dev->flags &= ~EDV_DEVICE_NO_UNMOUNT;
-	/* Read Only */
+	/* Read-Only */
 	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(d->read_only_check))
 	    dev->flags |= EDV_DEVICE_READ_ONLY;
 	else
@@ -811,6 +828,11 @@ static void EDVDevicesEditWinApplyCB(
 	    dev->flags |= EDV_DEVICE_UNLISTED;
 	else
 	    dev->flags &= ~EDV_DEVICE_UNLISTED;
+	/* No Scan */
+	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(d->no_scan_check))
+	    dev->flags |= EDV_DEVICE_NO_SCAN;
+	else
+	    dev->flags &= ~EDV_DEVICE_NO_SCAN;
 
 	/* Begin applying icons file paths */
 
@@ -881,6 +903,9 @@ static void EDVDevicesEditWinCancelCB(
 	if(d == NULL)
 	    return;
 
+	if(d->freeze_count > 0)
+	    return;
+
 	EDVDevicesEditWinUnmap(d);
 }
 
@@ -1013,7 +1038,7 @@ void EDVDevicesEditWinFetchValues(
 	    d->no_unmount_check,
 	    EDV_DEVICE_IS_NO_UNMOUNT(dev)
 	);
-	/* Read Only */
+	/* Read-Only */
 	GTK_TOGGLE_BUTTON_SET_ACTIVE(
 	    d->read_only_check,
 	    EDV_DEVICE_IS_READ_ONLY(dev)
@@ -1023,8 +1048,13 @@ void EDVDevicesEditWinFetchValues(
 	    d->unlisted_check,
 	    EDV_DEVICE_IS_UNLISTED(dev)
 	);
+	/* No Scan */
+	GTK_TOGGLE_BUTTON_SET_ACTIVE(
+	    d->no_scan_check,
+	    EDV_DEVICE_IS_NO_SCAN(dev)
+	);
 
-	/* Begin loading icons */
+	/* Begin loading the icons */
 
 #define DO_LOAD_ICON	{				\
  gint i;						\
@@ -1136,21 +1166,23 @@ void EDVDevicesEditWinDeviceRemovedCB(
 static edv_device_icon_picker_struct *EDVDevicesEditIconNew(
 	edv_device_edit_dlg_struct *d,
 	GtkWidget *parent, const gchar *title,
-	gint icon_width, gint icon_height,
-	gint btn_width, gint btn_height
+	const gint icon_width, const gint icon_height,
+	const gint btn_width, const gint btn_height
 )
 {
 	gint bw = 20, bh = 20;
 	GtkAdjustment *adj;
 	GtkWidget *w, *parent2, *parent3, *parent4;
-	edv_device_icon_picker_struct *ei = EDV_DEVICE_ICON_PICKER(
+	edv_device_icon_picker_struct *ei;
+
+	if(d == NULL)
+	    return(NULL);
+
+	ei = EDV_DEVICE_ICON_PICKER(
 	    g_malloc0(sizeof(edv_device_icon_picker_struct))
 	);
-	if((ei == NULL) || (d == NULL))
-	{
-	    g_free(ei);
+	if(ei == NULL)
 	    return(NULL);
-	}
 
 	ei->d = d;
 	ei->icon_width = icon_width;
@@ -1165,13 +1197,13 @@ static edv_device_icon_picker_struct *EDVDevicesEditIconNew(
 	    1.0f		/* Page size */
 	);
 
-	/* Create toplevel */
+	/* Toplevel GtkVBox */
 	ei->toplevel = w = gtk_vbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Hbox for title label */
+	/* Title GtkHBox */
 	w = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1182,7 +1214,7 @@ static edv_device_icon_picker_struct *EDVDevicesEditIconNew(
 	gtk_widget_show(w);
 
 
-	/* Hbox for button */
+	/* Button GtkHBox */
 	w = gtk_hbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1303,7 +1335,10 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gpointer entry_rtn, browse_rtn;
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
-	GtkWidget *w, *parent, *parent2, *parent3, *parent4, *parent5;
+	GtkWidget	*w,
+			*parent, *parent2, *parent3, *parent4,
+			*parent5,
+			*toplevel;
 	pulist_struct *pulist;
 	pulistbox_struct *pulistbox;
 	const edv_fs_type_struct *fs_type_ptr;
@@ -1317,29 +1352,21 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	    return(NULL);
 	}
 
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
-	d->processing = FALSE;
 	d->busy_count = 0;
 	d->freeze_count = 0;
-	d->core = core;
-
-	d->list_win = lw;
-
 	d->has_changes = 0;
-	d->title = NULL;
-
+	d->title = STRDUP(EDITWIN_TITLE);
+	d->core = core;
+	d->list_win = lw;
 	d->dev_num = -1;
 
-
-	/* Format title */
-	d->title = STRDUP(EDITWIN_TITLE);
-
-
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, EDITWIN_WIDTH, EDITWIN_HEIGHT);
+	gtk_window_set_title(GTK_WINDOW(w), d->title);
 	gtk_window_set_wmclass(
 	    GTK_WINDOW(w), "dialog", PROG_NAME
 	);
@@ -1371,8 +1398,7 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_window_add_accel_group(GTK_WINDOW(w), accelgrp);
 	parent = w;
 
-
-	/* Main vbox */
+	/* Main GtkVBox */
 	d->main_vbox = w = gtk_vbox_new(FALSE, border_major);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_container_add(GTK_CONTAINER(parent), w);
@@ -1380,21 +1406,21 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	parent = w;
 
 
-	/* Hbox to split things into two columns */
+	/* GtkHBox to make two columns */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
 
-	/* Left side vbox */
+	/* Left side GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent3 = w;
 
 
-	/* Identification frame */
+	/* Identification GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Identificacin"
@@ -1451,12 +1477,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Entre el nombre de este artefacto (este valor hace slo sea usado\
  para propsitos de despliegue)"
@@ -1486,7 +1514,7 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 
 
 	/* Device path entry */
-	w = GUIPromptBar(
+	w = GUIPromptBarWithBrowse(
 	    NULL,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Sendero Dispositivo"
@@ -1505,19 +1533,23 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 #else
 "Device Path"
 #endif
-	    ":", NULL, &entry_rtn
+	    ":", NULL, &entry_rtn, &browse_rtn,
+	    NULL, NULL
 	);
 	d->device_path_entry = (GtkWidget *)entry_rtn;
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	EDVEntrySetCompletePath(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Entre el sendero al objeto del artefacto"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1533,13 +1565,19 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "G inn i stien til innretnings objektet"
 #else
-"Enter the path to the device object"
+"Enter the path to the device node"
 #endif
 	);
 
+	w = (GtkWidget *)browse_rtn;
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "clicked",
+	    GTK_SIGNAL_FUNC(EDVDevicesEditWinBrowseDeviceCB), entry_rtn
+	);
+
 
 	/* Mount path entry */
-	w = GUIPromptBar(
+	w = GUIPromptBarWithBrowse(
 	    NULL,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Monte Sendero"
@@ -1558,19 +1596,23 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 #else
 "Mount Path"
 #endif
-	    ":", NULL, &entry_rtn
+	    ":", NULL, &entry_rtn, &browse_rtn,
+	    NULL, NULL
 	);
 	d->mount_path_entry = (GtkWidget *)entry_rtn;
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	EDVEntrySetCompletePath(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Entre el sendero al objeto de la gua que este artefacto\
  deber ser montado en"
@@ -1593,13 +1635,19 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 "G inn i stien til katalogobjektet som denne innretningen er montert\
  p"
 #else
-"Enter the path to the directory object that this device is to\
+"Enter the path to the directory that this device is to\
  be mounted on"
 #endif
 	);
 
+	w = (GtkWidget *)browse_rtn;
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "clicked",
+	    GTK_SIGNAL_FUNC(EDVDevicesEditWinBrowseMountPathCB), entry_rtn
+	);
 
-	/* Filesystem Frame */
+
+	/* Filesystem GtkFrame */
 	w = gtk_frame_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Archivesistema"
@@ -1645,12 +1693,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	d->fs_type_pulistbox = pulistbox = PUListBoxNew(
 	    parent5, -1, -1
 	);
+	pulist = PUListBoxGetPUList(pulistbox);
+        if(core->run_flags & EDV_RUN_SAFE_MODE)
+            PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
 	PUListBoxSetChangedCB(
 	    pulistbox,
 	    EDVDeviceEditDlgFSTypeChangedCB, d
 	);
 	PUListBoxMap(pulistbox);   
-	pulist = PUListBoxGetPUList(pulistbox);
 	for(i = 0; fs_type_list[i].fs_type; i++)
 	{
 	    fs_type_ptr = &fs_type_list[i];
@@ -1750,12 +1800,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 "Enter the command that will be executed to mount\
  this device (or leave it blank if it is not mountable)"
@@ -1796,12 +1848,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Enter the command that will be executed to unmount\
  this device (or leave it blank if it is not unmountable)"
@@ -1842,12 +1896,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Enter the command that will be executed to eject\
  the media from the device (or leave it blank if the device does not\
@@ -1892,12 +1948,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Enter the command that will be executed to run the\
  file system check program for this device (or leave it blank if\
@@ -1941,12 +1999,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Enter the command that will be executed to run the\
  tools program for this device (or leave it blank if you do not\
@@ -1990,12 +2050,14 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 
 	w = (GtkWidget *)entry_rtn;
-	EDVEntrySetDND(core, w);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
 	);
-	GUISetWidgetTip(w,
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
+	GUISetWidgetTip(
+	    w,
 #ifdef PROG_LANGUAGE_ENGLISH
 "Enter the command that will be executed to run the\
  format program for this device (or leave it blank if you do not\
@@ -2020,7 +2082,7 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	);
 
 
-	/* Options frame */
+	/* Options GtkFrame */
 	w = gtk_frame_new(
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Options"
@@ -2042,7 +2104,7 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	gtk_widget_show(w);
 	parent4 = w;
 
-	/* No unmount check */
+	/* No Unmount GtkCheckButton */
 	d->no_unmount_check = w = gtk_check_button_new_with_label(
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "No Unmount"
@@ -2066,10 +2128,10 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	);
 	gtk_widget_show(w);
 
-	/* Read only check */
+	/* Read-Only GtkCheckButton */
 	d->read_only_check = w = gtk_check_button_new_with_label(
 #ifdef PROG_LANGUAGE_ENGLISH
-	    "Read Only"
+	    "Read-Only"
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 	    "Lea Slo"
@@ -2089,7 +2151,7 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	);
 	gtk_widget_show(w);
 
-	/* Unlisted check */
+	/* Unlisted GtkCheckButton */
 	d->unlisted_check = w = gtk_check_button_new_with_label(
 #ifdef PROG_LANGUAGE_ENGLISH
 	    "Unlisted"
@@ -2113,9 +2175,26 @@ edv_device_edit_dlg_struct *EDVDevicesEditWinNew(
 	);
 	gtk_widget_show(w);
 
+	/* No Scan GtkCheckButton */
+	d->no_scan_check = w = gtk_check_button_new_with_label(
+	    "No Scan"
+	);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "toggled",
+	    GTK_SIGNAL_FUNC(EDVDevicesEditWinAnyChangedCB), d
+	);
+	GUISetWidgetTip(
+	    w,
+"Check this to avoid scanning the mount path for objects\
+ unless you explicitly select it (useful for\
+ conserving bandwidth on NFS filesystems)"
+	);
+	gtk_widget_show(w);
 
 
-	/* Right side vbox */
+
+	/* Right side GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -2415,16 +2494,16 @@ void EDVDevicesEditWinUpdateMenus(edv_device_edit_dlg_struct *d)
 	gboolean        sensitive,
 			has_changes;
 	gint dev_num;
+	gchar *s;
 	const edv_device_struct *dev;
-	GtkWidget *w;
 	edv_core_struct *core;
 
 	if(d == NULL)
 	    return;
 
+	d->freeze_count++;
+
 	core = d->core;
-	if(core == NULL)
-	    return;
 
 	has_changes = d->has_changes;
 	dev_num = d->dev_num;
@@ -2433,45 +2512,39 @@ void EDVDevicesEditWinUpdateMenus(edv_device_edit_dlg_struct *d)
 	else
 	    dev = NULL;
 
-#define MAP(w)                  \
-{ if((w) != NULL) { gtk_widget_show(w); } }
-#define UNMAP(w)                \
-{ if((w) != NULL) { gtk_widget_hide(w); } }
+	/* Title */
+	s = g_strconcat(
+	    (d->title != NULL) ? d->title : "",
+	    (has_changes) ? " (*)" : "",
+	    NULL
+	);
+	gtk_window_set_title(GTK_WINDOW(d->toplevel), s);
+	g_free(s);
 
-	/* Update title */
-	w = d->toplevel;
-	if(w != NULL)
-	{
-	    gchar *title = g_strdup_printf(
-		"%s%s",
-		STRISEMPTY(d->title) ? "Untitled" : d->title,
-		(has_changes) ? " (*)" : ""
-	    );
-	    gtk_window_set_title(GTK_WINDOW(w), title);
-	    g_free(title);
-	}
-
-	/* Update OK, Apply, Save, Cancel Buttons */
+	/* OK, Apply, Cancel, Close Buttons */
+	sensitive = has_changes ? TRUE : FALSE;
+	gtk_widget_set_sensitive(d->ok_btn, sensitive);
+	gtk_widget_set_sensitive(d->apply_btn, sensitive);
 	sensitive = has_changes;
-	GTK_WIDGET_SET_SENSITIVE(d->ok_btn, sensitive)
-	GTK_WIDGET_SET_SENSITIVE(d->apply_btn, sensitive)
-
-	if(has_changes)
+	gtk_widget_set_sensitive(d->cancel_btn, sensitive);
+	sensitive = has_changes ? FALSE : TRUE;      
+	gtk_widget_set_sensitive(d->close_btn, sensitive);
+	if(has_changes)                                  
 	{
-	    MAP(d->cancel_btn);
-	    UNMAP(d->close_btn);
+	    gtk_widget_show(d->ok_btn);
+	    gtk_widget_show(d->apply_btn);
+	    gtk_widget_show(d->cancel_btn);
+	    gtk_widget_hide(d->close_btn);
 	}
 	else
 	{
-	    UNMAP(d->cancel_btn);
-	    MAP(d->close_btn);
+	    gtk_widget_show(d->ok_btn);
+	    gtk_widget_show(d->apply_btn);
+	    gtk_widget_hide(d->cancel_btn);
+	    gtk_widget_show(d->close_btn);
 	}
-	sensitive = TRUE;
-	GTK_WIDGET_SET_SENSITIVE(d->cancel_btn, sensitive)
-	GTK_WIDGET_SET_SENSITIVE(d->close_btn, sensitive)
 
-#undef UNMAP
-#undef MAP
+	d->freeze_count--;
 }
 
 /*
@@ -2481,8 +2554,10 @@ gboolean EDVDevicesEditWinIsMapped(
 	edv_device_edit_dlg_struct *d
 )
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(d == NULL)
+            return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(d->toplevel));
 }
 
 /*
@@ -2492,14 +2567,12 @@ void EDVDevicesEditWinMap(
 	edv_device_edit_dlg_struct *d
 )
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	if(d == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(d->toplevel);
 
-	w = d->name_entry;
-	gtk_widget_grab_focus(w);
+	gtk_widget_grab_focus(d->name_entry);
 }
 
 /*
@@ -2509,11 +2582,10 @@ void EDVDevicesEditWinUnmap(
 	edv_device_edit_dlg_struct *d
 )
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	if(d == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(d->toplevel);
 }
 
 /*
@@ -2526,36 +2598,22 @@ void EDVDevicesEditWinDelete(
 	if(d == NULL)
 	    return;
 
+	EDVDevicesEditWinUnmap(d);
+
+	d->freeze_count++;
+
 	EDVDevicesEditIconDelete(d->icon_small);
 	EDVDevicesEditIconDelete(d->icon_medium);
 	EDVDevicesEditIconDelete(d->icon_large);
 
-	GTK_WIDGET_DESTROY(d->ok_btn);
-	GTK_WIDGET_DESTROY(d->apply_btn);
-	GTK_WIDGET_DESTROY(d->cancel_btn);
-	GTK_WIDGET_DESTROY(d->close_btn);
-
-	GTK_WIDGET_DESTROY(d->name_entry);
-	GTK_WIDGET_DESTROY(d->device_path_entry);
-	GTK_WIDGET_DESTROY(d->mount_path_entry);
 	PUListBoxDelete(d->fs_type_pulistbox);
 
-	GTK_WIDGET_DESTROY(d->command_mount_entry);
-	GTK_WIDGET_DESTROY(d->command_unmount_entry);
-	GTK_WIDGET_DESTROY(d->command_eject_entry);
-	GTK_WIDGET_DESTROY(d->command_check_entry);
-	GTK_WIDGET_DESTROY(d->command_tools_entry);
-	GTK_WIDGET_DESTROY(d->command_format_entry);
-
-	GTK_WIDGET_DESTROY(d->no_unmount_check);
-	GTK_WIDGET_DESTROY(d->read_only_check);
-	GTK_WIDGET_DESTROY(d->unlisted_check);
-
-	GTK_WIDGET_DESTROY(d->toplevel);
-
-	GTK_ACCEL_GROUP_UNREF(d->accelgrp);
+	gtk_widget_destroy(d->toplevel);
+	gtk_accel_group_unref(d->accelgrp);
 
 	g_free(d->title);
 
+	d->freeze_count--;
+
 	g_free(d);
 }
