@@ -1,5 +1,6 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -142,38 +143,38 @@ void EDVDevicesListWinDelete(
 );
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
-
-
 #if defined(PROG_LANGUAGE_SPANISH)
-# define LISTWIN_TITLE	"Artefactos"
+# define LISTWIN_TITLE			"Artefactos"
 #elif defined(PROG_LANGUAGE_FRENCH)
-# define LISTWIN_TITLE	"D'appareils"
+# define LISTWIN_TITLE			"D'appareils"
 #elif defined(PROG_LANGUAGE_GERMAN)
-# define LISTWIN_TITLE	"Vorrichtungen"
+# define LISTWIN_TITLE			"Vorrichtungen"
 #elif defined(PROG_LANGUAGE_ITALIAN)
-# define LISTWIN_TITLE	"I Congegni"
+# define LISTWIN_TITLE			"I Congegni"
 #elif defined(PROG_LANGUAGE_DUTCH)
-# define LISTWIN_TITLE	"Apparaten"
+# define LISTWIN_TITLE			"Apparaten"
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
-# define LISTWIN_TITLE	"Os Artifcios"
+# define LISTWIN_TITLE			"Os Artifcios"
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
-# define LISTWIN_TITLE	"Innretninger"
+# define LISTWIN_TITLE			"Innretninger"
 #else
-# define LISTWIN_TITLE	"Devices"
+# define LISTWIN_TITLE			"Devices"
 #endif
 
-#define LISTWIN_WIDTH		550
-#define LISTWIN_HEIGHT		460
+#define LISTWIN_WIDTH			550
+#define LISTWIN_HEIGHT			460
+
+
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
+
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -391,7 +392,7 @@ static gint EDVDevicesListWinButtonPressEventCB(
 		/* Handle by button number */
 		switch(button->button)
 		{
-		  case 1:
+		  case GDK_BUTTON1:
 		    EDVDevicesListWinEditCB(NULL, lw);
 		    break;
 		}
@@ -402,7 +403,7 @@ static gint EDVDevicesListWinButtonPressEventCB(
 		/* Handle by button number */
 		switch(button->button)
 		{
-		  case 3:
+		  case GDK_BUTTON3:
 		    /* Select row before mapping menu? */
 		    if(EDV_GET_B(EDV_CFG_PARM_RIGHT_CLICK_MENU_SELECTS) &&
 		       (row >= 0) && (row < clist->rows)
@@ -559,7 +560,7 @@ static void EDVDevicesListWinMountCB(GtkWidget *widget, gpointer data)
 	/* Mount error? */
 	if(status)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -643,7 +644,7 @@ static void EDVDevicesListWinEjectCB(GtkWidget *widget, gpointer data)
 	/* Mount error? */
 	if(status)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -671,6 +672,7 @@ static void EDVDevicesListWinPropertiesCB(GtkWidget *widget, gpointer data)
 {
 	gint row, dev_num;
 	const gchar *mount_path;
+	GtkWidget *toplevel;
 	GtkCList *clist;
 	edv_device_struct *dev;
 	edv_core_struct *core;
@@ -678,10 +680,9 @@ static void EDVDevicesListWinPropertiesCB(GtkWidget *widget, gpointer data)
 	if(lw == NULL)
 	    return;
 
-	clist = (GtkCList *)lw->devices_clist;
+	toplevel = lw->toplevel;
+	clist = GTK_CLIST(lw->devices_clist);
 	core = lw->core;
-	if((clist == NULL) || (core == NULL))
-	    return;
 
 	/* Get selected row and device number index as the row index */
 	dev_num = row = EDVCListGetSelectedLast(clist, NULL);
@@ -694,30 +695,40 @@ static void EDVDevicesListWinPropertiesCB(GtkWidget *widget, gpointer data)
 
 	EDVDevicesListSetBusy(lw, TRUE);
 
-	/* Get mount path */
+	/* Get the mount path */
 	mount_path = dev->mount_path;
 	if(!STRISEMPTY(mount_path))
 	{
-	    struct stat lstat_buf;
-
-	    /* Check if mount path exists locally */
-	    if(!lstat(mount_path, &lstat_buf))
+	    edv_object_struct *obj = EDVObjectNew();
+	    if(obj != NULL)
 	    {
-		edv_object_struct *obj = EDVObjectNew();
-		if(obj != NULL)
-		{
-		    EDVObjectSetPath(obj, mount_path);
-		    EDVObjectSetStat(obj, &lstat_buf);
-		    EDVObjectUpdateLinkFlags(obj);
+		struct stat lstat_buf;
 
-		    /* Create a new Properties Dialog */
-		    EDVNewPropertiesDialogPage(
-			core,
-			obj,		/* Object */
-			"Device",	/* Page Name */
-			lw->toplevel	/* Toplevel */
-		    );
+		EDVObjectSetPath(obj, mount_path);
+		if(lstat(mount_path, &lstat_buf))
+		{
+	            const gint error_code = (gint)errno;
+	            EDVObjectPropSet(
+	                obj,
+	                EDV_OBJECT_PROP_NAME_ERROR,
+	                g_strerror(error_code),
+	                TRUE
+	            );
 		}
+		else
+		{
+		    EDVObjectSetStat(obj, &lstat_buf);
+		}
+		EDVObjectUpdateLinkFlags(obj);
+
+		/* Create a new Properties Dialog */
+		EDVNewPropertiesDialogPage(
+		    core,
+		    obj,
+		    "Device",			/* Page Name */
+		    toplevel
+		);
+
 		EDVObjectDelete(obj);
 	    }
 	}
@@ -794,7 +805,7 @@ static void EDVDevicesListWinToolsCB(GtkWidget *widget, gpointer data)
 	EDVDevicesListSetBusy(lw, TRUE);
 
 	/* Run the device tools */
-	EDVRunDeviceTools(core, dev, toplevel);                           
+	EDVRunDeviceTools(core, dev, toplevel);
 
 	EDVDevicesListSetBusy(lw, FALSE);
 }
@@ -1676,7 +1687,7 @@ static void EDVDevicesListWinUpdateDisplay(
 "Mount/unmount the device"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(w, !EDV_DEVICE_IS_NO_UNMOUNT(dev));
+	gtk_widget_set_sensitive(w, !EDV_DEVICE_IS_NO_UNMOUNT(dev));
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1710,10 +1721,10 @@ static void EDVDevicesListWinUpdateDisplay(
 "Eject the media from the device"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(
+	gtk_widget_set_sensitive(
 	    w,
 	    !STRISEMPTY(dev->command_eject) ? TRUE : FALSE
-	)
+	);
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1747,10 +1758,10 @@ static void EDVDevicesListWinUpdateDisplay(
 "Modify the properties of the mount path"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(
+	gtk_widget_set_sensitive(
 	    w,
 	    !STRISEMPTY(dev->mount_path) ? TRUE : FALSE
-	)
+	);
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1784,10 +1795,10 @@ static void EDVDevicesListWinUpdateDisplay(
 "Check the device's file system"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(
+	gtk_widget_set_sensitive(
 	    w,
 	    !STRISEMPTY(dev->command_check) ? TRUE : FALSE
-	)
+	);
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1802,7 +1813,8 @@ static void EDVDevicesListWinUpdateDisplay(
 	    GTK_OBJECT(w), "clicked",
 	    GTK_SIGNAL_FUNC(EDVDevicesListWinToolsCB), lw
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Corra el programa de instrumentos de dispositivo"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1821,10 +1833,10 @@ static void EDVDevicesListWinUpdateDisplay(
 "Run the device's tools program"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(
+	gtk_widget_set_sensitive(
 	    w,
 	    !STRISEMPTY(dev->command_tools) ? TRUE : FALSE
-	)
+	);
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1858,10 +1870,10 @@ static void EDVDevicesListWinUpdateDisplay(
 "Format the media in the device"
 #endif
 	);
-	GTK_WIDGET_SET_SENSITIVE(
+	gtk_widget_set_sensitive(
 	    w,
 	    !STRISEMPTY(dev->command_format) ? TRUE : FALSE
-	)
+	);
 	if(standard_rcstyle != NULL)
 	    gtk_widget_modify_style_recursive(w, standard_rcstyle);
 	gtk_widget_show(w);
@@ -1877,6 +1889,7 @@ static void EDVDevicesListWinUpdateDisplay(
 	if(EDV_DEVICE_IS_MOUNTED(dev))
 	{
 	    const gulong	adj_division = 10l,
+				block_size = EDV_GET_UL(EDV_CFG_PARM_BLOCK_SIZE),
 				total_kb = dev->blocks_total,
 				free_kb = dev->blocks_free,
 				available_kb = dev->blocks_available,
@@ -1891,13 +1904,12 @@ static void EDVDevicesListWinUpdateDisplay(
 	        (gfloat)(total_kb / adj_division),
 	        0.0f, 0.0f, 0.0f
 	    );
-	    GDK_COLOR_SET_COEFF(&c, 0.0f, 0.0f, 1.0f)
-	    ss_kb = STRDUP(
-		EDVGetObjectSizeStr(core, total_kb)
-	    );
-	    ss_mb = STRDUP(
-		EDVGetObjectSizeStr(core, total_kb / 1000l)
-	    );
+	    GDK_COLOR_SET_COEFF(&c, 0.0f, 0.0f, 1.0f);
+	    ss_kb = STRDUP(EDVSizeStrDelim(total_kb));
+	    ss_mb = STRDUP(EDVSizeStrDelim(
+		(block_size > 0l) ?
+		    (total_kb / block_size) : (total_kb / 1024l)
+	    ));
 	    text = g_strdup_printf(
 	        "%s: %s kb (%s mb)",
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1922,12 +1934,11 @@ static void EDVDevicesListWinUpdateDisplay(
 	    g_free(ss_kb);
 	    g_free(ss_mb);
 
-	    ss_kb = STRDUP(
-		EDVGetObjectSizeStr(core, dev->blocks_free)
-	    );
-	    ss_mb = STRDUP(
-		EDVGetObjectSizeStr(core, dev->blocks_free / 1000)
-	    );
+	    ss_kb = STRDUP(EDVSizeStrDelim(free_kb));
+	    ss_mb = STRDUP(EDVSizeStrDelim(
+		(block_size > 0l) ?
+		    (free_kb / block_size) : (free_kb / 1024l)
+	    ));
 	    text2 = g_strdup_printf(
 	        "%s kb (%s mb)",
 		ss_kb, ss_mb
@@ -1936,7 +1947,8 @@ static void EDVDevicesListWinUpdateDisplay(
 	    g_free(ss_mb);
 
 	    lw->dev_pc = pc = PieChartNew(
-	        adj, &c, 110, 70,
+		110, 70,
+	        adj, &c,
 	        NULL, text,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Libre"
@@ -1962,20 +1974,19 @@ static void EDVDevicesListWinUpdateDisplay(
 	    g_free(text2);
 	    w = pc->toplevel;
 	    gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, FALSE, 0);
-	    gtk_widget_show(w);
+	    PieChartMap(pc);
 
-	    GDK_COLOR_SET_COEFF(&c, 0.0f, 1.0f, 1.0f)
+	    GDK_COLOR_SET_COEFF(&c, 0.0f, 1.0f, 1.0f);
 	    adj = (GtkAdjustment *)gtk_adjustment_new(
 	        0.0f, 0.0f,
 	        (gfloat)(available_kb / adj_division),
 	        0.0f, 0.0f, 0.0f
 	    );
-	    ss_kb = STRDUP(
-		EDVGetObjectSizeStr(core, available_kb)
-	    );
-	    ss_mb = STRDUP(
-		EDVGetObjectSizeStr(core, available_kb / 1000l)
-	    );
+	    ss_kb = STRDUP(EDVSizeStrDelim(available_kb));
+	    ss_mb = STRDUP(EDVSizeStrDelim(
+		(block_size > 0l) ?
+		    (available_kb / block_size) : (available_kb / 1024l)
+	    ));
 	    text = g_strdup_printf(
 	        "%s kb (%s mb)",
 		ss_kb, ss_mb
@@ -2012,17 +2023,16 @@ static void EDVDevicesListWinUpdateDisplay(
 	        (gfloat)(used_kb / adj_division),
 	        0.0f, 0.0f, 0.0f
 	    );
-	    ss_kb = STRDUP(
-		EDVGetObjectSizeStr(core, used_kb)
-	    );
-	    ss_mb = STRDUP(
-		EDVGetObjectSizeStr(core, used_kb / 1000l)
-	    );
+	    ss_kb = STRDUP(EDVSizeStrDelim(used_kb));
+	    ss_mb = STRDUP(EDVSizeStrDelim(
+		(block_size > 0l) ?
+		    (used_kb / block_size) : (used_kb / 1024l)
+	    ));
 	    text = g_strdup_printf(
 	        "%s kb (%s mb)",
 		ss_kb, ss_mb
 	    );
-	    g_free(ss_kb);   
+	    g_free(ss_kb);
 	    g_free(ss_mb);
  	    PieChartValueAdd(
 	        pc, adj, &c,
@@ -2531,8 +2541,10 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	GtkRcStyle *standard_rcstyle, *lists_rcstyle;
 	GtkAccelGroup *accelgrp;
 	GtkStyle *style;
-	GtkWidget *w, *menu, *parent, *parent2, *parent3, *parent4, *parent5;
-	GtkEntry *entry;
+	GtkWidget	*w, *menu,
+			*parent, *parent2, *parent3, *parent4,
+			*parent5,
+			*toplevel;
 	GtkCList *clist;
 	edv_devices_list_win_struct *lw = EDV_DEVICE_LISTWIN(
 	    g_malloc0(sizeof(edv_devices_list_win_struct))
@@ -2549,17 +2561,17 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	wm_name = core->wm_name;
 	wm_class = core->wm_class;
 
+	lw->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	lw->accelgrp = accelgrp = gtk_accel_group_new();
-	lw->processing = FALSE;
 	lw->busy_count = 0;
 	lw->freeze_count = 0;
 	lw->core = core;
 
+	lw->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	lw->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, LISTWIN_WIDTH, LISTWIN_HEIGHT);
 	gtk_window_set_title(GTK_WINDOW(w), LISTWIN_TITLE);
 	if(!STRISEMPTY(wm_name) && !STRISEMPTY(wm_class))
@@ -2617,7 +2629,7 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 #undef LOAD_PIXMAP
 #endif
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	lw->main_vbox = w = gtk_vbox_new(FALSE, border_major);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_container_add(GTK_CONTAINER(parent), w);
@@ -2625,7 +2637,7 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	parent = w;
 
 
-	/* Hbox for find prompt */
+	/* Find GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -2653,17 +2665,18 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	    ":", NULL, &entry_rtn
 	);
 	lw->find_entry = (GtkWidget *)entry_rtn;
-	entry = (GtkEntry *)entry_rtn;
-	gtk_signal_connect(
-	    GTK_OBJECT(entry), "activate",
-	    GTK_SIGNAL_FUNC(EDVDevicesListWinFindCB), lw
-	);
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
-	EDVEntrySetDND(core, (GtkWidget *)entry_rtn);
 	gtk_widget_show(w);
 
+	w = (GtkWidget *)entry_rtn;
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "activate",
+	    GTK_SIGNAL_FUNC(EDVDevicesListWinFindCB), lw
+	);
+	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
 
-	/* Vbox for devices clist and buttons */
+	/* Devices clist and buttons GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -2779,10 +2792,25 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	gtk_container_add(GTK_CONTAINER(parent3), w);
 	gtk_widget_realize(w);
 	gtk_clist_set_column_width(clist, 0, 130);
+	gtk_clist_set_column_justification(
+	    clist, 0, GTK_JUSTIFY_LEFT
+	);
 	gtk_clist_set_column_width(clist, 1, 100);
+	gtk_clist_set_column_justification(
+	    clist, 1, GTK_JUSTIFY_LEFT
+	);
 	gtk_clist_set_column_width(clist, 2, 100);
+	gtk_clist_set_column_justification(
+	    clist, 2, GTK_JUSTIFY_LEFT
+	);
 	gtk_clist_set_column_width(clist, 3, 80);
+	gtk_clist_set_column_justification(
+	    clist, 3, GTK_JUSTIFY_LEFT
+	);
 	gtk_clist_set_column_width(clist, 4, 50);
+	gtk_clist_set_column_justification(
+	    clist, 4, GTK_JUSTIFY_LEFT
+	);
 	gtk_clist_column_titles_passive(clist);
 	gtk_clist_set_row_height(clist, EDV_LIST_ROW_SPACING);
 	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);
@@ -2843,7 +2871,7 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	gtk_accel_group_add(
 	    accelgrp, GDK_a, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE,
 	    GTK_OBJECT(w), "clicked"
-	);                          
+	);
 	GUIButtonLabelUnderline(w, GDK_a);
 	gtk_widget_show(w);
 
@@ -2881,7 +2909,7 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	gtk_accel_group_add(
 	    accelgrp, GDK_e, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE,
 	    GTK_OBJECT(w), "clicked"
-	);                          
+	);
 	GUIButtonLabelUnderline(w, GDK_e);
 	gtk_widget_show(w);
 
@@ -2919,58 +2947,44 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 	gtk_accel_group_add(
 	    accelgrp, GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE,
 	    GTK_OBJECT(w), "clicked"
-	);                          
+	);
 	GUIButtonLabelUnderline(w, GDK_r);
 	gtk_widget_show(w);
 
 
-	/* Hbox for up and down set of buttons */
+	/* GtkHBox for the up and down buttons */
 	w = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 
-	/* Vbox for up button */
+	/* GtkVBox for the up button */
 	w = gtk_vbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
 	/* Up button */
-	lw->up_btn = w = gtk_button_new();
+	lw->up_btn = w = GUIButtonArrow(GTK_ARROW_UP, 20, 20);
 	gtk_box_pack_start(GTK_BOX(parent5), w, TRUE, FALSE, 0);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "clicked",
 	    GTK_SIGNAL_FUNC(EDVDevicesListWinUpCB), lw
 	);
 	gtk_widget_show(w);
-	parent5 = w;
-	/* Arrow */
-	w = gtk_arrow_new(GTK_ARROW_UP, GTK_SHADOW_OUT);
-	gtk_widget_set_usize(w, 20, 20);
-	gtk_container_add(GTK_CONTAINER(parent5), w);
-	gtk_widget_show(w);
 
-	/* Vbox for down button */
+	/* GtkVBox for the down button */
 	w = gtk_vbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent4), w, TRUE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
 	/* Down button */
-	lw->down_btn = w = gtk_button_new();
-	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
+	lw->down_btn = w = GUIButtonArrow(GTK_ARROW_DOWN, 20, 20);
+	gtk_box_pack_start(GTK_BOX(parent5), w, TRUE, FALSE, 0);
 	gtk_signal_connect(
 	    GTK_OBJECT(w), "clicked",
 	    GTK_SIGNAL_FUNC(EDVDevicesListWinDownCB), lw
 	);
 	gtk_widget_show(w);
-	parent5 = w;
-	/* Arrow */
-	w = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
-	gtk_widget_set_usize(w, 20, 20);
-	gtk_container_add(GTK_CONTAINER(parent5), w);
-	gtk_widget_show(w);
-
-
 
 
 
@@ -3224,6 +3238,8 @@ edv_devices_list_win_struct *EDVDevicesListWinNew(
 
 	EDVDevicesListWinUpdateMenus(lw);
 
+	lw->freeze_count--;
+
 	return(lw);
 }
 
@@ -3236,32 +3252,48 @@ void EDVDevicesListWinUpdateMenus(
 )
 {
 	gboolean sensitive;
-	gint sel_row;
+	gint nrows, sel_row, nselected;
 	GtkCList *clist;
 
 	if(lw == NULL)
 	    return;
 
-	/* Get selected row on clist */
-	clist = (GtkCList *)lw->devices_clist;
+	lw->freeze_count++;
+
+	/* Devices GtkCList */
+	clist = GTK_CLIST(lw->devices_clist);
+	nrows = clist->rows;
+	nselected = g_list_length(clist->selection);
 	sel_row = EDVCListGetSelectedLast(clist, NULL);
 
-	sensitive = (sel_row > -1) ? TRUE : FALSE;
-	GTK_WIDGET_SET_SENSITIVE(lw->edit_btn, sensitive);
-	GTK_WIDGET_SET_SENSITIVE(lw->remove_btn, sensitive);
+	/* Add */
+	sensitive = (nselected <= 1) ? TRUE : FALSE;
+	gtk_widget_set_sensitive(lw->add_btn, sensitive);
+	gtk_widget_set_sensitive(lw->add_mi, sensitive);
 
-	GTK_WIDGET_SET_SENSITIVE(lw->edit_mi, sensitive);
-	GTK_WIDGET_SET_SENSITIVE(lw->remove_mi, sensitive);
+	/* Edit */
+	sensitive = (nselected == 1) ? TRUE : FALSE;
+	gtk_widget_set_sensitive(lw->edit_btn, sensitive);
+	gtk_widget_set_sensitive(lw->edit_mi, sensitive);
 
-	sensitive = (sel_row > 0) ? TRUE : FALSE;
-	GTK_WIDGET_SET_SENSITIVE(lw->up_btn, sensitive);
-	GTK_WIDGET_SET_SENSITIVE(lw->up_mi, sensitive);
+	/* Remove */
+	sensitive = (nselected >= 1) ? TRUE : FALSE;
+	gtk_widget_set_sensitive(lw->remove_btn, sensitive);
+	gtk_widget_set_sensitive(lw->remove_mi, sensitive);
 
-	sensitive = ((sel_row > -1) && (sel_row < (clist->rows - 1))) ?
-	    TRUE : FALSE;
-	GTK_WIDGET_SET_SENSITIVE(lw->down_btn, sensitive);
-	GTK_WIDGET_SET_SENSITIVE(lw->down_mi, sensitive);
+	/* Shift Up */
+	sensitive = ((g_list_find(clist->selection, (gpointer)0) == NULL) &&
+	    (nselected > 0)) ? TRUE : FALSE;
+	gtk_widget_set_sensitive(lw->up_btn, sensitive);
+	gtk_widget_set_sensitive(lw->up_mi, sensitive);
 
+	/* Shift Down */
+	sensitive = ((g_list_find(clist->selection, (gpointer)(nrows - 1)) == NULL) &&
+	    (nselected > 0)) ? TRUE : FALSE;
+	gtk_widget_set_sensitive(lw->down_btn, sensitive);
+	gtk_widget_set_sensitive(lw->down_mi, sensitive);
+
+	lw->freeze_count--;
 }
 
 /*
@@ -3323,8 +3355,10 @@ gboolean EDVDevicesListWinIsMapped(
 	edv_devices_list_win_struct *lw
 )
 {
-	GtkWidget *w = (lw != NULL) ? lw->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(lw == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(lw->toplevel));
 }
 
 /*
@@ -3334,14 +3368,11 @@ void EDVDevicesListWinMap(
 	edv_devices_list_win_struct *lw
 )
 {
-	GtkWidget *w = (lw != NULL) ? lw->toplevel : NULL;
-	if(w == NULL)
+	if(lw == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
-
-	w = lw->devices_clist;
-	gtk_widget_grab_focus(w);
+	gtk_widget_show_raise(lw->toplevel);
+	gtk_widget_grab_focus(lw->devices_clist);
 }
 
 /*
@@ -3351,11 +3382,10 @@ void EDVDevicesListWinUnmap(
 	edv_devices_list_win_struct *lw
 )
 {
-	GtkWidget *w = (lw != NULL) ? lw->toplevel : NULL;
-	if(w == NULL)
+	if(lw == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(lw->toplevel);
 }
 
 /*
@@ -3365,50 +3395,30 @@ void EDVDevicesListWinDelete(
 	edv_devices_list_win_struct *lw
 )
 {
+	GtkCList *clist;
+
 	if(lw == NULL)
 	    return;
 
-	/* Delete Device Edit dialog */
+	/* Delete the Device Edit dialog */
 	EDVDevicesEditWinDelete(lw->edit_dlg);
 	lw->edit_dlg = NULL;
 
-	/* Destroy right click menu */
-	GTK_WIDGET_DESTROY(lw->menu)
-	lw->menu = NULL;
+	EDVDevicesListWinUnmap(lw);
 
-	/* Begin destroying widgets */
-	GTK_WIDGET_DESTROY(lw->dev_icon_pm);
-	GTK_WIDGET_DESTROY(lw->dev_name_label);
+	clist = GTK_CLIST(lw->devices_clist);
+	gtk_clist_freeze(clist);
+	gtk_clist_clear(clist);
+	gtk_clist_thaw(clist);
 
-	GTK_WIDGET_DESTROY(lw->dev_device_path_label);
-	GTK_WIDGET_DESTROY(lw->dev_mount_path_label);
+	lw->freeze_count++;
 
-	GTK_WIDGET_DESTROY(lw->dev_mount_btn);
-	GTK_WIDGET_DESTROY(lw->dev_eject_btn);
-	GTK_WIDGET_DESTROY(lw->dev_properties_btn);
-
-	GTK_WIDGET_DESTROY(lw->dev_fsck_btn);
-	GTK_WIDGET_DESTROY(lw->dev_tools_btn);
-	GTK_WIDGET_DESTROY(lw->dev_format_btn);
+	gtk_widget_destroy(lw->menu);
 
 	PieChartDelete(lw->dev_pc);
 
-	GTK_WIDGET_DESTROY(lw->display_client);
-	GTK_WIDGET_DESTROY(lw->display_parent);
-
-	GTK_WIDGET_DESTROY(lw->find_entry);
-	GTK_WIDGET_DESTROY(lw->devices_clist);
-
-	GTK_WIDGET_DESTROY(lw->add_btn);
-	GTK_WIDGET_DESTROY(lw->edit_btn);
-	GTK_WIDGET_DESTROY(lw->remove_btn);
-	GTK_WIDGET_DESTROY(lw->up_btn);
-	GTK_WIDGET_DESTROY(lw->down_btn);
-	GTK_WIDGET_DESTROY(lw->close_btn);
-
-	GTK_WIDGET_DESTROY(lw->toplevel);
-
-	GTK_ACCEL_GROUP_UNREF(lw->accelgrp)
+	gtk_widget_destroy(lw->toplevel);
+	gtk_accel_group_unref(lw->accelgrp);
 
 #if 0
 	GDK_PIXMAP_UNREF(lw->mounted_pm);
@@ -3417,5 +3427,7 @@ void EDVDevicesListWinDelete(
 	GDK_BITMAP_UNREF(lw->not_mounted_mask);
 #endif
 
+	lw->freeze_count--;
+
 	g_free(lw);
 }
