@@ -16,11 +16,11 @@
 
 
 typedef struct _edv_devices_list_win_struct	edv_devices_list_win_struct;
-#define EDV_DEVICE_ICON_PICKER(p)	((edv_device_icon_picker_struct *)(p)) 
+#define EDV_DEVICE_ICON_PICKER(p)		((edv_device_icon_picker_struct *)(p)) 
 typedef struct _edv_device_edit_dlg_struct	edv_device_edit_dlg_struct;
-#define EDV_DEVICE_EDIT_DLG(p)	((edv_device_edit_dlg_struct *)(p))
+#define EDV_DEVICE_EDIT_DLG(p)			((edv_device_edit_dlg_struct *)(p))
 typedef struct _edv_device_icon_picker_struct	edv_device_icon_picker_struct;
-#define EDV_DEVICE_LISTWIN(p)	((edv_devices_list_win_struct *)(p))
+#define EDV_DEVICE_LISTWIN(p)			((edv_devices_list_win_struct *)(p))
 
 
 /*
@@ -48,22 +48,20 @@ struct _edv_device_icon_picker_struct {
 };
 
 /*
- *	Device Edit Window:
+ *	Device Edit Dialog:
  */
 struct _edv_device_edit_dlg_struct {
 
 	GtkWidget       *toplevel;
 	GtkAccelGroup   *accelgrp;
-	gboolean        processing;
 	gint            busy_count,
 			freeze_count;
+	gboolean	has_changes;
+	gchar		*title;
 	edv_core_struct *core;
 
 	edv_devices_list_win_struct	*list_win;
 
-	gboolean	has_changes;
-	gchar		*title;
-
 	GtkWidget       *main_vbox,
 			*name_entry,
 			*device_path_entry,
@@ -77,6 +75,7 @@ struct _edv_device_edit_dlg_struct {
 			*no_unmount_check,
 			*read_only_check,
 			*unlisted_check,
+			*no_scan_check,
 			*ok_btn,
 			*apply_btn,
 			*cancel_btn,
@@ -101,7 +100,6 @@ struct _edv_devices_list_win_struct {
 
 	GtkWidget       *toplevel;
 	GtkAccelGroup   *accelgrp;
-	gboolean        processing;
 	gint            busy_count,
 			freeze_count;
 	edv_core_struct	*core;
