@@ -27,7 +27,7 @@
 #include "edv_cfg_list.h"
 #include "config.h"
 
-#include "images/pdi_packagefile_32x32.xpm"
+#include "animations/packagefile_32x32.xpm"
 
 
 /*
@@ -36,16 +36,18 @@
  *	0	Success.
  *	-1	General error.
  *	-2	Invalid value.
- *	-3	Systems error; out of memory or out of disk space.
+ *	-3	Systems error, out of memory, or out of disk space.
  *	-4	User responded with "Cancel".
  *	-5	User responded with "No" or response was not available.
- *	-6	Call would cause reentry.
+ *	-6	An operation is already in progress.
  */
 
 
 /* Error Message */
-const gchar *EDVArchChkGetError(void);
-static void EDVArchChkCopyErrorMessage(const gchar *msg);
+const gchar *EDVArchChkGetError(edv_core_struct *core);
+static void EDVArchChkCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+);
 
 static void EDVArchChkMapProgressDialog(
 	const gchar *label, const gfloat progress,
@@ -59,28 +61,28 @@ static void EDVArchChkMapProgressDialogUnknown(
 /* Delete Object From Archive */
 static gint EDVArchChkARJ(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchChkLHA(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchChkRAR(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchChkTar(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all,
@@ -90,7 +92,7 @@ static gint EDVArchChkTar(
 );
 static gint EDVArchChkZip(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
@@ -98,7 +100,7 @@ static gint EDVArchChkZip(
 );
 gint EDVArchChk(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	gboolean const show_progress, const gboolean interactive,
@@ -106,12 +108,6 @@ gint EDVArchChk(
 );
 
 
-static const gchar *last_error = NULL;
-static gchar last_error_buf[256];
-
-
-#define ISCR(c)		(((c) == '\n') || ((c) == '\r'))
-
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -126,29 +122,36 @@ static gchar last_error_buf[256];
 #define UNLINK(p)	(((p) != NULL) ? (gint)unlink((const char *)(p)) : -1)
 #define INTERRUPT(i)	(((i) > 0) ? (gint)kill((int)(i), SIGINT) : -1)
 
+#define ISCR(c)		(((c) == '\n') || ((c) == '\r'))
+
 
 /*
  *	Returns the last error message as a statically allocated
  *	string or NULL if there was no previous error.
  */
-const gchar *EDVArchChkGetError(void)
+const gchar *EDVArchChkGetError(edv_core_struct *core)
 {
-	return(last_error);
+	return((core != NULL) ? core->archive_last_error : NULL);
 }
 
 
 /*
- *	Coppies the error message specified by msg to the last error
- *	message buffer and sets last_error to point to that buffer.
+ *	Coppies the error message to the core's archive_last_error_buf
+ *	and sets the core's archive_last_error to point to it.
  */
-static void EDVArchChkCopyErrorMessage(const gchar *msg)
+static void EDVArchChkCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+)
 {
-	if(msg == NULL)
+	if(core == NULL)
 	    return;
 
-	strncpy(last_error_buf, msg, sizeof(last_error_buf));
-	last_error_buf[sizeof(last_error_buf) - 1] = '\0';
-	last_error = last_error_buf;
+	core->archive_last_error = NULL;
+
+	g_free(core->archive_last_error_buf);
+	core->archive_last_error_buf = STRDUP(msg);
+
+	core->archive_last_error = core->archive_last_error_buf;
 }
 
 
@@ -187,7 +190,7 @@ static void EDVArchChkMapProgressDialog(
 	ProgressDialogMap(
 	    "Checking",
 	    label,
-	    (const guint8 **)pdi_packagefile_32x32_xpm,
+	    (const guint8 **)packagefile_32x32_xpm,
 	    "Stop"
 	);
 	ProgressDialogUpdate(
@@ -236,7 +239,7 @@ static void EDVArchChkMapProgressDialogUnknown(
 	ProgressDialogMap(
 	    "Checking",
 	    label,
-	    (const guint8 **)pdi_packagefile_32x32_xpm,
+	    (const guint8 **)packagefile_32x32_xpm,
 	    "Stop"
 	);
 	ProgressDialogUpdateUnknown(
@@ -255,7 +258,7 @@ static void EDVArchChkMapProgressDialogUnknown(
  */
 static gint EDVArchChkARJ(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
@@ -270,14 +273,12 @@ static gint EDVArchChkARJ(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(cmd);				\
+ g_free(stdout_path);			\
+ g_free(stderr_path);			\
+					\
+ return(_v_);				\
 }
 
 	/* Format the check archive command */
@@ -285,13 +286,12 @@ static gint EDVArchChkARJ(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" t -i -y \"%s\"",
-		prog_arj, arch_obj
+		prog_arj, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the check command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -306,9 +306,8 @@ static gint EDVArchChkARJ(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the check archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -411,7 +410,7 @@ static gint EDVArchChkARJ(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Checking:\n\
@@ -447,8 +446,8 @@ In:\n\
 			while(ISBLANK(*s))
 			    s++;
 
-			if(last_error == NULL)
-			    EDVArchChkCopyErrorMessage(s);
+			if(core->archive_last_error == NULL)
+			    EDVArchChkCopyErrorMessage(core, s);
 			if((status == 0) || (status == -4))
 			    status = -1;
 
@@ -494,7 +493,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchChkCopyErrorMessage(s);
+		    EDVArchChkCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -508,9 +507,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -518,7 +516,7 @@ In:\n\
  */
 static gint EDVArchChkLHA(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
@@ -533,14 +531,12 @@ static gint EDVArchChkLHA(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(cmd);				\
+ g_free(stdout_path);			\
+ g_free(stderr_path);			\
+					\
+ return(_v_);				\
 }
 
 	/* Format the check archive command */
@@ -548,13 +544,12 @@ static gint EDVArchChkLHA(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" -tq1 \"%s\"",
-		prog_lha, arch_obj
+		prog_lha, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the check command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -569,9 +564,8 @@ static gint EDVArchChkLHA(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the check archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -662,7 +656,7 @@ static gint EDVArchChkLHA(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Checking:\n\
@@ -723,7 +717,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchChkCopyErrorMessage(s);
+		    EDVArchChkCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -737,9 +731,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -747,7 +740,7 @@ In:\n\
  */
 static gint EDVArchChkRAR(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
@@ -762,14 +755,12 @@ static gint EDVArchChkRAR(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(cmd);				\
+ g_free(stdout_path);			\
+ g_free(stderr_path);			\
+					\
+ return(_v_);				\
 }
 
 	/* Format the check archive command */
@@ -777,13 +768,12 @@ static gint EDVArchChkRAR(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" t -y \"%s\"",
-		prog_rar, arch_obj
+		prog_rar, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the check command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -798,9 +788,8 @@ static gint EDVArchChkRAR(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the check archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -904,7 +893,7 @@ static gint EDVArchChkRAR(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Checking:\n\
@@ -965,7 +954,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchChkCopyErrorMessage(s);
+		    EDVArchChkCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -979,9 +968,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -989,7 +977,7 @@ In:\n\
  */
 static gint EDVArchChkTar(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all,
@@ -1011,19 +999,17 @@ static gint EDVArchChkTar(
 	    core->cfg_list, EDV_CFG_PARM_PROG_BUNZIP2
 	);
 	FILE *fp;
-	gint status = 0, p;
+	gint status, p;
 	gchar	*cmd = NULL,
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(cmd);				\
+ g_free(stdout_path);			\
+ g_free(stderr_path);			\
+					\
+ return(_v_);				\
 }
 
 	/* Format the check archive command */
@@ -1032,28 +1018,27 @@ static gint EDVArchChkTar(
 	    if(is_compress_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_uncompress, arch_obj
+		    prog_tar, prog_uncompress, arch_path
 		);
 	    else if(is_gzip_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_gunzip, arch_obj
+		    prog_tar, prog_gunzip, arch_path
 		);
 	    else if(is_bzip2_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_bunzip2, arch_obj
+		    prog_tar, prog_bunzip2, arch_path
 		);
 	    else
 		cmd = g_strdup_printf(
 		    "\"%s\" -t -f \"%s\"",
-		    prog_tar, arch_obj
+		    prog_tar, arch_path
 		);
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the check command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -1068,9 +1053,8 @@ static gint EDVArchChkTar(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the check archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -1078,6 +1062,8 @@ static gint EDVArchChkTar(
 	    cmd = NULL;
 	}
 
+	status = 0;
+
 	/* Open the output file for reading */
 	fp = fopen((const char *)stdout_path, "rb");
 	if(fp != NULL)
@@ -1153,7 +1139,7 @@ static gint EDVArchChkTar(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Checking:\n\
@@ -1214,7 +1200,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchChkCopyErrorMessage(s);
+		    EDVArchChkCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -1228,9 +1214,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -1238,7 +1223,7 @@ In:\n\
  */
 static gint EDVArchChkZip(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
@@ -1259,23 +1244,40 @@ static gint EDVArchChkZip(
 	guint8 *read_buf;
 	edv_object_type type;
 
-	if(stat(arch_obj, &stat_buf))
+	if(stat((const char *)arch_path, &stat_buf))
 	{
 	    const gint error_code = (int)errno;
-	    EDVArchChkCopyErrorMessage(g_strerror(error_code));
+	    gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+		g_strerror(error_code), arch_path
+	    );
+	    EDVArchChkCopyErrorMessage(core, msg);
+	    g_free(msg);
 	    return(-1);
 	}
 
 	/* Open the PKZip archive for reading */
-	archive = zip_open(arch_obj, 0, &zip_error_code);
+	archive = zip_open(arch_path, 0, &zip_error_code);
 	if(archive == NULL)
 	{
 	    const gint sys_error_code = (gint)errno;
+	    gchar *msg, err_msg[1024];
 	    zip_error_to_str(
-		last_error_buf, sizeof(last_error_buf),
+		err_msg, sizeof(err_msg),
 		zip_error_code, sys_error_code
 	    );
-	    last_error = last_error_buf;
+	    msg = g_strdup_printf(
+"Unable to open the PKZip Archive for reading:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+		arch_path, err_msg
+	    );
+	    EDVArchChkCopyErrorMessage(core, msg);
+	    g_free(msg);
 	    return(-1);
 	}
 
@@ -1297,7 +1299,14 @@ static gint EDVArchChkZip(
 	{
 	    if(zip_stat_index(archive, i, 0, &zip_stat_buf))
 	    {
-		EDVArchChkCopyErrorMessage(zip_strerror(archive));
+		gchar *msg = g_strdup_printf(
+"Unable to obtain the statistics for the PKZip Archive object #%i.\n\
+\n\
+%s.",
+		    i + 1, zip_strerror(archive)
+		);
+		EDVArchChkCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 		break;
 	    }
@@ -1332,7 +1341,7 @@ static gint EDVArchChkZip(
 		    clean_name, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 		),
 			*p2 = EDVShortenPath(
-		    arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+		    arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 		),
 			*msg = g_strdup_printf(
 "Checking:\n\
@@ -1366,18 +1375,29 @@ In:\n\
 	    zip_fp = zip_fopen_index(archive, i, 0);
 	    if(zip_fp == NULL)
 	    {
-		EDVArchChkCopyErrorMessage(zip_strerror(archive));
+		gchar *msg = g_strdup_printf(
+"Unable to open the PKZip Archive object #%i for reading.\n\
+\n\
+%s.",
+		    i + 1, zip_strerror(archive)
+		);
+		EDVArchChkCopyErrorMessage(core, msg);
+		g_free(msg);
 		g_free(clean_name);
 		status = -1;
 		break;
 	    }
 
 	    /* Allocate the read buffer */
-	    read_buf_len = MAX((int)stat_buf.st_blksize, 1);
+#if defined(_WIN32)
+	    read_buf_len = 1024;
+#else
+	    read_buf_len = MAX((gint)stat_buf.st_blksize, 1);
+#endif
 	    read_buf = (guint8 *)g_malloc(read_buf_len * sizeof(guint8));
 	    if(read_buf == NULL)
 	    {
-		last_error = "Memory allocation error";
+		core->archive_last_error = "Memory allocation error.";
 		g_free(clean_name);
 		zip_fclose(zip_fp);
 		status = -3;
@@ -1394,7 +1414,14 @@ In:\n\
 		{
 		    if(bytes_read < 0)
 		    {
-			EDVArchChkCopyErrorMessage(zip_file_strerror(zip_fp));
+			gchar *msg = g_strdup_printf(
+"Unable to read from the PKZip Archive object #%i.\n\
+\n\
+%s.",
+			    i + 1, zip_file_strerror(zip_fp)
+			);
+			EDVArchChkCopyErrorMessage(core, msg);
+			g_free(msg);
 			status = -1;
 		    }
 		    break; 
@@ -1436,7 +1463,16 @@ In:\n\
 	{
 	    if((status == 0) || (status == -4))
 	    {
-		EDVArchChkCopyErrorMessage(zip_strerror(archive));
+		gchar *msg = g_strdup_printf(
+"Unable to close the PKZip Archive:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+		    arch_path, zip_strerror(archive)
+		);
+		EDVArchChkCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 	}
@@ -1465,14 +1501,12 @@ In:\n\
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(cmd);				\
+ g_free(stdout_path);			\
+ g_free(stderr_path);			\
+					\
+ return(_v_);				\
 }
 
 	/* Format the check archive command */
@@ -1480,13 +1514,12 @@ In:\n\
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" -T -v \"%s\"",
-		prog_zip, arch_obj
+		prog_zip, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the check command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -1501,9 +1534,8 @@ In:\n\
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the check archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the check archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -1605,7 +1637,7 @@ In:\n\
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Checking:\n\
@@ -1666,7 +1698,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchChkCopyErrorMessage(s);
+		    EDVArchChkCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -1680,66 +1712,65 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 #endif	/* !HAVE_LIBZIP */
 }
 
 /*
  *	Checks the archive.
  *
- *	The arch_obj specifies the archive.
+ *	The arch_path specifies the archive.
  */
 gint EDVArchChk(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 )
 {
-	static gboolean reenterent = FALSE;
 /*	const gulong time_start = (gulong)time(NULL); */
 	gint status;
 	const gchar *arch_name;
 
-#define DO_FREE_LOCALS		{	\
+#define CLEANUP_RETURN(_v_)	{	\
+					\
+ return(_v_);				\
 }
 
-	if(reenterent)
+	if(ProgressDialogIsQuery())
 	{
-	    last_error =
-"An operation is already in progress, please try again later";
+	    EDVArchChkCopyErrorMessage(
+		core,
+"An operation is already in progress, please try again later."
+	    );
 	    return(-6);
 	}
-	else
-	{
-	    reenterent = TRUE;
-	}
 
-	last_error = NULL;
+	/* Clear the last error message */
+	EDVArchChkCopyErrorMessage(core, NULL);
 
-	if((core == NULL) || STRISEMPTY(arch_obj) ||
+	if((core == NULL) || STRISEMPTY(arch_path) ||
 	   (yes_to_all == NULL)
 	)
 	{
-	    reenterent = FALSE;
+	    EDVArchChkCopyErrorMessage(core, "Invalid value.");
 	    return(-2);
 	}
 
 	/* Begin deleting the source object from the archive object
-	 * arch_obj. The deleting method will be determined by taking
+	 * arch_path. The deleting method will be determined by taking
 	 * the extension of the archive object's name
 	 */
 
 	/* ARJ Archive */
-	arch_name = g_basename(arch_obj);
+	arch_name = g_basename(arch_path);
 	if(EDVIsExtension(arch_name, ".arj"))
 	{
 	    status = EDVArchChkARJ(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1747,7 +1778,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".lha"))
 	{
 	    status = EDVArchChkLHA(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1755,7 +1786,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".rar"))
 	{
 	    status = EDVArchChkRAR(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1763,7 +1794,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".tar.Z"))
 	{
 	    status = EDVArchChkTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		TRUE,		/* Is compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1774,7 +1805,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".tgz .tar.gz"))
 	{
 	    status = EDVArchChkTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		TRUE,		/* Is gzip compressed */
@@ -1785,7 +1816,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".tar.bz2"))
 	{
 	    status = EDVArchChkTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1796,7 +1827,7 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".tar"))
 	{
 	    status = EDVArchChkTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1807,18 +1838,16 @@ gint EDVArchChk(
 	else if(EDVIsExtension(arch_name, ".xpi .zip"))
 	{
 	    status = EDVArchChkZip(
-		core, arch_obj, password,
+		core, arch_path, password,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
 	else
 	{
-	    last_error = "Unsupported archive format";
+	    core->archive_last_error = "Unsupported archive format.";
 	    status = -2;
 	}
 
-	DO_FREE_LOCALS
-	reenterent = FALSE;
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
