@@ -25,7 +25,7 @@
 #include "edv_cfg_list.h"
 #include "config.h"
 
-#include "images/pdi_packagefile_32x32.xpm"
+#include "animations/packagefile_32x32.xpm"
 
 
 /*
@@ -34,16 +34,18 @@
  *	0	Success.
  *	-1	General error.
  *	-2	Invalid value.
- *	-3	Systems error; out of memory or out of disk space.
+ *	-3	Systems error, out of memory, or out of disk space.
  *	-4	User responded with "Cancel".
  *	-5	User responded with "No" or response was not available.
- *	-6	Call would cause reentry.
+ *	-6	An operation is already in progress.
  */
 
 
 /* Error Message */
-const gchar *EDVArchFixGetError(void);
-static void EDVArchFixCopyErrorMessage(const gchar *msg);
+const gchar *EDVArchFixGetError(edv_core_struct *core);
+static void EDVArchFixCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+);
 
 static void EDVArchFixMapProgressDialog(
 	const gchar *label, const gfloat progress,
@@ -57,28 +59,28 @@ static void EDVArchFixMapProgressDialogUnknown(
 /* Delete Object From Archive */
 static gint EDVArchFixARJ(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchFixLHA(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchFixRAR(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
 static gint EDVArchFixTar(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all,
@@ -88,7 +90,7 @@ static gint EDVArchFixTar(
 );
 static gint EDVArchFixZip(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
@@ -96,7 +98,7 @@ static gint EDVArchFixZip(
 );
 gint EDVArchFix(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	gboolean const show_progress, const gboolean interactive,
@@ -104,12 +106,6 @@ gint EDVArchFix(
 );
 
 
-static const gchar *last_error = NULL;
-static gchar last_error_buf[256];
-
-
-#define ISCR(c)		(((c) == '\n') || ((c) == '\r'))
-
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -124,29 +120,36 @@ static gchar last_error_buf[256];
 #define UNLINK(p)	(((p) != NULL) ? (gint)unlink((const char *)(p)) : -1)
 #define INTERRUPT(i)	(((i) > 0) ? (gint)kill((int)(i), SIGINT) : -1)
 
+#define ISCR(c)		(((c) == '\n') || ((c) == '\r'))
+
 
 /*
  *	Returns the last error message as a statically allocated
  *	string or NULL if there was no previous error.
  */
-const gchar *EDVArchFixGetError(void)
+const gchar *EDVArchFixGetError(edv_core_struct *core)
 {
-	return(last_error);
+	return((core != NULL) ? core->archive_last_error : NULL);
 }
 
 
 /*
- *	Coppies the error message specified by msg to the last error
- *	message buffer and sets last_error to point to that buffer.
+ *	Coppies the error message to the core's archive_last_error_buf
+ *	and sets the core's archive_last_error to point to it.
  */
-static void EDVArchFixCopyErrorMessage(const gchar *msg)
+static void EDVArchFixCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+)
 {
-	if(msg == NULL)
+	if(core == NULL)
 	    return;
 
-	strncpy(last_error_buf, msg, sizeof(last_error_buf));
-	last_error_buf[sizeof(last_error_buf) - 1] = '\0';
-	last_error = last_error_buf;
+	core->archive_last_error = NULL;
+
+	g_free(core->archive_last_error_buf);
+	core->archive_last_error_buf = STRDUP(msg);
+
+	core->archive_last_error = core->archive_last_error_buf;
 }
 
 
@@ -185,7 +188,7 @@ static void EDVArchFixMapProgressDialog(
 	ProgressDialogMap(
 	    "Fixing",
 	    label,
-	    (const guint8 **)pdi_packagefile_32x32_xpm,
+	    (const guint8 **)packagefile_32x32_xpm,
 	    "Stop"
 	);
 	ProgressDialogUpdate(
@@ -234,7 +237,7 @@ static void EDVArchFixMapProgressDialogUnknown(
 	ProgressDialogMap(
 	    "Fixing",
 	    label,
-	    (const guint8 **)pdi_packagefile_32x32_xpm,
+	    (const guint8 **)packagefile_32x32_xpm,
 	    "Stop"
 	);
 	ProgressDialogUpdateUnknown(
@@ -253,14 +256,14 @@ static void EDVArchFixMapProgressDialogUnknown(
  */
 static gint EDVArchFixARJ(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 )
 {
 #if 1
-	last_error = "There is no support for fixing ARJ archives";
+	core->archive_last_error = "There is no support for fixing ARJ archives.";
 	return(-2);
 #else
 	const gchar *prog_arj = CFGItemListGetValueS(
@@ -272,14 +275,12 @@ static gint EDVArchFixARJ(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{		\
+ g_free(cmd);					\
+ g_free(stdout_path);				\
+ g_free(stderr_path);				\
+						\
+ return(_v_);					\
 }
 
 	/* Format the fix archive command */
@@ -287,13 +288,12 @@ static gint EDVArchFixARJ(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" ??? \"%s\"",
-		prog_arj, arch_obj
+		prog_arj, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the fix command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the fix command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -308,9 +308,8 @@ static gint EDVArchFixARJ(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the fix archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the fix archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -413,7 +412,7 @@ static gint EDVArchFixARJ(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Fixing:\n\
@@ -449,8 +448,8 @@ In:\n\
 			while(ISBLANK(*s))
 			    s++;
 
-			if(last_error == NULL)
-			    EDVArchFixCopyErrorMessage(s);
+			if(core->archive_last_error == NULL)
+			    EDVArchFixCopyErrorMessage(core, s);
 			if((status == 0) || (status == -4))
 			    status = -1;
 
@@ -496,7 +495,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchFixCopyErrorMessage(s);
+		    EDVArchFixCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -510,9 +509,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 #endif
 }
 
@@ -521,14 +519,14 @@ In:\n\
  */
 static gint EDVArchFixLHA(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 )
 {
 #if 1
-	last_error = "There is no support for fixing LHA archives";
+	core->archive_last_error = "There is no support for fixing LHA archives.";
 	return(-1);
 #else
 	const gchar *prog_lha = CFGItemListGetValueS(
@@ -540,14 +538,12 @@ static gint EDVArchFixLHA(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{		\
+ g_free(cmd);					\
+ g_free(stdout_path);				\
+ g_free(stderr_path);				\
+						\
+ return(_v_);					\
 }
 
 	/* Format the fix archive command */
@@ -555,13 +551,12 @@ static gint EDVArchFixLHA(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" ??? \"%s\"",
-		prog_lha, arch_obj
+		prog_lha, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the fix command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the fix command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -576,9 +571,8 @@ static gint EDVArchFixLHA(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the fix archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the fix archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -669,7 +663,7 @@ static gint EDVArchFixLHA(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Fixing:\n\
@@ -730,7 +724,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchFixCopyErrorMessage(s);
+		    EDVArchFixCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -744,9 +738,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 #endif
 }
 
@@ -755,7 +748,7 @@ In:\n\
  */
 static gint EDVArchFixRAR(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
@@ -772,22 +765,22 @@ static gint EDVArchFixRAR(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	{	\
- EDVSetCWD(cwd);		\
- g_free(cwd);			\
- cwd = NULL;			\
- g_free(cmd);			\
- cmd = NULL;			\
- g_free(stdout_path);		\
- stdout_path = NULL;		\
- g_free(stderr_path);		\
- stderr_path = NULL;		\
+#define CLEANUP_RETURN(_v_)	{		\
+ if(cwd != NULL) {				\
+  EDVSetCWD(cwd);				\
+  g_free(cwd);					\
+ }						\
+ g_free(cmd);					\
+ g_free(stdout_path);				\
+ g_free(stderr_path);				\
+						\
+ return(_v_);					\
 }
 
 	/* Set the archive's parent directory as the current working
 	 * directory
 	 */
-	parent = g_dirname(arch_obj);
+	parent = g_dirname(arch_path);
 	EDVSetCWD(parent);
 	g_free(parent);
 
@@ -796,13 +789,12 @@ static gint EDVArchFixRAR(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" r -y \"%s\"",
-		prog_rar, arch_obj
+		prog_rar, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the fix command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the fix command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -817,9 +809,8 @@ static gint EDVArchFixRAR(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the fix archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the fix archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -903,7 +894,7 @@ static gint EDVArchFixRAR(
 				op_msg, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Fixing:\n\
@@ -964,7 +955,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchFixCopyErrorMessage(s);
+		    EDVArchFixCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -984,33 +975,33 @@ In:\n\
 	if(status == 0)
 	{
 	    struct stat stat_buf;
-	    gchar	*parent = g_dirname(arch_obj),
+	    gchar	*parent = g_dirname(arch_path),
 			*rebuilt_arch_path = g_strconcat(
 			    parent,
 			    G_DIR_SEPARATOR_S,
 			    "rebuilt.",
-			    g_basename(arch_obj),
+			    g_basename(arch_path),
 			    NULL
 			);
 	    if((rebuilt_arch_path != NULL) ?
 		(stat(rebuilt_arch_path, &stat_buf) == 0) : FALSE
 	    )
 	    {
-		if(stat(arch_obj, &stat_buf) == 0)
+		if(stat(arch_path, &stat_buf) == 0)
 		{
 		    /* Remove the original archive and rename the
 		     * rebuilt archive to that of the original
 		     * archive
 		     */
-		    UNLINK(arch_obj);
-		    rename(rebuilt_arch_path, arch_obj);
+		    UNLINK(arch_path);
+		    rename(rebuilt_arch_path, arch_path);
 
 		    chmod(
-			(const char *)arch_obj,
+			(const char *)arch_path,
 			stat_buf.st_mode
 		    );
 		    chown(
-			(const char *)arch_obj,
+			(const char *)arch_path,
 			stat_buf.st_uid,
 			stat_buf.st_gid
 		    );
@@ -1021,9 +1012,8 @@ In:\n\
 	}
 
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -1031,7 +1021,7 @@ In:\n\
  */
 static gint EDVArchFixTar(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all,
@@ -1041,7 +1031,7 @@ static gint EDVArchFixTar(
 )
 {
 #if 1
-	last_error = "There is no support for fixing TAR archives";
+	core->archive_last_error = "There is no support for fixing TAR archives.";
 	return(-1);
 #else
 	const gchar *prog_tar = CFGItemListGetValueS(
@@ -1062,14 +1052,12 @@ static gint EDVArchFixTar(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{		\
+ g_free(cmd);					\
+ g_free(stdout_path);				\
+ g_free(stderr_path);				\
+						\
+ return(_v_);					\
 }
 
 	/* Format the fix archive command */
@@ -1078,28 +1066,27 @@ static gint EDVArchFixTar(
 	    if(is_compress_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_uncompress, arch_obj
+		    prog_tar, prog_uncompress, arch_path
 		);
 	    else if(is_gzip_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_gunzip, arch_obj
+		    prog_tar, prog_gunzip, arch_path
 		);
 	    else if(is_bzip2_compressed)
 		cmd = g_strdup_printf(
 		    "\"%s\" \"--use-compress-program=%s\" -t -f \"%s\"",
-		    prog_tar, prog_bunzip2, arch_obj
+		    prog_tar, prog_bunzip2, arch_path
 		);
 	    else
 		cmd = g_strdup_printf(
 		    "\"%s\" -t -f \"%s\"",
-		    prog_tar, arch_obj
+		    prog_tar, arch_path
 		);
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the fix command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the fix command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -1114,9 +1101,8 @@ static gint EDVArchFixTar(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the fix archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the fix archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -1199,7 +1185,7 @@ static gint EDVArchFixTar(
 				chk_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Fixing:\n\
@@ -1260,7 +1246,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchFixCopyErrorMessage(s);
+		    EDVArchFixCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -1274,9 +1260,8 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 #endif
 }
 
@@ -1285,7 +1270,7 @@ In:\n\
  */
 static gint EDVArchFixZip(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
@@ -1301,14 +1286,12 @@ static gint EDVArchFixZip(
 		*stdout_path = NULL,
 		*stderr_path = NULL;
 
-#define DO_FREE_LOCALS	\
-{			\
- g_free(cmd);		\
- cmd = NULL;		\
- g_free(stdout_path);	\
- stdout_path = NULL;	\
- g_free(stderr_path);	\
- stderr_path = NULL;	\
+#define CLEANUP_RETURN(_v_)	{		\
+ g_free(cmd);					\
+ g_free(stdout_path);				\
+ g_free(stderr_path);				\
+						\
+ return(_v_);					\
 }
 
 	/* Format the fix archive command */
@@ -1316,13 +1299,12 @@ static gint EDVArchFixZip(
 	{
 	    cmd = g_strdup_printf(
 		"\"%s\" -FF -v \"%s\"",
-		prog_zip, arch_obj
+		prog_zip, arch_path
 	    );
 	    if(cmd == NULL)
 	    {
-		last_error = "Unable to generate the fix command";
-		DO_FREE_LOCALS
-		return(-2);
+		core->archive_last_error = "Unable to generate the fix command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate the output file paths */
@@ -1337,9 +1319,8 @@ static gint EDVArchFixZip(
 	    );
 	    if(p <= 0)
 	    {
-		last_error = "Unable to execute the fix archive command";
-		DO_FREE_LOCALS
-		return(-1);
+		core->archive_last_error = "Unable to execute the fix archive command.";
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Delete the command */
@@ -1423,7 +1404,7 @@ static gint EDVArchFixZip(
 				op_msg, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*p2 = EDVShortenPath(
-				arch_obj, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
+				arch_path, EDV_DEF_PROGRESS_BAR_PATH_DISPLAY_MAX
 			    ),
 					*msg = g_strdup_printf(
 "Fixing:\n\
@@ -1484,7 +1465,7 @@ In:\n\
 
 		if(!STRISEMPTY(s))
 		{
-		    EDVArchFixCopyErrorMessage(s);
+		    EDVArchFixCopyErrorMessage(core, s);
 		    if((status == 0) || (status == -4))
 			status = -1;
 		    break;
@@ -1498,65 +1479,64 @@ In:\n\
 	UNLINK(stdout_path);
 	UNLINK(stderr_path);
 
-	DO_FREE_LOCALS
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 /*
  *	Fixes the archive.
  *
- *	The arch_obj specifies the archive.
+ *	The arch_path specifies the archive.
  */
 gint EDVArchFix(
 	edv_core_struct *core,
-	const gchar *arch_obj,
+	const gchar *arch_path,
 	const gchar *password,
 	GtkWidget *toplevel,
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 )
 {
-	static gboolean reenterent = FALSE;
 	const gulong time_start = (gulong)time(NULL);
 	gint status;
 	const gchar *arch_name;
 
-#define DO_FREE_LOCALS		{	\
+#define CLEANUP_RETURN(_v_)	{		\
+						\
+ return(_v_);					\
 }
 
-	if(reenterent)
+	if(ProgressDialogIsQuery())
 	{
-	    last_error =
-"An operation is already in progress, please try again later";
+	    EDVArchFixCopyErrorMessage(
+		core,
+"An operation is already in progress, please try again later."
+	    );
 	    return(-6);
 	}
-	else
-	{
-	    reenterent = TRUE;
-	}
 
-	last_error = NULL;
+	/* Clear the last error message */
+	EDVArchFixCopyErrorMessage(core, NULL);
 
-	if((core == NULL) || STRISEMPTY(arch_obj) ||
+	if((core == NULL) || STRISEMPTY(arch_path) ||
 	   (yes_to_all == NULL)
 	)
 	{
-	    reenterent = FALSE;
+	    EDVArchFixCopyErrorMessage(core, "Invalid value.");
 	    return(-2);
 	}
 
 	/* Begin deleting the source object from the archive object
-	 * arch_obj. The deleting method will be determined by taking
+	 * arch_path. The deleting method will be determined by taking
 	 * the extension of the archive object's name
 	 */
 
 	/* ARJ Archive */
-	arch_name = g_basename(arch_obj);
+	arch_name = g_basename(arch_path);
 	if(EDVIsExtension(arch_name, ".arj"))
 	{
 	    status = EDVArchFixARJ(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1564,7 +1544,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".lha"))
 	{
 	    status = EDVArchFixLHA(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1572,7 +1552,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".rar"))
 	{
 	    status = EDVArchFixRAR(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
@@ -1580,7 +1560,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".tar.Z"))
 	{
 	    status = EDVArchFixTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		TRUE,		/* Is compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1591,7 +1571,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".tgz .tar.gz"))
 	{
 	    status = EDVArchFixTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		TRUE,		/* Is gzip compressed */
@@ -1602,7 +1582,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".tar.bz2"))
 	{
 	    status = EDVArchFixTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1613,7 +1593,7 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".tar"))
 	{
 	    status = EDVArchFixTar(
-		core, arch_obj,
+		core, arch_path,
 		toplevel, show_progress, interactive, yes_to_all,
 		FALSE,		/* Not compress compressed */
 		FALSE,		/* Not gzip compressed */
@@ -1624,13 +1604,13 @@ gint EDVArchFix(
 	else if(EDVIsExtension(arch_name, ".xpi .zip"))
 	{
 	    status = EDVArchFixZip(
-		core, arch_obj, password,
+		core, arch_path, password,
 		toplevel, show_progress, interactive, yes_to_all
 	    );
 	}
 	else
 	{
-	    last_error = "Unsupported archive format";
+	    core->archive_last_error = "Unsupported archive format.";
 	    status = -2;
 	}
 
@@ -1643,9 +1623,9 @@ gint EDVArchFix(
 		EDV_HISTORY_ARCHIVE_FIX,
 		time_start, time_end,
 		status,
-		arch_obj,		/* Source */
+		arch_path,		/* Source */
 		NULL,			/* Target */
-		last_error		/* Comment */
+		core->archive_last_error		/* Comment */
 	    );
 	}
 	else
@@ -1656,9 +1636,9 @@ gint EDVArchFix(
 		EDV_HISTORY_ARCHIVE_FIX,
 		time_start, time_end,
 		status,
-		arch_obj,		/* Source */
+		arch_path,		/* Source */
 		NULL,			/* Target */
-		last_error		/* Comment */
+		core->archive_last_error		/* Comment */
 	    );
 	}
 
@@ -1668,8 +1648,6 @@ gint EDVArchFix(
 	 */
 	sync();
 
-	DO_FREE_LOCALS
-	reenterent = FALSE;
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
