@@ -13,7 +13,7 @@
 
 
 typedef struct _edv_archive_object_struct	edv_archive_object_struct;
-#define EDV_ARCHIVE_OBJECT(p)	((edv_archive_object_struct *)(p))
+#define EDV_ARCHIVE_OBJECT(p)		((edv_archive_object_struct *)(p))
 
 
 /*
@@ -36,7 +36,7 @@ struct _edv_archive_object_struct {
 			compressed_size;	/* In bytes */
 
 	/* Link */
-	gchar		*link_value;	/* Only for EDV_OBJECT_TYPE_LINK */
+	gchar		*link_target;	/* Only for EDV_OBJECT_TYPE_LINK */
 
 	/* Permissions */
 	edv_permission_flags	permissions;
@@ -74,7 +74,7 @@ struct _edv_archive_object_struct {
 
 extern edv_archive_object_struct *EDVArchObjectNew(void);
 extern edv_archive_object_struct *EDVArchObjectCopy(
-	const edv_archive_object_struct *obj
+	edv_archive_object_struct *obj
 );
 extern void EDVArchObjectDelete(edv_archive_object_struct *obj);
 
