@@ -6,7 +6,6 @@
 #include <gdk/gdkkeysyms.h>
 
 #include "../include/string.h"
-#include "../include/disk.h"
 #include "../include/prochandle.h"
 
 #include "guiutils.h"
@@ -129,6 +128,19 @@ void EDVLocBarIconDragDataGet(
 	gpointer data
 );
 
+/* File Selector Callbacks */
+void EDVFileSelectorObjectCreatedCB(
+	const gchar *path, gpointer data
+);
+void EDVFileSelectorObjectModifiedCB(
+	const gchar *old_path,
+	const gchar *new_path,
+	gpointer data
+);
+void EDVFileSelectorObjectDeletedCB(
+	const gchar *path, gpointer data
+);
+
 
 /* Write Protect Changed Emitters */
 void EDVWriteProtectChangedEmit(
@@ -220,7 +232,7 @@ void EDVDeviceRemovedEmit(
 /* History Event Emitters */
 void EDVHistoryAddedEmit(
 	edv_core_struct *core,
-	gint h_num, edv_history_struct *h_ptr
+	gint h_num, edv_history_struct *h
 );
 void EDVHistoryRemovedEmit(
 	edv_core_struct *core,
@@ -228,6 +240,15 @@ void EDVHistoryRemovedEmit(
 );
 
 
+/*
+ *	InterPS Idle Data:
+ */
+struct _interps_idle_data_struct {
+	edv_core_struct	*core;
+	gchar		**cmd_list;
+};
+
+
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -240,15 +261,6 @@ void EDVHistoryRemovedEmit(
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
-/*
- *	InterPS Idle Data:
- */
-struct _interps_idle_data_struct {
-	edv_core_struct	*core;
-	gchar		**cmd_list;
-};
-
-
 /*
  *	Signal callback.
  *
@@ -340,16 +352,15 @@ static gint EDVInterPSCommandsIdleCB(gpointer data)
 gint EDVManageTimeoutCB(gpointer data)
 {
 	gboolean	conserve_memory,
-			has_changes,
-			still_processing,
 			all_windows_unmapped;
 	gint i;
 	const cfg_item_struct *cfg_list;
 	edv_browser_struct *browser;
 	edv_imbr_struct *imbr;
 	edv_archiver_struct *archiver;
-	edv_archive_info_dlg_struct *arch_info;
+	edv_archive_info_dlg_struct *arch_info_dlg;
 	edv_recbin_struct *recbin;
+	edv_obj_op_dlg_struct *obj_op_dlg;
 	edv_find_win_struct *findwin;
 	edv_propdlg_struct *propdlg;
 	edv_history_win_struct *histwin;
@@ -363,15 +374,16 @@ gint EDVManageTimeoutCB(gpointer data)
 	    return(FALSE);
 
 /* Removes the timeout ID and sets it to 0 */
-#define DO_REMOVE_TOID(_id_)		\
-{ if(*(_id_) > 0) {			\
-  gtk_timeout_remove(*(_id_));		\
-  *(_id_) = 0;				\
-} }
+#define DO_REMOVE_TOID(_id_)	{		\
+ if(*(_id_) > 0) {				\
+  gtk_timeout_remove(*(_id_));			\
+  *(_id_) = 0;					\
+ }						\
+}
 
 /* Removes all timeout callbacks and breaks out of the GTK main loop */
 #define DO_REMOVE_TIMEOUTS_AND_BREAK_MAIN {	\
- DO_REMOVE_TOID(&core->manage_toid);	\
+ DO_REMOVE_TOID(&core->manage_toid);		\
 						\
  gtk_main_quit();				\
 }
@@ -381,7 +393,7 @@ gint EDVManageTimeoutCB(gpointer data)
  * been set for this function
  */
 #define DO_RETURN_RESCHEDUAL	{		\
- core->manage_toid = gtk_timeout_add(	\
+ core->manage_toid = gtk_timeout_add(		\
   1000l,		/* 1 second interval */	\
   EDVManageTimeoutCB, core			\
  );						\
@@ -393,7 +405,7 @@ gint EDVManageTimeoutCB(gpointer data)
  * return FALSE to never call this function again
  */
 #define DO_RETURN_NO_RESCHEDUAL		{	\
- core->manage_toid = 0;			\
+ core->manage_toid = 0;				\
 						\
  return(FALSE);					\
 }
@@ -403,16 +415,19 @@ gint EDVManageTimeoutCB(gpointer data)
  return(TRUE);					\
 }
 
-	/* Get configuration */
+	/* Get the configuration */
 	cfg_list = core->cfg_list;
-	conserve_memory = CFGItemListGetValueI(
-	    cfg_list, EDV_CFG_PARM_CONSERVE_MEMORY
-	);
+	conserve_memory = EDV_GET_B(EDV_CFG_PARM_CONSERVE_MEMORY);
 
 
 	/* Need to close all windows? */
-	if(core->pending_flags & EDV_CORE_PENDING_CLOSE_ALL_WINDOWS)
+	if(!(core->run_flags & EDV_RUN_STAY_RUNNING) &&
+	   (core->pending_flags & EDV_CORE_PENDING_CLOSE_ALL_WINDOWS)
+	)
 	{
+	    gboolean	still_processing = FALSE,
+			has_changes = FALSE;
+
 	    core->pending_flags &= ~EDV_CORE_PENDING_CLOSE_ALL_WINDOWS;
 
 	    /* Check if any windows/dialogs are in query or are still
@@ -458,12 +473,6 @@ gint EDVManageTimeoutCB(gpointer data)
 	    }
 	    /* All dialogs not in query */
 
-	    /* Reset still processing marker, which indicates that
-	     * there are still windows that are processing and cannot
-	     * be closed
-	     */
-	    still_processing = FALSE;
-
 	    /* Reset has changes marker, which indicates there is
 	     * atleast one window with unsaved modified data
 	     */
@@ -503,12 +512,12 @@ gint EDVManageTimeoutCB(gpointer data)
 		if(archiver->processing)
 		    still_processing = TRUE;
 
-		arch_info = archiver->arch_info;
-		if(arch_info != NULL)
+		arch_info_dlg = archiver->arch_info_dlg;
+		if(arch_info_dlg != NULL)
 		{
-		    if(arch_info->processing)
+		    if(arch_info_dlg->freeze_count > 0)
 			still_processing = TRUE;
-		    if(arch_info->has_changes)
+		    if(arch_info_dlg->has_changes)
 			has_changes = TRUE;
 		}
 	    }
@@ -519,6 +528,13 @@ gint EDVManageTimeoutCB(gpointer data)
 		if(recbin->processing)
 		    still_processing = TRUE;
 	    }
+	    /* Object Operations Dialog */
+	    obj_op_dlg = core->obj_op_dlg;
+	    if(obj_op_dlg != NULL)
+	    {
+		if(obj_op_dlg->processing)
+		    still_processing = TRUE;
+	    }
 	    /* Find Window */
 	    findwin = core->find_win;
 	    if(findwin != NULL)
@@ -533,7 +549,7 @@ gint EDVManageTimeoutCB(gpointer data)
 		if(propdlg == NULL)
 		    continue;
 
-		if(propdlg->processing)
+		if(propdlg->freeze_count > 0)
 		    still_processing = TRUE;
 		if(propdlg->has_changes)
 		    has_changes = TRUE;
@@ -556,21 +572,21 @@ gint EDVManageTimeoutCB(gpointer data)
 	    deviceswin = core->device_listwin;
 	    if(deviceswin != NULL)
 	    {
-		if(deviceswin->processing)
+		if(deviceswin->freeze_count > 0)
 		    still_processing = TRUE;
 	    }
 	    /* MIME Types Window */
 	    mimetypeswin = core->mimetype_listwin;
 	    if(mimetypeswin != NULL)
 	    {
-		if(mimetypeswin->processing)
+		if(mimetypeswin->freeze_count > 0)
 		    still_processing = TRUE;
 	    }
 	    /* Options Window */
 	    optwin = core->options_window;
 	    if(optwin != NULL)
 	    {
-		if(optwin->processing)
+		if(optwin->freeze_count > 0)
 		    still_processing = TRUE;
 		if(optwin->has_changes)
 		    has_changes = TRUE;
@@ -579,7 +595,7 @@ gint EDVManageTimeoutCB(gpointer data)
 	    optwin = core->customize_window;
 	    if(optwin != NULL)
 	    {
-		if(optwin->processing)
+		if(optwin->freeze_count > 0)
 		    still_processing = TRUE;
 		if(optwin->has_changes)
 		    has_changes = TRUE;
@@ -690,6 +706,20 @@ gint EDVManageTimeoutCB(gpointer data)
 	    if(archiver == NULL)
 		continue;
 
+	    /* Archive Info Dialog mapped? */
+	    arch_info_dlg = archiver->arch_info_dlg;
+	    if(arch_info_dlg != NULL)
+	    {
+		if(arch_info_dlg->freeze_count <= 0)
+		{
+		    if(!EDVArchiveInfoIsMapped(arch_info_dlg))
+		    {
+			EDVArchiveInfoDelete(arch_info_dlg);
+			archiver->arch_info_dlg = NULL;
+		    }
+		}
+	    }
+
 	    /* Mapped? */
 	    if(EDVArchiverIsMapped(archiver))
 	    {
@@ -720,6 +750,22 @@ gint EDVManageTimeoutCB(gpointer data)
 		recbin = NULL;
 	    }
 	}
+	/* Object Operations Dialog */
+	obj_op_dlg = core->obj_op_dlg;
+	if(obj_op_dlg != NULL)
+	{
+	    if(EDVObjOpDlgIsMapped(obj_op_dlg))
+	    {
+		all_windows_unmapped = FALSE;
+	    }
+	    else
+	    {
+		/* Never delete the object operations dialog since
+		 * it holds the last used values that needs to be
+		 * preserved
+		 */
+	    }
+	}
 	/* Find Window */
 	findwin = core->find_win;
 	if(findwin != NULL)
@@ -874,26 +920,26 @@ gint EDVManageTimeoutCB(gpointer data)
 	}
 
 
-	/* Were all windows unmapped? */
+	/* Were all the windows unmapped? */
 	if(all_windows_unmapped)
 	{
-	    /* All windows are unmapped, this implies that the
-	     * application must begin exiting
-	     */
+	    /* Begin exiting after all the windows have been unmapped? */
+	    if(!(core->run_flags & EDV_RUN_STAY_RUNNING))
+	    {
+		/* Removes all timeout callbacks and breaks out of
+		 * one GTK main loop
+		 */
+		DO_REMOVE_TIMEOUTS_AND_BREAK_MAIN
 
-	    /* Removes all timeout callbacks and breaks out of one GTK
-	     * main loop
-	     */
-	    DO_REMOVE_TIMEOUTS_AND_BREAK_MAIN
-
-	    /* Return and do not call this function again */
-	    DO_RETURN_NO_RESCHEDUAL
+		/* Return and do not call this function again */
+		DO_RETURN_NO_RESCHEDUAL
+	    }
 	}
 
 
 
-	/* Check and handle any InterPS commands if the InterPS
-	 * signal count is positive
+	/* Check for and handle any pending InterPS commands if the
+	 * InterPS signal count is positive
 	 */
 	if(core->interps_signal_count > 0)
 	{
@@ -933,6 +979,7 @@ gint EDVManageTimeoutCB(gpointer data)
 
 
 	DO_RETURN_RESCHEDUAL
+
 #undef DO_RETURN_RESCHEDUAL
 #undef DO_RETURN_NO_RESCHEDUAL
 #undef DO_RETURN_CURRENT_SCHEDUAL
@@ -1238,7 +1285,7 @@ void EDVClearHistoryCB(GtkWidget *widget, gpointer data)
 	if(core == NULL)
 	    return;
 
-	EDVClearHistoryEvents(core, toplevel);
+	EDVClearHistoryEvents(core, TRUE, toplevel);
 }
 
 /*
@@ -1252,7 +1299,7 @@ void EDVClearLocationsHistoryCB(GtkWidget *widget, gpointer data)
 	if(core == NULL)
 	    return;
 
-	EDVClearLocationsHistory(core, toplevel);
+	EDVClearLocationsHistory(core, TRUE, toplevel);
 }
 
 /*
@@ -1266,7 +1313,7 @@ void EDVClearRunHistoryCB(GtkWidget *widget, gpointer data)
 	if(core == NULL)
 	    return;
 
-	EDVClearRunHistory(core, toplevel);
+	EDVClearRunHistory(core, TRUE, toplevel);
 }
 
 
@@ -1309,7 +1356,6 @@ void EDVEntryDragDataReceivedCB(
 	gpointer data
 )
 {
-	GList *url_list = NULL;
 	GtkEntry *entry;
 	edv_core_struct *core = EDV_CORE(data);
 	if((widget == NULL) || (dc == NULL) || (selection_data == NULL) ||
@@ -1317,13 +1363,12 @@ void EDVEntryDragDataReceivedCB(
 	)
 	    return;
 
-#define DO_FREE_LOCALS	{				\
- g_list_foreach(url_list, (GFunc)URLDelete, NULL);	\
- g_list_free(url_list);					\
- url_list = NULL;					\
-}
+	if((selection_data->data == NULL) ||
+	   (selection_data->length <= 0)
+	)
+	    return;
 
-	/* Given widget must have or be a GtkEntry */
+	/* Get the widget as a GtkEntry */
 	if(GTK_IS_COMBO(widget))
 	{
 	    GtkCombo *combo = GTK_COMBO(widget);
@@ -1335,66 +1380,58 @@ void EDVEntryDragDataReceivedCB(
 	}
 	else
 	{
-	    DO_FREE_LOCALS
 	    return;
 	}
 
-	/* Decode buffer into a list of URLs */
-	url_list = URLDecode(
-	    (const guint8 *)selection_data->data,
-	    selection_data->length
-	);
-	if(url_list == NULL)
-	{
-	    DO_FREE_LOCALS
-	    return;
-	}
-
-	/* Drag data is a disk object path? */
+	/* Handle by the target type
+	 *
+	 * String
+	 */
 	if((info == EDV_DND_INFO_TEXT_PLAIN) ||
 	   (info == EDV_DND_INFO_TEXT_URI_LIST) ||
 	   (info == EDV_DND_INFO_STRING)
 	)
 	{
-	    gchar *s = STRDUP("");
-	    GList *glist;
-	    const url_struct *url;
+	    gint len = selection_data->length;
+	    const gchar *ss = (const gchar *)selection_data->data;
+	    gchar *s;
 
-	    /* Generate a new text value as a space separated list
-	     * of paths obtained from the URL list
-	     */
-	    for(glist = url_list;
-		glist != NULL;
-		glist = g_list_next(glist)
-	    )
-	    {
-		url = URL(glist->data);
-		if(url == NULL)
-		    continue;
+	    if(ss[len - 1] == '\0')
+		len--;
 
-		if(STRISEMPTY(url->path))
-		    continue;
-
-		/* Prefix a space if this is not the first path */
-		if(*s != '\0')
-		    s = strinschr(s, -1, ' ');
-
-		/* Append this path to the string */
-		s = strinsstr(s, -1, url->path);
-	    }
-
-	    /* Generated new text value? */
+	    s = (gchar *)g_malloc(len + (1 * sizeof(gchar)));
 	    if(s != NULL)
 	    {
-		gtk_entry_set_text(entry, s);
-		gtk_entry_set_position(entry, -1);
+		gint position;
+		gchar *s2, *s_end;
+		GtkEditable *editable = GTK_EDITABLE(entry);
+
+		if(len > 0)
+		    memcpy(s, selection_data->data, len);
+		s[len] = '\0';
+
+		/* Remove any null deliminators within the string */
+		s2 = s;
+		s_end = s2 + len;
+		while(s2 < s_end)
+		{
+		    if(*s2 == '\0')
+			*s2 = ' ';
+		    s2++;
+		}
+
+		/* Insert the string into the entry */
+		position = gtk_editable_get_position(editable);
+		if(len > 0)
+		    gtk_editable_insert_text(
+			editable,
+			s, len,
+			&position
+		    );
+
+		g_free(s);
 	    }
-
-	    g_free(s);
 	}
-
-	DO_FREE_LOCALS
-#undef DO_FREE_LOCALS
 }
 
 
@@ -1449,23 +1486,28 @@ gint EDVEntryCompletePathCB(
 
 	    if(press)
 	    {
-		gchar *s = STRDUP(gtk_entry_get_text(entry));
-		if(s != NULL)
+		gchar *path = STRDUP(gtk_entry_get_text(entry));
+		if(path != NULL)
 		{
-		    gint status;
+		    gchar *prev_path = STRDUP(path);
+		    edv_complete_path_status status;
 
-		    s = CompletePath(s, &status);
-		    gtk_entry_set_text(entry, (s != NULL) ? s : "");
+		    /* Complete the path */
+		    path = EDVCompletePath(path, &status);
+
+		    /* Set the completed path to the entry */
+		    gtk_entry_set_text(entry, (path != NULL) ? path : "");
 		    gtk_entry_set_position(entry, -1);
-		    g_free(s);
 
-		    switch(status)
+		    /* If there was no change in the path then beep */
+		    if((prev_path != NULL) && (path != NULL))
 		    {
-		      case COMPLETE_PATH_NONE:
-		      case COMPLETE_PATH_AMBIGUOUS:
-			gdk_beep();
-			break;
+			if(!strcmp((const char *)prev_path, (const char *)path))
+			    gdk_beep();
 		    }
+
+		    g_free(prev_path);
+		    g_free(path);
 		}
 	    }
 	    SIGNAL_EMIT_STOP
@@ -1544,6 +1586,58 @@ void EDVLocBarIconDragDataGet(
 }
 
 
+/*
+ *	File selector object created callback.
+ */
+void EDVFileSelectorObjectCreatedCB(
+	const gchar *path, gpointer data
+)
+{
+	struct stat lstat_buf;
+	edv_core_struct *core = EDV_CORE(data);
+	if((core == NULL) || (path == NULL))
+	    return;
+
+	if(!lstat((const char *)path, &lstat_buf))
+	    EDVObjectAddedEmit(core, path, &lstat_buf);
+}
+
+/*
+ *	File selector object modified callback.
+ */
+void EDVFileSelectorObjectModifiedCB(
+	const gchar *old_path,
+	const gchar *new_path,
+	gpointer data
+)
+{
+	struct stat lstat_buf;
+	edv_core_struct *core = EDV_CORE(data);
+	if((core == NULL) || (new_path == NULL))
+	    return;
+
+	if(old_path == NULL)
+	    old_path = new_path;
+
+	if(!lstat((const char *)new_path, &lstat_buf))
+	    EDVObjectModifiedEmit(core, old_path, new_path, &lstat_buf);
+}
+
+/*
+ *	File selector object deleted callback.
+ */
+void EDVFileSelectorObjectDeletedCB(
+	const gchar *path, gpointer data
+)
+{
+	edv_core_struct *core = EDV_CORE(data);
+	if((core == NULL) || (path == NULL))
+	    return;
+
+	EDVObjectRemovedEmit(core, path);
+}
+
+
 /*
  *	Write protect changed emit.
  *
@@ -1616,7 +1710,6 @@ void EDVObjectAddedEmit(
 	if((core == NULL) || STRISEMPTY(path))
 	    return;
 
-	/* Copy the specified path */
 	dpath = STRDUP(path);
 
 	/* Update the devices list */
@@ -1668,13 +1761,13 @@ void EDVObjectModifiedEmit(
 )
 {
 	gint i;
-	gchar *dpath;
+	gchar *dpath, *dnew_path;
 
 	if((core == NULL) || STRISEMPTY(path))
 	    return;
 
-	/* Copy the specified path */
 	dpath = STRDUP(path);
+	dnew_path = STRDUP(new_path);
 
 	/* Update the devices list */
 	EDVDevicesListUpdateStats(
@@ -1689,19 +1782,19 @@ void EDVObjectModifiedEmit(
 	/* File Browsers */
 	for(i = 0; i < core->total_browsers; i++)
 	    EDVBrowserObjectModifiedNotifyCB(
-		core->browser[i], dpath, new_path, lstat_buf
+		core->browser[i], dpath, dnew_path, lstat_buf
 	    );
 
 	/* Image browsers */
 	for(i = 0; i < core->total_imbrs; i++)
 	    EDVImbrObjectModifiedNotifyCB(
-		core->imbr[i], dpath, new_path, lstat_buf
+		core->imbr[i], dpath, dnew_path, lstat_buf
 	    );
 
 	/* Archivers */
 	for(i = 0; i < core->total_archivers; i++)
 	    EDVArchiverObjectModifiedNotifyCB(
-		core->archiver[i], dpath, new_path, lstat_buf
+		core->archiver[i], dpath, dnew_path, lstat_buf
 	    );
 
 	/* Skip the Recycle Bin */
@@ -1709,10 +1802,11 @@ void EDVObjectModifiedEmit(
 	/* Property Dialogs */
 	for(i = 0; i < core->total_propdlgs; i++)
 	    EDVPropDlgObjectModifiedNotifyCB(
-		core->propdlg[i], dpath, new_path, lstat_buf
+		core->propdlg[i], dpath, dnew_path, lstat_buf
 	    );
 
 	g_free(dpath);
+	g_free(dnew_path);
 }
 
 /*
@@ -1827,10 +1921,14 @@ void EDVArchiveObjectAddedEmit(
 )
 {
 	gint i;
+	gchar *darch_path, *dpath;
 
 	if(core == NULL)
 	    return;
 
+	darch_path = STRDUP(arch_path);
+	dpath = STRDUP(path);
+
 	/* Update the devices list */
 	EDVDevicesListUpdateStats(
 	    core->device, core->total_devices
@@ -1844,8 +1942,11 @@ void EDVArchiveObjectAddedEmit(
 	/* Archivers */
 	for(i = 0; i < core->total_archivers; i++)
 	    EDVArchiverArchiveObjectAddedNotifyCB(
-		core->archiver[i], arch_path, path, obj
+		core->archiver[i], darch_path, dpath, obj
 	    );
+
+	g_free(darch_path);
+	g_free(dpath);
 }
 
 /*
@@ -1864,10 +1965,15 @@ void EDVArchiveObjectModifiedEmit(
 )
 {
 	gint i;
+	gchar *darch_path, *dpath, *dnew_path;
 
 	if(core == NULL)
 	    return;
 
+	darch_path = STRDUP(arch_path);
+	dpath = STRDUP(path);
+	dnew_path = STRDUP(new_path);
+
 	/* Update the devices list */
 	EDVDevicesListUpdateStats(
 	    core->device, core->total_devices
@@ -1881,8 +1987,12 @@ void EDVArchiveObjectModifiedEmit(
 	/* Archivers */
 	for(i = 0; i < core->total_archivers; i++)
 	    EDVArchiverArchiveObjectModifiedNotifyCB(
-		core->archiver[i], arch_path, path, new_path, obj
+		core->archiver[i], darch_path, dpath, dnew_path, obj
 	    );
+
+	g_free(darch_path);
+	g_free(dpath);
+	g_free(dnew_path);
 }
 
 /*
@@ -1898,10 +2008,14 @@ void EDVArchiveObjectRemovedEmit(
 )
 {
 	gint i;
+	gchar *darch_path, *dpath;
 
 	if(core == NULL)
 	    return;
 
+	darch_path = STRDUP(arch_path);
+	dpath = STRDUP(path);
+
 	/* Update the devices list */
 	EDVDevicesListUpdateStats(
 	    core->device, core->total_devices
@@ -1915,8 +2029,11 @@ void EDVArchiveObjectRemovedEmit(
 	/* Archivers */
 	for(i = 0; i < core->total_archivers; i++)
 	    EDVArchiverArchiveObjectRemovedNotifyCB(
-		core->archiver[i], arch_path, path
+		core->archiver[i], darch_path, dpath
 	    );
+
+	g_free(darch_path);
+	g_free(dpath);
 }
 
 
@@ -2244,6 +2361,12 @@ void EDVMimeTypeAddedEmit(
 	EDVRecBinMimeTypeAddedCB(
 	    core->recbin, mt_num, mt
 	);
+
+	/* Property dialogs */
+	for(i = 0; i < core->total_propdlgs; i++)
+	    EDVPropDlgMIMETypeAddedNotifyCB(
+		core->propdlg[i], mt_num, mt
+	    );
 }
 
 /*
@@ -2305,6 +2428,12 @@ void EDVMimeTypeModifiedEmit(
 	EDVRecBinMimeTypeModifiedCB(
 	    core->recbin, mt_num, mt
 	);
+
+	/* Property dialogs */
+	for(i = 0; i < core->total_propdlgs; i++)
+	    EDVPropDlgMIMETypeModifiedNotifyCB(
+		core->propdlg[i], mt_num, mt
+	    );
 }
 
 /*
@@ -2366,6 +2495,12 @@ void EDVMimeTypeRemovedEmit(
 	EDVRecBinMimeTypeRemovedCB(
 	    core->recbin, mt_num
 	);
+
+	/* Property dialogs */
+	for(i = 0; i < core->total_propdlgs; i++)
+	    EDVPropDlgMIMETypeRemovedNotifyCB(
+		core->propdlg[i], mt_num
+	    );
 }
 
 
@@ -2485,7 +2620,7 @@ void EDVDeviceRemovedEmit(
  */
 void EDVHistoryAddedEmit(
 	edv_core_struct *core,
-	gint h_num, edv_history_struct *h_ptr
+	gint h_num, edv_history_struct *h
 )
 {
 	if(core == NULL)
@@ -2496,7 +2631,7 @@ void EDVHistoryAddedEmit(
 	/* History Window */
 	EDVHistoryWinHistoryAddedCB(
 	    core->history_listwin,
-	    h_num, h_ptr
+	    h_num, h
 	);
 }
 
