@@ -73,6 +73,19 @@ extern void EDVLocBarIconDragDataGet(
 	gpointer data
 );
 
+/* File Selector Callbacks */
+extern void EDVFileSelectorObjectCreatedCB(
+        const gchar *path, gpointer data
+);
+extern void EDVFileSelectorObjectModifiedCB(
+        const gchar *old_path,
+        const gchar *new_path,
+        gpointer data
+);
+extern void EDVFileSelectorObjectDeletedCB(
+        const gchar *path, gpointer data
+);
+
 
 /* Write Protect Changed Emitters */
 extern void EDVWriteProtectChangedEmit(
@@ -164,7 +177,7 @@ extern void EDVDeviceRemovedEmit(
 /* History Event Emitters */
 extern void EDVHistoryAddedEmit(
 	edv_core_struct *core,
-	gint h_num, edv_history_struct *h_ptr
+	gint h_num, edv_history_struct *h
 );
 extern void EDVHistoryRemovedEmit(
 	edv_core_struct *core,
