@@ -1,5 +1,5 @@
 /*
-                      Endeavour Configuration List
+		      Endeavour Configuration List
  */
 
 #ifndef EDV_CFG_LIST_H
@@ -62,6 +62,9 @@
 #define EDV_CFG_PARM_LISTS_MIDDLE_CLICK_SCROLL	"ListsMiddleClickScroll"
 #define EDV_CFG_PARM_RIGHT_CLICK_MENU_SELECTS	"RightClickMenuSelects"
 
+#define EDV_CFG_PARM_SIZE_FORMAT		"SizeFormat"
+#define EDV_CFG_PARM_BLOCK_SIZE			"BlockSize"
+
 #define EDV_CFG_PARM_DATE_RELATIVITY		"DateRelativity"
 #define EDV_CFG_PARM_DATE_FORMAT		"DateFormat"
 
@@ -119,6 +122,9 @@
 #define EDV_CFG_PARM_SOUND_PLAY_ERROR		"SoundPlayError"
 #define EDV_CFG_PARM_SOUND_PLAY_COMPLETED	"SoundPlayCompleted"
 
+#define EDV_CFG_PARM_OBJECT_OPS_MENU		"ObjectsOpsMenu"
+#define EDV_CFG_PARM_NEW_OBJECTS_MENU		"NewObjectsMenu"
+
 #define EDV_CFG_PARM_STARTUP_BROWSER		"StartupBrowser"
 #define EDV_CFG_PARM_STARTUP_IMBR		"StartupImageBrowser"
 #define EDV_CFG_PARM_STARTUP_ARCHIVER		"StartupArchiver"
@@ -193,6 +199,8 @@
 #define EDV_CFG_PARM_PRESENTATION_COLOR_FG	"PresentationColorFG"
 #define EDV_CFG_PARM_PRESENTATION_COLOR_FG_INVERT	"PresentationColorFGInvert"
 #define EDV_CFG_PARM_PRESENTATION_COLOR_BG	"PresentationColorBG"
+#define EDV_CFG_PARM_PRESENTATION_TRANSISION	"PresentationTransision"
+#define EDV_CFG_PARM_PRESENTATION_TRANSISION_INT	"PresentationTransisionInt"
 #define EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY	"PresentationHidePointerDelay"
 #define EDV_CFG_PARM_PRESENTATION_SLIDESHOW		"PresentationSlideshow"
 #define EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY	"PresentationSlideshowDelay"
@@ -333,6 +341,9 @@
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_LISTS_MIDDLE_CLICK_SCROLL,		NULL }, \
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_RIGHT_CLICK_MENU_SELECTS,		NULL }, \
 										\
+{ CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_SIZE_FORMAT,			NULL },	\
+{ CFG_ITEM_TYPE_UINT64,	EDV_CFG_PARM_BLOCK_SIZE,			NULL },	\
+										\
 { CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_DATE_RELATIVITY,			NULL },	\
 { CFG_ITEM_TYPE_STRING,	EDV_CFG_PARM_DATE_FORMAT,			NULL },	\
 										\
@@ -383,6 +394,9 @@
 { CFG_ITEM_TYPE_STRING,	EDV_CFG_PARM_SOUND_PLAY_ERROR,			NULL },	\
 { CFG_ITEM_TYPE_STRING,	EDV_CFG_PARM_SOUND_PLAY_COMPLETED,		NULL }, \
 										\
+{ CFG_ITEM_TYPE_MENU,	EDV_CFG_PARM_OBJECT_OPS_MENU,			NULL }, \
+{ CFG_ITEM_TYPE_MENU,	EDV_CFG_PARM_NEW_OBJECTS_MENU,			NULL }, \
+										\
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_STARTUP_BROWSER,			NULL },	\
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_STARTUP_IMBR,			NULL },	\
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_STARTUP_ARCHIVER,			NULL },	\
@@ -450,9 +464,11 @@
 { CFG_ITEM_TYPE_COLOR,	EDV_CFG_PARM_PRESENTATION_COLOR_FG,		NULL }, \
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_PRESENTATION_COLOR_FG_INVERT,	NULL }, \
 { CFG_ITEM_TYPE_COLOR,	EDV_CFG_PARM_PRESENTATION_COLOR_BG,		NULL }, \
-{ CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY,	NULL }, \
+{ CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_PRESENTATION_TRANSISION,		NULL }, \
+{ CFG_ITEM_TYPE_UINT64,	EDV_CFG_PARM_PRESENTATION_TRANSISION_INT,	NULL }, \
+{ CFG_ITEM_TYPE_UINT64,	EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY,	NULL }, \
 { CFG_ITEM_TYPE_UINT8,	EDV_CFG_PARM_PRESENTATION_SLIDESHOW,		NULL }, \
-{ CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY,	NULL }, \
+{ CFG_ITEM_TYPE_UINT64,	EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY,	NULL }, \
 										\
 { CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_RECBIN_X,				NULL },	\
 { CFG_ITEM_TYPE_INT32,	EDV_CFG_PARM_RECBIN_Y,				NULL },	\
@@ -535,6 +551,7 @@
 #define EDV_GET_INTLIST(_parm_)	(CFGItemListGetValueIntList(cfg_list, (_parm_)))
 #define EDV_GET_ACCELKEY_LIST(_parm_) (CFGItemListGetValueAccelkeyList(cfg_list, (_parm_)))
 #define EDV_GET_STYLE(_parm_)	(CFGItemListGetValueStyle(cfg_list, (_parm_)))
+#define EDV_GET_MENU(_parm_)	(CFGItemListGetValueMenu(cfg_list, (_parm_)))
 
 /*
  *	Configuration Set Macros:
@@ -572,6 +589,9 @@
 #define EDV_SET_STYLE(_parm_,_v_)	(CFGItemListSetValueStyle( \
  cfg_list, (_parm_), (_v_), (FALSE) \
 ))
+#define EDV_SET_MENU(_parm_,_v_)	(CFGItemListSetValueMenu( \
+ cfg_list, (_parm_), (_v_), (FALSE) \
+))
 
 
 #endif	/* EDV_CFG_LIST_H */
