@@ -131,7 +131,7 @@ gint EDVConfirmLink(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 
 	if(core == NULL)
 	    return(CDIALOG_RESPONSE_NOT_AVAILABLE);
@@ -164,7 +164,7 @@ gint EDVConfirmLink(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El enlace:\n\
 \n\
@@ -233,7 +233,7 @@ To:\n\
 		, src_path, tar_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Enlace %i objetos a:\n\
 \n\
@@ -273,14 +273,14 @@ To:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_link2_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);	/* Delete message */
+	g_free(msg);	/* Delete message */
 
 	return(response);
 }
@@ -300,7 +300,7 @@ gint EDVConfirmMove(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 
 	if(core == NULL)
 	    return(CDIALOG_RESPONSE_NOT_AVAILABLE);
@@ -333,7 +333,7 @@ gint EDVConfirmMove(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mueva:\n\
 \n\
@@ -402,7 +402,7 @@ To:\n\
 		, src_path, tar_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mueva %i objetos a:\n\
 \n\
@@ -442,14 +442,14 @@ To:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_move_file_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);    /* Delete message */
+	g_free(msg);    /* Delete message */
 
 	return(response);
 }
@@ -469,7 +469,7 @@ gint EDVConfirmCopy(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 
 
 	if(core == NULL)
@@ -503,7 +503,7 @@ gint EDVConfirmCopy(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La Copia:\n\
 \n\
@@ -572,7 +572,7 @@ To:\n\
 		, src_path, tar_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La copia %i se opone a:\n\
 \n\
@@ -612,14 +612,14 @@ To:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_copy_file_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);    /* Delete message */
+	g_free(msg);    /* Delete message */
 
 	return(response);
 }
@@ -639,7 +639,7 @@ gint EDVConfirmDelete(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 	edv_delete_method delete_method;
 	guint8 **icon_data = (guint8 **)icon_trash_32x32_xpm;
 	const cfg_item_struct *cfg_list;
@@ -699,7 +699,7 @@ gint EDVConfirmDelete(
 	    if(type_str != NULL)
 		strtolower(type_str);
 
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Borre %s \"%s\"?"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -730,8 +730,8 @@ gint EDVConfirmDelete(
 	    if(FALSE)
 #endif
 	    {
-		buf = G_STRCAT(
-		    buf,
+		msg = G_STRCAT(
+		    msg,
 #if defined(PROG_LANGUAGE_SPANISH)
 "\n\
 \n\
@@ -771,7 +771,7 @@ Including any contents within that directory."
 	    g_free(type_str);
 	}
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Borra %i objetos escogidos?"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -798,8 +798,8 @@ Including any contents within that directory."
 	 */
 	if(delete_method == EDV_DELETE_METHOD_PURGE)
 	{
-	    buf = G_STRCAT(
-		buf,
+	    msg = G_STRCAT(
+		msg,
 #if defined(PROG_LANGUAGE_SPANISH)
 "\n\nUsted no ser capaz de recuperar el object(s)\n\
 una vez ellos han sido borrados!"
@@ -835,13 +835,13 @@ once they have been deleted!"
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL, icon_data,
+	    title, msg, NULL, icon_data,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);    /* Delete message */
+	g_free(msg);    /* Delete message */
 
 	return(response);
 }
@@ -861,7 +861,7 @@ gint EDVConfirmArchiveAdd(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 
 
 	if(core == NULL)
@@ -895,7 +895,7 @@ gint EDVConfirmArchiveAdd(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Agregue:\n\
 \n\
@@ -964,7 +964,7 @@ To Archive:\n\
 		, src_path, arch_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Agregue %i objetos al archivo:\n\
 \n\
@@ -1004,14 +1004,14 @@ To Archive:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_archive_add_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);	/* Delete message */
+	g_free(msg);	/* Delete message */
 
 	return(response);
 }
@@ -1032,7 +1032,7 @@ gint EDVConfirmArchiveExtract(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 
 
 	if(core == NULL)
@@ -1066,7 +1066,7 @@ gint EDVConfirmArchiveExtract(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El Extracto:\n\
 \n\
@@ -1135,7 +1135,7 @@ To:\n\
 		, src_path, tar_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El extracto %i los objetos a:\n\
 \n\
@@ -1175,14 +1175,14 @@ To:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_archive_extract_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);	/* Delete message */
+	g_free(msg);	/* Delete message */
 
 	return(response);
 }
@@ -1201,8 +1201,7 @@ gint EDVConfirmArchiveDelete(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
-
+	gchar *msg;
 
 	if(core == NULL)
 	    return(CDIALOG_RESPONSE_NOT_AVAILABLE);
@@ -1235,7 +1234,7 @@ gint EDVConfirmArchiveDelete(
 	;
 
 	if((total_objects == 1) && (src_path != NULL))
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Borre:\n\
 \n\
@@ -1290,13 +1289,13 @@ som det stryker!"
 \n\
     %s\n\
 \n\
-This object in the archive cannot be recovered once\n\
-it has been deleted!"
+You will not be able to recover this object in the\n\
+archive once it has been deleted!"
 #endif
 		, src_path
 	    );
 	else
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Borra %i objetos escogidos?\n\
 \n\
@@ -1344,14 +1343,14 @@ they have been deleted!"
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_purge_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);	/* Delete message */
+	g_free(msg);	/* Delete message */
 
 	return(response);
 }
@@ -1373,7 +1372,7 @@ gint EDVConfirmRecover(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 	const gchar *recycled_index_file;
 
 
@@ -1416,7 +1415,7 @@ gint EDVConfirmRecover(
 
 	if((total_objects == 1) && (src_path != NULL))
 	{
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Recupere:\n\
 \n\
@@ -1487,7 +1486,7 @@ To:\n\
 	}
 	else if(tar_path != NULL)
 	{
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Recupere %i objetos a:\n\
 \n\
@@ -1526,7 +1525,7 @@ To:\n\
 	}
 	else
 	{
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Recupere %i objetos?"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1551,14 +1550,14 @@ To:\n\
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_recover_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);	/* Delete message */
+	g_free(msg);	/* Delete message */
 
 	return(response);
 }
@@ -1577,7 +1576,7 @@ gint EDVConfirmPurge(
 {
 	gint response;
 	const gchar *title;
-	gchar *buf;
+	gchar *msg;
 	const gchar *recycled_index_file;
 
 	if(core == NULL)
@@ -1619,7 +1618,7 @@ gint EDVConfirmPurge(
 
 	if((total_objects == 1) && (src_path != NULL))
 	{
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Purga recycled \"%s\" objetivo?\n\
 \n\
@@ -1666,7 +1665,7 @@ once it has been purged!"
 	}
 	else
 	{
-	    buf = g_strdup_printf(
+	    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Purga %i objetos de recycled?\n\
 \n\
@@ -1715,14 +1714,14 @@ once they have been purged!"
 	EDVPlaySoundQuestion(core);
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponseIconData(
-	    title, buf, NULL,
+	    title, msg, NULL,
 	    (guint8 **)icon_purge_32x32_xpm,
 	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 	    CDIALOG_BTNFLAG_YES
 	);
 	CDialogSetTransientFor(NULL);
 
-	g_free(buf);    /* Delete message */
+	g_free(msg);    /* Delete message */
 
 	return(response);
 }
