@@ -1,16 +1,19 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
+#if defined(HAVE_IMLIB)
+# include <Imlib.h>
+#endif
 #include <gtk/gtk.h>
 #if defined(_WIN32)
+# include <gdk/gdkprivate.h>
 # include <gdk/gdkwin32.h>
-# define HAVE_WIN32
+# include <windows.h>
 #else
 # include <gdk/gdkx.h>
-# define HAVE_X
+# include <unistd.h>
 #endif
-#include <unistd.h>
 
-#include "../include/string.h"
 #include "../include/disk.h"
 
 #include "guiutils.h"
@@ -33,8 +36,10 @@
 #include "edv_devices_list.h"
 #include "edv_mime_type.h"
 #include "edv_mime_types_list.h"
+#include "obj_op_dlg.h"
 #include "browser.h"
 #include "imbr.h"
+#include "presentation_mode.h"
 #include "archiver.h"
 #include "recbin.h"
 #include "recbin_desktop_icon.h"
@@ -55,15 +60,16 @@
 #include "config.h"
 
 
-static GList *g_list_new_length(guint length, gpointer datas);
-static void g_list_set_nth_data(GList *glist, guint n, gpointer data);
+static GList *EDVNewGListLength(const guint length, gpointer datas);
+static void EDVNewGListSetNthData(GList *glist, const guint n, gpointer data);
 
 static cfg_item_struct *EDVNewDefaultConfigurationList(
         const gchar *startup_path,
         const gchar *home_path
 );
-static void EDVCreateStandardCursors(edv_core_struct *core);
+static void EDVNewCreateStandardCursors(edv_core_struct *core);
 
+/* Endeavour Core */
 edv_core_struct *EDVNew(
 	gint argc, gchar *argv[],
 	gint *error_rtn
@@ -89,7 +95,7 @@ void EDVDelete(
 /*
  *	Creates a new GList of the specified length.
  */
-static GList *g_list_new_length(guint length, gpointer datas)
+static GList *EDVNewGListLength(const guint length, gpointer datas)
 {
 	gint i;
 	GList *glist = NULL;
@@ -103,11 +109,11 @@ static GList *g_list_new_length(guint length, gpointer datas)
 /*
  *	Sets the data of the nth item in the GList.
  */
-static void g_list_set_nth_data(GList *glist, guint n, gpointer data)
+static void EDVNewGListSetNthData(GList *glist, const guint n, gpointer data)
 {
-	glist = g_list_nth(glist, n);
-	if(glist != NULL)
-	    glist->data = data;
+	GList *glist_nth = g_list_nth(glist, n);
+	if(glist_nth != NULL)
+	    glist_nth->data = data;
 }
 
 
@@ -291,6 +297,12 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	/* Right Click Menus Selects */
 	EDV_SET_B(EDV_CFG_PARM_RIGHT_CLICK_MENU_SELECTS, TRUE);
 
+	/* Size Format */
+	EDV_SET_I(EDV_CFG_PARM_SIZE_FORMAT, (gint)EDV_SIZE_FORMAT_DELIMINATED);
+
+	/* Block Size */
+	EDV_SET_UL(EDV_CFG_PARM_BLOCK_SIZE, 1000l);
+
 	/* Date Relativity */
 	EDV_SET_I(EDV_CFG_PARM_DATE_RELATIVITY, (gint)EDV_DATE_RELATIVITY_ABSOLUTE);
 	/* Date Format (see strftime()) */
@@ -391,6 +403,9 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	EDV_SET_S(EDV_CFG_PARM_SOUND_PLAY_ERROR, "");
 	EDV_SET_S(EDV_CFG_PARM_SOUND_PLAY_COMPLETED, "");
 
+	/* New Objects Menu */
+	/* Skip */
+
 	/* Startup Window */
 	EDV_SET_B(EDV_CFG_PARM_STARTUP_BROWSER, TRUE);
 	EDV_SET_B(EDV_CFG_PARM_STARTUP_IMBR, FALSE);
@@ -464,56 +479,56 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	CFGIntListDelete(intlist);
 
 	/* Browser Contents Column Widths */
-	glist = g_list_new_length(
+	glist = EDVNewGListLength(
 	    EDV_BROWSER_CONTENTS_CLIST_COLUMNS_MAX,
 	    (gpointer)0
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_NAME, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_SIZE, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_TYPE, (gpointer)100
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_PERMISSIONS, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_OWNER, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_GROUP, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_DATE_ACCESS, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_DATE_MODIFIED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_DATE_CHANGED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_HARD_LINKS, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_LINKED_TO, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_DEVICE_INDEX, (gpointer)60
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_INDEX, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_DEVICE_TYPE, (gpointer)90
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_BLOCK_SIZE, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_BROWSER_COLUMN_TYPE_BLOCKS, (gpointer)60
 	);
 	intlist = CFGIntListNew(glist);
@@ -625,9 +640,14 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	color = CFGColorNew(0.0f, 0.0f, 0.5f, 1.0f);
 	EDV_SET_COLOR(EDV_CFG_PARM_PRESENTATION_COLOR_BG, color);
 	CFGColorDelete(color);
-	EDV_SET_I(EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY, 3);
+	EDV_SET_I(
+	    EDV_CFG_PARM_PRESENTATION_TRANSISION,
+	    PRESENTATION_MODE_TRANSISION_NONE
+	);
+	EDV_SET_L(EDV_CFG_PARM_PRESENTATION_TRANSISION_INT, 1500l);
+	EDV_SET_L(EDV_CFG_PARM_PRESENTATION_HIDE_POINTER_DELAY, 2l);
 	EDV_SET_B(EDV_CFG_PARM_PRESENTATION_SLIDESHOW, FALSE);
-	EDV_SET_I(EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY, 3);
+	EDV_SET_L(EDV_CFG_PARM_PRESENTATION_SLIDESHOW_DELAY, 3l);
 
 	/* Archiver Geometry */
 	EDV_SET_I(EDV_CFG_PARM_ARCHIVER_X, 0);
@@ -686,56 +706,56 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	CFGIntListDelete(intlist);
 
 	/* Archiver Contents Column Widths */
-	glist = g_list_new_length(
+	glist = EDVNewGListLength(
 	    EDV_ARCHIVER_CONTENTS_CLIST_COLUMNS_MAX,
 	    (gpointer)0
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_NAME, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_SIZE, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_COMPRESSED_SIZE, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_TYPE, (gpointer)100
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_PERMISSIONS, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_OWNER, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_GROUP, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_DATE_ACCESS, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_DATE_MODIFIED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_DATE_CHANGED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_LOCATION, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_LINKED_TO, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_DEVICE_TYPE, (gpointer)90
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_COMPRESSION, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_METHOD, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_ARCHIVER_COLUMN_TYPE_CRC, (gpointer)80
 	);
 	intlist = CFGIntListNew(glist);
@@ -806,50 +826,50 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 	CFGIntListDelete(intlist);
 
 	/* Recycle Bin Contents Column Widths */
-	glist = g_list_new_length(
+	glist = EDVNewGListLength(
 	    EDV_RECBIN_CONTENTS_CLIST_COLUMNS_MAX,
 	    (gpointer)0
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_NAME, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_SIZE, (gpointer)70
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_TYPE, (gpointer)100
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_PERMISSIONS, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_OWNER, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_GROUP, (gpointer)80
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_DATE_ACCESS, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_DATE_MODIFIED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_DATE_CHANGED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_DATE_DELETED, (gpointer)120
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_LINKED_TO, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_ORIGINAL_LOCATION, (gpointer)150
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_INDEX, (gpointer)60
 	);
-	g_list_set_nth_data(glist,
+	EDVNewGListSetNthData(glist,
 	    EDV_RECBIN_COLUMN_TYPE_CAPACITY_USED, (gpointer)85
 	);
 	intlist = CFGIntListNew(glist);
@@ -907,7 +927,7 @@ static cfg_item_struct *EDVNewDefaultConfigurationList(
 /*
  *	Creates a standard list of cursors on the core.
  */
-static void EDVCreateStandardCursors(edv_core_struct *core)
+static void EDVNewCreateStandardCursors(edv_core_struct *core)
 {
 	GdkWindow *root = GDK_ROOT_PARENT();
 	GdkColor	c_fg = { 0, 0xffff, 0xffff, 0xffff },
@@ -1175,13 +1195,14 @@ static void EDVCreateStandardCursors(edv_core_struct *core)
 edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 {
 	gint i, p;
-	const gchar *s, *arg;
+	const gchar *arg;
 	gboolean cfg_version_different;
 	gint cfg_file_load_status;
 	gint		old_version_major,
 			old_version_minor,
 			old_version_release;
 	gboolean	no_splash = FALSE,
+			safe_mode = FALSE,
 			startup_browser,
 			startup_imbr,
 			startup_archiver,
@@ -1191,27 +1212,21 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			*recbin_index_file,
 			*alternate_config_file = NULL,
 			*startup_path = NULL;
-	gchar		*cwd,
+	gchar		*sd,
+			*cwd,
 			*startup_full_path = NULL,
 			hostname[64 + 1];
 	cfg_item_struct	*cfg_list;
-	edv_core_struct *core = EDV_CORE(
-	    g_malloc0(sizeof(edv_core_struct))
-	);
-	if(core == NULL)
-	{
-	    if(error_rtn != NULL)
-		*error_rtn = -3;
-	    return(core);
-	}
-
+	edv_core_struct *core;
 
 	if(error_rtn != NULL)
 	    *error_rtn = 0;
 
-#define DO_FREE_LOCALS	{		\
- g_free(cwd);				\
- g_free(startup_full_path);		\
+#define CLEANUP_RETURN(_v_)	{		\
+ g_free(cwd);					\
+ g_free(startup_full_path);			\
+						\
+ return(_v_);					\
 }
 
 	/* Get the current working directory */
@@ -1223,7 +1238,20 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	    cwd = STRDUP("/");
 #endif
 
-	/* Begin resetting values */
+	/* Create the Endeavour Mark II Core */
+	core = EDV_CORE(g_malloc0(sizeof(edv_core_struct)));
+	if(core == NULL)
+	{
+	    if(error_rtn != NULL)
+		*error_rtn = -3;
+	    return(NULL);
+	}
+
+	/* Set the initial runlevel at construct */
+	core->runlevel = EDV_RUNLEVEL_CONSTRUCT;
+
+	core->run_flags = 0;
+
 	core->cfg_file = NULL;
 	core->cfg_list = NULL;
 
@@ -1246,6 +1274,8 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	core->recbin_deskicon = NULL;
 	core->last_recbin_items = 0;
 
+	core->obj_op_dlg = NULL;
+
 	core->find_win = NULL;
 
 	core->propdlg = NULL;
@@ -1261,9 +1291,18 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	core->check_devices_toid = 0;
 
 	core->pending_flags = 0;
-
 	core->interps_signal_count = 0;
+	core->op_level = 0;
 
+	core->archive_last_error_buf = NULL;
+	core->device_last_error_buf = NULL;
+	core->obj_last_error_buf = NULL;
+	core->recbin_last_error_buf = NULL;
+
+	core->archive_last_error = NULL;
+	core->device_last_error = NULL;
+	core->obj_last_error = NULL;
+	core->recbin_last_error = NULL;
 
 	core->pid = getpid();
 	core->prog_file = NULL;
@@ -1274,11 +1313,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	core->group_id = getgid();
 	core->effective_group_id = getegid();
 
-	core->uid = NULL;
-	core->total_uids = 0;
+	core->uids_list = NULL;
 	core->users_pulist = NULL;
-	core->gid = NULL;
-	core->total_gids = 0;
+	core->gids_list = NULL;
 	core->groups_pulist = NULL;
 
 	core->effective_user_id_str = NULL;
@@ -1324,6 +1361,11 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	core->standard_rcstyle = NULL;
 	core->lists_rcstyle = NULL;
 
+#if defined(HAVE_IMLIB)
+        /* Initialize Imlib */
+        core->imlib_handle = Imlib_init(GDK_DISPLAY());
+#endif	/* HAVE_IMLIB */
+
 
 	/* Parse arguments */
 	for(i = 1; i < argc; i++)
@@ -1354,10 +1396,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* No Splash? */
@@ -1524,21 +1565,37 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 		    window_name++;
 	    }
 
+	    /* Safe mode? */
+	    else if(!g_strcasecmp(arg, "--safe-mode") ||
+		    !g_strcasecmp(arg, "-safe-mode") ||
+		    !g_strcasecmp(arg, "--safe_mode") ||
+		    !g_strcasecmp(arg, "-safe_mode") ||
+		    !g_strcasecmp(arg, "--safe") ||
+		    !g_strcasecmp(arg, "-safe")
+	    )
+	    {
+		safe_mode = TRUE;
+		core->run_flags |= EDV_RUN_SAFE_MODE;
+	    }
+
 	    /* Geometry? */
 	    else if(!g_strcasecmp(arg, "--geometry") ||
 		    !g_strcasecmp(arg, "-geometry")
 	    )
 	    {
 		i++;
-		arg = (i < argc) ? argv[i] : NULL;
-		if(!STRISEMPTY(arg))
+		if(i < argc)
 		{
 		    gint x, y, width, height;
+		    GdkGeometryFlags status;
 		    GdkRectangle *rect = &core->geometry;
-		    GdkGeometryFlags status = gdk_parse_geometry(
+
+		    arg = argv[i];
+
+		    status = gdk_parse_geometry(
 			arg, &x, &y, &width, &height
 		    );
-		    core->geometry_set = status ? TRUE : FALSE;
+		    core->geometry_set = (status != 0x00000000) ? TRUE : FALSE;
 		    rect->x = (status & GDK_GEOMETRY_X) ? x : 0;
 		    rect->y = (status & GDK_GEOMETRY_Y) ? y : 0;
 		    rect->width = (status & GDK_GEOMETRY_WIDTH) ? width : 0;
@@ -1548,13 +1605,12 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 		{
 		    g_printerr(
 			"%s: Requires argument.\n",
-			argv[i - 1]
+			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}		
 	    }
 	    /* Font name? */
@@ -1578,10 +1634,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Foreground color name? */
@@ -1605,10 +1660,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Background color name? */
@@ -1632,10 +1686,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Selected foreground color name? */
@@ -1659,10 +1712,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Selected background color name? */
@@ -1686,10 +1738,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Pointer cursor foreground color name? */
@@ -1713,10 +1764,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Pointer cursor background color name? */
@@ -1740,10 +1790,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Background pixmap path? */
@@ -1767,10 +1816,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Selected background pixmap path? */
@@ -1794,10 +1842,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 
@@ -1820,10 +1867,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Class? */
@@ -1845,10 +1891,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Title? */
@@ -1870,10 +1915,9 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 			arg
 		    );
 		    EDVDelete(core, FALSE);
-		    DO_FREE_LOCALS
 		    if(error_rtn != NULL)
 			*error_rtn = -2;
-		    return(NULL);
+		    CLEANUP_RETURN(NULL);
 		}
 	    }
 	    /* Non-switch argument? */
@@ -1889,6 +1933,7 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 		    /* The startup_path may be a relative path so
 		     * complete it as needed as startup_full_path
 		     */
+		    g_free(startup_full_path);
 		    startup_full_path = EDVEvaluatePath(cwd, startup_path);
 		}
 		else if(arg_extra == NULL)
@@ -1899,6 +1944,13 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	}
 
 
+	/* Disable certain options if running in safe mode */
+	if(safe_mode)
+	{
+	    no_splash = TRUE;
+	}
+
+
 	/* Record the path to the program file from the specified
 	 * command line arguments
 	 */
@@ -1908,39 +1960,50 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	    core->prog_file = STRDUP(argv[0]);
 	}
 
-	/* Get the Host's Name */
+	/* Get the host's name */
 	if(gethostname((char *)hostname, sizeof(hostname)))
 	    *hostname = '\0';
 	else
 	    hostname[sizeof(hostname) - 1] = '\0';
 
 	/* Open the User & Group IDs list files */
-	core->uid = EDVUIDGetSystem(&core->total_uids);
-	core->gid = EDVGIDGetSystem(&core->total_gids);
+	core->uids_list = EDVUIDGetSystem();
+	core->gids_list = EDVGIDGetSystem();
 
 	/* Get the Effective User ID string */
-	s = EDVUIDGetNameFromUID(
-	    core->uid, core->total_uids,
+	sd = EDVUIDGetNameFromUID(
+	    core->uids_list,
 	    core->effective_user_id, NULL
 	);
+
 	g_free(core->effective_user_id_str);
 	core->effective_user_id_str = STRDUP(
-	    !STRISEMPTY(s) ? s : "anonymous"
+	    STRISEMPTY(sd) ? "anonymous" : sd
 	);
 
 	/* Get the Effective User ID & Hostname string */
 	g_free(core->effective_user_id_host_str);
-	core->effective_user_id_host_str = g_strdup_printf(
-	    "%s@%s",
-	    !STRISEMPTY(s) ? s : "anonymous",
-	    !STRISEMPTY(hostname) ? hostname : "unknown"
+	core->effective_user_id_host_str = g_strconcat(
+	    STRISEMPTY(sd) ? "anonymous" : sd,
+	    "@",
+	    STRISEMPTY(hostname) ? "unknown" : hostname,
+	    NULL
 	);
 
+	g_free(sd);
+
 	/* Set the configuration to default values */
 	core->cfg_list = cfg_list = EDVNewDefaultConfigurationList(
-	    startup_path,
+	    startup_full_path,
 	    core->home_dir
 	);
+	if(cfg_list == NULL)
+	{
+	    if(safe_mode)
+		g_printerr(
+"EDVNewDefaultConfigurationList(): Unable to generate the default configuration list.\n"
+		);
+	}
 
 	/* Need to set an alternate configuration file path? */
 	if(alternate_config_file != NULL)
@@ -1992,6 +2055,17 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	else if(old_version_release != PROG_VERSION_RELEASE)
 	    cfg_version_different = TRUE;
 
+	if(cfg_version_different)
+	{
+	    if(safe_mode)
+		g_print(
+"%s: Current configuration file version %i.%i.%i is different from the program version %s.\n",
+		    core->cfg_file,
+		    old_version_major, old_version_minor, old_version_release,
+		    PROG_VERSION
+		);
+	}
+
 	/* Now set the new version to the configuration since the
 	 * version just fetched may be old
 	 */
@@ -2000,9 +2074,13 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	EDV_SET_I(EDV_CFG_PARM_VERSION_RELEASE, PROG_VERSION_RELEASE);
 
 #if 0
-	/* Override the startup directory from command line? */
-	if(startup_path != NULL)
-	    EDV_SET_S(EDV_CFG_PARM_DIR_START_UP, startup_path);
+/* This is no longer used because startup_full_path will be passed
+ * to either EDVNewWindow() or to the curent running process via
+ * InterPS
+ */
+	/* Override the startup directory from the command line? */
+	if(startup_full_path != NULL)
+	    EDV_SET_S(EDV_CFG_PARM_DIR_START_UP, startup_full_path);
 #endif
 
 	/* Get the startup window options */
@@ -2033,6 +2111,17 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	    /* Issue a command to the already running process */
 	    gchar *cmd, *cmd_list[2];
 
+	    if(safe_mode)
+		g_print(
+"%s is already running on process ID %i, sending InterPS command.\n\
+\n\
+This process ID %i running in safe mode will exit after sending the\n\
+InterPS command.\n",
+		    PROG_NAME,
+		    p,
+		    core->pid
+		);
+
 	    /* If no window name was specified at startup then use
 	     * the one specified from the configuration
 	     */
@@ -2064,11 +2153,10 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	    g_free(cmd);
 
 	    EDVDelete(core, FALSE);
-	    DO_FREE_LOCALS
-	    return(NULL);
+	    CLEANUP_RETURN(NULL);
 	}
 
-	/* (Re)create InterPS lock link and remove any queued
+	/* (Re)create the InterPS lock link and remove any queued
 	 * interprocess commands
 	 */
 	switch(EDVInterPSMakeLock(cfg_list, core->pid, TRUE))
@@ -2078,18 +2166,19 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	  default:
 	    if(cfg_list != NULL)
 	    {
-		const gchar *s = CFGItemListGetValueS(
+		const gint error_code = (gint)errno;
+		const gchar *local_dir = CFGItemListGetValueS(
 		    cfg_list, EDV_CFG_PARM_DIR_LOCAL
 		);
-		gchar *lock_link = g_strdup_printf(
-		    "%s%c%s",
-		    (s != NULL) ? s : "/",
-		    G_DIR_SEPARATOR,
+		gchar *lock_link = g_strconcat(
+		    (local_dir != NULL) ? local_dir : "/",
+		    G_DIR_SEPARATOR_S,
 		    EDV_INTERPS_LOCK_LINK
 		);
 		g_printerr(
-"Unable to create the process ID lock \"%s\".\n",
-		    lock_link
+"%s: Unable to create the process ID lock: %s.\n",
+		    lock_link,
+		    g_strerror(error_code)
 		);
 		g_free(lock_link);
 	    }
@@ -2125,13 +2214,13 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	GUISetGlobalTipsState(EDV_GET_B(EDV_CFG_PARM_SHOW_TOOLTIPS));
 
 	/* Create the standard cursors */
-	EDVCreateStandardCursors(core);
+	EDVNewCreateStandardCursors(core);
 
 
 
 
-	/* Open the devices from the system, then open devices from
-	 * the local devices file
+	/* Open the devices from the system, then from the local
+	 * devices file
 	 */
 	EDVDevicesListLoadFromSystem(
 	    &core->device, &core->total_devices,
@@ -2151,10 +2240,15 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	);
 
 
-	/* Open the MIME Types from the system, then open MIME Types
-	 * from the local and then global MIME Types files
+	/* Open the MIME Types from the following files in the 
+	 * following order:
+	 *
+	 *	System
+	 *	Local
+	 *	Global
 	 */
 	EDVMimeTypesListSystemFileOpen(
+	    EDV_GET_S(EDV_CFG_PARM_DIR_GLOBAL),
 	    &core->mimetype, &core->total_mimetypes,
 	    -1,			/* Append */
 	    NULL, core,
@@ -2185,6 +2279,7 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	);
 	/* Update the MIME Types class list index hints */
 	EDVUpdateMIMETypeHintIndices(core);
+/* TODO: Warn if no local or global MIME Types were opened */
 
 
 	/* Create the Users & Groups popup lists */
@@ -2197,13 +2292,14 @@ edv_core_struct *EDVNew(gint argc, gchar *argv[], gint *error_rtn)
 	EDVUpdateOpenWithPUList(core);
 
 
+#if !defined(DEBIAN)
 	/* If the configuration file version is different from the
 	 * compile time version of this program, then, print warning at
 	 * this point
 	 */
-	if(cfg_version_different)
+	if(cfg_version_different && !safe_mode)
 	{
-	    gchar *s = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "%s ha cambiado de la versin %i.%i.%i a la versin %s.\n\
 \n\
@@ -2264,7 +2360,6 @@ by going to Settings->Options... and Settings->Customize...\n"
 		old_version_major, old_version_minor, old_version_release,
 		PROG_VERSION
 	    );
-
 	    EDVPlaySoundInfo(core);
 	    CDialogSetTransientFor(NULL);
 	    CDialogGetResponse(
@@ -2286,15 +2381,15 @@ by going to Settings->Options... and Settings->Customize...\n"
 "Version Change Notice"
 #endif
 
-		, s, NULL,
+		, msg, NULL,
 		CDIALOG_ICON_INSTALL,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
+	    g_free(msg);
 	    CDialogSetTransientFor(NULL);
-	    g_free(s);
 	}
-
+#endif /* !DEBIAN */
 
 	/* Check the recycle bin size and warn the user as needed */
 	EDVRecycleBinSizeCheck(core, NULL);
@@ -2364,9 +2459,12 @@ by going to Settings->Options... and Settings->Customize...\n"
 	 * file (suggesting the configuration file does not exist)
 	 */
 	if(cfg_version_different || cfg_file_load_status)
-	    EDVAboutPage(
-		core, "Copyright", NULL
-	    );
+	{
+	    if(!safe_mode)
+		EDVAboutPage(
+		    core, "Copyright", NULL
+		);
+	}
 #endif
 
 	/* Set main management timeout function */
@@ -2375,10 +2473,11 @@ by going to Settings->Options... and Settings->Customize...\n"
 	    EDVManageTimeoutCB, core
 	);
 
-	DO_FREE_LOCALS
-#undef DO_FREE_LOCALS
+	/* Set the runlevel to idle */
+	core->runlevel = EDV_RUNLEVEL_IDLE;
 
-	return(core);
+	CLEANUP_RETURN(core);
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -2399,6 +2498,9 @@ void EDVDelete(
 	if(core == NULL)
 	    return;
 
+	/* Set the runlevel to deconstruct */
+	core->runlevel = EDV_RUNLEVEL_CONSTRUCT;
+
 	/* Save the configuration? */
 	if((core->cfg_list != NULL) && save_config)
 	{
@@ -2467,6 +2569,10 @@ void EDVDelete(
 	core->recbin_deskicon = NULL;
 
 
+	/* Delete the Object Operations Dialog */
+	EDVObjOpDlgDelete(core->obj_op_dlg);
+	core->obj_op_dlg = NULL;
+
 	/* Delete the Find Window */
 	EDVFindWinDelete(core->find_win);
 	core->find_win = NULL;
@@ -2512,11 +2618,7 @@ void EDVDelete(
 	core->open_with_pulist = NULL;
 
 	/* Delete the MIME Types list */
-	for(i = 0; i < core->total_mimetypes; i++)
-	    EDVMimeTypeDelete(core->mimetype[i]);
-	g_free(core->mimetype);
-	core->mimetype = NULL;
-	core->total_mimetypes = 0;
+	EDVMimeTypesListClear(&core->mimetype, &core->total_mimetypes);
 
 
 	/* Delete the Devices popup list */
@@ -2550,17 +2652,23 @@ void EDVDelete(
 	core->groups_pulist = NULL;
 
 	/* Delete the User & Group IDs lists */
-	for(i = 0; i < core->total_uids; i++)
-	    EDVUIDDelete(core->uid[i]);
-	g_free(core->uid);
-	core->uid = NULL;
-	core->total_uids = 0;
+	if(core->uids_list != NULL)
+	{
+	    g_list_foreach(
+		core->uids_list, (GFunc)EDVUIDDelete, NULL
+	    );
+	    g_list_free(core->uids_list);
+	    core->uids_list = NULL;
+	}
 
-	for(i = 0; i < core->total_gids; i++)
-	    EDVGIDDelete(core->gid[i]);
-	g_free(core->gid);
-	core->gid = NULL;
-	core->total_gids = 0;
+	if(core->gids_list != NULL)
+	{
+	    g_list_foreach(
+		core->gids_list, (GFunc)EDVGIDDelete, NULL
+	    );
+	    g_list_free(core->gids_list);
+	    core->gids_list = NULL;
+	}
 
 
 	/* Reset the DND Icon */
@@ -2568,19 +2676,25 @@ void EDVDelete(
 
 
 	/* Delete the user defined GtkRcStyles */
-	GTK_RC_STYLE_UNREF(core->standard_rcstyle)
+	GTK_RC_STYLE_UNREF(core->standard_rcstyle);
 	core->standard_rcstyle = NULL;
-	GTK_RC_STYLE_UNREF(core->lists_rcstyle)
+	GTK_RC_STYLE_UNREF(core->lists_rcstyle);
 	core->lists_rcstyle = NULL;
 
 
 	/* Delete the Pixmaps List */
-	core->pixmap_list = EDVPixmapListClean(core->pixmap_list);
-	g_list_foreach(
-	    core->pixmap_list, (GFunc)EDVPixmapDelete, NULL
-	);
-	g_list_free(core->pixmap_list);
-	core->pixmap_list = NULL;
+	if(core->pixmap_list != NULL)
+	{
+	    core->pixmap_list = EDVPixmapListClean(core->pixmap_list);
+	    if(core->pixmap_list != NULL)
+	    {
+		g_list_foreach(
+		    core->pixmap_list, (GFunc)EDVPixmapDelete, NULL
+		);
+		g_list_free(core->pixmap_list);
+		core->pixmap_list = NULL;
+	    }
+	}
 
 
 	core->pending_flags = 0;
@@ -2589,56 +2703,38 @@ void EDVDelete(
 
 
 	/* Delete other memory on the core */
-	core->geometry_set = FALSE;
-	memset(&core->geometry, 0x00, sizeof(GdkRectangle));
+	EDVClearErrorMessages(core);
 
 	g_free(core->font_name);
-	core->font_name = NULL;
-
 	GDK_FONT_UNREF(core->font);
-	core->font = NULL;
 
 	g_free(core->fg_color_name);
-	core->fg_color_name = NULL;
 	g_free(core->bg_color_name);
-	core->bg_color_name = NULL;
 
 	g_free(core->sfg_color_name);
-	core->sfg_color_name = NULL;
 	g_free(core->sbg_color_name);
-	core->sbg_color_name = NULL;
 
 	g_free(core->cfg_color_name);
-	core->cfg_color_name = NULL;
 	g_free(core->cbg_color_name);
-	core->cbg_color_name = NULL;
 
 	g_free(core->bg_pixmap_path);
-	core->bg_pixmap_path = NULL;
 	g_free(core->sbg_pixmap_path);
-	core->sbg_pixmap_path = NULL;
 
 	g_free(core->wm_name);
-	core->wm_name = NULL;
 	g_free(core->wm_class);
-	core->wm_class = NULL;
 	g_free(core->title);
-	core->title = NULL;
-
 
 	g_free(core->cfg_file);
-	core->cfg_file = NULL;
-
 	g_free(core->prog_file);
-	core->prog_file = NULL;
-
 	g_free(core->home_dir);
-	core->home_dir = NULL;
 
 	g_free(core->effective_user_id_str);
-	core->effective_user_id_str = NULL;
 	g_free(core->effective_user_id_host_str);
-	core->effective_user_id_host_str = NULL;
+
+#if defined(HAVE_IMLIB)
+        /* Imlib does not need to be shut down */
+/*	core->imlib_handle = NULL; */
+#endif	/* HAVE_IMLIB */
 
 	g_free(core);
 }
