@@ -91,8 +91,17 @@ void EDVDeviceListFileSave(
 
 
 /*
- *	Matches the Device from the Devices List who's mount_path 
- *	matches the specified mount_path. 
+ *	Matches the Device by its mount path.
+ *
+ *	The list and total specifies the devices list.
+ *
+ *	The n specifies the device index return value.
+ *
+ *	The mount_path specifies the string describing the full path
+ *	to the directory that the device is mounted on.
+ *
+ *	Returns the device or NULL if the device could not be found or
+ *	an error occured.
  */
 edv_device_struct *EDVDevicesListMatchMountPath(
 	edv_device_struct **list, const gint total,
@@ -113,10 +122,13 @@ edv_device_struct *EDVDevicesListMatchMountPath(
 	for(i = 0; i < total; i++)
 	{
 	    d = list[i];
-	    if((d != NULL) ? STRISEMPTY(d->mount_path) : TRUE)
+	    if(d == NULL)
 		continue;
 
-	    if(!strcmp(d->mount_path, mount_path))
+	    if(d->mount_path == NULL)
+		continue;
+
+	    if(!strcmp((const char *)d->mount_path, (const char *)mount_path))
 	    {
 		if(n != NULL)
 		    *n = i;
@@ -128,8 +140,17 @@ edv_device_struct *EDVDevicesListMatchMountPath(
 }
 
 /*
- *	Matches the Device in the Devices List who's device_path
- *	matches the specified device_path.
+ *	Matches the Device by its device path.
+ *
+ *	The list and total specifies the devices list.
+ *
+ *	The n specifies the device index return value.
+ *
+ *	The device_path specifies the string describing the full path
+ *	to the device.
+ *
+ *	Returns the device or NULL if the device could not be found or
+ *	an error occured.
  */
 edv_device_struct *EDVDevicesListMatchDevicePath(
 	edv_device_struct **list, const gint total,
@@ -150,10 +171,13 @@ edv_device_struct *EDVDevicesListMatchDevicePath(
 	for(i = 0; i < total; i++)
 	{
 	    d = list[i];
-	    if((d != NULL) ? STRISEMPTY(d->device_path) : TRUE)
+	    if(d == NULL)
 		continue;
 
-	    if(!strcmp(d->device_path, device_path))
+	    if(d->device_path == NULL)
+		continue;
+
+	    if(!strcmp((const char *)d->device_path, (const char *)device_path))
 	    {
 		if(n != NULL)
 		    *n = i;
@@ -165,8 +189,17 @@ edv_device_struct *EDVDevicesListMatchDevicePath(
 }
 
 /*
- *	Matches the Device in the Devices List that references the 
- *	object specified by path.
+ *	Matches the Device that the object is on.
+ *
+ *	The list and total specifies the devices list.
+ *
+ *	The n specifies the device index return value.
+ *
+ *	The path specifies the string describing the full path to the
+ *	object.
+ *
+ *	Returns the device that the object reside on or NULL if the
+ *	device could not be found or an error occured.
  */
 edv_device_struct *EDVDevicesListMatchObject(
 	edv_device_struct **list, const gint total,
@@ -174,8 +207,8 @@ edv_device_struct *EDVDevicesListMatchObject(
 	const gchar *path
 )
 {
-	gint i;
-	edv_device_struct *d;
+	gint i, matched_dev_num, matched_mount_path_len;
+	edv_device_struct *d, *matched_d;
 
 	if(n != NULL)
 	    *n = -1;
@@ -183,22 +216,47 @@ edv_device_struct *EDVDevicesListMatchObject(
 	if(STRISEMPTY(path))
 	    return(NULL);
 
-	/* Iterate through the devices list */
-	for(i = 0; i < total; i++)
-	{
-	    d = list[i];
-	    if((d != NULL) ? STRISEMPTY(d->mount_path) : TRUE)
-		continue;
+	matched_mount_path_len = 0;
+        matched_dev_num = -1;
+        matched_d = NULL;
+        for(i = 0; i < total; i++)
+        {
+            d = list[i];
+            if(d == NULL)
+                continue;
 
-	    if(strpfx(path, d->mount_path))
-	    {
-		if(n != NULL)
-		    *n = i;
-		return(d);
-	    }
-	}    
-	 
-	return(NULL);
+            if(d->mount_path == NULL)
+                continue;
+
+            if(strpfx(path, d->mount_path))
+            {
+		/* Was there a previously matched device? */
+		if(matched_d != NULL)
+		{
+		    /* Is this device's mount path longer than
+		     * the previously matched device's mount path?
+		     */
+		    const gint len = STRLEN(d->mount_path);
+		    if(len > matched_mount_path_len)
+		    {
+			matched_mount_path_len = len;
+			matched_dev_num = i;
+			matched_d = d;
+		    }
+		}
+		else
+		{
+		    matched_mount_path_len = STRLEN(d->mount_path);
+		    matched_dev_num = i;
+		    matched_d = d;
+		}
+            }
+        }
+
+	if(n != NULL)
+	    *n = matched_dev_num;
+
+	return(matched_d);
 }
 
 
@@ -234,9 +292,9 @@ void EDVDevicesListLoadFromSystem(
 	/* Open the system's devices list file */
 #if defined(__FreeBSD__)
 	setfsent();
-	fp = FOpen("/etc/fstab", "rb");
+	fp = fopen("/etc/fstab", "rb");
 #elif defined(__SOLARIS__)
-	fp = FOpen("/etc/vfstab", "rb");
+	fp = fopen("/etc/vfstab", "rb");
 #else
 	fp = setmntent("/etc/fstab", "rb");
 #endif
@@ -442,15 +500,15 @@ void EDVDevicesListLoadFromSystem(
 	/* Close the system's devices list file */
 #if defined(__FreeBSD__)
 	endfsent();
-	FClose(fp);
+	fclose(fp);
 #elif defined(__SOLARIS__)
-	FClose(fp);
+	fclose(fp);
 	g_free(vfs_ptr);
 #else
 	endmntent(fp);
 #endif
 
-	/* Report final progress */
+	/* Report the final progress */
 	if(progress_cb != NULL)
 	    progress_cb(
 		client_data, file_size, file_size
@@ -533,7 +591,7 @@ void EDVDevicesListUpdateStats(
 		     * really just the block size), is larger than or
 		     * equal to the base of 1024 bytes per block
 		     */
-		    const gulong block_size = buf.f_bsize;
+		    const gulong block_size = (gulong)buf.f_bsize;
 		    if(block_size >= 1024l)
 		    {
 			const gulong block_div = block_size / 1024l;
@@ -584,8 +642,8 @@ void EDVDevicesListFileOpen(
 	if((list == NULL) || (total == NULL) || STRISEMPTY(filename))
 	    return;
 
-	/* Open devices file */
-	fp = FOpen(filename, "rb");
+	/* Open the devices file for reading */
+	fp = fopen((const char *)filename, "rb");
 	if(fp == NULL)
 	    return;
 
@@ -715,6 +773,19 @@ void EDVDevicesListFileOpen(
 			dev->flags &= ~EDV_DEVICE_UNLISTED;
 		}
 	    }
+	    /* NoScan */
+	    else if(!g_strcasecmp(parm, "NoScan"))
+	    {
+		gint v[1];
+		FGetValuesI(fp, v, 1);
+		if(dev != NULL)
+		{
+		    if(v[0])
+			dev->flags |= EDV_DEVICE_NO_SCAN;
+		    else
+			dev->flags &= ~EDV_DEVICE_NO_SCAN;
+		}
+	    }
 	    /* Name */
 	    else if(!g_strcasecmp(parm, "Name"))
 	    {
@@ -969,8 +1040,8 @@ void EDVDevicesListFileOpen(
 	/* Delete parameter */
 	g_free(parm);
 
-	/* Close devices file */
-	FClose(fp);
+	/* Close the devices file */
+	fclose(fp);
 }
 
 /*
@@ -1005,8 +1076,8 @@ void EDVDeviceListFileSave(
 	    parent_path = NULL;
 	}
 
-	/* Open devices file for writing */
-	fp = FOpen(filename, "wb");
+	/* Open the devices file for writing */
+	fp = fopen((const char *)filename, "wb");
 	if(fp == NULL)
 	    return;
 
@@ -1070,6 +1141,12 @@ void EDVDeviceListFileSave(
 		"\tUnlisted = %i\n",
 		EDV_DEVICE_IS_UNLISTED(dev)
 	    );
+	    /* NoScan */
+	    fprintf(
+		fp,
+		"\tNoScan = %i\n",
+		EDV_DEVICE_IS_NO_SCAN(dev)
+	    );
 	    /* Name */
 	    if(!STRISEMPTY(dev->name))
 		fprintf(
@@ -1225,6 +1302,6 @@ void EDVDeviceListFileSave(
 	    );
 	}
 
-	/* Close devices file */
-	FClose(fp);
+	/* Close the devices file */
+	fclose(fp);
 }
