@@ -26,116 +26,149 @@ static gchar *EDVFindGrepExcerptFile(
 /* Object In Archive Finding */
 static void EDVFindArchiveObjectByNameIterate(
 	edv_core_struct *core,
-	const gchar *arch_path, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *arch_path,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 );
 gint EDVFindArchiveObjectByName(
 	edv_core_struct *core,
-	const gchar *arch_path,	const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *arch_path,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 );
 
 /* Recycled Object Finding */
 static void EDVFindRecycledObjectByContentIterate(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
 	),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 );
 gint EDVFindRecycledObjectByContent(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
-	)
+	),
+	gpointer match_data
 );
 
 static void EDVFindRecycledObjectByNameIterate(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 );
 gint EDVFindRecycledObjectByName(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 );
 
 
 /* Object Finding */
 static void EDVFindObjectByContentIterate(
-	const gchar *path, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
- 	gpointer client_data,
+	const gchar *path,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
 	),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches,
+	gboolean *stop_find
 );
 gint EDVFindObjectByContent(
-	const gchar *start_dir, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *start_dir,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
-	)
+	),
+	gpointer match_data
 );
 
 static void EDVFindObjectByNameIterate(
-	const gchar *path, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *path,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches,
+	gboolean *stop_find
 );
 gint EDVFindObjectByName(
-	const gchar *start_dir, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *start_dir,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 );
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define EDV_FIND_EXCERPT_PAD_MAX	40
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
+
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 #define ISCR(c)		(((c) == '\n') || ((c) == '\r'))
 #define ISBLANK(c)	(((c) == ' ') || ((c) == '\t'))
@@ -153,11 +186,11 @@ static gchar *EDVFindGrepExcerptFile(
 	gboolean case_sensitive, gint *line_index
 )
 {
-	gulong cur_line_start_pos = 0l;
-	const gchar *expre_ptr;
-	gchar *excerpt = NULL;
-	gint c;
 	FILE *fp;
+	gint c;
+	gulong line_start_pos;
+	const gchar *expre_ptr;
+	gchar *excerpt;
 
 	if(line_index != NULL)
 	    *line_index = 0;
@@ -170,8 +203,13 @@ static gchar *EDVFindGrepExcerptFile(
 	if(fp == NULL)
 	    return(NULL);
 
-	/* Begin searching for the given expression in the file */
+	/* Set the expression to the beginning */
 	expre_ptr = expression;
+
+	line_start_pos = 0l;
+	excerpt = NULL;
+
+	/* Search for the expression in the file */
 	while(TRUE)
 	{
 	    /* Get the next character */
@@ -194,23 +232,25 @@ static gchar *EDVFindGrepExcerptFile(
 		if(line_index != NULL)
 		    *line_index = (*line_index) + 1;
 
-		cur_line_start_pos = (gulong)ftell(fp);
+		/* Record the line start position */
+		line_start_pos = (gulong)ftell(fp);
 	    }
 
-	    /* Does this character matches the current character of
+	    /* Does this character matches the current character of the
 	     * expression?
-	     *
-	     * If so then increment the expression pointer
-	     *
-	     * Otherwise reset the expression pointer back to the start
-	     * of the expression
 	     */
-	    if(((gchar)c == *expre_ptr) && (c != '\0'))
+	    if(((gchar)c == *expre_ptr) && ((gchar)c != '\0'))
+	    {
+		/* Increment to the next character in the expression */
 		expre_ptr++;
-	    else
+	    }
+	    /* Reset expression back to the beginning as needed */
+	    else if(expre_ptr != expression)
+	    {
 		expre_ptr = expression;
+	    }
 
-	    /* Matched entire expression?
+	    /* Matched the entire expression?
 	     *
 	     * Test if the expression pointer has been incremented all
 	     * the way to the end which suggests that the entire
@@ -220,62 +260,57 @@ static gchar *EDVFindGrepExcerptFile(
 	    {
 		gchar *s;
 
-		/* Seek to start of the current line */
-		fseek(fp, (long)cur_line_start_pos, SEEK_SET);
+		/* Seek to the start of the line */
+		fseek(fp, (long)line_start_pos, SEEK_SET);
 
 		/* Get current line as a string */
 		s = (gchar *)FGetStringLiteral(fp);
 		if(s != NULL)
 		{
-		    const gint	left_pad_max = 40,
-				exp_len_max = 80;
-		    gchar *s_exp = (case_sensitive) ?
+		    /* Seek to the expression in the excerpt */
+		    gchar *s_exp_start = (case_sensitive) ?
 			(gchar *)strstr((char *)s, (const char *)expression) :
 			(gchar *)strcasestr((char *)s, (const char *)expression);
-		    gchar *s2;
-
-		    /* Sanitize and limit length of string */
-		    for(s2 = s; *s2 != '\0'; s2++)
+		    if(s_exp_start != NULL)
 		    {
-			/* Non-ACSII character? */
-			if(!isascii((int)(*s2)))
-			    *s2 = ' ';		/* Replace with space */
+			const gint	left_pad_max = EDV_FIND_EXCERPT_PAD_MAX,
+					right_pad_max = left_pad_max;
+			gchar	*s2,
+				*s_exp_end = s_exp_start + STRLEN(expression);
 
-			/* Is s2 past the start of the expression? */
-			if((s_exp != NULL) ? (s2 > s_exp) : FALSE)
+			/* Sanitize and limit length of the excerpt */
+			for(s2 = s; *s2 != '\0'; s2++)
 			{
-			    if((gint)(s2 - s_exp) >= exp_len_max)
+			    /* Non-ACSII character? */
+			    if(!isascii((int)(*s2)))
+				*s2 = ' ';	/* Replace with space */
+
+			    /* Exceeded the maximum excerpt length? */
+			    if(s2 > s_exp_end)
 			    {
-				*s2 = '\0';
-				break;
+				if((gint)(s2 - s_exp_end) >= right_pad_max)
+				{
+				    /* Cap the excerpt and break */
+				    *s2 = '\0';
+				    break;
+				}
 			    }
 			}
+
+			/* Copy the sanitized excerpt to the return */
+			g_free(excerpt);
+			if((s_exp_start - s) > left_pad_max)
+			    excerpt = STRDUP(s_exp_start - left_pad_max);
 			else
-			{
-			    if((gint)(s2 - s) >= exp_len_max)
-			    {
-				*s2 = '\0';
-				break;
-			    }
-			}
+			    excerpt = STRDUP(s);
 		    }
 
-		    /* Copy sanitized line as the excerpt */
-		    g_free(excerpt);
-		    if(s_exp != NULL)
-			excerpt = STRDUP(
-			    ((s_exp - s) > left_pad_max) ?
-				(s_exp - left_pad_max) : s
-			);
-		    else
-			excerpt = STRDUP(s);
-
 		    g_free(s);
 		}
 
-		break;	/* Break after first match */
+		break;	/* Break after the first match */
 
-	    }	/* Matched entire expression? */
+	    }	/* Matched the entire expression? */
 	}
 
 	/* Close the file */
@@ -291,14 +326,17 @@ static gchar *EDVFindGrepExcerptFile(
  */
 static void EDVFindArchiveObjectByNameIterate(
 	edv_core_struct *core,
-	const gchar *arch_path, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *arch_path,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 )
 {
+	gint i, nobjs;
 	GList *glist, *obj_list;
 	edv_archive_object_struct *obj;
 
@@ -316,7 +354,11 @@ static void EDVFindArchiveObjectByNameIterate(
 	);
 
 	/* Iterate through all the objects in the archive */
-	for(glist = obj_list; glist != NULL; glist = g_list_next(glist))
+	nobjs = g_list_length(obj_list);
+	for(glist = obj_list, i = 0;
+	    glist != NULL;
+	    glist = g_list_next(glist), i++
+	)
 	{
 	    obj = EDV_ARCHIVE_OBJECT(glist->data);
 	    if(obj == NULL)
@@ -329,38 +371,55 @@ static void EDVFindArchiveObjectByNameIterate(
 	    /* Report progress */
 	    if(progress_cb != NULL)
 	    {
-		if(progress_cb(obj->name, -1.0f, client_data))
+		if(progress_cb(
+		    obj->name,
+		    (gfloat)i / (gfloat)nobjs,
+		    progress_data
+		))
 		{
 		    *stop_find = TRUE;
 		    continue;
 		}
 	    }
 
-	    /* Does the object in archive has a name? */
+	    /* Does the object in the archive have a name? */
 	    if(!STRISEMPTY(obj->name))
 	    {
-		/* Do match on object's name */
 		gchar *s = STRDUP(obj->name);
-
-		/* If not performing case sensitive search then make
-		 * the name uppercase
-		 */
 		if(!case_sensitive)
-		    strtoupper((char *)s);
+		    g_strup(s);
 
-		/* Check if the object's name matches the expression */
-		if(!fnmatch((const char *)expression, (char *)s, 0))
+		/* Names match? */
+		if(!fnmatch((const char *)expression, (const char *)s, 0))
 		{
-		    /* Got match, increment number of matches and call
-		     * match callback
-		     */
-		    *total_matches = (*total_matches) + 1;
+		    /* Convert the object in the archive stats */
+		    struct stat *lstat_buf = (struct stat *)g_malloc0(
+			sizeof(struct stat)
+		    );
+		    if(lstat_buf != NULL)
+		    {
+			lstat_buf->st_mode = 
+ EDVObjectGetTypeFromEDVType(obj->type) |
+ EDVObjectGetPermissionsFromEDVPermissions(obj->permissions)
+			;
+			lstat_buf->st_size = (size_t)obj->size; 
+			lstat_buf->st_atime = (time_t)obj->access_time;
+			lstat_buf->st_mtime = (time_t)obj->modify_time;
+			lstat_buf->st_ctime = (time_t)obj->change_time;
+		    }
+
+		    /* Increment the match count */
+		    *nmatches = (*nmatches) + 1;
+
+		    /* Repor the match */
 		    if(match_cb != NULL)
 			match_cb(
 			    obj->full_path,	/* Path */
-			    NULL,		/* No stats */
-			    client_data		/* Data */
+			    lstat_buf,		/* Stats */
+			    match_data		/* Data */
 			);
+
+		    g_free(lstat_buf);
 		}
 
 		g_free(s);	/* Delete copy of object's name */
@@ -368,8 +427,11 @@ static void EDVFindArchiveObjectByNameIterate(
 	}
 
 	/* Delete the archive object stats */
-	g_list_foreach(obj_list, (GFunc)EDVArchObjectDelete, NULL);
-	g_list_free(obj_list);
+	if(obj_list != NULL)
+	{
+	    g_list_foreach(obj_list, (GFunc)EDVArchObjectDelete, NULL);
+	    g_list_free(obj_list);
+	}
 }
 
 /*
@@ -387,18 +449,18 @@ static void EDVFindArchiveObjectByNameIterate(
  */
 gint EDVFindArchiveObjectByName(
 	edv_core_struct *core,
-	const gchar *arch_path, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *arch_path,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 )
 {
-	gint i;
+	gboolean stop_find;
+	gint nmatches;
 	gchar *lexpression;
-	gboolean *stop_find;
-	gint *total_matches;
-
 
 	if((core == NULL) || STRISEMPTY(arch_path) ||
 	   STRISEMPTY(expression)
@@ -413,35 +475,23 @@ gint EDVFindArchiveObjectByName(
 	 */
 	lexpression = STRDUP(expression);
 	if(!case_sensitive)
-	    strtoupper((char *)lexpression);
-
-	/* Allocate total number of matches variable for passing to
-	 * iteration calls
-	 */
-	stop_find = (gboolean *)g_malloc(sizeof(gboolean));
-	*stop_find = FALSE;
-
-	total_matches = (gint *)g_malloc(sizeof(gint));
-	*total_matches = 0;
+	    g_strup(lexpression);
 
 	/* Begin search */
+	nmatches = 0;
+	stop_find = FALSE;
 	EDVFindArchiveObjectByNameIterate(
 	    core,
 	    arch_path, lexpression,
 	    case_sensitive,
-	    client_data,
-	    progress_cb, match_cb,
-	    total_matches, stop_find
+	    progress_cb, progress_data,
+	    match_cb, match_data,
+	    &nmatches, &stop_find
 	);
 
-	/* Record total matches */
-	i = *total_matches;
-
 	g_free(lexpression);
-	g_free(stop_find);
-	g_free(total_matches);
 
-	return(i);
+	return(nmatches);
 }
 
 
@@ -450,18 +500,21 @@ gint EDVFindArchiveObjectByName(
  *	iteration on the given path.
  */
 static void EDVFindRecycledObjectByContentIterate(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
 	),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 )
 {
+	gint i, nobjs;
 	gchar *recycled_dir;
 	edv_recbin_index_struct *rbi_ptr;
 	edv_recycled_object_struct *obj;
@@ -474,16 +527,21 @@ static void EDVFindRecycledObjectByContentIterate(
 	 */
 	recycled_dir = g_dirname(recycled_index_file);
 
+	nobjs = EDVRecBinIndexGetTotal(recycled_index_file);
+
 	/* Get recycled objects directory index pointer used to iterate
 	 * through all recycled objects
 	 */
+	i = 0;
 	rbi_ptr = EDVRecBinIndexOpen(recycled_index_file);
 	while(!EDVRecBinIndexNext(rbi_ptr))
 	{
-	    /* Get recycled object */
 	    obj = rbi_ptr->obj;
 	    if(obj == NULL)
+	    {
+		i++;
 		continue;
+	    }
 
 	    if(*stop_find)
 		break;
@@ -491,9 +549,14 @@ static void EDVFindRecycledObjectByContentIterate(
 	    /* Report progress */
 	    if(progress_cb != NULL)
 	    {
-		if(progress_cb(obj->name, -1.0f, client_data))
+		if(progress_cb(
+		    obj->name,
+		    (gfloat)i / (gfloat)nobjs,
+		    progress_data
+		))
 		{
 		    *stop_find = TRUE;
+		    i++;
 		    continue;
 		}
 	    }
@@ -501,6 +564,7 @@ static void EDVFindRecycledObjectByContentIterate(
 	    /* Do match */
 	    if(TRUE)
 	    {
+		/* Contents match? */
 		gint line_index;
 		gchar	*full_path = g_strdup_printf(
 		    "%s%c%i",
@@ -512,32 +576,45 @@ static void EDVFindRecycledObjectByContentIterate(
 		);
 		if(excerpt != NULL)
 		{
-		    gchar index_str[80];
-		    struct stat lstat_buf;
-
-		    /* Got match, increment match count and call match
-		     * callback. The path passed should be the object's
-		     * name which is a string specifying a recycled object
-		     * index number
+		    /* Got match, increment the match count and report
+		     * the match
 		     */
-		    *total_matches = (*total_matches) + 1;
-		    g_snprintf(
-			index_str, sizeof(index_str),
-			"%i",
-			obj->index
+		    gchar *index_str = g_strdup_printf("%u", obj->index);
+
+		    /* Convert the recycled object's stats */
+		    struct stat *lstat_buf = (struct stat *)g_malloc0(
+			sizeof(struct stat)
 		    );
-		    if((match_cb != NULL) &&
-		       !lstat((char *)full_path, &lstat_buf)
-		    )
+		    if(lstat_buf != NULL)
+		    {
+			lstat_buf->st_mode =
+ EDVObjectGetTypeFromEDVType(obj->type) |
+ EDVObjectGetPermissionsFromEDVPermissions(obj->permissions)
+			;
+			lstat_buf->st_size = (size_t)obj->size;
+			lstat_buf->st_uid = (uid_t)obj->owner_id;
+			lstat_buf->st_gid = (gid_t)obj->group_id;
+			lstat_buf->st_atime = (time_t)obj->access_time;
+			lstat_buf->st_mtime = (time_t)obj->modify_time;
+			lstat_buf->st_ctime = (time_t)obj->change_time;
+		    }
+
+		    /* Increment the match count */
+		    *nmatches = (*nmatches) + 1;
+
+		    /* Report the match */
+		    if(match_cb != NULL)
 			match_cb(
 			    index_str,		/* Path as index string */
-			    &lstat_buf,		/* Local Stats */
+			    lstat_buf,		/* Stats */
 			    excerpt,		/* Excerpt */
 			    line_index,		/* Line Index */
-			    client_data		/* Data */
+			    match_data		/* Data */
 			); 
 
-		    g_free(excerpt);	/* Delete copy of excerpt */
+		    g_free(lstat_buf);
+		    g_free(index_str);
+		    g_free(excerpt);
 		}
 
 		g_free(full_path);	/* Delete full path */
@@ -564,21 +641,22 @@ static void EDVFindRecycledObjectByContentIterate(
  *      Returns the number of occurances found.
  */
 gint EDVFindRecycledObjectByContent(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
-	)
+	),
+	gpointer match_data
 )
 {
-	gint i;
+	gboolean stop_find;
+	gint nmatches;
 	gchar *lexpression;
-	gboolean *stop_find;
-	gint *total_matches;
 
 	if(STRISEMPTY(recycled_index_file) || STRISEMPTY(expression))
 	    return(0);
@@ -588,34 +666,22 @@ gint EDVFindRecycledObjectByContent(
 	 */
 	lexpression = STRDUP(expression);
 	if(!case_sensitive)
-	    strtoupper((char *)lexpression);
-
-	/* Allocate total number of matches variable for passing to
-	 * iteration calls
-	 */
-	stop_find = (gboolean *)g_malloc(sizeof(gboolean));
-	*stop_find = FALSE;
-
-	total_matches = (gint *)g_malloc(sizeof(gint));
-	*total_matches = 0;
+	    g_strup(lexpression);
 
 	/* Begin search */
+	nmatches = 0;
+	stop_find = FALSE;
 	EDVFindRecycledObjectByContentIterate(
 	    recycled_index_file, lexpression,
 	    case_sensitive,
-	    client_data,
-	    progress_cb, match_cb,
-	    total_matches, stop_find
+	    progress_cb, progress_data,
+	    match_cb, match_data,
+	    &nmatches, &stop_find
 	);
 
-	/* Record total matches */
-	i = *total_matches;
-
 	g_free(lexpression);
-	g_free(stop_find);
-	g_free(total_matches);
 
-	return(i);
+	return(nmatches);
 }
 
 
@@ -624,14 +690,17 @@ gint EDVFindRecycledObjectByContent(
  *	on the given path.
  */
 static void EDVFindRecycledObjectByNameIterate(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches, gboolean *stop_find
 )
 {
+	gint i, nobjs;
 	gchar *recycled_dir;
 	edv_recbin_index_struct *rbi_ptr;
 	const edv_recycled_object_struct *obj;
@@ -644,16 +713,21 @@ static void EDVFindRecycledObjectByNameIterate(
 	 */
 	recycled_dir = g_dirname(recycled_index_file);
 
+	nobjs = EDVRecBinIndexGetTotal(recycled_index_file);
+
 	/* Get recycled objects directory index pointer used to iterate
 	 * through all recycled objects
 	 */
+	i = 0;
 	rbi_ptr = EDVRecBinIndexOpen(recycled_index_file);
 	while(!EDVRecBinIndexNext(rbi_ptr))
 	{
-	    /* Get recycled object */
 	    obj = rbi_ptr->obj;
 	    if(obj == NULL)
+	    {
+		i++;
 		continue;
+	    }
 
 	    if(*stop_find)
 		break;
@@ -661,9 +735,14 @@ static void EDVFindRecycledObjectByNameIterate(
 	    /* Report progress */
 	    if(progress_cb != NULL)
 	    {
-		if(progress_cb(obj->name, -1.0f, client_data))
+		if(progress_cb(
+		    obj->name,
+		    (gfloat)i / (gfloat)nobjs,
+		    progress_data
+		))
 		{
 		    *stop_find = TRUE;
+		    i++;
 		    continue;
 		}
 	    }
@@ -674,42 +753,47 @@ static void EDVFindRecycledObjectByNameIterate(
 		gchar *s = STRDUP(obj->name);
 
 		if(!case_sensitive)
-		    strtoupper((char *)s);
+		    g_strup(s);
 
-		if(!fnmatch((const char *)expression, (char *)s, 0))
+		/* Names match? */
+		if(!fnmatch((const char *)expression, (const char *)s, 0))
 		{
-		    gchar *full_path, index_str[80];
-
-		    /* Got match, increment match count and call match
-		     * callback
-		     *
-		     * The path passed should be the object's name
-		     * which is a string specifying a recycled object
-		     * index number
+		    /* Got match, increment the match count and report
+		     * the match
 		     */
-		    *total_matches = (*total_matches) + 1;
+		    gchar *index_str = g_strdup_printf("%u", obj->index);
 
-		    g_snprintf(
-			index_str, sizeof(index_str),
-			"%i",
-			obj->index
+		    /* Convert the recycled object's stats */
+		    struct stat *lstat_buf = (struct stat *)g_malloc0(
+			sizeof(struct stat)
 		    );
-		    full_path = g_strdup_printf(
-			"%s%c%i",
-			recycled_dir, G_DIR_SEPARATOR, obj->index
-		    );
-		    if((full_path != NULL) && (match_cb != NULL))
+		    if(lstat_buf != NULL)
 		    {
-			struct stat lstat_buf;
-			if(!lstat((char *)full_path, &lstat_buf))
-			    match_cb(
-				index_str,
-				&lstat_buf,
-				client_data
-			    );
+			lstat_buf->st_mode = 
+ EDVObjectGetTypeFromEDVType(obj->type) |
+ EDVObjectGetPermissionsFromEDVPermissions(obj->permissions)
+			;
+			lstat_buf->st_size = (size_t)obj->size;
+			lstat_buf->st_uid = (uid_t)obj->owner_id;
+			lstat_buf->st_gid = (gid_t)obj->group_id;
+			lstat_buf->st_atime = (time_t)obj->access_time;
+			lstat_buf->st_mtime = (time_t)obj->modify_time;
+			lstat_buf->st_ctime = (time_t)obj->change_time;
 		    }
 
-		    g_free(full_path);
+		    /* Increment the match count */
+		    *nmatches = (*nmatches) + 1;
+
+		    /* Report the match */
+		    if(match_cb != NULL)
+			match_cb(
+			    index_str,		/* Index as path */
+			    lstat_buf,		/* Stats */
+			    match_data		/* Data */
+			);
+
+		    g_free(lstat_buf);
+		    g_free(index_str);
 		}
 
 		g_free(s);
@@ -736,21 +820,22 @@ static void EDVFindRecycledObjectByNameIterate(
  *      Returns the number of occurances found.
  */
 gint EDVFindRecycledObjectByName(
-	const gchar *recycled_index_file, const gchar *expression,
-	gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *recycled_index_file,
+	const gchar *expression,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 )
 {
-	gint i;
+	gboolean stop_find;
+	gint nmatches;
 	gchar *lexpression;
-	gboolean *stop_find;
-	gint *total_matches;
 
 	if(STRISEMPTY(recycled_index_file) || STRISEMPTY(expression))
 	    return(0);
-	if(!strcmp(expression, "*"))
+	if(!strcmp((const char *)expression, "*"))
 	    return(0);
 
 	/* Convert expression to upper case if not matching case
@@ -758,34 +843,22 @@ gint EDVFindRecycledObjectByName(
 	 */
 	lexpression = STRDUP(expression);
 	if(!case_sensitive)
-	    strtoupper((char *)lexpression);
-
-	/* Allocate total number of matches variable for passing to
-	 * iteration calls
-	 */
-	stop_find = (gboolean *)g_malloc(sizeof(gboolean));
-	*stop_find = FALSE;
-
-	total_matches = (gint *)g_malloc(sizeof(gint));
-	*total_matches = 0;
+	    g_strup(lexpression);
 
 	/* Begin search */
+	nmatches = 0;
+	stop_find = FALSE;
 	EDVFindRecycledObjectByNameIterate(
 	    recycled_index_file, lexpression,
 	    case_sensitive,
-	    client_data,
-	    progress_cb, match_cb,
-	    total_matches, stop_find
+	    progress_cb, progress_data,
+	    match_cb, match_data,
+	    &nmatches, &stop_find
 	);
 
-	/* Record total matches */
-	i = *total_matches;
-
 	g_free(lexpression);
-	g_free(stop_find);
-	g_free(total_matches);
 
-	return(i);
+	return(nmatches);
 }
 
 
@@ -795,16 +868,21 @@ gint EDVFindRecycledObjectByName(
  *	recurisive is TRUE.
  */
 static void EDVFindObjectByContentIterate(
-	const gchar *path, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *path,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
 	),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches,
+	gboolean *stop_find
 )
 {
 	gint strc;
@@ -817,7 +895,7 @@ static void EDVFindObjectByContentIterate(
 	strv = (gchar **)GetDirEntNames2((const char *)path, (int *)&strc);
 	if(strv != NULL)
 	{
-	    gint i, result;
+	    gint i, status;
 	    const gchar *name;
 	    gchar *full_path = NULL;
 	    struct stat stat_buf;
@@ -859,7 +937,11 @@ static void EDVFindObjectByContentIterate(
 		/* Report progress */
 		if(progress_cb != NULL)
 		{
-		    if(progress_cb(full_path, -1.0f, client_data))
+		    if(progress_cb(
+			full_path,
+			recursive ? -1.0f : ((gfloat)i / (gfloat)strc),
+			progress_data
+		    ))
 		    {
 			*stop_find = TRUE;
 			DO_FREE_CONTINUE
@@ -880,17 +962,17 @@ static void EDVFindObjectByContentIterate(
 			struct stat lstat_buf;
 
 			/* Increment number of matches */
-			*total_matches = (*total_matches) + 1;
+			*nmatches = (*nmatches) + 1;
 
 			/* Get local stats and call match callback */
-			result = (gint)lstat((char *)full_path, &lstat_buf);
-			if((match_cb != NULL) && !result)
+			status = (gint)lstat((const char *)full_path, &lstat_buf);
+			if((match_cb != NULL) && (status == 0))
 			    match_cb(
 				full_path,	/* Path */
 				&lstat_buf,	/* Local Stats */
 				excerpt,	/* Excerpt */
 				line_index,	/* Line Index */
-				client_data	/* Data */
+				match_data	/* Data */
 			    );
 
 			g_free(excerpt);
@@ -910,14 +992,14 @@ static void EDVFindObjectByContentIterate(
 		{
 		    struct stat lstat_buf;
 
-		    result = (gint)lstat(
-			(char *)full_path, &lstat_buf
+		    status = (gint)lstat(
+			(const char *)full_path, &lstat_buf
 		    );
 #ifdef S_ISLNK
 		    /* Skip links that go to directories in order to
 		     * avoid possible infinate recusion
 		     */
-		    if(!result)
+		    if(status == 0)
 		    {
 			if(S_ISLNK(lstat_buf.st_mode) && !follow_links)
 			    DO_FREE_CONTINUE
@@ -927,9 +1009,9 @@ static void EDVFindObjectByContentIterate(
 		    EDVFindObjectByContentIterate(
 			full_path, expression,
 			recursive, follow_links, case_sensitive,
-			client_data,
-			progress_cb, match_cb,
-			total_matches, stop_find
+			progress_cb, progress_data,
+			match_cb, match_data,
+			nmatches, stop_find
 		    );
 		}
 
@@ -956,21 +1038,24 @@ static void EDVFindObjectByContentIterate(
  *	Returns the number of occurances found.
  */
 gint EDVFindObjectByContent(
-	const gchar *start_dir, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *start_dir,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(
 		const gchar *, const struct stat *,
 		const gchar *, gint,
 		gpointer
-	)
+	),
+	gpointer match_data
 )
 {
-	gint i;
+	gboolean stop_find;
+	gint nmatches;
 	gchar *lexpression;
-	gboolean *stop_find;
-	gint *total_matches;
 
 	if(STRISEMPTY(start_dir) || STRISEMPTY(expression))
 	    return(0);
@@ -980,34 +1065,22 @@ gint EDVFindObjectByContent(
 	 */
 	lexpression = STRDUP(expression);
 	if(!case_sensitive)
-	    strtoupper((char *)lexpression);
-
-	/* Allocate total number of matches variable for passing to
-	 * iteration calls
-	 */
-	stop_find = (gboolean *)g_malloc(sizeof(gboolean));
-	*stop_find = FALSE;
-
-	total_matches = (gint *)g_malloc(sizeof(gint));
-	*total_matches = 0;
+	    g_strup(lexpression);
 
 	/* Begin search */
+	nmatches = 0;
+	stop_find = FALSE;
 	EDVFindObjectByContentIterate(
 	    start_dir, lexpression,
 	    recursive, follow_links, case_sensitive,
-	    client_data,
-	    progress_cb, match_cb,
-	    total_matches, stop_find
+	    progress_cb, progress_data,
+	    match_cb, match_data,
+	    &nmatches, &stop_find
 	);
 
-	/* Record total matches */
-	i = *total_matches;
-
 	g_free(lexpression);
-	g_free(stop_find);
-	g_free(total_matches);
 
-	return(i);
+	return(nmatches);
 }
 
 
@@ -1017,12 +1090,17 @@ gint EDVFindObjectByContent(
  *	recurisive is TRUE.
  */
 static void EDVFindObjectByNameIterate(
-	const gchar *path, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *path,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
+	gpointer progress_data,
 	void (*match_cb)(const gchar *, const struct stat *, gpointer),
-	gint *total_matches, gboolean *stop_find
+	gpointer match_data,
+	gint *nmatches,
+	gboolean *stop_find
 )
 {
 	gint strc;
@@ -1035,7 +1113,7 @@ static void EDVFindObjectByNameIterate(
 	strv = (gchar **)GetDirEntNames2((const char *)path, (int *)&strc);
 	if(strv != NULL)
 	{
-	    gint i, result;
+	    gint i, status;
 	    const gchar *name;
 	    gchar *full_path = NULL;
 	    struct stat stat_buf;
@@ -1077,7 +1155,11 @@ static void EDVFindObjectByNameIterate(
 		/* Report progress */
 		if(progress_cb != NULL)
 		{
-		    if(progress_cb(full_path, -1.0f, client_data))
+		    if(progress_cb(
+			full_path,
+			recursive ? -1.0f : ((gfloat)i / (gfloat)strc),
+			progress_data
+		    ))
 		    {
 			*stop_find = TRUE;
 			DO_FREE_CONTINUE
@@ -1090,23 +1172,23 @@ static void EDVFindObjectByNameIterate(
 		    gchar *s = STRDUP(name);
 
 		    if(!case_sensitive)
-			strtoupper((char *)s);
+			g_strup(s);
 
-		    if(!fnmatch((const char *)expression, (char *)s, 0))
+		    if(!fnmatch((const char *)expression, (const char *)s, 0))
 		    {
 			/* Got match */
 			struct stat lstat_buf;
 
 			/* Increment number of matches */
-			*total_matches = (*total_matches) + 1;
+			*nmatches = (*nmatches) + 1;
 
 			/* Get local stats and call match callback */
-			result = (gint)lstat((char *)full_path, &lstat_buf);
-			if((match_cb != NULL) && !result)
+			status = (gint)lstat((const char *)full_path, &lstat_buf);
+			if((match_cb != NULL) && (status == 0))
 			    match_cb(
 				full_path,	/* Path */
 				&lstat_buf,	/* Stats */
-				client_data	/* Data */
+				match_data	/* Data */
 			    );
 		    }
 
@@ -1126,14 +1208,14 @@ static void EDVFindObjectByNameIterate(
 		{
 		    struct stat lstat_buf;
 
-		    result = (gint)lstat(
+		    status = (gint)lstat(
 			(char *)full_path, &lstat_buf
 		    );
 #ifdef S_ISLNK
 		    /* Skip links that go to directories in order to
 		     * avoid possible infinate recusion
 		     */
-		    if(!result)
+		    if(status == 0)
 		    {
 			if(S_ISLNK(lstat_buf.st_mode) && !follow_links)
 			    DO_FREE_CONTINUE
@@ -1143,9 +1225,9 @@ static void EDVFindObjectByNameIterate(
 		    EDVFindObjectByNameIterate(
 			full_path, expression,
 			recursive, follow_links, case_sensitive,
-			client_data,
-			progress_cb, match_cb,
-			total_matches, stop_find
+			progress_cb, progress_data,
+			match_cb, match_data,
+			nmatches, stop_find
 		    );
 		}
 
@@ -1171,17 +1253,20 @@ static void EDVFindObjectByNameIterate(
  *	Returns the number of occurances found.
  */
 gint EDVFindObjectByName(
-	const gchar *start_dir, const gchar *expression,
-	gboolean recursive, gboolean follow_links, gboolean case_sensitive,
-	gpointer client_data,
+	const gchar *start_dir,
+	const gchar *expression,
+	const gboolean recursive,
+	const gboolean follow_links,
+	const gboolean case_sensitive,
 	gint (*progress_cb)(const gchar *, gfloat, gpointer),
-	void (*match_cb)(const gchar *, const struct stat *, gpointer)
+	gpointer progress_data,
+	void (*match_cb)(const gchar *, const struct stat *, gpointer),
+	gpointer match_data
 )
 {
-	gint i;
+	gboolean stop_find;
+	gint nmatches;
 	gchar *lexpression;
-	gboolean *stop_find;
-	gint *total_matches;
 
 	if(STRISEMPTY(start_dir) || STRISEMPTY(expression))
 	    return(0);
@@ -1193,32 +1278,20 @@ gint EDVFindObjectByName(
 	 */
 	lexpression = STRDUP(expression);
 	if(!case_sensitive)
-	    strtoupper((char *)lexpression);
-
-	/* Allocate total number of matches variable for passing to
-	 * iteration calls
-	 */
-	stop_find = (gboolean *)g_malloc(sizeof(gboolean));
-	*stop_find = FALSE;
-
-	total_matches = (gint *)g_malloc(sizeof(gint));
-	*total_matches = 0;
+	    g_strup(lexpression);
 
 	/* Begin search */
+	nmatches = 0;
+	stop_find = FALSE;
 	EDVFindObjectByNameIterate(
 	    start_dir, lexpression,
 	    recursive, follow_links, case_sensitive,
-	    client_data,
-	    progress_cb, match_cb,
-	    total_matches, stop_find
+	    progress_cb, progress_data,
+	    match_cb, match_data,
+	    &nmatches, &stop_find
 	);
 
-	/* Record total matches */
-	i = *total_matches;
-
 	g_free(lexpression);
-	g_free(stop_find);
-	g_free(total_matches);
 
-	return(i);
+	return(nmatches);
 }
