@@ -47,8 +47,8 @@ void EDVFindBarSetSearch(
 edv_findbar_operation EDVFindBarCurrentFindOP(edv_findbar_struct *fb);
 
 edv_findbar_struct *EDVFindBarNew(
-	gpointer core, GtkWidget *parent,
-	edv_findbar_role role,
+	edv_core_struct *core, GtkWidget *parent,
+	const edv_findbar_role role,
 	const gchar *(*get_location_cb)(
 		edv_findbar_struct *, gpointer
 	),
@@ -248,9 +248,8 @@ static void EDVFindBarFindCB(GtkWidget *widget, gpointer data)
 		    FALSE,			/* Do not recurse */
 		    FALSE,			/* Do not follow links */
 		    case_sensitive,
-		    fb,
-		    EDVFindBarProgressCB,
-		    EDVFindBarMatchCB
+		    EDVFindBarProgressCB, fb,
+		    EDVFindBarMatchCB, fb
 		);
 		break;
 
@@ -259,9 +258,8 @@ static void EDVFindBarFindCB(GtkWidget *widget, gpointer data)
 		    location,		/* Recycled objects index file */
  		    search_string,
 		    case_sensitive,
-		    fb,
-		    EDVFindBarProgressCB,
-		    EDVFindBarMatchCB
+		    EDVFindBarProgressCB, fb,
+		    EDVFindBarMatchCB, fb
 		);
 		break;
 
@@ -271,9 +269,8 @@ static void EDVFindBarFindCB(GtkWidget *widget, gpointer data)
 		    location,		/* Archive */
 		    search_string,
 		    case_sensitive,
-		    fb,
-		    EDVFindBarProgressCB,
-		    EDVFindBarMatchCB
+		    EDVFindBarProgressCB, fb,
+		    EDVFindBarMatchCB, fb
 		);
 		break;
 	    }
@@ -284,13 +281,13 @@ static void EDVFindBarFindCB(GtkWidget *widget, gpointer data)
 	    {
 	      case EDV_FINDBAR_ROLE_DISK_OBJECT:
 		total_matches = EDVFindObjectByContent(
-		    location, search_string,
+		    location,
+		    search_string,
 		    FALSE,			/* Do not recurse */
 		    FALSE,			/* Do not follow links */
 		    case_sensitive,		/* Case sensitive */
-		    fb,
-		    EDVFindBarProgressCB,
-		    EDVFindBarMatchExcerptCB
+		    EDVFindBarProgressCB, fb,
+		    EDVFindBarMatchExcerptCB, fb
 		);
 		break;
 
@@ -299,9 +296,8 @@ static void EDVFindBarFindCB(GtkWidget *widget, gpointer data)
 		    location,		/* Recycled objects index file */
 		    search_string,
 		    case_sensitive,
-		    fb,
-		    EDVFindBarProgressCB,
-		    EDVFindBarMatchExcerptCB
+		    EDVFindBarProgressCB, fb,
+		    EDVFindBarMatchExcerptCB, fb
 		);
 		break;
 
@@ -532,8 +528,8 @@ edv_findbar_operation EDVFindBarCurrentFindOP(edv_findbar_struct *fb)
  *	Create a new find bar.
  */
 edv_findbar_struct *EDVFindBarNew(
-	gpointer core, GtkWidget *parent,
-	edv_findbar_role role,
+	edv_core_struct *core, GtkWidget *parent,
+	const edv_findbar_role role,
 	const gchar *(*get_location_cb)(
 		edv_findbar_struct *, gpointer
 	),
@@ -557,16 +553,20 @@ edv_findbar_struct *EDVFindBarNew(
 	gint i;
 	gpointer combo_rtn;
 	GList *glist;
-	GtkWidget *w, *parent2, *parent3;
+	GtkWidget *w, *parent2, *parent3, *toplevel;
 	GtkCombo *combo;
 	pulist_struct *pulist;
 	pulistbox_struct *pulistbox;
-	edv_findbar_struct *fb = EDV_FINDBAR(
-	    g_malloc0(sizeof(edv_findbar_struct))
-	);
-	if(fb == NULL)
-	    return(fb);
+	edv_findbar_struct *fb;
 
+	if(core == NULL)
+	    return(NULL);
+
+	fb = EDV_FINDBAR(g_malloc0(sizeof(edv_findbar_struct)));
+	if(fb == NULL)
+	    return(NULL);
+
+	fb->toplevel = toplevel = gtk_hbox_new(FALSE, border_minor);
 	fb->freeze_count = 0;
 	fb->stop_count = 0;
 	fb->core = core;
@@ -579,13 +579,15 @@ edv_findbar_struct *EDVFindBarNew(
 	fb->status_progress_cb = status_progress_cb;
 	fb->data = data;
 
-	/* Create the toplevel GtkHBox */
-	fb->toplevel = w = gtk_hbox_new(FALSE, border_minor);
-	gtk_container_border_width(GTK_CONTAINER(w), 2);
+	fb->freeze_count++;
+
+	/* Toplevel GtkHBox */
+	w = toplevel;
+	gtk_container_border_width(GTK_CONTAINER(w), border_minor);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	parent2 = w;
 
-	/* GtkHBox for the find operation */
+	/* Find Operation GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -609,6 +611,7 @@ edv_findbar_struct *EDVFindBarNew(
 #else
 "Find"
 #endif
+	    ":"
 	);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -618,7 +621,8 @@ edv_findbar_struct *EDVFindBarNew(
 	    150, -1
 	);
 	pulist = PUListBoxGetPUList(pulistbox);
-	PUListBoxMap(pulistbox);
+        if(core->run_flags & EDV_RUN_SAFE_MODE)
+            PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
 #if defined(PROG_LANGUAGE_SPANISH)
 	i = PUListAddItem(pulist, "Por Nombre de archivo");
 	PUListSetItemData(pulist, i, (gpointer)EDV_FINDBAR_FIND_OBJECT_NAME);
@@ -661,8 +665,9 @@ edv_findbar_struct *EDVFindBarNew(
 	PUListSetItemData(pulist, i, (gpointer)EDV_FINDBAR_FIND_OBJECT_CONTENT);
 #endif
 	PUListBoxSetLinesVisible(pulistbox, i + 1);
+	PUListBoxMap(pulistbox);
 
-	/* Search string combo */
+	/* Search string GtkCombo */
 	glist = NULL;
 	w = GUIComboCreate(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -691,23 +696,20 @@ edv_findbar_struct *EDVFindBarNew(
 	    EDVFindBarFindCB,
 	    NULL
 	);
-	fb->search_combo = (GtkWidget *)combo_rtn;
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 
-	w = fb->search_combo;
+	w = (GtkWidget *)combo_rtn;
+	fb->search_combo = w;
 	combo = GTK_COMBO(w);
 	gtk_combo_set_case_sensitive(combo, TRUE);
 
 	w = combo->entry;
-	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_FOCUS | GTK_CAN_DEFAULT);
 	EDVEntrySetDND(core, w);
-#if 0
-	if(search_string != NULL)
-	    gtk_entry_set_text(GTK_ENTRY(w), search_string);
-#endif
+	GUIEditableEndowPopupMenu(w, 0);
 
-	/* Case sensitive check */
+
+	/* Case sensitive GtkCheckButton */
 	fb->case_sensitive_check = w = gtk_check_button_new_with_label(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Sensible May/min."
@@ -737,7 +739,8 @@ edv_findbar_struct *EDVFindBarNew(
 	    GTK_OBJECT(w), "leave_notify_event",
 	    GTK_SIGNAL_FUNC(EDVFindBarCrossingCB), fb
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Verifique esto para discriminar Mayusculas y minusculas"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -780,7 +783,8 @@ edv_findbar_struct *EDVFindBarNew(
 	    GTK_OBJECT(w), "leave_notify_event",
 	    GTK_SIGNAL_FUNC(EDVFindBarCrossingCB), fb
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Pare el procedimiento actual del hallazgo"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -805,6 +809,8 @@ edv_findbar_struct *EDVFindBarNew(
 
 	EDVFindBarUpdateMenus(fb);
 
+	fb->freeze_count--;
+
 	return(fb);
 }
 
@@ -821,14 +827,12 @@ void EDVFindBarUpdateMenus(edv_findbar_struct *fb)
 	    return;
 
 	core = fb->core;
-	if(core == NULL)
-	    return;
 
-#define DO_SHOW		\
+#define DO_SHOW				\
 { if(w != NULL) gtk_widget_show(w); }
-#define DO_HIDE		\
+#define DO_HIDE				\
 { if(w != NULL) gtk_widget_hide(w); }
-#define DO_SET_CHECK_STATE	\
+#define DO_SET_CHECK_STATE		\
 { GUIMenuItemSetCheck(w, state, FALSE); }
 
 	/* Update buttons */
@@ -845,7 +849,6 @@ void EDVFindBarUpdateMenus(edv_findbar_struct *fb)
 
 #undef DO_SHOW
 #undef DO_HIDE
-#undef DO_SET_SENSITIVE
 #undef DO_SET_CHECK_STATE
 }
 
@@ -854,11 +857,10 @@ void EDVFindBarUpdateMenus(edv_findbar_struct *fb)
  */
 void EDVFindBarMap(edv_findbar_struct *fb)
 {
-	GtkWidget *w = (fb != NULL) ? fb->toplevel : NULL;
-	if(w == NULL)
+	if(fb == NULL)
 	    return;
 
-	gtk_widget_show(w);
+	gtk_widget_show(fb->toplevel);
 }
 
 /*
@@ -866,11 +868,10 @@ void EDVFindBarMap(edv_findbar_struct *fb)
  */
 void EDVFindBarUnmap(edv_findbar_struct *fb)
 {
-	GtkWidget *w = (fb != NULL) ? fb->toplevel : NULL;
-	if(w == NULL)
-	    return;   
+	if(fb == NULL)
+	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(fb->toplevel);
 }
 
 /*
@@ -881,10 +882,14 @@ void EDVFindBarDelete(edv_findbar_struct *fb)
 	if(fb == NULL)
 	    return;
 
+	EDVFindBarUnmap(fb);
+
+	fb->freeze_count++;
+
 	PUListBoxDelete(fb->find_op_pulistbox);
-	GTK_WIDGET_DESTROY(fb->search_combo);
-	GTK_WIDGET_DESTROY(fb->case_sensitive_check);
-	GTK_WIDGET_DESTROY(fb->stop_btn);
-	GTK_WIDGET_DESTROY(fb->toplevel);
+	gtk_widget_destroy(fb->toplevel);
+
+	fb->freeze_count--;
+
 	g_free(fb);
 }
