@@ -107,8 +107,8 @@ extern edv_findbar_operation EDVFindBarCurrentFindOP(
 );
 
 extern edv_findbar_struct *EDVFindBarNew(
-	gpointer core, GtkWidget *parent,
-	edv_findbar_role role,
+	edv_core_struct *core, GtkWidget *parent,
+	const edv_findbar_role role,
 	const gchar *(*get_location_cb)(
 		edv_findbar_struct *, gpointer
 	),
