@@ -9,10 +9,12 @@
 #include "style_edit.h"
 #include "stacklist.h"
 #include "keymap_list.h"
+#include "menucfg_list.h"
 
 #include "edv_types.h"
 #include "edv_generic_options_win.h"
 #include "edv_generic_options_win_op.h"
+#include "edv_utils_gtk.h"
 #include "config.h"
 
 #include "images/icon_ok_20x20.xpm"
@@ -51,6 +53,9 @@ static void OptWinStackListChangedCB(
 static void OptWinKeymapListChangedCB(
 	keymap_list_struct *kmlist, gpointer data
 );
+static void OptWinMenuCfgListChangedCB(
+	menucfg_list_struct *list, gpointer data
+);
 
 
 /* Widget References */
@@ -131,10 +136,8 @@ static void OptWinSwitchPageCB(
 
 	if(optwin->freeze_count > 0)
 	    return;
-	else
-	    optwin->freeze_count++;
-
 
+	optwin->freeze_count++;
 
 	EDVGenOptWinUpdateMenus(optwin);
 
@@ -216,8 +219,8 @@ static void OptWinCancelCB(GtkWidget *widget, gpointer data)
 
 	if(optwin->freeze_count > 0)
 	    return;
-	else
-	    optwin->freeze_count++;
+
+	optwin->freeze_count++;
 
 	EDVGenOptWinUnmap(optwin);
 
@@ -252,7 +255,9 @@ static void OptWinAnyChangedCB(GtkWidget *widget, gpointer data)
 	    return;
 
 	if(optwin->freeze_count > 0)
-	    optwin->freeze_count++;
+	    return;
+
+	optwin->freeze_count++;
 
 	if(!wref->has_changes)
 	{
@@ -299,7 +304,12 @@ static void OptWinKeymapListChangedCB(
 {
 	OptWinAnyChangedCB(NULL, data);
 }
-
+static void OptWinMenuCfgListChangedCB(
+	menucfg_list_struct *list, gpointer data
+)
+{
+	OptWinAnyChangedCB(NULL, data);
+}
 
 /*
  *	Creates a new Options Window Widget Reference.
@@ -336,18 +346,46 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefNew(
  */
 void EDVGenOptWinWRefDelete(edv_gen_opt_wref_struct *wref)
 {
+	GtkWidget *toplevel = NULL;
+	edv_gen_opt_win_struct *optwin;
+	edv_core_struct *core = NULL;
+
 	if(wref == NULL)
 	    return;
 
+	optwin = wref->optwin;
+	if(optwin != NULL)
+	{
+	    toplevel = optwin->toplevel;
+	    core = optwin->core;
+	}
+
 	/* Destroy the widget */
 	switch(wref->type)
 	{
 	  case EDV_GEN_OPT_WIDGET_UNKNOWN:
 	    if(wref->w != NULL)
-		g_printerr(
-"EDVGenOptWinWRefDelete(): Widget %p: Unable to destroy a widget of unknown type.\n",
+	    {
+		gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefDelete():\n\
+optwin=%p\n\
+wref->cfg_parm=\"%s\"\n\
+wref->w=%p\n\
+\n\
+Unable to destroy of type EDV_GEN_OPT_WIDGET_UNKNOWN.",
+		    optwin,
+		    wref->cfg_parm,
 		    wref->w
 		);
+		EDVPlaySoundError(core);
+		EDVMessageError(
+		    "Internal Error",
+		    msg,
+		    NULL,
+		    toplevel
+		);
+		g_free(msg);
+	    }
 	    break;
 
 	  case EDV_GEN_OPT_WIDGET_DRAWING_AREA:
@@ -385,6 +423,9 @@ void EDVGenOptWinWRefDelete(edv_gen_opt_wref_struct *wref)
 	  case EDV_GEN_OPT_WIDGET_KEYMAP_LIST:
 	    KeymapListDelete(KEYMAP_LIST(wref->w));
 	    break;
+	  case EDV_GEN_OPT_WIDGET_MENUCFG_LIST:
+	    MenuCfgListDelete(MENUCFG_LIST(wref->w));
+	    break;
 	}
 
 	GTK_WIDGET_DESTROY(wref->browse);
@@ -396,21 +437,39 @@ void EDVGenOptWinWRefDelete(edv_gen_opt_wref_struct *wref)
 
 /*
  *	Appends a new Widget Reference to the Options Window.
- *	the new wref structure.
+ *
+ *	The optwin specifies the Options Window.
+ *
+ *	The type specifies the what type of "widget" w is.
+ *
+ *	The w specifies the "widget".
+ *
+ *	The browse specifies the browse GtkButton.
+ *
+ *	The cfg_parm specifies the Cfg Item's parameter
+ *
+ *	Returns the new Widget Reference, the new Widget Reference
+ *	must not be deleted since it has already been appended to
+ *	the Options Window.
  */
 edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	edv_gen_opt_win_struct *optwin,
 	const edv_gen_opt_widget_type type,
 	gpointer w,
-	GtkWidget *browse,	/* Browse button */
-	const gchar *cfg_parm	/* Cfg item parameter */
+	GtkWidget *browse,
+	const gchar *cfg_parm
 )
 {
+	GtkWidget *toplevel;
 	edv_gen_opt_wref_struct *wref;
+	edv_core_struct *core;
 
 	if(optwin == NULL)
 	    return(NULL);
 
+	toplevel = optwin->toplevel;
+	core = optwin->core;
+
 	/* Create a new widget reference */
 	wref = EDVGenOptWinWRefNew(
 	    type, w, browse, cfg_parm, optwin
@@ -422,14 +481,34 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	    gpointer data = wref;
 	    guint sig_id = 0;
 
-	    /* Attach changed callback depending on widget type */
+	    /* Attach the "changed" signal callback based on the
+	     * widget's type
+	     */
 	    switch(type)
 	    {
 	      case EDV_GEN_OPT_WIDGET_UNKNOWN:
-		g_printerr(
-"EDVGenOptWinWRefAppend(): Widget %p: Unknown widget type specified.\n",
-		    w
-		);
+		if(TRUE)
+		{
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+Unable to append a widget of type EDV_GEN_OPT_WIDGET_UNKNOWN.",
+			optwin,
+			cfg_parm,
+			w
+		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
+		}
 		break;
 
 	      case EDV_GEN_OPT_WIDGET_DRAWING_AREA:
@@ -439,10 +518,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkDrawingArea.\n"
+		    /* Not a GtkDrawingArea */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_DRAWING_AREA\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkDrawingArea.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_BUTTON:
@@ -454,17 +550,55 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 			GTK_SIGNAL_FUNC(optwin_change_cb), data
 		    );
 		    if(GTK_IS_TOGGLE_BUTTON(w))
-			g_printerr(
-"EDVGenOptWinWRefAppend() The specified widget was specified\
- as a GtkButton but is detected to also be a GtkToggleButton.\n"
+		    {
+			gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget's type was specified as\n\
+EDV_GEN_OPT_WIDGET_BUTTON but the widget was detected\n\
+to be a GtkToggleButton, you should use specify the\n\
+type as EDV_GEN_OPT_WIDGET_TOGGLE_BUTTON instead.",
+			    optwin,
+			    cfg_parm,
+			    w
 			);
+			EDVPlaySoundError(core);
+			EDVMessageError(
+			    "Internal Error",
+			    msg,
+			    NULL,
+			    toplevel
+			);
+			g_free(msg);
+		    }
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkButton.\n"
+		    /* Not a GtkButton */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkButton.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_TOGGLE_BUTTON:
@@ -476,17 +610,55 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 			GTK_SIGNAL_FUNC(optwin_change_cb), data
 		    );
 		    if(GTK_IS_RADIO_BUTTON(w))
-			g_printerr(
-"EDVGenOptWinWRefAppend() The specified widget was specified\
- as a GtkToggleButton but is detected to also be a GtkRadioButton.\n"
+		    {
+			gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_TOGGLE_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget's type was specified as\n\
+EDV_GEN_OPT_WIDGET_TOGGLE_BUTTON but the widget was detected\n\
+to be a GtkRadioButton, you should use specify the\n\
+type as EDV_GEN_OPT_WIDGET_RADIO_BUTTON instead.",
+			    optwin,
+			    cfg_parm,
+			    w
 			);
+			EDVPlaySoundError(core);
+			EDVMessageError(
+			    "Internal Error",
+			    msg,
+			    NULL,
+			    toplevel
+			);
+			g_free(msg);
+		    }
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkToggleButton.\n"
+		    /* Not a GtkToggleButton */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_TOGGLE_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkToggleButton.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_RADIO_BUTTON:
@@ -500,10 +672,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkRadioButton.\n"
+		    /* Not a GtkRadioButton */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_RADIO_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkRadioButton.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_RANGE:
@@ -518,10 +707,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkRange.\n"
+		    /* Not a GtkRange */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_RANGE\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkRange.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_EDITABLE:
@@ -533,17 +739,55 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 			GTK_SIGNAL_FUNC(optwin_change_cb), data
 		    );
 		    if(GTK_IS_SPIN_BUTTON(w))
-			g_printerr(
-"EDVGenOptWinWRefAppend() The specified widget was specified\
- as a GtkEditable but is detected to also be a GtkSpinButton.\n"
+		    {
+			gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_EDITABLE\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget's type was specified as\n\
+EDV_GEN_OPT_WIDGET_EDITABLE but the widget was detected\n\
+to be a GtkSpinButton, you should use specify the\n\
+type as EDV_GEN_OPT_WIDGET_SPIN_BUTTON instead.",
+			    optwin,
+			    cfg_parm,
+			    w
 			);
+			EDVPlaySoundError(core);
+			EDVMessageError(
+			    "Internal Error",
+			    msg,
+			    NULL,
+			    toplevel
+			);
+			g_free(msg);
+		    }
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkEditable.\n"
+		    /* Not a GtkEditable */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_EDITABLE\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkEditable.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_SPIN_BUTTON:
@@ -557,10 +801,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkSpinButton.\n"
+		    /* Not a GtkSpinButton */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_SPIN_BUTTON\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkSpinButton.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_COMBO:
@@ -575,10 +836,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkCombo.\n"
+		    /* Not a GtkCombo */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_COMBO\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkCombo.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 	      case EDV_GEN_OPT_WIDGET_CLIST:
@@ -592,10 +870,27 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 		}
 		else
 		{
-		    g_printerr(
-"EDVGenOptWinWRefAppend(): The specified widget is not a\
- GtkCList.\n"
+		    /* Not a GtkCList */
+		    gchar *msg = g_strdup_printf(
+"EDVGenOptWinWRefAppend():\n\
+optwin=%p\n\
+type=EDV_GEN_OPT_WIDGET_CLIST\n\
+cfg_parm=\"%s\"\n\
+widget=%p\n\
+\n\
+The specified widget was not, in fact, a GtkCList.",
+			optwin,
+			cfg_parm,
+			w
 		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Internal Error",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
 		}
 		break;
 
@@ -612,8 +907,9 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	      case EDV_GEN_OPT_WIDGET_POPUP_LIST_BOX:
 		if(w != NULL)
 		{
+		    pulistbox_struct *pulistbox = PULISTBOX(w);
 		    PUListBoxSetChangedCB(
-			PULISTBOX(w),
+			pulistbox,
 			OptWinPopupListBoxChangedCB, wref
 		    );
 		}
@@ -621,8 +917,9 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	      case EDV_GEN_OPT_WIDGET_STYLE_EDIT:
 		if(w != NULL)
 		{
+		    style_edit_struct *style_edit = STYLE_EDIT(w);
 		    StyleEditSetChangedCB(
-			STYLE_EDIT(w),
+			style_edit,
 			OptWinStyleEditChangedCB, wref
 		    );
 		}
@@ -630,8 +927,9 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	      case EDV_GEN_OPT_WIDGET_STACK_LIST:
 		if(w != NULL)
 		{
+		    stack_list_struct *slist = STACK_LIST(w);
 		    StackListSetChangedCB(
-			STACK_LIST(w),
+			slist,
 			OptWinStackListChangedCB, wref
 		    );
 		}
@@ -639,12 +937,23 @@ edv_gen_opt_wref_struct *EDVGenOptWinWRefAppend(
 	      case EDV_GEN_OPT_WIDGET_KEYMAP_LIST:
 		if(w != NULL)
 		{
+		    keymap_list_struct *kmlist = KEYMAP_LIST(w);
 		    KeymapListSetChangedCB(
-			KEYMAP_LIST(w),
+			kmlist,
 			OptWinKeymapListChangedCB, wref
 		    );
 		}
 		break;
+	      case EDV_GEN_OPT_WIDGET_MENUCFG_LIST:
+		if(w != NULL)
+		{
+		    menucfg_list_struct *menucfg_list = MENUCFG_LIST(w);
+		    MenuCfgListSetChangedCB(
+			menucfg_list,
+			OptWinMenuCfgListChangedCB, wref
+		    );
+		}
+		break;
 	    }
 
 	    /* Append this widget reference to the list */
@@ -669,29 +978,32 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
 	GtkStyle *style;
-	GtkWidget *w, *parent, *parent2;
-	edv_gen_opt_win_struct *optwin = EDV_GEN_OPT_WIN(
-	    g_malloc0(sizeof(edv_gen_opt_win_struct))
-	);
+	GtkWidget	*w,
+			*parent, *parent2,
+			*toplevel;
+	edv_gen_opt_win_struct *optwin;
+
+	if(core == NULL)
+	    return(NULL);
+
+	optwin = EDV_GEN_OPT_WIN(g_malloc0(sizeof(edv_gen_opt_win_struct)));
 	if(optwin == NULL)
-	    return(optwin);
+	    return(NULL);
 
+	optwin->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	optwin->accelgrp = accelgrp = gtk_accel_group_new();
-	optwin->processing = FALSE;
-	optwin->busy_count = 0;
 	optwin->freeze_count = 0;
-	optwin->core = core;
-
+	optwin->busy_count = 0;
 	optwin->has_changes = FALSE;
-
 	optwin->title = STRDUP(title);
 	optwin->icon_data = icon_data;
+	optwin->core = core;
 
+	optwin->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	optwin->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, OPTWIN_WIDTH, OPTWIN_HEIGHT);
 	gtk_window_set_title(GTK_WINDOW(w), title);
 	gtk_window_set_wmclass(
@@ -721,21 +1033,21 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 	parent = w;
 
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	w = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	gtk_widget_show(w);
 	parent = w;
 
 
-	/* Vbox for main notebook */
+	/* Notebook GtkVBox */
 	w = gtk_vbox_new(FALSE, 0);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Main notebook */
+	/* Main GtkNotebook */
 	optwin->notebook = w = gtk_notebook_new();
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(w), GTK_POS_TOP);
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
@@ -752,13 +1064,13 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 
 
 
-	/* Separator */
+	/* Horizontal GtkSeparator */
 	w = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 
 
-	/* Buttons hbox */
+	/* Buttons GtkHBox */
 	w = gtk_hbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, border_major);
 	gtk_widget_show(w);
@@ -785,7 +1097,6 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 	    GTK_OBJECT(w), "clicked"
 	);
 	GUIButtonLabelUnderline(w, GDK_o);
-	gtk_widget_show(w);
 
 	/* Apply button */
 	optwin->apply_btn = w = GUIButtonPixmapLabelH(
@@ -824,7 +1135,6 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 	    GTK_OBJECT(w), "clicked"
 	);
 	GUIButtonLabelUnderline(w, GDK_a);
-	gtk_widget_show(w);
 
 	/* Save button */
 	optwin->save_btn = w = GUIButtonPixmapLabelH(
@@ -949,6 +1259,8 @@ edv_gen_opt_win_struct *EDVGenOptWinNew(
 	GUIButtonLabelUnderline(w, GDK_c);
 
 
+	optwin->freeze_count--;
+
 	return(optwin);
 }
 
@@ -1009,50 +1321,48 @@ GtkWidget *EDVGenOptWinPageAppend(
  */
 void EDVGenOptWinUpdateMenus(edv_gen_opt_win_struct *optwin)
 {
-	gboolean sensitive, has_changes;
-	GtkWidget *w;
+	gboolean	sensitive,
+			has_changes;
+	gchar *s;
 
 	if(optwin == NULL)
 	    return;
 
+	optwin->freeze_count++;
+
 	has_changes = optwin->has_changes;
 
-#define MAP(w)                  \
-{ if((w) != NULL) { gtk_widget_show(w); } }
-#define UNMAP(w)                \
-{ if((w) != NULL) { gtk_widget_hide(w); } }
+	s = g_strconcat(
+	    (optwin->title != NULL) ? optwin->title : "",
+	    (has_changes) ? " (*)" : "",
+	    NULL
+	);
+	gtk_window_set_title(GTK_WINDOW(optwin->toplevel), s);
+	g_free(s);
 
-	/* Update title */
-	w = optwin->toplevel;
-	if(w != NULL)
-	{
-	    gchar *title = g_strdup_printf(
-		"%s%s",
-		STRISEMPTY(optwin->title) ? "Untitled" : optwin->title,
-		(has_changes) ? " (*)" : ""
-	    );
-	    gtk_window_set_title(GTK_WINDOW(w), title);
-	    g_free(title);
-	}
-
-	/* Update OK, Apply, Save, Cancel Buttons */
-	sensitive = has_changes;
-	GTK_WIDGET_SET_SENSITIVE(optwin->ok_btn, sensitive)
-	GTK_WIDGET_SET_SENSITIVE(optwin->apply_btn, sensitive)
-	GTK_WIDGET_SET_SENSITIVE(optwin->save_btn, sensitive)
+	/* OK, Apply, Cancel, Close Buttons */
+	sensitive = has_changes ? TRUE : FALSE;
+	gtk_widget_set_sensitive(optwin->ok_btn, sensitive);
+	gtk_widget_set_sensitive(optwin->apply_btn, sensitive);
+	gtk_widget_set_sensitive(optwin->cancel_btn, sensitive);
+	sensitive = has_changes ? FALSE : TRUE;
+	gtk_widget_set_sensitive(optwin->close_btn, sensitive);
 	if(has_changes)
 	{
-	    MAP(optwin->cancel_btn);
-	    UNMAP(optwin->close_btn);
+	    gtk_widget_show(optwin->ok_btn);
+	    gtk_widget_show(optwin->apply_btn);
+	    gtk_widget_show(optwin->cancel_btn);
+	    gtk_widget_hide(optwin->close_btn);
 	}
 	else
 	{
-	    UNMAP(optwin->cancel_btn);
-	    MAP(optwin->close_btn);
+	    gtk_widget_show(optwin->ok_btn);
+	    gtk_widget_show(optwin->apply_btn);
+	    gtk_widget_hide(optwin->cancel_btn);
+	    gtk_widget_show(optwin->close_btn);
 	}
 
-#undef UNMAP
-#undef MAP
+	optwin->freeze_count--;
 }
 
 /*
@@ -1065,13 +1375,10 @@ void EDVGenOptWinSetBusy(edv_gen_opt_win_struct *optwin, gboolean is_busy)
 	GtkWidget *w;
 	edv_core_struct *core;
 
-
 	if(optwin == NULL)
 	    return;
 
 	core = optwin->core;
-	if(core == NULL)
-	    return;
 
 	w = optwin->toplevel;
 	if(w != NULL)
@@ -1116,8 +1423,10 @@ void EDVGenOptWinSetBusy(edv_gen_opt_win_struct *optwin, gboolean is_busy)
  */
 gboolean EDVGenOptWinIsMapped(edv_gen_opt_win_struct *optwin)
 {
-	GtkWidget *w = (optwin != NULL) ? optwin->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(optwin == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(optwin->toplevel));
 }
 
 /*
@@ -1125,18 +1434,13 @@ gboolean EDVGenOptWinIsMapped(edv_gen_opt_win_struct *optwin)
  */
 void EDVGenOptWinMap(edv_gen_opt_win_struct *optwin)
 {
-	GtkWidget *w = (optwin != NULL) ? optwin->toplevel : NULL;
-	if(w == NULL)
+	if(optwin == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(optwin->toplevel);
 
-	w = optwin->ok_btn;
-	if(w != NULL)
-	{
-	    gtk_widget_grab_focus(w);
-	    gtk_widget_grab_default(w);
-	}
+	gtk_widget_grab_focus(optwin->ok_btn);
+	gtk_widget_grab_default(optwin->ok_btn);
 }
 
 /*
@@ -1144,11 +1448,10 @@ void EDVGenOptWinMap(edv_gen_opt_win_struct *optwin)
  */
 void EDVGenOptWinUnmap(edv_gen_opt_win_struct *optwin)
 {
-	GtkWidget *w = (optwin != NULL) ? optwin->toplevel : NULL;
-	if(w == NULL)
+	if(optwin == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(optwin->toplevel);
 }
 
 /*
@@ -1159,6 +1462,10 @@ void EDVGenOptWinDelete(edv_gen_opt_win_struct *optwin)
 	if(optwin == NULL)
 	    return;
 
+	EDVGenOptWinUnmap(optwin);
+
+	optwin->freeze_count++;
+
 	/* Delete the widget references list */
 	if(optwin->wrefs_list != NULL)
 	{
@@ -1179,28 +1486,13 @@ void EDVGenOptWinDelete(edv_gen_opt_win_struct *optwin)
 	    optwin->page_toplevels_list = NULL;
 	}
 
-	GTK_WIDGET_DESTROY(optwin->notebook);
-	optwin->notebook = NULL;
-
-	GTK_WIDGET_DESTROY(optwin->ok_btn);
-	optwin->ok_btn = NULL;
-	GTK_WIDGET_DESTROY(optwin->apply_btn);
-	optwin->apply_btn = NULL;
-	GTK_WIDGET_DESTROY(optwin->save_btn);
-	optwin->save_btn = NULL;
-	GTK_WIDGET_DESTROY(optwin->cancel_btn);
-	optwin->cancel_btn = NULL;
-	GTK_WIDGET_DESTROY(optwin->close_btn);
-	optwin->close_btn = NULL;
-
-	GTK_WIDGET_DESTROY(optwin->toplevel);
-	optwin->toplevel = NULL;
-
-	GTK_ACCEL_GROUP_UNREF(optwin->accelgrp);
-	optwin->accelgrp = NULL;
+	gtk_widget_destroy(optwin->notebook);
+	gtk_widget_destroy(optwin->toplevel);
+	gtk_accel_group_unref(optwin->accelgrp);
 
 	g_free(optwin->title);
-	optwin->title = NULL;
+
+	optwin->freeze_count--;
 
 	g_free(optwin);
 }
