@@ -8,6 +8,7 @@
 #include "style_edit.h"
 #include "stacklist.h"
 #include "keymap_list.h"
+#include "menucfg_list.h"
 
 #include "cfg.h"
 #include "edv_types.h"
@@ -206,8 +207,7 @@ static void OptWinFetchWidgetValueFromCFG(
 	        }
 	        else
 	        {
-		    gfloat valuef = EDV_GET_F(cfg_parm);
-		    gtk_spin_button_set_value(spin_button, valuef);
+		    gtk_spin_button_set_value(spin_button, EDV_GET_F(cfg_parm));
 	        }
 	        THAW
 	    }
@@ -381,6 +381,13 @@ static void OptWinFetchWidgetValueFromCFG(
 		    const gint i = EDV_GET_I(cfg_parm);
 		    PUListBoxSelect(pulistbox, i);
 		}
+		else if((cfg_type == CFG_ITEM_TYPE_INT64) ||
+		        (cfg_type == CFG_ITEM_TYPE_UINT64)
+		)
+		{
+		    const glong l = EDV_GET_L(cfg_parm);
+		    PUListBoxSelect(pulistbox, (gint)l);
+		}
 		THAW
 	    }
 	    break;
@@ -531,7 +538,6 @@ static void OptWinFetchWidgetValueFromCFG(
 		    {
 		        gint row;
 		        GList *glist;
-		        keymap_key_struct *km;
 
 		        /* Iterate through Accelkey list and set them to
 		         * the Keymap List
@@ -545,15 +551,67 @@ static void OptWinFetchWidgetValueFromCFG(
 			    if(ak == NULL)
 			        continue;
 
-			    km = KeymapKeyNew(
-			        ak->key,			/* Keyval */
-			        ak->modifiers,		/* State */
-			        (gpointer)ak->opid		/* Data */
-			    );
 			    KeymapListSetKey(
-			        kmlist, row, km
+			        kmlist, row,
+				KeymapKeyNew(
+				    ak->key,		/* Keyval */
+				    ak->modifiers,	/* State */
+				    (gpointer)ak->opid	/* Data */
+				)
 			    );
-			    KeymapKeyDelete(km);
+		        }
+		    }
+	        }
+		THAW
+	    }
+	    break;
+	  case EDV_GEN_OPT_WIDGET_MENUCFG_LIST:
+	    if(w != NULL)
+	    {
+		menucfg_list_struct *list = MENUCFG_LIST(w);
+	        FREEZE
+	        if(cfg_type == CFG_ITEM_TYPE_MENU)
+	        {
+		    cfg_menu_item_struct *item;
+		    cfg_menu_struct *menu = CFG_MENU(
+		        cfg_item->value
+		    );
+		    if(menu != NULL)
+		    {
+			const gint ncustom_datas = MeuCfgListGetTotalCustomDatas(list);
+			gint row;
+		        GList *glist;
+			menucfg_item_struct *item2;
+
+			MenuCfgListClear(list);
+
+		        for(glist = menu->list;
+			    glist != NULL;
+			    glist = g_list_next(glist)
+		        )
+		        {
+			    item = CFG_MENU_ITEM(glist->data);
+			    if(item == NULL)
+			        continue;
+
+			    item2 = MenuCfgItemNew(
+                                item->label,
+                                item->command,
+                                item->icon_file,
+                                item->description
+                            );
+			    if(item2 == NULL)
+				break;
+
+			    if(ncustom_datas > 0)
+			    {
+				item2->custom_datas_list = g_list_append(
+				    item2->custom_datas_list,
+				    STRDUP(item->ext_data)
+				);
+			    }
+
+			    row = MenuCfgListAppend(list, item2);
 		        }
 		    }
 	        }
@@ -683,19 +741,31 @@ static void OptWinSetWidgetValueToCFG(
 	        }
 	        else
 	        {
-		    if(spin_button->digits > 0)
+		    if((spin_button->digits > 0) ||
+		       (cfg_type == CFG_ITEM_TYPE_FLOAT) ||
+		       (cfg_type == CFG_ITEM_TYPE_DOUBLE)
+		    )
 		    {
-		        gfloat valuef = gtk_spin_button_get_value_as_float(
+			const gfloat v = gtk_spin_button_get_value_as_float(
 			    spin_button
 		        );
-		        EDV_SET_F(cfg_parm, valuef);
+		        EDV_SET_F(cfg_parm, v);
+		    }
+		    else if((cfg_type == CFG_ITEM_TYPE_UINT64) ||
+			    (cfg_type == CFG_ITEM_TYPE_INT64)
+		    )
+		    {
+		        const gint v = gtk_spin_button_get_value_as_int(
+			    spin_button
+		        );
+		        EDV_SET_L(cfg_parm, v);
 		    }
 		    else
 		    {
-		        gfloat valuei = gtk_spin_button_get_value_as_int(
+		        const gint v = gtk_spin_button_get_value_as_int(
 			    spin_button
 		        );
-		        EDV_SET_I(cfg_parm, valuei);
+		        EDV_SET_I(cfg_parm, v);
 		    }
 	        }
 	    }
@@ -849,6 +919,12 @@ static void OptWinSetWidgetValueToCFG(
 		{
 		    EDV_SET_I(cfg_parm, i);
 		}
+		else if((cfg_type == CFG_ITEM_TYPE_INT64) ||
+			(cfg_type == CFG_ITEM_TYPE_UINT64)
+		)
+		{
+		    EDV_SET_L(cfg_parm, (glong)i);
+		}
 	    }
 	    break;
 	  case EDV_GEN_OPT_WIDGET_STYLE_EDIT:
@@ -997,8 +1073,11 @@ static void OptWinSetWidgetValueToCFG(
 		 * containing the Accelkeys
 		 */
 		ak_list = CFGAccelkeyListNew(glist);
-		g_list_foreach(glist, (GFunc)CFGAccelkeyDelete, NULL);
-		g_list_free(glist);
+		if(glist != NULL)
+		{
+		    g_list_foreach(glist, (GFunc)CFGAccelkeyDelete, NULL);
+		    g_list_free(glist);
+		}
 
 		/* Set/copy the Accelkey List to cfg item */
 		EDV_SET_ACCELKEY_LIST(cfg_parm, ak_list);
@@ -1006,6 +1085,55 @@ static void OptWinSetWidgetValueToCFG(
 		CFGAccelkeyListDelete(ak_list);
 	    }
 	    break;
+	  case EDV_GEN_OPT_WIDGET_MENUCFG_LIST:
+	    if((w != NULL) && (cfg_type == CFG_ITEM_TYPE_MENU))
+	    {
+		gint row;
+		GList *menu_items_list;
+		cfg_menu_struct *menu;
+		menucfg_item_struct *item;
+		menucfg_list_struct *list = MENUCFG_LIST(w);
+
+		/* Iterate through each row on the Keymap List and
+		 * append/copy each key to the glist
+		 */
+		menu_items_list = NULL;
+		for(row = 0; TRUE; row++)
+		{
+		    item = MenuCfgListGet(list, row);
+		    if(item == NULL)
+			break;
+
+		    menu_items_list = g_list_append(
+			menu_items_list,
+			CFGMenuItemNew(
+			    item->label,
+			    item->command,
+			    item->icon_file,
+			    item->description,
+			    MenuCfgItemGetCustomDataValue(item, 0)
+			)
+		    );
+		}
+
+		/* Create a new Accelkey List and then delete the glist
+		 * containing the Accelkeys
+		 */
+		menu = CFGMenuNew(menu_items_list);
+		if(menu_items_list != NULL)
+		{
+		    g_list_foreach(
+			menu_items_list, (GFunc)CFGMenuItemDelete, NULL
+		    );
+		    g_list_free(menu_items_list);
+		}
+
+		/* Set/copy the Menu List to cfg item */
+		EDV_SET_MENU(cfg_parm, menu);
+
+		CFGMenuDelete(menu);
+	    }
+	    break;
 	}
 }
 
