@@ -1,9 +1,9 @@
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#include <errno.h>
 #include <gtk/gtk.h>
+#include <unistd.h>
 
-//#include "../include/string.h"
 #include "../include/disk.h"
 #include "../include/prochandle.h"
 
@@ -426,7 +426,7 @@ please look in the contents by going to Help->Contents.\n",
 	    );
 	    EDVPlaySoundWarning(core);
 	    EDVMessageWarning(
-		"Help Display Failed",
+		"Help Failed",
 		buf,
 		NULL,
 		toplevel
@@ -539,28 +539,31 @@ Save the file and then restart this application.\n",
 		break;
 	    }
 
-	    /* Execute help viewer */
+	    /* Execute the help viewer */
 	    p = Exec(cmd);
-	    if(p == 0)
+	    if(p <= 0)
 	    {
 		/* Unable to execute help viewer */
-		gchar *buf = g_strdup_printf(
+		const gint error_code = (gint)errno;
+		gchar *msg = g_strdup_printf(
 "Unable to execute help viewer command:\n\
 \n\
-    %s\n",
-		    cmd
+    %s\n\
+\n\
+%s.",
+		    cmd, g_strerror(error_code)
 		);
 		EDVPlaySoundError(core);
 		EDVMessageError(
-"Help Display Failed",
-		    buf,
+"Help Failed",
+		    msg,
 "An error was encountered while attempting to run the help\n\
 viewer to display the requested document. Please verify that\n\
 the help viewer and the requested document are both\n\
 installed properly.",
 		    toplevel
 		);
-		g_free(buf);
+		g_free(msg);
 	    }
 
 	    /* Delete command */
@@ -568,7 +571,7 @@ installed properly.",
 	}
 	else
 	{
-	    gchar *buf = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 "Unable to find a help viewer to display help documentation:\n\
 \n\
     %s\n",
@@ -576,12 +579,12 @@ installed properly.",
 	    );
 	    EDVPlaySoundWarning(core);
 	    EDVMessageWarning(
-		"Help Display Failed",
-		buf,
+		"Help Failed",
+		msg,
 		NULL,
 		toplevel
 	    );
-	    g_free(buf);
+	    g_free(msg);
 	}
 
 	g_free(help_file);
