@@ -12,9 +12,8 @@ extern edv_history_struct *EDVHistoryListAppend(
 	gint *n, const gint max
 );
 
-extern void EDVHistoryListOpen(
+extern GList *EDVHistoryListOpen(
 	const gchar *path,
-	edv_history_struct ***list, gint *total,
 	gint (*progress_cb)(gpointer, const gulong, const gulong),
 	gpointer data
 );
