@@ -101,11 +101,13 @@ static gchar *EDVInterPSGetCmdPath(const cfg_item_struct *cfg_list)
 
 
 /*
- *	Returns the pid of the currently running process of Endeavour or
- *	0 if it is not running/error.
+ *	Gets the pid of the currently running process of Endeavour.
  *
  *	The lock link will be checked and the actual process will be
  *	checked to see if it is actually running.
+ *
+ *	Returns the pid of the currently running process of Endeavour
+ *	or 0 if it is not.
  */
 gint EDVInterPSGetLock(const cfg_item_struct *cfg_list)
 {
@@ -116,7 +118,7 @@ gint EDVInterPSGetLock(const cfg_item_struct *cfg_list)
 	    return(0);
 
 	/* Get the lock link's value */
-	v = EDVGetLinkValue(lock_link);
+	v = EDVGetLinkTarget(lock_link);
 	if(v == NULL)
 	{
 	    /* Lock link does not exist or error reading it */
@@ -126,7 +128,7 @@ gint EDVInterPSGetLock(const cfg_item_struct *cfg_list)
 
 	g_free(lock_link);	/* Don't need this anymore */
 
-	/* Get pid from the link's destination and check if it is
+	/* Get pid from the link's target and check if it is
 	 * actually running
 	 */
 	p = ATOI(v);
@@ -169,7 +171,7 @@ gint EDVInterPSMakeLock(
 	    return(0);
 	}
 
-	/* Format the link destination, which is the pid of the
+	/* Format the link's target, which is the pid of the
 	 * specified process p
 	 */
 	g_snprintf(
@@ -181,17 +183,20 @@ gint EDVInterPSMakeLock(
 	/* (Re)create the lock link */
 	if(unlink((const char *)lock_link))
 	{
-	    const gint error_code = errno;
+	    const gint error_code = (gint)errno;
 	    if(error_code != ENOENT)
 	    {
 		/* The object exists but was not able to be removed */
 		g_free(lock_link);
+		errno = (int)error_code;
 		return(-1);
 	    }
 	}
 	if(symlink((const char *)num_str, (const char *)lock_link))
 	{
+	    const gint error_code = (gint)errno;
 	    g_free(lock_link);
+	    errno = (int)error_code;
 	    return(-1);
 	}
 
@@ -340,10 +345,14 @@ gchar **EDVInterPSGetCommandsList(const cfg_item_struct *cfg_list)
 	}
 
 	/* Allocate the read buffer */
-	read_buf_len = (gulong)(stat_buf.st_blksize / sizeof(gchar));
+#if defined(_WIN32)
+	read_buf_len = 1024l;
+#else
+	read_buf_len = (gulong)stat_buf.st_blksize;
+#endif
 	if(read_buf_len == 0l)
 	    read_buf_len = 1024l;
-	read_buf = (gchar *)g_malloc(read_buf_len);
+	read_buf = (gchar *)g_malloc(read_buf_len * sizeof(gchar));
 	if(read_buf == NULL)
 	{
 	    fclose(fp);
