@@ -4,7 +4,6 @@
 #include <sys/stat.h>
 #include <gtk/gtk.h>
 
-#include "../include/string.h"
 #include "../include/fio.h"
 #include "../include/disk.h"
 
@@ -15,6 +14,7 @@
 #include "config.h"
 
 
+/* Matching */
 edv_mime_type_struct *EDVMimeTypesListMatchType(
 	edv_mime_type_struct **list, const gint total,
 	gint *n,
@@ -25,7 +25,45 @@ edv_mime_type_struct *EDVMimeTypesListMatchPath(
 	const gchar *path
 );
 
+/* Insert, Append, Remove */
+gint EDVMimeTypesListInsert(
+	edv_mime_type_struct ***list, gint *total,
+	const gint i,
+	edv_mime_type_struct *m
+);
+gint EDVMimeTypesListAppend(
+	edv_mime_type_struct ***list, gint *total,
+	edv_mime_type_struct *m
+);
+void EDVMimeTypesListRemove(
+	edv_mime_type_struct ***list, gint *total,
+	const gint i
+);
+void EDVMimeTypesListClear(
+	edv_mime_type_struct ***list, gint *total 
+);
+
+
+/* DDE Buffer */
+GList *EDVMimeTypesListFromDDEBuffer(
+	edv_mime_type_struct **list, const gint total,
+	const guint8 *buf, const gint buf_len
+);
+guint8 *EDVMimeTypesListToDDEBuffer(
+	edv_mime_type_struct **list, const gint total,
+	GList *selection_list,
+	gint *buf_len_rtn
+);
+GList *EDVMimeTypeCommandsListFromDDEBuffer(
+	const guint8 *buf, const gint buf_len
+);
+guint8 *EDVMimeTypeCommandsListToDDEBuffer(
+	GList *commands_list,
+	gint *buf_len_rtn
+);
+
 void EDVMimeTypesListSystemFileOpen(
+	const gchar *data_path,
 	edv_mime_type_struct ***list, gint *total,
 	const gint insert_index,
 	gint (*progress_cb)(gpointer, const gulong, const gulong),
@@ -104,9 +142,9 @@ edv_mime_type_struct *EDVMimeTypesListMatchType(
 
 	    if(m->type == type)
 	    {
-                if(n != NULL)
-                   *n = i;
-                return(m);
+		if(n != NULL)
+		   *n = i;
+		return(m);
 	    }
 
 	    if(case_sensitive)
@@ -196,9 +234,551 @@ edv_mime_type_struct *EDVMimeTypesListMatchPath(
 }
 
 
+/*
+ *	Inserts a new MIME Type.
+ *
+ *	The list and total specifies the MIME Types list.
+ *
+ *	The i specifies the insert position. If i is -1 then the
+ *	MIME Type will be appended to the list instead.
+ *
+ *	The m specifies the MIME Type to insert into the list. This
+ *	MIME Type will be transfered to the list and should not be
+ *	referenced again after this call.
+ *
+ *	Returns the MIME Type's index in the list or negative on error.
+ */
+gint EDVMimeTypesListInsert(
+	edv_mime_type_struct ***list, gint *total,
+	const gint i,
+	edv_mime_type_struct *m
+)
+{
+	gint insert_pos = i, mt_num;
+
+	if((list == NULL) || (total == NULL))
+	{
+	    EDVMimeTypeDelete(m);
+	    return(-2);
+	}
+
+	if((insert_pos < 0) || (insert_pos >= *total))
+	{
+	    /* Append */
+	    gint n = MAX((*total), 0);
+	    *total = n + 1;
+	    *list = (edv_mime_type_struct **)g_realloc(
+		*list,
+		(*total) * sizeof(edv_mime_type_struct *)
+	    );
+	    if(*list == NULL)
+	    {
+		*total = 0;
+		EDVMimeTypeDelete(m);
+		return(-3);
+	    }
+
+	    mt_num = n;
+	}
+	else
+	{
+	    /* Insert */
+	    gint j, n = MAX((*total), 0);
+	    *total = n + 1;
+	    *list = (edv_mime_type_struct **)g_realloc(
+		*list,
+		(*total) * sizeof(edv_mime_type_struct *)
+	    );
+	    if(*list == NULL)
+	    {
+		*total = 0;
+		EDVMimeTypeDelete(m);
+		return(-3);
+	    }
+
+	    /* Shift pointers */
+	    mt_num = insert_pos;
+	    for(j = n; j > mt_num; j--)
+		(*list)[j] = (*list)[j - 1];
+
+	    insert_pos++;
+	}
+
+	(*list)[mt_num] = m;
+
+	return(mt_num);
+}
+
+/*
+ *	Appends the MIME Type to the list.
+ *
+ *	The list and total specifies the MIME Types list.
+ *
+ *	The m specifies the MIME Type to append to the list. This
+ *	MIME Type will be transfered to the list and should not be
+ *	referenced again after this call.
+ *
+ *	Returns the MIME Type's index in the list or negative on error.
+ */
+gint EDVMimeTypesListAppend(
+	edv_mime_type_struct ***list, gint *total,
+	edv_mime_type_struct *m
+)
+{
+	return(EDVMimeTypesListInsert(
+	    list, total,
+	    -1,
+	    m
+	));
+}
+
+/*
+ *	Removes the MIME Type from the list.
+ *
+ *	The list and total specifies the MIME Types list.
+ *
+ *	The i specifies the MIME Type to remove from the list.
+ */
+void EDVMimeTypesListRemove(
+	edv_mime_type_struct ***list, gint *total,
+	const gint i
+)
+{
+	if((list == NULL) || (total == NULL))
+	    return;
+
+	if((i < 0) || (i >= (*total)))
+	    return;
+
+	/* Delete this MIME Type */
+	EDVMimeTypeDelete((*list)[i]);
+
+	/* Reduce the total */
+	*total = (*total) - 1;
+	if(*total < 0)
+	    *total = 0;
+
+	/* Shift the pointers and reduce the allocation of the
+	 * MIME Types list
+	 */
+	if(*total > 0)
+	{
+	    gint n;
+
+	    for(n = i;  n < *total; n++)
+		(*list)[n] = (*list)[n + 1];
+
+	    *list = (edv_mime_type_struct **)g_realloc(
+		*list,
+		(*total) * sizeof(edv_mime_type_struct *)
+	    );
+	    if(*list == NULL)
+		*total = 0;
+	}
+	else
+	{
+	    g_free(*list);
+	    *list = NULL;
+	}
+}
+
+/*
+ *	Removes all the MIME Types from the list.
+ *
+ *	The list and total specifies the MIME Types list.
+ */
+void EDVMimeTypesListClear(
+	edv_mime_type_struct ***list, gint *total
+)
+{
+	gint i, n;
+
+	if((list == NULL) || (total == NULL))
+	    return;
+
+	if((*list == NULL) || (*total <= 0))
+	    return;
+
+	n = *total;
+	for(i = 0; i < n; i++)
+	    EDVMimeTypeDelete((*list)[i]);
+
+	g_free(*list);
+	*list = NULL;
+	*total = 0;
+}
+
+
+/*
+ *	Creates a list of MIME Types from the DDE Buffer.
+ *
+ *	The list and total specifies the MIME Types list.
+ *
+ *	The buf and buf_len specifies the DDE Buffer created by
+ *	EDVMimeTypesListToDDEBuffer().
+ *
+ *	Returns a GList of edv_mime_type_struct * that must be
+ *	deleted by the calling function. Each MIME Type in the
+ *	returned list is a copy (not a reference) to a MIME Type in
+ *	the MIME Types list.
+ */
+GList *EDVMimeTypesListFromDDEBuffer(
+	edv_mime_type_struct **list, const gint total,
+	const guint8 *buf, const gint buf_len
+)
+{
+	gint i, len;
+	gchar *type;
+	const guint8 *buf_ptr, *buf_end, *buf_next;
+	GList *list_rtn;
+	edv_mime_type_struct *m;
+
+	if((list == NULL) || (total <= 0) ||
+	   (buf == NULL) || (buf_len <= 0)
+	)
+	    return(NULL);
+
+	list_rtn = NULL;
+
+	/* Begin parsing the DDE buffer
+	 *
+	 * Format of each segment is as follows:
+	 *
+	 *	<type>			String
+	 */
+	buf_ptr = buf;
+	buf_end = buf_ptr + buf_len;
+	while(buf_ptr < buf_end)
+	{
+	    /* Get the type */
+	    buf_next = buf_ptr;
+	    while(buf_next < buf_end)
+	    {
+		if(*((const gchar *)buf_next) == '\0')
+		    break;
+
+		buf_next++;
+	    }
+	    len = buf_next - buf_ptr;
+	    type = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+	    if(type == NULL)
+		break;
+	    if(len > 0)
+		memcpy(type, buf_ptr, len);
+	    type[len] = '\0';
+
+	    /* Look for a MIME Type in the list that matches this
+	     * type
+	     */
+	    m = EDVMimeTypesListMatchType(
+		list, total,
+		&i,
+		type, FALSE
+	    );
+	    if(m != NULL)
+		list_rtn = g_list_append(
+		    list_rtn,
+		    EDVMimeTypeCopy(m)
+		);
+
+	    g_free(type);
+
+	    buf_ptr = buf_next + 1;	/* Seek past the null character */
+	}
+
+	return(list_rtn);
+}
+
+/*
+ *	Creates a DDE Buffer from selected objects in the
+ *	MIME Types list.
+ *
+ *	The list and total specifies the MIME Types list.
+ *
+ *	The selection_list specifies a list of gints that specify
+ *	the indices of which MIME Types in the list that are to
+ *	be added to the DDE Buffer. If selection_list is NULL
+ *	then all the MIME Types in the list will be added to the
+ *	DDE Buffer.
+ *
+ *	The buf_len_rtn specifies the length of the DDE buffer.
+ *
+ *	Returns the DDE Buffer which describes a null character
+ *	deliminated list of MIME Type types. This buffer must be
+ *	deleted by the calling function.
+ */
+guint8 *EDVMimeTypesListToDDEBuffer(
+	edv_mime_type_struct **list, const gint total,
+	GList *selection_list,
+	gint *buf_len_rtn
+)
+{
+	gint buf_len;
+	guint8 *buf;
+
+	if(buf_len_rtn == NULL)
+	    return(NULL);
+
+	*buf_len_rtn = 0;
+
+	if((list == NULL) || (total <= 0))
+	    return(NULL);
+
+	buf_len = 0;
+	buf = NULL;
+
+#define APPEND_BUF(_type_)	{		\
+ if((_type_) != NULL) {				\
+  const gint	buf_i = buf_len,		\
+		type_len = STRLEN(_type_);	\
+						\
+  buf_len += type_len + 1;	/* Include the null character */ \
+  buf = (guint8 *)g_realloc(			\
+   buf,						\
+   buf_len * sizeof(guint8)			\
+  );						\
+  if(buf != NULL) {				\
+   if(type_len > 0)				\
+    memcpy(					\
+     buf + buf_i,				\
+     (_type_),					\
+     type_len					\
+    );						\
+   buf[buf_i + type_len] = '\0';	/* Set the null character */ \
+  } else {					\
+   buf_len = 0;					\
+  }						\
+ }						\
+}
+
+	/* Selection list specified? */
+	if(selection_list != NULL)
+	{
+	    gint i;
+	    GList *glist;
+	    edv_mime_type_struct *m;
+
+	    for(glist = selection_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
+	    {
+		i = (gint)glist->data;
+		if((i < 0) || (i >= total))
+		    continue;
+
+		m = list[i];
+		if(m == NULL)
+		    continue;
+
+		APPEND_BUF(m->type);
+	    }
+	}
+	else
+	{
+	    gint i;
+	    edv_mime_type_struct *m;
+
+	    /* Iterate through the MIME Types list */
+	    for(i = 0; i < total; i++)
+	    {
+		m = list[i];
+		if(m == NULL)
+		    continue;
+
+		APPEND_BUF(m->type);
+	    }
+	}
+
+#undef APPEND_BUF
+
+	*buf_len_rtn = buf_len;
+
+	return(buf);
+}
+
+/*
+ *	Creates a list of MIME Type commands from the DDE Buffer.
+ *
+ *	The buf and buf_len specifies the DDE Buffer created by
+ *	EDVMimeTypeCommandsListToDDEBuffer().
+ *
+ *	Returns a GList of edv_mime_type_command_struct * that must
+ *	be deleted by the calling function.
+ */
+GList *EDVMimeTypeCommandsListFromDDEBuffer(
+	const guint8 *buf, const gint buf_len
+)
+{
+	gint len;
+	const guint8 *buf_ptr, *buf_end, *buf_next;
+	GList *list_rtn;
+	edv_mime_type_command_struct *cmd;
+
+	if((buf == NULL) || (buf_len <= 0))
+	    return(NULL);
+
+	list_rtn = NULL;
+
+	/* Begin parsing the DDE buffer
+	 *
+	 * Format of each segment is as follows:
+	 *
+	 *	<name>			String
+	 *	<command>		String
+	 */
+	buf_ptr = buf;
+	buf_end = buf_ptr + buf_len;
+	while(buf_ptr < buf_end)
+	{
+	    /* Create a new command */
+	    cmd = EDVMimeTypeCommandNew();
+	    if(cmd == NULL)
+		break;
+
+	    /* Append the new command to the return list */
+	    list_rtn = g_list_append(list_rtn, cmd);
+
+	    /* Begin parsing this segment */
+
+	    /* Get the name */
+	    buf_next = buf_ptr;
+	    while(buf_next < buf_end)
+	    {
+		if(*((const gchar *)buf_next) == '\0')
+		    break;
+
+		buf_next++;
+	    }
+	    len = buf_next - buf_ptr;
+	    g_free(cmd->name);
+	    cmd->name = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+	    if(cmd->name != NULL)
+	    {
+		if(len > 0)
+		    memcpy(cmd->name, buf_ptr, len);
+		cmd->name[len] = '\0';
+	    }
+	    buf_ptr = buf_next + 1;	/* Seek past the null character */
+	    if(buf_ptr >= buf_end)
+		break;
+
+	    /* Get the command */
+	    buf_next = buf_ptr;
+	    while(buf_next < buf_end)
+	    {
+		if(*((const gchar *)buf_next) == '\0')
+		    break;
+
+		buf_next++;
+	    }
+	    len = buf_next - buf_ptr;
+	    g_free(cmd->command);
+	    cmd->command = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+	    if(cmd->command != NULL)
+	    {
+		if(len > 0)
+		    memcpy(cmd->command, buf_ptr, len);
+		cmd->command[len] = '\0';
+	    }
+	    buf_ptr = buf_next + 1;	/* Seek past the null character */
+	}
+
+	return(list_rtn);
+}
+
+/*
+ *	Creates a DDE Buffer from the MIME Type commands list.
+ *
+ *	The commands_list specifies the MIME Type commands list,
+ *	which is a GList of edv_mime_type_command_struct *.
+ *
+ *	The buf_len_rtn specifies the length of the DDE buffer.
+ *
+ *	Returns the DDE buffer which describes the MIME Type
+ *	commands. This buffer must be deleted by the calling
+ *	function.
+ */
+guint8 *EDVMimeTypeCommandsListToDDEBuffer(
+	GList *commands_list,
+	gint *buf_len_rtn
+)
+{
+	gint buf_len;
+	guint8 *buf;
+	GList *glist;
+	edv_mime_type_command_struct *cmd;
+
+	if(buf_len_rtn == NULL)
+	    return(NULL);
+
+	*buf_len_rtn = 0;
+
+	if(commands_list == NULL)
+	    return(NULL);
+
+	buf_len = 0;
+	buf = NULL;
+
+#define APPEND_BUF_S(_s_)	{		\
+ const gint	buf_i = buf_len,		\
+		s_len = STRLEN(_s_);		\
+						\
+ buf_len += s_len + 1;		/* Include the null character */ \
+ buf = (guint8 *)g_realloc(			\
+  buf,						\
+  buf_len * sizeof(guint8)			\
+ );						\
+ if(buf != NULL) {				\
+  if(s_len > 0)					\
+   memcpy(					\
+    buf + buf_i,				\
+    (_s_),					\
+    s_len					\
+   );						\
+  buf[buf_i + s_len] = '\0';	/* Set the null character */ \
+ } else {					\
+  buf_len = 0;					\
+ }						\
+}
+
+	/* Iterate through the commands list */
+	for(glist = commands_list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
+	{
+	    cmd = EDV_MIME_TYPE_COMMAND(glist->data);
+	    if(cmd == NULL)
+		continue;
+
+	    /* Append this command as a segment
+	     *
+	     * Format of each segment is as follows:
+	     *
+	     *	<name>			String
+	     *	<command>		String
+	     */
+	    APPEND_BUF_S(cmd->name);
+	    APPEND_BUF_S(cmd->command);
+	}
+
+#undef APPEND_BUF_S
+
+	*buf_len_rtn = buf_len;
+
+	return(buf);
+}
+
+
 /*
  *	Opens the MIME Types that are specified on the system.
  *
+ *	The data_path specifies the full path to the data directury
+ *	where the icons are to be loaded from. The EDV_DEF_ICONS_SUBDIR
+ *	will be postfixed to this value in which all the icons will
+ *	be loaded from this directory.
+ *
  *	The list and total specifies the pointers to the MIME Types
  *	list.
  *
@@ -223,6 +803,7 @@ edv_mime_type_struct *EDVMimeTypesListMatchPath(
  *	from this file will be marked as read only.
  */
 void EDVMimeTypesListSystemFileOpen(
+	const gchar *data_path,
 	edv_mime_type_struct ***list, gint *total,
 	const gint insert_index,
 	gint (*progress_cb)(gpointer, const gulong, const gulong),
@@ -237,6 +818,15 @@ void EDVMimeTypesListSystemFileOpen(
 			*icon_file_l[EDV_MIME_TYPE_TOTAL_ICON_STATES];
 	edv_mime_type_struct *m;
 	gint cur_insert_index = insert_index;
+	gchar *icons_dir = g_strconcat(
+	    data_path,
+	    G_DIR_SEPARATOR_S,
+	    EDV_DEF_ICONS_SUBDIR,
+	    NULL
+	);
+
+	if(icons_dir == NULL)
+	    return;
 
 	memset(icon_file_s, 0x00, EDV_MIME_TYPE_TOTAL_ICON_STATES * sizeof(const gchar *));
 	memset(icon_file_m, 0x00, EDV_MIME_TYPE_TOTAL_ICON_STATES * sizeof(const gchar *));
@@ -244,78 +834,63 @@ void EDVMimeTypesListSystemFileOpen(
 
 #define EDV_ICONS_DIR   PREFIX "/share/endeavour2/icons"
 
-#define APPEND_MIMETYPE(			\
- _val_,_type_,_desc_,				\
- _icon_file_s_,					\
- _icon_file_m_,					\
- _icon_file_l_					\
-) {						\
- gint mt_num;					\
-						\
- /* Insert? */					\
- if((cur_insert_index >= 0) &&			\
-    (cur_insert_index < *total)			\
- ) {						\
-  gint i = MAX(*total, 0);			\
-  *total = i + 1;				\
-  *list = (edv_mime_type_struct **)g_realloc(	\
-   *list,					\
-   (*total) * sizeof(edv_mime_type_struct *)	\
-  );						\
-  if(*list == NULL) {				\
-   *total = 0;					\
-   return;					\
-  }						\
-  mt_num = cur_insert_index;			\
-  for(i = (*total) - 1; i > mt_num; i--)	\
-   (*list)[i] = (*list)[i - 1];			\
-						\
-  (*list)[mt_num] = m = EDVMimeTypeNew(		\
-   EDV_MIME_TYPE_CLASS_SYSTEM, (_val_),		\
-   (_type_), (_desc_)				\
-  );						\
-  cur_insert_index++;				\
- } else {					\
-  mt_num = MAX(*total, 0);			\
-  *total = mt_num + 1;				\
-  *list = (edv_mime_type_struct **)g_realloc(	\
-   *list,					\
-   (*total) * sizeof(edv_mime_type_struct *)	\
-  );						\
-  if(*list == NULL) {				\
-   *total = 0;					\
-   return;					\
-  }						\
-  (*list)[mt_num] = m = EDVMimeTypeNew(		\
-   EDV_MIME_TYPE_CLASS_SYSTEM, (_val_),		\
-   (_type_), (_desc_)				\
-  );						\
-  cur_insert_index = -1;			\
- }						\
-						\
- if(m != NULL) {				\
-  gint n;					\
-						\
-  m->read_only = mark_all_loaded_read_only;	\
-  m->realized = FALSE;				\
-						\
-  for(n = 0; n < EDV_MIME_TYPE_TOTAL_ICON_STATES; n++) { \
-   m->small_icon_file[n] = STRDUP(PrefixPaths(	\
-    EDV_ICONS_DIR, (_icon_file_s_)[n]		\
-   ));						\
-   m->medium_icon_file[n] = STRDUP(PrefixPaths(	\
-    EDV_ICONS_DIR, (_icon_file_m_)[n]		\
-   ));						\
-   m->large_icon_file[n] = STRDUP(PrefixPaths(	\
-    EDV_ICONS_DIR, (_icon_file_l_)[n]		\
-   ));						\
-						\
-   if(added_cb != NULL)				\
-    added_cb(mt_num, m, added_data);		\
-  }						\
- }						\
+#define APPEND_MIMETYPE(				\
+ _val_,_type_,_desc_,					\
+ _icon_file_s_,						\
+ _icon_file_m_,						\
+ _icon_file_l_						\
+) {							\
+ gint mt_num;						\
+							\
+ /* Create a new MIME Type */				\
+ m = EDVMimeTypeNew(					\
+  EDV_MIME_TYPE_CLASS_SYSTEM, (_val_),			\
+  (_type_), (_desc_)					\
+ );							\
+							\
+ /* Insert/append the MIME Type to the list */		\
+ mt_num = EDVMimeTypesListInsert(			\
+  list, total, cur_insert_index, m			\
+ );							\
+ if(mt_num > -1) {					\
+  m = (*list)[mt_num];					\
+  if(m != NULL) {					\
+   /* Set the new MIME Type's values */			\
+   gint i;						\
+							\
+   m->read_only = mark_all_loaded_read_only;		\
+   m->realized = FALSE;					\
+							\
+   for(i = 0; i < EDV_MIME_TYPE_TOTAL_ICON_STATES; i++) { \
+    m->small_icon_file[i] = g_strconcat(		\
+     icons_dir,						\
+     G_DIR_SEPARATOR_S,					\
+     (_icon_file_s_)[i],				\
+     NULL						\
+    );							\
+    m->medium_icon_file[i] = g_strconcat(		\
+     icons_dir,						\
+     G_DIR_SEPARATOR_S,					\
+     (_icon_file_m_)[i],				\
+     NULL						\
+    );							\
+    m->large_icon_file[i] = g_strconcat(		\
+     icons_dir,						\
+     G_DIR_SEPARATOR_S,					\
+     (_icon_file_l_)[i],				\
+     NULL						\
+    );							\
+   }							\
+							\
+   /* Notify about this MIME Type being added */	\
+   if(added_cb != NULL)					\
+    added_cb(mt_num, m, added_data);			\
+  }							\
+  if(cur_insert_index > -1)				\
+   cur_insert_index++;					\
+ }							\
 }
-
+	/* Unknown */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_object_misc_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] =
@@ -342,6 +917,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Error */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_object_error_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] =
@@ -368,6 +944,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* File */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_file_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -391,6 +968,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Executable */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_executable_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -411,25 +989,29 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Directory */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_folder_closed_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] =
 	    "icon_folder_opened_20x20.xpm";
-	icon_file_s[EDV_MIME_TYPE_ICON_STATE_EXTENDED] = NULL;
+	icon_file_s[EDV_MIME_TYPE_ICON_STATE_EXTENDED] =
+	    "icon_folder_noaccess_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_HIDDEN] =
 	    "icon_folder_hidden_20x20.xpm";
 	icon_file_m[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_folder_closed_32x32.xpm";
 	icon_file_m[EDV_MIME_TYPE_ICON_STATE_SELECTED] =      
 	    "icon_folder_opened_32x32.xpm";
-	icon_file_m[EDV_MIME_TYPE_ICON_STATE_EXTENDED] = NULL;
+	icon_file_m[EDV_MIME_TYPE_ICON_STATE_EXTENDED] =
+	    "icon_folder_noaccess_32x32.xpm";
 	icon_file_m[EDV_MIME_TYPE_ICON_STATE_HIDDEN] =
 	    "icon_folder_hidden_32x32.xpm";
 	icon_file_l[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_folder_closed_48x48.xpm";
 	icon_file_l[EDV_MIME_TYPE_ICON_STATE_SELECTED] =
 	    "icon_folder_opened_48x48.xpm";
-	icon_file_l[EDV_MIME_TYPE_ICON_STATE_EXTENDED] = NULL;
+	icon_file_l[EDV_MIME_TYPE_ICON_STATE_EXTENDED] =
+	    "icon_folder_noaccess_48x48.xpm";
 	icon_file_l[EDV_MIME_TYPE_ICON_STATE_HIDDEN] =
 	    "icon_folder_hidden_48x48.xpm";
 	APPEND_MIMETYPE(
@@ -437,6 +1019,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Link */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_link2_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -460,6 +1043,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Device Block */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_device_block_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -480,6 +1064,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Device Character */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_device_character_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -500,6 +1085,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* FIFO Pipe */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_pipe_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -520,6 +1106,7 @@ void EDVMimeTypesListSystemFileOpen(
 	    icon_file_s, icon_file_m, icon_file_l
 	);
 
+	/* Socket */
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_STANDARD] =
 	    "icon_socket_20x20.xpm";
 	icon_file_s[EDV_MIME_TYPE_ICON_STATE_SELECTED] = NULL;
@@ -542,6 +1129,8 @@ void EDVMimeTypesListSystemFileOpen(
 
 #undef APPEND_MIMETYPE
 #undef EDV_ICONS_DIR
+
+	g_free(icons_dir);
 }
 
 
@@ -926,10 +1515,10 @@ void EDVMimeTypesListFileOpen(
 			    if(len > 0)
 				memcpy(cmd->name, command, len * sizeof(gchar));
 			    cmd->name[len] = '\0';
-			    strstrip((char *)cmd->name);
+			    cmd->name = g_strstrip(cmd->name);
 
 			    cmd->command = STRDUP(s_cmd);
-			    strstrip((char *)cmd->command);
+			    cmd->command = g_strstrip(cmd->command);
 			}
 			else
 			{
@@ -944,7 +1533,7 @@ void EDVMimeTypesListFileOpen(
 			    );
 
 			    cmd->command = STRDUP(command);
-			    strstrip((char *)cmd->command);
+			    cmd->command = g_strstrip(cmd->command);
 			}
 
 			mt->commands_list = g_list_append(
