@@ -153,6 +153,8 @@ void EDVMimeTypesListMailcapImport(
 	gpointer modified_data
 )
 {
+	struct stat stat_buf;
+	FILE *fp;
 	gboolean aborted = FALSE, in_quotes, mt_exists;
 	gint	c,
 		cur_insert_index = insert_index,
@@ -160,10 +162,8 @@ void EDVMimeTypesListMailcapImport(
 	gulong	file_size = 0l,
 		file_last_modified = 0l;
 	gchar *s, *s_end, buf[10000];
-	FILE *fp;
 	GList *val_list, *mt_imported_list = NULL;
 	edv_mime_type_struct *mt = NULL;
-	struct stat stat_buf;
 
 	if((list == NULL) || (total == NULL) || STRISEMPTY(path))
 	    return;
@@ -337,12 +337,11 @@ void EDVMimeTypesListMailcapImport(
 
 		/* Null terminate the value and strip spaces */
 		buf[sizeof(buf) - 1] = '\0';
-		strstrip(buf);
 
 		/* Append this value to the values list */
-		val_list = g_list_append(
-		    val_list, STRDUP(buf)
-		);
+		s = STRDUP(buf);
+		s = g_strstrip(s);
+		val_list = g_list_append(val_list, s);
 
 		/* End of line reached? */
 		if(ISCR(c))
@@ -362,9 +361,9 @@ void EDVMimeTypesListMailcapImport(
 	    /* Check if the MIME Type already exists? */
 	    if(update)
 	    {
-		GList *glist = g_list_nth(val_list, 0);
-		const gchar *type = (glist != NULL) ? 
-		    (gchar *)glist->data : NULL;
+		const gchar *type = (const gchar *)g_list_nth_data(
+		    val_list, 0
+		);
 		if(!STRISEMPTY(type))
 		{
 		    /* Check if this MIME Type is already in the list
@@ -381,25 +380,11 @@ void EDVMimeTypesListMailcapImport(
 			/* Check if the MIME Type was already defined
 			 * in this file
 			 */
-			for(glist = mt_imported_list;
-			    glist != NULL;
-			    glist = g_list_next(glist)
-			)
-			{
-			    if(EDV_MIME_TYPE(glist->data) == mt)
-				break;
-			}
-			if(glist != NULL)
+			if(g_list_find(mt_imported_list, mt) != NULL)
 			{
 			    /* Already defined in this file, so set
 			     * it NULL so that it is not defined again
 			     */
-#if 0
-			    g_printerr(
- "%s: Warning: Redefination of \"%s\" ignored.\n",
-				path, type
-			    );
-#endif
 			    mt = NULL;
 			    mt_num = -1;
 			}
@@ -468,10 +453,6 @@ void EDVMimeTypesListMailcapImport(
 		    );
 		    if(*list == NULL)
 		    {
-			g_printerr(
-			    "%s: Memory allocation error.\n",
-			    path
-			);
 		        *total = 0;
 		        break;
 		    }
@@ -840,11 +821,13 @@ void EDVMimeTypesListMailcapImport(
 		    }
 		    else
 		    {
+#if 0
 			g_printerr(
 "%s: MIME Type %s: Unsupported value \"%s\".\n",
 			    path, m->type,
 			    s
 			);
+#endif
 		    }
 
 
