@@ -117,8 +117,9 @@ void EDVMimeTypesListMediaTypesImport(
 	gpointer modified_data
 )
 {
-	gboolean aborted = FALSE, c_literal, mt_exists;
 	FILE *fp;
+	struct stat stat_buf;
+	gboolean aborted = FALSE, c_literal, mt_exists;
 	gint	i, c,
 		cur_insert_index = insert_index,
 		mt_num = -1;
@@ -129,7 +130,6 @@ void EDVMimeTypesListMediaTypesImport(
 		ext_str[2048];
 	GList *mt_imported_list = NULL;
 	edv_mime_type_struct *mt = NULL;
-	struct stat stat_buf;
 
 	if((list == NULL) || (total == NULL) || STRISEMPTY(path))
 	    return;
@@ -249,7 +249,6 @@ void EDVMimeTypesListMediaTypesImport(
 	    else
 		type_str[sizeof(type_str) - 1] = '\0';
 
-
 	    /* Begin fetching extensions up until the next newline
 	     * character
 	     */
@@ -300,31 +299,17 @@ void EDVMimeTypesListMediaTypesImport(
 		    );
 		    if(mt != NULL)
 		    {
-			GList *glist;
-
 			/* MIME Type already exists */
 			mt_exists = TRUE;
 
 			/* Check if the MIME Type was already defined
 			 * in this file
 			 */
-			for(glist = mt_imported_list;
-			    glist != NULL;
-			    glist = g_list_next(glist)
-			)
-			{
-			    if(EDV_MIME_TYPE(glist->data) == mt)
-				break;
-			}
-			if(glist != NULL)
+			if(g_list_find(mt_imported_list, mt) != NULL)
 			{
 			    /* Already defined in this file, so set
 			     * it NULL so that it is not defined again
 			     */
-			    g_printerr(
- "%s: Warning: Redefination of \"%s\" ignored.\n",
-				path, type_str
-			    );
 			    mt = NULL;
 			    mt_num = -1;
 			}
@@ -360,10 +345,6 @@ void EDVMimeTypesListMediaTypesImport(
 		    );
 		    if(*list == NULL)
 		    {
-			g_printerr(
-			    "%s: Memory allocation error.\n",
-			    path
-			);
 		        *total = 0;
 		        break;
 		    }
@@ -397,10 +378,6 @@ void EDVMimeTypesListMediaTypesImport(
 		    );
 		    if(*list == NULL)
 		    {
-			g_printerr(
-			    "%s: Memory allocation error.\n",
-			    path
-			);
 		        *total = 0;
 		        break;
 		    }
