@@ -1,5 +1,6 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -62,8 +63,7 @@ static void EDVMountBarSetStats(edv_mount_bar_struct *mb);
 
 /* Mount Bar */
 edv_mount_bar_struct *EDVMountBarNew(
-	edv_core_struct *core,
-	GtkWidget *parent,
+	edv_core_struct *core, GtkWidget *parent,
 	void (*mount_cb)(
 		edv_mount_bar_struct *, gint, edv_device_struct *,
 		gpointer
@@ -754,8 +754,6 @@ static void EDVMountBarPropertiesCB(GtkWidget *widget, gpointer data)
 	    return;
 
 	core = mb->core;
-	if(core == NULL)  
-	    return;
 
 	/* Get the selected device */
 	dev_num = mb->selected_dev_num;
@@ -768,15 +766,25 @@ static void EDVMountBarPropertiesCB(GtkWidget *widget, gpointer data)
 	if(STRISEMPTY(mount_path))
 	    return;
 
-	if(lstat(mount_path, &lstat_buf))
-	    return;
-
 	obj = EDVObjectNew();
 	if(obj == NULL)
 	    return;
 
 	EDVObjectSetPath(obj, mount_path);
-	EDVObjectSetStat(obj, &lstat_buf);
+	if(lstat((const char *)mount_path, &lstat_buf))
+	{
+            const gint error_code = (gint)errno;
+	    EDVObjectPropSet(
+		obj,
+		EDV_OBJECT_PROP_NAME_ERROR,
+		g_strerror(error_code),
+		TRUE
+	    );
+	}
+	else
+	{
+	    EDVObjectSetStat(obj, &lstat_buf);
+	}
 	EDVObjectUpdateLinkFlags(obj);
 
 	EDVNewPropertiesDialogPage(
@@ -1254,18 +1262,10 @@ static void EDVMountBarSetStats(edv_mount_bar_struct *mb)
 	    if(EDV_DEVICE_IS_MOUNTED(dev))
 	    {
 		const gchar *fmt;
-		gchar	*s_used = STRDUP(EDVGetObjectSizeStr(
-			core, used
-		)),
-			*s_total = STRDUP(EDVGetObjectSizeStr(
-			core, total
-		)),
-			*s_free_avail = STRDUP(EDVGetObjectSizeStr(
-			core, dev->blocks_available
-		)),
-			*s_free = STRDUP(EDVGetObjectSizeStr(
-			core, dev->blocks_free
-		));
+		gchar	*s_used = STRDUP(EDVSizeStrDelim(used)),
+			*s_total = STRDUP(EDVSizeStrDelim(total)),
+			*s_free_avail = STRDUP(EDVSizeStrDelim(dev->blocks_available)),
+			*s_free = STRDUP(EDVSizeStrDelim(dev->blocks_free));
 
 		switch(mb->dev_stats_page)
 		{
@@ -1339,8 +1339,7 @@ Free & Available: %s kb  Free: %s kb";
  *	Create a new mount bar.
  */
 edv_mount_bar_struct *EDVMountBarNew(
-	edv_core_struct *core,
-	GtkWidget *parent,
+	edv_core_struct *core, GtkWidget *parent,
 	void (*mount_cb)(
 		edv_mount_bar_struct *, gint, edv_device_struct *,
 		gpointer
@@ -1358,19 +1357,22 @@ edv_mount_bar_struct *EDVMountBarNew(
 )
 {
 	const gint border_minor = 2;
-	GtkWidget *w, *parent2, *parent3;
-	edv_mount_bar_struct *mb = EDV_MOUNT_BAR(
-	    g_malloc0(sizeof(edv_mount_bar_struct))
-	);
-	if((mb == NULL) || (core == NULL))
-	{
-	    g_free(mb);
-	    return(NULL);
-	}
+	GtkWidget *w, *parent2, *parent3, *toplevel;
+	edv_mount_bar_struct *mb;
 
-	mb->gc = NULL;
-	mb->busy_count = 0;
+	if(core == NULL)
+	    return(NULL);
+
+	mb = EDV_MOUNT_BAR(
+            g_malloc0(sizeof(edv_mount_bar_struct))
+        );
+	if(mb == NULL)
+	    return(NULL);
+
+	mb->toplevel = toplevel = gtk_hbox_new(FALSE, border_minor);
 	mb->freeze_count = 0;
+	mb->busy_count = 0;
+	mb->gc = NULL;
 	mb->core = core;
 	mb->selected_dev_num = 0;
 	mb->dev_text = NULL;
@@ -1385,11 +1387,11 @@ edv_mount_bar_struct *EDVMountBarNew(
 	mb->status_message_cb = status_message_cb;
 	mb->data = data;
 
+	mb->freeze_count++;
 
-	/* Set first device from the core structure as the initially
-	 * selected device. This first device is not always device
-	 * index 0, but rather the first one that is not marked as
-	 * unlisted
+	/* Set the first device as the initially selected device,
+	 * this first device is not always index 0, but rather the
+	 * first one that is not marked as unlisted
 	 */
 	if(core != NULL)
 	{
@@ -1401,6 +1403,7 @@ edv_mount_bar_struct *EDVMountBarNew(
 		dev = core->device[i];
 		if(dev == NULL)
 		    continue;
+
 		if(EDV_DEVICE_IS_UNLISTED(dev))
 		    continue;
 
@@ -1409,14 +1412,13 @@ edv_mount_bar_struct *EDVMountBarNew(
 	    }
 	}
 
-	/* Create toplevel */
-	mb->toplevel = w = gtk_hbox_new(FALSE, border_minor);
+	/* Toplevel GtkHBox */
+	w = toplevel;
 	gtk_container_border_width(GTK_CONTAINER(w), border_minor);
 	gtk_container_add(GTK_CONTAINER(parent), w);
 	parent = w;
 
-
-	/* GtkHbox for the devices list */
+	/* Devices list GtkHbox */
 	w = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1820,6 +1822,8 @@ edv_mount_bar_struct *EDVMountBarNew(
 
 	EDVMountBarUpdateMenus(mb);
 
+	mb->freeze_count--;
+
 	return(mb);
 }
 
@@ -1948,11 +1952,10 @@ void EDVMountBarUpdateMenus(edv_mount_bar_struct *mb)
  */
 void EDVMountBarMap(edv_mount_bar_struct *mb)
 {
-	GtkWidget *w = (mb != NULL) ? mb->toplevel : NULL;
-	if(w == NULL)
+	if(mb == NULL)
 	    return;
 
-	gtk_widget_show(w);
+	gtk_widget_show(mb->toplevel);
 }
 
 /*
@@ -1960,11 +1963,10 @@ void EDVMountBarMap(edv_mount_bar_struct *mb)
  */
 void EDVMountBarUnmap(edv_mount_bar_struct *mb)
 {
-	GtkWidget *w = (mb != NULL) ? mb->toplevel : NULL;
-	if(w == NULL)
+	if(mb == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(mb->toplevel);
 }
 
 /*
@@ -1975,27 +1977,17 @@ void EDVMountBarDelete(edv_mount_bar_struct *mb)
 	if(mb == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(mb->dev_da);
-	GTK_WIDGET_DESTROY(mb->map_btn);
-	GTK_WIDGET_DESTROY(mb->mount_btn);
-	GTK_WIDGET_DESTROY(mb->eject_btn);
-	GTK_WIDGET_DESTROY(mb->refresh_btn);
-	GTK_WIDGET_DESTROY(mb->goto_btn);
-	GTK_WIDGET_DESTROY(mb->properties_btn);
-	GTK_WIDGET_DESTROY(mb->devices_list_btn);
-	GTK_WIDGET_DESTROY(mb->fsck_btn);
-	GTK_WIDGET_DESTROY(mb->tools_btn);
-	GTK_WIDGET_DESTROY(mb->format_btn);
-	GTK_WIDGET_DESTROY(mb->stats_da);
-	GTK_WIDGET_DESTROY(mb->toplevel);
+	mb->freeze_count++;
 
+	gtk_widget_destroy(mb->toplevel);
 	GDK_GC_UNREF(mb->gc);
 
 	GDK_PIXMAP_UNREF(mb->dev_pixmap);
 	GDK_BITMAP_UNREF(mb->dev_mask);
 	g_free(mb->dev_text);
-
 	g_free(mb->dev_stats_label);
 
+	mb->freeze_count--;
+
 	g_free(mb);
 }
