@@ -13,11 +13,15 @@
 #include "edv_utils.h"
 
 
+/* Type */
 edv_object_type EDVObjectGetTypeFromStatMode(const mode_t m);
 mode_t EDVObjectGetTypeFromEDVType(const edv_object_type type);
+
+/* Type Name */
 const gchar *EDVObjectGetTypeName(const edv_object_type type);
 const gchar *EDVObjectGetTypeNameLower(const edv_object_type type);
 
+/* Permissions */
 edv_permission_flags EDVObjectGetPermissionsFromStatMode(
 	const mode_t m
 );
@@ -25,6 +29,7 @@ mode_t EDVObjectGetPermissionsFromEDVPermissions(
 	const edv_permission_flags permissions
 );
 
+/* Comparing */
 gboolean EDVObjectCompareIndex(
 	edv_object_struct *obj1, edv_object_struct *obj2
 );
@@ -32,8 +37,27 @@ gboolean EDVObjectCompareIndexWithStat(
 	edv_object_struct *obj, const struct stat *stat_buf
 );
 
+/* Object Property */
+void EDVObjectPropSet(
+	edv_object_struct *obj,
+	const gchar *name, const gchar *value,
+	const gboolean create_as_needed
+);
+void EDVObjectPropRemove(
+	edv_object_struct *obj,
+        const gchar *name
+);
+edv_object_prop_struct *EDVObjectPropGet(
+	edv_object_struct *obj, 
+        const gchar *name
+);
+edv_object_prop_struct *EDVObjectPropNew(void);
+edv_object_prop_struct *EDVObjectPropCopy(edv_object_prop_struct *prop);
+void EDVObjectPropDelete(edv_object_prop_struct *prop);
+
+/* Object */
 edv_object_struct *EDVObjectNew(void);
-edv_object_struct *EDVObjectCopy(const edv_object_struct *obj);
+edv_object_struct *EDVObjectCopy(edv_object_struct *obj);
 void EDVObjectSetPath(
 	edv_object_struct *obj, const gchar *path
 );
@@ -356,60 +380,246 @@ gboolean EDVObjectCompareIndexWithStat(
 }
 
 
+/*
+ *	Sets the propert value.
+ *
+ *	The obj specifies the object that the properties list is on.
+ *
+ *	The name specifies the name of the property to set.
+ *
+ *	The value specifies the property value to set. If value is NULL
+ *	or an empty string then the property will be removed.
+ *
+ *	If create_as_needed is TRUE and the property does not yet exist
+ *	then it will be created.
+ */
+void EDVObjectPropSet(
+	edv_object_struct *obj,
+	const gchar *name, const gchar *value,
+	const gboolean create_as_needed
+)
+{
+	edv_object_prop_struct *prop;
+
+	if((obj == NULL) || STRISEMPTY(name))
+	    return;
+
+	/* Remove the property? */
+	if(STRISEMPTY(value))
+	{
+	    EDVObjectPropRemove(obj, name);
+	    return;
+	}
+
+	/* Find the property that matches the specified name */
+	prop = EDVObjectPropGet(obj, name);
+	if(prop != NULL)
+	{
+	    /* Update the property name for case-insensitive corrections */
+	    g_free(prop->name);
+	    prop->name = STRDUP(name);
+	}
+	else
+	{
+	    /* No such property exists, do not create it? */
+	    if(!create_as_needed)
+		return;
+
+	    /* Create the property */
+	    prop = EDVObjectPropNew();
+	    if(prop == NULL)
+		return;
+
+	    /* Set the new property's initial values and append it to
+	     * the properties list
+	     */
+	    prop->name = STRDUP(name);
+	    obj->ext_props_list = g_list_append(
+		obj->ext_props_list,
+		prop
+	    );
+	}
+
+	/* Set the new value */
+	g_free(prop->value);
+	prop->value = STRDUP(value);
+}
+
+/*
+ *	Removes the property.
+ *
+ *	The obj specifies the object that the properties list is on.
+ *
+ *	The name specifies the name of the property to remove.
+ */
+void EDVObjectPropRemove(
+	edv_object_struct *obj,
+        const gchar *name
+)
+{
+	edv_object_prop_struct *prop;
+
+	if((obj == NULL) || STRISEMPTY(name))
+	    return;
+
+	/* Find the property that matches the specified name */
+	prop = EDVObjectPropGet(obj, name);
+	if(prop == NULL)
+	    return;
+
+	/* Remove this property from the list */
+	obj->ext_props_list = g_list_remove(
+	    obj->ext_props_list,
+	    prop
+	);
+}
+
+/*
+ *	Gets the property.
+ *
+ *	The obj specifies the object that the properties list is on.
+ *
+ *	The name specifies the name of the property to get.
+ *
+ *	Returns the property or NULL on error.
+ */
+edv_object_prop_struct *EDVObjectPropGet(
+	edv_object_struct *obj,
+        const gchar *name
+)
+{
+	GList *glist;
+	edv_object_prop_struct *prop;
+
+	if((obj == NULL) || STRISEMPTY(name))
+	    return(NULL);
+
+	for(glist = obj->ext_props_list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
+	{
+	    prop = EDV_OBJECT_PROP(glist->data);
+	    if(prop == NULL)
+		continue;
+
+	    if(prop->name == NULL)
+		continue;
+
+	    if(!g_strcasecmp(prop->name, name))
+		return(prop);
+	}
+
+	return(NULL);
+}
+
+/*
+ *	Creates a new Property.
+ */
+edv_object_prop_struct *EDVObjectPropNew(void)
+{
+	return(EDV_OBJECT_PROP(g_malloc0(sizeof(edv_object_prop_struct))));
+}
+
+/*
+ *	Coppies the Property.
+ */
+edv_object_prop_struct *EDVObjectPropCopy(edv_object_prop_struct *prop)
+{
+	edv_object_prop_struct *new_prop;
+
+	if(prop == NULL)
+	    return(NULL);
+
+	new_prop = EDVObjectPropNew();
+	if(new_prop == NULL)
+	    return(NULL);
+
+	new_prop->name = STRDUP(prop->name);
+	new_prop->value = STRDUP(prop->value);
+
+	return(new_prop);
+}
+
+/*
+ *	Deletes the Property.
+ */
+void EDVObjectPropDelete(edv_object_prop_struct *prop)
+{
+	if(prop == NULL)
+	    return;
+
+	g_free(prop->name);
+	g_free(prop->value);
+	g_free(prop);
+}
+
+
 /*
  *	Creates a new Object.
  */
 edv_object_struct *EDVObjectNew(void)
 {
-	return(EDV_OBJECT(
-	    g_malloc0(sizeof(edv_object_struct))
-	));
+	return(EDV_OBJECT(g_malloc0(sizeof(edv_object_struct))));
 }
 
 /*
  *	Coppies the object.
  */
-edv_object_struct *EDVObjectCopy(const edv_object_struct *obj)
+edv_object_struct *EDVObjectCopy(edv_object_struct *obj)
 {
-	const edv_object_struct *src = obj;
-	edv_object_struct *tar;
+	edv_object_struct *new_obj;
 
-	if(src == NULL)
+	if(obj == NULL)
 	    return(NULL);
 
-	/* Create target Object and copy values to it */
-	tar = EDVObjectNew();
+	new_obj = EDVObjectNew();
+	if(new_obj == NULL)
+	    return(NULL);
 
-	tar->type = src->type;
+	new_obj->type = obj->type;
 
-	tar->name = STRDUP(src->name);
-	tar->full_path = STRDUP(src->full_path);
+	new_obj->name = STRDUP(obj->name);
+	new_obj->full_path = STRDUP(obj->full_path);
 
-	tar->device_index = src->device_index;
-	tar->index = src->index;
+	new_obj->device_index = obj->device_index;
+	new_obj->index = obj->index;
 
-	tar->size = src->size;
+	new_obj->size = obj->size;
 
-	tar->link_value = STRDUP(src->link_value);
-	tar->link_flags = src->link_flags;
+	new_obj->link_target = STRDUP(obj->link_target);
+	new_obj->link_flags = obj->link_flags;
 
-	tar->permissions = src->permissions;
+	new_obj->permissions = obj->permissions;
 
-	tar->access_time = src->access_time;
-	tar->modify_time = src->modify_time;
-	tar->change_time = src->change_time;
+	new_obj->access_time = obj->access_time;
+	new_obj->modify_time = obj->modify_time;
+	new_obj->change_time = obj->change_time;
 
-	tar->owner_id = src->owner_id;
-	tar->group_id = src->group_id;
+	new_obj->owner_id = obj->owner_id;
+	new_obj->group_id = obj->group_id;
 
-	tar->device_type = src->device_type;
+	new_obj->device_type = obj->device_type;
 
-	tar->block_size = src->block_size;
-	tar->blocks = src->blocks;
+	new_obj->block_size = obj->block_size;
+	new_obj->blocks = obj->blocks;
 
-	tar->hard_links = src->hard_links;
+	new_obj->hard_links = obj->hard_links;
 
-	return(tar);
+	if(obj->ext_props_list != NULL)
+	{
+	    GList *glist;
+	    for(glist = obj->ext_props_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
+		new_obj->ext_props_list = g_list_append(
+		    new_obj->ext_props_list,
+		    EDVObjectPropCopy(EDV_OBJECT_PROP(glist->data))
+		);
+	}
+
+	return(new_obj);
 }
 
 
@@ -425,7 +635,7 @@ void EDVObjectSetPath(
 	if(obj == NULL)
 	    return;
 
-	/* Reset names first */
+	/* Reset the name and path first */
 	g_free(obj->name);
 	obj->name = NULL;
 	g_free(obj->full_path);
@@ -461,7 +671,7 @@ void EDVObjectSetPath2(
 	if(obj == NULL)
 	    return;
 
-	/* Reset names first */
+	/* Reset the names first */
 	g_free(obj->name);
 	obj->name = NULL;
 	g_free(obj->full_path);
@@ -528,8 +738,10 @@ void EDVObjectSetStat(
 	obj->device_type = (gint)lstat_buf->st_rdev;
 
 	/* Block transfer size and blocks allocated */
+#if !defined(_WIN32)
 	obj->block_size = (gulong)lstat_buf->st_blksize;
 	obj->blocks = (gulong)lstat_buf->st_blocks;
+#endif
 
 	/* Number of hard links */
 	obj->hard_links = (gint)lstat_buf->st_nlink;
@@ -538,8 +750,8 @@ void EDVObjectSetStat(
 	/* If this is a link then update the Object's link_value */
 	if(S_ISLNK(m))
 	{
-	    g_free(obj->link_value);
-	    obj->link_value = EDVGetLinkValue(obj->full_path);
+	    g_free(obj->link_target);
+	    obj->link_target = EDVGetLinkTarget(obj->full_path);
 	}
 #endif
 }
@@ -562,7 +774,7 @@ void EDVObjectUpdateLinkFlags(edv_object_struct *obj)
 	obj->link_flags = 0;
 
 	if(!EDV_OBJECT_IS_LINK(obj) ||
-	   (obj->full_path == NULL) || (obj->link_value == NULL)
+	   (obj->full_path == NULL) || (obj->link_target == NULL)
 	)
 	    return;
 
@@ -570,7 +782,7 @@ void EDVObjectUpdateLinkFlags(edv_object_struct *obj)
 	if(parent == NULL)
 	    return;
 
-	path = EDVEvaluatePath(parent, obj->link_value);
+	path = EDVEvaluatePath(parent, obj->link_target);
 	if(path == NULL)
 	{
 	    g_free(parent);
@@ -604,8 +816,11 @@ void EDVObjectDelete(edv_object_struct *obj)
 
 	g_free(obj->name);
 	g_free(obj->full_path);
-
-	g_free(obj->link_value);
-
+	g_free(obj->link_target);
+	if(obj->ext_props_list != NULL)
+	{
+	    g_list_foreach(obj->ext_props_list, (GFunc)EDVObjectPropDelete, NULL);
+	    g_list_free(obj->ext_props_list);
+	}
 	g_free(obj);
 }
