@@ -12,7 +12,16 @@
 
 
 typedef struct _edv_object_struct	edv_object_struct;
-#define EDV_OBJECT(p)	((edv_object_struct *)(p))
+#define EDV_OBJECT(p)			((edv_object_struct *)(p))
+typedef struct _edv_object_prop_struct	edv_object_prop_struct;
+#define EDV_OBJECT_PROP(p)		((edv_object_prop_struct *)(p))
+
+
+/*
+ *	Standard Property Names:
+ */
+#define EDV_OBJECT_PROP_NAME_ERROR	"Error"
+#define EDV_OBJECT_PROP_NAME_NOTES	"Notes"
 
 
 /*
@@ -52,7 +61,7 @@ struct _edv_object_struct {
 	gulong		size;		/* In bytes */
 
 	/* Link (for type EDV_OBJECT_TYPE_LINK) */
-	gchar		*link_value;
+	gchar		*link_target;
 	edv_object_link_flags	link_flags;
 
 	/* Permissions */
@@ -85,6 +94,9 @@ struct _edv_object_struct {
 
 	gint		hard_links;	/* Number of hard links to this object */
 
+	/* Extended Properties List */
+	GList		*ext_props_list;	/* GList of edv_object_prop_struct * */
+
 };
 
 #define EDV_OBJECT_IS_FILE(p)		(((p) != NULL) ? \
@@ -110,6 +122,17 @@ struct _edv_object_struct {
  ((p)->link_flags & EDV_OBJECT_LINK_TAR_GRAND_PARENT) : FALSE)
 
 
+/*
+ *	Object Property:
+ */
+struct _edv_object_prop_struct {
+
+	gchar	*name,
+		*value;
+
+};
+
+
 /* Type */
 extern edv_object_type EDVObjectGetTypeFromStatMode(const mode_t m);
 extern mode_t EDVObjectGetTypeFromEDVType(const edv_object_type type);
@@ -134,9 +157,27 @@ extern gboolean EDVObjectCompareIndexWithStat(
 	edv_object_struct *obj, const struct stat *stat_buf
 );
 
+/* Object Property */
+extern void EDVObjectPropSet(
+        edv_object_struct *obj,
+        const gchar *name, const gchar *value,
+        const gboolean create_as_needed
+);
+extern void EDVObjectPropRemove(
+        edv_object_struct *obj,
+        const gchar *name
+);
+extern edv_object_prop_struct *EDVObjectPropGet(
+        edv_object_struct *obj,
+        const gchar *name
+);
+extern edv_object_prop_struct *EDVObjectPropNew(void);
+extern edv_object_prop_struct *EDVObjectPropCopy(edv_object_prop_struct *prop);
+extern void EDVObjectPropDelete(edv_object_prop_struct *prop);
+
 /* Object */
 extern edv_object_struct *EDVObjectNew(void);
-extern edv_object_struct *EDVObjectCopy(const edv_object_struct *obj);
+extern edv_object_struct *EDVObjectCopy(edv_object_struct *obj);
 extern void EDVObjectSetPath(
 	edv_object_struct *obj, const gchar *path
 );
