@@ -10,15 +10,16 @@
 #include <unistd.h>
 
 #include "../include/string.h"
-#include "../include/fio.h"
 #include "../include/disk.h"
 
 #include "guiutils.h"
 #include "cdialog.h"
+#include "pdialog.h"
 #include "progressdialog.h"
 
 #include "edv_types.h"
 #include "edv_obj_create.h"
+#include "endeavour2.h"
 #include "edv_utils.h"
 #include "edv_utils_gtk.h"
 #include "config.h"
@@ -30,22 +31,36 @@
  *	0	Success.                        
  *	-1	General error.
  *	-2	Invalid value.
- *	-3	Systems error; out of memory or out of disk space.    
+ *	-3	Systems error, out of memory, or out of disk space.    
  *	-4	User responded with "Cancel".                     
  *	-5	User responded with "No" or response was not available.
- *	-6	Call would cause reentry.    
+ *	-6	An operation is already in progress.    
  */
 
 
 /* Error Message */  
-const gchar *EDVObjCreateGetError(void);
-static void EDVObjCreateCopyErrorMessage(const gchar *msg);
+const gchar *EDVObjCreateGetError(edv_core_struct *core);
+static void EDVObjCreateCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+);
 
 /* New Path */
 static gchar *EDVObjCreateGenerateNewPath(
 	const gchar *path, const edv_object_type type
 );
 
+/* Create New File Query Name */
+gint EDVObjCreateFileQueryName(
+	edv_core_struct *core,
+	const gchar *parent_path,
+        const gchar *suggested_name,
+	gchar **new_path_rtn,
+	GtkWidget *toplevel,
+	const gboolean show_progress,
+	gboolean *yes_to_all
+);
+
+/* Create New Object Nexus */
 static gint EDVObjCreateNexus(
 	edv_core_struct *core,
 	const gchar *path,
@@ -56,6 +71,7 @@ static gint EDVObjCreateNexus(
 	gboolean *yes_to_all
 );
 
+/* Create New File */
 gint EDVObjCreateFile(
 	edv_core_struct *core,
 	const gchar *path,
@@ -64,6 +80,7 @@ gint EDVObjCreateFile(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New Directory */
 gint EDVObjCreateDirectory(
 	edv_core_struct *core,
 	const gchar *path,
@@ -72,6 +89,7 @@ gint EDVObjCreateDirectory(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New Link */
 gint EDVObjCreateLink(
 	edv_core_struct *core,
 	const gchar *path,
@@ -80,6 +98,7 @@ gint EDVObjCreateLink(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New FIFO Pipe */
 gint EDVObjCreateFifo(
 	edv_core_struct *core,
 	const gchar *path,
@@ -88,6 +107,7 @@ gint EDVObjCreateFifo(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New Block Device */
 gint EDVObjCreateDeviceBlock(
 	edv_core_struct *core,
 	const gchar *path,
@@ -96,6 +116,7 @@ gint EDVObjCreateDeviceBlock(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New Character Device */
 gint EDVObjCreateDeviceCharacter(
 	edv_core_struct *core,
 	const gchar *path,
@@ -104,6 +125,7 @@ gint EDVObjCreateDeviceCharacter(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+/* Create New Socket */
 gint EDVObjCreateSocket(
 	edv_core_struct *core,
 	const gchar *path,
@@ -114,20 +136,16 @@ gint EDVObjCreateSocket(
 );
 
 
-static const gchar *last_error = NULL;
-static gchar last_error_buf[256];
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -135,23 +153,28 @@ static gchar last_error_buf[256];
  *
  *      The returned pointer must not be deallocated.
  */
-const gchar *EDVObjCreateGetError(void)
+const gchar *EDVObjCreateGetError(edv_core_struct *core)
 {
-	return(last_error);
+	return((core != NULL) ? core->obj_last_error : NULL);
 }
 
 /*
- *	Coppies the error message specified by msg to the last error
- *	message buffer and sets last_error to point to that buffer.
+ *	Coppies the error message to the core's obj_last_error_buf
+ *	and sets the core's obj_last_error to point to it.
  */
-static void EDVObjCreateCopyErrorMessage(const gchar *msg)
+static void EDVObjCreateCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+)
 {
-	if(msg == NULL)
+	if(core == NULL)
 	    return;
 
-	strncpy(last_error_buf, msg, sizeof(last_error_buf));
-	last_error_buf[sizeof(last_error_buf) - 1] = '\0';
-	last_error = last_error_buf;
+	core->obj_last_error = NULL;
+
+	g_free(core->obj_last_error_buf);
+	core->obj_last_error_buf = STRDUP(msg);
+
+	core->obj_last_error = core->obj_last_error_buf;
 }
 
 
@@ -171,6 +194,7 @@ static gchar *EDVObjCreateGenerateNewPath(
 	const gchar *path, const edv_object_type type
 )
 {
+	struct stat lstat_buf;
 	guint16 i;
 	gchar *type_str, *new_path;
 
@@ -192,18 +216,25 @@ static gchar *EDVObjCreateGenerateNewPath(
 	    type_str,
 	    NULL
 	);
-
 	if(new_path == NULL)
 	{
 	    g_free(type_str);
 	    return(NULL);
 	}
 
-	/* Does the new path not refer to an existing object? */
-	if(access((const char *)new_path, F_OK))
+	/* Does the new path refer to a non-existant object? */
+	if(lstat((const char *)new_path, &lstat_buf))
 	{
-	    g_free(type_str);
-	    return(new_path);
+	    const gint error_code = (gint)errno;
+#ifdef ENOENT
+	    if(error_code == ENOENT)
+#else
+	    if(FALSE)
+#endif
+	    {
+		g_free(type_str);
+		return(new_path);
+	    }
 	}
 
 	/* An object already exists at the generated new path, so
@@ -221,12 +252,22 @@ static gchar *EDVObjCreateGenerateNewPath(
 		"%s%cnew_%s%u",
 		path, G_DIR_SEPARATOR, type_str, i
 	    );
+	    if(new_path == NULL)
+		break;
 
-	    /* Does the new path not refer to an existing object? */
-	    if(access((const char *)new_path, F_OK))
+	    /* Does the new path refer to a non-existant object? */
+	    if(lstat((const char *)new_path, &lstat_buf))
 	    {
-		g_free(type_str);
-		return(new_path);
+		const gint error_code = (gint)errno;
+#ifdef ENOENT
+		if(error_code == ENOENT)
+#else
+		if(FALSE)
+#endif
+		{
+		    g_free(type_str);
+		    return(new_path);
+		}
 	    }
 	}
 
@@ -241,6 +282,260 @@ static gchar *EDVObjCreateGenerateNewPath(
 	return(NULL);
 }
 
+
+/*
+ *	Prompts the user to create a new file.
+ *
+ *	The parent_path specifies the full path to the directory at
+ *	which the new object is to be created at.
+ *
+ *	The suggested_name specifies the suggested name for the new
+ *	file or NULL.
+ *
+ *	If new_path_rtn is not NULL then a dynamically allocated
+ *	string describing the full path to the new object will be set
+ *	to it.
+ *
+ *	If an error occures then the last error will be set to the
+ *	error message that describes what error occured.
+ *
+ *	Returns 0 on success and non-zero on error.
+ */
+gint EDVObjCreateFileQueryName(
+	edv_core_struct *core,
+	const gchar *parent_path,
+        const gchar *suggested_name,
+	gchar **new_path_rtn,
+	GtkWidget *toplevel,
+	const gboolean show_progress,
+	gboolean *yes_to_all
+)
+{
+	const gboolean interactive = TRUE;
+	struct stat lstat_buf;
+	gint status, fd, strc;
+	guint m;
+	gulong time_start;
+	gchar *name, *path, **strv;
+
+	if(new_path_rtn != NULL)
+	    *new_path_rtn = NULL;
+
+	/* Clear the last error message */
+	EDVObjCreateCopyErrorMessage(core, NULL);
+
+	if((core == NULL) || STRISEMPTY(parent_path) ||
+	   (yes_to_all == NULL)
+	)
+	{
+	    EDVObjCreateCopyErrorMessage(core, "Invalid value.");
+	    return(-2);
+	}
+
+        /* Is there an operation already in progress? */
+        if(core->op_level > 0)
+        {
+            core->obj_last_error =
+"An operation is already in progress, please try again later.";
+            return(-6);
+        }
+
+        core->op_level++;
+
+	if(!g_path_is_absolute(parent_path))
+	{
+	    core->obj_last_error = "Invalid value.";
+	    core->op_level--;
+	    return(-2);
+	}
+
+	/* Prompt the user for a file name */
+        PDialogDeleteAllPrompts();
+        PDialogSetTransientFor(toplevel);
+        PDialogAddPrompt(NULL, "New Name:", suggested_name);
+        PDialogSetSize(320, -1);
+        strv = PDialogGetResponse(
+            "Create New File",
+            NULL, NULL,
+            PDIALOG_ICON_FILE,
+            "Create", "Cancel",
+            PDIALOG_BTNFLAG_SUBMIT | PDIALOG_BTNFLAG_CANCEL,
+            PDIALOG_BTNFLAG_SUBMIT,
+            &strc
+        );
+        PDialogSetTransientFor(NULL);
+	if((strv == NULL) || (strc < 1))
+	{
+	    PDialogDeleteAllPrompts();
+	    core->op_level--;
+	    return(-4);
+	}
+
+	/* Get a copy of the name */
+	name = STRDUP(strv[0]);
+
+	PDialogDeleteAllPrompts();
+
+	/* Unable to get a copy of the name? */
+	if(name == NULL)
+	{
+	    core->obj_last_error = "Memory allocation error.";
+	    core->op_level--;
+            return(-3);
+	}
+
+	/* Generate the full path to the new object to be created */
+	path = g_strconcat(
+	    parent_path,
+	    G_DIR_SEPARATOR_S,
+	    name,
+	    NULL
+	);
+	if(path == NULL)
+	{
+	    g_free(name);
+	    core->obj_last_error = "Memory allocation error.";
+	    core->op_level--;
+            return(-3);
+	}
+
+	EDVSimplifyPath(path);
+
+	/* Check if the object already exists */
+	if(!stat((const char *)path, &lstat_buf))
+	{
+	    if(interactive)
+	    {
+		/* Confirm overwrite */
+		gint response;
+                gchar *msg = g_strdup_printf(
+"Overwrite existing file:\n\
+\n\
+    %s",
+		    path
+		);
+                EDVPlaySoundWarning(core);
+                CDialogSetTransientFor(toplevel);
+                response = CDialogGetResponse(
+#if defined(PROG_LANGUAGE_SPANISH)
+"Confirme Escriba Para Reemplazar"
+#elif defined(PROG_LANGUAGE_FRENCH)
+"Confirmer craser"
+#elif defined(PROG_LANGUAGE_GERMAN)
+"Besttigen Sie berschreibt"
+#elif defined(PROG_LANGUAGE_ITALIAN)
+"Confermare Sovrascrivere"
+#elif defined(PROG_LANGUAGE_DUTCH)
+"Bevestiig Beschrijft"   
+#elif defined(PROG_LANGUAGE_PORTUGUESE)
+"Confirme Overwrite"
+#elif defined(PROG_LANGUAGE_NORWEGIAN)
+"Bekreft Overskriver"
+#else
+"Confirm Overwrite"
+#endif
+                    , msg,
+                    NULL,
+                    CDIALOG_ICON_WARNING,
+                    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+                    CDIALOG_BTNFLAG_NO
+                );
+                g_free(msg);
+                CDialogSetTransientFor(NULL);
+                if((response != CDIALOG_RESPONSE_YES) &&
+		   (response != CDIALOG_RESPONSE_YES_TO_ALL)
+		)
+		{
+		    g_free(name);
+		    g_free(path);
+	            core->op_level--;
+	            return(-5);
+		}
+		if(response == CDIALOG_RESPONSE_YES_TO_ALL)
+		    *yes_to_all = TRUE;
+	    }
+
+	    /* Remove the existing file */
+	    if(unlink((const char *)path))
+	    {
+		const gint error_code = (gint)errno;
+		gchar *msg = g_strconcat(
+		    "Unable to create a new ",
+		    EDVObjectGetTypeNameLower(EDV_OBJECT_TYPE_FILE),
+		    ", ",
+		    g_strerror(error_code),
+		    ".",
+		    NULL
+		);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
+		g_free(name);
+		g_free(path);
+	        core->op_level--;
+	        return(-1);
+	    }
+	}
+
+	time_start = (gulong)time(NULL);
+	status = 0;
+
+	/* Create the new file */
+	m = EDVGetUMask();
+	fd = (gint)open(
+	    (const char *)path,
+	    O_WRONLY | O_CREAT | O_EXCL,
+	    (mode_t)(~m) &
+		(S_IRUSR | S_IWUSR |
+		 S_IRGRP | S_IWGRP |
+		 S_IROTH | S_IWOTH)
+	);
+	if(fd > -1)
+	{
+	    close((int)fd);
+	}
+	else
+	{
+	    const gint error_code = (gint)errno;
+	    gchar *msg = g_strconcat(
+		"Unable to create a new ",
+		EDVObjectGetTypeNameLower(EDV_OBJECT_TYPE_FILE),
+		", ",
+		g_strerror(error_code),
+		".",
+	        NULL
+	    );
+	    EDVObjCreateCopyErrorMessage(core, msg);
+	    g_free(msg);
+	    status = -1;
+	}
+
+	/* Update new path return if creation of object was successful */
+	if((status == 0) && (new_path_rtn != NULL))
+	{
+	    g_free(*new_path_rtn);
+	    *new_path_rtn = STRDUP(path);
+	}
+
+	/* Record history */
+	EDVAppendHistory(
+	    core,
+	    EDV_HISTORY_DISK_OBJECT_CREATE,
+	    time_start, (gulong)time(NULL),
+	    status,
+	    parent_path,			/* Source */
+	    path,				/* Target */
+	    core->obj_last_error		/* Comment */
+	);
+
+	g_free(name);
+	g_free(path);
+
+	core->op_level--;
+
+	return(status);
+}
+
+
 /*
  *	Creates a new object of the specified type.
  *
@@ -252,8 +547,8 @@ static gchar *EDVObjCreateGenerateNewPath(
  *	string describing the full path to the new object will be set
  *	to it.
  *
- *	If an error occures then last_error will be set to the error
- *	message that describes what error occured.
+ *	If an error occures then the last error will be set to the
+ *	error message that describes what error occured.
  *
  *	Returns 0 on success and non-zero on error.
  */
@@ -267,7 +562,6 @@ static gint EDVObjCreateNexus(
 	gboolean *yes_to_all
 )
 {
-	static gboolean reenterent = FALSE;
 	const gulong time_start = (gulong)time(NULL);
 	struct stat stat_buf;
 	gint status;
@@ -276,35 +570,34 @@ static gint EDVObjCreateNexus(
 	if(new_path_rtn != NULL)
 	    *new_path_rtn = NULL;
 
-	/* Leave the value of yes_to_all as is */
-
-	if(reenterent)
-	{
-	    last_error =
-"An operation is already in progress, please try again later";
-	    return(-6);
-	}
-	else
-	{
-	    reenterent = TRUE;
-	}
-
-	last_error = NULL;
-
-#define DO_FREE_LOCALS	{	\
- g_free(lpath);			\
- g_free(new_path);		\
-}
+	/* Clear the last error message */
+	EDVObjCreateCopyErrorMessage(core, NULL);
 
 	if((core == NULL) || STRISEMPTY(path) ||
 	   (yes_to_all == NULL)
 	)
 	{
-	    DO_FREE_LOCALS
-	    reenterent = FALSE;
+	    EDVObjCreateCopyErrorMessage(core, "Invalid value.");
 	    return(-2);
 	}
 
+        /* Is there an operation already in progress? */
+        if(core->op_level > 0)
+        {
+            core->obj_last_error =
+"An operation is already in progress, please try again later.";
+            return(-6);
+        }
+
+        core->op_level++;
+
+#define CLEANUP_RETURN(_v_)	{	\
+ g_free(lpath);				\
+ g_free(new_path);			\
+					\
+ return(_v_);				\
+}
+
 	/* Copy the specified path */
 	lpath = STRDUP(path);
 
@@ -315,7 +608,11 @@ static gint EDVObjCreateNexus(
 	if(stat((const char *)lpath, &stat_buf))
 	{
 	    const gint error_code = (gint)errno;
+#ifdef ENOENT
 	    if(error_code == ENOENT)
+#else
+	    if(FALSE)
+#endif
 	    {
 		/* No such object exists at the specified path's
 		 * location so use it as the path to the new object
@@ -324,18 +621,18 @@ static gint EDVObjCreateNexus(
 	    }
 	    else
 	    {
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
-		DO_FREE_LOCALS
-		reenterent = FALSE;
-		return(-1);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
+		core->op_level--;
+		CLEANUP_RETURN(-1);
 	    }
 	}
 	else
@@ -351,11 +648,10 @@ static gint EDVObjCreateNexus(
 	    if(TRUE)
 #endif
 	    {
-		last_error =
-"The location to create the new object at is not a directory";
-		DO_FREE_LOCALS
-		reenterent = FALSE;
-		return(-1);
+		core->obj_last_error =
+"The location to create the new object at is not a directory.";
+		core->op_level--;
+		CLEANUP_RETURN(-1);
 	    }
 
 	    /* Generate a new name based on the specified path */
@@ -364,10 +660,9 @@ static gint EDVObjCreateNexus(
 	/* Unable to generate the full path to the new object? */
 	if(new_path == NULL)
 	{
-	    last_error = "Unable to generate the full path for the new object";
-	    DO_FREE_LOCALS
-	    reenterent = FALSE;
-	    return(-2);
+	    core->obj_last_error = "Unable to generate the full path for the new object.";
+	    core->op_level--;
+	    CLEANUP_RETURN(-1);
 	}
 
 	/* At this point new_path specifies the full path to the new
@@ -380,23 +675,32 @@ static gint EDVObjCreateNexus(
 	/* File? */
 	if(type == EDV_OBJECT_TYPE_FILE)
 	{
-	    FILE *fp = fopen((const char *)new_path, "wb");
-	    if(fp != NULL)
+	    const guint m = EDVGetUMask();
+	    const gint fd = (gint)open(
+		(const char *)new_path,
+		O_WRONLY | O_CREAT | O_EXCL,
+		(mode_t)(~m) &
+		    (S_IRUSR | S_IWUSR |
+		     S_IRGRP | S_IWGRP |
+		     S_IROTH | S_IWOTH)
+	    );
+	    if(fd > -1)
 	    {
-		fclose(fp);
+		close((int)fd);
 	    }
 	    else
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 	}
@@ -413,15 +717,16 @@ static gint EDVObjCreateNexus(
 	    ))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 	}
@@ -431,15 +736,16 @@ static gint EDVObjCreateNexus(
 	    if(symlink("undefined", (const char *)new_path))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 	}
@@ -467,19 +773,20 @@ static gint EDVObjCreateNexus(
 	    ))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 #else
-	    last_error = "Unsupported object type";
+	    core->obj_last_error = "Unsupported object type.";
 	    status = -2;
 #endif
 	}
@@ -500,19 +807,20 @@ static gint EDVObjCreateNexus(
 	    ))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 #else
-	    last_error = "Unsupported object type";
+	    core->obj_last_error = "Unsupported object type.";
 	    status = -2;
 #endif
 	}
@@ -533,19 +841,20 @@ static gint EDVObjCreateNexus(
 	    ))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 #else
-	    last_error = "Unsupported object type";
+	    core->obj_last_error = "Unsupported object type.";
 	    status = -2;
 #endif
 	}
@@ -565,25 +874,26 @@ static gint EDVObjCreateNexus(
 	    ))
 	    {
 		const gint error_code = (gint)errno;
-		gchar *s = g_strconcat(
+		gchar *msg = g_strconcat(
 		    "Unable to create a new ",
 		    EDVObjectGetTypeNameLower(type),
 		    ", ",
 		    g_strerror(error_code),
+		    ".",
 		    NULL
 		);
-		EDVObjCreateCopyErrorMessage(s);
-		g_free(s);
+		EDVObjCreateCopyErrorMessage(core, msg);
+		g_free(msg);
 		status = -1;
 	    }
 #else
-	    last_error = "Unsupported object type";
+	    core->obj_last_error = "Unsupported object type.";
 	    status = -2;
 #endif
 	}
 	else
 	{
-	    last_error = "Unsupported object type";
+	    core->obj_last_error = "Unsupported object type.";
 	    status = -2;
 	}
 
@@ -601,17 +911,15 @@ static gint EDVObjCreateNexus(
 	    EDV_HISTORY_DISK_OBJECT_CREATE,
 	    time_start, (gulong)time(NULL),
 	    status,
-	    path,		/* Source */
-	    new_path,		/* Target */
-	    last_error		/* Comment */
+	    path,				/* Source */
+	    new_path,				/* Target */
+	    core->obj_last_error		/* Comment */
 	);
 
-	DO_FREE_LOCALS
+	core->op_level--;
 
-	reenterent = FALSE;
-
-	return(status);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(status);
+#undef CLEANUP_RETURN
 }
 
 
