@@ -11,7 +11,7 @@
 #include "edv_types.h"
 
 
-extern const gchar *EDVObjectOPGetError(void);
+extern const gchar *EDVObjectOPGetError(edv_core_struct *core);
 
 extern gint EDVObjectOPCopy(
 	edv_core_struct *core,
@@ -29,6 +29,9 @@ extern gint EDVObjectOPMove(
 	const gboolean show_progress, const gboolean interactive,
 	gboolean *yes_to_all
 );
+extern gchar *EDVPlotRelativePath(
+        const gchar *path, const gchar *dest_path
+);
 extern gint EDVObjectOPLink(
 	edv_core_struct *core,
 	const gchar *path,
