@@ -1,5 +1,6 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <time.h>
 #include <gtk/gtk.h>
 #include <unistd.h>
@@ -15,6 +16,7 @@
 #include "cfg_fio.h"
 #include "url.h"
 #include "edv_types.h"
+#include "edv_id.h"
 #include "edv_history.h"
 #include "edv_obj.h"
 #include "edv_device.h"
@@ -114,6 +116,10 @@ void EDVRunTerminal(
 );
 
 
+/* Object Operations Dialog */
+edv_obj_op_dlg_struct *EDVGetObjOpDlg(edv_core_struct *core);
+
+
 /* Window Mapping */
 void EDVMapRecBinDeskIcon(edv_core_struct *core);
 void EDVMapRecBin(edv_core_struct *core);
@@ -199,14 +205,21 @@ void EDVSyncDisks(edv_core_struct *core);
 
 
 /* Clearing */
+void EDVClearErrorMessages(edv_core_struct *core);
 void EDVClearHistoryEvents(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 );
 void EDVClearLocationsHistory(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 );
 void EDVClearRunHistory(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 );
 
 
@@ -339,8 +352,21 @@ gint EDVNewWindow(
 	    if(!STRISEMPTY(path))
 	    {
 		EDVObjectSetPath(obj, path);
-		if(!lstat(path, &lstat_buf))
+		if(lstat(path, &lstat_buf))
+		{
+		    const gint error_code = (gint)errno;
+		    EDVObjectPropSet(
+			obj,
+			EDV_OBJECT_PROP_NAME_ERROR,
+			g_strerror(error_code),
+			TRUE
+		    );
+		}
+		else
+		{
 		    EDVObjectSetStat(obj, &lstat_buf);
+		}
+		EDVObjectUpdateLinkFlags(obj);
 	    }
 	    EDVNewPropertiesDialogPage(core, obj, extra, NULL);
 	    EDVObjectDelete(obj);
@@ -906,7 +932,7 @@ gint EDVNewPropertiesDialogPage(
 	    }
 
 	    EDVPropDlgMap(propdlg);
-	    EDVPropDlgResetHasChanges(propdlg, FALSE);
+	    EDVPropDlgSetHasChanges(propdlg, FALSE);
 	    EDVPropDlgUpdateMenus(propdlg);
 
 	    return(i);
@@ -961,29 +987,36 @@ void EDVRunDeviceCheck(
 	}
 	else
 	{
-	    gchar *buf = g_strdup_printf(
+	    const gint error_code = (gint)errno;
+	    gchar *msg = g_strdup_printf(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Unable to execute command:\n\
+"Unable to execute the command:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "Incapaz de ejecutar la orden:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Incapable pour excuter l'ordre:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
-		cmd
+		cmd, g_strerror(error_code)
 	    );
 	    EDVPlaySoundError(core);
 	    EDVMessageError(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Cannot Run Program",
-		    buf,
+"Device Check Failed",
+		    msg,
 "An error was encountered while attempting to run the\n\
 specified program. Please verify that the command\n\
 is specified properly by going to Device->Devices...\n\
@@ -991,7 +1024,7 @@ and edit the device reference.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "No Puede Correr El Programa",
-		    buf,
+		    msg,
 "Un error se encontr al procurar para correr el\n\
 el programa especificado. Verifique por favor que la\n\
 orden es especificado apropiadamente yendo\n\
@@ -1000,7 +1033,7 @@ artefacto.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Ne Peut Pas Courir Le Programme",
-		    buf,
+		    msg,
 "Une erreur a t rencontre en tentant pour courir\n\
 le programme spcifi. S'il vous plat vrifier que\n\
 l'ordre est convenablement spcifi en allant\n\
@@ -1008,7 +1041,7 @@ Appareil->Appareils... et dite la rfrence d'appareil.",
 #endif
 		toplevel
 	    );
-	    g_free(buf);
+	    g_free(msg);
 	}
 }
 
@@ -1023,7 +1056,6 @@ void EDVRunDeviceTools(
 	pid_t p;
 	const gchar *cmd;
 
-
 	if((core == NULL) || (dev == NULL))
 	    return;
 
@@ -1040,31 +1072,37 @@ void EDVRunDeviceTools(
 	    return;
 
 	p = Exec(cmd);
-	if(p == 0)
+	if(p <= 0)
 	{
-	    gchar *buf = g_strdup_printf(
+	    const gint error_code = (gint)errno;
+	    gchar *msg = g_strdup_printf(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Unable to execute command:\n\
+"Unable to execute the command:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "Incapaz de ejecutar la orden:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Incapable pour excuter l'ordre:\n\
 \n\
-    %s\n",
+    %s\n\
+%s.",
 #endif
-		cmd
+		cmd, g_strerror(error_code)
 	    );
 	    EDVPlaySoundError(core);
 	    EDVMessageError(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Cannot Run Program",
-		    buf,
+"Device Tool Failed",
+		    msg,
 "An error was encountered while attempting to run the\n\
 specified program. Please verify that the command\n\
 is specified properly by going to Device->Devices...\n\
@@ -1072,7 +1110,7 @@ and edit the device reference.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "No Puede Correr El Programa",
-		    buf,
+		    msg,
 "Un error se encontr al procurar para correr el\n\
 el programa especificado. Verifique por favor que la\n\
 orden es especificado apropiadamente yendo\n\
@@ -1081,7 +1119,7 @@ artefacto.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Ne Peut Pas Courir Le Programme",
-		    buf,
+		    msg,
 "Une erreur a t rencontre en tentant pour courir\n\
 le programme spcifi. S'il vous plat vrifier que\n\
 l'ordre est convenablement spcifi en allant\n\
@@ -1089,7 +1127,7 @@ Appareil->Appareils... et dite la rfrence d'appareil.",
 #endif
 		toplevel
 	    );
-	    g_free(buf);
+	    g_free(msg);
 	}
 }
 
@@ -1120,31 +1158,38 @@ void EDVRunDeviceFormat(
 	    return;
 
 	p = Exec(cmd);
-	if(p == 0)
+	if(p <= 0)
 	{
-	    gchar *buf = g_strdup_printf(
+	    const gint error_code = (gint)errno;
+	    gchar *msg = g_strdup_printf(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Unable to execute command:\n\
+"Unable to execute the command:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "Incapaz de ejecutar la orden:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Incapable pour excuter l'ordre:\n\
 \n\
-    %s\n",
+    %s\n\
+\n\
+%s.",
 #endif
-		cmd
+		cmd, g_strerror(error_code)
 	    );
 	    EDVPlaySoundError(core);
 	    EDVMessageError(
 #ifdef PROG_LANGUAGE_ENGLISH
-"Cannot Run Program",
-		    buf,
+"Device Format Failed",
+		    msg,
 "An error was encountered while attempting to run the\n\
 specified program. Please verify that the command\n\
 is specified properly by going to Device->Devices...\n\
@@ -1152,7 +1197,7 @@ and edit the device reference.",
 #endif
 #ifdef PROG_LANGUAGE_SPANISH
 "No Puede Correr El Programa",
-		    buf,
+		    msg,
 "Un error se encontr al procurar para correr el\n\
 el programa especificado. Verifique por favor que la\n\
 orden es especificado apropiadamente yendo\n\
@@ -1161,7 +1206,7 @@ artefacto.",
 #endif
 #ifdef PROG_LANGUAGE_FRENCH
 "Ne Peut Pas Courir Le Programme",
-		    buf,
+		    msg,
 "Une erreur a t rencontre en tentant pour courir\n\
 le programme spcifi. S'il vous plat vrifier que\n\
 l'ordre est convenablement spcifi en allant\n\
@@ -1169,7 +1214,7 @@ Appareil->Appareils... et dite la rfrence d'appareil.",
 #endif
 		toplevel
 	    );
-	    g_free(buf);
+	    g_free(msg);
 	}
 }
 
@@ -1207,35 +1252,89 @@ void EDVRunTerminal(
 	    cwd = NULL;
 	}
 
+	/* Execute the run terminal command
+	 *
+	 * If the cmd is specified then append cmd to the run
+	 * terminal command defined in the configuration,
+	 * otherwise just run the terminal command defined in the
+	 * configuration
+	 */
 	if(!STRISEMPTY(cmd))
 	{
 	    /* Get terminal command for running a program with a
 	     * terminal, format the command, and then run the command
 	     */
-	    gchar *cmd_src = STRDUP(EDV_GET_S(
+	    gchar *cmd_terminal = STRDUP(EDV_GET_S(
 		EDV_CFG_PARM_PROG_TERMINAL_RUN
 	    ));
-	    if(!STRISEMPTY(cmd_src))
+	    if(!STRISEMPTY(cmd_terminal))
 	    {
-		gchar *cmd_tar = g_strdup_printf(
-		    "%s %s",
-		    cmd_src,
-		    cmd
+		gchar *cmd_tar = g_strconcat(
+		    cmd_terminal,
+		    " ",
+		    cmd,
+		    NULL
 		);
-		Exec(cmd_tar);
+		const gint p = Exec(cmd_tar);
+		if(p <= 0)
+		{
+		    const gint error_code = (gint)errno;
+		    gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			cmd_tar, g_strerror(error_code)
+		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+"Run Terminal Failed",
+			msg,
+"Make sure that the run terminal command is\n\
+defined correctly, please go to\n\
+Settings->Options->Programs.",
+			toplevel
+		    );
+		    g_free(msg);
+		}
 		g_free(cmd_tar);
 	    }
-	    g_free(cmd_src);
+	    g_free(cmd_terminal);
 	}
 	else
 	{
 	    /* No command specified, just run the terminal */
-	    gchar *cmd_src = STRDUP(EDV_GET_S(
+	    gchar *cmd_terminal = STRDUP(EDV_GET_S(
 		EDV_CFG_PARM_PROG_TERMINAL
 	    ));
-	    if(!STRISEMPTY(cmd_src))
-		Exec(cmd_src);
-	    g_free(cmd_src);
+	    if(!STRISEMPTY(cmd_terminal))
+	    {
+		const gint p = Exec(cmd_terminal);
+		if(p <= 0)
+		{
+		    const gint error_code = (gint)errno;
+		    gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			cmd_terminal, g_strerror(error_code)
+		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+"Run Terminal Failed",
+			msg,
+"Make sure that the run terminal command is\n\
+defined correctly, please go to\n\
+Settings->Options->Programs.",
+			toplevel
+		    );
+		    g_free(msg);
+		}
+	    }
+	    g_free(cmd_terminal);
 	}
 
 	EDVSetCWD(cwd);
@@ -1244,6 +1343,21 @@ void EDVRunTerminal(
 }
 
 
+/*
+ *	Gets the Object Operations Dialog, creating it as needed.
+ */
+edv_obj_op_dlg_struct *EDVGetObjOpDlg(edv_core_struct *core)
+{
+	if(core == NULL)
+	    return(NULL);
+
+	if(core->obj_op_dlg == NULL)
+	    core->obj_op_dlg = EDVObjOpDlgNew(core);
+
+	return(core->obj_op_dlg);
+}
+
+
 /*
  *	Maps the Recycle Bin Desktop Icon, creating it as needed.
  */
@@ -1358,6 +1472,7 @@ void EDVMapOptionsWinPage(
 	if(core == NULL)
 	    return;
 
+	/* Get the Options Window, creating it as needed */
 	optwin = core->options_window;
 	if(optwin == NULL)
 	    core->options_window = optwin = EDVCreateOptionsWindow(
@@ -1366,14 +1481,32 @@ void EDVMapOptionsWinPage(
 	if(optwin == NULL)
 	    return;
 
-	EDVCenterWindowToWindow(
-	    toplevel, optwin->toplevel
-	);
+	/* Center the Options Window to the specified toplevel window */
+	EDVCenterWindowToWindow(toplevel, optwin->toplevel);
 
+	/* Get the values from the configuration */
 	EDVGenOptWinGetValues(optwin);
 
+	/* Page specified? */
 	if(!STRISEMPTY(page_name))
-	    EDVOptionsWindowSelectTab(optwin, page_name);
+	{
+	    /* Switch to the specified page */
+	    if(!EDVOptionsWindowSelectTab(optwin, page_name))
+	    {
+		gchar *msg = g_strdup_printf(
+"Invalid Options Window page name \"%s\".",
+		    page_name
+		);
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(
+		    "Options Window Page Switch Failed",
+		    msg,
+		    NULL,
+		    toplevel
+		);
+		g_free(msg);
+	    }
+	}
 
 	EDVGenOptWinMap(optwin);
 	EDVGenOptWinResetHasChanges(optwin, FALSE);
@@ -1406,8 +1539,7 @@ void EDVMapCustomizeWin(
 	if(core == NULL)
 	    return;
 
-
-	/* Get Customize Window, create it as needed */
+	/* Get the Customize Window, creating it as needed */
 	optwin = core->customize_window;
 	if(optwin == NULL)
 	    core->customize_window = optwin = EDVCustomizeWindowNew(
@@ -1416,12 +1548,10 @@ void EDVMapCustomizeWin(
 	if(optwin == NULL)
 	    return;
 
-	/* Center Customize Window */
+	/* Center the Customize Window to the specified toplevel window */
 	EDVCenterWindowToWindow(toplevel, optwin->toplevel);
 
-	/* Get values from configuration and set to Customize Window's
-	 * widgets
-	 */
+	/* Get the values from the configuration */
 	EDVGenOptWinGetValues(optwin);
 
 	/* Match the given toplevel with the corresponding window */
@@ -1450,8 +1580,26 @@ void EDVMapCustomizeWin(
 	    page_name = NULL;
 	}
 
-	/* Switch page */
-	EDVCustomizeWindowSelectTab(optwin, page_name);
+	/* Page specified? */
+	if(!STRISEMPTY(page_name))
+	{
+	    /* Switch to the specified page */
+	    if(!EDVCustomizeWindowSelectTab(optwin, page_name))
+	    {
+		gchar *msg = g_strdup_printf(
+"Invalid Customize Window page name \"%s\".",
+		    page_name
+		);
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(
+		    "Customize Window Page Switch Failed",
+		    msg,
+		    NULL,
+		    toplevel
+		);
+		g_free(msg);
+	    }
+	}
 
 	EDVGenOptWinMap(optwin);
 	EDVGenOptWinResetHasChanges(optwin, FALSE);
@@ -2041,6 +2189,7 @@ void EDVUpdateRCStyles(edv_core_struct *core)
 void EDVUpdateIDPULists(edv_core_struct *core)
 {
 	gint i, n;
+	GList *glist;
 	GtkRcStyle *rcstyle;
 	GtkWidget *w;
 	const edv_uid_struct *uid;
@@ -2067,10 +2216,16 @@ void EDVUpdateIDPULists(edv_core_struct *core)
 	PUListClear(pulist);
 
 	/* Add Users to the Popup List */
-	for(i = 0; i < core->total_uids; i++)
+	for(glist = core->uids_list, i = 0;
+	    glist != NULL;
+	    glist = g_list_next(glist), i++
+	)
 	{
-	    uid = core->uid[i];
-	    if((uid != NULL) ? STRISEMPTY(uid->name) : TRUE)
+	    uid = EDV_UID(glist->data);
+	    if(uid == NULL)
+		continue;
+
+	    if(STRISEMPTY(uid->name))
 		continue;
 
 	    n = PUListAddItem(pulist, uid->name);
@@ -2098,10 +2253,16 @@ void EDVUpdateIDPULists(edv_core_struct *core)
 	PUListClear(pulist);
 
 	/* Add Groups to the Popup List */
-	for(i = 0; i < core->total_gids; i++)
+	for(glist = core->gids_list, i = 0;
+	    glist != NULL;
+	    glist = g_list_next(glist), i++
+	)
 	{
-	    gid = core->gid[i];
-	    if((gid != NULL) ? STRISEMPTY(gid->name) : TRUE)
+	    gid = EDV_GID(glist->data);
+	    if(gid == NULL)
+		continue;
+
+	    if(STRISEMPTY(gid->name))
 		continue;
 
 	    n = PUListAddItem(pulist, gid->name);
@@ -2409,8 +2570,7 @@ void EDVRefresh(edv_core_struct *core)
 void EDVReset(edv_core_struct *core)
 {
 	gint i;
-	const gchar *s;
-	gchar hostname[64 + 1];
+	gchar *sd, hostname[64 + 1];
 	cfg_item_struct *cfg_list;
 
 	if(core == NULL)
@@ -2425,33 +2585,48 @@ void EDVReset(edv_core_struct *core)
 	    hostname[sizeof(hostname) - 1] = '\0';
 
 	/* Reget the User & Group IDs */
-	for(i = 0; i < core->total_uids; i++)
-	    EDVUIDDelete(core->uid[i]);
-	g_free(core->uid);
-	core->uid = EDVUIDGetSystem(&core->total_uids);
-	for(i = 0; i < core->total_gids; i++)
-	    EDVGIDDelete(core->gid[i]);
-	g_free(core->gid);
-	core->gid = EDVGIDGetSystem(&core->total_gids);
+	if(core->uids_list != NULL)
+	{
+	    g_list_foreach(
+		core->uids_list, (GFunc)EDVUIDDelete, NULL
+	    );
+	    g_list_free(core->uids_list);
+	    core->uids_list = NULL;
+	}
+	core->uids_list = EDVUIDGetSystem();
+
+	if(core->gids_list != NULL)
+	{
+	    g_list_foreach(
+		core->gids_list, (GFunc)EDVGIDDelete, NULL
+	    );
+	    g_list_free(core->gids_list);
+	    core->gids_list = NULL;
+	}
+	core->gids_list = EDVGIDGetSystem();
 
 	/* Reget the Effective User ID string */
-	s = EDVUIDGetNameFromUID(
-	    core->uid, core->total_uids,
+	sd = EDVUIDGetNameFromUID(
+	    core->uids_list,
 	    core->effective_user_id, NULL
 	);
+
 	g_free(core->effective_user_id_str);
 	core->effective_user_id_str = STRDUP(
-	    !STRISEMPTY(s) ? s : "anonymous"
+	    STRISEMPTY(sd) ? "anonymous" : sd
 	);
 
 	/* Reget the Effective User ID & Hostname string */
 	g_free(core->effective_user_id_host_str);
-	core->effective_user_id_host_str = g_strdup_printf(
-	    "%s@%s",
-	    !STRISEMPTY(s) ? s : "anonymous",
-	    !STRISEMPTY(hostname) ? hostname : "unknown"
+	core->effective_user_id_host_str = g_strconcat(
+	    STRISEMPTY(sd) ? "anonymous" : sd,
+	    "@",
+	    STRISEMPTY(hostname) ? "unknown" : hostname,
+	    NULL
 	);
 
+	g_free(sd);
+
 	/* Reopen the configuration list from the configuration file */
 	if(core->cfg_file != NULL)
 	    CFGFileOpen(core->cfg_file, cfg_list);
@@ -2479,6 +2654,7 @@ void EDVReset(edv_core_struct *core)
 	core->mimetype = NULL;
 	core->total_mimetypes = 0;
 	EDVMimeTypesListSystemFileOpen(
+	    EDV_GET_S(EDV_CFG_PARM_DIR_GLOBAL),
 	    &core->mimetype, &core->total_mimetypes,
 	    -1,			/* Append */
 	    NULL, core,
@@ -2562,7 +2738,7 @@ void EDVSyncDisks(edv_core_struct *core)
 		FALSE,		/* Do not write MIME Types marked read_only */
 		NULL, core
 	    );
-	    if(!lstat(path, &lstat_buf))
+	    if(!lstat((const char *)path, &lstat_buf))
 	    {
 		if(object_existed)
 		    EDVObjectModifiedEmit(core, path, NULL, &lstat_buf);
@@ -2626,6 +2802,29 @@ void EDVSyncDisks(edv_core_struct *core)
 }
 
 
+/*
+ *	Clears all the error message buffers and pointers.
+ */
+void EDVClearErrorMessages(edv_core_struct *core)
+{
+	if(core == NULL)
+	    return;
+
+	core->archive_last_error = NULL;
+	core->device_last_error = NULL;
+	core->obj_last_error = NULL;
+	core->recbin_last_error = NULL;
+
+	g_free(core->archive_last_error_buf);
+	core->archive_last_error_buf = NULL;
+	g_free(core->device_last_error_buf);
+	core->device_last_error_buf = NULL;
+	g_free(core->obj_last_error_buf);
+	core->obj_last_error_buf = NULL;
+	g_free(core->recbin_last_error_buf);
+	core->recbin_last_error_buf = NULL;
+}
+
 /*
  *	Clears all history events.
  *
@@ -2633,7 +2832,9 @@ void EDVSyncDisks(edv_core_struct *core)
  *	log file.
  */
 void EDVClearHistoryEvents(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 )
 {
 	if(core == NULL)
@@ -2653,16 +2854,17 @@ void EDVClearHistoryEvents(
  *	Clears the locations history on all windows.
  */
 void EDVClearLocationsHistory(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 )
 {
-	gint i, response;
+	gint i;
 	const gchar *path;
+	const cfg_item_struct *cfg_list;
 	edv_browser_struct *browser;
 	edv_imbr_struct *imbr;
 	edv_archiver_struct *archiver;
-	const cfg_item_struct *cfg_list;
-
 
 	if(core == NULL)
 	    return;
@@ -2670,9 +2872,13 @@ void EDVClearLocationsHistory(
 	cfg_list = core->cfg_list;
 
 	/* Confirm */
-	EDVPlaySoundQuestion(core);
-	CDialogSetTransientFor(toplevel);
-	response = CDialogGetResponse(
+	if(confirm)
+	{
+	    gint response;
+
+	    EDVPlaySoundQuestion(core);
+	    CDialogSetTransientFor(toplevel);
+	    response = CDialogGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Confirme Claro",
 "Usted est seguro que usted quiere borrar la historia de\n\
@@ -2705,17 +2911,17 @@ as janelas?\n",
 "Are you sure you want to delete the locations history on\n\
 all the windows?\n",
 #endif
-	    NULL,
-	    CDIALOG_ICON_QUESTION,
-	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
-	    CDIALOG_BTNFLAG_YES
-	);
-	CDialogSetTransientFor(NULL);
-	if(response != CDIALOG_RESPONSE_YES)
-	    return;
+		NULL,
+		CDIALOG_ICON_QUESTION,
+		CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+		CDIALOG_BTNFLAG_YES
+	    );
+	    CDialogSetTransientFor(NULL);
+	    if(response != CDIALOG_RESPONSE_YES)
+		return;
+	}
 
-
-	/* Clear File Browser Locations List */
+	/* Clear the File Browser's Locations List */
 	for(i = 0; i < core->total_browsers; i++)
 	{
 	    browser = core->browser[i];
@@ -2785,14 +2991,14 @@ all the windows?\n",
  *	file.
  */
 void EDVClearRunHistory(
-	edv_core_struct *core, GtkWidget *toplevel
+	edv_core_struct *core,
+	const gboolean confirm,
+	GtkWidget *toplevel
 )
 {
-	gint response;
 	const gchar *path;
-	edv_run_dlg_struct *d;
 	const cfg_item_struct *cfg_list;
-
+	edv_run_dlg_struct *d;
 
 	if(core == NULL)
 	    return;
@@ -2800,9 +3006,13 @@ void EDVClearRunHistory(
 	cfg_list = core->cfg_list;
 
 	/* Confirm */
-	EDVPlaySoundQuestion(core);
-	CDialogSetTransientFor(toplevel);
-	response = CDialogGetResponse(
+	if(confirm)
+	{
+	    gint response;
+
+	    EDVPlaySoundQuestion(core);
+	    CDialogSetTransientFor(toplevel);
+	    response = CDialogGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Confirme Claro",
 "Usted est seguro que usted quiere borrar el corre la historia?\n",
@@ -2828,15 +3038,15 @@ void EDVClearRunHistory(
 "Confirm Clear",
 "Are you sure you want to delete the run history?\n",
 #endif
-	    NULL,
-	    CDIALOG_ICON_QUESTION,
-	    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
-	    CDIALOG_BTNFLAG_YES
-	);
-	CDialogSetTransientFor(NULL);
-	if(response != CDIALOG_RESPONSE_YES)
-	    return;
-
+		NULL,
+		CDIALOG_ICON_QUESTION,
+		CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+		CDIALOG_BTNFLAG_YES
+	    );
+	    CDialogSetTransientFor(NULL);
+	    if(response != CDIALOG_RESPONSE_YES)
+		return;
+	}
 
 	/* Clear Run Dialog Run History */
 	d = core->run_dlg;
@@ -2951,20 +3161,25 @@ to Settings->Options->Programs.",
 	    p = Exec(cmd);
 	    if(p <= 0)
 	    {
-		gchar *s = g_strdup_printf(
-"Unable to execute command:\n\n    %s",
-		    cmd
+		const gint error_code = (gint)errno;
+		gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+		    cmd, g_strerror(error_code)
 		);
 		EDVPlaySoundError(core);
 		EDVMessageError(
 		    "Download Failed",
-		    s,
+		    msg,
 "Make sure that the network download program is\n\
 defined correctly, please go to\n\
 Settings->Options->Programs.",
 		    toplevel
 		);
-		g_free(s);
+		g_free(msg);
 	    }
 
 	    g_free(cmd);
@@ -3036,13 +3251,11 @@ void EDVRecycleBinSizeCheck(
 	 */
 	if(total_size >= recbin_size_warn)
 	{
-	    gchar	*total_size_s = STRDUP(
-		EDVGetObjectSizeStr(core, total_size)
-			),
-			*recbin_size_warn_s = STRDUP(
-		EDVGetObjectSizeStr(core, recbin_size_warn)
-			),
-			*buf = g_strdup_printf(
+	    gchar	*total_size_s = STRDUP(EDVSizeStrDelim(total_size)),
+			*recbin_size_warn_s = STRDUP(EDVSizeStrDelim(
+			    recbin_size_warn
+			)),
+			*msg = g_strdup_printf(
 #ifdef PROG_LANGUAGE_ENGLISH
 "The size of the recycle bin is now %s bytes,\n\
 it has exceeded the user specified size of %s bytes\n\
@@ -3083,15 +3296,14 @@ au Recycle Huche et alors va File->Purge All.",
 #ifdef PROG_LANGUAGE_FRENCH
 		"Recycler l'Avertissement d'Huche",
 #endif
-		buf, NULL,
+		msg, NULL,
 		(guint8 **)icon_trash_32x32_xpm,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
-	    CDialogSetTransientFor(NULL);
-
-	    g_free(buf);
+	    g_free(msg);
 	    g_free(total_size_s);
 	    g_free(recbin_size_warn_s);
+	    CDialogSetTransientFor(NULL);
 	}
 }
