@@ -1,10 +1,13 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <gtk/gtk.h>
 #include <unistd.h>
 
+#include "cdialog.h"
+
 #include "edv_types.h"
 #include "edv_mime_type.h"
 #include "edv_mime_types_list.h"
@@ -20,7 +23,9 @@
 
 static gchar *EDVOpenStripExt(const gchar *s);
 static gboolean EDVOpenIsFileExecutable(const gchar *path);
-static gchar *EDVOpenFormatCommand(
+
+/* Command Formatting */
+gchar *EDVOpenFormatCommand(
 	edv_core_struct *core, const gchar *cmd, GList *paths_list
 );
 
@@ -52,6 +57,16 @@ gint EDVOpenWith(
 	gchar **stdout_path_rtn, gchar **stderr_path_rtn
 );
 
+/* Open Command */
+gint EDVOpenCommand(
+	edv_core_struct *core,
+        GList *paths_list,
+	const gchar *command,
+	GtkWidget *toplevel,
+        const gboolean verbose,
+        gchar **stdout_path_rtn, gchar **stderr_path_rtn
+);
+
 
 #define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
@@ -86,9 +101,9 @@ static gchar *EDVOpenStripExt(const gchar *s)
 
 	/* Get the extension */
 	if(*name == '.') 
-	    ext = strchr(name + 1, '.');
+	    ext = (gchar *)strchr((char *)name + 1, '.');
 	else
-	    ext = strchr(name, '.');
+	    ext = (gchar *)strchr((char *)name, '.');
 
 	/* No extension? */
 	if(ext == NULL)
@@ -98,7 +113,11 @@ static gchar *EDVOpenStripExt(const gchar *s)
 	len = (gint)(ext - s);
 	path_wo_ext = (gchar *)g_malloc((len + 1) * sizeof(gchar));
 	if(len > 0)
-	    memcpy(path_wo_ext, s, len * sizeof(gchar));
+	    memcpy(
+		path_wo_ext,
+		s,
+		len * sizeof(gchar)
+	    );
 	path_wo_ext[len] = '\0';
 
 	/* Need to tack on the ending double quote character? */
@@ -120,16 +139,16 @@ static gchar *EDVOpenStripExt(const gchar *s)
 static gboolean EDVOpenIsFileExecutable(const gchar *path)
 {
 	mode_t m;
-	struct stat stat_buf;
+	struct stat stat_msg;
 
 	if(STRISEMPTY(path))
 	    return(FALSE);
 
 	/* Get the destination stats */
-	if(stat(path, &stat_buf))
+	if(stat(path, &stat_msg))
 	    return(FALSE);
 
-	m = stat_buf.st_mode;
+	m = stat_msg.st_mode;
 
 #ifdef S_ISREG
 	/* Does the path lead to a file? */
@@ -150,8 +169,16 @@ static gboolean EDVOpenIsFileExecutable(const gchar *path)
 }
 
 /*
- *	Performs substitutions to the command string specified by cmd 
- *	with the values specified in the core and the paths list.
+ *	Performs token substitutions on the command string.
+ *
+ *	The core specifies the core, who's values will be used in the
+ *	substitutions.
+ *
+ *	The cmd specifies the original (unsubstituted) command string
+ *	with tokens.
+ *
+ *	The paths_list specifies the list of gchar * full paths to
+ *	the selected objects.
  *
  *	Returns a dynamically allocated string describing the command
  *	with all the substitutions made.
@@ -159,7 +186,7 @@ static gboolean EDVOpenIsFileExecutable(const gchar *path)
  *	Reminder: There is a sister function in lib/edv_open.c which
  *	needs to perform equvilently to this function.
  */
-static gchar *EDVOpenFormatCommand(
+gchar *EDVOpenFormatCommand(
 	edv_core_struct *core, const gchar *cmd, GList *paths_list
 )
 {
@@ -179,7 +206,7 @@ static gchar *EDVOpenFormatCommand(
 
 	/* Begin creating substitution values */
 
-	/* Current working directory */
+	/* Current the current working directory */
 	s = EDVGetCWD();
 	cwd_quoted = g_strdup_printf("\"%s\"", s);
 	g_free(s);
@@ -444,18 +471,42 @@ static gint EDVOpenIteration(
 		g_free(parent);
 
 		/* Format the command to execute the object */
-		cmd = g_strdup_printf(
-		    "%s &",
-		    path
+		cmd = g_strconcat(
+		    path,
+		    NULL
 		);
 
-		/* Execute command */
+		/* Execute the command */
 		pid = EDVSystem(cmd);
-		if(pid <= 0)
+		if(pid < 0)
+		{
+		    const gint error_code = (gint)errno;
+		    if(verbose)
+		    {
+			gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			    cmd, g_strerror(error_code)
+		        );
+			EDVPlaySoundError(core);
+			EDVMessageError(
+			    "Open Failed",
+			    msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+			    toplevel
+			);
+			g_free(msg);
+		    }
 		    status = -1;
+		}
 		else
+		{
 		    status = 0;
-
+		}
 		g_free(cmd);
 
 		/* Change back to the previous working dir */
@@ -465,7 +516,7 @@ static gint EDVOpenIteration(
 		/* Failed to execute the object? */
 		if((status != 0) && verbose)
 		{
-		    gchar *buf = g_strdup_printf(
+		    gchar *msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Incapaz de ejecutar objeto:\n\
 \n\
@@ -505,53 +556,53 @@ static gint EDVOpenIteration(
 		    EDVMessageError(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Si usted desea abrir este objeto ejecutable\n\
 con un especfico de tipo MIME, el Archivo de\n\
 La Prueba->Abre Con.",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Si vous souhaitez ouvrir cet objet ralisable\n\
 avec un Type de MIME spcifique, essayez Le\n\
 Fichier->Ouvre Avec.",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Wenn sie wnschen, dieses ausfhrbare objekt\n\
 mit einem spezifischen MIME Typ zu offnen,\n\
 Legt Versuch->Offen Mit Ab.",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Se lei desidera aprire quest'oggetto eseguibile\n\
 con un Tipo di MIME specifico, tenta\n\
 Il File->Apre Con.",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Indien u wenst deze uitvoerbaar voorwerp met een\n\
 specifiek MIME Type, poging Dossier->Open Met te\n\
 openen.",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Se deseja abrir este objeto de executable com um\n\
 tipo especfico de MIMO, tenta Arquivo->Abre Com.",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Om De nsker pne dette gjennomfrbare objektet med\n\
 en spesifikk MIME Type, forsk File->Open With.",
 #else
 "Open Failed",
-buf,
+msg,
 "If you wish to open this executable object with\n\
 a specific MIME Type, try File->Open With.",
 #endif
 			toplevel
 		    );
-		    g_free(buf);
+		    g_free(msg);
 		}
 		return(status);
 	    }
@@ -574,22 +625,49 @@ a specific MIME Type, try File->Open With.",
 		EDVSetCWD(parent);
 		g_free(parent);
 
-		/* Format command */
+		/* Format the command by substituting the tokens */
 		paths_list = g_list_append(
 		    paths_list,
 		    STRDUP(path)
 		);
 		cmd = EDVOpenFormatCommand(core, def_viewer_cmd, paths_list);
-		g_list_foreach(paths_list, (GFunc)g_free, NULL);
-		g_list_free(paths_list);
+		if(paths_list != NULL)
+		{
+		    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+		    g_list_free(paths_list);
+		}
 
-		/* Execute command */
+		/* Execute the command */
 		pid = EDVSystem(cmd);
-		if(pid <= 0)
+		if(pid < 0)
+		{
+		    const gint error_code = (gint)errno;
+		    if(verbose)
+		    {
+			gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			    cmd, g_strerror(error_code)
+		        );
+			EDVPlaySoundError(core);
+			EDVMessageError(
+			    "Open Failed",
+			    msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+			    toplevel
+			);
+			g_free(msg);
+		    }
 		    status = -1;
+		}
 		else
+		{
 		    status = 0;
-
+		}
 		g_free(cmd);
 
 		/* Change back to the previous working dir */
@@ -604,7 +682,7 @@ a specific MIME Type, try File->Open With.",
 	     */
 	    else if(verbose)
 	    {
-		gchar *buf = g_strdup_printf(
+		gchar *msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Incapaz de encontrar de tipo MIME asociado con objeto:\n\
 \n\
@@ -644,7 +722,7 @@ a specific MIME Type, try File->Open With.",
 		EDVMessageWarning(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Para definir un de tipo MIME para este objeto,\n\
 va a Ver->De Tipo MIME...\n\
 \n\
@@ -652,7 +730,7 @@ Para la ayuda adicional a establecer de tipo MIME,\n\
 vea Ayuda->De Tipo MIME...",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Pour dfinir un Type de MIME pour cet objet,\n\
 Aller Regarder->Les Types De MIME...\n\
 \n\
@@ -660,7 +738,7 @@ Pour l'aide supplmentaire sur monte les Types de MIME,\n\
 L'Aide D'vch -> Les Types De MIME...",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Um einen MIME Typ fr dieses objekt zu definieren,\n\
 gehen sie, Um->MIME Typen Anzusehen...\n\
 \n\
@@ -668,7 +746,7 @@ Fr zustzliche Hilfe stellend auf MIME Typen auf,\n\
 sieht Hilfe->MIME Typen...",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Per definire un Tipo di MIME per quest'oggetto,\n\
 andare Osservare->I Tipi Di MIME...\n\
 \n\
@@ -676,7 +754,7 @@ Per l'aiuto ulteriore su installa i Tipi di MIME,\n\
 vedere L'Aiuto->I Tipi Di MIME...",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Om een MIME Type voor deze voorwerp te definiren,\n\
 Ga Om->MIME Typen Te Bekijken...\n\
 \n\
@@ -684,7 +762,7 @@ Voor bijkomende hulp op opstellen van MIME Typen,\n\
 ziet Hulp->MIME Typen...",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Definir um Tipo de MIMO para este objeto,\n\
 vai Ver->Tipos de MIMO...\n\
 \n\
@@ -692,7 +770,7 @@ Para ajuda adicional em armar Tipos de MIMO,\n\
 vem Ajuda->Tipos De MIMO...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Definere en MIME Type for dette objektet,\n\
 dra til View->MIME Types...\n\
 \n\
@@ -700,7 +778,7 @@ For ytterligere hjelp p innstilling ser opp MIME Types,\n\
 Help->MIME Types...",
 #else
 "Open Failed",
-buf,
+msg,
 "To define a MIME Type for this object, go to\n\
 View->MIME Types...\n\
 \n\
@@ -709,7 +787,7 @@ Help->MIME Types...",
 #endif
 		    toplevel
 		);
-		g_free(buf);
+		g_free(msg);
 	    }
 	    return(-2);
 	}
@@ -830,9 +908,9 @@ with this object is not supported.\n",
 	{
 	    if(verbose)
 	    {
-		gchar *buf;
+		gchar *msg;
 		if(!STRISEMPTY(command_name))
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "MIME Type \"%s\" no tiene una orden de \"%s\".\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -848,12 +926,12 @@ with this object is not supported.\n",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "MIME Type \"%s\" har ikke en \"%s\" kommando.\n"
 #else
-"MIME Type \"%s\" does not have a \"%s\" command.\n"
+"The MIME Type \"%s\" does not have a \"%s\" command.\n"
 #endif
 			, mt->type, command_name
 		    );
 		else
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "MIME Type \"%s\" no tiene ninguna orden definida.\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -869,7 +947,7 @@ with this object is not supported.\n",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "MIME Type \"%s\" har ikke noen definerte kommandoer.\n"
 #else
-"MIME Type \"%s\" does not have any defined commands.\n"
+"The MIME Type \"%s\" does not have any defined commands.\n"
 #endif
 			, mt->type
 		    );
@@ -877,7 +955,7 @@ with this object is not supported.\n",
 		EDVMessageWarning(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Para redactar el de tipo MIME para este objeto,\n\
 va a Ver->De Tipo MIME...\n\
 \n\
@@ -885,7 +963,7 @@ Para la ayuda adicional a establecer de tipo MIME,\n\
 Vea Ayuda->De Tipo MIME...",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Pour diter le Type de MIME pour cet objet,\n\
 aller Regarder->Les Types De MIME...\n\
 \n\
@@ -893,7 +971,7 @@ Pour l'aide supplmentaire sur monte les Types de MIME,\n\
 L'Aide D'vch->Les Types De MIME...",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Um den MIME Typ fr dieses Objekt zu redigieren,\n\
 gehen Sie, Um->MIME Typen Anzusehen...\n\
 \n\
@@ -901,7 +979,7 @@ Fr zustzliche Hilfe stellend auf MIME Typen auf,\n\
 sieht Hilfe->MIME Typen...",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Per redigere il Tipo di MIME per quest'oggetto,\n\
 andare Osservare->I Tipi Di MIME...\n\
 \n\
@@ -909,7 +987,7 @@ Per l'aiuto ulteriore su installa i Tipi di MIME,\n\
 vedere L'Aiuto->I Tipi Di MIME...",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Om het MIME Type voor deze voorwerp te redigeren,\n\
 ga Om->MIME Typen Te Bekijken...\n\
 \n\
@@ -917,7 +995,7 @@ Voor bijkomende hulp op opstellen van MIME Typen,\n\
 ziet Hulp->MIME Typen...",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Editar o Tipo de MIMO para este objeto,\n\
 vai Ver->Tipos De MIMO...\n\
 \n\
@@ -925,7 +1003,7 @@ Para ajuda adicional em armar Tipos de MIMO,\n\
 vem Ajuda->Tipos De MIMO...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Redigere MIME Type for dette objektet,\n\
 dra til View->MIME Types...\n\
 \n\
@@ -933,7 +1011,7 @@ For ytterligere hjelp p innstilling ser opp MIME Types,\n\
 Help->MIME Types...",
 #else
 "Open Failed",
-buf,
+msg,
 "To edit the MIME Type for this object, go to\n\
 View->MIME Types...\n\
 \n\
@@ -942,7 +1020,7 @@ Help->MIME Types...",
 #endif
 		    toplevel
 		);
-		g_free(buf);
+		g_free(msg);
 	    }
 	    return(-2);
 	}
@@ -971,7 +1049,7 @@ Help->MIME Types...",
 		if(mt->mt_class == EDV_MIME_TYPE_CLASS_PROGRAM)
 		{
 		    if((mt->type != NULL) ?
-			!strcmp(mt->type, command_base) : FALSE
+			!strcmp((const char *)mt->type, (const char *)command_base) : FALSE
 		    )
 		    {
 			/* Found a referenced MIME Type, now get its
@@ -998,7 +1076,7 @@ Help->MIME Types...",
 		/* No MIME Type associated for the given path */
 		if(verbose)
 		{
-		    gchar *buf = g_strdup_printf(
+		    gchar *msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Incapaz de encontrar la clase de la Aplicacin \"%s\" de tipo MIME."
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1014,7 +1092,7 @@ Help->MIME Types...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "Maktesls finne Application klasse MIME Type \"%s\"."
 #else
-"Unable to find Application class MIME Type \"%s\"."
+"Unable to find an Application class MIME Type \"%s\"."
 #endif
 			, command_base
 		    );
@@ -1022,7 +1100,7 @@ Help->MIME Types...",
 		    EDVMessageWarning(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Para redactar el de tipo MIME para este objeto,\n\
 va a Ver->De Tipo MIME...\n\
 \n\
@@ -1030,7 +1108,7 @@ Para la ayuda adicional a establecer de tipo MIME,\n\
 Vea Ayuda->De Tipo MIME...",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Pour diter le Type de MIME pour cet objet,\n\
 aller Regarder->Les Types De MIME...\n\
 \n\
@@ -1038,7 +1116,7 @@ Pour l'aide supplmentaire sur monte les Types de MIME,\n\
 L'Aide D'vch->Les Types De MIME...",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Um den MIME Typ fr dieses Objekt zu redigieren,\n\
 gehen Sie, Um->MIME Typen Anzusehen...\n\
 \n\
@@ -1046,7 +1124,7 @@ Fr zustzliche Hilfe stellend auf MIME Typen auf,\n\
 sieht Hilfe->MIME Typen...",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Per redigere il Tipo di MIME per quest'oggetto,\n\
 andare Osservare->I Tipi Di MIME...\n\
 \n\
@@ -1054,7 +1132,7 @@ Per l'aiuto ulteriore su installa i Tipi di MIME,\n\
 vedere L'Aiuto->I Tipi Di MIME...",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Om het MIME Type voor deze voorwerp te redigeren,\n\
 ga Om->MIME Typen Te Bekijken...\n\
 \n\
@@ -1062,7 +1140,7 @@ Voor bijkomende hulp op opstellen van MIME Typen,\n\
 ziet Hulp->MIME Typen...",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Editar o Tipo de MIMO para este objeto,\n\
 vai Ver->Tipos De MIMO...\n\
 \n\
@@ -1070,7 +1148,7 @@ Para ajuda adicional em armar Tipos de MIMO,\n\
 vem Ajuda->Tipos De MIMO...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Redigere MIME Type for dette objektet,\n\
 dra til View->MIME Types...\n\
 \n\
@@ -1078,7 +1156,7 @@ For ytterligere hjelp p innstilling ser opp MIME Types,\n\
 Help->MIME Types...",
 #else
 "Open Failed",
-buf,
+msg,
 "To edit the MIME Type for this object, go to\n\
 View->MIME Types...\n\
 \n\
@@ -1087,7 +1165,7 @@ Help->MIME Types...",
 #endif
 			toplevel
 		    );
-		    g_free(buf);
+		    g_free(msg);
 		}
 		return(-2);
 	    }
@@ -1099,9 +1177,9 @@ Help->MIME Types...",
 	{
 	    if(verbose)
 	    {
-		gchar *buf;
+		gchar *msg;
 		if(mt != NULL)
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "\"%s\" de tipo MIME no tiene una orden definida."
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1117,19 +1195,19 @@ Help->MIME Types...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "MIME Type \"%s\" har ikke en definert kommando."
 #else
-"MIME Type \"%s\" does not have a defined command."
+"The MIME Type \"%s\" does not have a defined command."
 #endif
 			, mt->type
 		    );
 		else
-		    buf = STRDUP(
-"MIME Type does not have a defined command."
+		    msg = STRDUP(
+"Unable to find a MIME Type associated with this object."
 		    );
 		EDVPlaySoundWarning(core);
 		EDVMessageWarning(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Para redactar el de tipo MIME para este objeto,\n\
 va a Ver->De Tipo MIME...\n\
 \n\
@@ -1137,7 +1215,7 @@ Para la ayuda adicional a establecer de tipo MIME,\n\
 Vea Ayuda->De Tipo MIME...",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Pour diter le Type de MIME pour cet objet,\n\
 aller Regarder->Les Types De MIME...\n\
 \n\
@@ -1145,7 +1223,7 @@ Pour l'aide supplmentaire sur monte les Types de MIME,\n\
 L'Aide D'vch->Les Types De MIME...",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Um den MIME Typ fr dieses Objekt zu redigieren,\n\
 gehen Sie, Um->MIME Typen Anzusehen...\n\
 \n\
@@ -1153,7 +1231,7 @@ Fr zustzliche Hilfe stellend auf MIME Typen auf,\n\
 sieht Hilfe->MIME Typen...",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Per redigere il Tipo di MIME per quest'oggetto,\n\
 andare Osservare->I Tipi Di MIME...\n\
 \n\
@@ -1161,7 +1239,7 @@ Per l'aiuto ulteriore su installa i Tipi di MIME,\n\
 vedere L'Aiuto->I Tipi Di MIME...",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Om het MIME Type voor deze voorwerp te redigeren,\n\
 ga Om->MIME Typen Te Bekijken...\n\
 \n\
@@ -1169,7 +1247,7 @@ Voor bijkomende hulp op opstellen van MIME Typen,\n\
 ziet Hulp->MIME Typen...",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Editar o Tipo de MIMO para este objeto,\n\
 vai Ver->Tipos De MIMO...\n\
 \n\
@@ -1177,7 +1255,7 @@ Para ajuda adicional em armar Tipos de MIMO,\n\
 vem Ajuda->Tipos De MIMO...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Redigere MIME Type for dette objektet,\n\
 dra til View->MIME Types...\n\
 \n\
@@ -1185,7 +1263,7 @@ For ytterligere hjelp p innstilling ser opp MIME Types,\n\
 Help->MIME Types...",
 #else
 "Open Failed",
-buf,
+msg,
 "To edit the MIME Type for this object, go to\n\
 View->MIME Types...\n\
 \n\
@@ -1194,7 +1272,7 @@ Help->MIME Types...",
 #endif
 		    toplevel
 		);
-		g_free(buf);
+		g_free(msg);
 	    }
 	    return(-2);
 	}
@@ -1217,22 +1295,49 @@ Help->MIME Types...",
 	    EDVSetCWD(parent);
 	    g_free(parent);
 
-	    /* Format command */
+	    /* Format the command by substituting the tokens */
 	    paths_list = g_list_append(
 		paths_list,
 		STRDUP(path)
 	    );
 	    cmd = EDVOpenFormatCommand(core, command_base, paths_list);
-	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
-	    g_list_free(paths_list);
+	    if(paths_list != NULL)
+	    {
+		g_list_foreach(paths_list, (GFunc)g_free, NULL);
+		g_list_free(paths_list);
+	    }
 
-	    /* Execute command */
+	    /* Execute the command */
 	    pid = EDVSystem(cmd);
-	    if(pid <= 0)
+	    if(pid < 0)
+	    {
+		const gint error_code = (gint)errno;
+		if(verbose)
+		{
+		    gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			cmd, g_strerror(error_code)
+		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Open Failed",
+			msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+			toplevel
+		    );
+		    g_free(msg);
+		}
 		status = -1;
+	    }
 	    else
+	    {
 		status = 0;
-
+	    }
 	    g_free(cmd);
 
 	    /* Change back to the previous working dir */
@@ -1246,20 +1351,33 @@ Help->MIME Types...",
 /*
  *	Opens the object(s) specified in the paths_list.
  *
- *	The command_name specifies which MIME Type command must be used
- *	to open the object.
+ *	The core specifies the core.
  *
- *	If stdout_path_rtn or stderr_path_rtn are not NULL, then paths
- *	to the stdout and stderr files will be returned. The calling
- *	function must delete the returned strings.
+ *	The paths_list specifies a list of gchar * full paths of
+ *	the objects to open.
+ *
+ *	The command_name specifies the name of the command on the
+ *	matched MIME Type. If command_name is NULL then the default
+ *	command on the matched MIME Type will be used.
+ *
+ *	If verbose is TRUE then error and warning messages will be
+ *	displayed in a dialog.
+ *
+ *	The toplevel specifies the toplevel GtkWidget for the dialogs.
+ *
+ *	If stdout_path_rtn or stderr_path_rtn are not NULL then they
+ *	will be set with a dynamically allocated string describing the
+ *	stdout and stderr output file paths. The calling function must
+ *	delete these returned strings.
  *
  *	Returns:
  *
  *	0	Success.
  *	-1	General error.
- *	-2	Bad value, no MIME Type, or no application.
- *	-3	System error.
- *	-6	Call is a re-enterent to an existing call to this function.
+ *	-2	Invalid value.
+ *	-3	System error or memory allocation error.
+ *	-4	User responded with `Cancel'.
+ *	-6	Call is a renterent to an existing call to this function.
  */
 gint EDVOpen(
 	edv_core_struct *core,
@@ -1299,22 +1417,36 @@ gint EDVOpen(
 
 
 /*
- *	Opens the object(s) specified in the paths_list by prompting
- *	the user to select a method from the Open With Popup List.
+ *	Opens the objects by prompting the user to select a method from
+ *	the Open With Popup List.
  *
- *	If and only if stdout_path_rtn or stderr_path_rtn are not NULL,
- *	then they will be set with a dynamically allocated string
- *	containing the stdout and stderr output file paths. Which the
- *	calling function must deallocate.
+ *	The core specifies the core.
+ *
+ *	The paths_list specifies a list of gchar * full paths of
+ *	the objects to open.
+ *
+ *	The command_name specifies the name of the command on the
+ *	matched MIME Type. If command_name is NULL then the default
+ *	command on the matched MIME Type will be used.
+ *
+ *	If verbose is TRUE then error and warning messages will be
+ *	displayed in a dialog.
+ *
+ *	The toplevel specifies the toplevel GtkWidget for the dialogs.
+ *
+ *	If stdout_path_rtn or stderr_path_rtn are not NULL then they
+ *	will be set with a dynamically allocated string describing the
+ *	stdout and stderr output file paths. The calling function must
+ *	delete these returned strings.
  *
  *	Returns:
  *
- *	0	Success
- *	-1	General error
- *	-2	Bad value
- *	-3	System error
- *	-4	User responded with `Cancel'
- *	-6	Call is a renterent to an existing call to this function
+ *	0	Success.
+ *	-1	General error.
+ *	-2	Invalid value.
+ *	-3	System error or memory allocation error.
+ *	-4	User responded with `Cancel'.
+ *	-6	Call is a renterent to an existing call to this function.
  */
 gint EDVOpenWith(
 	edv_core_struct *core,
@@ -1383,9 +1515,7 @@ gint EDVOpenWith(
 	{
 	    if(verbose)
 	    {
-		gchar *buf;
-		if(mt != NULL)
-		    buf = g_strdup_printf(
+		gchar *msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "\"%s\" de tipo MIME no tiene una orden definida."
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1401,19 +1531,15 @@ gint EDVOpenWith(
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "MIME Type \"%s\" har ikke en definert kommando."
 #else
-"MIME Type \"%s\" does not have a defined command."
+"The MIME Type \"%s\" does not have a defined command."
 #endif
-			, mt->type
-		    );
-		else
-		    buf = STRDUP(
-"MIME Type does not have a defined command."
-		    );
+		    , mt->type
+		);
 		EDVPlaySoundWarning(core);
 		EDVMessageWarning(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Abra Fallado",
-buf,
+msg,
 "Para redactar el de tipo MIME para este objeto,\n\
 va a Ver->De Tipo MIME...\n\
 \n\
@@ -1421,7 +1547,7 @@ Para la ayuda adicional a establecer de tipo MIME,\n\
 Vea Ayuda->De Tipo MIME...",
 #elif defined(PROG_LANGUAGE_FRENCH)
 "Ouvrir A Echou",
-buf,
+msg,
 "Pour diter le Type de MIME pour cet objet,\n\
 aller Regarder->Les Types De MIME...\n\
 \n\
@@ -1429,7 +1555,7 @@ Pour l'aide supplmentaire sur monte les Types de MIME,\n\
 L'Aide D'vch->Les Types De MIME...",
 #elif defined(PROG_LANGUAGE_GERMAN)
 "Offen Versagt",
-buf,
+msg,
 "Um den MIME Typ fr dieses Objekt zu redigieren,\n\
 gehen Sie, Um->MIME Typen Anzusehen...\n\
 \n\
@@ -1437,7 +1563,7 @@ Fr zustzliche Hilfe stellend auf MIME Typen auf,\n\
 sieht Hilfe->MIME Typen...",
 #elif defined(PROG_LANGUAGE_ITALIAN)
 "Aprire Fallito",
-buf,
+msg,
 "Per redigere il Tipo di MIME per quest'oggetto,\n\
 andare Osservare->I Tipi Di MIME...\n\
 \n\
@@ -1445,7 +1571,7 @@ Per l'aiuto ulteriore su installa i Tipi di MIME,\n\
 vedere L'Aiuto->I Tipi Di MIME...",
 #elif defined(PROG_LANGUAGE_DUTCH)
 "Open Verzuimde",
-buf,
+msg,
 "Om het MIME Type voor deze voorwerp te redigeren,\n\
 ga Om->MIME Typen Te Bekijken...\n\
 \n\
@@ -1453,7 +1579,7 @@ Voor bijkomende hulp op opstellen van MIME Typen,\n\
 ziet Hulp->MIME Typen...",
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 "Abra Fracassado",
-buf,
+msg,
 "Editar o Tipo de MIMO para este objeto,\n\
 vai Ver->Tipos De MIMO...\n\
 \n\
@@ -1461,7 +1587,7 @@ Para ajuda adicional em armar Tipos de MIMO,\n\
 vem Ajuda->Tipos De MIMO...",
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 "pen Failed",
-buf,
+msg,
 "Redigere MIME Type for dette objektet,\n\
 dra til View->MIME Types...\n\
 \n\
@@ -1469,7 +1595,7 @@ For ytterligere hjelp p innstilling ser opp MIME Types,\n\
 Help->MIME Types...",
 #else
 "Open Failed",
-buf,
+msg,
 "To edit the MIME Type for this object, go to\n\
 View->MIME Types...\n\
 \n\
@@ -1478,7 +1604,7 @@ Help->MIME Types...",
 #endif
 		    toplevel
 		);
-		g_free(buf);
+		g_free(msg);
 	    }
 	    return(-2);
 	}
@@ -1488,30 +1614,59 @@ Help->MIME Types...",
 	if(TRUE)
 	{
 	    gint pid;
-	    gchar *parent, *cmd;
+	    gchar *cmd;
 
 	    /* Record the previous working dir */
 	    gchar *pwd = EDVGetCWD();
 
-	    /* Set new working dir as the parent of the specified
-	     * path
+	    /* Set the working directory as the parent of the first
+	     * path in the list
 	     */
-	    parent = g_dirname(
-		(paths_list != NULL) ? (gchar *)paths_list->data : NULL
-	    );
-	    EDVSetCWD(parent);
-	    g_free(parent);
+	    if(paths_list != NULL)
+	    {
+		const gchar *path = (const gchar *)paths_list->data;
+		if(path != NULL)
+		{
+		    gchar *parent = g_dirname(path);
+		    EDVSetCWD(parent);
+		    g_free(parent);
+		}
+	    }
 
-	    /* Format command */
+	    /* Format the command by substituting the tokens */
 	    cmd = EDVOpenFormatCommand(core, command_base, paths_list);
 
-	    /* Execute command */
+	    /* Execute the command */
 	    pid = EDVSystem(cmd);
-	    if(pid <= 0)
+	    if(pid < 0)
+	    {
+		const gint error_code = (gint)errno;
+		if(verbose)
+		{
+		    gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			cmd, g_strerror(error_code)
+		    );
+		    EDVPlaySoundError(core);
+		    EDVMessageError(
+			"Open Failed",
+			msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+			toplevel
+		    );
+		    g_free(msg);
+		}
 		status = -1;
+	    }
 	    else
+	    {
 		status = 0;
-
+	    }
 	    g_free(cmd);
 
 	    /* Change back to the previous working dir */
@@ -1521,3 +1676,111 @@ Help->MIME Types...",
 
 	return(status);
 }
+
+/*
+ *	Opens the objects with the command.
+ *
+ *	The core specifies the core.
+ *
+ *	The paths_list specifies a list of gchar * full paths of
+ *	the objects to open.
+ *
+ *	The command specifies the unsubstituted command string.
+ *
+ *	If verbose is TRUE then error and warning messages will be
+ *	displayed in a dialog.
+ *
+ *	The toplevel specifies the toplevel GtkWidget for the dialogs.
+ *
+ *	If stdout_path_rtn or stderr_path_rtn are not NULL then they
+ *	will be set with a dynamically allocated string describing the
+ *	stdout and stderr output file paths. The calling function must
+ *	delete these returned strings.
+ *
+ *	Returns:
+ *
+ *	0	Success.
+ *	-1	General error.
+ *	-2	Invalid value.
+ *	-3	System error or memory allocation error.
+ *	-4	User responded with `Cancel'.
+ *	-6	Call is a renterent to an existing call to this function.
+ */
+gint EDVOpenCommand(
+	edv_core_struct *core,
+        GList *paths_list,
+	const gchar *command,
+	GtkWidget *toplevel,
+        const gboolean verbose,
+        gchar **stdout_path_rtn, gchar **stderr_path_rtn
+)
+{
+	gint pid, status;
+	const gchar *path;
+	gchar *cmd, *pwd;
+
+	/* Reset returns */
+	if(stdout_path_rtn != NULL)
+	    *stdout_path_rtn = NULL;
+	if(stderr_path_rtn != NULL)
+	    *stderr_path_rtn = NULL;
+
+	if((core == NULL) || (paths_list == NULL) || STRISEMPTY(command))
+	    return(-2);
+
+	/* Record the previous working dir */
+	pwd = EDVGetCWD();
+
+	/* Set the working directory as the parent of the first
+	 * path in the list
+	 */
+	path = (const gchar *)paths_list->data;
+	if(path != NULL)
+	{
+	    gchar *parent = g_dirname(path);
+	    EDVSetCWD(parent);
+	    g_free(parent);
+	}
+
+	/* Format the command by substituting the tokens */
+	cmd = EDVOpenFormatCommand(core, command, paths_list);
+
+	/* Execute the command */
+	pid = EDVSystem(cmd);
+	if(pid < 0)
+	{
+	    const gint error_code = (gint)errno;
+	    if(verbose)
+	    {
+		gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+		    cmd, g_strerror(error_code)
+		);
+		EDVPlaySoundError(core);
+		EDVMessageError(
+		    "Open Failed",
+		    msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+		    toplevel
+		);
+		g_free(msg);
+	    }
+	    status = -1;
+	}
+	else
+	{
+	    status = 0;
+	}
+	g_free(cmd);
+
+	/* Change back to the previous working dir */
+	EDVSetCWD(pwd);
+	g_free(pwd);
+
+	return(status);
+}
