@@ -25,20 +25,20 @@
 #include "edv_cfg_list.h"
 #include "config.h"
 
-#include "images/pdi_file01_20x20.xpm"
-#include "images/pdi_file02_20x20.xpm"
-#include "images/pdi_file03_20x20.xpm"
-#include "images/pdi_file04_20x20.xpm"
-#include "images/pdi_file05_20x20.xpm"
-#include "images/pdi_file06_20x20.xpm"
-#include "images/pdi_folder_32x32.xpm"
-#include "images/pdi_folderfile_32x32.xpm"
-#include "images/pdi_trash01_20x20.xpm"
-#include "images/pdi_trash02_20x20.xpm"
-#include "images/pdi_trash03_20x20.xpm"
-#include "images/pdi_trash04_20x20.xpm"
-#include "images/pdi_trash05_20x20.xpm"
-#include "images/pdi_trash06_20x20.xpm"
+#include "animations/file01_20x20.xpm"
+#include "animations/file02_20x20.xpm"
+#include "animations/file03_20x20.xpm"
+#include "animations/file04_20x20.xpm"
+#include "animations/file05_20x20.xpm"
+#include "animations/file06_20x20.xpm"
+#include "animations/folder_32x32.xpm"
+#include "animations/folderfile_32x32.xpm"
+#include "animations/trash01_20x20.xpm"
+#include "animations/trash02_20x20.xpm"
+#include "animations/trash03_20x20.xpm"
+#include "animations/trash04_20x20.xpm"
+#include "animations/trash05_20x20.xpm"
+#include "animations/trash06_20x20.xpm"
 
 #include "images/icon_trash_32x32.xpm"
 
@@ -49,14 +49,17 @@
  *	0	Success.
  *	-1	General error.
  *	-2	Invalid value.
- *	-3	Systems error; out of memory or out of disk space.
+ *	-3	Systems error, out of memory, or out of disk space.
  *	-4	User responded with "Cancel".
  *	-5	User responded with "No" or response was not available.
- *	-6	Call would cause reentry.
+ *	-6	An operation is already in progress.
  */
 
 
-const gchar *EDVRecBinDeleteObjectGetError(void);
+const gchar *EDVRecBinDeleteObjectGetError(edv_core_struct *core);
+static void EDVRecBinDeleteCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+);
 
 static gboolean EDVRecBinNoDeleteCheck(
 	edv_core_struct *core, GtkWidget *toplevel,
@@ -93,19 +96,16 @@ gint EDVRecBinDeleteObject(
 );
 
 
-static const gchar *last_error = NULL;
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -113,9 +113,28 @@ static const gchar *last_error = NULL;
  *	error message (or NULL if there was no error) since the last
  *	call to a EDVRecBinFOP*() function.
  */
-const gchar *EDVRecBinDeleteObjectGetError(void)
+const gchar *EDVRecBinDeleteObjectGetError(edv_core_struct *core)
 {
-	return(last_error);
+	return((core != NULL) ? core->recbin_last_error : NULL);
+}
+
+/*
+ *	Coppies the error message to the core's recbin_last_error_buf
+ *	and sets the core's recbin_last_error to point to it.
+ */
+static void EDVRecBinDeleteCopyErrorMessage(
+	edv_core_struct *core, const gchar *msg
+)
+{
+	if(core == NULL)
+	    return;
+
+	core->recbin_last_error = NULL;
+
+	g_free(core->recbin_last_error_buf);
+	core->recbin_last_error_buf = STRDUP(msg);
+
+	core->recbin_last_error = core->recbin_last_error_buf;
 }
 
 
@@ -613,30 +632,30 @@ static void EDVRecBinDeleteMapProgressDialog(
 
 	if(delete_method == EDV_DELETE_METHOD_PURGE)
 	{
-	    start_icon_data[0] = (guint8 **)pdi_folderfile_32x32_xpm;
-	    start_icon_data[1] = (guint8 **)pdi_folder_32x32_xpm;
-	    start_icon_data[2] = (guint8 **)pdi_folder_32x32_xpm;
-	    icon_data[0] = (guint8 **)pdi_trash01_20x20_xpm;
-	    icon_data[1] = (guint8 **)pdi_trash02_20x20_xpm;
-	    icon_data[2] = (guint8 **)pdi_trash03_20x20_xpm;
-	    icon_data[3] = (guint8 **)pdi_trash04_20x20_xpm;
-	    icon_data[4] = (guint8 **)pdi_trash05_20x20_xpm;
-	    icon_data[5] = (guint8 **)pdi_trash06_20x20_xpm;
+	    start_icon_data[0] = (guint8 **)folderfile_32x32_xpm;
+	    start_icon_data[1] = (guint8 **)folder_32x32_xpm;
+	    start_icon_data[2] = (guint8 **)folder_32x32_xpm;
+	    icon_data[0] = (guint8 **)trash01_20x20_xpm;
+	    icon_data[1] = (guint8 **)trash02_20x20_xpm;
+	    icon_data[2] = (guint8 **)trash03_20x20_xpm;
+	    icon_data[3] = (guint8 **)trash04_20x20_xpm;
+	    icon_data[4] = (guint8 **)trash05_20x20_xpm;
+	    icon_data[5] = (guint8 **)trash06_20x20_xpm;
 	    end_icon_data[0] = (guint8 **)NULL;
 	    end_icon_data[1] = (guint8 **)NULL;
 	    end_icon_data[2] = (guint8 **)NULL;
 	}
 	else
 	{
-	    start_icon_data[0] = (guint8 **)pdi_folderfile_32x32_xpm;
-	    start_icon_data[1] = (guint8 **)pdi_folder_32x32_xpm;
-	    start_icon_data[2] = (guint8 **)pdi_folder_32x32_xpm;
-	    icon_data[0] = (guint8 **)pdi_file01_20x20_xpm;
-	    icon_data[1] = (guint8 **)pdi_file02_20x20_xpm;
-	    icon_data[2] = (guint8 **)pdi_file03_20x20_xpm;
-	    icon_data[3] = (guint8 **)pdi_file04_20x20_xpm;
-	    icon_data[4] = (guint8 **)pdi_file05_20x20_xpm;
-	    icon_data[5] = (guint8 **)pdi_file06_20x20_xpm;
+	    start_icon_data[0] = (guint8 **)folderfile_32x32_xpm;
+	    start_icon_data[1] = (guint8 **)folder_32x32_xpm;
+	    start_icon_data[2] = (guint8 **)folder_32x32_xpm;
+	    icon_data[0] = (guint8 **)file01_20x20_xpm;
+	    icon_data[1] = (guint8 **)file02_20x20_xpm;
+	    icon_data[2] = (guint8 **)file03_20x20_xpm;
+	    icon_data[3] = (guint8 **)file04_20x20_xpm;
+	    icon_data[4] = (guint8 **)file05_20x20_xpm;
+	    icon_data[5] = (guint8 **)file06_20x20_xpm;
 	    end_icon_data[0] = (guint8 **)icon_trash_32x32_xpm;
 	    end_icon_data[1] = (guint8 **)icon_trash_32x32_xpm;
 	    end_icon_data[2] = (guint8 **)icon_trash_32x32_xpm;
@@ -746,28 +765,35 @@ static guint EDVRecBinDeleteObjectIterate(
 
 
 	/* Error or user aborted from prior call? */
-	if(*status)
+	if(*status != 0)
 	    return(0);
 
-	/* Get local stats of the given disk object to be deleted */
+	/* Get the local stats of the object to be deleted */
 	if(lstat((const char *)path, &lstat_buf))
 	{
-	    *status = -2;
-	    last_error =
-"Unable to get the object's statistics";
+	    const gint error_code = (gint)errno;
+	    gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+		g_strerror(error_code), path
+	    );
+	    EDVRecBinDeleteCopyErrorMessage(core, msg);
+	    g_free(msg);
+	    *status = -1;
 	    return(0);
 	}
 
 #if 0
-/* Let system determine this */
-	/* If effective user ID of this process does not own object
-	 * then the object unable to be deleted
+/* Let the system determine this */
+	/* If the effective user ID of this process does not own
+	 * object then the object unable to be deleted
 	 */
 	if(!EDVCheckIsOwner(core, lstat_buf.st_uid))
 	{
-	    *status = -2;
-	    last_error =
-"Effective user id of this process does not own the object";
+	    *status = -1;
+	    core->recbin_last_error =
+"Effective user id of this process does not own the object.";
 	    return(0);
 	}
 #endif
@@ -785,14 +811,25 @@ static guint EDVRecBinDeleteObjectIterate(
 	    );
 	if(no_delete)
 	{
-	    *status = -2;
+	    *status = -1;
 	    /* Set last error only if not interactive since calling
 	     * EDVRecBinNoDeleteCheck() would have already displayed 
 	     * appropriate error message
 	     */
 	    if(!interactive)
-		last_error =
-"Object is part of the function of this process or system and may not be deleted";
+	    {
+		gchar *msg = g_strdup_printf(
+"The object:\n\
+\n\
+    %s\n\
+\n\
+Is part of the function of this process or system\n\
+and may not be deleted.",
+		    path
+		);
+		EDVRecBinDeleteCopyErrorMessage(core, msg);
+		g_free(msg);
+	    }
 	    return(0);
 	}
 
@@ -815,8 +852,8 @@ static guint EDVRecBinDeleteObjectIterate(
 	    strv = StringQSort(strv, strc);
 	    if(strv == NULL)
 	    {
-		*status = -2;
-		last_error = "Unable to sort directory entries";
+		*status = -1;
+		core->recbin_last_error = "Unable to sort directory entries.";
 		return(0);
 	    }
 
@@ -836,7 +873,7 @@ static guint EDVRecBinDeleteObjectIterate(
 		    FREE_AND_CONTINUE
 
 		/* Error encountered or user abort? */
-		if(*status)
+		if(*status != 0)
 		    FREE_AND_CONTINUE
 
 		/* Skip special dir notations */
@@ -871,7 +908,7 @@ static guint EDVRecBinDeleteObjectIterate(
 	}
 
 	/* Error or user aborted while deleting any child objects? */
-	if(*status)
+	if(*status != 0)
 	    return(0);
 
 	/* Update progress to display the path of this object that is
@@ -940,8 +977,6 @@ static guint EDVRecBinDeleteObjectIterate(
 	if(delete_method == EDV_DELETE_METHOD_PURGE)
 	{
 	    /* Permanently purge this object */
-
-	    /* Get system permissions of object */
 	    const mode_t m = lstat_buf.st_mode;
 
 	    /* Delete by object type */
@@ -954,7 +989,17 @@ static guint EDVRecBinDeleteObjectIterate(
 	    {
 		if(rmdir((const char *)path))
 		{
-		    last_error = "Unable to delete the directory";
+		    const gint error_code = (gint)errno;
+		    gchar *msg = g_strdup_printf(
+"Unable to delete the directory:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			path, g_strerror(error_code)
+		    );
+		    EDVRecBinDeleteCopyErrorMessage(core, msg);
+		    g_free(msg);
 		    *status = -1;
 		}
 	    }
@@ -963,7 +1008,17 @@ static guint EDVRecBinDeleteObjectIterate(
 	    {
 		if(unlink((const char *)path))
 		{
-		    last_error = "Unable to delete the object";
+		    const gint error_code = (gint)errno;
+		    gchar *msg = g_strdup_printf(
+"Unable to delete:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			path, g_strerror(error_code)
+		    );
+		    EDVRecBinDeleteCopyErrorMessage(core, msg);
+		    g_free(msg);
 		    *status = -1;
 	        }
 	    }
@@ -993,13 +1048,11 @@ static guint EDVRecBinDeleteObjectIterate(
 		return(0);
 
 	    /* Set values from the object to new recycled object */
-	    g_free(obj->name);
 	    obj->name = STRDUP(g_basename(path));
 
 	    parent_path = g_dirname(path);
 	    if(parent_path == NULL)
 		parent_path = STRDUP("/");
-	    g_free(obj->original_path);
 	    obj->original_path = STRDUP(parent_path);
 
 	    obj->deleted_time = cur_time;
@@ -1008,24 +1061,21 @@ static guint EDVRecBinDeleteObjectIterate(
 
 #ifdef S_ISLNK
 	    if(S_ISLNK(m))
-	    {
-		g_free(obj->link_value);
-		obj->link_value = EDVGetLinkValue(path);
-	    }
+		obj->link_target = EDVGetLinkTarget(path);
 #endif
 
 	    obj->permissions = EDVObjectGetPermissionsFromStatMode(m);
 
-	    obj->access_time = lstat_buf.st_atime;
-	    obj->modify_time = lstat_buf.st_mtime;
-	    obj->change_time = lstat_buf.st_ctime;
+	    obj->access_time = (gulong)lstat_buf.st_atime;
+	    obj->modify_time = (gulong)lstat_buf.st_mtime;
+	    obj->change_time = (gulong)lstat_buf.st_ctime;
 
-	    obj->owner_id = lstat_buf.st_uid;
-	    obj->group_id = lstat_buf.st_gid;
+	    obj->owner_id = (gint)lstat_buf.st_uid;
+	    obj->group_id = (gint)lstat_buf.st_gid;
 
-	    obj->size = lstat_buf.st_size;
+	    obj->size = (gulong)lstat_buf.st_size;
 
-	    /* Add object to recycled index file */
+	    /* Add this object to the recycled objects index file */
 	    index = EDVRecBinIndexAdd(recycled_index_file, obj);
 	    if(index != 0)
 	    {
@@ -1048,7 +1098,16 @@ static guint EDVRecBinDeleteObjectIterate(
 		/* Other error occured? */
 		else if(*status != 0)
 		{
-		    last_error = EDVRecBinIndexGetError();
+		    gchar *msg = g_strdup_printf(
+"Unable to delete:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+			path, EDVRecBinIndexGetError()
+		    );
+		    EDVRecBinDeleteCopyErrorMessage(core, msg);
+		    g_free(msg);
 		    EDVRecBinIndexRemove(recycled_index_file, index);
 		    index = 0;
 		}
@@ -1064,13 +1123,11 @@ static guint EDVRecBinDeleteObjectIterate(
 	    }
 	    else
 	    {
-		last_error = "Unable to generate new recycled object index";
+		core->recbin_last_error = "Unable to generate a new recycled object index.";
 		*status = -1;
 	    }
 
-	    /* Delete tempory recycled object structure, it is no
-	     * longer needed
-	     */
+	    /* Delete the tempory recycled object */
 	    EDVRecycledObjectDelete(obj);
 
 	    g_free(parent_path);
@@ -1108,14 +1165,15 @@ gint EDVRecBinDeleteObject(
 	gchar *lsrc_obj = NULL;
 	const cfg_item_struct *cfg_list;
 
-	last_error = NULL;
+	/* Clear the last error message */
+	EDVRecBinDeleteCopyErrorMessage(core, NULL);
 
 	if(index_list_rtn != NULL)
 	    *index_list_rtn = NULL;
 
 	if((core == NULL) || STRISEMPTY(path))
 	{
-	    last_error = "Invalid value";
+	    EDVRecBinDeleteCopyErrorMessage(core, "Invalid value.");
 	    return(-1);
 	}
 
@@ -1132,7 +1190,7 @@ gint EDVRecBinDeleteObject(
 	);
 	if(recycled_index_file == NULL)
 	{
-	    last_error = "Unable to get the recycled objects index file";
+	    core->recbin_last_error = "Unable to get the recycled objects index file.";
 	    DO_FREE_LOCALS
 	    return(-1);
 	}
@@ -1150,7 +1208,7 @@ gint EDVRecBinDeleteObject(
 	lsrc_obj = STRDUP(path);
 	if(lsrc_obj == NULL)
 	{
-	    last_error = "Memory allocation error";
+	    core->recbin_last_error = "Memory allocation error.";
 	    DO_FREE_LOCALS
 	    return(-3);
 	}
@@ -1174,7 +1232,7 @@ gint EDVRecBinDeleteObject(
 	    status,
 	    lsrc_obj,
 	    NULL,		/* No target */
-	    last_error
+	    core->recbin_last_error
 	);
 
 	DO_FREE_LOCALS
