@@ -11,7 +11,7 @@
 #include "edv_recycled_obj.h"
 
 
-extern const gchar *EDVRecBinPurgeGetError(void);
+extern const gchar *EDVRecBinPurgeGetError(edv_core_struct *core);
 
 extern gint EDVRecBinPurge(
 	edv_core_struct *core,
