@@ -11,7 +11,7 @@
 #include "edv_recycled_obj.h"
 
 
-extern const gchar *EDVRecBinRecoverGetError(void);
+extern const gchar *EDVRecBinRecoverGetError(edv_core_struct *core);
 
 extern gint EDVRecBinRecover(
 	edv_core_struct *core,
