@@ -32,10 +32,10 @@
  *	0	Success.
  *	-1	General error.
  *	-2	Invalid value.
- *	-3	Systems error; out of memory or out of disk space.
+ *	-3	Systems error, out of memory, or out of disk space.
  *	-4	User responded with "Cancel".
  *	-5	User responded with "No" or response was not available.
- *	-6	Call would cause reentry.
+ *	-6	An operation is already in progress.
  */
 
 
@@ -75,9 +75,6 @@ gint EDVRecBinSync(
 );
 
 
-static const gchar *last_error = NULL;
-
-
 #define STATUS_MESSAGE(_s_)	{		\
  if(status_message_cb != NULL)			\
   status_message_cb((_s_), data);		\
@@ -1204,51 +1201,29 @@ gint EDVRecBinSync(
 	gpointer data
 )
 {
-	static gboolean reenterent = FALSE;
-	static gint status;
-	static gboolean lyes_to_all;
+	gint status = 0;
+	gboolean lyes_to_all = FALSE;
 	const gchar *index_path;
 	gchar *recbin_path = NULL;
 	gulong time_start = (gulong)time(NULL);
 	struct stat stat_buf;
 	const cfg_item_struct *cfg_list;
 
-	if(reenterent)
-	{
-	    status = -6;
-	    return(status);
-	}
-	else
-	{
-	    reenterent = TRUE;
-	}
-
 #define POSTOP_AND_RETURN(rtn_val)	{	\
- /* Report final progress */			\
+ /* Report the final progress */		\
  if(status_progress_cb != NULL)			\
   status_progress_cb(1.0f, data);		\
 						\
  g_free(recbin_path);				\
- recbin_path = NULL;				\
 						\
- reenterent = FALSE;				\
  return(rtn_val);				\
 }
 
-	/* Reset last error */
-	last_error = NULL;
-
-	/* Reset status */
-	status = 0;
-
-	/* Use local yes_to_all if none is given */
+	/* Use the local yes_to_all if none is given */
 	if(yes_to_all == NULL)
-	{
-	    lyes_to_all = FALSE;	/* Reset when using local */
 	    yes_to_all = &lyes_to_all;
-	}
 
-	/* Report initial progress */
+	/* Report the initial progress */
 	if(status_progress_cb != NULL)
 	    status_progress_cb(0.0f, data);
 
@@ -1260,10 +1235,13 @@ gint EDVRecBinSync(
 "Invalid value."
 	    );
 	    if(status == 0)
-		status = -1;
+		status = -2;
 	    POSTOP_AND_RETURN(status);
 	}
 
+	/* Clear the last error message */
+	core->recbin_last_error = NULL;
+
 	cfg_list = core->cfg_list;
 
 	/* Get path to the recycle bin index file */
@@ -1469,7 +1447,7 @@ The recycle bin has been compacted."
 	    status,
 	    index_path,	/* Source */
 	    NULL,		/* Target */
-	    last_error		/* Comment */
+	    core->recbin_last_error
 	);
 
 	POSTOP_AND_RETURN(status);
