@@ -47,7 +47,7 @@ struct _edv_recycled_object_struct {
 				 * EDV_OBJECT_TYPE_FILE */
 
 	/* Link */
-	gchar		*link_value;	/* Only for EDV_OBJECT_TYPE_LINK */
+	gchar		*link_target;	/* Only for EDV_OBJECT_TYPE_LINK */
 
 	/* Permissions */
 	edv_permission_flags permissions;
@@ -64,10 +64,9 @@ struct _edv_recycled_object_struct {
 };
 
 
-/* edv_recycled_obj.c */
 extern edv_recycled_object_struct *EDVRecycledObjectNew(void);
 extern edv_recycled_object_struct *EDVRecycledObjectCopy(
-	const edv_recycled_object_struct *obj
+	edv_recycled_object_struct *obj
 );
 extern void EDVRecycledObjectDelete(
 	edv_recycled_object_struct *obj
