@@ -30,37 +30,37 @@ edv_status_bar_struct *EDVStatusBarNew(
 );
 void EDVStatusBarUpdate(edv_status_bar_struct *sb);
 void EDVStatusBarSetBusy(
-	edv_status_bar_struct *sb, gboolean busy  
+	edv_status_bar_struct *sb, const gboolean busy  
 );
 void EDVStatusBarMap(edv_status_bar_struct *sb);
 void EDVStatusBarUnmap(edv_status_bar_struct *sb);
 void EDVStatusBarDelete(edv_status_bar_struct *sb);
 
 void EDVStatusBarMessage(
-	edv_status_bar_struct *sb, const gchar *message,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gchar *msg,
+	const gboolean allow_gtk_iteration
 );
 void EDVStatusBarProgress(
-	edv_status_bar_struct *sb, gfloat percent,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gfloat v,
+	const gboolean allow_gtk_iteration
 );
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
-
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
-
-
 #define STATUS_BAR_HEIGHT		26
 
 
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
+
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
+
+
 /*
  *	Status Bar "button_press_event" or "button_release_event"
  *	signal callback.
@@ -128,7 +128,7 @@ static gint EDVStatusBarButtonEventCB(
 }
 
 /*
- *      Status Bar "enter_notify_event" or "leave_notify_event"
+ *	Status Bar "enter_notify_event" or "leave_notify_event"
  *	signal callback.
  */
 static gint EDVStatusBarCrossingEventCB(
@@ -149,12 +149,12 @@ static gint EDVStatusBarCrossingEventCB(
 	/* Write protect enabled */
 	if(widget == sb->write_protect_enabled_eb)
 	{
-	    const gchar *mesg = NULL;
+	    const gchar *msg = NULL;
 
 	    switch(etype)
 	    {
 	      case GDK_ENTER_NOTIFY:
-		mesg =
+		msg =
 #if defined(PROG_LANGUAGE_SPANISH)
 "Proteccion de escritura habilitada, haga clic para anularla"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -175,21 +175,21 @@ static gint EDVStatusBarCrossingEventCB(
 		;
 		break;
 	      case GDK_LEAVE_NOTIFY:
-		mesg = NULL;
+		msg = NULL;
 		break;
 	    }
-	    EDVStatusBarMessage(sb, mesg, FALSE);
+	    EDVStatusBarMessage(sb, msg, FALSE);
 	    status = TRUE;
 	}
 	/* Write protect disabled */
 	else if(widget == sb->write_protect_disabled_eb)
 	{
-	    const gchar *mesg = NULL;
+	    const gchar *msg = NULL;
 
 	    switch(etype)
 	    {
 	      case GDK_ENTER_NOTIFY:
-		mesg =
+		msg =
 #if defined(PROG_LANGUAGE_SPANISH)
 "Proteccion de escritura deshabilitada, haga clic para habilitarla"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -211,21 +211,21 @@ static gint EDVStatusBarCrossingEventCB(
 		break;
 
 	      case GDK_LEAVE_NOTIFY:
-		mesg = "";
+		msg = "";
 		break;
 	    }
-	    EDVStatusBarMessage(sb, mesg, FALSE);
+	    EDVStatusBarMessage(sb, msg, FALSE);
 	    status = TRUE;
 	}
 	/* Delete method recycle */
 	else if(widget == sb->delete_method_recycle_eb)
 	{
-	    const gchar *mesg = NULL;
+	    const gchar *msg = NULL;
 
 	    switch(etype)
 	    {
 	      case GDK_ENTER_NOTIFY:
-		mesg =
+		msg =
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mtodo de borrado establecido a reciclar, hacer clic para ponerlo a purgar"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -250,21 +250,21 @@ static gint EDVStatusBarCrossingEventCB(
 		break;
 
 	      case GDK_LEAVE_NOTIFY: 
-		mesg = "";
+		msg = "";
 		break;
 	    }
-	    EDVStatusBarMessage(sb, mesg, FALSE);
+	    EDVStatusBarMessage(sb, msg, FALSE);
 	    status = TRUE;
 	}
 	/* Delete method purge */
 	else if(widget == sb->delete_method_purge_eb)
 	{
-	    const gchar *mesg = NULL;
+	    const gchar *msg = NULL;
 
 	    switch(etype)
 	    {
 	      case GDK_ENTER_NOTIFY:
-		mesg =
+		msg =
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mtodo de borrado establecido a purgar, hacer clic para ponerlo a reciclar"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -288,10 +288,10 @@ static gint EDVStatusBarCrossingEventCB(
 		break;
 
 	      case GDK_LEAVE_NOTIFY:
-		mesg = "";
+		msg = "";
 		break;
 	    }
-	    EDVStatusBarMessage(sb, mesg, FALSE);
+	    EDVStatusBarMessage(sb, msg, FALSE);
 	    status = TRUE;
 	}
 
@@ -307,26 +307,26 @@ edv_status_bar_struct *EDVStatusBarNew(
 )
 {
 	GtkStyle *style;
-	GtkWidget *w, *parent2, *parent3;
+	GtkWidget *w, *parent2, *parent3, *toplevel;
 	GtkAdjustment *adj;
-	edv_status_bar_struct *sb = EDV_STATUS_BAR(
-	    g_malloc0(sizeof(edv_status_bar_struct))
-	);
-	if((sb == NULL) || (core == NULL))
-	{
-	    g_free(sb);
-	    return(NULL);
-	}
+	edv_status_bar_struct *sb;
 
-	sb->busy_count = 0;
+	if(core == NULL)
+	    return(NULL);
+
+	sb = EDV_STATUS_BAR(g_malloc0(sizeof(edv_status_bar_struct)));
+	if(sb == NULL)
+	    return(NULL);
+
+	sb->toplevel = toplevel = gtk_frame_new(NULL);
 	sb->freeze_count = 0;
+	sb->busy_count = 0;
 	sb->core = core;
 
+	sb->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel frame */
-	sb->toplevel = w = gtk_frame_new(NULL);
+	/* Toplevel GtkFrame */
+	w = toplevel;
 	gtk_widget_set_usize(w, -1, STATUS_BAR_HEIGHT);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
 	gtk_container_border_width(GTK_CONTAINER(w), 0);
@@ -334,24 +334,17 @@ edv_status_bar_struct *EDVStatusBarNew(
 	    gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	else if(GTK_IS_CONTAINER(parent))
 	    gtk_container_add(GTK_CONTAINER(parent), w);
-	else
-	    g_printerr(
-"EDVStatusBarNew(): Warning:\
- parent 0x%.8x is not a GtkContainer.\n",
-		(guint)parent
-	    );
 	style = gtk_widget_get_style(w);
 	parent2 = w;
 
-	/* Table in main frame */
+	/* Main GtkTable */
 	w = gtk_table_new(1, 4, FALSE);
 	gtk_container_add(GTK_CONTAINER(parent2), w);
 	gtk_widget_show(w);
 	parent2 = w;
 
-
 	/* Write protect enabled/disabled */
-	/* Write protect enabled */
+	/* Write protect enabled GtkEventBox */
 	sb->write_protect_enabled_eb = w = gtk_event_box_new();
 	gtk_widget_add_events(
 	    w,
@@ -376,7 +369,8 @@ edv_status_bar_struct *EDVStatusBarNew(
 	    0, 0,
 	    2, 0
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Proteccion de escritura habilitada"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -406,8 +400,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 	gtk_container_add(GTK_CONTAINER(parent3), w);
 	gtk_widget_show(w);
 
-
-	/* Write protect disabled */
+	/* Write protect disabled GtkEventBox */
 	sb->write_protect_disabled_eb = w = gtk_event_box_new();
 	gtk_widget_add_events(
 	    w,
@@ -432,7 +425,8 @@ edv_status_bar_struct *EDVStatusBarNew(
 	    0, 0,
 	    2, 0
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Proteccion de escritura deshabilitada"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -465,7 +459,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 
 
 	/* Delete method */
-	/* Delete method recycle */         
+	/* Delete method recycle GtkEventBox */
 	sb->delete_method_recycle_eb = w = gtk_event_box_new();
 	gtk_widget_add_events(
 	    w,
@@ -490,7 +484,8 @@ edv_status_bar_struct *EDVStatusBarNew(
 	    0, 0,
 	    2, 0
 	);
-	GUISetWidgetTip(w,
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mtodo de borrado seleccionado a reciclar"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -510,7 +505,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 #endif
 	);
 	gtk_widget_show(w);
-	parent3 = w;  
+	parent3 = w;
 	/* Icon */
 	w = EDVNewPixmapWidget(
 	    core,
@@ -521,7 +516,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 	gtk_container_add(GTK_CONTAINER(parent3), w);
 	gtk_widget_show(w);
 
-	/* Delete method purge */
+	/* Delete method purge GtkEventBox */
 	sb->delete_method_purge_eb = w = gtk_event_box_new();
 	gtk_widget_add_events(
 	    w,
@@ -545,8 +540,9 @@ edv_status_bar_struct *EDVStatusBarNew(
 	    1, 2, 0, 1,
 	    0, 0,
 	    2, 0 
-	);      
-	GUISetWidgetTip(w,
+	);
+	GUISetWidgetTip(
+	    w,
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mtodo de borrado seleccionado a purgar"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -577,7 +573,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 	gtk_widget_show(w);
 
 
-	/* Progress bar */
+	/* GtkProgressBar */
 	adj = (GtkAdjustment *)gtk_adjustment_new(
 	    0.0f, 1.0f, 100.0f,
 	    0.0f, 0.0f, 0.0f
@@ -602,7 +598,7 @@ edv_status_bar_struct *EDVStatusBarNew(
 	);
 	gtk_widget_show(w);
 
-	/* Label */
+	/* Label GtkFrame */
 	w = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
 	gtk_container_border_width(GTK_CONTAINER(w), 1);
@@ -621,11 +617,14 @@ edv_status_bar_struct *EDVStatusBarNew(
 	gtk_widget_show(w);
 	parent3 = w;
 
+	/* GtkLabel */
 	sb->label = w = gtk_label_new("");
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 2);
 	gtk_widget_show(w);
 
+	sb->freeze_count--;
+
 	return(sb);
 }
 
@@ -640,27 +639,22 @@ void EDVStatusBarUpdate(edv_status_bar_struct *sb)
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
 
-
 	if(sb == NULL)
 	    return;
 
-	core = sb->core;
-	if(core == NULL)
-	    return;
+	sb->freeze_count++;
 
+	core = sb->core;
 	cfg_list = core->cfg_list;
 
-	/* Get configuration */
+	/* Get the configuration values */
 	write_protect = CFGItemListGetValueI(
 	    cfg_list, EDV_CFG_PARM_WRITE_PROTECT
 	);
-	delete_method = CFGItemListGetValueI(          
+	delete_method = CFGItemListGetValueI(
 	    cfg_list, EDV_CFG_PARM_DELETE_METHOD
 	);
 
-
-	/* Begin updating */
-
 	/* Write protect */
 	w = sb->write_protect_enabled_eb;
 	if(w != NULL)
@@ -692,29 +686,33 @@ void EDVStatusBarUpdate(edv_status_bar_struct *sb)
 	if(w != NULL)
 	{
 	    if(delete_method == EDV_DELETE_METHOD_PURGE)
-		gtk_widget_show(w);                              
+		gtk_widget_show(w);
 	    else   
 		gtk_widget_hide(w);
 	}
+
+	sb->freeze_count--;
 }
 
 /*
  *	Sets the Status Bar as busy or ready.
  */
 void EDVStatusBarSetBusy(
-	edv_status_bar_struct *sb, gboolean busy  
+	edv_status_bar_struct *sb, const gboolean busy  
 )
 {
 	GdkCursor *cur;
-	GtkWidget *w = (sb != NULL) ? sb->toplevel : NULL;
+	GtkWidget *w;
 	edv_core_struct *core;
+
+	if(sb == NULL)
+	    return;
+
+	w = gtk_widget_get_toplevel(sb->toplevel);
 	if(w == NULL)
 	    return;
 
-	w = gtk_widget_get_toplevel(w);
 	core = sb->core;
-	if((w == NULL) || (core == NULL))
-	    return;
 
 	if(busy)
 	{
@@ -728,7 +726,7 @@ void EDVStatusBarSetBusy(
 	    cur = EDVGetCursor(core, EDV_CURSOR_CODE_BUSY);
 	}
 	else
-	{   
+	{
 	    /* Reduce busy count */
 	    sb->busy_count--;
 	    if(sb->busy_count < 0)
@@ -738,10 +736,10 @@ void EDVStatusBarSetBusy(
 	    if(sb->busy_count > 0)
 		return;
 
-	    cur = NULL;		/* Use default cursor */
+	    cur = NULL;		/* Use the default cursor */
 	}
 
-	/* Update toplevel window's cursor */
+	/* Update the toplevel GdkWindow's GdkCursor */
 	if(w->window != NULL)
 	{
 	    gdk_window_set_cursor(w->window, cur);
@@ -754,11 +752,10 @@ void EDVStatusBarSetBusy(
  */
 void EDVStatusBarMap(edv_status_bar_struct *sb)
 {
-	GtkWidget *w = (sb != NULL) ? sb->toplevel : NULL;
-	if(w == NULL)
+	if(sb == NULL)
 	    return;
 
-	gtk_widget_show(w);
+	gtk_widget_show(sb->toplevel);
 }
 
 /*
@@ -766,11 +763,10 @@ void EDVStatusBarMap(edv_status_bar_struct *sb)
  */
 void EDVStatusBarUnmap(edv_status_bar_struct *sb)
 {
-	GtkWidget *w = (sb != NULL) ? sb->toplevel : NULL;
-	if(w == NULL)
+	if(sb == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(sb->toplevel);
 }
 
 /*
@@ -781,13 +777,11 @@ void EDVStatusBarDelete(edv_status_bar_struct *sb)
 	if(sb == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(sb->progress_bar);
-	GTK_WIDGET_DESTROY(sb->label);
-	GTK_WIDGET_DESTROY(sb->write_protect_enabled_eb);
-	GTK_WIDGET_DESTROY(sb->write_protect_disabled_eb);
-	GTK_WIDGET_DESTROY(sb->delete_method_recycle_eb);
-	GTK_WIDGET_DESTROY(sb->delete_method_purge_eb);
-	GTK_WIDGET_DESTROY(sb->toplevel);
+	sb->freeze_count++;
+
+	gtk_widget_destroy(sb->toplevel);
+
+	sb->freeze_count--;
 
 	g_free(sb);
 }
@@ -797,17 +791,16 @@ void EDVStatusBarDelete(edv_status_bar_struct *sb)
  *	Sets the Status Bar's message.
  */
 void EDVStatusBarMessage(
-	edv_status_bar_struct *sb, const gchar *message,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gchar *msg,
+	const gboolean allow_gtk_iteration
 )
 {
-	GtkWidget *w = (sb != NULL) ? sb->label : NULL;
-	if(w == NULL)
+	if(sb == NULL)
 	    return;
 
 	gtk_label_set_text(
-	    GTK_LABEL(w),
-	    (message != NULL) ? message : ""
+	    GTK_LABEL(sb->label),
+	    (msg != NULL) ? msg : ""
 	);
 
 	/* Flush events? */
@@ -818,21 +811,23 @@ void EDVStatusBarMessage(
 /*
  *	Sets the Status Bar's progress.
  *
- *	If value is between 0.0 to 1.0 then normal finite progress
- *	is performed, otherwise if value is negative then unknown time
- *	progress is performed.
+ *	The v specifies the progress value from 0.0 to 1.0. If v is
+ *	-1.0 then unknown progress is displayed.
  */
 void EDVStatusBarProgress(
-	edv_status_bar_struct *sb, gfloat value,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gfloat v,
+	const gboolean allow_gtk_iteration
 )
 {
+	gfloat value = v;
 	GtkProgress *progress;
-	GtkWidget *w = (sb != NULL) ? sb->progress_bar : NULL;
-	if(w == NULL)
+	GtkProgressBar *progress_bar;
+
+	if(sb == NULL)
 	    return;
 
-	progress = GTK_PROGRESS(w);
+	progress_bar = GTK_PROGRESS_BAR(sb->progress_bar);
+	progress = GTK_PROGRESS(progress_bar);
 
 	/* Do activity? */
 	if(value < 0.0f)
@@ -859,6 +854,7 @@ void EDVStatusBarProgress(
 	}
 	else
 	{
+
 	    /* Clip value to 0.0, 1.0 */
 	    if(value > 1.0f)
 		value = 1.0f;
@@ -887,7 +883,7 @@ void EDVStatusBarProgress(
 	    gtk_progress_set_show_text(
 		progress, (value > 0.0f) ? TRUE : FALSE
 	    );
-	    gtk_progress_bar_update(GTK_PROGRESS_BAR(w), value);
+	    gtk_progress_bar_update(progress_bar, value);
 
 	    /* Record last position */
 	    sb->progress_pos_last = value;
