@@ -13,7 +13,7 @@
 
 
 typedef struct _edv_status_bar_struct	edv_status_bar_struct;
-#define EDV_STATUS_BAR(p)	((edv_status_bar_struct *)(p))
+#define EDV_STATUS_BAR(p)		((edv_status_bar_struct *)(p))
 
 
 /*
@@ -22,8 +22,8 @@ typedef struct _edv_status_bar_struct	edv_status_bar_struct;
 struct _edv_status_bar_struct {
 
 	GtkWidget	*toplevel;		/* GtkFrame */
-	gint		busy_count,
-			freeze_count;
+	gint		freeze_count,
+			busy_count;
 	edv_core_struct	*core;
 
 	GtkWidget	*write_protect_enabled_eb,
@@ -34,6 +34,7 @@ struct _edv_status_bar_struct {
 			*label;		/* GtkLabel */
 
 	gfloat		progress_pos_last;
+
 };
 
 
@@ -42,19 +43,19 @@ extern edv_status_bar_struct *EDVStatusBarNew(
 );
 extern void EDVStatusBarUpdate(edv_status_bar_struct *sb);
 extern void EDVStatusBarSetBusy(
-	edv_status_bar_struct *sb, gboolean busy
+	edv_status_bar_struct *sb, const gboolean busy
 );
 extern void EDVStatusBarMap(edv_status_bar_struct *sb);
 extern void EDVStatusBarUnmap(edv_status_bar_struct *sb);
 extern void EDVStatusBarDelete(edv_status_bar_struct *sb);
 
 extern void EDVStatusBarMessage(
-	edv_status_bar_struct *sb, const gchar *message,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gchar *msg,
+	const gboolean allow_gtk_iteration
 );
 extern void EDVStatusBarProgress(
-	edv_status_bar_struct *sb, gfloat percent,
-	gboolean allow_gtk_iteration
+	edv_status_bar_struct *sb, const gfloat v,
+	const gboolean allow_gtk_iteration
 );
 
 
