@@ -30,11 +30,12 @@
 #include "config.h"
 
 
-typedef struct _edv_scrolled_window_data	edv_scrolled_window_data;
-#define EDV_SCROLLED_WINDOW_DATA(p)		((edv_scrolled_window_data *)(p))
+#define EDV_MENU_BUTTON_MAP_DATA_KEY	"EDVMenuButtonMapData"
 
+typedef struct _EDVScrolledWindowData	EDVScrolledWindowData;
+#define EDV_SCROLLED_WINDOW_DATA(p)	((EDVScrolledWindowData *)(p))
+#define EDV_SCROLLED_WINDOW_DATA_KEY	"EDVScrolledWindowData"
 
-static gchar *G_STRCAT(gchar *s, const gchar *s2);
 
 /* Accelerator Key Matching */
 gint EDVMatchAccelKeyOPID(
@@ -46,13 +47,10 @@ cfg_accelkey_struct *EDVMatchAccelKey(
 	const gint opid 
 );
 
-/* String Formatting */
-gchar *EDVGetObjectSizeStr(edv_core_struct *core, const gulong x);
-
 /* Pixmaps */
 void EDVResizePixmap(
 	GdkPixmap *pixmap, GdkBitmap *mask,
-	gint width, gint height,
+	const gint width, const gint height,
 	GdkPixmap **pixmap_rtn, GdkBitmap **mask_rtn
 );
 gboolean EDVLoadPixmap(
@@ -220,7 +218,7 @@ void EDVCenterWindowToWindowOffset(
 	GtkWidget *w1, GtkWidget *w2, gint x_offset, gint y_offset
 );
 
-/* GtkEntry Setup Utilities */
+/* GtkEditable & GtkEntry Behavior Additions */
 void EDVEntrySetDND(edv_core_struct *core, GtkWidget *w);
 void EDVEntrySetCompletePath(edv_core_struct *core, GtkWidget *w);
 
@@ -239,12 +237,12 @@ static void EDVScrolledWindowDataDelete(gpointer data);
 /* Copy Object Path Utilities */
 void EDVCopyDiskObjectsToDDEPath(
 	edv_core_struct *core,
-	edv_object_struct **list, gint total,
+	GList *objs_list,
 	GtkWidget *owner
 );
 void EDVCopyDiskObjectsToDDEURL(
 	edv_core_struct *core,
-	edv_object_struct **list, gint total,
+	GList *objs_list,
 	GtkWidget *owner
 );
 
@@ -254,16 +252,45 @@ GtkRcStyle *EDVCreateRCStyleFromCfg(const cfg_style_struct *style);
 /* History Utilities */
 void EDVAppendHistory(
 	edv_core_struct *core,
-	edv_history_type type,	/* One of EDV_HISTORY_* */
-	gulong time_start,	/* Time the operation first started */
-	gulong time_end,	/* Time the operation ended */
-	gint status,		/* Result of operation */
-	const gchar *src_path,	/* Source Object/Operation/Value */
-	const gchar *tar_path,	/* Target Object/Operation/Value */
-	const gchar *comments	/* Comments */
+	const edv_history_type type,
+	const gulong time_start,
+	const gulong time_end,
+	const gint status,
+	const gchar *src_path,
+	const gchar *tar_path,
+	const gchar *comments
 );
 
 
+/*
+ *	GtkScrolledWindow Data:
+ */
+struct _EDVScrolledWindowData {
+	gint		button,
+			drag_last_x,
+			drag_last_y;
+	GdkCursor	*translate_cur;
+};
+
+/*
+ *	GtkEditable Popup Menu Data:
+ */
+struct _EDVEditablePopupMenuData {
+
+	GtkWidget	*menu,
+			*editable,
+			*cut_mi,
+			*copy_mi,
+			*paste_mi,
+			*delete_mi,
+			*select_all_mi,
+			*unselect_all_mi;
+
+	gint		freeze_count;
+
+};
+
+
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -275,38 +302,6 @@ void EDVAppendHistory(
 #define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
-#define EDV_MENU_BUTTON_MAP_DATA_KEY	\
-	"edv_menu_button_map_data"
-
-#define EDV_SCROLLED_WINDOW_DATA_KEY	\
-	"EDV_SCROLLED_WINDOW_DATA"
-
-
-struct _edv_scrolled_window_data {
-	gint		button,
-			drag_last_x,
-			drag_last_y;
-	GdkCursor	*translate_cur;
-};
-
-
-static gchar *G_STRCAT(gchar *s, const gchar *s2)
-{
-	if(s != NULL) {
-	    if(s2 != NULL) {
-		gchar *sr = g_strconcat(s, s2, NULL);
-		g_free(s);
-		s = sr;
-	    }
-	} else {
-	    if(s2 != NULL)
-		s = STRDUP(s2);
-	    else
-		s = STRDUP("");
-	}
-	return(s);
-}
-
 
 /*
  *	Returns the Operation ID of the specified Accelerator Key
@@ -401,65 +396,12 @@ cfg_accelkey_struct *EDVMatchAccelKey(
 }
 
 
-/*
- *	Returns a statically allocated string describing the size
- *	of the object specified by i in units defined by the
- *	configuration on the given core structure.
- *
- *	The returned string will be no longer than 256 bytes and
- *	never NULL.
- */
-gchar *EDVGetObjectSizeStr(edv_core_struct *core, const gulong x)
-{
-	gint comma_countdown, slen;
-	gchar ss[80], *ss_ptr;
-	static gchar ts[80], *ts_ptr;
-
-	g_snprintf(ss, sizeof(ss), "%ld", x);
-	slen = STRLEN(ss);
-
-	/* 3 digits or less? (no commas needed) */
-	if(slen <= 3)
-	{
-	    strcpy(ts, ss);
-	    return(ts);
-	}
-
-	ts_ptr = ts;
-	ss_ptr = ss;
-
-	/* Initialize comma counter */
-	comma_countdown = slen % 3;
-	if(comma_countdown <= 0)
-	    comma_countdown = 3;
-
-	/* Iterate through size string until end is reached */
-	while(*ss_ptr != '\0')
-	{
-	    /* Reset comma counter and put in a comma? */
-	    if(comma_countdown <= 0)
-	    {
-		*ts_ptr++ = ',';
-		comma_countdown = 3;
-	    }
-
-	    *ts_ptr++ = *ss_ptr++;
-	    comma_countdown--;
-	}
-
-	/* Null terminate return string */
-	*ts_ptr = '\0';
-
-	return(ts);
-}
-
-
 /*
  *	Resizes the pixmap and mask to the specified new size.
  */
 void EDVResizePixmap(
 	GdkPixmap *pixmap, GdkBitmap *mask,
-	gint width, gint height,
+	const gint width, const gint height,
 	GdkPixmap **pixmap_rtn, GdkBitmap **mask_rtn
 )
 {
@@ -470,7 +412,9 @@ void EDVResizePixmap(
 	if(mask_rtn != NULL)
 	    *mask_rtn = NULL;
 
-	if((pixmap == NULL) || (width <= 0) || (height <= 0))
+	if(pixmap == NULL)
+	    return;
+	if((width <= 0) || (height <= 0))
 	    return;
 
 	gdk_window_get_size(pixmap, &old_width, &old_height);
@@ -482,40 +426,39 @@ void EDVResizePixmap(
 	{
 	    if(pixmap_rtn != NULL)
 	    {
-		if(pixmap != NULL)
-		    gdk_pixmap_ref(pixmap);
+		GDK_PIXMAP_REF(pixmap);
 		*pixmap_rtn = pixmap;
 	    }
 	    if(mask_rtn != NULL)
 	    {
-		if(mask != NULL)
-		    gdk_bitmap_ref(mask);
+		GDK_BITMAP_REF(mask);
 		*mask_rtn = mask;
 	    }
 	    return;
 	}
 
-	/* Resize pixmap */
+	/* Resize the pixmap */
 	if(pixmap_rtn != NULL)
 	{
+	    const gint bpp = 4 * sizeof(guint8);
 	    gint sw, sh, sbpl, tbpl;
 	    guint8 *tdata, *sdata;
 	    GdkGC *gc = GDK_GC_NEW();
 
 	    /* Get source RGBA data from the specified pixmap */
 	    sdata = gdk_get_rgba_image(
-		pixmap,
+		(GdkDrawable *)pixmap,
 		NULL,
 		&sw, &sh, &sbpl
 	    );
 	    if(sdata == NULL)
 	    {
-		GDK_GC_UNREF(gc)
+		GDK_GC_UNREF(gc);
 		return;
 	    }
 
-	    /* Create target RGBA data */
-	    tbpl = width * 4;
+	    /* Create the resized target RGBA data */
+	    tbpl = width * bpp;
 	    tdata = (guint8 *)g_malloc(tbpl * height);
 	    if(tdata == NULL)
 	    {
@@ -524,44 +467,77 @@ void EDVResizePixmap(
 		return;
 	    }
 
-	    /* Copy/resize source RGBA data to the target RGBA data */
+	    /* Copy/resize */
 	    GUIImageBufferResize(
-		4,
+		bpp,
 		sdata, sw, sh, sbpl,
 		tdata, width, height, tbpl,
 		NULL, NULL
 	    );
 
-	    g_free(sdata);
+	    g_free(sdata);	/* Delete the source RGBA data */
 
-	    /* Create target pixmap */
+	    /* Create the resized pixmap */
 	    *pixmap_rtn = GDK_PIXMAP_NEW(width, height);
 	    if(*pixmap_rtn == NULL)
 	    {
 		g_free(tdata);
-		GDK_GC_UNREF(gc)
+		GDK_GC_UNREF(gc);
 		return;
 	    }
 
-	    /* Put resized target RGBA data to the target pixmap */
+	    /* Put the resized RGBA data to the resized pixmap */
 	    gdk_draw_rgb_32_image(
-		*pixmap_rtn, gc,
+		*pixmap_rtn,
+		gc,
 		0, 0,
 		width, height,
 		GDK_RGB_DITHER_NORMAL,
 		tdata, tbpl
 	    );
 
-	    /* Resize mask */
-	    if(mask_rtn != NULL)
-		*mask_rtn = GUICreateBitmapFromDataRGBA(
-		    width, height, tbpl,
-		    tdata, 0x80,
-		    NULL
-		);
+	    g_free(tdata);	/* Delete the resized RGBA data */
 
-	    g_free(tdata);
-	    GDK_GC_UNREF(gc)
+	    /* Resize the mask */
+	    if(mask_rtn != NULL)
+	    {
+		/* Get the source RGBA image data from the mask */
+		sdata = gdk_get_rgba_image(
+		    (GdkDrawable *)mask,
+		    NULL,
+		    &sw, &sh, &sbpl
+		);
+		if(sdata != NULL)
+		{
+		    /* Create the resized target RGBA data */
+		    tbpl = width * bpp;
+		    tdata = (guint8 *)g_malloc(tbpl * height);
+		    if(tdata != NULL)
+		    {
+			/* Copy/resize */
+			GUIImageBufferResize(
+			    bpp,
+			    sdata, sw, sh, sbpl,
+			    tdata, width, height, tbpl,
+			    NULL, NULL
+			);
+
+			/* Create the resized mask */
+			*mask_rtn = GUICreateBitmapFromDataRGBA(
+			    width, height, tbpl,
+			    tdata, 0x80,
+			    NULL
+			);
+
+			g_free(tdata);	/* Delete the resized RGBA data */
+		    }
+
+		    g_free(sdata);	/* Delete the source RGBA data */
+		}
+
+	    }
+
+	    GDK_GC_UNREF(gc);
 	}
 }
 
@@ -703,18 +679,18 @@ GdkPixmap *EDVNewProgressPixmap(
 	    return(NULL);
 
 	/* Draw the background */
-        gdk_draw_rectangle(
-            pixmap, style->base_gc[GTK_STATE_NORMAL],
-            TRUE,
-            0, 0, _width, _height
-        );
+	gdk_draw_rectangle(
+	    pixmap, style->base_gc[GTK_STATE_NORMAL],
+	    TRUE,
+	    0, 0, _width, _height
+	);
 
 	/* Draw the outline */
 	gdk_draw_rectangle(
-            pixmap, style->fg_gc[GTK_STATE_NORMAL],
-            FALSE,
-            0, 0, _width - 1, _height - 1
-        );
+	    pixmap, style->fg_gc[GTK_STATE_NORMAL],
+	    FALSE,
+	    0, 0, _width - 1, _height - 1
+	);
 
 	/* Calculate the geometry of the trough */
 	trough_rect.x = 2;
@@ -727,7 +703,7 @@ GdkPixmap *EDVNewProgressPixmap(
 	/* Calculate the geometry of the bar */
 	switch(orientation)
 	{
-          case GTK_ORIENTATION_HORIZONTAL:
+	  case GTK_ORIENTATION_HORIZONTAL:
 	    bar_rect.width = (coeff >= 0.0f) ?
 		(gint)(trough_rect.width * MIN(coeff, 1.0f)) :
 		trough_rect.width;
@@ -742,7 +718,7 @@ GdkPixmap *EDVNewProgressPixmap(
 		bar_rect.x = trough_rect.x;
 		bar_rect.y = trough_rect.y;
 	    }
-            break;
+	    break;
 	  case GTK_ORIENTATION_VERTICAL:
 	    bar_rect.width = trough_rect.width;
 	    bar_rect.height = (coeff >= 0.0f) ?
@@ -784,7 +760,7 @@ GdkPixmap *EDVNewProgressPixmap(
 		"%.0f%%",
 		coeff * 100.0f
 	    );
-            gdk_string_bounds(font, s, &b);
+	    gdk_string_bounds(font, s, &b);
 	    x = 0;
 	    y = 0;
 	    switch(orientation)
@@ -800,20 +776,20 @@ GdkPixmap *EDVNewProgressPixmap(
 	    }
 	    gdk_draw_string(
 		pixmap, font, gc,
-                x - b.lbearing,  
-                y + font->ascent,
+		x - b.lbearing,
+		y + font->ascent,
 		s
 	    );
 
 	    gc = style->text_gc[GTK_STATE_SELECTED];
-            gdk_gc_set_clip_rectangle(gc, &bar_rect);
-            gdk_draw_string(
-                pixmap, font, gc,
-                x - b.lbearing,   
-                y + font->ascent,
-                s
-            );   
-            gdk_gc_set_clip_rectangle(gc, NULL);
+	    gdk_gc_set_clip_rectangle(gc, &bar_rect);
+	    gdk_draw_string(
+		pixmap, font, gc,
+		x - b.lbearing,
+		y + font->ascent,
+		s
+	    );   
+	    gdk_gc_set_clip_rectangle(gc, NULL);
 	    g_free(s);
 	}
 
@@ -2441,7 +2417,10 @@ void EDVPlaySoundBeep(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-	        gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 	        EDVSystem(cmd);
 	        g_free(cmd);
 	    }
@@ -2466,7 +2445,10 @@ void EDVPlaySoundInfo(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-		gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 		EDVSystem(cmd);
 		g_free(cmd);
 	    }
@@ -2491,7 +2473,10 @@ void EDVPlaySoundQuestion(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-		gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 		EDVSystem(cmd);
 		g_free(cmd);
 	    }
@@ -2516,7 +2501,10 @@ void EDVPlaySoundWarning(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-		gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 		EDVSystem(cmd);
 		g_free(cmd);
 	    }
@@ -2541,7 +2529,10 @@ void EDVPlaySoundError(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-		gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 		EDVSystem(cmd);
 		g_free(cmd);
 	    }
@@ -2566,7 +2557,10 @@ void EDVPlaySoundCompleted(edv_core_struct *core)
 	    );
 	    if(!STRISEMPTY(s))
 	    {
-		gchar *cmd = g_strdup_printf("%s &", s);
+	        gchar *cmd = g_strconcat(
+		    s,
+		    NULL
+		);
 		EDVSystem(cmd);
 		g_free(cmd);
 	    }
@@ -2747,15 +2741,16 @@ void EDVCenterWindowToWindowOffset(
 	);
 }
 
+
 /*
  *	Sets up the GtkEntry w to allow it to receive drag and drop.
  */
 void EDVEntrySetDND(edv_core_struct *core, GtkWidget *w)
 {
 	const GtkTargetEntry dnd_tar_types[] = {
-{"text/plain",			0,	EDV_DND_INFO_TEXT_PLAIN},
-{"text/uri-list",		0,	EDV_DND_INFO_TEXT_URI_LIST},
-{"STRING",			0,	EDV_DND_INFO_STRING}
+{GUI_TARGET_NAME_TEXT_PLAIN,	0,	EDV_DND_INFO_TEXT_PLAIN},
+{GUI_TARGET_NAME_TEXT_URI_LIST,	0,	EDV_DND_INFO_TEXT_URI_LIST},
+{GUI_TARGET_NAME_STRING,	0,	EDV_DND_INFO_STRING}
 	};
 
 	if((core == NULL) || (w == NULL))
@@ -2813,8 +2808,8 @@ GtkWidget *EDVScrolledWindowNew(
 	GtkWidget **vbox_rtn
 )
 {
-	edv_scrolled_window_data *d = EDV_SCROLLED_WINDOW_DATA(g_malloc0(
-	    sizeof(edv_scrolled_window_data)
+	EDVScrolledWindowData *d = EDV_SCROLLED_WINDOW_DATA(g_malloc0(
+	    sizeof(EDVScrolledWindowData)
 	));
 	GtkWidget *parent, *w = gtk_scrolled_window_new(NULL, NULL);
 	GtkScrolledWindow *sw = GTK_SCROLLED_WINDOW(w);
@@ -2883,7 +2878,7 @@ static gint EDVScrolledWindowEventCB(
 )
 {
 	gint status = FALSE;
-	edv_scrolled_window_data *d;
+	EDVScrolledWindowData *d;
 	gboolean press;
 	GdkEventFocus *focus;
 	GdkEventKey *key;
@@ -3132,7 +3127,7 @@ static gint EDVScrolledWindowEventCB(
 
 static void EDVScrolledWindowDataDelete(gpointer data)
 {
-	edv_scrolled_window_data *d = EDV_SCROLLED_WINDOW_DATA(data);
+	EDVScrolledWindowData *d = EDV_SCROLLED_WINDOW_DATA(data);
 	if(d == NULL)
 	    return;
 
@@ -3142,91 +3137,124 @@ static void EDVScrolledWindowDataDelete(gpointer data)
 
 
 /*
- *	Coppies the paths of the given list of disk objects to the
- *	DDE as a space separated list of absolute paths.
+ *	Coppies the object paths to the DDE as a space-separated
+ *	list of absolute paths.
+ *
+ *	The objs_list specifies the list of edv_object_struct *
+ *	objects.
+ *
+ *	The owner specifies the owner GtkWidget.
  */
 void EDVCopyDiskObjectsToDDEPath(
 	edv_core_struct *core,
-	edv_object_struct **list, gint total,
+	GList *objs_list,
 	GtkWidget *owner
 )
 {
-	gint i;
-	gchar *buf = NULL;
+	gchar *s, *s2;
+	GList *glist;
 	edv_object_struct *obj;
 
-	if((core == NULL) || (list == NULL) || (total <= 0))
+	if((core == NULL) || (objs_list == NULL))
 	    return;
 
-	for(i = 0; i < total; i++)
+	s = NULL;
+	for(glist = objs_list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
 	{
-	    obj = list[i];
+	    obj = EDV_OBJECT(glist->data);
 	    if(obj == NULL)
 		continue;
 
 	    if(obj->full_path == NULL)
 		continue;
 
-	    buf = G_STRCAT(buf, obj->full_path);
-	    if((i + 1) < total)
-		buf = G_STRCAT(buf, " ");
+	    if(s != NULL)
+		s2 = g_strconcat(
+		    s,
+		    " ",
+		    obj->full_path,
+		    NULL
+		);
+	    else
+		s2 = STRDUP(obj->full_path);
+	    g_free(s);
+	    s = s2;
 	}
 
-	if(buf != NULL)
-	{
-	    GUIDDESetString(
+	if(s != NULL)
+	    GUIDDESetDirect(
 		owner,			/* Owner */
 		GDK_SELECTION_PRIMARY,	/* Selection */
 		GDK_CURRENT_TIME,	/* Time */
-		buf			/* Data */
+		gdk_atom_intern(GUI_TARGET_NAME_STRING, FALSE),
+		s, STRLEN(s)		/* Data */
 	    );
-	    g_free(buf);
-	}
 }
 
 /*
- *	Coppies the paths of the given list of disk objects to the
- *	DDE as a space separated list of urls (without the localhost
- *	address specified).
+ *	Coppies the object paths to the DDE as a space-separated
+ *	list of URLs (without the localhost address specified).
+ *
+ *	The objs_list specifies the list of edv_object_struct *
+ *	objects.
+ *
+ *	The owner specifies the owner GtkWidget.
  */
 void EDVCopyDiskObjectsToDDEURL(
 	edv_core_struct *core,
-	edv_object_struct **list, gint total,
+	GList *objs_list,
 	GtkWidget *owner
 )
 {
-	gint i;
-	gchar *buf = NULL;
+	gchar *s, *s2;
+	GList *glist;
 	edv_object_struct *obj;
 
-	if((core == NULL) || (list == NULL) || (total <= 0))
+	if((core == NULL) || (objs_list == NULL))
 	    return;
 
-	for(i = 0; i < total; i++)
+	s = NULL;
+	for(glist = objs_list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
 	{
-	    obj = list[i];
+	    obj = EDV_OBJECT(glist->data);
 	    if(obj == NULL)
 		continue;
 
 	    if(obj->full_path == NULL)
 		continue;
 
-	    buf = G_STRCAT(buf, "file://");
-	    buf = G_STRCAT(buf, obj->full_path);
-	    if((i + 1) < total)
-		buf = G_STRCAT(buf, " ");
+	    if(s != NULL)
+		s2 = g_strconcat(
+		    s,
+		    " ",
+		    "file://",
+		    obj->full_path,
+		    NULL
+		);
+	    else
+		s2 = g_strconcat(
+		    "file://",
+		    obj->full_path,
+		    NULL
+		);
+	    g_free(s);
+	    s = s2;
 	}
 
-	if(buf != NULL)
-	{
-	    GUIDDESetString(
+	if(s != NULL)
+	    GUIDDESetDirect(
 		owner,			/* Owner */
 		GDK_SELECTION_PRIMARY,	/* Selection */
 		GDK_CURRENT_TIME,	/* Time */
-		buf			/* Data */
+		gdk_atom_intern(GUI_TARGET_NAME_STRING, FALSE),
+		s, STRLEN(s)		/* Data */
 	    );
-	    g_free(buf);
-	}
 }
 
 
@@ -3303,32 +3331,57 @@ GtkRcStyle *EDVCreateRCStyleFromCfg(const cfg_style_struct *style)
 }
 
 /*
- *	Appends a new operation history event item to the list on the
- *	given core structure.
+ *	Appends a history event to the history list.
+ *
+ *	The core specifies the core containing the history list to
+ *	append to.
+ *
+ *	The type specifies the event type.
+ *
+ *	The time_start specifies the start time in seconds since EPOCH.
+ *
+ *	The time_end specifies the end time in seconds since EPOCH.
+ *
+ *	The status specifies the resulting status of the history event,
+ *	valid values are:
+ *
+ *	0	Success.
+ *	-1	General error.
+ *	-2	Invalid value.
+ *	-3	Systems error, out of memory, or out of disk space.
+ *	-4	User aborted.
+ *	-5	User responded with "no" to a query.
+ *	-6	Operation already in progress or would cause reentry.
+ *
+ *	The src_path specifies the source object/operation/value.
+ *
+ *	The tar_path specifies the target object/operation/value.
+ *
+ *	The comments specifies any comments or error message pertaining
+ *	to the history event.
  */
 void EDVAppendHistory(
 	edv_core_struct *core,
-	edv_history_type type,	/* One of EDV_HISTORY_* */
-	gulong time_start,	/* Time the operation first started */
-	gulong time_end,	/* Time the operation ended */
-	gint status,		/* Result of operation */
-	const gchar *src_path,	/* Source Object/Operation/Value */
-	const gchar *tar_path,	/* Target Object/Operation/Value */
-	const gchar *comments	/* Comments */
+	const edv_history_type type,
+	const gulong time_start,
+	const gulong time_end,
+	const gint status,
+	const gchar *src_path,
+	const gchar *tar_path,
+	const gchar *comments
 )
 {
 	edv_history_struct *h;
 
-
 	if(core == NULL)
 	    return;
 
-	/* Create new history event */
+	/* Create a new history event */
 	h = EDVHistoryNew();
 	if(h == NULL)
 	    return;
 
-	/* Set values */
+	/* Set the values for the new history event */
 	h->type = type;
 	h->time_start = time_start;
 	h->time_end = time_end;
@@ -3337,17 +3390,18 @@ void EDVAppendHistory(
 	h->tar_path = STRDUP(tar_path);
 	h->comments = STRDUP(comments);
 
-	/* Append to history file */
+	/* Append this history event to the list (which is on file) */
 	EDVHistoryAppendToFile(
 	    CFGItemListGetValueS(
-		core->cfg_list, EDV_CFG_PARM_FILE_HISTORY
+		core->cfg_list,
+		EDV_CFG_PARM_FILE_HISTORY
 	    ),
 	    h
 	);
 
-	/* Emit history event added signal */
+	/* Notify about this history event being appended */
 	EDVHistoryAddedEmit(core, -1, h);
 
-	/* Delete history event */
+	/* Delete the history event */
 	EDVHistoryDelete(h);
 }
