@@ -213,14 +213,14 @@ edv_findwin_operation EDVFindWinCurrentOperation(edv_find_win_struct *fw)
 	if(fw == NULL)
 	    return(EDV_FIND_WIN_FIND_OBJECT_NAME);
 
-        pulist = PUListBoxGetPUList(fw->find_op_pulistbox);
-        if(pulist == NULL)
-            return(EDV_FIND_WIN_FIND_OBJECT_NAME);
+	pulist = PUListBoxGetPUList(fw->find_op_pulistbox);
+	if(pulist == NULL)
+	    return(EDV_FIND_WIN_FIND_OBJECT_NAME);
 
-        return((edv_findwin_operation)PUListGetItemData(
-            pulist,
-            PUListGetSelectedLast(pulist)
-        ));
+	return((edv_findwin_operation)PUListGetItemData(
+	    pulist,
+	    PUListGetSelectedLast(pulist)
+	));
 }
 
 /*
@@ -265,39 +265,38 @@ void EDVFindWinListResetColumns(
 	edv_find_win_struct *fw, const edv_findwin_operation operation
 )
 {
-	gint i, columns;
+	gint i, ncolumns;
 	gchar **title;
 	gint *justify;
 	GtkRcStyle *lists_rcstyle;
+	GtkWidget *w;
+	GtkCList *clist;
 	edv_core_struct *core;
-	GtkCList *clist = (GtkCList *)((fw != NULL) ?
-	    fw->results_clist : NULL
-	);
-	if(clist == NULL)
+
+	if(fw == NULL)
 	    return;
 
-	columns = clist->columns;
+	w = fw->results_clist;
+	clist = GTK_CLIST(w);
+	ncolumns = MAX(clist->columns, 1);
 	core = fw->core;
-	if((columns <= 0) || (core == NULL))
-	    return;
-
 	lists_rcstyle = core->lists_rcstyle;
 
 	/* Allocate column property values */
-	title = (gchar **)g_malloc0(columns * sizeof(gchar *));
-	justify = (gint *)g_malloc0(columns * sizeof(gint));
+	title = (gchar **)g_malloc0(ncolumns * sizeof(gchar *));
+	justify = (gint *)g_malloc0(ncolumns * sizeof(gint));
 
 
 	/* Begin resetting columns */
 
 	gtk_clist_freeze(clist);
 
-	/* Update clist column settings */
+	/* Update the column settings */
 	gtk_clist_column_titles_show(clist);
-	gtk_clist_column_titles_passive(clist); 
+	gtk_clist_column_titles_active(clist);
 
-	/* Iterate through columns */
-	for(i = 0; i < columns; i++)
+	/* Set up each column */
+	for(i = 0; i < ncolumns; i++)
 	{
 	    /* Set column attributes by the find operation code */
 	    switch(operation)
@@ -357,7 +356,7 @@ void EDVFindWinListResetColumns(
 	    /* Is this column's properties defined? */
 	    if(title[i] != NULL)
 	    {
-		/* Show column and set column properties */
+		/* Show this column and set its attributes */
 		gtk_clist_set_column_visibility(
 		    clist, i, TRUE
 		);
@@ -373,7 +372,7 @@ void EDVFindWinListResetColumns(
 	    }
 	    else
 	    {
-		/* Hide column */
+		/* Hide this column */
 		gtk_clist_set_column_visibility(
 		    clist, i, FALSE
 		);
@@ -382,11 +381,9 @@ void EDVFindWinListResetColumns(
 
 	gtk_clist_thaw(clist);
 
-	/* Set RC style */
+	/* Set the RC style */
 	if(lists_rcstyle != NULL)
-	    gtk_widget_modify_style_recursive(
-		GTK_WIDGET(clist), lists_rcstyle
-	    );
+	    gtk_widget_modify_style_recursive(w, lists_rcstyle);
 
 	g_free(title);
 	g_free(justify);
@@ -402,11 +399,11 @@ void EDVFindWinListAppend(
 	const gchar *excerpt, const gint line_index
 )
 {
-	edv_findwin_operation operation;
-	gint columns;
-	edv_date_relativity relativity;
+	gint ncolumns;
 	const gchar *format;
 	const cfg_item_struct *cfg_list;
+	edv_date_relativity relativity;
+	edv_findwin_operation operation;
 	edv_core_struct *core;
 	GtkCList *clist = (GtkCList *)((fw != NULL) ?
 	    fw->results_clist : NULL
@@ -414,10 +411,8 @@ void EDVFindWinListAppend(
 	if((clist == NULL) || STRISEMPTY(path))
 	    return;
 
-	columns = clist->columns;
+	ncolumns = MAX(clist->columns, 1);
 	core = fw->core;
-	if((columns <= 0) || (core == NULL))
-	    return;
 
 	/* Get current operation */
 	operation = EDVFindWinCurrentOperation(fw);
@@ -433,8 +428,8 @@ void EDVFindWinListAppend(
 	    gint i, row;
 
 	    /* Allocate row cell values */
-	    gchar **strv = (gchar **)g_malloc(columns * sizeof(gchar *));
-	    for(i = 0; i < columns; i++)
+	    gchar **strv = (gchar **)g_malloc(ncolumns * sizeof(gchar *));
+	    for(i = 0; i < ncolumns; i++)
 		strv[i] = "";
 
 	    gtk_clist_freeze(clist);
@@ -452,9 +447,12 @@ void EDVFindWinListAppend(
 		{
 		  case EDV_FIND_WIN_ROLE_DISK_OBJECT:
 		    obj = EDVObjectNew();
-		    EDVObjectSetPath(obj, path);
-		    EDVObjectSetStat(obj, lstat_buf);
-		    EDVObjectUpdateLinkFlags(obj);
+		    if(obj != NULL)
+		    {
+			EDVObjectSetPath(obj, path);
+			EDVObjectSetStat(obj, lstat_buf);
+			EDVObjectUpdateLinkFlags(obj);
+		    }
 		    break;
 
 		  case EDV_FIND_WIN_ROLE_RECYCLED_OBJECT:
@@ -465,31 +463,32 @@ void EDVFindWinListAppend(
 			EDV_GET_S(EDV_CFG_PARM_FILE_RECYCLE_BIN_INDEX),
 			index
 		    );
-		    /* Set recycled object stats to object */
-		    g_free(obj->name);
-		    obj->name = STRDUP(path);
-		    g_free(obj->full_path);
-		    obj->full_path = STRDUP(
-			PrefixPaths(recobj->original_path, recobj->name)
-		    );
-		    obj->type = recobj->type;
-		    obj->permissions = recobj->permissions;
-		    obj->access_time = recobj->access_time;
-		    obj->modify_time = recobj->modify_time;
-		    obj->change_time = recobj->change_time;
-		    obj->owner_id = recobj->owner_id;
-		    obj->group_id = recobj->group_id;
-		    obj->size = recobj->size;
-		    /* Delete recycled object stats */
+		    if((obj != NULL) && (recobj != NULL))
+		    {
+			obj->name = STRDUP(path);
+			obj->full_path = STRDUP(PrefixPaths(
+			    recobj->original_path, recobj->name
+			));
+			obj->type = recobj->type;
+			obj->permissions = recobj->permissions;
+			obj->access_time = recobj->access_time;
+			obj->modify_time = recobj->modify_time;
+			obj->change_time = recobj->change_time;
+			obj->owner_id = recobj->owner_id;
+			obj->group_id = recobj->group_id;
+			obj->size = recobj->size;
+		    }
 		    EDVRecycledObjectDelete(recobj);
 		    break;
 
 		  case EDV_FIND_WIN_ROLE_ARCHIVE_OBJECT:
 		    obj = EDVObjectNew();
-		    g_free(obj->name);
-		    obj->name = STRDUP(g_basename(path));
-		    g_free(obj->full_path);
-		    obj->full_path = STRDUP(path);
+		    if(obj != NULL)
+		    {
+			obj->name = STRDUP(g_basename(path));
+			obj->full_path = STRDUP(path);
+			EDVObjectSetStat(obj, lstat_buf);
+		    }
 		    break;
 		}
 		/* Object created? */
@@ -532,7 +531,7 @@ void EDVFindWinListAppend(
 		     * find operation)
 		     */
 		    i = 0;
-		    if(i < columns)
+		    if(i < ncolumns)
 		    {
 			if(pixmap != NULL)
 			    gtk_clist_set_pixtext(
@@ -553,7 +552,7 @@ void EDVFindWinListAppend(
 		      case EDV_FIND_WIN_FIND_OBJECT_NAME:
 			/* Last modified */
 			i = 1;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    const gulong last_modified = obj->modify_time;
 			    const gchar *s = (last_modified > 0l) ?
@@ -567,7 +566,7 @@ void EDVFindWinListAppend(
 			}
 			/* Location */
 			i = 2;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    gchar *location = (obj->full_path != NULL) ?
 				g_dirname(obj->full_path) : NULL;
@@ -575,46 +574,50 @@ void EDVFindWinListAppend(
 			    {
 				struct stat lstat_buf;
 				edv_object_struct *loc_obj = EDVObjectNew();
-				EDVObjectSetPath(loc_obj, location);
-				if(lstat(location, &lstat_buf))
+				if(loc_obj != NULL)
 				{
-				    loc_obj->type = EDV_OBJECT_TYPE_DIRECTORY;
-				}
-				else
-				{
-				    EDVObjectSetStat(loc_obj, &lstat_buf);
-				}
-				EDVMatchObjectIcon(
-				    core->device, core->total_devices,
-				    core->mimetype, core->total_mimetypes,
-				    loc_obj->type, loc_obj->full_path,
-				    EDV_OBJECT_IS_LINK_VALID(loc_obj),
-				    loc_obj->permissions,
-				    0,		/* Small icons */
-				    &pixmap, &mask,
-				    NULL, NULL,
-				    NULL, NULL,
-				    &pixmap_hid, &mask_hid
-				);
-				/* Hidden? */
-			        if(EDVIsObjectNameHidden(loc_obj->name))
-				{
-				    pixmap = pixmap_hid;
-				    mask = mask_hid;
-				}
-				if(pixmap != NULL)
-				    gtk_clist_set_pixtext(
-					clist, row, i,
-					location,
-					EDV_LIST_PIXMAP_TEXT_SPACING,
-					pixmap, mask
+				    EDVObjectSetPath(loc_obj, location);
+				    if(lstat((const char *)location, &lstat_buf))
+				    {
+					loc_obj->type = EDV_OBJECT_TYPE_DIRECTORY;
+				    }
+				    else
+				    {
+					EDVObjectSetStat(loc_obj, &lstat_buf);
+				    }
+				    EDVMatchObjectIcon(
+					core->device, core->total_devices,
+					core->mimetype, core->total_mimetypes,
+					loc_obj->type,
+					loc_obj->full_path,
+					EDV_OBJECT_IS_LINK_VALID(loc_obj),
+					loc_obj->permissions,
+					0,          /* Small icons */
+					&pixmap, &mask,
+					NULL, NULL,
+					NULL, NULL,
+					&pixmap_hid, &mask_hid
 				    );
-				else
-				    gtk_clist_set_text(
-					clist, row, i,
-					location
-				    );
-				EDVObjectDelete(loc_obj);
+				    /* Hidden? */
+				    if(EDVIsObjectNameHidden(loc_obj->name))
+				    {
+					pixmap = pixmap_hid;
+					mask = mask_hid;
+				    }
+				    if(pixmap != NULL)
+					gtk_clist_set_pixtext(
+					    clist, row, i,
+					    location,
+					    EDV_LIST_PIXMAP_TEXT_SPACING,
+					    pixmap, mask
+					);
+				    else
+					gtk_clist_set_text(
+					    clist, row, i,
+					    location
+					);
+				    EDVObjectDelete(loc_obj);
+				}
 			    }
 			    g_free(location);
 			}
@@ -623,7 +626,7 @@ void EDVFindWinListAppend(
 		      case EDV_FIND_WIN_FIND_OBJECT_CONTENT:
 			/* Line */
 			i = 1;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    gchar *s;
 			    if(line_index > -1)
@@ -639,7 +642,7 @@ void EDVFindWinListAppend(
 			}
 			/* Excerpt */
 			i = 2;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    gtk_clist_set_text(
 				clist, row, i,
@@ -648,7 +651,7 @@ void EDVFindWinListAppend(
 			}
 			/* Last modified */
 			i = 3;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    const gulong last_modified = obj->modify_time;
 			    const gchar *s = (last_modified > 0l) ?
@@ -662,7 +665,7 @@ void EDVFindWinListAppend(
 			}
 			/* Location */
 			i = 4;
-			if(i < columns)
+			if(i < ncolumns)
 			{
 			    gchar *location = (obj->full_path != NULL) ?
 				g_dirname(obj->full_path) : NULL;
@@ -670,58 +673,62 @@ void EDVFindWinListAppend(
 			    {
 				struct stat lstat_buf;
 				edv_object_struct *loc_obj = EDVObjectNew();
-				EDVObjectSetPath(loc_obj, location);
-				if(lstat(location, &lstat_buf))
+				if(loc_obj != NULL)
 				{
-				    loc_obj->type = EDV_OBJECT_TYPE_DIRECTORY;
-				}
-				else
-				{
-				    EDVObjectSetStat(loc_obj, &lstat_buf);
-				}
-				EDVMatchObjectIcon(
-				    core->device, core->total_devices,
-				    core->mimetype, core->total_mimetypes,
-				    loc_obj->type, loc_obj->full_path,
-				    EDV_OBJECT_IS_LINK_VALID(loc_obj),
-				    loc_obj->permissions,
-				    0,          /* Small icons */
-				    &pixmap, &mask,
-				    NULL, NULL,
-				    NULL, NULL,
-				    &pixmap_hid, &mask_hid
-				);
-				/* Hidden? */
-				if(EDVIsObjectNameHidden(loc_obj->name))
-				{
-				    pixmap = pixmap_hid;
-				    mask = mask_hid;
-				}
-				if(pixmap != NULL)
-				    gtk_clist_set_pixtext(
-					clist, row, i,
-					location,
-					EDV_LIST_PIXMAP_TEXT_SPACING,
-					pixmap, mask
+				    EDVObjectSetPath(loc_obj, location);
+				    if(lstat((const char *)location, &lstat_buf))
+				    {
+					loc_obj->type = EDV_OBJECT_TYPE_DIRECTORY;
+				    }
+				    else
+				    {
+					EDVObjectSetStat(loc_obj, &lstat_buf);
+				    }
+				    EDVMatchObjectIcon(
+					core->device, core->total_devices,
+					core->mimetype, core->total_mimetypes,
+					loc_obj->type,
+					loc_obj->full_path,
+					EDV_OBJECT_IS_LINK_VALID(loc_obj),
+					loc_obj->permissions,
+					0,          /* Small icons */
+					&pixmap, &mask,
+					NULL, NULL,
+					NULL, NULL,
+					&pixmap_hid, &mask_hid
 				    );
-				else
-				    gtk_clist_set_text(
-					clist, row, i,
-					location
-				    );
-				EDVObjectDelete(loc_obj);
+				    /* Hidden? */
+				    if(EDVIsObjectNameHidden(loc_obj->name))
+				    {
+					pixmap = pixmap_hid;
+					mask = mask_hid;
+				    }
+				    if(pixmap != NULL)
+					gtk_clist_set_pixtext(
+					    clist, row, i,
+					    location,
+					    EDV_LIST_PIXMAP_TEXT_SPACING,
+					    pixmap, mask
+					);
+				    else
+					gtk_clist_set_text(
+					    clist, row, i,
+					    location
+					);
+				    EDVObjectDelete(loc_obj);
+				}
 			    }
 			    g_free(location);
 			}
 			break;
 		    }
 
-		    /* Set row data as the object */
+		    /* Set the new row's data as the object */
 		    gtk_clist_set_row_data_full(
 			clist, row,
 			obj, EDVFindWinListItemDestroyCB
 		    );
-		    obj = NULL;
+/*		    obj = NULL; */
 		}
 	    }
 
@@ -738,12 +745,13 @@ void EDVFindWinListAppend(
  */
 void EDVFindWinListClear(edv_find_win_struct *fw)
 {
-	GtkCList *clist = (GtkCList *)((fw != NULL) ?
-	    fw->results_clist : NULL
-	);
-	if(clist == NULL)
+	GtkCList *clist;
+
+	if(fw == NULL)
 	    return;
 
+	clist = GTK_CLIST(fw->results_clist);
+
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 	gtk_clist_thaw(clist);
@@ -761,14 +769,10 @@ void EDVFindWinSyncConfiguration(edv_find_win_struct *fw)
 	cfg_item_struct *cfg_list;
 	edv_core_struct *core;
 
-
 	if(fw == NULL)
 	    return;
 
 	core = fw->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Get size of toplevel */
@@ -845,26 +849,24 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 	edv_findwin_operation find_op;
 	const gchar *search_string;
 	gboolean case_sensitive, recursive;
-	edv_status_bar_struct *status_bar;
 	GList *glist;
+	GdkWindow *window;
 	GtkRcStyle *standard_rcstyle, *lists_rcstyle;
-	GtkWidget *w, *parent, *parent2, *parent3, *parent4, *parent5;
+	GtkAccelGroup *accelgrp;
+	GtkWidget	*w,
+			*parent, *parent2, *parent3, *parent4, *parent5,
+			*toplevel;
 	gpointer combo_rtn;
 	GtkCombo *combo;
 	GtkCList *clist;
-	GdkWindow *window;
-	GtkAccelGroup *accelgrp;
-        pulist_struct *pulist;
-        pulistbox_struct *pulistbox;
+	pulist_struct *pulist;
+	pulistbox_struct *pulistbox;
 	const cfg_item_struct *cfg_list;
-	edv_find_win_struct *fw = EDV_FIND_WIN(
-	    g_malloc0(sizeof(edv_find_win_struct))
-	);
-	if((fw == NULL) || (core == NULL))
-	{
-	    g_free(fw);
+	edv_status_bar_struct *status_bar;
+	edv_find_win_struct *fw;
+
+	if(core == NULL)
 	    return(NULL);
-	}
 
 	cfg_list = core->cfg_list;
 
@@ -882,12 +884,19 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 	case_sensitive = EDV_GET_B(EDV_CFG_PARM_FINDWIN_CASE_SENSITIVE);
 	recursive = EDV_GET_B(EDV_CFG_PARM_FINDWIN_RECURSIVE);
 
+
+	/* Create the Find Window */
+	fw = EDV_FIND_WIN(g_malloc0(sizeof(edv_find_win_struct)));
+	if(fw == NULL)
+	    return(NULL);
+
+	fw->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	fw->accelgrp = accelgrp = gtk_accel_group_new();
 	fw->processing = FALSE;
+	fw->freeze_count = 0;
 	fw->busy_count = 0;
-	fw->core = core;
-
 	fw->stop_count = 0;
+	fw->core = core;
 
 	fw->status_bar_map_state = status_bar_map_state;
 
@@ -900,20 +909,16 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 	fw->find_op = EDV_FIND_WIN_FIND_OBJECT_NAME;
 	fw->last_write_protect_state = -1;
 
+	fw->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	fw->toplevel = w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_uposition(w, toplevel_x, toplevel_y);
 	gtk_widget_set_usize(w, toplevel_width, toplevel_height);
 	gtk_window_set_wmclass(
 	    GTK_WINDOW(w), "find", PROG_NAME
 	);
-	gtk_window_set_policy(
-	    GTK_WINDOW(w),
-	    TRUE, TRUE, TRUE
-	);
 	gtk_window_set_title(GTK_WINDOW(w), "Find Objects");
 	gtk_widget_realize(w);
 	window = w->window;
@@ -980,12 +985,12 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 	parent4 = w;
 
 	/* GtkHBox for the find operation */
-        w = gtk_hbox_new(FALSE, border_minor);
-        gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
-        gtk_widget_show(w);
-        parent5 = w;
+	w = gtk_hbox_new(FALSE, border_minor);
+	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
+	parent5 = w;
 
-        w = gtk_label_new(
+	w = gtk_label_new(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Buscar"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -1005,60 +1010,62 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 #endif
 	    ":"
 	);
-        gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
-        gtk_widget_show(w);
+	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
+	gtk_widget_show(w);
 
-        fw->find_op_pulistbox = pulistbox = PUListBoxNew(
-            parent5,
-            150, -1
-        );
-        pulist = PUListBoxGetPUList(pulistbox);
-        PUListBoxMap(pulistbox);
+	fw->find_op_pulistbox = pulistbox = PUListBoxNew(
+	    parent5,
+	    150, -1
+	);
+	pulist = PUListBoxGetPUList(pulistbox);
+        if(core->run_flags & EDV_RUN_SAFE_MODE)
+            PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
 #if defined(PROG_LANGUAGE_SPANISH)
 	i = PUListAddItem(pulist, "Por Nombre de archivo");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Por Contenido");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_FRENCH)
 	i = PUListAddItem(pulist, "Nom D'Objet");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Contenu D'Objet");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_GERMAN)
 	i = PUListAddItem(pulist, "Wenden Sie Gegen Namen Ein");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Wenden Sie Gegen Inhalt Ein");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_ITALIAN)
 	i = PUListAddItem(pulist, "Il Nome Di Oggetto");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Il Contenuto Di Oggetto");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_DUTCH)
 	i = PUListAddItem(pulist, "Heb Bezwaar Naam");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Heb Bezwaar Inhoud");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_PORTUGUESE)
 	i = PUListAddItem(pulist, "O Nome De Objeto");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "O Contedo De Objeto");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #elif defined(PROG_LANGUAGE_NORWEGIAN)
 	i = PUListAddItem(pulist, "Protester Navn");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Protester Innhold");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #else
 	i = PUListAddItem(pulist, "Object Name");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_NAME);
 	i = PUListAddItem(pulist, "Object Content");
-        PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
+	PUListSetItemData(pulist, i, (gpointer)EDV_FIND_WIN_FIND_OBJECT_CONTENT);
 #endif
-        PUListBoxSetLinesVisible(pulistbox, i + 1);
+	PUListBoxSetLinesVisible(pulistbox, i + 1);
+	PUListBoxMap(pulistbox);
 
 
-	/* Search string combo */
+	/* Search string GtkCombo */
 	glist = NULL;
 	w = GUIComboCreate(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1097,16 +1104,17 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 
 	w = combo->entry;
 	EDVEntrySetDND(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
 	if(search_string != NULL)
 	    gtk_entry_set_text(GTK_ENTRY(w), search_string);
 
-	/* Hbox for location combo */
+	/* Location GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 
-	/* Location combo */
+	/* Location GtkCombo */
 	glist = NULL;
 	w = GUIComboCreate(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -1146,15 +1154,16 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 	w = combo->entry;
 	EDVEntrySetDND(core, w);
 	EDVEntrySetCompletePath(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
 
 
-	/* Hbox for options */
+	/* Options GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 
-	/* Case sensitive check */
+	/* Case sensitive GtkCheckButton */
 	fw->case_sensitive_check = w = gtk_check_button_new_with_label(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Sensible May/min."
@@ -1961,6 +1970,8 @@ edv_find_win_struct *EDVFindWinNew(edv_core_struct *core)
 
 	EDVFindWinUpdateMenus(fw);
 
+	fw->freeze_count--;
+
 	return(fw);
 }
 
@@ -2120,8 +2131,10 @@ void EDVFindWinSetBusy(edv_find_win_struct *fw, const gboolean is_busy)
  */
 gboolean EDVFindWinIsMapped(edv_find_win_struct *fw)
 {
-	GtkWidget *w = (fw != NULL) ? fw->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(fw == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(fw->toplevel));
 }
 
 /*
@@ -2129,11 +2142,12 @@ gboolean EDVFindWinIsMapped(edv_find_win_struct *fw)
  */
 void EDVFindWinMap(edv_find_win_struct *fw)
 {
-	GtkWidget *w = (fw != NULL) ? fw->toplevel : NULL;
-	if(w == NULL)
+	GtkWidget *w;
+
+	if(fw == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(fw->toplevel);
 
 	w = fw->search_combo;
 	w = GTK_COMBO(w)->entry;
@@ -2145,11 +2159,10 @@ void EDVFindWinMap(edv_find_win_struct *fw)
  */
 void EDVFindWinUnmap(edv_find_win_struct *fw)
 {
-	GtkWidget *w = (fw != NULL) ? fw->toplevel : NULL;
-	if(w == NULL)
+	if(fw == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(fw->toplevel);
 }
 
 /*
@@ -2160,29 +2173,17 @@ void EDVFindWinDelete(edv_find_win_struct *fw)
 	if(fw == NULL)
 	    return;
 
-	EDVStatusBarDelete(fw->status_bar);
+	EDVFindWinUnmap(fw);
+
+	fw->freeze_count++;
 
 	GTK_WIDGET_DESTROY(fw->results_clist_menu);
-
+	EDVStatusBarDelete(fw->status_bar);
 	PUListBoxDelete(fw->find_op_pulistbox);
-	GTK_WIDGET_DESTROY(fw->search_combo);
-	GTK_WIDGET_DESTROY(fw->location_combo);
-	GTK_WIDGET_DESTROY(fw->case_sensitive_check);
-	GTK_WIDGET_DESTROY(fw->recursive_check);
-
-	GTK_WIDGET_DESTROY(fw->search_btn);
-	GTK_WIDGET_DESTROY(fw->stop_btn);
-	GTK_WIDGET_DESTROY(fw->clear_btn);
-	GTK_WIDGET_DESTROY(fw->close_btn);
-	GTK_WIDGET_DESTROY(fw->results_clist);
-	GTK_WIDGET_DESTROY(fw->open_btn);
-	GTK_WIDGET_DESTROY(fw->open_with_btn);
-	GTK_WIDGET_DESTROY(fw->goto_btn);
-
-	GTK_WIDGET_DESTROY(fw->main_vbox);
 	GTK_WIDGET_DESTROY(fw->toplevel);
-
 	GTK_ACCEL_GROUP_UNREF(fw->accelgrp);
 
+	fw->freeze_count--;
+
 	g_free(fw);
 }
