@@ -46,26 +46,27 @@ struct _edv_find_win_struct {
 	GtkWidget	*toplevel;
 	GtkAccelGroup	*accelgrp;
 	gboolean	processing;
-	gint		busy_count;
+	gint		freeze_count,
+			busy_count,
+			stop_count;
 	edv_core_struct *core;
 
-	gint		stop_count;
-
 	gboolean	status_bar_map_state;
 
 	GtkWidget	*main_vbox;
 	pulistbox_struct	*find_op_pulistbox;
 	GtkWidget	*search_combo,
 			*location_combo,
-
 			*case_sensitive_check,
 			*recursive_check,
-
-			*search_btn, *stop_btn, *clear_btn, *close_btn,
-
+			*search_btn,
+			*stop_btn,
+			*clear_btn,
+			*close_btn,
 			*results_clist,
-
-			*open_btn, *open_with_btn, *goto_btn;
+			*open_btn,
+			*open_with_btn,
+			*goto_btn;
 
 	/* Results clist right click menu */
 	GtkWidget	*results_clist_menu,
@@ -74,7 +75,7 @@ struct _edv_find_win_struct {
 			*results_clist_goto_mi;
 
 	/* Status Bar */
-	edv_status_bar_struct *status_bar;
+	edv_status_bar_struct	*status_bar;
 
 	/* Reference to the window that created this find window
 	 *
