@@ -534,9 +534,18 @@ void EDVFindWinClickColumnCB(
 	if((clist == NULL) || (fw == NULL))
 	    return;
 
-	if(fw->processing)
+	if(fw->processing || (fw->freeze_count > 0))
 	    return;
 
+	gtk_clist_freeze(clist);
+	gtk_clist_set_sort_type(
+	    clist,
+	    (clist->sort_type == GTK_SORT_ASCENDING) ?
+		GTK_SORT_DESCENDING : GTK_SORT_ASCENDING
+	);
+	gtk_clist_set_sort_column(clist, column);
+	gtk_clist_sort(clist);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -551,7 +560,7 @@ void EDVFindWinSelectRowCB(
 	if((clist == NULL) || (fw == NULL))
 	    return;
 
-	if(fw->processing)
+	if(fw->processing || (fw->freeze_count > 0))
 	    return;
 
 	/* Check which clist this signal is for */
@@ -682,7 +691,7 @@ void EDVFindWinUnselectRowCB(
 	if((clist == NULL) || (fw == NULL))
 	    return;
 
-	if(fw->processing)
+	if(fw->processing || (fw->freeze_count > 0))
 	    return;
 
 	/* Check which clist this signal is for */
@@ -708,7 +717,7 @@ void EDVFindWinSearchCB(GtkWidget *widget, gpointer data)
 	gchar		*location,
 			*parent_path,
 			*search_string;
-	gint		total_matches = 0;
+	gint		nmatches = 0;
 	edv_findwin_operation operation;
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
@@ -716,17 +725,14 @@ void EDVFindWinSearchCB(GtkWidget *widget, gpointer data)
 	if(fw == NULL)
 	    return;
 
-	if(fw->processing)
+	if(fw->processing || (fw->freeze_count > 0))
 	    return;
 
 	core = fw->core;
-	if(core == NULL)
-	    return;
+	cfg_list = core->cfg_list;
 
 	fw->processing = TRUE;
 
-	cfg_list = core->cfg_list;
-
 
 	/* Get search parameters */
 
@@ -812,37 +818,34 @@ void EDVFindWinSearchCB(GtkWidget *widget, gpointer data)
 	    switch(fw->role)
 	    {
 	      case EDV_FIND_WIN_ROLE_DISK_OBJECT:
-		total_matches = EDVFindObjectByName(
+		nmatches = EDVFindObjectByName(
 		    location, search_string,
 		    recursive,		/* Recursive */
 		    FALSE,		/* Do not follow links */
 		    case_sensitive,	/* Case sensitive */
-		    fw,
-		    EDVFindWinFindProgressCB,
-		    EDVFindWinFindMatchedCB
+		    EDVFindWinFindProgressCB, fw,
+		    EDVFindWinFindMatchedCB, fw
 		);
 		break;
 
 	      case EDV_FIND_WIN_ROLE_RECYCLED_OBJECT:
-		total_matches = EDVFindRecycledObjectByName(
+		nmatches = EDVFindRecycledObjectByName(
 		    location,		/* Recycled objects index file */
 		    search_string,
 		    case_sensitive,	/* Case sensitive */
-		    fw,
-		    EDVFindWinFindProgressCB,
-		    EDVFindWinFindMatchedCB
+		    EDVFindWinFindProgressCB, fw,
+		    EDVFindWinFindMatchedCB, fw
 		);
 		break;
 
 	      case EDV_FINDBAR_ROLE_ARCHIVE_OBJECT:
-		total_matches = EDVFindArchiveObjectByName(
+		nmatches = EDVFindArchiveObjectByName(
 		    core,
 		    location,		/* Archive */
 		    search_string,
 		    case_sensitive,	/* Case sensitive */
-		    fw,
-		    EDVFindWinFindProgressCB,
-		    EDVFindWinFindMatchedCB
+		    EDVFindWinFindProgressCB, fw,
+		    EDVFindWinFindMatchedCB, fw
 		);
 		break;
 	    }
@@ -852,31 +855,30 @@ void EDVFindWinSearchCB(GtkWidget *widget, gpointer data)
 	    switch(fw->role)
 	    {
 	      case EDV_FIND_WIN_ROLE_DISK_OBJECT:
-		total_matches = EDVFindObjectByContent(
-		    location, search_string,
+		nmatches = EDVFindObjectByContent(
+		    location,
+		    search_string,
 		    recursive,		/* Recursive */
 		    FALSE,		/* Do not follow links */
 		    case_sensitive,	/* Case sensitive */
-		    fw,
-		    EDVFindWinFindProgressCB,
-		    EDVFindWinFindMatchedExcerptCB
+		    EDVFindWinFindProgressCB, fw,
+		    EDVFindWinFindMatchedExcerptCB, fw
 		);
 		break;
 
 	      case EDV_FIND_WIN_ROLE_RECYCLED_OBJECT:
-		total_matches = EDVFindRecycledObjectByContent(
+		nmatches = EDVFindRecycledObjectByContent(
 		    location,		/* Recycled objects index file */
 		    search_string,
 		    case_sensitive,	/* Case sensitive */
-		    fw,
-		    EDVFindWinFindProgressCB,
-		    EDVFindWinFindMatchedExcerptCB
+		    EDVFindWinFindProgressCB, fw,
+		    EDVFindWinFindMatchedExcerptCB, fw
 		);
 		break;
 
 	      case EDV_FINDBAR_ROLE_ARCHIVE_OBJECT:
 		/* Finding archive object by content is not supported */
-		total_matches = 0;
+		nmatches = 0;
 		break;
 	    }
 	    break;
@@ -886,17 +888,17 @@ void EDVFindWinSearchCB(GtkWidget *widget, gpointer data)
 
 	/* Begin tallying results */
 	EDVStatusBarProgress(fw->status_bar, 0.0f, FALSE);
-	if(total_matches > 0)
+	if(nmatches > 0)
 	{
-	    gchar *buf = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 		"Matched %i object%s%s",
-		total_matches,
-		(total_matches == 1) ? "" : "s",
+		nmatches,
+		(nmatches == 1) ? "" : "s",
 		(fw->stop_count > 0) ?
 		    " (search interrupted)" : ""
 	    );
-	    EDVStatusBarMessage(fw->status_bar, buf, FALSE);
-	    g_free(buf);
+	    EDVStatusBarMessage(fw->status_bar, msg, FALSE);
+	    g_free(msg);
 	}
 	else
 	{
