@@ -3,6 +3,7 @@
 #include <gdk/gdkprivate.h>
 #if defined(_WIN32)
 # include <gdk/gdkwin32.h>
+# define HAVE_WIN32
 #else
 # include <gdk/gdkx.h>
 # define HAVE_X
@@ -11,62 +12,79 @@
 #include "guirgbimg.h"
 
 
+static void GUIImagePixelRGBABlend(
+	const gint bpp, guint8 *tar, const guint8 *src
+);
+static void GUIImagePixelGreyscaleAlphaBlend(
+	const gint bpp, guint8 *tar, const guint8 *src
+);
+static void GUIImagePixelAnySet(
+	const gint bpp, guint8 *tar, const guint8 *src
+);
+
 void GUIImageBufferCopyArea(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl,
-	gint tar_x, gint tar_y
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	const gint tar_x, const gint tar_y,
+	const gboolean blend,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 void GUIImageBufferResize(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 void GUIImageBufferRotateCW90(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 );
 void GUIImageBufferRotateCCW90(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 );
 void GUIImageBufferRotateCW180(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 );
 void GUIImageBufferMirrorH(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data, 
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 );
 void GUIImageBufferMirrorV(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
-	guint8 *tar_data, 
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint src_width, const gint src_height, const gint src_bpl,
+	guint8 *tar_data,
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 );
 
 void GUIImageBufferFlattenWithBG(
-	gint bpp,
+	const gint bpp,
 	guint8 *data,
-	gint width, gint height, gint bpl,
-	const guint8 *bg_color
+	const gint width, const gint height, const gint bpl,
+	const guint8 *bg_color,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 );
 
 guint8 *GUIGetRGBImage(
@@ -93,23 +111,112 @@ guint8 *GUIGetRGBAImage(
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
-/*
- *	Copies the source image data to the target image at the
- *	specified coordinates.
- */
-void GUIImageBufferCopyArea(
-	gint bpp,
-	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
-	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl,
-	gint tar_x, gint tar_y
+static void GUIImagePixelRGBABlend(
+	const gint bpp, guint8 *tar, const guint8 *src
 )
 {
-	guint src_alpha, tar_alpha;
-	gint x, y;
-	const guint8 *src_line, *src_line_end, *src_ptr, *src_end;
-	guint8 *tar_ptr;
+	const guint32 src_alpha = src[3];
+	if(src_alpha == 0xff)
+	{
+	    *(guint32 *)tar = *(guint32 *)src;
+	}
+	else if(src_alpha > 0x00)
+	{
+	    const guint32 tar_alpha = 0xff - src_alpha;
+	    tar[0] = (guint8)(
+		((guint32)tar[0] * tar_alpha / 0xff) +
+		((guint32)src[0] * src_alpha / 0xff)
+	    );
+	    tar[1] = (guint8)(
+		((guint32)tar[1] * tar_alpha / 0xff) +
+		((guint32)src[1] * src_alpha / 0xff)
+	    );
+	    tar[2] = (guint8)(
+		((guint32)tar[2] * tar_alpha / 0xff) +
+		((guint32)src[2] * src_alpha / 0xff)
+	    );
+	    tar[3] = (guint8)MIN(
+		(guint32)tar[3] + src_alpha,
+		0xff
+	    );
+	}
+}
+
+static void GUIImagePixelGreyscaleAlphaBlend(
+	const gint bpp, guint8 *tar, const guint8 *src
+)
+{
+	const guint32 src_alpha = src[1];
+	if(src_alpha == 0xff)
+	{
+	    *(guint16 *)tar = *(guint16 *)src;
+	}
+	else if(src_alpha > 0x00)
+	{
+	    const guint32 tar_alpha = 0xff - src_alpha;
+	    tar[0] = (guint8)(
+		((guint32)tar[0] * tar_alpha / 0xff) +
+		((guint32)src[0] * src_alpha / 0xff)
+	    );
+	    tar[1] = (guint8)MIN(
+		(guint32)tar[1] + src_alpha,
+		0xff
+	    );
+	}
+}
+
+static void GUIImagePixelAnySet(
+	const gint bpp, guint8 *tar, const guint8 *src
+)
+{
+	memcpy(tar, src, bpp);
+}
+
+
+
+
+/*
+ *	Coppies an area of the source image to the target image.
+ *
+ *	The bpp specifies the bytes per pixel.
+ *
+ *	The src_data, src_width, and src_height specifies the
+ *	source image.
+ *
+ *	The src_bpl specifies the number of bytes per line on the
+ *	source image.
+ *
+ *	The tar_data, tar_width, and tar_height specifies the
+ *	source image.
+ *
+ *	The tar_bpl specifies the number of bytes per line on the
+ *	source image.
+ *
+ *	The tar_x and tar_y specifies the coordinates on the target
+ *	image to copy the source image to.
+ *
+ *	If blend is TRUE then if bpp is 4 or 2 (if there is an alpha
+ *	channel) then the source image will be blended on to the
+ *	target image, otherwise the source image's pixel is set to
+ *	the target image's pixel.
+ */
+void GUIImageBufferCopyArea(
+	const gint bpp,
+	const guint8 *src_data,
+	const gint src_width, const gint src_height, const gint src_bpl,
+	guint8 *tar_data,
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	const gint tar_x, const gint tar_y,
+	const gboolean blend,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
+)
+{
+	gint	x, y, src_y,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
+	const guint8 *src_ptr, *src_end;
+	void (*pixel_op_func)(const gint, guint8 *, const guint8 *);
 
 	if((src_width <= 0) || (src_height <= 0) ||
 	   (tar_width <= 0) || (tar_height <= 0) ||
@@ -118,19 +225,42 @@ void GUIImageBufferCopyArea(
 	)
 	    return;
 
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)src_height, progress_data))
+		return;
+	}
+
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
+
+	/* Determine the pixel operation function */
+	switch(bpp)
+	{
+	  case 4:
+	    if(blend)
+		pixel_op_func = GUIImagePixelRGBABlend;
+	    else
+		pixel_op_func = GUIImagePixelAnySet;
+	    break;
+	  case 2:
+	    if(blend)
+		pixel_op_func = GUIImagePixelGreyscaleAlphaBlend;
+	    else
+		pixel_op_func = GUIImagePixelAnySet;
+	    break;
+	  default:
+	    pixel_op_func = GUIImagePixelAnySet;
+	    break;
+	}
 
 	/* Iterate through source image */
-	for(y = tar_y,
-	    src_line = src_data,
-	    src_line_end = src_line + (src_bpl * src_height);
-	    src_line < src_line_end;
-	    y++,
-	    src_line += src_bpl
+	for(y = tar_y, src_y = 0;
+	    src_y < src_height;
+	    y++, src_y++
 	)
 	{
 	    if(y < 0)
@@ -138,8 +268,17 @@ void GUIImageBufferCopyArea(
 	    if(y >= tar_height)
 		break;
 
+	    if((progress_cb != NULL) && ((y % 10) == 0))
+	    {
+		if(progress_cb(
+		    (gulong)src_y, (gulong)src_height,
+		    progress_data
+		))
+		    return;
+	    }
+
 	    x = tar_x;
-	    src_ptr = src_line;
+	    src_ptr = src_data + (src_y * _src_bpl);
 	    src_end = src_ptr + (src_width * bpp);
 	    while(src_ptr < src_end)
 	    {
@@ -148,82 +287,19 @@ void GUIImageBufferCopyArea(
 		 */
 		if((x >= 0) && (x < tar_width))
 		{
-		    /* Current target coordinates are valid, get pointer
-		     * to target pixel
+		    /* Target coordinates are in bounds, set this
+		     * pixel
 		     */
-		    tar_ptr = tar_data + (y * tar_bpl) + (x * bpp);
-
-		    switch(bpp)
-		    {
-		      case 4:
-			src_alpha = src_ptr[3];
-			if(src_alpha == 0xff)
-			{
-			    *tar_ptr++ = *src_ptr++;
-			    *tar_ptr++ = *src_ptr++;
-			    *tar_ptr++ = *src_ptr++;
-			    *tar_ptr   = *src_ptr++;
-			}
-			else if(src_alpha > 0x00)
-			{
-			    tar_alpha = 0xff - src_alpha;
-			    tar_ptr[0] = (guint8)(
-				((guint)(*src_ptr++) * src_alpha / 0xff) +
-				((guint)tar_ptr[0] * tar_alpha / 0xff)
-			    );
-			    tar_ptr[1] = (guint8)(
-				((guint)(*src_ptr++) * src_alpha / 0xff) +
-				((guint)tar_ptr[1] * tar_alpha / 0xff)
-			    );
-			    tar_ptr[2] = (guint8)(
-				((guint)(*src_ptr++) * src_alpha / 0xff) +
-				((guint)tar_ptr[2] * tar_alpha / 0xff)
-			    );
-			    tar_ptr[3] = MIN(
-				(guint)(*src_ptr++) + (guint)tar_ptr[3],
-				0xff
-			    );
-			}
-			else
-			{
-			    src_ptr += bpp;
-			}
-			break;
-
-		      case 2:
-			src_alpha = src_ptr[1];
-			if(src_alpha == 0xff)
-			{
-			    *tar_ptr++ = *src_ptr++;
-			    *tar_ptr   = *src_ptr++;
-			}
-			else if(src_alpha > 0x00)
-			{
-			    tar_alpha = 0xff - src_alpha;
-			    tar_ptr[0] = (guint8)(
-				((guint)(*src_ptr++) * src_alpha / 0xff) +
-				((guint)tar_ptr[0] * tar_alpha / 0xff)
-			    );
-			    tar_ptr[1] = MIN(
-				(guint)(*src_ptr++) + (guint)tar_ptr[1],
-				0xff
-			    );
-			}
-			else
-			{   
-			    src_ptr += bpp;
-			}
-			break;
-
-		      default:
-			memcpy(tar_ptr, src_ptr, bpp);
-			src_ptr += bpp;
-			break;
-		    }
+		    pixel_op_func(
+			bpp,
+			tar_data + (y * _tar_bpl) + (x * bpp),
+			src_ptr
+		    );
+		    src_ptr += bpp;
 		}
 		else
 		{
-		    /* Target coordinates out of bounds, so skip this
+		    /* Target coordinates out of bounds, skip this
 		     * pixel
 		     */
  		    src_ptr += bpp;
@@ -232,6 +308,12 @@ void GUIImageBufferCopyArea(
 		x++;
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)src_height, (gulong)src_height, progress_data))
+		return;
+	}
 }
 
 
@@ -240,13 +322,17 @@ void GUIImageBufferCopyArea(
  *	a different size.
  */
 void GUIImageBufferResize(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 )
 {
+	gint	_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_ptr8;
 	guint8 *tar_ptr8;
 	const guint16 *src_ptr16;
@@ -270,11 +356,17 @@ void GUIImageBufferResize(
 	)
 	    return;
 
-	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)tar_height, progress_data))
+		return;
+	}
+
+	/* Calculate the bytes per line values as needed */
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* 8 bits */
 	if(bpp == 1)
@@ -295,19 +387,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr8 = (const guint8 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr8 = (guint8 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr8 = *src_ptr8;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -343,19 +442,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr16 = (const guint16 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr16 = (guint16 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr16 = *src_ptr16;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -391,20 +497,28 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr8 = (const guint8 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr8 = (guint8 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr8++ = *src_ptr8++;
 		*tar_ptr8++ = *src_ptr8++;
 		*tar_ptr8 = *src_ptr8;
 
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -440,19 +554,26 @@ void GUIImageBufferResize(
 		  (src_y_row < src_dh_height)
 	    )
 	    {
-		/* Get buffer position */
+		if((progress_cb != NULL) && ((tar_y_row % 10) == 0))
+		{
+		    if(progress_cb((gulong)tar_y_row, (gulong)tar_height, progress_data))
+			return;
+		}
+
+		/* Get the pointers to the current pixel in the data */
 		src_ptr32 = (const guint32 *)(src_data +
-		    ((src_y_row >> 8) * src_bpl) +
+		    ((src_y_row >> 8) * _src_bpl) +
 		    ((src_x_col >> 8) * bpp)
 		);
 		tar_ptr32 = (guint32 *)(tar_data +
-		    (tar_y_row * tar_bpl) +
+		    (tar_y_row * _tar_bpl) +
 		    (tar_x_col * bpp)
 		);
+
+		/* Set the pixel */
 		*tar_ptr32 = *src_ptr32;
 
-
-		/* Increment colum positions */
+		/* Increment the column positions */
 		tar_x_col += 1;
 		src_x_col += src_dh_x_col_inc;
 
@@ -469,6 +590,12 @@ void GUIImageBufferResize(
 		}
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)tar_height, (gulong)tar_height, progress_data))
+		return;
+	}
 }
 
 /*
@@ -476,14 +603,16 @@ void GUIImageBufferResize(
  *	clockwise 90 degrees.
  */
 void GUIImageBufferRotateCW90(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 )
 {
-	gint sx, sy, tx, ty;
+	gint	sx, sy, tx, ty,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_line;
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -498,10 +627,10 @@ void GUIImageBufferRotateCW90(
 	    return;
 
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* Copy/rotate clockwise 90 degrees */
 	for(sy = 0, tx = tar_width - 1;
@@ -509,13 +638,13 @@ void GUIImageBufferRotateCW90(
 	    sy++, tx--
 	)
 	{
-	    src_line = src_data + (sy * src_bpl);
+	    src_line = src_data + (sy * _src_bpl);
 	    for(sx = 0, ty = 0;
 		sx < src_width;
 		sx++, ty++
 	    )
 		memcpy(
-		    tar_data + (ty * tar_bpl) + (tx * bpp),
+		    tar_data + (ty * _tar_bpl) + (tx * bpp),
 		    src_line + (sx * bpp),
 		    bpp
 		);
@@ -527,14 +656,16 @@ void GUIImageBufferRotateCW90(
  *	counter-clockwise 90 degrees.
  */
 void GUIImageBufferRotateCCW90(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 )
 {
-	gint sx, sy, tx, ty;
+	gint	sx, sy, tx, ty,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_line;
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -549,10 +680,10 @@ void GUIImageBufferRotateCCW90(
 	    return;
 
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* Copy/rotate counter-clockwise 90 degrees */
 	for(sy = 0, tx = 0;
@@ -560,13 +691,13 @@ void GUIImageBufferRotateCCW90(
 	    sy++, tx++
 	)
 	{
-	    src_line = src_data + (sy * src_bpl);
+	    src_line = src_data + (sy * _src_bpl);
 	    for(sx = 0, ty = tar_height - 1;
 		sx < src_width;
 		sx++, ty--
 	    )
 		memcpy(
-		    tar_data + (ty * tar_bpl) + (tx * bpp),
+		    tar_data + (ty * _tar_bpl) + (tx * bpp),
 		    src_line + (sx * bpp),
 		    bpp
 		);
@@ -578,14 +709,16 @@ void GUIImageBufferRotateCCW90(
  *	clockwise 180 degrees.
  */
 void GUIImageBufferRotateCW180(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 ) 
 {
-	gint sx, sy, tx, ty;
+	gint	sx, sy, tx, ty,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_line;
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -600,10 +733,10 @@ void GUIImageBufferRotateCW180(
 	    return;
 
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* Copy/rotate clockwise 90 degrees */
 	for(sy = 0, ty = tar_height - 1;
@@ -611,13 +744,13 @@ void GUIImageBufferRotateCW180(
 	    sy++, ty--
 	)
 	{
-	    src_line = src_data + (sy * src_bpl);
+	    src_line = src_data + (sy * _src_bpl);
 	    for(sx = 0, tx = tar_width - 1;
 		sx < src_width;
 		sx++, tx--
 	    )
 		memcpy(
-		    tar_data + (ty * tar_bpl) + (tx * bpp),
+		    tar_data + (ty * _tar_bpl) + (tx * bpp),
 		    src_line + (sx * bpp),
 		    bpp
 		);
@@ -629,14 +762,16 @@ void GUIImageBufferRotateCW180(
  *	horizontally.
  */
 void GUIImageBufferMirrorH(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 )
 {
-	gint sx, sy, tx, ty;
+	gint	sx, sy, tx, ty,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_line;
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -651,10 +786,10 @@ void GUIImageBufferMirrorH(
 	    return;
 
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* Copy/mirror horizontally */
 	for(sy = 0, ty = 0;
@@ -662,13 +797,13 @@ void GUIImageBufferMirrorH(
 	    sy++, ty++
 	)
 	{
-	    src_line = src_data + (sy * src_bpl);
+	    src_line = src_data + (sy * _src_bpl);
 	    for(sx = 0, tx = tar_width - 1;
 		sx < src_width;
 		sx++, tx--
 	    )
 		memcpy(
-		    tar_data + (ty * tar_bpl) + (tx * bpp),
+		    tar_data + (ty * _tar_bpl) + (tx * bpp),
 		    src_line + (sx * bpp),
 		    bpp
 		);
@@ -680,14 +815,16 @@ void GUIImageBufferMirrorH(
  *	vertically.
  */
 void GUIImageBufferMirrorV(
-	gint bpp,
+	const gint bpp,
 	const guint8 *src_data,
-	gint src_width, gint src_height, gint src_bpl,
+	const gint src_width, const gint src_height, const gint src_bpl,
 	guint8 *tar_data,
-	gint tar_width, gint tar_height, gint tar_bpl
+	const gint tar_width, const gint tar_height, const gint tar_bpl
 )
 {
-	gint sx, sy, tx, ty;
+	gint	sx, sy, tx, ty,
+		_src_bpl = src_bpl,
+		_tar_bpl = tar_bpl;
 	const guint8 *src_line;
 
 	if((src_width <= 0) || (src_height <= 0) ||
@@ -702,10 +839,10 @@ void GUIImageBufferMirrorV(
 	    return;
 
 	/* Calculate bytes per line values as needed */
-	if(src_bpl <= 0)
-	    src_bpl = src_width * bpp;
-	if(tar_bpl <= 0)
-	    tar_bpl = tar_width * bpp;
+	if(_src_bpl <= 0)
+	    _src_bpl = src_width * bpp;
+	if(_tar_bpl <= 0)
+	    _tar_bpl = tar_width * bpp;
 
 	/* Copy/mirror horizontally */
 	for(sy = 0, ty = tar_height - 1;
@@ -713,13 +850,13 @@ void GUIImageBufferMirrorV(
 	    sy++, ty--
 	)
 	{
-	    src_line = src_data + (sy * src_bpl);
+	    src_line = src_data + (sy * _src_bpl);
 	    for(sx = 0, tx = 0;
 		sx < src_width;
 		sx++, tx++
 	    )
 		memcpy(
-		    tar_data + (ty * tar_bpl) + (tx * bpp),
+		    tar_data + (ty * _tar_bpl) + (tx * bpp),
 		    src_line + (sx * bpp),
 		    bpp
 		);
@@ -727,40 +864,54 @@ void GUIImageBufferMirrorV(
 }
 
 /*
- *	Combines the image data with the specified background color.
+ *	Combines the image data with the background color.
  *
- *	Only bpp=2 (Greyscale Alpha) and bpp=4 (RGBA) are supported.
+ *	The bpp specifies the bytes per pixel and this value must
+ *	be either 2 (Greyscale Alpha) or 4 (RGBA).
  */
 void GUIImageBufferFlattenWithBG(
-	gint bpp,
+	const gint bpp,
 	guint8 *data,
-	gint width, gint height, gint bpl,
-	const guint8 *bg_color
+	const gint width, const gint height, const gint bpl,
+	const guint8 *bg_color,
+	gint (*progress_cb)(const gulong, const gulong, gpointer),
+	gpointer progress_data
 )
 {
+	gint _bpl = bpl;
 	guint tar_alpha, bg_alpha;
-	guint8 *line, *line_end, *ptr, *ptr_end;
 
 	if((data == NULL) || (bg_color == NULL))
 	    return;
 
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb(0l, (gulong)height, progress_data))
+		return;
+	}
+
 	/* Calculate bytes per line value as needed */
-	if(bpl <= 0)
-	    bpl = width * bpp;
+	if(_bpl <= 0)
+	    _bpl = width * bpp;
 
 	/* Greyscale Alpha? */
 	if(bpp == 2)
 	{
-	    for(line = data,
-		line_end = line + (bpl * height);
-		line < line_end;
-		line += bpl
-	    )
-	    {
-		ptr = line;
-		ptr_end = ptr + (width * bpp);
+	    gint y;
+	    guint8 *ptr, *end;
 
-		while(ptr < ptr_end)
+	    for(y = 0; y < height; y++)
+	    {
+		if((progress_cb != NULL) && ((y % 10) == 0))
+		{
+		    if(progress_cb((gulong)y, (gulong)height, progress_data))
+			return;
+		}
+
+		ptr = data + (y * _bpl);
+		end = ptr + (width * bpp);
+
+		while(ptr < end)
 		{
 		    tar_alpha = (guint)ptr[3];
 		    if(tar_alpha == 0xff)
@@ -787,16 +938,21 @@ void GUIImageBufferFlattenWithBG(
 	/* RGBA */
 	else if(bpp == 4)
 	{
-	    for(line = data, 
-		line_end = line + (bpl * height);
-		line < line_end;
-		line += bpl
-	    )
-	    {
-		ptr = line;
-		ptr_end = ptr + (width * bpp);
+	    gint y;
+	    guint8 *ptr, *end;
 
-		while(ptr < ptr_end)
+	    for(y = 0; y < height; y++)
+	    {
+		if((progress_cb != NULL) && ((y % 10) == 0))
+		{
+		    if(progress_cb((gulong)y, (gulong)height, progress_data))
+			return;
+		}
+
+		ptr = data + (y * _bpl);
+		end = ptr + (width * bpp);
+
+		while(ptr < end)
 		{
 		    tar_alpha = (guint)ptr[3];
 		    if(tar_alpha == 0xff)
@@ -828,6 +984,12 @@ void GUIImageBufferFlattenWithBG(
 		}
 	    }
 	}
+
+	if(progress_cb != NULL)
+	{
+	    if(progress_cb((gulong)height, (gulong)height, progress_data))
+		return;
+	}
 }
 
 
@@ -843,15 +1005,18 @@ guint8 *GUIGetRGBImage(
 	gint *width_rtn, gint *height_rtn, gint *bpl_rtn  
 )
 {
-#if defined(__GDK_X_H__)
+#if defined(HAVE_X)
 	gint width, height, src_bpp, src_bpl, tar_bpp, tar_bpl;
 	const guint8 *src_data;
 	guint8 *tar_data;
 	Display *xdpy;
+	Window xwin;
 	XImage *ximg;
 	GdkRectangle lrect;
-	GdkWindowPrivate *private = (GdkWindowPrivate *)drawable;
-	Window xwin = (private != NULL) ? private->xwindow : None;
+	GdkVisualType visual_type;
+	GdkVisual *visual;
+	GdkColormap *cmap;
+	GdkWindowPrivate *private;
 
 	if(width_rtn != NULL)
 	    *width_rtn = 0;
@@ -860,21 +1025,30 @@ guint8 *GUIGetRGBImage(
 	if(bpl_rtn != NULL)
 	    *bpl_rtn = 0;
 
-	if(xwin == None)
+	if(drawable == NULL)
 	    return(NULL);
 
+	private = (GdkWindowPrivate *)drawable;
 	xdpy = private->xdisplay;
-	if(xdpy == NULL)
+	xwin = private->xwindow;
+	if((xdpy == NULL) || (xwin == None))
 	    return(NULL);
 
-	/* If no rectangle is specified then set rectangle to cover the
-	 * entire drawable
+	cmap = private->colormap;
+	visual = (cmap != NULL) ? gdk_colormap_get_visual(cmap) : NULL;
+	if(visual != NULL)
+	    visual_type = visual->type;
+	else
+	    visual_type = gdk_visual_get_best_type();
+
+	/* If no rectangle was specified then set the rectangle to
+	 * cover the entire drawable
 	 */
 	if(rect == NULL)
 	{
 	    rect = &lrect;
-	    lrect.x = private->x;
-	    lrect.y = private->y;
+	    lrect.x = 0;
+	    lrect.y = 0;
 	    lrect.width = private->width;
 	    lrect.height = private->height;
 	}
@@ -890,14 +1064,16 @@ guint8 *GUIGetRGBImage(
 	    AllPlanes,
 	    ZPixmap
 	);
-	if((ximg != NULL) ? (ximg->data == NULL) : TRUE)
+	if(ximg == NULL)
+	    return(NULL);
+
+	if(ximg->data == NULL)
 	{
-	    if(ximg != NULL)
-		XDestroyImage(ximg);
+	    XDestroyImage(ximg);
 	    return(NULL);
 	}
 
-	/* Get values from XImage */
+	/* Get the values from the XImage */
 	width = ximg->width;
 	height = ximg->height;
 	src_bpl = ximg->bytes_per_line;
@@ -918,38 +1094,94 @@ guint8 *GUIGetRGBImage(
 	 */
 	if(ximg->depth == 8)
 	{
-	    gint xoffset_bytes, x, y;
 	    const gint	r_div = 7,
 			g_div = 7,
 			b_div = 3;
-	    guint src_v;
+	    gint xoffset_bytes, x, y;
+	    guint32 src_v;
 	    const guint8 *src_ptr;
 	    guint8 *tar_ptr;
+	    GdkColor *c;
 
 	    src_bpp = 1;
 	    xoffset_bytes = ximg->xoffset * src_bpp;
 
-	    for(y = 0; y < height; y++)
+	    switch(visual_type)
 	    {
-		src_ptr = (const guint8 *)(
-		    src_data + (y * src_bpl) + xoffset_bytes
-		);
-		tar_ptr = tar_data + (y * tar_bpl);
-
-		for(x = 0; x < width; x++)
+	      case GDK_VISUAL_STATIC_GRAY:
+	      case GDK_VISUAL_GRAYSCALE:
+		for(y = 0; y < height; y++)
 		{
-		    /* 8 Bits Format: rrrgggbb */
-		    src_v = (guint)*src_ptr++;
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0xE0) >> 5) * 0xff / r_div  
-		    );
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0x1C) >> 2) * 0xff / g_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0x03) >> 0) * 0xff / b_div
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
 		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			*tar_ptr++ = *src_ptr;
+			*tar_ptr++ = *src_ptr;
+			*tar_ptr++ = *src_ptr++;
+		    }
 		}
+		break;
+
+	      case GDK_VISUAL_STATIC_COLOR:
+	      case GDK_VISUAL_PSEUDO_COLOR:
+		if(cmap == NULL)
+		{
+		    g_free(tar_data);
+		    XDestroyImage(ximg);
+		    return(NULL);
+		}
+		if((cmap->colors == NULL) || (cmap->size <= 0))
+		{
+		    g_free(tar_data);
+		    XDestroyImage(ximg);
+		    return(NULL);
+		}
+		for(y = 0; y < height; y++)
+		{
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
+		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			c = &cmap->colors[*src_ptr];
+			src_ptr++;
+			*tar_ptr++ = (guint8)(c->red >> 8);
+			*tar_ptr++ = (guint8)(c->green >> 8);
+			*tar_ptr++ = (guint8)(c->blue >> 8);
+		    }
+		}
+		break;
+
+	      default:		/* GDK_VISUAL_TRUE_COLOR */
+		for(y = 0; y < height; y++)
+		{
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
+		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			/* 8 Bits Format: rrrgggbb */
+			src_v = (guint32)*src_ptr++;
+			*tar_ptr++ = (guint8)(
+		((src_v & 0xE0) >> 5) * 0xff / r_div  
+			);
+			*tar_ptr++ = (guint8)(
+		((src_v & 0x1C) >> 2) * 0xff / g_div
+			);
+			*tar_ptr++ = (guint8)(
+		((src_v & 0x03) >> 0) * 0xff / b_div
+			);
+		    }
+		}
+		break;
 	    }
 	}
 	else if(ximg->depth == 15)
@@ -958,6 +1190,7 @@ guint8 *GUIGetRGBImage(
 	    const gint  r_div = 31,
 			g_div = 31,
 			b_div = 31;
+	    guint16 v16;
 	    const guint16 *src_ptr;
 	    guint8 *tar_ptr;
 
@@ -973,18 +1206,18 @@ guint8 *GUIGetRGBImage(
 
 		for(x = 0; x < width; x++)
 		{
-		    /* 16 Bits Format: arrrrrgg gggbbbbb */
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x7C00) >> 10) * 0xff / r_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x03E0) >> 5) * 0xff / g_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x001F) >> 0) * 0xff / b_div
-		    );
+		    /* 15 Bits Format: arrrrrgg gggbbbbb */
+		    v16 = *src_ptr++;
 
-		    src_ptr++;
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0x7C00) >> 10) * 0xFF / r_div
+		    );
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0x03E0) >> 5) * 0xFF / g_div
+		    );
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0x001F) >> 0) * 0xFF / b_div
+		    );
 		}
 	    }
 	}
@@ -994,6 +1227,7 @@ guint8 *GUIGetRGBImage(
 	    const gint  r_div = 31,
 			g_div = 63,
 			b_div = 31;
+	    guint16 v16;
 	    const guint16 *src_ptr;
 	    guint8 *tar_ptr;
 
@@ -1010,17 +1244,17 @@ guint8 *GUIGetRGBImage(
 		for(x = 0; x < width; x++)
 		{
 		    /* 16 Bits Format: rrrrrggg gggbbbbb */
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0xF800) >> 11) * 0xff / r_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x07E0) >> 5) * 0xff / g_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x001F) >> 0) * 0xff / b_div
-		    );
+		    v16 = *src_ptr++;
 
-		    src_ptr++;
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0xF800) >> 11) * 0xFF / r_div
+		    );
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0x07E0) >> 5) * 0xFF / g_div
+		    );
+		    *tar_ptr++ = (guint8)(
+		(guint32)((v16 & 0x001F) >> 0) * 0xFF / b_div
+		    );
 		}
 	    }
 	}
@@ -1112,15 +1346,18 @@ guint8 *GUIGetRGBAImage(
 	gint *width_rtn, gint *height_rtn, gint *bpl_rtn  
 )
 {
-#if defined(__GDK_X_H__)
+#if defined(HAVE_X)
 	gint width, height, src_bpp, src_bpl, tar_bpp, tar_bpl;
 	const guint8 *src_data;
 	guint8 *tar_data;
 	Display *xdpy;
+	Window xwin;
 	XImage *ximg;
 	GdkRectangle lrect;
-	GdkWindowPrivate *private = (GdkWindowPrivate *)drawable;
-	Window xwin = (private != NULL) ? private->xwindow : None;
+	GdkVisualType visual_type;
+	GdkVisual *visual;
+	GdkColormap *cmap;
+	GdkWindowPrivate *private;
 
 	if(width_rtn != NULL)
 	    *width_rtn = 0;
@@ -1129,21 +1366,30 @@ guint8 *GUIGetRGBAImage(
 	if(bpl_rtn != NULL)
 	    *bpl_rtn = 0;
 
-	if(xwin == None)
+	if(drawable == NULL)
 	    return(NULL);
 
+	private = (GdkWindowPrivate *)drawable;
 	xdpy = private->xdisplay;
-	if(xdpy == NULL)
+	xwin = private->xwindow;
+	if((xdpy == NULL) || (xwin == None))
 	    return(NULL);
 
-	/* If no rectangle is specified then set rectangle to cover the
-	 * entire drawable
+	cmap = private->colormap;
+	visual = (cmap != NULL) ? gdk_colormap_get_visual(cmap) : NULL;
+	if(visual != NULL)
+	    visual_type = visual->type;
+	else
+	    visual_type = gdk_visual_get_best_type();
+
+	/* If no rectangle was specified then set the rectangle to
+	 * cover the entire drawable
 	 */
 	if(rect == NULL)
 	{
 	    rect = &lrect;
-	    lrect.x = private->x;
-	    lrect.y = private->y;
+	    lrect.x = 0;
+	    lrect.y = 0;
 	    lrect.width = private->width;
 	    lrect.height = private->height;
 	}
@@ -1159,10 +1405,12 @@ guint8 *GUIGetRGBAImage(
 	    AllPlanes,
 	    ZPixmap
 	);
-	if((ximg != NULL) ? (ximg->data == NULL) : TRUE)
+	if(ximg == NULL)
+	    return(NULL);
+
+	if(ximg->data == NULL)
 	{
-	    if(ximg != NULL)
-		XDestroyImage(ximg);
+	    XDestroyImage(ximg);
 	    return(NULL);
 	}
 
@@ -1173,7 +1421,7 @@ guint8 *GUIGetRGBAImage(
 	src_data = (const guint8 *)ximg->data;
 
 	/* Allocate the target RGBA image data */
-	tar_bpp = 4;
+	tar_bpp = 4 * sizeof(guint8);
 	tar_bpl = width * tar_bpp;
 	tar_data = (guint8 *)g_malloc(tar_bpl * height);
 	if(tar_data == NULL)
@@ -1183,18 +1431,14 @@ guint8 *GUIGetRGBAImage(
 	}
 
 	/* Begin copying the XImage data to the return data */
-	if(ximg->depth == 8)
+	if(ximg->depth == 1)
 	{
-	    gint xoffset_bytes, x, y;
-	    const gint	r_div = 7,
-			g_div = 7,
-			b_div = 3;
-	    guint src_v;
+	    gint x, y, xoffset_bytes;
 	    const guint8 *src_ptr;
 	    guint8 *tar_ptr;
 
-	    src_bpp = 1;
-	    xoffset_bytes = ximg->xoffset * src_bpp;
+            src_bpp = 1;
+            xoffset_bytes = ximg->xoffset * src_bpp;
 
 	    for(y = 0; y < height; y++)
 	    {
@@ -1205,27 +1449,125 @@ guint8 *GUIGetRGBAImage(
 
 		for(x = 0; x < width; x++)
 		{
-		    /* 8 Bits Format: rrrgggbb */
-		    src_v = (guint)*src_ptr++;
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0xE0) >> 5) * 0xff / r_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0x1C) >> 2) * 0xff / g_div
-		    );
-		    *tar_ptr++ = (guint8)(
-		((src_v & 0x03) >> 0) * 0xff / b_div
-		    );
-		    *tar_ptr++ = 0xff;
+		    if(src_ptr[x / 8] & (1 << (x % 8)))
+		    {
+			*tar_ptr++ = 0xff;
+			*tar_ptr++ = 0xff;
+			*tar_ptr++ = 0xff;
+			*tar_ptr++ = 0xff;
+		    }
+		    else
+		    {
+			*tar_ptr++ = 0x00;
+			*tar_ptr++ = 0x00;
+			*tar_ptr++ = 0x00;
+			*tar_ptr++ = 0x00;
+		    }
 		}
 	    }
 	}
+	else if(ximg->depth == 8)
+	{
+	    const gint	r_div = 7,
+			g_div = 7,
+			b_div = 3;
+	    gint xoffset_bytes, x, y;
+	    guint32 src_v;
+	    const guint8 *src_ptr;
+	    guint8 *tar_ptr;
+	    GdkColor *c;
+
+	    src_bpp = 1;
+	    xoffset_bytes = ximg->xoffset * src_bpp;
+
+	    switch(visual_type)
+	    {
+	      case GDK_VISUAL_STATIC_GRAY:
+	      case GDK_VISUAL_GRAYSCALE:
+		for(y = 0; y < height; y++)
+		{
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
+		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			*tar_ptr++ = *src_ptr;
+			*tar_ptr++ = *src_ptr;
+			*tar_ptr++ = *src_ptr++;
+			*tar_ptr++ = 0xff;
+		    }
+		}
+		break;
+
+	      case GDK_VISUAL_STATIC_COLOR:
+	      case GDK_VISUAL_PSEUDO_COLOR:
+		if(cmap == NULL)
+		{
+		    g_free(tar_data);
+		    XDestroyImage(ximg);
+		    return(NULL);
+		}
+		if((cmap->colors == NULL) || (cmap->size <= 0))
+		{
+		    g_free(tar_data);
+		    XDestroyImage(ximg);
+		    return(NULL);
+		}
+		for(y = 0; y < height; y++)
+		{
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
+		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			c = &cmap->colors[*src_ptr];
+			src_ptr++;
+			*tar_ptr++ = (guint8)(c->red >> 8);
+			*tar_ptr++ = (guint8)(c->green >> 8);
+			*tar_ptr++ = (guint8)(c->blue >> 8);
+			*tar_ptr++ = 0xff;
+		    }
+		}
+		break;
+
+	      default:		/* GDK_VISUAL_TRUE_COLOR */
+		for(y = 0; y < height; y++)
+		{
+		    src_ptr = (const guint8 *)(
+			src_data + (y * src_bpl) + xoffset_bytes
+		    );
+		    tar_ptr = tar_data + (y * tar_bpl);
+
+		    for(x = 0; x < width; x++)
+		    {
+			/* 8 Bits Format: rrrgggbb */
+			src_v = (guint32)*src_ptr++;
+			*tar_ptr++ = (guint8)(
+		((src_v & 0xE0) >> 5) * 0xff / r_div  
+			);
+			*tar_ptr++ = (guint8)(
+		((src_v & 0x1C) >> 2) * 0xff / g_div
+			);
+			*tar_ptr++ = (guint8)(
+		((src_v & 0x03) >> 0) * 0xff / b_div
+			);
+			*tar_ptr++ = 0xff;
+		    }
+		}
+		break;
+	    }
+	}
 	else if(ximg->depth == 15)
 	{
 	    gint xoffset_bytes, x, y;
 	    const gint	r_div = 31,
 			g_div = 31,
 			b_div = 31;
+	    guint16 v16;
 	    const guint16 *src_ptr;
 	    guint8 *tar_ptr;
 
@@ -1241,23 +1583,22 @@ guint8 *GUIGetRGBAImage(
 
 		for(x = 0; x < width; x++)
 		{
-		    /* 16 Bits Format: arrrrrgg gggbbbbb */
+		    /* 15 Bits Format: arrrrrgg gggbbbbb */
+		    v16 = *src_ptr++;
+
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x7C00) >> 10) * 0xff / r_div
+		(guint32)((v16 & 0x7C00) >> 10) * 0xFF / r_div
 		    );
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x03E0) >> 5) * 0xff / g_div
+		(guint32)((v16 & 0x03E0) >> 5) * 0xFF / g_div
 		    );
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x001F) >> 0) * 0xff / b_div
+		(guint32)((v16 & 0x001F) >> 0) * 0xFF / b_div
 		    );
 /* Alpha channel may not be defined
-		    *tar_ptr++ = (((guint)*src_ptr) & (1 << 15)) ?
-			0xff : 0x00;
+		    *tar_ptr++ = (v16 & (1 << 15)) ? 0xff : 0x00;
  */
 		    *tar_ptr++ = 0xff;
-
-		    src_ptr++;
 		}
 	    }
 	}
@@ -1267,6 +1608,7 @@ guint8 *GUIGetRGBAImage(
 	    const gint  r_div = 31,
 			g_div = 63,
 			b_div = 31;
+	    guint16 v16;
 	    const guint16 *src_ptr;
 	    guint8 *tar_ptr;
 
@@ -1283,18 +1625,18 @@ guint8 *GUIGetRGBAImage(
 		for(x = 0; x < width; x++)
 		{
 		    /* 16 Bits Format: rrrrrggg gggbbbbb */
+		    v16 = *src_ptr++;
+
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0xF800) >> 11) * 0xff / r_div
+		(guint32)((v16 & 0xF800) >> 11) * 0xff / r_div
 		    );
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x07E0) >> 5) * 0xff / g_div
+		(guint32)((v16 & 0x07E0) >> 5) * 0xff / g_div
 		    );
 		    *tar_ptr++ = (guint8)(
-		((((guint)*src_ptr) & 0x001F) >> 0) * 0xff / b_div
+		(guint32)((v16 & 0x001F) >> 0) * 0xff / b_div
 		    );
 		    *tar_ptr++ = 0xff;
-
-		    src_ptr++;
 		}
 	    }
 	}	    
