@@ -22,6 +22,20 @@
 #endif	/* End of GTK 2 wrappers */
 
 
+/*
+ *	Standard Target Names:
+ */
+#define GUI_TARGET_NAME_STRING		"STRING"
+#define GUI_TARGET_NAME_TEXT_PLAIN	"text/plain"
+#define GUI_TARGET_NAME_TEXT_URI_LIST	"text/uri-list"
+
+
+/*
+ *	Standard Type Names:
+ */
+#define GUI_TYPE_NAME_STRING		"STRING"
+
+
 /*
  *	Button Numbers:
  */
@@ -137,6 +151,15 @@ typedef enum {
 } gui_menu_bar_item_alignment;
 
 
+/*
+ *	GtkEditable Popup Menu Flags:
+ */
+typedef enum {
+	GUI_EDITABLE_POPUP_MENU_READ_ONLY	= (1 << 0),
+	GUI_EDITABLE_POPUP_MENU_UNDO		= (1 << 1)	/* And redo */
+} gui_editable_popup_menu_flags;
+
+
 /*
  *	String Macros:
  */
@@ -219,8 +242,8 @@ typedef enum {
  */
 #define GTK_OBJECT_UNREF(_o_)			\
 { if((_o_) != NULL) gtk_object_unref(GTK_OBJECT(_o_)); }
-#define GTK_OBJECT_GET_DATA(_o_,_k_)		\
- (((_o_) != NULL) ? gtk_object_get_data((GtkObject *)(_o_), (_k_)) : NULL)
+#define GTK_OBJECT_GET_DATA(_o_,_key_)		\
+ (((_o_) != NULL) ? gtk_object_get_data((GtkObject *)(_o_), (_key_)) : NULL)
 
 /*
  *	GtkSignal Macros:
@@ -440,9 +463,11 @@ extern void GUISetGlobalTipsState(gboolean state);
 
 /* Buttons Modifying */
 extern void GUIButtonChangeLayout(
-	GtkWidget *w, gboolean show_pixmap, gboolean show_label
+	GtkWidget *w,
+	const gboolean show_pixmap,
+	const gboolean show_label
 );
-extern void GUIButtonLabelUnderline(GtkWidget *w, guint c);
+extern void GUIButtonLabelUnderline(GtkWidget *w, const guint c);
 extern void GUIButtonPixmapUpdate(
 	GtkWidget *w, guint8 **icon, const gchar *label
 );
@@ -510,18 +535,18 @@ extern void GUIDDESetDirect(
 	guint8 *data, const gint length
 );
 extern void GUIDDESet(
-        GtkWidget *w,
-        GdkAtom selection_atom,
-        const gulong t,
-        GdkAtom target_atom,
-        const guint8 *data, const gint length
+	GtkWidget *w,
+	GdkAtom selection_atom,
+	const gulong t,
+	GdkAtom target_atom,
+	const guint8 *data, const gint length
 );
 extern guint8 *GUIDDEGet(
-        GtkWidget *w,
-        GdkAtom selection_atom,
-        const gulong t,
-        GdkAtom target_atom,
-        gint *length
+	GtkWidget *w,
+	GdkAtom selection_atom,
+	const gulong t,
+	GdkAtom target_atom,
+	gint *length
 );
 extern void GUIDDEClear(GtkWidget *w);
 extern void GUIDDESetBinary(
@@ -551,27 +576,50 @@ extern gchar *GUIDDEGetString(
 
 /* Drag & Drop Utilities */
 extern void GUIDNDSetSrc(
-	void *w, const void *drag_type, int ndrag_types,
-	unsigned int actions, unsigned int buttons,
-	void (*begin_cb)(GtkWidget *, GdkDragContext *, gpointer),
-	void (*request_data_cb)(GtkWidget *, GdkDragContext *,
-		GtkSelectionData *, guint, guint, gpointer),
-	void (*delete_data_cb)(GtkWidget *, GdkDragContext *, gpointer),
-	void (*end_cb)(GtkWidget *, GdkDragContext *, gpointer),
-	gpointer client_data
+	GtkWidget *w,
+	const GtkTargetEntry *target_entry,
+	const gint ntarget_entries,
+	const GdkDragAction actions,
+	const GdkModifierType buttons,
+	void (*drag_begin_cb)(
+		GtkWidget *, GdkDragContext *,
+		gpointer
+	),
+	void (*drag_data_get_cb)(
+		GtkWidget *, GdkDragContext *,
+		GtkSelectionData *,
+		guint, guint,
+		gpointer
+	),
+	void (*drag_data_delete_cb)(
+		GtkWidget *, GdkDragContext *,
+		gpointer
+	),
+	void (*drag_end_cb)(
+		GtkWidget *, GdkDragContext *,
+		gpointer
+	),
+	gpointer data
 );
 extern void GUIDNDSetTar(
-	void *w, const void *drag_type, int ndrag_types,
-	unsigned int actions, unsigned int default_action_same,
-	unsigned int default_action,
-	void (*recieved_data_cb)(GtkWidget *, GdkDragContext *,
-		gint, gint, GtkSelectionData *, guint, guint,
-		gpointer  
+	GtkWidget *w,
+	const GtkTargetEntry *target_entry,
+	const gint ntarget_entries,
+	const GdkDragAction actions,
+	const GdkDragAction default_action_same,
+	const GdkDragAction default_action,
+	void (*drag_data_recieved_cb)(
+		GtkWidget *, GdkDragContext *,
+		gint, gint,
+		GtkSelectionData *,
+		guint, guint,
+		gpointer
 	),
-	gpointer client_data
+	gpointer drag_data_received_data
 );
 extern void GUIDNDSetDragIcon(
-	GdkPixmap *pixmap, GdkBitmap *mask, gint hot_x, gint hot_y
+	GdkPixmap *pixmap, GdkBitmap *mask,
+	const gint hot_x, const gint hot_y
 );
 
 
@@ -634,6 +682,10 @@ extern GtkWidget *GUIMenuAddToMenuBar(
 );
 extern void GUIMenuItemSetLabel(GtkWidget *menu_item, const gchar *label);
 extern void GUIMenuItemSetPixmap(GtkWidget *menu_item, guint8 **icon_data);
+extern void GUIMenuItemSetPixmap2(
+	GtkWidget *menu_item,
+	GdkPixmap *pixmap, GdkBitmap *mask
+);
 extern void GUIMenuItemSetAccelKey(
 	GtkWidget *menu_item, GtkAccelGroup *accelgrp,
 	guint accel_key, guint accel_mods
@@ -652,23 +704,30 @@ extern void GUIMenuItemSetSubMenu(
 );
 
 
+/* Editable Popup Menu */
+extern GtkWidget *GUIEditableGetPopupMenu(GtkWidget *w);
+extern void GUIEditableEndowPopupMenu(
+	GtkWidget *w, const gui_editable_popup_menu_flags flags
+);
+
+
 /* Pull Out Window */
 extern void *GUIPullOutCreateH(
-	void *parent_box,
-	int homogeneous, int spacing,		/* Of client vbox */
-	int expand, int fill, int padding,	/* Of holder hbox */
-	int toplevel_width, int toplevel_height,
+	gpointer parent_box,
+	gint homogeneous, gint spacing,		/* Of client vbox */
+	gint expand, gint fill, gint padding,	/* Of holder hbox */
+	gint toplevel_width, gint toplevel_height,
 	void *pull_out_client_data,
 	void (*pull_out_cb)(void *, void *),
 	void *push_in_client_data,
 	void (*push_in_cb)(void *, void *)
 );
 extern void *GUIPullOutGetToplevelWindow(
-	void *client_box,
-	int *x, int *y, int *width, int *height
+	gpointer client_box,
+	gint *x, gint *y, gint *width, gint *height
 );
-extern void GUIPullOutPullOut(void *client_box);
-extern void GUIPullOutPushIn(void *client_box);
+extern void GUIPullOutPullOut(gpointer client_box);
+extern void GUIPullOutPushIn(gpointer client_box);
 
 
 #endif	/* GUIUTILS_H */
