@@ -506,17 +506,24 @@ ascii_chart_dlg_struct *ASCIIChartDlgNew(GtkWidget *ref_toplevel)
 			border_minor = 2;
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
-	GtkWidget *w, *parent, *parent2, *parent3, *parent4;
+	GtkWidget	*w,
+			*parent, *parent2, *parent3, *parent4,
+			*toplevel;
 	ascii_chart_dlg_struct *d = ASCII_CHART_DLG(g_malloc0(
 	    sizeof(ascii_chart_dlg_struct)
 	));
+	if(d == NULL)
+	    return(NULL);
 
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
 	d->processing = FALSE;
 	d->freeze_count = 0;
 
-	/* Create toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	d->freeze_count++;
+
+	/* Toplevel GtkWindow */
+	w = toplevel;
 	gtk_window_set_title(
 	    GTK_WINDOW(w),
 	    PROG_NAME_FULL ": Extended ASCII Chart"
@@ -728,6 +735,7 @@ ascii_chart_dlg_struct *ASCIIChartDlgNew(GtkWidget *ref_toplevel)
 
 	ASCIIChartDlgUpdateMenus(d);
 
+	d->freeze_count--;
 
 	return(d);
 }
@@ -740,6 +748,11 @@ void ASCIIChartDlgUpdateMenus(ascii_chart_dlg_struct *d)
 {
 	if(d == NULL)
 	    return;
+
+	d->freeze_count++;
+
+
+	d->freeze_count--;
 }
 
 /*
@@ -747,8 +760,10 @@ void ASCIIChartDlgUpdateMenus(ascii_chart_dlg_struct *d)
  */
 gboolean ASCIIChartDlgIsMapped(ascii_chart_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(d == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(d->toplevel));
 }
 
 /*
@@ -756,19 +771,17 @@ gboolean ASCIIChartDlgIsMapped(ascii_chart_dlg_struct *d)
  */
 void ASCIIChartDlgMap(ascii_chart_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	GtkWidget *w;
+
+	if(d == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(d->toplevel);
 
 	/* Grab focus and default on the OK button */
 	w = d->ok_btn;
-	if(w != NULL)
-	{
-	    gtk_widget_grab_focus(w);
-	    gtk_widget_grab_default(w);
-	}
+	gtk_widget_grab_focus(w);
+	gtk_widget_grab_default(w);
 }
 
 /*
@@ -776,11 +789,10 @@ void ASCIIChartDlgMap(ascii_chart_dlg_struct *d)
  */
 void ASCIIChartDlgUnmap(ascii_chart_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	if(d == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(d->toplevel);
 }
 
 /*
@@ -791,14 +803,16 @@ void ASCIIChartDlgDelete(ascii_chart_dlg_struct *d)
 	if(d == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(d->ok_btn);
-	GTK_WIDGET_DESTROY(d->main_vbox);
-
+	ASCIIChartDlgUnmap(d);
 	gtk_window_set_modal(GTK_WINDOW(d->toplevel), FALSE);
 	gtk_window_set_transient_for(GTK_WINDOW(d->toplevel), NULL);
-	GTK_WIDGET_DESTROY(d->toplevel);
 
+	d->freeze_count++;
+
+	GTK_WIDGET_DESTROY(d->toplevel);
 	GTK_ACCEL_GROUP_UNREF(d->accelgrp);
 
+	d->freeze_count--;
+
 	g_free(d);
 }
