@@ -1,13 +1,14 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <time.h>
 #include <signal.h>
-#include <unistd.h>
-
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-
-#include "../../include/disk.h"
+#include <unistd.h>
 
 #include "../guiutils.h"
 #include "../fprompt.h"
@@ -48,6 +49,7 @@ typedef struct {
 			*menu_bar_handle,
 			*cut_mi,
 			*copy_mi,
+			*paste_mi,
 			*delete_mi,
 			*edit_mode_hex_mi,
 			*edit_mode_ascii_mi,
@@ -62,6 +64,7 @@ typedef struct {
 
 	guint8		*last_find_needle;
 	gint		last_find_needle_len;
+	gboolean	last_find_case_sensitive;
 
 	ascii_chart_dlg_struct	*ac_dlg;
 
@@ -72,7 +75,8 @@ typedef struct {
 
 
 static void HViewSignalCB(int s);
-static gint HEditCloseCB(GtkWidget *widget, GdkEvent *event, gpointer data);
+static gint HEditDeleteEventCB(GtkWidget *widget, GdkEvent *event, gpointer data);
+static gint HEditKeyEventCB(GtkWidget *widget, GdkEventKey *key, gpointer data);
 static void HEditHViewChangedCB(hview_struct *hv, gpointer data);
 static void HEditHViewSelectCB(
 	hview_struct *hv, int start_pos, int end_pos, gpointer data
@@ -89,7 +93,7 @@ static void HEditNewCB(GtkWidget *widget, gpointer data);
 static void HEditOpenCB(GtkWidget *widget, gpointer data);
 static void HEditSaveCB(GtkWidget *widget, gpointer data);
 static void HEditSaveAsCB(GtkWidget *widget, gpointer data);
-static void HEditExitCB(GtkWidget *widget, gpointer data);
+static void HEditCloseCB(GtkWidget *widget, gpointer data);
 static void HEditCutCB(GtkWidget *widget, gpointer data);
 static void HEditCopyCB(GtkWidget *widget, gpointer data);
 static void HEditPasteCB(GtkWidget *widget, gpointer data);
@@ -133,10 +137,13 @@ static void HEditUpdate(hedit_struct *he);
 
 
 #define DO_CHECK_CHANGES_AND_QUERY_USER(_he_,_hv_) {	\
+							\
+ HViewSetBusy((_hv_), TRUE);				\
+							\
  /* Check if current data has modifications and		\
   * if so, query user to save first			\
   */							\
- if(((_hv_) != NULL) ? ((_hv_)->flags & HVIEW_FLAG_HAS_CHANGES) : FALSE) { \
+ if((_hv_)->flags & HVIEW_FLAG_HAS_CHANGES) {		\
   gint response;					\
 							\
   EDVPlaySoundQuestion((_he_)->ctx);			\
@@ -157,19 +164,24 @@ saved, do you want to save those changes?", \
    case CDIALOG_RESPONSE_YES_TO_ALL:			\
    case CDIALOG_RESPONSE_YES:				\
     HEditSaveCB(NULL, data);				\
-    /* Check if changes were not saved? */		\
-    if((_hv_)->flags & HVIEW_FLAG_HAS_CHANGES)		\
+    /* Check if the changes were not saved? */		\
+    if((_hv_)->flags & HVIEW_FLAG_HAS_CHANGES) {	\
+     HViewSetBusy((_hv_), FALSE);			\
      return;						\
+    }							\
     break;						\
 							\
    case CDIALOG_RESPONSE_NO:				\
     break;						\
 							\
    default:						\
+    HViewSetBusy((_hv_), FALSE);			\
     return;						\
     break;						\
   }							\
  }							\
+							\
+ HViewSetBusy((_hv_), FALSE);				\
 }
 
 
@@ -191,12 +203,51 @@ static void HViewSignalCB(int s)
 /*
  *	Hex Editor toplevel GtkWindow "delete_event" callback.
  */
-static gint HEditCloseCB(GtkWidget *widget, GdkEvent *event, gpointer data)
+static gint HEditDeleteEventCB(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
-	HEditExitCB(widget, data);
-	return(TRUE);
+	gint status = FALSE;
+	hedit_struct *he = HEDIT(data);
+	if((widget == NULL) || (he == NULL))
+	    return(status);
+
+	if(he->freeze_count > 0)
+	    return(status);
+
+	HEditCloseCB(widget, data);
+	status = TRUE;
+
+	return(status);
 }
 
+/*
+ *	Hex Editor toplevel GtkWindow "key_press_event" or
+ *	"key_release_event" signal callback.
+ */
+static gint HEditKeyEventCB(GtkWidget *widget, GdkEventKey *key, gpointer data)
+{
+	gint status = FALSE;
+	gboolean press;
+	hedit_struct *he = HEDIT(data);
+	if((widget == NULL) || (key == NULL) || (he == NULL))
+	    return(status);
+
+	if(he->freeze_count > 0)
+	    return(status);
+
+	press = (key->type == GDK_KEY_PRESS) ? TRUE : FALSE;
+
+	switch(key->keyval)
+	{
+	  case GDK_F3:
+	    if(press)
+		HEditOpenCB(widget, data);
+	    status = TRUE;
+	    break;
+
+	}
+
+	return(status);
+}
 
 /*
  *	Hex View changed callback.
@@ -266,10 +317,8 @@ static void HEditDNDDragReceivedCB(
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
-	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv)
+	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv);
 
 	HViewSetBusy(hv, TRUE);
 
@@ -294,20 +343,23 @@ static void HEditDNDDragReceivedCB(
 		 * it and get the absolute path portion of that URL
 		 * string
 		 */
-		s = strstr(url, "://");
+		s = (const gchar *)strstr((const char *)url, "://");
 		if(s != NULL)
 		    s += STRLEN("://");
 		else
 		    s = url;
 
-		s = strchr(s, '/');
+		s = (const gchar *)strchr((const char *)s, '/');
 		path = STRDUP(s);
 
 		/* Got absolute path to object to be loaded? */
 		if(path != NULL)
 		{
-		    g_free(he->filename);
-		    he->filename = STRDUP(path);
+		    if(path != he->filename)
+		    {
+			g_free(he->filename);
+			he->filename = STRDUP(path);
+		    }
 		    HViewOpenFileProgress(
 			hv, he->filename,
 			HEditHViewOpenProgressCB, he
@@ -338,10 +390,8 @@ static void HEditNewCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
-	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv)
+	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv);
 
 	HViewSetBusy(hv, TRUE);
 
@@ -361,12 +411,11 @@ static void HEditNewCB(GtkWidget *widget, gpointer data)
  */
 static void HEditOpenCB(GtkWidget *widget, gpointer data)
 {
-	gboolean status;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
 	GtkWidget *toplevel;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	hview_struct *hv;
 	hedit_struct *he = HEDIT(data);
 	if((he == NULL) || FileBrowserIsQuery())
@@ -377,47 +426,153 @@ static void HEditOpenCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
-	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv)
+	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv);
 
 	HViewSetBusy(hv, TRUE);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
-	/* Query user for file to open */
+	/* Query the user for the file to open */
+	he->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 	    "Open File",
 	    "Open", "Cancel",
-	    NULL,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    he->filename,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	he->freeze_count--;
 
 	/* Got user response? */
-	if(status)
+	if(response)
 	{
-	    const gchar *new_path = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *new_path = (npaths > 0) ?
+		paths_list[0] : NULL;
+	    if(!STRISEMPTY(new_path))
+	    {
+		struct stat stat_buf;
+		gboolean file_exists;
 
-	    g_free(he->filename);
-	    he->filename = STRDUP(new_path);
-	    HViewOpenFileProgress(
-		hv, he->filename,
-		HEditHViewOpenProgressCB, he
-	    );
+		/* Check if the file exists */
+		if(stat(new_path, &stat_buf))
+		{
+		    const gint error_code = (gint)errno;
+		    if(error_code == ENOENT)
+			file_exists = FALSE;
+		    else
+			file_exists = TRUE;
+		}
+		else
+		{
+		    file_exists = TRUE;
+		}
+		/* Does the file exist? */
+		if(file_exists)
+		{
+		    /* File exists, open it */
+		    if(new_path != he->filename)
+		    {
+			g_free(he->filename);
+			he->filename = STRDUP(new_path);
+		    }
+		    HViewOpenFileProgress(
+			hv, he->filename,
+			HEditHViewOpenProgressCB, he
+		    );
+		}
+		else
+		{
+		    /* File does not exist, prompt the user to
+		     * to create a new file
+		     */
+		    gint response;
+		    gchar *msg = g_strdup_printf(
+"The file \"%s\" does not exist,\n\
+do you want to create it?",
+			g_basename(new_path)
+		    );
+		    EDVPlaySoundQuestion(he->ctx);
+		    CDialogSetTransientFor(toplevel);
+		    response = CDialogGetResponse(
+			"Confirm Create",
+			msg,
+			NULL,
+			CDIALOG_ICON_QUESTION,
+			CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+			CDIALOG_BTNFLAG_NO
+		    );
+		    g_free(msg);
+		    CDialogSetTransientFor(NULL);
+		    if(response == CDIALOG_RESPONSE_YES)
+		    {
+			/* Create the new file */
+			const guint m = EDVGetUMask();
+			const gint fd = (gint)open(
+			    (const char *)new_path,
+			    O_WRONLY | O_CREAT | O_EXCL,
+			    (mode_t)(~m) &
+				(S_IRUSR | S_IWUSR |
+				 S_IRGRP | S_IWGRP |
+				 S_IROTH | S_IWOTH)
+			);
+			if(fd > -1)
+			{
+			    close((int)fd);
+
+			    /* Set the new file name */
+			    if(new_path != he->filename)
+			    {
+				g_free(he->filename);
+				he->filename = STRDUP(new_path);
+			    }
+
+			    /* Notify about the file being created */
+			    EDVNotifyQueueObjectAdded(he->ctx, he->filename);
+			    EDVContextSync(he->ctx);
+
+			    /* Open the new file */
+			    HViewOpenFileProgress(
+				hv, he->filename,
+				HEditHViewOpenProgressCB, he
+			    );
+			}
+			else
+			{
+			    const gint error_code = (gint)errno;
+			    gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+				g_strerror(error_code), new_path
+			    );
+			    EDVPlaySoundError(he->ctx);
+			    CDialogSetTransientFor(toplevel);
+			    CDialogGetResponse(
+				"Create Failed",
+				msg,
+				NULL,
+				CDIALOG_ICON_ERROR,
+				CDIALOG_BTNFLAG_OK,
+				CDIALOG_BTNFLAG_OK
+			    );
+			    g_free(msg);
+			    CDialogSetTransientFor(NULL);
+			}
+		    }
+		}	/* Does the file exist? */
+	    }
 	}
 
 	/* Delete the file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	HEditUpdate(he);
 	HViewSetBusy(hv, FALSE);
@@ -439,47 +594,50 @@ static void HEditSaveCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
-	/* If there is no filename to save to then call save as callback
-	 * so user can be queried for a file name
+	HViewSetBusy(hv, TRUE);
+
+	/* If there was no filename to save to then call the save as
+	 * callback so user can be prompted for a file name
 	 */
 	if(he->filename == NULL)
 	{
 	    HEditSaveAsCB(widget, data);
+	    HViewSetBusy(hv, FALSE);
 	    return;
 	}
 
-	HViewSetBusy(hv, TRUE);
-
 	/* Save the data to file */
 	if(HViewSaveFileProgress(
 	    hv, he->filename,
 	    HEditHViewSaveProgressCB, he
 	))
 	{
-	    /* Error occured while saving file */
-	    gchar *buf = g_strdup_printf(
-"An error occured while saving the file:\n\n    %s",
+	    /* Error occured while saving the file */
+	    gchar *msg = g_strdup_printf(
+"An error occured while saving the file:\n\
+\n\
+    %s",
 		he->filename
 	    );
 	    EDVPlaySoundError(he->ctx);
 	    CDialogSetTransientFor(toplevel);
 	    CDialogGetResponse(
 		"Save Error",
-		buf,
+		msg,
 		NULL,
 		CDIALOG_ICON_ERROR,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
+	    g_free(msg);
 	    CDialogSetTransientFor(NULL);
-	    g_free(buf);
 	}
 	else
 	{
-	    /* File saved successfully */
+	    /* File was saved successfully, notify about the file
+	     * being saved
+	     */
 	    EDVNotifyQueueObjectModified(
 		he->ctx, he->filename, NULL
 	    );
@@ -497,12 +655,11 @@ static void HEditSaveCB(GtkWidget *widget, gpointer data)
  */
 static void HEditSaveAsCB(GtkWidget *widget, gpointer data)
 {
-	gboolean status;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
 	GtkWidget *toplevel;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	hview_struct *hv;
 	hedit_struct *he = HEDIT(data);
 	if((he == NULL) || FileBrowserIsQuery())
@@ -513,67 +670,80 @@ static void HEditSaveAsCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
 	HViewSetBusy(hv, TRUE);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
-	/* Query user for file to save */
+	/* Query the user for the file to save */
+	he->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 	    "Save File As",
 	    "Save", "Cancel",
-	    NULL,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    he->filename,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	he->freeze_count--;
 
 	/* Got user response? */
-	if(status)
+	if(response)
 	{
 	    gboolean file_exists = FALSE;
-	    const gchar *new_path = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *new_path = (npaths > 0) ?
+		paths_list[0] : NULL;
 
 	    if(new_path != NULL)
 	    {
+		struct stat stat_buf;
 		gboolean allow_save = TRUE;
 
-		/* Check if file exists */
-		file_exists = access(new_path, F_OK) ? FALSE : TRUE;
+		/* Check if the file exists */
+		if(stat((const char *)new_path, &stat_buf))
+		{
+		    const gint error_code = (gint)errno;
+		    if(error_code == ENOENT)
+			file_exists = FALSE;
+		    else
+			file_exists = TRUE;
+		}
+		else
+		{
+		    file_exists = TRUE;
+		}
 		if(file_exists)
 		{
 		    gint response;
-		    gchar *buf = g_strdup_printf(
-"Overwrite existing file:\n\n    %s\n",
+		    gchar *msg = g_strdup_printf(
+"Overwrite existing file:\n\
+\n\
+    %s",
 			new_path
 		    );
 		    EDVPlaySoundWarning(he->ctx);
 		    CDialogSetTransientFor(toplevel);
 		    response = CDialogGetResponse(
 			"Confirm Overwrite",
-			buf,
+			msg,
 			NULL,
 			CDIALOG_ICON_WARNING,
 			CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
 			CDIALOG_BTNFLAG_YES
 		    );
+		    g_free(msg);
 		    CDialogSetTransientFor(NULL);
-		    g_free(buf);
 		    switch(response)
 		    {
 		      case CDIALOG_RESPONSE_YES_TO_ALL:
 		      case CDIALOG_RESPONSE_YES:
 			break;
-
 		      default:
 			allow_save = FALSE;
 			break;
@@ -583,37 +753,44 @@ static void HEditSaveAsCB(GtkWidget *widget, gpointer data)
 		/* File not exist or user permits ovewrite? */
 		if(allow_save)
 		{
-		    /* Update filename */
-		    g_free(he->filename);
-		    he->filename = STRDUP(new_path);
+		    /* Update the filename */
+		    if(new_path != he->filename)
+		    {
+			g_free(he->filename);
+			he->filename = STRDUP(new_path);
+		    }
 
-		    /* Save data to new file name */
+		    /* Save the data to the new file name */
 		    if(HViewSaveFileProgress(
 			hv, he->filename,
 			HEditHViewSaveProgressCB, he
 		    ))
 		    {
 			/* Error occured while saving file */
-			gchar *buf = g_strdup_printf(
-"An error occured while saving the file:\n\n    %s",
+			gchar *msg = g_strdup_printf(
+"An error occured while saving the file:\n\
+\n\
+    %s",
 			    he->filename
 			);
 			EDVPlaySoundError(he->ctx);
 			CDialogSetTransientFor(toplevel);
 			CDialogGetResponse(
 			    "Save Error",
-			    buf,
+			    msg,
 			    NULL,
 			    CDIALOG_ICON_ERROR,
 			    CDIALOG_BTNFLAG_OK,
 			    CDIALOG_BTNFLAG_OK
 			);
+			g_free(msg);
 			CDialogSetTransientFor(NULL);
-			g_free(buf);
 		    }
 		    else
 		    {
-			/* File saved successfully */
+			/* File was saved successfully, notify about
+			 * the file being saved
+			 */
 			if(file_exists)
 			    EDVNotifyQueueObjectModified(
 				he->ctx, he->filename, NULL
@@ -631,7 +808,7 @@ static void HEditSaveAsCB(GtkWidget *widget, gpointer data)
 	}	/* Got user response? */
 
 	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	/* Reset due to possible file related change */
 	FileBrowserReset();
@@ -641,9 +818,9 @@ static void HEditSaveAsCB(GtkWidget *widget, gpointer data)
 }
 
 /*
- *	Exit callback.
+ *	Close callback.
  */
-static void HEditExitCB(GtkWidget *widget, gpointer data)
+static void HEditCloseCB(GtkWidget *widget, gpointer data)
 {
 	GtkWidget *toplevel;
 	hview_struct *hv;
@@ -656,10 +833,8 @@ static void HEditExitCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
-	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv)
+	DO_CHECK_CHANGES_AND_QUERY_USER(he, hv);
 
 	gtk_main_quit();
 }
@@ -787,8 +962,6 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
 	HViewSetBusy(hv, TRUE);
 
@@ -809,6 +982,7 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
  	PDialogAddPrompt(NULL, "Find:", last_needle);
 	g_free(last_needle);
 	last_needle = NULL;
+	PDialogAddPromptToggle(NULL, "Case Sensitive", he->last_find_case_sensitive);
 	PDialogSetSize(320, -1);
 	PDialogSetTransientFor(toplevel);
 	strv = PDialogGetResponse(
@@ -821,25 +995,28 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
 	);
 	PDialogSetTransientFor(NULL);
 
-	if((strv != NULL) && (strc > 0))
+	if((strv != NULL) && (strc >= 2))
 	{
 	    gint i;
-	    const gchar *s = strv[0];
-	    if(!STRISEMPTY(s))
+	    const gchar *new_needle = strv[0];
+	    const gboolean case_sensitive = ATOI(strv[1]) ? TRUE : FALSE;
+
+	    if(!STRISEMPTY(new_needle))
 	    {
 		/* Need to convert the user given string from ASCII to
 		 * hex and update the last find needle
 		 */
 		g_free(he->last_find_needle);
-		he->last_find_needle = (guint8 *)STRDUP(s);
-		he->last_find_needle_len = STRLEN(s);
+		he->last_find_needle = (guint8 *)STRDUP(new_needle);
+		he->last_find_needle_len = STRLEN(new_needle);
+		he->last_find_case_sensitive = case_sensitive;
 
 		/* Do search */
 		i = HViewFind(
 		    hv,
 		    he->last_find_needle, he->last_find_needle_len,
 		    hv->buf_pos + 1,	/* Start index */
-		    FALSE		/* Not case sensitive */
+		    he->last_find_case_sensitive
 		);
 		/* Got match? */
 		if(i > -1)
@@ -850,7 +1027,7 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
 		    if(HViewIsBufferPositionVisible(hv, i) !=
 			GTK_VISIBILITY_FULL
 		    )
-			HViewScrollTo(hv, i, 0.5);
+			HViewScrollTo(hv, i, 0.5f);
 		}
 		else
 		{
@@ -861,7 +1038,7 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
 			hv,
 			he->last_find_needle, he->last_find_needle_len,
 			0,		/* Start index */
-			FALSE		/* Not case sensitive */
+			he->last_find_case_sensitive
 		    );
 		    /* Got match the second time? */
 		    if(i > -1)
@@ -872,21 +1049,29 @@ static void HEditFindCB(GtkWidget *widget, gpointer data)
 			if(HViewIsBufferPositionVisible(hv, i) !=
 			    GTK_VISIBILITY_FULL
 			)
-			    HViewScrollTo(hv, i, 0.5);
+			    HViewScrollTo(hv, i, 0.5f);
 		    }
 		    else
 		    {
 			/* No match */
-			HViewSetStatusMessage(
-			    hv,
-			    "String not found",
-			    FALSE
+			EDVPlaySoundInfo(he->ctx);
+			CDialogSetTransientFor(toplevel);
+			CDialogGetResponse(
+			    "Find",
+"The string that you entered was not found.",
+			    NULL,
+			    CDIALOG_ICON_INFO,
+			    CDIALOG_BTNFLAG_OK,
+			    CDIALOG_BTNFLAG_OK
 			);
+			CDialogSetTransientFor(NULL);
 		    }	/* Got match the second time? */
 		}	/* Got match? */
 	    }
 	}
 
+	PDialogDeleteAllPrompts();
+
 	HEditUpdate(he);
 	HViewSetBusy(hv, FALSE);
 }
@@ -907,8 +1092,6 @@ static void HEditFindNextCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
 	if(he->last_find_needle == NULL)
 	    return;
@@ -922,7 +1105,7 @@ static void HEditFindNextCB(GtkWidget *widget, gpointer data)
 		hv,
 		he->last_find_needle, he->last_find_needle_len,
 		hv->buf_pos + 1,	/* Start index */
-		FALSE			/* Not case sensitive */
+		he->last_find_case_sensitive
 	    );
 	    /* Got match? */
 	    if(i > -1)
@@ -933,7 +1116,7 @@ static void HEditFindNextCB(GtkWidget *widget, gpointer data)
 		if(HViewIsBufferPositionVisible(hv, i) !=
 		    GTK_VISIBILITY_FULL
 		)
-		    HViewScrollTo(hv, i, 0.5);
+		    HViewScrollTo(hv, i, 0.5f);
 	    }
 	    else
 	    {
@@ -944,7 +1127,7 @@ static void HEditFindNextCB(GtkWidget *widget, gpointer data)
 		    hv,
 		    he->last_find_needle, he->last_find_needle_len,
 		    0,		/* Start index */
-		    FALSE	/* Not case sensitive */
+		    he->last_find_case_sensitive
 		);
 		/* Got match the second time? */
 		if(i > -1)
@@ -955,16 +1138,22 @@ static void HEditFindNextCB(GtkWidget *widget, gpointer data)
 		    if(HViewIsBufferPositionVisible(hv, i) !=
 			GTK_VISIBILITY_FULL
 		    )
-			HViewScrollTo(hv, i, 0.5);
+			HViewScrollTo(hv, i, 0.5f);
 		}
 		else
 		{
 		    /* No match */
-		    HViewSetStatusMessage(
-			hv,
-			"String not found",
-			FALSE
+		    EDVPlaySoundInfo(he->ctx);
+		    CDialogSetTransientFor(toplevel);
+		    CDialogGetResponse(
+			"Find",
+"The string that you entered was not found.",
+			NULL,
+			CDIALOG_ICON_INFO,
+			CDIALOG_BTNFLAG_OK,
+			CDIALOG_BTNFLAG_OK
 		    );
+		    CDialogSetTransientFor(NULL);
 		}	/* Got match the second time? */
 	    }		/* Got match? */
 	}
@@ -989,10 +1178,9 @@ static void HEditEditModeHexCB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
 	HViewSetEditMode(hv, HVIEW_EDIT_MODE_HEX);
+	HEditUpdate(he);
 }
 
 /*
@@ -1011,10 +1199,9 @@ static void HEditEditModeASCIICB(GtkWidget *widget, gpointer data)
 
 	toplevel = he->toplevel;
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
 
 	HViewSetEditMode(hv, HVIEW_EDIT_MODE_ASCII);
+	HEditUpdate(he);
 }
 
 /*
@@ -1129,7 +1316,7 @@ static void HEditCalculateSelectedCB(GtkWidget *widget, gpointer data)
 
 	if(TRUE)
 	{
-	    gchar *buf = g_strdup_printf(
+	    gchar *msg = g_strdup_printf(
 "Position: %i\n\
 8 Bit Value: %i  Unsigned: %i  0x%.2X\n\
 16 Bit Value: %i  Unsigned: %i  0x%.4X\n\
@@ -1163,14 +1350,14 @@ Swapped Values:\n\
 	    CDialogSetTransientFor(toplevel);
 	    CDialogGetResponse(
 		"Results",
-		buf,
+		msg,
 		NULL,
 		CDIALOG_ICON_INFO,
 		CDIALOG_BTNFLAG_OK,
 		CDIALOG_BTNFLAG_OK
 	    );
+	    g_free(msg);
 	    CDialogSetTransientFor(NULL);
-	    g_free(buf);
 	}
 
 
@@ -1408,7 +1595,7 @@ static void HEditBuildMenuBar(hedit_struct *he, GtkWidget *parent)
 	    ;
 	    accel_key = GDK_w;
 	    accel_mods = GDK_CONTROL_MASK;
-	    func_cb = HEditExitCB;
+	    func_cb = HEditCloseCB;
 	    DO_ADD_MENU_ITEM_LABEL
 
 	}
@@ -1514,6 +1701,7 @@ static void HEditBuildMenuBar(hedit_struct *he, GtkWidget *parent)
 	    accel_mods = GDK_CONTROL_MASK;
 	    func_cb = HEditPasteCB;
 	    DO_ADD_MENU_ITEM_LABEL
+	    he->paste_mi = w;
 
 	    DO_ADD_MENU_SEP
 
@@ -1590,16 +1778,16 @@ static void HEditBuildMenuBar(hedit_struct *he, GtkWidget *parent)
 
 	    icon = NULL;
 	    label = "Select All";
-	    accel_key = GDK_a;
-	    accel_mods = GDK_CONTROL_MASK;
+	    accel_key = 0;
+	    accel_mods = 0;
 	    func_cb = HEditSelectAllCB;
 	    DO_ADD_MENU_ITEM_LABEL
 	    he->select_all_mi = w;
 
 	    icon = NULL;
 	    label = "Unselect All";
-	    accel_key = GDK_u;
-	    accel_mods = GDK_CONTROL_MASK;
+	    accel_key = 0;
+	    accel_mods = 0;
 	    func_cb = HEditUnselectAllCB;
 	    DO_ADD_MENU_ITEM_LABEL
 	    he->unselect_all_mi = w;
@@ -1806,7 +1994,7 @@ static void HEditBuildMenuBar(hedit_struct *he, GtkWidget *parent)
  */
 static void HEditUpdate(hedit_struct *he)
 {
-	gboolean sensitive;
+	gboolean sensitive, has_changes;
 	GtkWidget *w;
 	hview_struct *hv;
 
@@ -1816,32 +2004,46 @@ static void HEditUpdate(hedit_struct *he)
 	he->freeze_count++;
 
 	hv = he->hv;
-	if(hv == NULL)
-	    return;
+	has_changes = HVIEW_HAS_CHANGES(hv);
 
 	w = he->toplevel;
 	if(w != NULL)
 	{
-	    const gchar *s = (he->filename != NULL) ?
-		strrchr(he->filename, DIR_DELIMINATOR) : NULL;
-	    gchar *title = g_strdup_printf(
-		PROG_NAME_FULL ": %s%s",
-		(s != NULL) ? s + 1 :
-		    ((he->filename != NULL) ? he->filename : "Untitled"),
-		HVIEW_HAS_CHANGES(hv) ? " (*)" : ""
+	    const gchar *name;
+	    gchar *s;
+
+	    if(he->filename != NULL)
+	    {
+		name = (const gchar *)strrchr(
+		    (const char *)he->filename, G_DIR_SEPARATOR
+		);
+		if(name != NULL)
+		    name++;
+		else
+		    name = he->filename;
+	    }
+	    else
+	    {
+		name = "Untitled";
+	    }
+
+	    s = g_strconcat(
+		PROG_NAME_FULL,
+		": ",
+		name,
+		(has_changes) ? " (*)" : "",
+		NULL
 	    );
-	    gtk_window_set_title(GTK_WINDOW(w), title);
-	    g_free(title);
+	    gtk_window_set_title(GTK_WINDOW(w), s);
+	    g_free(s);
 	}
 
 	sensitive = ((hv->buf_sel_start >= 0) && (hv->buf_sel_end >= 0)) ?
 	    TRUE : FALSE;
-	w = he->cut_mi;
-	if(w != NULL)
-	    gtk_widget_set_sensitive(w, sensitive);
-	w = he->copy_mi;
-	if(w != NULL)
-	    gtk_widget_set_sensitive(w, sensitive);
+	gtk_widget_set_sensitive(he->cut_mi, sensitive);
+	gtk_widget_set_sensitive(he->copy_mi, sensitive);
+	sensitive = TRUE;
+	gtk_widget_set_sensitive(he->paste_mi, sensitive);
 
 	GTK_CHECK_MENU_ITEM_SET_ACTIVE(
 	    he->edit_mode_hex_mi,
@@ -1862,6 +2064,7 @@ int main(int argc, char *argv[])
 	gint i;
 	gint		cursor_pos = 0;
 	const gchar	*arg,
+			*view_font_name = NULL,
 			*filename = NULL;
 	GdkWindow *window;
 	GtkWidget *w;
@@ -1869,7 +2072,9 @@ int main(int argc, char *argv[])
 	hview_struct *hv;
 	edv_context_struct *ctx;
 
-#define DO_FREE_LOCALS	{		\
+#define CLEANUP_RETURN(_v_)	{	\
+					\
+ return(_v_);				\
 }
 
 	/* Set up signal callbacks */
@@ -1899,35 +2104,33 @@ int main(int argc, char *argv[])
 	    if(arg == NULL)
 		continue;
 
-	    /* Help? */
-	    if(!strcasecmp(arg, "--help") ||
-	       !strcasecmp(arg, "-help") ||
-	       !strcasecmp(arg, "--h") ||
-	       !strcasecmp(arg, "-h") ||
-	       !strcasecmp(arg, "-?")
+	    /* Help */
+	    if(!g_strcasecmp(arg, "--help") ||
+	       !g_strcasecmp(arg, "-help") ||
+	       !g_strcasecmp(arg, "--h") ||
+	       !g_strcasecmp(arg, "-h") ||
+	       !g_strcasecmp(arg, "-?")
 	    )
 	    {
-		g_print(PROG_HELP_MESG);
-		DO_FREE_LOCALS
-		return(0);
+		g_print("%s", PROG_HELP_MESG);
+		CLEANUP_RETURN(0);
 	    }
 	    /* Version? */
-	    else if(!strcasecmp(arg, "--version") ||
-		    !strcasecmp(arg, "-version")
+	    else if(!g_strcasecmp(arg, "--version") ||
+		    !g_strcasecmp(arg, "-version")
 	    )
 	    {
 		g_print(
 		    "%s %s\n%s",
 		    PROG_NAME_FULL, PROG_VERSION, PROG_COPYRIGHT
 		);
-		DO_FREE_LOCALS
-		return(0);
+		CLEANUP_RETURN(0);
 	    }
-	    /* Initial cursor position? */
-	    else if(!strcasecmp(arg, "--address") ||
-		    !strcasecmp(arg, "-address") ||
-		    !strcasecmp(arg, "--a") ||
-		    !strcasecmp(arg, "-a")
+	    /* Address */
+	    else if(!g_strcasecmp(arg, "--address") ||
+		    !g_strcasecmp(arg, "-address") ||
+		    !g_strcasecmp(arg, "--a") ||
+		    !g_strcasecmp(arg, "-a")
 	    )
 	    {
 		i++;
@@ -1942,8 +2145,29 @@ int main(int argc, char *argv[])
 			"%s: Requires argument\n",
 			argv[i - 1]
 		    );
-		    DO_FREE_LOCALS
-		    return(2);
+		    CLEANUP_RETURN(2);
+		}
+	    }
+	    /* View Font */
+	    else if(!g_strcasecmp(arg, "--view-font-name") ||
+		    !g_strcasecmp(arg, "-view-font-name") ||
+		    !g_strcasecmp(arg, "--view-font") ||
+		    !g_strcasecmp(arg, "-view-font")
+	    )
+	    {
+		i++;
+		arg = (i < argc) ? argv[i] : NULL;
+		if(arg != NULL)
+		{
+		    view_font_name = arg;
+		}
+		else
+		{
+		    g_printerr(
+			"%s: Requires argument\n",
+			argv[i - 1]
+		    );
+		    CLEANUP_RETURN(2);
 		}
 	    }
 	    /* Skip these arguments so that gtk_window_apply_args()
@@ -1973,8 +2197,7 @@ int main(int argc, char *argv[])
 "-%c: Unsupported argument.\n",
 			    c
 			);
-			DO_FREE_LOCALS
-			return(2);
+			CLEANUP_RETURN(2);
 		    }
 #endif
 		    v++;
@@ -1991,8 +2214,7 @@ int main(int argc, char *argv[])
 "%s: Unsupported argument.\n",
 		    arg
 		);
-		DO_FREE_LOCALS
-		return(2);
+		CLEANUP_RETURN(2);
 	    }
 	}
 
@@ -2005,8 +2227,7 @@ int main(int argc, char *argv[])
 	    if(!gtk_init_check(&argc, &argv))
 	    {
 		g_printerr("Unable to initialize GTK.\n");
-		DO_FREE_LOCALS
-		return(1);
+		CLEANUP_RETURN(1);
 	    }
 	    initialized_gtk = TRUE;
 	}
@@ -2026,23 +2247,43 @@ int main(int argc, char *argv[])
 
 
 	/* Begin creating the HEdit window */
-	he = HEDIT(
-	    g_malloc0(sizeof(hedit_struct))
-	);
+	he = HEDIT(g_malloc0(sizeof(hedit_struct)));
+	he->toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	he->accelgrp = gtk_accel_group_new();
 	he->freeze_count = 0;
 	he->ac_dlg = NULL;
 	he->ctx = ctx;
+	he->filename = NULL;
+	he->last_find_needle = NULL;
+	he->last_find_needle_len = 0;
+	he->last_find_case_sensitive = FALSE;
 
-	/* Toplevel */
-	he->toplevel = w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	he->freeze_count++;
+
+	/* Toplevel GtkWindow */
+	w = he->toplevel;
+	gtk_widget_set_name(w, HEDIT_TOPLEVEL_WIDGET_NAME);
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, HEDIT_WIDTH, HEDIT_HEIGHT);
 	gtk_window_set_wmclass(
 	    GTK_WINDOW(w), "hexeditor", PROG_NAME
 	);
-	gtk_window_set_policy(
-	    GTK_WINDOW(w),
-	    TRUE, TRUE, TRUE
+	gtk_widget_add_events(
+	    w,
+	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
+	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "delete_event",
+	    GTK_SIGNAL_FUNC(HEditDeleteEventCB), he
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "key_press_event",
+	    GTK_SIGNAL_FUNC(HEditKeyEventCB), he
+	);
+	gtk_signal_connect(
+	    GTK_OBJECT(w), "key_release_event",
+	    GTK_SIGNAL_FUNC(HEditKeyEventCB), he
 	);
 	gtk_widget_realize(w);
 	window = w->window;
@@ -2096,53 +2337,160 @@ int main(int argc, char *argv[])
 	HEditBuildMenuBar(he, he->menu_bar_handle);
 
 	he->hv = hv = HViewNew(he->main_vbox);
+	gtk_widget_set_name(hv->view_da, HEDIT_HVIEW_WIDGET_NAME);
+	if(view_font_name != NULL)
+	    HViewSetViewFont(hv, view_font_name);
 	HViewSetChangedCB(hv, HEditHViewChangedCB, he);
 	HViewSetSelectCB(hv, HEditHViewSelectCB, he);
 	HViewSetEditModeChangedCB(hv, HEditHViewEditModeChangedCB, he);
 	HViewMap(hv);
-	gtk_signal_connect(
-	    GTK_OBJECT(he->toplevel), "delete_event",
-	    GTK_SIGNAL_FUNC(HEditCloseCB), he
-	);
+
 	/* Set up the HexView for DND */
 	w = hv->view_da;
 	if(w != NULL)
 	{
 	    const GtkTargetEntry dnd_tar_types[] = {
-{"text/plain",          0,              HEDIT_DND_TYPE_INFO_TEXT_PLAIN},
-{"text/uri-list",       0,              HEDIT_DND_TYPE_INFO_TEXT_URI_LIST},
-{"STRING",              0,              HEDIT_DND_TYPE_INFO_STRING}
+{GUI_TARGET_NAME_TEXT_PLAIN,	0,	HEDIT_DND_TYPE_INFO_TEXT_PLAIN},
+{GUI_TARGET_NAME_TEXT_URI_LIST,	0,	HEDIT_DND_TYPE_INFO_TEXT_URI_LIST},
+{GUI_TARGET_NAME_STRING,	0,	HEDIT_DND_TYPE_INFO_STRING}
 	    };
 	    GUIDNDSetTar(
 		w,
 		dnd_tar_types,
 		sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
-		GDK_ACTION_DEFAULT |            /* Drag actions */
-		GDK_ACTION_LINK |
-		GDK_ACTION_MOVE |
-		GDK_ACTION_COPY,
-		GDK_ACTION_MOVE,                /* Def act if same */
-		GDK_ACTION_COPY,                /* Def act */
+		GDK_ACTION_COPY,		/* Drag actions */
+		GDK_ACTION_MOVE,                /* Default action if same */
+		GDK_ACTION_COPY,                /* Default action */
 		HEditDNDDragReceivedCB,
 		he
 	    );
 	}
+
 	HEditUpdate(he);
+
+	he->freeze_count--;
+
 	gtk_widget_show_raise(he->toplevel);
 	gtk_widget_grab_focus(hv->view_da);
 
-	if(filename != NULL)
+	/* Open file at startup? */
+	if(!STRISEMPTY(filename))
 	{
+	    struct stat stat_buf;
+	    gboolean file_exists;
+	    GtkWidget *toplevel = he->toplevel;
+
 	    HViewSetBusy(hv, TRUE);
 
-	    g_free(he->filename);
-	    he->filename = STRDUP(filename);
-	    HViewOpenFileProgress(
-		hv, he->filename,
-		HEditHViewOpenProgressCB, he
-	    );
+	    /* Check if the file exists */
+	    if(stat(filename, &stat_buf))
+	    {
+		const gint error_code = (gint)errno;
+		if(error_code == ENOENT)
+		    file_exists = FALSE;
+		else
+		    file_exists = TRUE;
+	    }
+	    else
+	    {
+		file_exists = TRUE;
+	    }
+	    /* Does the file exist? */
+	    if(file_exists)
+	    {
+		/* File exists, open it */
+		if(filename != he->filename)
+		{
+		    g_free(he->filename);
+		    he->filename = STRDUP(filename);
+		}
+		HViewOpenFileProgress(
+		    hv, he->filename,
+		    HEditHViewOpenProgressCB, he
+		);
+	    }
+	    else
+	    {
+		/* File does not exist, prompt the user to create
+		 * a new file
+		 */
+		gint response;
+		gchar *msg = g_strdup_printf(
+"The file \"%s\" does not exist,\n\
+do you want to create it?",
+		    g_basename(filename)
+		);
+		EDVPlaySoundQuestion(he->ctx);
+		CDialogSetTransientFor(toplevel);
+		response = CDialogGetResponse(
+		    "Confirm Create",
+		    msg,
+		    NULL,
+		    CDIALOG_ICON_QUESTION,
+		    CDIALOG_BTNFLAG_YES | CDIALOG_BTNFLAG_NO,
+		    CDIALOG_BTNFLAG_NO
+		);
+		g_free(msg);
+		CDialogSetTransientFor(NULL);
+		if(response == CDIALOG_RESPONSE_YES)
+		{
+		    /* Create the new file */
+		    const guint m = EDVGetUMask();
+		    const gint fd = (gint)open(
+			(const char *)filename,
+			O_WRONLY | O_CREAT | O_EXCL,
+			(mode_t)(~m) &
+			    (S_IRUSR | S_IWUSR |
+			     S_IRGRP | S_IWGRP |
+			     S_IROTH | S_IWOTH)
+		    );
+		    if(fd > -1)
+		    {
+			close((int)fd);
 
-	    /* Set cursor position at startup? */
+			/* Set the new file name */
+			if(filename != he->filename)
+			{
+			    g_free(he->filename);
+			    he->filename = STRDUP(filename);
+			}
+
+			/* Notify about the file being created */
+			EDVNotifyQueueObjectAdded(he->ctx, he->filename);
+			EDVContextSync(he->ctx);
+
+			/* Open the new file */
+			HViewOpenFileProgress(
+			    hv, he->filename,
+			    HEditHViewOpenProgressCB, he
+			);
+		    }
+		    else
+		    {
+			const gint error_code = (gint)errno;
+			gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+			    g_strerror(error_code), filename
+			);
+			EDVPlaySoundError(he->ctx);
+			CDialogSetTransientFor(toplevel);
+			CDialogGetResponse(
+			    "Create Failed",
+			    msg,
+			    NULL,
+			    CDIALOG_ICON_ERROR,
+			    CDIALOG_BTNFLAG_OK,
+			    CDIALOG_BTNFLAG_OK
+			);
+			g_free(msg);
+			CDialogSetTransientFor(NULL);
+		    }
+		}
+	    }	/* Does the file exist? */
+
+	    /* Set the cursor position at startup? */
 	    if(cursor_pos > 0)
 	    {
 		if(cursor_pos >= hv->buf_len)
@@ -2170,20 +2518,19 @@ int main(int argc, char *argv[])
 	ASCIIChartDlgDelete(he->ac_dlg);
 	he->ac_dlg = NULL;
 
+	gtk_widget_hide(he->toplevel);
+
+	he->freeze_count++;
 	HViewClear(hv);
 	HViewDelete(hv);
-	he->hv = hv = NULL;
-
-	gtk_widget_destroy(he->menu_bar_handle);
-	he->menu_bar_handle = NULL;
 	gtk_widget_destroy(he->toplevel);
-	he->toplevel = NULL;
-
 	gtk_accel_group_unref(he->accelgrp);
-	he->accelgrp = NULL;
 
 	g_free(he->filename);
 	g_free(he->last_find_needle);
+
+	he->freeze_count--;
+
 	g_free(he);
 	he = NULL;
 
@@ -2202,8 +2549,6 @@ int main(int argc, char *argv[])
 	/* Reset the DND Icon */
 	GUIDNDSetDragIcon(NULL, NULL, 0, 0);
 
-	DO_FREE_LOCALS
-
-	return(0);
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN(0);
+#undef CLEANUP_RETURN
 }
