@@ -79,19 +79,20 @@ GtkMenu *HViewGetMenuWidget(hview_struct *hv);
 
 hview_struct *HViewNew(GtkWidget *parent);
 hview_struct *HViewNewWithToplevel(
-	gint width, gint height
+	const gint width, const gint height
 );
 void HViewShowHeading(hview_struct *hv, const gboolean show);
 void HViewShowStatusBar(hview_struct *hv, const gboolean show);
+void HViewSetViewFont(hview_struct *hv, const gchar *font_name);
 void HViewSetReadOnly(hview_struct *hv, const gboolean read_only);
 void HViewSetChangedCB(
 	hview_struct *hv,
-	void (*changed_cb)(hview_struct *, gpointer),
+	void (*func_cb)(hview_struct *, gpointer),
 	gpointer data
 );
 void HViewSetSelectCB(
 	hview_struct *hv,
-	void (*select_cb)(
+	void (*func_cb)(
 		hview_struct *,
 		int, int,
 		gpointer
@@ -100,7 +101,7 @@ void HViewSetSelectCB(
 );
 void HViewSetEditModeChangedCB(
 	hview_struct *hv,
-	void (*edit_mode_changed_cb)(
+	void (*func_cb)(
 		hview_struct *,
 		hview_edit_mode,                               
 		gpointer                        
@@ -109,12 +110,12 @@ void HViewSetEditModeChangedCB(
 ); 
 void HViewUpdate(hview_struct *hv);
 void HViewDraw(hview_struct *hv);
-void HViewQueueDraw(hview_struct *hv);  
-void HViewSetBusy(hview_struct *hv, gboolean is_busy);
+void HViewQueueDraw(hview_struct *hv);
 void HViewSetStatusMessage(
-	hview_struct *hv, const gchar *mesg,
-	gboolean allow_gtk_iteration
+	hview_struct *hv, const gchar *msg,
+	const gboolean allow_gtk_iteration
 );
+void HViewSetBusy(hview_struct *hv, const gboolean is_busy);
 void HViewMap(hview_struct *hv);
 void HViewUnmap(hview_struct *hv);
 void HViewDelete(hview_struct *hv);
@@ -137,6 +138,8 @@ void HViewDelete(hview_struct *hv);
 #define HVIEW_DEF_HEIGHT		480
 #define HVIEW_STATUS_BAR_HEIGHT		26
 
+#define HVIEW_DEF_VALUES_PER_ROW	16
+
 
 /*
  *	Returns the buffer position based on the given x and y
@@ -366,7 +369,11 @@ gint HViewOpenFileProgress(
 	    }
 	    else
 	    {
+#if defined(_WIN32)
+		read_buf_len = 1024l;
+#else
 		read_buf_len = (gulong)stat_buf.st_blksize;
+#endif
 		size = (gulong)stat_buf.st_size;
 	    }
 
@@ -571,7 +578,11 @@ gint HViewSaveFileProgress(
 	    }
 	    else
 	    {
+#if defined(_WIN32)
+		write_buf_len = 1024l;
+#else
 		write_buf_len = (gulong)stat_buf.st_blksize;
+#endif
 		size = (gulong)stat_buf.st_size;
 	    }
 
@@ -1137,14 +1148,11 @@ GtkMenu *HViewGetMenuWidget(hview_struct *hv)
 hview_struct *HViewNew(GtkWidget *parent)
 {
 	const gint border_minor = 2;
-	GdkFont *font;
 	GdkColormap *colormap;
 	GtkAdjustment *adj;
 	GdkWindow *window;
 	GtkWidget *w, *parent2, *parent3, *parent4, *parent5;
-	hview_struct *hv = HVIEW(
-	    g_malloc0(sizeof(hview_struct))
-	);
+	hview_struct *hv = HVIEW(g_malloc0(sizeof(hview_struct)));
 	if(hv == NULL)
 	    return(NULL);
 
@@ -1168,23 +1176,23 @@ hview_struct *HViewNew(GtkWidget *parent)
 	hv->edit_mode_changed_cb = NULL;
 	hv->edit_mode_changed_data = NULL;
 
-	/* Load cursors */
+	/* Load the cursors */
 	hv->busy_cur = gdk_cursor_new(GDK_WATCH);
 	hv->text_cur = gdk_cursor_new(GDK_XTERM);
 	hv->translate_cur = gdk_cursor_new(GDK_FLEUR);
 
-	/* Load fonts */
-	hv->font = font = gdk_font_load(
-"-adobe-courier-medium-r-normal-*-12-*-*-*-m-*-iso8859-1"
-	);
-	hv->cell_width = 7;
-	hv->cell_height = font->ascent + font->descent;
-	hv->cells_per_row = 16;
+	/* Load the fonts */
+	hv->font = NULL;
+	hv->cell_width = 1;
+	hv->cell_height = 1;
+	hv->cells_per_row = HVIEW_DEF_VALUES_PER_ROW;
 	hv->address_x = 0;
-	hv->hex_x = 90;
-	hv->ascii_x = 450;
+	hv->hex_x = 0;
+	hv->ascii_x = 0;
 
-	/* Toplevel */
+	hv->freeze_count++;
+
+	/* Toplevel GtkVBox */
 	hv->toplevel = w = gtk_vbox_new(FALSE, 0);
 	if(GTK_IS_BOX(parent))
 	    gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
@@ -1192,8 +1200,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	    gtk_container_add(GTK_CONTAINER(parent), w);
 	parent = w;
 
-
-	/* Table for view */
+	/* View GtkTable */
 	w = gtk_table_new(1, 2, 0);
 	gtk_table_set_row_spacing(GTK_TABLE(w), 0, border_minor);
 	gtk_table_set_col_spacing(GTK_TABLE(w), 0, border_minor);
@@ -1201,7 +1208,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Drawing area frame */
+	/* View GtkFrame */
 	w = gtk_frame_new(NULL);
 	gtk_table_attach(
 	    GTK_TABLE(parent2), w,
@@ -1214,20 +1221,21 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 	parent3 = w;
 
+	/* Columns & View GtkVBox */
 	w = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(parent3), w);
 	gtk_widget_show(w);
 	parent3 = w;
 
-	/* Hbox for column headings */	
+	/* Column headings GtkHBox */
 	hv->heading_toplevel = w = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent4 = w;
 
-	/* Address column heading frame and label */
-	w = gtk_frame_new(NULL);
-	gtk_widget_set_usize(w, hv->hex_x - hv->address_x, 22);
+	/* Address column heading */
+	hv->heading_address_toplevel = w = gtk_frame_new(NULL);
+	gtk_widget_set_usize(w, -1, 22);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1241,8 +1249,8 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 
 	/* Hex column heading frame and label */
-	w = gtk_frame_new(NULL);
-	gtk_widget_set_usize(w, hv->ascii_x - hv->hex_x, 22);
+	hv->heading_hex_toplevel = w = gtk_frame_new(NULL);
+	gtk_widget_set_usize(w, -1, 22);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -1256,7 +1264,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 
 	/* ASCII column heading frame and label */
-	w = gtk_frame_new(NULL);
+	hv->heading_ascii_toplevel = w = gtk_frame_new(NULL);
 	gtk_widget_set_usize(w, -1, 22);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
 	gtk_box_pack_start(GTK_BOX(parent4), w, TRUE, TRUE, 0);
@@ -1271,9 +1279,10 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 
 
-	/* View drawing area */
+	/* View GtkDrawingArea */
 	hv->view_da = w = gtk_drawing_area_new();
 	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_FOCUS);
+	gtk_widget_set_name(w, HVIEW_WIDGET_NAME);
 	gtk_widget_add_events(
 	    w,
 	    GDK_STRUCTURE_MASK | GDK_EXPOSURE_MASK | GDK_VISIBILITY_NOTIFY_MASK |
@@ -1327,13 +1336,13 @@ hview_struct *HViewNew(GtkWidget *parent)
 
 	/* Create adjustment for vertical scrolling */
 	hv->vadj = adj = (GtkAdjustment *)gtk_adjustment_new(
-	    0.0, 0.0, 0.0, 0.0, 0.0, 0.0
+	    0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f
 	);
 	gtk_signal_connect(
 	    GTK_OBJECT(adj), "value_changed",
 	    GTK_SIGNAL_FUNC(HViewValueChangedCB), hv
 	);
-	/* Vertical scroll bar */
+	/* Vertical GtkScrollbar */
 	hv->vscrollbar = w = gtk_vscrollbar_new(adj);
 	gtk_table_attach(
 	    GTK_TABLE(parent2), w,
@@ -1346,7 +1355,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 
 
 
-	/* Frame for status bar */
+	/* Status Bar GtkFrame */
 	hv->status_bar_toplevel = w = gtk_frame_new(NULL);
 	gtk_widget_set_usize(w, -1, HVIEW_STATUS_BAR_HEIGHT);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
@@ -1354,13 +1363,13 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Table in status bar frame */
+	/* Status Bar GtkTable */
 	w = gtk_table_new(1, 1, FALSE);
 	gtk_container_add(GTK_CONTAINER(parent2), w);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Status label frame */
+	/* Status label GtkFrame */
 	w = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
 	gtk_container_border_width(GTK_CONTAINER(w), 1);
@@ -1385,7 +1394,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	gtk_widget_show(w);
 
 
-	/* Right-click menu */
+	/* Right-click popup GtkMenu */
 	if(TRUE)
 	{
 #define DO_ADD_MENU_ITEM_LABEL	{		\
@@ -1499,6 +1508,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	    hv->paste_mi = w;
 
 	    DO_ADD_MENU_SEP
+	    hv->insert_msep = w;
 
 	    icon = (guint8 **)icon_add_20x20_xpm;
 	    label =
@@ -1568,7 +1578,7 @@ hview_struct *HViewNew(GtkWidget *parent)
 	    accel_mods = 0;
 	    func_cb = HViewUnselectAllCB;
 	    DO_ADD_MENU_ITEM_LABEL  
-	    hv->unselect_all_mi = w;    
+	    hv->unselect_all_mi = w;
 
 	    DO_ADD_MENU_SEP   
 	    hv->edit_mode_msep = w;
@@ -1633,10 +1643,16 @@ hview_struct *HViewNew(GtkWidget *parent)
 #undef DO_ADD_MENU_SEP
 	}
 
-
+	HViewSetViewFont(
+	    hv,
+"-adobe-courier-medium-r-normal-*-12-*-*-*-m-*-iso8859-1"
+	);
 	HViewUpdate(hv);
 	HViewSetStatusMessage(hv, NULL, FALSE);
 
+
+	hv->freeze_count--;
+
 	return(hv);
 }
 
@@ -1644,7 +1660,7 @@ hview_struct *HViewNew(GtkWidget *parent)
  *	Creates a new Hex View with a toplevel GtkWindow.
  */
 hview_struct *HViewNewWithToplevel(
-	gint width, gint height
+	const gint width, const gint height
 )
 {
 	hview_struct *hv;
@@ -1663,7 +1679,7 @@ hview_struct *HViewNewWithToplevel(
 	    GTK_WINDOW(w), "hexview", PROG_NAME
 	);
 #endif
-	gtk_widget_realize(w);                                           
+	gtk_widget_realize(w);
 	window = w->window;
 	if(window != NULL)
 	{
@@ -1703,13 +1719,10 @@ void HViewShowHeading(hview_struct *hv, const gboolean show)
 	    return;
 
 	w = hv->heading_toplevel;
-	if(w != NULL)
-	{
-	    if(show)
-		gtk_widget_show(w);
-	    else
-		gtk_widget_hide(w);
-	}
+	if(show)
+	    gtk_widget_show(w);
+	else
+	    gtk_widget_hide(w);
 }
 
 /*
@@ -1723,17 +1736,91 @@ void HViewShowStatusBar(hview_struct *hv, const gboolean show)
 	    return;
 
 	w = hv->status_bar_toplevel;
-	if(w != NULL)
-	{
-	    if(show)
-		gtk_widget_show(w);
-	    else
-		gtk_widget_hide(w);
-	}
+	if(show)
+	    gtk_widget_show(w);
+	else
+	    gtk_widget_hide(w);
 }
 
 /*
- *	Sets the Hex View's as read only or writable.
+ *	Sets the view font.
+ *
+ *	The font_name specifies the font. The font must be a
+ *	fixed-width font.
+ */
+void HViewSetViewFont(hview_struct *hv, const gchar *font_name)
+{
+	if(hv == NULL)   
+	    return;
+
+	if(font_name != NULL)
+	{
+	    gint i, len, column_width[3];
+	    gchar *s;
+
+	    GdkTextBounds b;
+
+	    GdkFont *font = gdk_font_load(font_name);
+	    if(font == NULL)
+		return;
+
+	    GDK_FONT_UNREF(hv->font);
+	    hv->font = font;
+
+	    gdk_text_bounds(font, "X", 1, &b);
+
+	    hv->cell_width = b.width;
+	    hv->cell_height = font->ascent + font->descent;
+	    hv->cells_per_row = 16;		/* Fixed */
+
+	    hv->address_x = 0;
+
+	    s = STRDUP("0x00000000  ");
+	    gdk_string_bounds(font, s, &b);
+	    g_free(s);
+	    column_width[0] = b.width;
+	    hv->hex_x = column_width[0];
+
+	    len = (hv->cells_per_row * 3) + 2;
+	    s = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+	    for(i = 0; i < len; i++)
+		s[i] = 'X';
+	    s[len] = '\0';
+	    gdk_string_bounds(font, s, &b);
+	    g_free(s);
+	    column_width[1] = b.width;
+	    hv->ascii_x = column_width[0] + column_width[1];
+
+	    column_width[2] = -1;
+
+	    /* Update the column heading sizes */
+	    gtk_widget_set_usize(
+		hv->heading_address_toplevel,
+		column_width[0],
+		22
+	    );
+	    gtk_widget_set_usize(
+		hv->heading_hex_toplevel,
+		column_width[1],
+		22
+	    );
+	    gtk_widget_set_usize(
+		hv->heading_ascii_toplevel,
+		column_width[2],
+		22
+	    );
+	    gtk_widget_queue_resize(hv->toplevel);
+	}
+	else
+	{
+
+	}
+
+	HViewQueueDraw(hv);
+}
+
+/*
+ *	Sets read only or writable.
  */
 void HViewSetReadOnly(hview_struct *hv, const gboolean read_only)
 {
@@ -1757,14 +1844,14 @@ void HViewSetReadOnly(hview_struct *hv, const gboolean read_only)
  */
 void HViewSetChangedCB(
 	hview_struct *hv,
-	void (*changed_cb)(hview_struct *, gpointer),
+	void (*func_cb)(hview_struct *, gpointer),
 	gpointer data
 )
 {
 	if(hv == NULL)
 	    return;
 
-	hv->changed_cb = changed_cb;
+	hv->changed_cb = func_cb;
 	hv->changed_data = data;
 }
 
@@ -1773,7 +1860,7 @@ void HViewSetChangedCB(
  */
 void HViewSetSelectCB(
 	hview_struct *hv,
-	void (*select_cb)(
+	void (*func_cb)(
 		hview_struct *,
 		int, int,
 		gpointer
@@ -1784,7 +1871,7 @@ void HViewSetSelectCB(
 	if(hv == NULL)
 	    return;
 
-	hv->select_cb = select_cb;
+	hv->select_cb = func_cb;
 	hv->select_data = data;
 }
 
@@ -1793,7 +1880,7 @@ void HViewSetSelectCB(
  */
 void HViewSetEditModeChangedCB(
 	hview_struct *hv,
-	void (*edit_mode_changed_cb)(
+	void (*func_cb)(
 		hview_struct *,
 		hview_edit_mode,
 		gpointer
@@ -1804,7 +1891,7 @@ void HViewSetEditModeChangedCB(
 	if(hv == NULL)
 	    return;
 
-	hv->edit_mode_changed_cb = edit_mode_changed_cb;
+	hv->edit_mode_changed_cb = func_cb;
 	hv->edit_mode_changed_data = data;
 }
 
@@ -1828,25 +1915,49 @@ void HViewUpdate(hview_struct *hv)
 	/* Cut, Copy and Paste */
 	sensitive = read_only ?
 	    FALSE : HVIEW_IS_BUF_SELECTED(hv);
-	GTK_WIDGET_SET_SENSITIVE(hv->cut_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(hv->cut_mi, sensitive);
 	sensitive = (HVIEW_IS_BUF_SELECTED(hv)) ? TRUE : FALSE;
-	GTK_WIDGET_SET_SENSITIVE(hv->copy_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(hv->copy_mi, sensitive);
 	sensitive = !read_only;
-	GTK_WIDGET_SET_SENSITIVE(hv->paste_mi, sensitive) 
+	GTK_WIDGET_SET_SENSITIVE(hv->paste_mi, sensitive);
+	if(read_only)
+	{
+	    gtk_widget_hide(hv->cut_mi);
+	    gtk_widget_show(hv->copy_mi);
+	    gtk_widget_hide(hv->paste_mi);
+	}
+	else
+	{
+	    gtk_widget_show(hv->cut_mi);
+	    gtk_widget_show(hv->copy_mi);
+	    gtk_widget_show(hv->paste_mi);
+	}
 
 	/* Insert and Delete */
 	sensitive = !read_only;
-	GTK_WIDGET_SET_SENSITIVE(hv->insert_mi, sensitive) 
-	GTK_WIDGET_SET_SENSITIVE(hv->delete_mi, sensitive) 
+	GTK_WIDGET_SET_SENSITIVE(hv->insert_mi, sensitive);
+	GTK_WIDGET_SET_SENSITIVE(hv->delete_mi, sensitive);
+	if(read_only)
+	{
+	    gtk_widget_hide(hv->insert_msep);
+	    gtk_widget_hide(hv->insert_mi);
+	    gtk_widget_hide(hv->delete_mi);
+	}
+	else
+	{
+	    gtk_widget_show(hv->insert_msep);
+	    gtk_widget_show(hv->insert_mi);
+	    gtk_widget_show(hv->delete_mi);
+	}
 
 	/* Select All and Unselect All */
 	sensitive = TRUE;
-	GTK_WIDGET_SET_SENSITIVE(hv->select_all_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(hv->select_all_mi, sensitive);
 	sensitive = HVIEW_IS_BUF_SELECTED(hv);
-	GTK_WIDGET_SET_SENSITIVE(hv->unselect_all_mi, sensitive)
+	GTK_WIDGET_SET_SENSITIVE(hv->unselect_all_mi, sensitive);
 
 	/* Edit mode */
-	if(HVIEW_READ_ONLY(hv))
+	if(read_only)
 	{
 	    gtk_widget_hide(hv->edit_mode_msep);
 	    gtk_widget_hide(hv->edit_mode_hex_mi);
@@ -1885,41 +1996,36 @@ void HViewDraw(hview_struct *hv)
  */
 void HViewQueueDraw(hview_struct *hv)
 {
-	GtkWidget *w;
-
 	if(hv == NULL)
 	    return;
 
-	w = hv->view_da;
-	if(w != NULL)
-	    gtk_widget_queue_draw(w);
+	gtk_widget_queue_draw(hv->view_da);
 }
 
 
 /*
  *	Updates the Hex View's Status Bar message.
  *
- *	If mesg is NULL then the cursor position status message will
+ *	If msg is NULL then the cursor position status message will
  *	be set.
  */
 void HViewSetStatusMessage(
-	hview_struct *hv, const gchar *mesg,
-	gboolean allow_gtk_iteration
+	hview_struct *hv, const gchar *msg,
+	const gboolean allow_gtk_iteration
 )
 {
-	GtkWidget *w = (hv != NULL) ? hv->status_bar_label : NULL;
-	if(w == NULL)
+	if(hv == NULL)
 	    return;
 
-	if(mesg == NULL)
+	if(msg == NULL)
 	{
 	    gint i = hv->buf_pos;
-	    gchar *buf;
+	    gchar *msg;
 
 	    /* Is a value being edited? */
 	    if((hv->edit_buf_i > 0) && (sizeof(hv->edit_buf) >= 2))
 	    {
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 "Address: 0x%.8X - Position: %i - Size: %i -\
  Mode: %s - Hex: 0x%c%c%c",
 		    i, i,
@@ -1935,7 +2041,7 @@ void HViewSetStatusMessage(
 	    else
 	    {
 		if((i >= 0) && (i < hv->buf_len) && (hv->buf != NULL))
-		    buf = g_strdup_printf(
+		    msg = g_strdup_printf(
 "Address: 0x%.8X - Position: %i - Size: %i -\
  Mode: %s - Hex: 0x%.2X - ASCII: %c",
 			i, i,
@@ -1947,14 +2053,14 @@ void HViewSetStatusMessage(
 			    (gchar)hv->buf[i] : ' '
 		    );
 		else
-		    buf = STRDUP("");
+		    msg = STRDUP("");
 	    }
-	    gtk_label_set_text(GTK_LABEL(w), buf);
-	    g_free(buf);
+	    gtk_label_set_text(GTK_LABEL(hv->status_bar_label), msg);
+	    g_free(msg);
 	}
 	else
 	{
-	    gtk_label_set_text(GTK_LABEL(w), mesg);
+	    gtk_label_set_text(GTK_LABEL(hv->status_bar_label), msg);
 	}
 
 	while((gtk_events_pending() > 0) && allow_gtk_iteration)
@@ -1964,26 +2070,27 @@ void HViewSetStatusMessage(
 /*
  *	Marks the Hex View as busy or ready.
  */
-void HViewSetBusy(hview_struct *hv, gboolean is_busy)
+void HViewSetBusy(hview_struct *hv, const gboolean is_busy)
 {
 	GdkCursor *cur;
-	GtkWidget *w = (hv != NULL) ? hv->toplevel : NULL;
-	if(w == NULL)
+	GtkWidget *w;
+
+	if(hv == NULL)
 	    return;
 
 	/* If the toplevel is not a GtkWindow then set only for the
 	 * view GtkDrawingArea
 	 */
-	if(!GTK_IS_WINDOW(hv))
-	{
+	if(GTK_IS_WINDOW(hv->toplevel))
+	    w = hv->toplevel;
+	else
 	    w = hv->view_da;
-	    if(w == NULL)
-		return;
-	}
+	if(w == NULL)
+	    return;
 
 	if(is_busy)
 	{
-	    /* Increase busy count */
+	    /* Increase the busy count */
 	    hv->busy_count++;
 
 	    /* If already busy then don't change anything */
@@ -1994,19 +2101,18 @@ void HViewSetBusy(hview_struct *hv, gboolean is_busy)
 	}
 	else
 	{
-	    /* Reduce busy count */
+	    /* Reduce the busy count */
 	    hv->busy_count--;
 	    if(hv->busy_count < 0)
 		hv->busy_count = 0;
 
-	    /* If still busy do not change anything */
+	    /* If still busy then do not change anything */
 	    if(hv->busy_count > 0)
 		return;
 
-	    cur = NULL;		/* Use default cursor */
+	    cur = NULL;		/* Use the default cursor */
 	}
 
-	/* Update toplevel window's cursor */
 	if(w->window != NULL)
 	{
 	    gdk_window_set_cursor(w->window, cur);
@@ -2019,10 +2125,12 @@ void HViewSetBusy(hview_struct *hv, gboolean is_busy)
  */
 void HViewMap(hview_struct *hv)
 {
-	GtkWidget *w = (hv != NULL) ? hv->toplevel : NULL;
-	if(w == NULL)
+	GtkWidget *w;
+
+	if(hv == NULL)
 	    return;
 
+	w = hv->toplevel;
 	if(GTK_IS_WINDOW(w))
 	    gtk_widget_show_raise(w);
 	else
@@ -2035,11 +2143,10 @@ void HViewMap(hview_struct *hv)
  */
 void HViewUnmap(hview_struct *hv)
 {
-	GtkWidget *w = (hv != NULL) ? hv->toplevel : NULL;
-	if(w == NULL)
+	if(hv == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(hv->toplevel);
 	hv->map_state = FALSE;
 }
 
@@ -2056,6 +2163,10 @@ void HViewDelete(hview_struct *hv)
 	GTK_TIMEOUT_REMOVE(hv->scroll_toid);
 	hv->scroll_toid = 0;
 
+	HViewUnmap(hv);
+
+	hv->freeze_count++;
+
 	colormap = hv->colormap;
 
 	g_free(hv->buf);
@@ -2067,25 +2178,22 @@ void HViewDelete(hview_struct *hv)
 	hv->edit_buf_i = 0;
 	*hv->edit_buf = '\0';
 
-	GTK_WIDGET_DESTROY(hv->menu)
-	GTK_WIDGET_DESTROY(hv->status_bar_label)
-	GTK_WIDGET_DESTROY(hv->status_bar_toplevel)
-	GTK_WIDGET_DESTROY(hv->vscrollbar)
-	GTK_WIDGET_DESTROY(hv->view_da)
-	GTK_WIDGET_DESTROY(hv->heading_toplevel)
-	GTK_WIDGET_DESTROY(hv->toplevel)
+	GTK_WIDGET_DESTROY(hv->menu);
+	GTK_WIDGET_DESTROY(hv->toplevel);
 
-	GDK_PIXMAP_UNREF(hv->view_pm)
+	GDK_PIXMAP_UNREF(hv->view_pm);
 
-	GDK_CURSOR_DESTROY(hv->busy_cur)
-	GDK_CURSOR_DESTROY(hv->text_cur)
-	GDK_CURSOR_DESTROY(hv->translate_cur)
+	GDK_CURSOR_DESTROY(hv->busy_cur);
+	GDK_CURSOR_DESTROY(hv->text_cur);
+	GDK_CURSOR_DESTROY(hv->translate_cur);
 
-	GDK_GC_UNREF(hv->gc)
+	GDK_GC_UNREF(hv->gc);
 
-	GDK_COLORMAP_UNREF(hv->colormap)
+	GDK_COLORMAP_UNREF(hv->colormap);
 
-	GDK_FONT_UNREF(hv->font)
+	GDK_FONT_UNREF(hv->font);
+
+	hv->freeze_count--;
 
 	g_free(hv);
 }
