@@ -7,6 +7,7 @@
 #include "../include/string.h"
 
 #include "guiutils.h"
+#include "progressdialog.h"
 
 #include "edv_types.h"
 #include "cfg.h"
@@ -24,6 +25,8 @@
 #include "imbr_img_view.h"
 #include "imbr_dnd.h"
 #include "endeavour2.h"
+#include "edv_obj_create.h"
+#include "edv_open.h"
 #include "edv_op.h"
 #include "edv_cb.h"
 #include "edv_utils.h"
@@ -34,6 +37,7 @@
 
 
 /* GTK+ Signal Callbacks */
+void EDVImbrCmdDestroyCB(gpointer data);
 void EDVImbrTListItemDestroyCB(gpointer data);
 
 gint EDVImbrDeleteEventCB(
@@ -64,7 +68,7 @@ void EDVImbrTListUnselectCB(
 
 void EDVImbrComboActivateCB(GtkWidget *widget, gpointer data);
 
-gint EDVImbrLoadingTOCB(gpointer data);
+gint EDVImbrLoadThumbsPassiveTOCB(gpointer data);
 
 /* Menu Item Callbacks */
 void EDVImbrMenuItemCB(GtkWidget *widget, gpointer data);
@@ -74,6 +78,14 @@ gint EDVImbrMenuItemEnterCB(
 gint EDVImbrMenuItemLeaveCB(
 	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
 );
+void EDVImbrMenuItemCmdObjectOpCB(GtkWidget *widget, gpointer data);
+void EDVImbrMenuItemCmdNewObjectCB(GtkWidget *widget, gpointer data);
+gint EDVImbrMenuItemCmdEnterCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+);
+gint EDVImbrMenuItemCmdLeaveCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+);
 
 /* Paths List Get Callback */
 GList *EDVImbrGetSelectedPathsListCB(gpointer data);
@@ -189,6 +201,22 @@ void EDVImbrDeviceRemovedCB(
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
+/*
+ *	Image Browser command "destroy" signal callback.
+ */
+void EDVImbrCmdDestroyCB(gpointer data)
+{
+	edv_imbr_cmd_struct *d = EDV_IMBR_CMD(data);
+	if(d == NULL)
+	    return;
+
+	g_free(d->label);
+	g_free(d->command);
+	g_free(d->description);
+	g_free(d->ext_data);
+	g_free(d);
+}
+
 /*
  *	Image Browser thumbs list "destroy" signal callback.
  */
@@ -207,10 +235,10 @@ gint EDVImbrDeleteEventCB(
 {
 	edv_imbr_struct *imbr = EDV_IMBR(data);
 	if(imbr == NULL)
-	    return(TRUE);
+	    return(FALSE);
 
-	if(imbr->processing)
-	    return(TRUE);
+	if(imbr->processing || (imbr->freeze_count > 0))
+	    return(FALSE);
 
 	EDVImbrOPClose(imbr);
 
@@ -246,17 +274,18 @@ gint EDVImbrKeyEventCB(
 	    return(status);
 
 	w = imbr->toplevel;
-	focus_widget = (w != NULL) ? GTK_WINDOW(w)->focus_widget : NULL;
+	focus_widget = GTK_WINDOW(w)->focus_widget;
 	cfg_list = core->cfg_list;
+
 	etype = key->type;
 	is_press = (etype == GDK_KEY_PRESS) ? TRUE : FALSE;
 	keyval = key->keyval;
 	state = key->state;
 
 /* Stop emit of signal */
-#define DO_STOP_KEY_SIGNAL_EMIT	{		\
+#define STOP_SIGNAL_EMISSION(_w_)	{	\
  gtk_signal_emit_stop_by_name(			\
-  GTK_OBJECT(widget),				\
+  GTK_OBJECT(_w_),				\
   is_press ?					\
    "key_press_event" : "key_release_event"	\
  );						\
@@ -283,7 +312,7 @@ gint EDVImbrKeyEventCB(
 		    if(opid != NULL)
 			EDVImbrOPCB(NULL, -1, opid);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		return(status);
 	    }
@@ -302,17 +331,22 @@ gint EDVImbrKeyEventCB(
 		 * item who's name starts with the letter of the key
 		 * that was pressed
 		 */
-		if(isalnum((int)keyval) && is_press &&
-		   !(state & GDK_CONTROL_MASK) &&
-		   !(state & GDK_SHIFT_MASK)    
-		)
+		if(isalnum((int)keyval) && is_press)
 		{
+                    const gboolean backwards = (state & GDK_MOD1_MASK) ? TRUE : FALSE;
+                    const gint start_thumb_num = (backwards) ?
+                        (tlist->total_thumbs - 1) : 0;
+
 		    TListFreeze(tlist);
 		    EDVTListSeekCharacter(
-			tlist, 0, keyval
+			tlist,
+			start_thumb_num,
+			backwards,
+			0,			/* Text index */
+			keyval			/* Character */
 		    );
 		    TListThaw(tlist);
-		    DO_STOP_KEY_SIGNAL_EMIT
+		    STOP_SIGNAL_EMISSION(widget);
 		    status = TRUE;
 		}
 		break;
@@ -320,7 +354,7 @@ gint EDVImbrKeyEventCB(
 	}
 
 	return(status);
-#undef DO_STOP_KEY_SIGNAL_EMIT
+#undef STOP_SIGNAL_EMISSION
 }
 
 /*
@@ -345,14 +379,8 @@ gint EDVImbrButtonPressEventCB(
 
 	tlist = imbr->tlist;
 	core = imbr->core;
-	if((tlist == NULL) || (core == NULL))
-	    return(status);
-
 	cfg_list = core->cfg_list;
-
-	/* Get event type */
-	etype = button->type;
-
+	etype = (gint)button->type;
 	state = button->state;
 
 	/* Event occured on the thumbs list? */
@@ -388,8 +416,6 @@ gint EDVImbrButtonPressEventCB(
 	      case 3:
 		if(etype == GDK_BUTTON_PRESS)
 		{
-		    GtkMenu *menu;
-
 		    /* Select item before mapping menu? */
 		    if(EDV_GET_B(EDV_CFG_PARM_RIGHT_CLICK_MENU_SELECTS) &&
 		       (thumb_num > -1)
@@ -407,15 +433,13 @@ gint EDVImbrButtonPressEventCB(
 			TListThaw(tlist);
 		    }
 
-		    /* Update all menus and map right click menu */
+		    /* Update all menus and map the right click menu */
 		    EDVImbrUpdateMenus(imbr);
-		    menu = (GtkMenu *)imbr->tlist_menu;
-		    if(menu != NULL)
-			gtk_menu_popup(
-			    menu, NULL, NULL,
-			    NULL, NULL,
-			    button->button, button->time
-			);
+		    gtk_menu_popup(
+			GTK_MENU(imbr->tlist_menu), NULL, NULL,
+			NULL, NULL,
+			button->button, button->time
+		    );
 		}
 		status = TRUE;
 		break;
@@ -424,7 +448,7 @@ gint EDVImbrButtonPressEventCB(
 		if(etype == GDK_BUTTON_PRESS)
 		{
 		    if(thumb_num > -1)
-			EDVImbrTListDoFPromptRename(imbr, thumb_num);
+			EDVImbrTListPromptRename(imbr, thumb_num);
 		}
 		status = TRUE;
 		break;
@@ -655,12 +679,9 @@ void EDVImbrTListSelectCB(
 		      case EDV_OBJECT_TYPE_FILE:
 			type_str = "File";
 			size_str = g_strdup_printf(
-			    " (%s byte%s)",
-			    EDVGetObjectSizeStr(
-				imbr->core,
-				obj->size
-			    ),
-			    (obj->size == 1) ? "" : "s"
+			    " (%s %s)",
+			    EDVSizeStrDelim(obj->size),
+			    (obj->size == 1l) ? "byte" : "bytes"
 			);
 			break;
 		      case EDV_OBJECT_TYPE_DIRECTORY:
@@ -697,7 +718,7 @@ void EDVImbrTListSelectCB(
 		    if(nselected > 1)
 			s = g_strdup_printf(
 			    "%s objects selected",
-			    EDVGetObjectSizeStr(core, nselected)
+			    EDVSizeStrDelim(nselected)
 			);
 		    else if(!strcmp(obj->name, ".."))
 			s = g_strdup_printf(
@@ -794,9 +815,9 @@ void EDVImbrTListUnselectCB(
 	    /* Update the status bar message */
 	    if(nselected > 0)
 		s = g_strdup_printf(
-		    "%s object%s selected",
-		    EDVGetObjectSizeStr(core, nselected),
-		    (nselected == 1) ? "" : "s"
+		    "%s %s selected",
+		    EDVSizeStrDelim(nselected),
+		    (nselected == 1) ? "object" : "objects"
 		);
 	    else
 		s = STRDUP("No objects selected");
@@ -864,12 +885,13 @@ void EDVImbrComboActivateCB(GtkWidget *widget, gpointer data)
 
 
 /*
- *	Loading procedure timeout callback.
+ *	Load Thumbs Passive timeout callback.
  */
-gint EDVImbrLoadingTOCB(gpointer data)
+gint EDVImbrLoadThumbsPassiveTOCB(gpointer data)
 {
-	gint status;
 	const cfg_item_struct *cfg_list;
+	edv_status_bar_struct *sb;
+	edv_runlevel prev_runlevel;
 	edv_core_struct *core;
 	edv_imbr_struct *imbr = EDV_IMBR(data);
 	if(imbr == NULL)
@@ -881,83 +903,85 @@ gint EDVImbrLoadingTOCB(gpointer data)
 	if(imbr->processing)
 	    return(TRUE);
 
-#define DO_FINAL_RETURN		{	\
-					\
- /* Reset loading timeout id */		\
- imbr->loading_tocb = 0;		\
-					\
- EDVImbrSetPassiveBusy(imbr, FALSE);	\
-					\
- return(FALSE);				\
+#define RETURN_FINAL(_imbr_) {			\
+						\
+ /* Reset loading timeout id */			\
+ (_imbr_)->loading_tocb = 0;			\
+						\
+ /* Restore the previous runlevel */		\
+ core->runlevel = prev_runlevel;		\
+						\
+ EDVImbrSetPassiveBusy((_imbr_), FALSE);	\
+						\
+ return(FALSE);					\
 }
 
-#define DO_RESCHEDUAL_RETURN	{	\
-					\
- /* Set new loading timeout callback */	\
- imbr->loading_tocb = gtk_timeout_add(	\
-  100l,					\
-  EDVImbrLoadingTOCB, imbr		\
- );					\
-					\
- /* Return FALSE since we reschedualed	\
-  * a timeout callback to this function	\
-  */					\
- return(FALSE);				\
+#define RETURN_RESCHEDUAL(_imbr_) {		\
+						\
+ /* Set new loading timeout callback */		\
+ (_imbr_)->loading_tocb = gtk_timeout_add(	\
+  100l,						\
+  EDVImbrLoadThumbsPassiveTOCB, imbr		\
+ );						\
+						\
+ /* Restore the previous runlevel */		\
+ core->runlevel = prev_runlevel;		\
+						\
+ /* Return FALSE since we reschedualed a	\
+  * timeout callback to this function		\
+  */						\
+ return(FALSE);					\
 }
 
+	sb = imbr->status_bar;
 	core = imbr->core;
-	if(core == NULL)
-	{
-	    DO_FINAL_RETURN
-	}
-
 	cfg_list = core->cfg_list;
 
-	/* Stop loading? */
+	/* Update the core's runlevel to background */
+	if(core->runlevel < EDV_RUNLEVEL_BACKGROUND)
+	{
+	    prev_runlevel = core->runlevel;
+	    core->runlevel = EDV_RUNLEVEL_BACKGROUND;
+	}
+	else
+	{
+	    prev_runlevel = core->runlevel;
+	}
+
+	/* Loading interrupted? */
 	if(imbr->stop_count > 0)
 	{
 	    imbr->stop_count = 0;
 	    imbr->loading_tocb = 0;
 
-	    EDVStatusBarMessage(
-		imbr->status_bar, "Loading interrupted", FALSE
-	    );
-	    EDVStatusBarProgress(imbr->status_bar, 0.0f, FALSE);
+	    EDVStatusBarMessage(sb, "Loading interrupted", FALSE);
+	    EDVStatusBarProgress(sb, 0.0f, FALSE);
 	    EDVImbrUpdateMenus(imbr);
-	    EDVImbrSetPassiveBusy(imbr, FALSE);
 
-	    return(FALSE);
+	    RETURN_FINAL(imbr);
 	}
 
 	/* Load the next thumb */
-	status = EDVImbrTListLoadIterate(
+	switch(EDVImbrTListLoadIterate(
 	    imbr, TRUE,
 	    EDV_GET_B(EDV_CFG_PARM_IMBR_THUMB_NO_ENLARGE)
-	);
-#if 0
-g_print(
- "EDVImbrTListLoadIterate(): status=%i\n",
- status
-);
-#endif
-	switch(status)
+	))
 	{
-	  case 0:	/* All thumb loaded */
+	  case 0:	/* All thumbs loaded */
 	    imbr->loading_tocb = 0;
 	    EDVImbrUpdateMenus(imbr);
-	    EDVImbrSetPassiveBusy(imbr, FALSE);
-	    return(FALSE);
+	    RETURN_FINAL(imbr);
 	    break;
 
-	  case 1:
-	    /* Thumb loaded successfully, but there still may be more
-	     * thumbs that need to be loaded
+	  case 1:	/* Thumb loaded successfully */
+	    /* There still may be more thumbs that need to be loaded
+	     * so reschedual
 	     */
-	    DO_RESCHEDUAL_RETURN
+	    RETURN_RESCHEDUAL(imbr);
 	    break;
 
 	  case -3:	/* System error */
-	    DO_FINAL_RETURN
+	    RETURN_FINAL(imbr);
 	    break;
 
 	  default:	/* General error */
@@ -965,11 +989,11 @@ g_print(
 	     * this thumb that failed to load will not be attempted
 	     * to be loaded again
 	     */
-	    DO_RESCHEDUAL_RETURN
+	    RETURN_RESCHEDUAL(imbr);
 	    break;
 	}
-#undef DO_FINAL_RETURN
-#undef DO_RESCHEDUAL_RETURN
+#undef RETURN_FINAL
+#undef RETURN_RESCHEDUAL
 }
 
 
@@ -1009,39 +1033,302 @@ gint EDVImbrMenuItemLeaveCB(
 	return(TRUE);
 }
 
+/*
+ *	Menu Item command object op "activate" signal callback.
+ */
+void EDVImbrMenuItemCmdObjectOpCB(GtkWidget *widget, gpointer data)
+{
+	const gchar *command;
+	GtkWidget *toplevel;
+	GList *paths_list;
+	edv_imbr_struct *imbr;
+	edv_core_struct *core;
+	edv_imbr_cmd_struct *d = EDV_IMBR_CMD(data);
+	if(d == NULL)
+	    return;
+
+	imbr = d->imbr;
+	toplevel = imbr->toplevel;
+	core = imbr->core;
+
+	if(imbr->processing)
+	    return;
+
+	EDVImbrSetBusy(imbr, TRUE);
+
+	/* Get the command for this menu item */
+	command = d->command;
+	if(STRISEMPTY(command))
+	{
+	    gchar *msg = g_strdup_printf(
+"The custom menu item:\n\
+\n\
+    \"%s\"\n\
+\n\
+Does not have a defined command.",
+		d->label
+	    );
+	    EDVPlaySoundWarning(core);
+	    EDVMessageWarning(
+		"Open Failed",
+		msg,
+"To set the command for this custom menu item,\n\
+go to Settings->Customize...->Global->Object Ops Menu.",
+		toplevel
+	    );
+	    g_free(msg);
+	    EDVImbrSetBusy(imbr, FALSE);
+	    return;
+	}
+
+	/* Get the selected paths list */
+	paths_list = EDVImbrGetSelectedPathsList(imbr);
+	if(paths_list != NULL)
+	{
+	    /* Open the selected objects */
+	    gchar *stdout_path, *stderr_path;
+
+	    EDVOpenCommand(
+		core,
+		paths_list,
+		command,
+		toplevel,
+		TRUE,
+		&stdout_path, &stderr_path
+	    );
+
+	    g_free(stdout_path);
+	    g_free(stderr_path);
+
+	    /* Delete the selected paths list */
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
+	}
+
+	EDVImbrSetBusy(imbr, FALSE);
+}
+
+/*
+ *	Menu Item command new object "activate" signal callback.
+ */
+void EDVImbrMenuItemCmdNewObjectCB(GtkWidget *widget, gpointer data)
+{
+	gboolean yes_to_all = FALSE;
+	gint status;
+	gchar *new_name, *new_path;
+	const gchar *command, *error_msg;
+	GList *new_paths_list;
+	GtkWidget *toplevel;
+	edv_imbr_struct *imbr;
+	edv_core_struct *core;
+	edv_imbr_cmd_struct *d = EDV_IMBR_CMD(data);
+	if(d == NULL)
+	    return;
+
+	imbr = d->imbr;
+	toplevel = imbr->toplevel;
+	core = imbr->core;
+
+	if(imbr->processing)
+	    return;
+
+	EDVImbrSetBusy(imbr, TRUE);
+
+	/* Get the command for this menu item */
+	command = d->command;
+	if(STRISEMPTY(command))
+	{
+	    gchar *msg = g_strdup_printf(
+"The custom menu item:\n\
+\n\
+    %s\n\
+\n\
+Does not have a defined command.",
+		d->label
+	    );
+	    EDVPlaySoundWarning(core);
+	    EDVMessageWarning(
+		"Open Failed",
+		msg,
+"To set the command for this custom menu item,\n\
+go to Settings->Customize...->Global->New Objects Menu.",
+		toplevel
+	    );
+	    g_free(msg);
+	    EDVImbrSetBusy(imbr, FALSE);
+	    return;
+	}
+
+	/* Format the suggested name for the new object */
+	if(!STRISEMPTY(d->ext_data))
+	{
+	    const gchar *ext = d->ext_data;
+	    gchar *s, *new_ext;
+
+	    if(*ext != '.')
+		new_ext = g_strconcat(".", ext, NULL);
+	    else
+		new_ext = STRDUP(ext);
+	    s = (gchar *)strpbrk((char *)new_ext, " \t,;");
+	    if(s != NULL)
+		*s = '\0';
+
+	    new_name = g_strconcat(
+		"new",
+		new_ext,
+		NULL
+	    );
+
+	    g_free(new_ext);
+	}
+	else
+	{
+	    new_name = STRDUP("new");
+	}
+
+	/* Prompt the user for the name of the new file and create the
+	 * new file
+	 */
+	status = EDVObjCreateFileQueryName(
+	    core,
+	    EDVImbrCurrentLocation(imbr),	/* Parent path */
+	    new_name,				/* Suggested name */
+	    &new_path,
+	    toplevel,
+	    TRUE,				/* Show progress */
+	    &yes_to_all
+	);
+
+	g_free(new_name);
+
+	/* Check for errors */
+	error_msg = EDVObjCreateGetError(core);
+	if(!STRISEMPTY(error_msg))
+	{
+	    /* Report the error */
+	    EDVPlaySoundError(core);
+	    EDVMessageError(
+		"Create New Object Failed",
+		error_msg,   
+		NULL,
+		toplevel
+	    );
+	}
+
+	/* Report the new object being added and create the new paths
+	 * list
+	 */
+	new_paths_list = NULL;
+	if(new_path != NULL)
+	{
+	    struct stat lstat_buf;
+	    if(!lstat((const char *)new_path, &lstat_buf))
+		EDVObjectAddedEmit(
+		    core,
+		    new_path,
+		    &lstat_buf
+		);
+
+	    new_paths_list = g_list_append(
+		new_paths_list,
+		STRDUP(new_path)
+	    );
+	}
+
+	/* Unmap the progress dialog */
+	ProgressDialogBreakQuery(TRUE);
+	ProgressDialogSetTransientFor(NULL);
+
+	/* Open the new object */
+	if(new_paths_list != NULL)
+	{
+	    /* Open the selected objects */
+	    gchar *stdout_path, *stderr_path;
+
+	    EDVOpenCommand(
+		core,
+		new_paths_list,
+		command,
+		toplevel,
+		TRUE,
+		&stdout_path, &stderr_path
+	    );
+
+	    g_free(stdout_path);
+	    g_free(stderr_path);
+
+	    /* Delete the new paths list */
+	    g_list_foreach(new_paths_list, (GFunc)g_free, NULL);
+	    g_list_free(new_paths_list);
+	}
+
+	g_free(new_path);
+
+	/* Play the "completed" sound on success */
+	if(status == 0)
+	    EDVPlaySoundCompleted(core);
+
+	EDVImbrSetBusy(imbr, FALSE);
+}
+
+/*
+ *	Menu item command "leave_enter_event" signal callback.
+ */
+gint EDVImbrMenuItemCmdEnterCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+)
+{
+	edv_imbr_struct *imbr;
+	edv_imbr_cmd_struct *d = EDV_IMBR_CMD(data);
+	if(d == NULL)
+	    return(FALSE);
+
+	imbr = d->imbr;
+
+	EDVStatusBarMessage(
+	    imbr->status_bar,
+	    STRISEMPTY(d->description) ?
+		d->command : d->description,
+	    FALSE
+	);
+
+	return(TRUE);
+}
+
+/*
+ *	Menu item command "leave_notify_event" signal callback.
+ */
+gint EDVImbrMenuItemCmdLeaveCB(
+	GtkWidget *widget, GdkEventCrossing *crossing, gpointer data
+)
+{
+	edv_imbr_struct *imbr;
+	edv_imbr_cmd_struct *d = EDV_IMBR_CMD(data);
+	if(d == NULL)
+	    return(FALSE);
+
+	imbr = d->imbr;
+
+	EDVStatusBarMessage(
+	    imbr->status_bar,
+	    NULL,
+	    FALSE
+	);
+
+	return(TRUE);
+}
+
 
 /*
  *	Open paths List Get callback.
  */
 GList *EDVImbrGetSelectedPathsListCB(gpointer data)
 {
-	GList *paths_list = NULL;
 	edv_imbr_struct *imbr = EDV_IMBR(data);
 	if(imbr == NULL)
-	    return(paths_list);
+	    return(NULL);
 
-	if(TRUE)
-	{
-	    tlist_struct *tlist = imbr->tlist;
-	    if(tlist != NULL)
-	    {
-		gint thumb_num;
-		GList *glist = tlist->selection;
-		edv_object_struct *obj;
-		while(glist != NULL)
-		{
-		    thumb_num = (gint)glist->data;
-		    obj = EDV_OBJECT(TListGetThumbData(tlist, thumb_num));
-		    if(obj != NULL)
-			paths_list = g_list_append(
-			    paths_list, STRDUP(obj->full_path)
-			);
-		    glist = g_list_next(glist);
-		}
-	    }
-	}
-
-	return(paths_list);
+	return(EDVImbrGetSelectedPathsList(imbr));
 }
 
 /*
@@ -1051,7 +1338,7 @@ void EDVImbrGotoDirectoryCB(gpointer data, const gchar *path)
 {
 	GtkWidget *toplevel;
 	edv_imbr_struct *imbr = EDV_IMBR(data);
-	if((imbr == NULL) || (path == NULL))         
+	if((imbr == NULL) || (path == NULL))
 	    return;
 
 	toplevel = imbr->toplevel;
@@ -1122,7 +1409,7 @@ void EDVImbrMountBarMountCB(
 	/* Mount error? */
 	if(status)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -1199,7 +1486,7 @@ void EDVImbrMountBarEjectCB(
 	/* Mount error? */
 	if(status)
 	{
-	    const gchar *last_error = EDVDeviceUnmountGetError();
+	    const gchar *last_error = EDVDeviceMountGetError(core);
 	    if(!STRISEMPTY(last_error))
 	    {
 		EDVPlaySoundError(core);
@@ -1382,7 +1669,7 @@ void EDVImbrObjectAddedNotifyCB(
 	const struct stat *lstat_buf
 )
 {
-	if(imbr == NULL)
+	if((imbr == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(imbr->processing)
@@ -1411,40 +1698,34 @@ void EDVImbrObjectModifiedNotifyCB(
 	const struct stat *lstat_buf
 )
 {
-	if(imbr == NULL)
+	gchar *cur_path;
+
+	if((imbr == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(imbr->processing)
 	    return;
 
-	/* Check current location first, if the old path matches
-	 * the current location then the current location needs to be
-	 * updated  to reflect the new path before notifying the
-	 * Thumbs List
-	 */
-	if(!STRISEMPTY(path))
+	if(new_path == NULL)
+	    new_path = path;
+
+	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
+	if(cur_path == NULL)
+	    return;
+
+	/* Is the modified path the current location? */
+	if(!strcmp((const char *)path, (const char *)cur_path))
 	{
-	    gchar *cur_loc = STRDUP(EDVImbrCurrentLocation(imbr));
-
-	    if(new_path == NULL)
-		new_path = path;
-
-	    /* Check if the old path matches the current location, if
-	     * it does then the current location needs to be updated to
-	     * the new path
-	     */
-	    if((cur_loc != NULL) ? !strcmp(cur_loc, path) : FALSE)
+	    /* Path changed? */
+	    if(path != new_path)
 	    {
-		/* Old path matches current location, change values
-		 * reflecting the current location to the value of
-		 * new_path
-		 */
-		EDVImbrSetTitle(imbr, new_path);
-		EDVImbrSetLocation(imbr, new_path, FALSE);
-		EDVImbrUpdateLocationIcon(imbr, new_path);
+		if(strcmp((const char *)path, (const char *)new_path))
+		{
+		    EDVImbrSetTitle(imbr, new_path);
+		    EDVImbrSetLocation(imbr, new_path, FALSE);
+		    EDVImbrUpdateLocationIcon(imbr, new_path);
+		}
 	    }
-
-	    g_free(cur_loc);
 	}
 
 	/* Thumbs List */
@@ -1456,6 +1737,8 @@ void EDVImbrObjectModifiedNotifyCB(
 	EDVMountBarUpdateMenus(imbr->mount_bar);
 
 /*	EDVImbrUpdateMenus(imbr); */
+
+	g_free(cur_path);
 }
 
 /*
@@ -1465,48 +1748,45 @@ void EDVImbrObjectRemovedNotifyCB(
 	edv_imbr_struct *imbr, const gchar *path
 )
 {
-	if(imbr == NULL)
+	gchar *cur_path;
+
+	if((imbr == NULL) || STRISEMPTY(path))
 	    return;
 
 	if(imbr->processing)
 	    return;
 
+	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
+	if(cur_path == NULL)
+	    return;
+
 	/* Thumbs List */
 	EDVImbrTListObjectRemovedNotify(imbr, path);
 
 	/* Mount Bar */
 	EDVMountBarUpdateMenus(imbr->mount_bar);
 
-	/* Check if the path that was removed is the current location,
-	 * if it is then set the current location to be the parent of
-	 * the path that was removed
+	/* Is the removed path the current location or one of its
+	 * parents?
 	 */
-	if(!STRISEMPTY(path))
+	if(strpfx((const char *)cur_path, (const char *)path))
 	{
-	    gchar *cur_loc = STRDUP(EDVImbrCurrentLocation(imbr));
-
-	    /* Check if the removed object is the current location */
-	    if((cur_loc != NULL) ? !strcmp(cur_loc, path) : FALSE)
+	    /* Change the location to the parent of the removed path */
+	    gchar *parent_path = g_dirname(path);
+	    if(parent_path != NULL)
 	    {
-		/* Current location has been removed, change the
-		 * current location to the parent
-		 */
-		gchar *parent_path = g_dirname(path);
-		if(parent_path != NULL)
-		{
-		    EDVImbrSetTitle(imbr, parent_path);
-		    EDVImbrSetLocation(imbr, parent_path, FALSE);
-		    EDVImbrUpdateLocationIcon(imbr, parent_path);
-		    g_free(parent_path);
-		}
+		EDVImbrSetTitle(imbr, parent_path);
+		EDVImbrSetLocation(imbr, parent_path, FALSE);
+		EDVImbrUpdateLocationIcon(imbr, parent_path);
+		EDVImbrTListGetListing(imbr, parent_path, TRUE);
+		g_free(parent_path);
 	    }
-
-	    g_free(cur_loc);
 	}
 
 /*	EDVImbrUpdateMenus(imbr); */
-}
 
+	g_free(cur_path);
+}
 
 /*
  *	Device mount/unmount callback.
@@ -1647,15 +1927,15 @@ void EDVImbrReconfiguredNotifyCB(edv_imbr_struct *imbr)
 
 	/* Begin updating */
 
-	/* Update Title */
+	/* Update the title */
 	EDVImbrSetTitle(imbr, cur_loc);
 
-	/* Recreate Accelkey Labels */
-	EDVImbrAccelkeysRegenerate(imbr);
-
-	/* Recreate Tool Bar */
+	/* Recreate the Tool Bar */
 	EDVImbrToolbarRegenerate(imbr);
 
+	/* Recreate the File menu items */
+	EDVImbrFileMenuItemsRegenerate(imbr);
+
 	/* Show/hide Tool Bar */
 	w = imbr->tool_bar_handle;
 	if(w != NULL)
@@ -1715,6 +1995,8 @@ void EDVImbrReconfiguredNotifyCB(edv_imbr_struct *imbr)
 	tlist = imbr->tlist;
 	if(tlist != NULL)
 	{
+	    GtkOrientation orientation;
+
 	    TListFreeze(tlist);
 
 	    TListThumbGeometry(
@@ -1726,9 +2008,9 @@ void EDVImbrReconfiguredNotifyCB(edv_imbr_struct *imbr)
 	    TListDoubleBuffer(
 		tlist, EDV_GET_B(EDV_CFG_PARM_LISTS_DOUBLE_BUFFER)
 	    );
-	    TListOrientation(
-		tlist, EDV_GET_B(EDV_CFG_PARM_IMBR_THUMB_LIST_HORIZONTAL)
-	    );
+	    orientation = (EDV_GET_B(EDV_CFG_PARM_IMBR_THUMB_LIST_HORIZONTAL)) ?
+		GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
+	    TListOrientation(tlist, orientation);
 	    TListShowThumbFrames(
 		tlist, EDV_GET_B(EDV_CFG_PARM_IMBR_THUMB_SHOW_FRAMES)
 	    );
@@ -1794,8 +2076,26 @@ void EDVImbrReconfiguredNotifyCB(edv_imbr_struct *imbr)
 	    ImgViewUpdateMenus(iv);
 	}
 
-	/* Recreate New Object Submenu */
-	EDVImbrNewObjectMenuRegenerate(imbr);
+	/* Recreate the New Object Submenu */
+	EDVImbrNewObjectMenuItemsRegenerate(imbr);
+
+	/* Recreate the Open To menus */
+	if(imbr->tlist_selected_thumb > -1)
+	{
+	    const gint thumb_num = imbr->tlist_selected_thumb;
+	    edv_object_struct *obj = EDV_OBJECT(TListGetThumbData(
+		imbr->tlist, thumb_num
+	    ));
+	    const gchar *path = (obj != NULL) ? obj->full_path : NULL;
+	    edv_mime_type_struct *m = EDVMimeTypesListMatchPath(
+		core->mimetype, core->total_mimetypes,
+		path
+	    );
+	    EDVImbrOpenToMenuRegenerate(imbr, m);
+	}
+
+	/* Recreate the Accelkey labels */
+	EDVImbrAccelkeysRegenerate(imbr);
 
 	/* Update RC styles */
 	w = imbr->toplevel;
