@@ -1,5 +1,6 @@
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <sys/stat.h>
 #if defined(HAVE_REGEX)
 # include <regex.h>
@@ -44,12 +45,13 @@
 #include "config.h"
 
 
-static edv_object_struct *EDVImbrNewErrorObject(
-	const gchar *path
-);
+typedef struct _EDVImbrTListFPromptData	EDVImbrTListFPromptData;
+#define EDV_IMBR_TLIST_FPROMPT_DATA(p)	((EDVImbrTListFPromptData *)(p))
 
-/* Callbacks */
-static void EDVImbrTListFPromptCancelCB(gpointer data);
+
+static edv_object_struct *EDVImbrNewErrorObject(
+	const gchar *path, const gchar *error_msg
+);
 
 /* Image Loading */
 static guint8 *EDVImbrTListConvertRGBToRGBA(
@@ -63,13 +65,14 @@ static guint8 *EDVImbrTListLoadImageRGBATarga(
 	GdkWindow *window			/* Reference window */
 );
 GList *EDVImbrTListLoadImageRGBA(
-	tlist_struct *tlist, const gchar *path,
+	edv_core_struct *core,
+	tlist_struct *tlist,
+	const gchar *path,
 	gint *width, gint *height, gint *bpl,
 	gint *nframes, GList **delay_list,
-	const gboolean resize_for_thumb,	/* Resize for tlist thumb */
-	const gboolean no_enlarge,		/* No enlarge if smaller
-						 * than req size */
-	GdkWindow *window			/* Reference window */
+	const gboolean resize_for_thumb,
+	const gboolean no_enlarge,
+ 	GdkWindow *window
 );
 
 static void EDVImbrTListSetThumbName(
@@ -126,7 +129,8 @@ void EDVImbrTListOpenWith(
 static void EDVImbrTListFPromptRenameApplyCB(
 	gpointer data, const gchar *value
 );
-void EDVImbrTListDoFPromptRename(
+static void EDVImbrTListFPromptRenameCancelCB(gpointer data);
+void EDVImbrTListPromptRename(
 	edv_imbr_struct *imbr, const gint thumb_num
 );
 
@@ -151,6 +155,13 @@ void EDVImbrTListMountNotify(
 );
 
 
+struct _EDVImbrTListFPromptData {
+	edv_imbr_struct	*imbr;
+	tlist_struct	*tlist;
+	gint		thumb_num;
+};
+
+
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -169,7 +180,7 @@ void EDVImbrTListMountNotify(
  *	The path specifies the full path to the error object.
  */
 static edv_object_struct *EDVImbrNewErrorObject(
-	const gchar *path
+	const gchar *path, const gchar *error_msg
 )
 {
 	edv_object_struct *obj = EDVObjectNew();
@@ -179,24 +190,18 @@ static edv_object_struct *EDVImbrNewErrorObject(
 	obj->type = EDV_OBJECT_TYPE_ERROR;
 	EDVObjectSetPath(obj, path);
 	EDVObjectUpdateLinkFlags(obj);
+	if(!STRISEMPTY(error_msg))
+	    EDVObjectPropSet(
+		obj,
+		EDV_OBJECT_PROP_NAME_ERROR,
+		error_msg,
+		TRUE
+	    );
 
 	return(obj);
 }
 
 
-/*
- *	All purpose fprompt cancel callback.
- */
-static void EDVImbrTListFPromptCancelCB(gpointer data)
-{
-	gpointer *cb_data = (gpointer *)data;
-	if(cb_data == NULL)
-	    return;
-
-	g_free(cb_data);
-}
-
-
 /*
  *	Converts the specified RGB and Alpha image data to RGBA.
  */
@@ -310,27 +315,49 @@ static guint8 *EDVImbrTListLoadImageRGBATarga(
 }
 
 /* 
- *	Opens the image specified by path and returns the RGBA image
- *	data and its geometry.
+ *	Opens the image data.
  *
- *	The specified Thumbs List can be NULL if resize_for_thumb is
- *	FALSE.
+ *	The tlist specifies the thumbs list. If tlist is NULL then
+ *	the image will not be resized even if resize_for_thumb is
+ *	TRUE.
  *
- *	Returns the RGBA image data or NULL on error.
+ *	The path specifies the image to be opened.
+ *
+ *	The width and height specifies the image geometry return values.
+ *
+ *	The bpl specifies the bytes per line return value.
+ *
+ *	The nframes specifies the number of frames return value.
+ *
+ *	The delay_list specifies the delay GList return value. Each
+ *	value in the GList will be a gulong.
+ *
+ *	If resize_for_thumb is TRUE then the image will be resized
+ *	to the thumb size specified by the thumbs list.
+ *
+ *	If no_enlarge is TRUE then the image will not be enlarged
+ *	if it is smalled than the thumb sized specified by the
+ *	thumbs list.
+ *
+ *	The window specifies the reference GdkWindow.
+ *
+ *	Returns a list of guint8 * RGBA image datas or NULL on error.
  */
 GList *EDVImbrTListLoadImageRGBA(
-	tlist_struct *tlist, const gchar *path,
+	edv_core_struct *core,
+	tlist_struct *tlist,
+	const gchar *path,
 	gint *width, gint *height, gint *bpl,
 	gint *nframes, GList **delay_list,
-	const gboolean resize_for_thumb,	/* Resize for tlist thumb */
-	const gboolean no_enlarge,		/* No enlarge if smaller
-						 * than req size */
- 	GdkWindow *window			/* Reference window */
+	const gboolean resize_for_thumb,
+	const gboolean no_enlarge,
+ 	GdkWindow *window
 )
 {
-#ifdef HAVE_IMLIB
+#if defined(HAVE_IMLIB)
 	const gchar *name;
 	gint swidth, sheight, twidth, theight;
+	gpointer imlib_handle;
 	GList *rgba_list = NULL;
 	ImlibImage *imlib_image;
 
@@ -346,9 +373,10 @@ GList *EDVImbrTListLoadImageRGBA(
 	if(delay_list != NULL)
 	    *delay_list = NULL;
 
-	if(STRISEMPTY(path))
+	if((core == NULL) || STRISEMPTY(path))
 	    return(NULL);
 
+	imlib_handle = core->imlib_handle;
 	name = g_basename(path);
 
 	/* Begin checking extension to see if we should use a native
@@ -428,16 +456,16 @@ GList *EDVImbrTListLoadImageRGBA(
 	    return(rgba_list);
 
 
-	/* Load Imlib image */
+	/* Open the Imlib image */
 	imlib_image = (imlib_handle != NULL) ?
 	    Imlib_load_image(imlib_handle, (char *)path) : NULL;
 	if(imlib_image == NULL)
 	    return(NULL);
 
-	/* Need to realize changes */
+	/* Need to realize the changes */
 	Imlib_changed_image(imlib_handle, imlib_image);
 
-	/* Get size of image */
+	/* Get the size of the image */
 	swidth = imlib_image->rgb_width;
 	sheight = imlib_image->rgb_height;
 
@@ -651,8 +679,11 @@ static void EDVImbrTListSetThumbName(
 		/* Directory not accessable? */
 		if(!EDVIsObjectAccessable(core, obj))
 		{
-		    pixmap = imbr->folder_noaccess_pixmap;
-		    mask = imbr->folder_noaccess_mask;
+		    if(pixmap_ext != NULL)
+		    {
+			pixmap = pixmap_ext;
+			mask = mask_ext;
+		    }
 		    if(text_color_fg == NULL)
 		    {
 			const GdkColor cs = EDV_GDKCOLOR_NO_ACCESS_FG;
@@ -765,9 +796,13 @@ static void EDVImbrTListSetThumbName(
 	}
 
 	/* Set thumb's text */
+	TListFreeze(tlist);
 	TListSetText(tlist, thumb_num, name);
 	if(text_color_fg != NULL)
 	    TListSetTextColor(tlist, thumb_num, text_color_fg, NULL);
+	else
+	    TListSetTextColor(tlist, thumb_num, NULL, NULL);
+	TListThaw(tlist);
 
 	g_free(text_color_fg);
 }
@@ -791,10 +826,15 @@ static gint EDVImbrTListAppendObject(
 	edv_object_struct *obj
 )
 {
+	gint thumb_num;
+
+	TListFreeze(tlist);
+
 	/* Append the new Thumb */
-	gint thumb_num = TListAppend(tlist, "");
+	thumb_num = TListAppend(tlist, "");
 	if(thumb_num < 0)
 	{
+	    TListThaw(tlist);
 	    EDVObjectDelete(obj);
 	    return(-1);
 	}
@@ -816,6 +856,8 @@ static gint EDVImbrTListAppendObject(
 	    thumb_num
 	);
 
+	TListThaw(tlist);
+
 	return(thumb_num);
 }
 
@@ -828,28 +870,24 @@ static gint EDVImbrTListAppendObject(
  *	The index specifies the index on the device that the object
  *	resides at.
  *
- *	Returns the matched thumb or -1 on error.
+ *	Returns the matched thumb or negative on error.
  */
 gint EDVImbrTListFindThumbByIndex(
 	edv_imbr_struct *imbr, 
 	const gulong device_index, const gulong index
 )
 {
-	gint i;
+	gint i, n;
 	tlist_struct *tlist;
 	edv_object_struct *obj;
 
 	if(imbr == NULL)
-	    return(-1);
+	    return(-2);
 
 	tlist = imbr->tlist;
-	if(tlist == NULL)
-	    return(-1);
 
-	/* Iterate through the thumbs list and look for a thumb
-	 * who's object matches the specified location index
-	 */
-	for(i = 0; i < tlist->total_thumbs; i++)
+	n = tlist->total_thumbs;
+	for(i = 0; i < n; i++)
 	{
 	    obj = EDV_OBJECT(TListGetThumbData(tlist, i));
 	    if(obj == NULL)
@@ -870,34 +908,33 @@ gint EDVImbrTListFindThumbByIndex(
  *
  *	The path specifies the full path of the object.
  *
- *	Returns the matched thumb or -1 on error.
+ *	Returns the matched thumb or negative on error.
  */
 gint EDVImbrTListFindThumbByPath(
 	edv_imbr_struct *imbr, const gchar *path
 )
 {
-	gint i;
+	gint i, n;
 	tlist_struct *tlist;
 	edv_object_struct *obj;
 
 	if((imbr == NULL) || (path == NULL))
-	    return(-1);
+	    return(-2);
 
 	tlist = imbr->tlist;
-	if(tlist == NULL)
-	    return(-1);
 
-	/* Iterate through the thumbs list and look for a thumb
-	 * who's object matches the specified path
-	 */
-	for(i = 0; i < tlist->total_thumbs; i++)
+	n = tlist->total_thumbs;
+	for(i = 0; i < n; i++)
 	{
 	    obj = EDV_OBJECT(TListGetThumbData(tlist, i));
-	    if((obj != NULL) ? STRISEMPTY(obj->full_path) : TRUE)
+	    if(obj == NULL)
+		continue;
+
+	    if(obj->full_path == NULL)
 		continue;
 
 	    /* Full paths match? */
-	    if(!strcmp(obj->full_path, path))
+	    if(!strcmp((const char *)obj->full_path, (const char *)path))
 		return(i);
 	}
 
@@ -912,12 +949,17 @@ gint EDVImbrTListFindThumbByPath(
  */
 GList *EDVImbrTListGetSelectedPaths(edv_imbr_struct *imbr)
 {
-	GList *glist, *paths_list = NULL;
+	GList *glist, *paths_list;
+	tlist_struct *tlist;
 	edv_object_struct *obj;
-	tlist_struct *tlist = (imbr != NULL) ? imbr->tlist : NULL;
-	if(tlist == NULL)
-	    return(paths_list);
 
+	if(imbr == NULL)
+	    return(NULL);
+
+	tlist = imbr->tlist;
+
+	/* Create the selected paths list */
+	paths_list = NULL;
 	for(glist = tlist->selection;
 	    glist != NULL;
 	    glist = g_list_next(glist)
@@ -926,7 +968,10 @@ GList *EDVImbrTListGetSelectedPaths(edv_imbr_struct *imbr)
 	    obj = EDV_OBJECT(TListGetThumbData(
 		tlist, (gint)glist->data
 	    ));
-	    if((obj != NULL) ? STRISEMPTY(obj->full_path) : TRUE)
+	    if(obj == NULL)
+		continue;
+
+	    if(STRISEMPTY(obj->full_path))
 		continue;
 
 	    paths_list = g_list_append(
@@ -952,7 +997,7 @@ GList *EDVImbrTListGetSelectedPaths(edv_imbr_struct *imbr)
  */
 void EDVImbrTListRealizeListing(edv_imbr_struct *imbr)
 {
-	gint i;
+	gint i, n;
 	tlist_thumb_struct *thumb;
 	tlist_struct *tlist;
 	edv_object_struct *obj;
@@ -963,10 +1008,10 @@ void EDVImbrTListRealizeListing(edv_imbr_struct *imbr)
 
 	tlist = imbr->tlist;
 	core = imbr->core;
-	if((tlist == NULL) || (core == NULL))
-	    return;
 
-	for(i = 0; i < tlist->total_thumbs; i++)
+	TListFreeze(tlist);
+	n = tlist->total_thumbs;
+	for(i = 0; i < n; i++)
 	{
 	    thumb = tlist->thumb[i];
 	    if(thumb == NULL)
@@ -978,6 +1023,7 @@ void EDVImbrTListRealizeListing(edv_imbr_struct *imbr)
 		    core, imbr, tlist, obj, i
 		);
 	}
+	TListThaw(tlist);
 }
 
 /*
@@ -1010,9 +1056,6 @@ void EDVImbrTListGetListing(
 	tlist = imbr->tlist;
 	sb = imbr->status_bar;
 	core = imbr->core;
-	if((tlist == NULL) || (core == NULL))
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	hide_object_hidden = !EDV_GET_B(
@@ -1148,12 +1191,13 @@ void EDVImbrTListGetListing(
 #endif
 
 		    /* Get this directory's local stats */
-		    if(lstat((char *)full_path, &stat_buf))
+		    if(lstat((const char *)full_path, &stat_buf))
 		    {
 			/* Unable to get the directory's local stats
 			 * so add it as an error object
 			 */
-			obj = EDVImbrNewErrorObject(full_path);
+			const gint error_code = (gint)errno;
+			obj = EDVImbrNewErrorObject(full_path, g_strerror(error_code));
 			if(obj != NULL)
 			    EDVImbrTListAppendObject(
 				core, imbr, tlist,
@@ -1268,7 +1312,8 @@ void EDVImbrTListGetListing(
 			/* Unable to get the object's local stats
 			 * so add it as an error object
 			 */
-			obj = EDVImbrNewErrorObject(full_path);
+			const gint error_code = (gint)errno;
+			obj = EDVImbrNewErrorObject(full_path, g_strerror(error_code));
 			if(obj != NULL)
 			    EDVImbrTListAppendObject(
 				core, imbr, tlist,
@@ -1352,9 +1397,6 @@ void EDVImbrTlistClear(edv_imbr_struct *imbr)
 	    return;
 
 	tlist = imbr->tlist;
-	if(tlist == NULL)
-	    return;
-
 	TListFreeze(tlist);
 	TListClear(tlist);
 	TListThaw(tlist);
@@ -1388,7 +1430,7 @@ gint EDVImbrTListLoadIterate(
 	gint thumbs_already_loaded = 0;
 	GtkWidget *toplevel;
 	tlist_thumb_struct *thumb, *thumb_to_load = NULL;
-	 tlist_struct *tlist;
+	tlist_struct *tlist;
 	edv_object_struct *obj;
 	edv_status_bar_struct *sb;
 	edv_core_struct *core;
@@ -1407,11 +1449,6 @@ gint EDVImbrTListLoadIterate(
 	tlist = imbr->tlist;
 	sb = imbr->status_bar;
 	core = imbr->core;
-	if((toplevel == NULL) || (tlist == NULL) || (core == NULL))
-	{
-	    DO_RESET_PROGRESS
-	    return(-1);
-	}
 
 	/* Count number of thumbs already loaded */
 	if(update_status_bar)
@@ -1496,9 +1533,15 @@ gint EDVImbrTListLoadIterate(
 	    return(0);
 	}
 
-	/* Get object from thumb data */
+	/* Get the object from the thumb data */
 	obj = EDV_OBJECT(thumb_to_load->data);
-	if((obj != NULL) ? STRISEMPTY(obj->full_path) : TRUE)
+	if(obj == NULL)
+	{
+	    thumb_to_load->load_state = TLIST_LOAD_STATE_FAILED;
+	    DO_RESET_PROGRESS
+	    return(-1);
+	}
+	if(STRISEMPTY(obj->full_path))
 	{
 	    thumb_to_load->load_state = TLIST_LOAD_STATE_FAILED;
 	    DO_RESET_PROGRESS
@@ -1516,15 +1559,17 @@ gint EDVImbrTListLoadIterate(
 
 	    if(update_status_bar)
 	    {
+		const gulong size = obj->size;
 		const gfloat progress = (tlist->total_thumbs > 0) ?
 		    ((gfloat)thumbs_already_loaded / (gfloat)tlist->total_thumbs) : 0.0f;
-		gchar *buf = g_strdup_printf(
-		    "Loading \"%s\" (%ld byte%s)",
-		    name, obj->size,
-		    (obj->size == 1) ? "" : "s"
+		gchar *msg = g_strdup_printf(
+		    "Loading \"%s\" (%s %s)",
+		    name,
+		    EDVSizeStrDelim(size),
+		    (size == 1l) ? "byte" : "bytes"
 		);
-		EDVStatusBarMessage(sb, buf, FALSE);
-		g_free(buf);
+		EDVStatusBarMessage(sb, msg, FALSE);
+		g_free(msg);
 
 		EDVStatusBarProgress(sb, progress, FALSE);
 	    }
@@ -1533,7 +1578,9 @@ gint EDVImbrTListLoadIterate(
 
 	    /* Load image to RGBA data */
 	    rgba_list = EDVImbrTListLoadImageRGBA(
-		tlist, full_path,
+		core,
+		tlist,
+		full_path,
 		&width, &height, &bpl, &nframes, &delay_list,
 		TRUE,			/* Resize for tlist thumb */
 		no_enlarge,		/* Do not enlarge? */
@@ -1546,11 +1593,13 @@ gint EDVImbrTListLoadIterate(
 		 * this thumb's pixmap
 		 */
 		thumb_to_load->load_state = TLIST_LOAD_STATE_FAILED;
+		TListFreeze(tlist);
 		TListSetPixmap(
 		    tlist, thumb_to_load_num,
 		    imbr->file_badimage_pixmap,
 		    imbr->file_badimage_mask
 		);
+		TListThaw(tlist);
 
 		DO_RESET_PROGRESS
 		return(-1);
@@ -1560,6 +1609,7 @@ gint EDVImbrTListLoadIterate(
 	    thumb_to_load->load_state = TLIST_LOAD_STATE_LOADED;
 
 	    /* Set image to thumb */
+	    TListFreeze(tlist);
 	    TListSetRGBA(
 		tlist, thumb_to_load_num,
 		width, height, bpl,
@@ -1567,6 +1617,7 @@ gint EDVImbrTListLoadIterate(
 		(guint8 *)rgba_list->data,	/* Use first image */
 		no_enlarge
 	    );
+	    TListThaw(tlist);
 
 	    /* Delete the loaded images */
 	    g_list_foreach(rgba_list, (GFunc)g_free, NULL);
@@ -1602,8 +1653,6 @@ void EDVImbrTListOpen(
 
 	toplevel = imbr->toplevel;
 	core = imbr->core;
-	if(core == NULL)
-	    return;
 
 	/* Use selected objects? */
 	if(thumb_num < 0)
@@ -1619,8 +1668,12 @@ void EDVImbrTListOpen(
 	    if(obj == NULL)
 		return;
 
+	    if(STRISEMPTY(obj->full_path))
+		return;
+
+	    paths_list = NULL;
 	    paths_list = g_list_append(
-		NULL,
+		paths_list,
 		STRDUP(obj->full_path)
 	    );
 	}
@@ -1632,7 +1685,13 @@ void EDVImbrTListOpen(
 	    gchar *path = EDVEvaluatePath(
 		NULL, (gchar *)paths_list->data
 	    );
-	    if((path != NULL) ? stat(path, &stat_buf) : TRUE)
+	    if(path == NULL)
+	    {
+		g_list_foreach(paths_list, (GFunc)g_free, NULL);
+		g_list_free(paths_list);
+		return;
+	    }
+	    if(stat((const char *)path, &stat_buf))
 	    {
 		g_list_foreach(paths_list, (GFunc)g_free, NULL);
 		g_list_free(paths_list);
@@ -1679,10 +1738,10 @@ void EDVImbrTListOpen(
 
 	    g_free(stdout_path_rtn);
 	    g_free(stderr_path_rtn);
-	}
 
-	g_list_foreach(paths_list, (GFunc)g_free, NULL);
-	g_list_free(paths_list);
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
+	}
 }
 
 /*
@@ -1703,8 +1762,6 @@ void EDVImbrTListOpenWith(
 
 	toplevel = imbr->toplevel;
 	core = imbr->core;
-	if(core == NULL)
-	    return;
 
 	/* Use selected objects? */
 	if(thumb_num < 0)
@@ -1721,8 +1778,12 @@ void EDVImbrTListOpenWith(
 	    if(obj == NULL)
 		return;
 
+	    if(STRISEMPTY(obj->full_path))
+		return;
+
+	    paths_list = NULL;
 	    paths_list = g_list_append(
-		NULL,
+		paths_list,
 		STRDUP(obj->full_path)
 	    );
 	}
@@ -1744,40 +1805,39 @@ void EDVImbrTListOpenWith(
 
 	    g_free(stdout_path_rtn);
 	    g_free(stderr_path_rtn);
-	}
 
-	g_list_foreach(paths_list, (GFunc)g_free, NULL);
-	g_list_free(paths_list);
+	    g_list_foreach(paths_list, (GFunc)g_free, NULL);
+	    g_list_free(paths_list);
+	}
 }
 
 
 /*
- *      FPrompt apply callback, set in EDVImbrTListDoFPromptRename().
+ *      FPrompt apply callback, set in EDVImbrTListPromptRename().
  */
 static void EDVImbrTListFPromptRenameApplyCB(
 	gpointer data, const gchar *value
 )
 {
 	gint thumb_num;
-	GtkWidget *toplevel;
 	tlist_struct *tlist;
+	edv_status_bar_struct *sb;
 	edv_imbr_struct *imbr;
-	gpointer *cb_data = (gpointer *)data;
-	if(cb_data == NULL)
-	    return;
+        EDVImbrTListFPromptData *d = EDV_IMBR_TLIST_FPROMPT_DATA(data);
+	if(d == NULL)
+            return;
 
-	/* Get callback data */
-	imbr = EDV_IMBR(cb_data[0]);
-	tlist = TLIST(cb_data[1]);
-	thumb_num = (gint)cb_data[2];
-
-	toplevel = imbr->toplevel;
+	imbr = d->imbr;
+	tlist = d->tlist;
+	thumb_num = d->thumb_num;
+	sb = imbr->status_bar;
 
 	/* Inputs valid? */
 	if((imbr != NULL) && (tlist != NULL) && (thumb_num >= 0) &&
 	   (value != NULL)
 	)
 	{
+	    GtkWidget *toplevel = imbr->toplevel;
 	    edv_core_struct *core = imbr->core;
 	    edv_object_struct *obj = EDV_OBJECT(
 		TListGetThumbData(tlist, thumb_num)
@@ -1802,23 +1862,20 @@ static void EDVImbrTListFPromptRenameApplyCB(
 		    &yes_to_all
 		);
 
-		/* Unmap progress dialog, it may have been mapped in the
-		 * above operation
-		 */
+		/* Unmap the progress dialog */
 		ProgressDialogBreakQuery(FALSE);
 		ProgressDialogSetTransientFor(NULL);
 
-		/* Get error message if any that might have occured in the
-		 * above operation
-		 */
-		error_msg = EDVObjectOPGetError();
+		/* Check for errors */
+		error_msg = EDVObjectOPGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
-			"Rename Object Error",
+		    EDVMessageError(
+			"Rename Error",
 			error_msg,
-			old_full_path,
+			NULL,
 			toplevel
 		    );
 		}
@@ -1839,15 +1896,15 @@ static void EDVImbrTListFPromptRenameApplyCB(
 			if(modified_path == NULL)
 			    continue;
 
-			if(!lstat(modified_path, &lstat_buf))
+			if(!lstat((const char *)modified_path, &lstat_buf))
 			{
-			    gchar *s = g_strdup_printf(
+			    gchar *msg = g_strdup_printf(
 				"Object \"%s\" renamed to \"%s\"",
 				g_basename(old_full_path),
 				g_basename(modified_path)
 			    );
-			    EDVStatusBarMessage(imbr->status_bar, s, FALSE);
-			    g_free(s);
+			    EDVStatusBarMessage(sb, msg, FALSE);
+			    g_free(msg);
 
 			    EDVObjectModifiedEmit(
 				core,
@@ -1858,8 +1915,13 @@ static void EDVImbrTListFPromptRenameApplyCB(
 			}
 		    }
 
-		    g_list_foreach(modified_paths_list, (GFunc)g_free, NULL);
-		    g_list_free(modified_paths_list);
+		    if(modified_paths_list != NULL)
+		    {
+			g_list_foreach(
+			    modified_paths_list, (GFunc)g_free, NULL
+			);
+			g_list_free(modified_paths_list);
+		    }
 		}
 		else
 		{
@@ -1867,7 +1929,7 @@ static void EDVImbrTListFPromptRenameApplyCB(
 		     * implies failure
 		     */
 		    EDVStatusBarMessage(
-			imbr->status_bar,
+			sb,
 			"Rename object failed",
 			FALSE
 		    );
@@ -1877,17 +1939,27 @@ static void EDVImbrTListFPromptRenameApplyCB(
 	    }
 	}
 
-	/* Delete the callback data, it is no longer needed */
-	g_free(cb_data);
+	g_free(d);
 }
 
 /*
- *	Procedure to map the floating prompt for renaming of the row
- *	specified by row.
- *
- *      If the column is -1 then it implies any column is valid.
+ *	FPrompt rename cancel callback.
  */
-void EDVImbrTListDoFPromptRename(
+static void EDVImbrTListFPromptRenameCancelCB(gpointer data)
+{
+        EDVImbrTListFPromptData *d = EDV_IMBR_TLIST_FPROMPT_DATA(data);
+	if(d == NULL)
+            return;
+
+	g_free(d);
+}
+
+/*
+ *	Prompt the user to rename an object.
+ *
+ *	The thumb_num specifies the thumb index.
+ */
+void EDVImbrTListPromptRename(
 	edv_imbr_struct *imbr, const gint thumb_num
 )
 {
@@ -1901,8 +1973,6 @@ void EDVImbrTListDoFPromptRename(
 
 	tlist = imbr->tlist;
 	core = imbr->core;
-	if((tlist == NULL) || (core == NULL))
-	    return;
 
 	/* Check and warn if write protect is enabled */
 	if(EDVCheckWriteProtect(core, TRUE, imbr->toplevel))
@@ -1936,9 +2006,9 @@ void EDVImbrTListDoFPromptRename(
 	if(obj == NULL)
 	    return;
 
-	/* Check if object name is a special notation that should not
-	 * be allowed to be renamed
-	 */
+        /* Check if the object's name is a special notation that
+         * may not be renamed
+         */
 	if(obj->name != NULL)
 	{
 	    const gchar *name = obj->name;
@@ -1950,17 +2020,15 @@ void EDVImbrTListDoFPromptRename(
 
 	if(TRUE)
 	{
-	    gpointer *cb_data = (gpointer *)g_malloc0(
-		3 * sizeof(gpointer)
-	    );
 	    gchar *value = STRDUP(obj->name);
-
-	    /* Set up callback data */
-	    if(cb_data != NULL)
+	    EDVImbrTListFPromptData *d = EDV_IMBR_TLIST_FPROMPT_DATA(
+		g_malloc0(sizeof(EDVImbrTListFPromptData))
+	    );
+	    if(d != NULL)
 	    {
-		cb_data[0] = imbr;
-		cb_data[1] = tlist;
-		cb_data[2] = (gpointer)thumb_num;
+		d->imbr = imbr;
+		d->tlist = tlist;
+		d->thumb_num = thumb_num;
 	    }
 
 	    /* Map floating prompt to change values */
@@ -1973,18 +2041,12 @@ void EDVImbrTListDoFPromptRename(
 		FPROMPT_MAP_NO_MOVE,	/* Map code */
 		pwidth, -1,		/* Width and height */
 		0,			/* Flags */
-		(gpointer)cb_data,	/* Callback data */
+		d,			/* Callback data */
 		NULL,			/* No browse callback */
 		EDVImbrTListFPromptRenameApplyCB,
-		EDVImbrTListFPromptCancelCB
+		EDVImbrTListFPromptRenameCancelCB
 	    );
 
-	    /* Do not reference cb_data after this call, it will be passed
-	     * to the callbacks where it will be deallocated.
-	     */
-	    cb_data = NULL;
-
-	    /* Delete original value */
 	    g_free(value);
 	}
 }
@@ -1999,85 +2061,87 @@ void EDVImbrTListObjectAddedNotify(
 )
 {
 	gint thumb_num;
-	const gchar *cur_path;
-	gchar *parent_path;
+	gchar *cur_path, *parent_path;
 	tlist_struct *tlist;
-	edv_object_struct *obj;
 	edv_core_struct *core;
 
-	if((imbr == NULL) || (path == NULL) || (lstat_buf == NULL))
+	if((imbr == NULL) || STRISEMPTY(path) || (lstat_buf == NULL))
+	    return;
+
+	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
+	if(cur_path == NULL)
 	    return;
 
-	cur_path = EDVImbrCurrentLocation(imbr);
 	tlist = imbr->tlist;
 	core = imbr->core;
-	if((cur_path == NULL) || (tlist == NULL) || (core == NULL))
-	    return;
 
-	parent_path = g_dirname(path);
-	if(parent_path == NULL)
-	    return;
-
-	/* Skip if the parent of the added path the same as the
-	 * current location
-	 */
-	if(strcmp(cur_path, parent_path))
+	/* Is the added path the same as the current location? */
+	if(!strcmp((const char *)path, (const char *)cur_path))
 	{
-	    g_free(parent_path);
+	    /* Reget the listing */
+	    EDVImbrTListGetListing(imbr, path, TRUE);
+	    g_free(cur_path);
 	    return;
 	}
 
-	/* Check if the added object refers to an object that is
-	 * already listed
-	 */
-	thumb_num = EDVImbrTListFindThumbByIndex(
-	    imbr,
-	    (gulong)lstat_buf->st_dev,
-	    (gulong)lstat_buf->st_ino
-	);
+	/* Update the object who's path matches the added path */
+	thumb_num = EDVImbrTListFindThumbByPath(imbr, path);
 	if(thumb_num > -1)
 	{
-	    /* Update the existing object */
-	    obj = EDV_OBJECT(TListGetThumbData(tlist, thumb_num));
+	    edv_object_struct *obj = EDV_OBJECT(
+		TListGetThumbData(tlist, thumb_num)
+	    );
 	    if(obj != NULL)
 	    {
 		EDVObjectSetPath(obj, path);
 		EDVObjectSetStat(obj, lstat_buf);
 		EDVObjectUpdateLinkFlags(obj);
 
-		/* Update the thumb's displayed name */
+		/* Update this thumb's values */
 		EDVImbrTListSetThumbName(
 		    core, imbr, tlist, obj, thumb_num
 		);
 	    }
+	    g_free(cur_path);
+	    return;
 	}
-	else
+
+	/* Get the parent path of the added path */
+	parent_path = g_dirname(path);
+	if(parent_path != NULL)
 	{
-	    /* Add the new object */
-	    obj = EDVObjectNew();
-	    if(obj != NULL)
-	    {
-		EDVObjectSetPath(obj, path);
-		EDVObjectSetStat(obj, lstat_buf);
-		EDVObjectUpdateLinkFlags(obj);
-
-		TListFreeze(tlist);
-
-		thumb_num = EDVImbrTListAppendObject(
-		    core, imbr, tlist,
-		    obj
-		);
-
-		TListThaw(tlist);
-	    }
-
-	    /* Need to (re)queue loading process so that any newly
-	     * added thumb gets its image loaded
+	    /* Is the parent directory of the added object the same as
+	     * the current location?
 	     */
-	    EDVImbrQueueLoadingProcess(imbr);
+	    if(!strcmp((const char *)parent_path, (const char *)cur_path))
+	    {
+		/* Add this object to the list */
+		edv_object_struct *obj = EDVObjectNew();
+		if(obj != NULL)
+		{
+		    gint new_thumb_num;
+
+		    EDVObjectSetPath(obj, path);
+		    EDVObjectSetStat(obj, lstat_buf);
+		    EDVObjectUpdateLinkFlags(obj);
+
+		    new_thumb_num = EDVImbrTListAppendObject(
+			core, imbr, tlist,
+			obj
+		    );
+		    if(new_thumb_num > -1)
+		    {
+			/* Need to (re)queue loading process so that
+			 * the added thumb gets its image loaded
+			 */
+			EDVImbrQueueLoadingProcess(imbr);
+		    }
+		}
+	    }
+	    g_free(parent_path);
 	}
 
-	g_free(parent_path);
+	g_free(cur_path);
 }
 
 /*
@@ -2089,64 +2153,55 @@ void EDVImbrTListObjectModifiedNotify(
 	const struct stat *lstat_buf
 )
 {
-	const gchar *cur_path;
-	gchar *parent_path;
+	gint thumb_num;
+	gchar *cur_path;
 	tlist_struct *tlist;
 	edv_core_struct *core;
 
-	if((imbr == NULL) || (path == NULL) || (lstat_buf == NULL))
+	if((imbr == NULL) || STRISEMPTY(path) || (lstat_buf == NULL))
 	    return;
 
 	if(new_path == NULL)
 	    new_path = path;
 
-	cur_path = EDVImbrCurrentLocation(imbr);
+	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
+	if(cur_path == NULL)
+	    return;
+
 	tlist = imbr->tlist;
 	core = imbr->core;
-	if((tlist == NULL) || (core == NULL))
-	    return;
-
-	parent_path = g_dirname(path);
-	if(parent_path == NULL)
-	    return;
 
 	/* Is the new modified path the same as the current location? */
-	if(!strcmp(new_path, cur_path))
+	if(!strcmp((const char *)new_path, (const char *)cur_path))
 	{
 	    /* Reget the listing */
 	    EDVImbrTListGetListing(imbr, new_path, TRUE);
+	    g_free(cur_path);
+	    return;
 	}
-	/* Is the modified path's parent directory the same as the
-	 * current location?
-	 */
-	else if(!strcmp(parent_path, cur_path))
-	{
-	    /* Update the existing object */
-	    const gint thumb_num = EDVImbrTListFindThumbByIndex(
-		imbr,
-		(gulong)lstat_buf->st_dev,
-		(gulong)lstat_buf->st_ino
-	    );
-	    if(thumb_num > -1)
-	    {
-		edv_object_struct *obj = EDV_OBJECT(
-		    TListGetThumbData(tlist, thumb_num)
-		);
-		if(obj != NULL)
-		{
-		    EDVObjectSetPath(obj, new_path);		/* Use new_path */
-		    EDVObjectSetStat(obj, lstat_buf);
-		    EDVObjectUpdateLinkFlags(obj);
 
-		    /* Update the thumb's displayed name */
-		    EDVImbrTListSetThumbName(
-			core, imbr, tlist, obj, thumb_num
-		    );
-		}
+	/* Update the object who's path matches the modified path */
+	thumb_num = EDVImbrTListFindThumbByPath(imbr, path);
+	if(thumb_num > -1)
+	{
+	    /* Update this object */
+	    edv_object_struct *obj = EDV_OBJECT(
+		TListGetThumbData(tlist, thumb_num)
+	    );
+	    if(obj != NULL)
+	    {
+		EDVObjectSetPath(obj, new_path);	/* Use the new path */
+		EDVObjectSetStat(obj, lstat_buf);
+		EDVObjectUpdateLinkFlags(obj);
+
+		/* Update this thumb's values */
+		EDVImbrTListSetThumbName(
+		    core, imbr, tlist, obj, thumb_num
+		);
 	    }
 	}
 
-	g_free(parent_path);
+	g_free(cur_path);
 }
 
 /*
@@ -2156,49 +2211,44 @@ void EDVImbrTListObjectRemovedNotify(
 	edv_imbr_struct *imbr, const gchar *path
 )
 {
-	const gchar *cur_path;
-	gchar *parent_path;
+	gchar *cur_path;
 	tlist_struct *tlist;
 
-	if((imbr == NULL) || (path == NULL))
+	if((imbr == NULL) || STRISEMPTY(path))
+	    return;
+
+	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
+	if(cur_path == NULL)
 	    return;
 
-	cur_path = EDVImbrCurrentLocation(imbr);
 	tlist = imbr->tlist;
-	if(tlist == NULL)
-	    return;
-
-	parent_path = g_dirname(path);
-	if(parent_path == NULL)
-	    return;
 
 	/* Is the removed path the same as the current location? */
-	if(!strcmp(path, cur_path))
+	if(!strcmp((const char *)path, (const char *)cur_path))
 	{
 	    /* Clear the thumbs list */
 	    TListFreeze(tlist);
 	    TListClear(tlist);
 	    TListThaw(tlist);
 	}
-	/* Is the removed path's parent directory the same as the
-	 * current location?
-	 */
-	else if(!strcmp(parent_path, cur_path))
+	else
 	{
-	    /* Remove the existing object */
-	    const gint thumb_num = EDVImbrTListFindThumbByPath(
-		imbr,
-		path
-	    );
+	    /* Remove all the thumbs who's object's path matches the
+	     * removed path
+	     */
+	    gint thumb_num = EDVImbrTListFindThumbByPath(imbr, path);
 	    if(thumb_num > -1)
 	    {
 		TListFreeze(tlist);
-		TListRemove(tlist, thumb_num);
-		TListThaw(tlist);;
+		do {
+		    TListRemove(tlist, thumb_num);
+		    thumb_num = EDVImbrTListFindThumbByPath(imbr, path);
+		} while(thumb_num > -1);
+		TListThaw(tlist);
 	    }
 	}
 
-	g_free(parent_path);
+	g_free(cur_path);
 }
 
 
@@ -2217,8 +2267,6 @@ void EDVImbrTListMountNotify(
 	    return;
 
 	tlist = imbr->tlist;
-	if(tlist == NULL)
-	    return;
 
 	cur_path = STRDUP(EDVImbrCurrentLocation(imbr));
 	mount_path = STRDUP(dev->mount_path);
@@ -2235,9 +2283,9 @@ void EDVImbrTListMountNotify(
 	/* Check if mount path is a parent of the current location,
 	 * if it is then the Thumbs List needs to be updated
 	 */
-	if(strpfx(cur_path, mount_path))
+	if(strpfx((const char *)cur_path, (const char *)mount_path))
 	{
-	    /* Reget listing */
+	    /* Reget the listing */
 	    EDVImbrTListGetListing(imbr, cur_path, TRUE);
 	}
 
