@@ -23,13 +23,14 @@ extern gint EDVImbrTListFindThumbByPath(
 
 /* Image Loading */
 extern GList *EDVImbrTListLoadImageRGBA(
-	tlist_struct *tlist, const gchar *path,
+	edv_core_struct *core,
+	tlist_struct *tlist,
+	const gchar *path,
 	gint *width, gint *height, gint *bpl,
 	gint *nframes, GList **delay_list,
-	const gboolean resize_for_thumb,	/* Resize for tlist thumb */
-	const gboolean no_enlarge,		/* No enlarge if smaller
-						 * than req size */
-	GdkWindow *window			/* Reference window */
+	const gboolean resize_for_thumb,
+	const gboolean no_enlarge,
+	GdkWindow *window
 );
 
 /* Realize Listing */
@@ -59,7 +60,7 @@ extern void EDVImbrTListOpenWith(
 );
 
 /* Renaming */
-extern void EDVImbrTListDoFPromptRename(
+extern void EDVImbrTListPromptRename(
 	edv_imbr_struct *imbr, const gint thumb_num
 );
 
