@@ -10,6 +10,7 @@
 
 int main(int argc, char *argv[])
 {
+        /* Initialize the Endeavour2 Context */
 	edv_context_struct *ctx = EDVContextNew();
 	EDVContextLoadConfigurationFile(ctx, NULL);
 
@@ -17,7 +18,7 @@ int main(int argc, char *argv[])
 	{
 	    const gchar *type_str = argv[1];
 	    edv_history_type type;
-	    gint status = (argc > 2) ? atoi(argv[2]) : 0;
+	    const gint status = (argc > 2) ? atoi(argv[2]) : 0;
 	    const gchar *source = (argc > 3) ? argv[3] : NULL;
 	    const gchar *target = (argc > 4) ? argv[4] : NULL;
 	    const gchar *comments = (argc > 5) ? argv[5] : NULL;
@@ -78,7 +79,10 @@ int main(int argc, char *argv[])
 	    );
 	}
 
+        /* Flush any pending Endeavour2 operations */
 	EDVContextSync(ctx);
+
+        /* Shutdown the Endeavour2 context */
 	EDVContextDelete(ctx);
 
 	return(0);
