@@ -3,11 +3,11 @@
 #include <sys/stat.h>
 #include <glib.h>
 
-#include "../../include/disk.h"
-
 #include "cfg.h"
 #include "../cfg_fio.h"
 #include "../edv_interps.h"
+#include "edv_id.h"
+#include "edv_ids_list.h"
 #include "edv_device.h"
 #include "edv_devices_list.h"
 #include "edv_mime_type.h"
@@ -58,8 +58,11 @@ void EDVContextDelete(edv_context_struct *ctx);
 /*
  *	Appends the Device to the Context.
  *
- *	The specified Device is transfered and must not be referenced
- *	again after this call.
+ *	The d specifies the Device to append to the list. This
+ *	Device will be transfered to the list and should not be
+ *	referenced again after this call.
+ *
+ *	Returns the Device's index in the list or negative on error.
  */
 static gint EDVContextDeviceAppend(
 	edv_context_struct *ctx, edv_device_struct *d
@@ -86,8 +89,11 @@ static gint EDVContextDeviceAppend(
 /*
  *	Appends the MIME Type to the Context.
  *
- *	The specified MIME Type is transfered and must not be referenced
- *	again after this call.
+ *	The m specifies the MIME Type to append to the list. This
+ *	MIME Type will be transfered to the list and should not be
+ *	referenced again after this call.
+ *
+ *	Returns the MIME Type's index in the list or negative on error.
  */
 static gint EDVContextMimeTypeAppend(
 	edv_context_struct *ctx, edv_mime_type_struct *m
@@ -103,7 +109,7 @@ static gint EDVContextMimeTypeAppend(
 	{
 	    ctx->total_mimetypes = 0;
 	    EDVMimeTypeDelete(m);
-	    return(-1);
+	    return(-3);
 	}
 
 	ctx->mimetype[i] = m;
@@ -124,28 +130,7 @@ static gint EDVContextMimeTypeAppend(
  */
 edv_context_struct *EDVContextNew(void)
 {
-	edv_context_struct *ctx = EDV_CONTEXT(g_malloc0(
-	    sizeof(edv_context_struct)
-	));
-	if(ctx == NULL)
-	    return(NULL);
-
-	ctx->cfg_list = NULL;
-
-	ctx->mimetype = NULL;
-	ctx->total_mimetypes = 0;
-
-	ctx->device = NULL;
-	ctx->total_devices = 0;
-
-	ctx->queued_command = NULL;
-	ctx->total_queued_commands = 0;
-
-	ctx->recycled_index_file = NULL;
-	ctx->prog_file = NULL;
-	ctx->prog_full_path = NULL;
-
-	return(ctx);
+	return(EDV_CONTEXT(g_malloc0(sizeof(edv_context_struct))));
 }
 
 /*
@@ -173,24 +158,26 @@ void EDVContextLoadConfigurationFile(
 	if(ctx == NULL)
 	    return;
 
-	/* Get the Endeavour program location */
+	/* Get the Endeavour 2 program location */
 	if(STRISEMPTY(ctx->prog_full_path))
 	{
-	    gint i;
 	    struct stat stat_buf;
+	    gint i;
 
 	    /* Search through the list of possible program locations */
 	    for(i = 0; prog_loc[i] != NULL; i++)
 	    {
 		/* Exists at this location? */
-		if(!stat(prog_loc[i], &stat_buf))
+		if(!stat((const char *)prog_loc[i], &stat_buf))
 		{
-		    const gchar *s = prog_loc[i];
+		    const gchar *path = prog_loc[i];
 
 		    g_free(ctx->prog_full_path);
-		    ctx->prog_full_path = STRDUP(s);
+		    ctx->prog_full_path = STRDUP(path);
+
 		    g_free(ctx->prog_file);
-		    ctx->prog_file = STRDUP(strrchr(s, G_DIR_SEPARATOR));
+		    ctx->prog_file = STRDUP(g_basename(path));
+
 		    break;
 		}
 	    }
@@ -298,7 +285,7 @@ If you have never runned %s before or have not configured it\
 	    }
 
 
-	    /* Recycled objects index file */
+	    /* Set the recycled objects index file */
 	    s = CFGItemListGetValueS(
 		cfg_list, EDV_CFG_PARM_FILE_RECYCLE_BIN_INDEX
 	    );
@@ -309,10 +296,27 @@ If you have never runned %s before or have not configured it\
 /* Add support for additional members that need to be coppied here */
 	}
 
-	/* Load all Devices */
+	/* Open the UIDS and GIDs */
+	if(ctx->uids_list != NULL)
+	{
+	    g_list_foreach(ctx->uids_list, (GFunc)EDVUIDDelete, NULL);
+	    g_list_free(ctx->uids_list);
+	}
+	ctx->uids_list = EDVUIDGetSystem();
+
+	if(ctx->gids_list != NULL)
+	{
+	    g_list_foreach(ctx->gids_list, (GFunc)EDVGIDDelete, NULL);
+	    g_list_free(ctx->gids_list);
+	}
+	ctx->gids_list = EDVGIDGetSystem();
+
+
+	/* Open the Devices list from the Devices file */
 	EDVContextLoadDevices(ctx);
 
-	/* Load all MIME Types */
+
+	/* Open the MIME Types list from the MIME Types file */
 	EDVContextLoadMimeTypes(ctx);
 
 
@@ -374,33 +378,46 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	gchar *icons_dir_global;
 	edv_mime_type_struct *m;
 
+	/* Get the global icons directory */
+	s = CFGItemListGetValueS(
+	    ctx->cfg_list, EDV_CFG_PARM_DIR_GLOBAL
+	);
+	icons_dir_global = g_strconcat(
+	    s,
+	    G_DIR_SEPARATOR_S,
+	    EDV_DEF_ICONS_SUBDIR,
+	    NULL
+	);
+
 #define SET_MIMETYPE_ICON_FILES(state,sfile,mfile,lfile) { \
  if(!STRISEMPTY(icons_dir_global)) {		\
 						\
  g_free(m->small_icon_file[(state)]);		\
- m->small_icon_file[(state)] = STRDUP(		\
-  PrefixPaths(icons_dir_global, (sfile))	\
+ m->small_icon_file[(state)] = g_strconcat(	\
+  icons_dir_global,				\
+  G_DIR_SEPARATOR_S,				\
+  (sfile),					\
+  NULL						\
  );						\
 						\
  g_free(m->medium_icon_file[(state)]);		\
- m->medium_icon_file[(state)] = STRDUP(		\
-  PrefixPaths(icons_dir_global, (mfile))	\
+ m->medium_icon_file[(state)] = g_strconcat(	\
+  icons_dir_global,				\
+  G_DIR_SEPARATOR_S,				\
+  (mfile),					\
+  NULL						\
  );						\
 						\
  g_free(m->large_icon_file[(state)]);		\
- m->large_icon_file[(state)] = STRDUP(		\
-  PrefixPaths(icons_dir_global, (lfile))	\
+ m->large_icon_file[(state)] = g_strconcat(	\
+  icons_dir_global,				\
+  G_DIR_SEPARATOR_S,				\
+  (lfile),					\
+  NULL						\
  );						\
 } }
 
-	/* Get global icons directory */
-	s = CFGItemListGetValueS(
-	    ctx->cfg_list, EDV_CFG_PARM_DIR_GLOBAL
-	);
-	icons_dir_global = STRDUP(PrefixPaths(s, "icons"));
-
-
-	/* Begin loading system MIME Types */
+	/* Begin loading the system MIME Types */
 
 	/* Unknown */
 	m = EDVMimeTypeNew(
@@ -414,7 +431,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_object_misc_20x20.xpm",
 	    "icon_object_misc_32x32.xpm",
 	    "icon_object_misc_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Error */
@@ -429,7 +446,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_object_error_20x20.xpm",
 	    "icon_object_error_32x32.xpm",
 	    "icon_object_error_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* File */
@@ -444,7 +461,13 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_file_20x20.xpm",
 	    "icon_file_32x32.xpm",
 	    "icon_file_48x48.xpm"
-	)
+	);
+	SET_MIMETYPE_ICON_FILES(
+	    EDV_MIME_TYPE_ICON_STATE_HIDDEN,
+	    "icon_file_hidden_20x20.xpm",
+	    "icon_file_hidden_32x32.xpm",
+	    "icon_file_hidden_48x48.xpm"
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Executable */
@@ -459,7 +482,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_executable_20x20.xpm",
 	    "icon_executable_32x32.xpm",
 	    "icon_executable_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Directory */
@@ -474,13 +497,25 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_folder_closed_20x20.xpm",
 	    "icon_folder_closed_32x32.xpm",
 	    "icon_folder_closed_48x48.xpm"
-	)
+	);
 	SET_MIMETYPE_ICON_FILES(
 	    EDV_MIME_TYPE_ICON_STATE_SELECTED,
 	    "icon_folder_opened_20x20.xpm",
 	    "icon_folder_opened_32x32.xpm",
 	    "icon_folder_opened_48x48.xpm"
-	)
+	);
+	SET_MIMETYPE_ICON_FILES(
+	    EDV_MIME_TYPE_ICON_STATE_EXTENDED,
+	    "icon_folder_noaccess_20x20.xpm",
+	    "icon_folder_noaccess_32x32.xpm",
+	    "icon_folder_noaccess_48x48.xpm"
+	);
+	SET_MIMETYPE_ICON_FILES(
+	    EDV_MIME_TYPE_ICON_STATE_HIDDEN,
+	    "icon_folder_hidden_20x20.xpm",
+	    "icon_folder_hidden_32x32.xpm",
+	    "icon_folder_hidden_48x48.xpm"
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Link */
@@ -495,13 +530,13 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_link2_20x20.xpm",
 	    "icon_link2_32x32.xpm",
 	    "icon_link2_48x48.xpm"
-	)
+	);
 	SET_MIMETYPE_ICON_FILES(
 	    EDV_MIME_TYPE_ICON_STATE_EXTENDED,
 	    "icon_link_broken_20x20.xpm",
-	    "icon_link2_32x32.xpm",
-	    "icon_link2_48x48.xpm"
-	)
+	    "icon_link_broken_32x32.xpm",
+	    "icon_link_broken_48x48.xpm"
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Device Block */
@@ -516,7 +551,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_device_block_20x20.xpm",
 	    "icon_device_block_32x32.xpm",
 	    "icon_device_block_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Device Character */
@@ -531,7 +566,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_device_character_20x20.xpm",
 	    "icon_device_character_32x32.xpm",
 	    "icon_device_character_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* FIFO Pipe */
@@ -546,7 +581,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_pipe_20x20.xpm",
 	    "icon_pipe_32x32.xpm",
 	    "icon_pipe_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 	/* Socket */
@@ -561,7 +596,7 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    "icon_socket_20x20.xpm",
 	    "icon_socket_32x32.xpm",
 	    "icon_socket_48x48.xpm"
-	)
+	);
 	EDVContextMimeTypeAppend(ctx, m);
 
 
@@ -597,10 +632,9 @@ static void EDVContextLoadMimeTypes(edv_context_struct *ctx)
 	    g_free(list);
 	}
 
+#undef SET_MIMETYPE_ICON_FILES
 
 	g_free(icons_dir_global);
-
-#undef SET_MIMETYPE_ICON_FILES
 }
 
 
@@ -734,9 +768,6 @@ void EDVContextDelete(edv_context_struct *ctx)
 	if(ctx == NULL)
 	    return;
 
-#define FREEP(p)	\
-{ if((p) != NULL) { g_free(*(p)); *(p) = NULL; } }
-
 	/* Delete any queued commands */
 	if(ctx->queued_command != NULL)
 	{
@@ -745,31 +776,43 @@ void EDVContextDelete(edv_context_struct *ctx)
 	    ctx->total_queued_commands = 0;
 	}
 
-	/* Delete other resources */
-	FREEP(&ctx->recycled_index_file);
-	FREEP(&ctx->prog_file);
-	FREEP(&ctx->prog_full_path);
-
-	/* Delete Devices list */
+	/* Delete the Devices list */
 	for(i = 0; i < ctx->total_devices; i++)
 	    EDVDeviceDelete(ctx->device[i]);
 	g_free(ctx->device);
 	ctx->device = NULL; 
 	ctx->total_devices = 0;
 
-	/* Delete MIME Types list */
+	/* Delete the MIME Types list */
 	for(i = 0; i < ctx->total_mimetypes; i++)
 	    EDVMimeTypeDelete(ctx->mimetype[i]);
 	g_free(ctx->mimetype);
 	ctx->mimetype = NULL;
 	ctx->total_mimetypes = 0;
 
-	/* Delete configuration list */
+	/* Delete the UIDs list */
+	if(ctx->uids_list != NULL)
+	{                         
+	    g_list_foreach(ctx->uids_list, (GFunc)EDVUIDDelete, NULL);
+	    g_list_free(ctx->uids_list);
+	    ctx->uids_list = NULL;
+	}
+
+	/* Delete the GIDs list */
+	if(ctx->gids_list != NULL)
+	{
+	    g_list_foreach(ctx->gids_list, (GFunc)EDVGIDDelete, NULL);
+	    g_list_free(ctx->gids_list);
+	    ctx->gids_list = NULL;
+	}
+
+	/* Delete other resources */
+	g_free(ctx->recycled_index_file);
+	g_free(ctx->prog_file);
+	g_free(ctx->prog_full_path);
+
+	/* Delete the configuration list */
 	CFGItemListDeleteList(ctx->cfg_list);
-	ctx->cfg_list = NULL;
 
-	/* Delete structure itself */
-	FREEP(&ctx);
-
-#undef FREEP
+	g_free(ctx);
 }
