@@ -1,15 +1,10 @@
-#include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 #include <time.h>
 #include <glib.h>
 
-#include "../../include/string.h"
-
-#include "edv_types.h"
+#include "edv_context.h"
 #include "edv_get.h"
 #include "edv_date.h"
-#include "edv_context.h"
 #include "edv_cfg_list.h"
 
 
@@ -66,7 +61,8 @@ const gchar *EDVDateString(
 	  case EDV_DATE_RELATIVITY_CURRENT:
 	    if(TRUE)
 	    {
-		gulong ct, dt = (gulong)time(NULL) - t;
+		const gulong dt = (gulong)time(NULL) - t;
+		gulong ct;
 		static gchar buf[80];
 
 		/* Less than one second? */
