@@ -1,12 +1,11 @@
 /*
-		     Endeavour Devices File IO
+		     Endeavour Devices List File IO
  */
 
 #ifndef EDV_DEVICES_LIST_H
 #define EDV_DEVICES_LIST_H
 
 #include <glib.h>
-#include "edv_types.h"
 #include "edv_device.h"
 
 
