@@ -82,13 +82,13 @@ static gchar *EDVOpenStripExt(const gchar *s)
 	    memcpy(path_wo_ext, s, len * sizeof(gchar));
 	path_wo_ext[len] = '\0';
 
-        /* Need to tack on the ending double quote character? */
-        if(*path_wo_ext == '"')
-        {
-            gchar *s2 = g_strconcat(path_wo_ext, "\"", NULL);
-            g_free(path_wo_ext);
-            path_wo_ext = s2;
-        }
+	/* Need to tack on the ending double quote character? */
+	if(*path_wo_ext == '"')
+	{
+	    gchar *s2 = g_strconcat(path_wo_ext, "\"", NULL);
+	    g_free(path_wo_ext);
+	    path_wo_ext = s2;
+	}
 
 	return(path_wo_ext);
 }
@@ -118,12 +118,12 @@ static gboolean EDVOpenIsFileExecutable(const gchar *path)
 #endif
 
 #if defined(S_IXUSR) && defined(S_IXGRP) && defined(S_IXOTH)
-        /* Is this file executable? */                      
-        if((m & S_IXUSR) ||
-           (m & S_IXGRP) ||
-           (m & S_IXOTH)   
-        )
-            return(TRUE);
+	/* Is this file executable? */                      
+	if((m & S_IXUSR) ||
+	   (m & S_IXGRP) ||
+	   (m & S_IXOTH)   
+	)
+	    return(TRUE);
 #endif
 
 	return(FALSE);
@@ -477,9 +477,10 @@ static gint EDVOpenIteration(
 			    EDVSetCWD(parent);
 			    g_free(parent);
 
-			    /* Format command */
+			    /* Format the command */
 			    paths_list = g_list_append(
-				paths_list, STRDUP(path)
+				paths_list,
+				STRDUP(path)
 			    );
 			    cmd = EDVOpenFormatCommand(
 				ctx, command_base, paths_list
@@ -487,7 +488,7 @@ static gint EDVOpenIteration(
 			    g_list_foreach(paths_list, (GFunc)g_free, NULL);
 			    g_list_free(paths_list);
 
-			    /* Execute command */
+			    /* Execute the command */
 			    pid = EDVSystem(cmd);
 			    g_free(cmd);
 			    if(pid <= 0)
@@ -560,7 +561,7 @@ static gint EDVOpenIteration(
 		    path
 		);
 
-		/* Execute command */
+		/* Execute the command */
 		pid = EDVSystem(cmd);
 		g_free(cmd);
 		if(pid <= 0)
@@ -589,7 +590,7 @@ static gint EDVOpenIteration(
 		EDVSetCWD(parent);
 		g_free(parent);
 
-		/* Format command */
+		/* Format the command */
 		paths_list = g_list_append(
 		    paths_list,
 		    STRDUP(path)
@@ -600,7 +601,7 @@ static gint EDVOpenIteration(
 		g_list_foreach(paths_list, (GFunc)g_free, NULL);
 		g_list_free(paths_list);
 
-		/* Execute command */
+		/* Execute the command */
 		pid = EDVSystem(cmd);
 		g_free(cmd);
 		if(pid <= 0)
