@@ -188,21 +188,26 @@ static Dlg *EDVDlgNew(
 	const gint	border_major = 5;
 	GdkWindow *window;
 	GtkAccelGroup *accelgrp;
-	GtkWidget *w, *parent, *parent2;
+	GtkWidget	*w,
+			*parent, *parent2,
+			*toplevel;
 	Dlg *d = (size > 0) ? DLG(g_malloc0(size)) : NULL;
 	if(d == NULL)
 	    return(NULL);
 
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
 	d->freeze_count = 0;
 	d->core = core;
 	d->path = STRDUP(path);
 	d->got_response = FALSE;
 
-	/* Toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, width, height);
-	gtk_window_set_title(GTK_WINDOW(w), title);
+	if(title != NULL)
+	    gtk_window_set_title(GTK_WINDOW(w), title);
 	gtk_window_set_wmclass(
 	    GTK_WINDOW(w), "dialog", PROG_NAME
 	);
@@ -212,12 +217,11 @@ static Dlg *EDVDlgNew(
 	{
 	    gdk_window_set_decorations(
 		window,
-		GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU |
-		GDK_DECOR_MINIMIZE
+		GDK_DECOR_BORDER | GDK_DECOR_TITLE
 	    );
 	    gdk_window_set_functions(
 		window,
-		GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE
+		GDK_FUNC_MOVE | GDK_FUNC_CLOSE
 	    );
 	    GUISetWMIcon(window, icon_data);
 	}
