@@ -80,6 +80,7 @@ static void EDVObjOpDlgCreatePermissionWidgets(
 	GtkWidget **sticky_w
 );
 static void EDVObjOpDlgCreateTimeStampWidgets(
+	edv_core_struct *core,
 	GtkWidget *parent,
 	const gchar *pfx_label,
 	GtkWidget **year_w,
@@ -96,9 +97,8 @@ void EDVObjOpDlgSetBusy(edv_obj_op_dlg_struct *d, const gboolean is_busy);
 gboolean EDVObjOpDlgIsMapped(edv_obj_op_dlg_struct *d);
 void EDVObjOpDlgMapValues(
 	edv_obj_op_dlg_struct *d,
-	const edv_obj_op_dlg_op op,	/* One of EDV_OBJ_OP_DLG_OP_* */
-	edv_object_struct **src_obj,	/* Source list, will be coppied */
-	const gint src_obj_total,
+	const edv_obj_op_dlg_op op,
+	GList *objs_list,		/* List of edv_object_struct * */
 	const gchar *src_dir,
 	GtkWidget *toplevel
 );
@@ -106,6 +106,9 @@ void EDVObjOpDlgUnmap(edv_obj_op_dlg_struct *d);
 void EDVObjOpDlgDelete(edv_obj_op_dlg_struct *d);
 
 
+#define OBJ_OP_DLG_USE_PERMISSION_CHECK_BUTTON	FALSE
+
+
 #define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
 #define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
 #define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
@@ -118,9 +121,6 @@ void EDVObjOpDlgDelete(edv_obj_op_dlg_struct *d);
 #define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
-#define FOPDLG_USE_PERMISSION_CHECK_BUTTON	FALSE
-
-
 /*
  *	"delete_event" signal callback.
  */
@@ -173,41 +173,40 @@ static void EDVObjOpDlgCancelCB(GtkWidget *widget, gpointer data)
  */
 static void EDVObjOpDlgBrowseCB(GtkWidget *widget, gpointer data)
 {
-	gboolean status;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	gchar **paths_list = NULL;
 	GtkWidget *toplevel;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	GtkEntry *entry = (GtkEntry *)data;
 	if((entry == NULL) || FileBrowserIsQuery())
 	    return;
 
 	toplevel = gtk_widget_get_toplevel(GTK_WIDGET(entry));
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
 	/* Query user for target object */
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 	    "Select Target Object",
 	    "Select", "Cancel",
 	    gtk_entry_get_text(entry),          /* Startup path */
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
 
 	/* Got user response? */
-	if(status)
+	if(response)
 	{
-	    const gchar *new_path = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *new_path = (npaths > 0) ?
+		paths_list[0] : NULL;
 	    if(!STRISEMPTY(new_path))
 	    {
 		gtk_entry_set_text(entry, new_path);
@@ -216,7 +215,7 @@ static void EDVObjOpDlgBrowseCB(GtkWidget *widget, gpointer data)
 	}
 
 	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	/* Reset due to possible file related change */
 	FileBrowserReset();
@@ -286,8 +285,8 @@ static void EDVObjOpDlgGroupMapPUListCB(GtkWidget *widget, gpointer data)
 	if(PUListIsQuery(pulist))
 	    return;
 
-        nitems = PUListGetTotalItems(pulist);
-        nitems_visible = MIN(15, nitems);
+	nitems = PUListGetTotalItems(pulist);
+	nitems_visible = MIN(15, nitems);
 
 	/* Block input and get value */
 	value = PUListMapQuery(
@@ -476,11 +475,11 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	const gboolean show_progress = TRUE;
 	gboolean	recursive = FALSE,
 			dereference_links = FALSE;
-	gint status, nobjs;
+	gint status;
 	gchar *tar_path;
-	edv_obj_op_dlg_op op;
-	edv_object_struct **objs_list;
+	GList *objs_list;
 	GtkWidget *w, *toplevel;
+	edv_obj_op_dlg_op op;
 	edv_core_struct *core;
 
 	edv_permission_flags permissions = 0x00000000;
@@ -490,27 +489,20 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	if(d == NULL)
 	    return;
 
+	toplevel = d->ref_toplevel;
 	core = d->core;
-	if(core == NULL)
-	    return;
 
 	/* Check and warn if write protect is enabled */
 	if(EDVCheckWriteProtect(core, TRUE, d->toplevel))
 	    return;
 
-	/* Get the operation code that indicates the operation that
-	 * is to be perfirmed
+	/* Get the operation code that indicates which operation is
+	 * to be perfirmed
 	 */
 	op = d->op;
 
-	/* Get the reference toplevel widget for setting the
-	 * transient for of subsequent dialogs
-	 */
-	toplevel = d->ref_toplevel;
-
 	/* Get the list of objects to operate on */
-	nobjs = d->src_obj_total;
-	objs_list = d->src_obj;
+	objs_list = d->objs_list;
 
 	/* Get the target value */
 	w = d->target_entry;
@@ -570,7 +562,7 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	    w = d->target_owner_entry;
 	    if(w != NULL)
 		owner_id = EDVUIDNameToUID(
-		    core->uid, core->total_uids,
+		    core->uids_list,
 		    gtk_entry_get_text(GTK_ENTRY(w))
 		);
 
@@ -578,7 +570,7 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	    w = d->target_group_entry;
 	    if(w != NULL)
 		group_id = EDVGIDNameToGID(
-		    core->gid, core->total_gids,
+		    core->gids_list,
 		    gtk_entry_get_text(GTK_ENTRY(w))
 		);
 	}
@@ -701,16 +693,19 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	{
 	    struct stat lstat_buf;
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *src_path;
 	    gchar *new_path;
 	    const gchar *error_msg;
+	    GList *glist;
 	    edv_object_struct *obj;
 
-	    /* Iterate through the objects list and copy each object */
-	    for(i = 0; i < nobjs; i++)
+	    /* Copy each object */
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -729,34 +724,35 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    &yes_to_all
 		);
 
-		/* Check for and report any errors */
-		error_msg = EDVObjectOPGetError();
+		/* Check for errors */
+		error_msg = EDVObjectOPGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
+		    EDVMessageError(
 			"Copy Error",
 			error_msg,
-			src_path,
+			NULL,
 			toplevel
 		    );
 		}
 
 		/* Get the source object's statistics */
-                if(lstat(src_path, &lstat_buf))
-                {
+		if(lstat((const char *)src_path, &lstat_buf))
+		{
 #ifdef ENOENT
 		    /* Report the source object being removed */
-                    const gint error_code = (gint)errno;
-                    if(error_code == ENOENT)
-                        EDVObjectRemovedEmit(core, src_path);
+		    const gint error_code = (gint)errno;
+		    if(error_code == ENOENT)
+			EDVObjectRemovedEmit(core, src_path);
 #endif
-                }
+		}
 
 		/* Report a new object being added? */
 		if(new_path != NULL)
 		{
-		    if(!lstat(new_path, &lstat_buf))
+		    if(!lstat((const char *)new_path, &lstat_buf))
 			EDVObjectAddedEmit(
 			    core,
 			    new_path,
@@ -777,16 +773,19 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	{
 	    struct stat lstat_buf;
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *src_path;
 	    gchar *new_path;
 	    const gchar *error_msg;
+	    GList *glist;
 	    edv_object_struct *obj;
 
-	    /* Iterate through the objects list and move each object */
-	    for(i = 0; i < nobjs; i++)
+	    /* Move each object */
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -805,34 +804,35 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    &yes_to_all
 		);
 
-		/* Check for and report any errors */
-		error_msg = EDVObjectOPGetError();
+		/* Check for errors */
+		error_msg = EDVObjectOPGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
+		    EDVMessageError(
 			"Move Error",
 			error_msg,
-			src_path,
+			NULL,
 			toplevel
 		    );
 		}
 
 		/* Get the source object's statistics */
-                if(lstat(src_path, &lstat_buf))
-                {
+		if(lstat((const char *)src_path, &lstat_buf))
+		{
 #ifdef ENOENT
 		    /* Report the source object being removed */
-                    const gint error_code = (gint)errno;
-                    if(error_code == ENOENT)
-                        EDVObjectRemovedEmit(core, src_path);
+		    const gint error_code = (gint)errno;
+		    if(error_code == ENOENT)
+			EDVObjectRemovedEmit(core, src_path);
 #endif
-                }
+		}
 
 		/* Report a new object being added? */
 		if(new_path != NULL)
 		{
-		    if(!lstat(new_path, &lstat_buf))
+		    if(!lstat((const char *)new_path, &lstat_buf))
 			EDVObjectAddedEmit(
 			    core,
 			    new_path,
@@ -853,16 +853,19 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	{
 	    struct stat lstat_buf;
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *src_path;
 	    gchar *new_path;
 	    const gchar *error_msg;
+	    GList *glist;
 	    edv_object_struct *obj;
 
-	    /* Iterate through the objects list and link each object */
-	    for(i = 0; i < nobjs; i++)
+	    /* Link each object */
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -882,34 +885,35 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    &yes_to_all
 		);
 
-		/* Check for and report any errors */
-		error_msg = EDVObjectOPGetError();
+		/* Check for errors */
+		error_msg = EDVObjectOPGetError(core);
 		if(!STRISEMPTY(error_msg))
 		{
+		    /* Report the error */
 		    EDVPlaySoundError(core);
-		    EDVMessageObjectOPError(
+		    EDVMessageError(
 			"Link Error",
 			error_msg,
-			src_path,
+			NULL,
 			toplevel
 		    );
 		}
 
 		/* Get the source object's statistics */
-                if(lstat(src_path, &lstat_buf))
-                {
+		if(lstat((const char *)src_path, &lstat_buf))
+		{
 #ifdef ENOENT
 		    /* Report the source object being removed */
-                    const gint error_code = (gint)errno;
-                    if(error_code == ENOENT)
-                        EDVObjectRemovedEmit(core, src_path);
+		    const gint error_code = (gint)errno;
+		    if(error_code == ENOENT)
+			EDVObjectRemovedEmit(core, src_path);
 #endif
-                }
+		}
 
 		/* Report a new object being added? */
 		if(new_path != NULL)
 		{
-		    if(!lstat(new_path, &lstat_buf))
+		    if(!lstat((const char *)new_path, &lstat_buf))
 			EDVObjectAddedEmit(
 			    core,
 			    new_path,
@@ -929,15 +933,18 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	else if(op == EDV_OBJ_OP_DLG_OP_CHMOD)
 	{
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *error_msg;
-	    GList *paths_list = NULL, *modified_paths_list;
+	    GList *glist, *paths_list, *modified_paths_list;
 	    edv_object_struct *obj;
 
 	    /* Create the paths list */
-	    for(i = 0; i < nobjs; i++)
+	    paths_list = NULL;
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -950,7 +957,8 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	    /* Change the permissions */
 	    status = EDVObjectOPChMod(
 		core,
-		paths_list, permissions,
+		paths_list,
+		permissions,
 		&modified_paths_list,
 		toplevel,
 		show_progress,
@@ -960,10 +968,11 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		!dereference_links	/* Archive */
 	    );
 
-	    /* Check for error */
-	    error_msg = EDVObjectOPGetError();
+	    /* Check for errors */
+	    error_msg = EDVObjectOPGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
 		EDVMessageError(
 		    "Change Permissions Error",
@@ -989,7 +998,7 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    if(modified_path == NULL)
 			continue;
 
-		    if(!lstat(modified_path, &lstat_buf))
+		    if(!lstat((const char *)modified_path, &lstat_buf))
 			EDVObjectModifiedEmit(
 			    core,
 			    modified_path,
@@ -1014,15 +1023,18 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	else if(op == EDV_OBJ_OP_DLG_OP_CHOWN)
 	{
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *error_msg;
-	    GList *paths_list = NULL, *modified_paths_list;
+	    GList *glist, *paths_list, *modified_paths_list;
 	    edv_object_struct *obj;
 
 	    /* Create the paths list */
-	    for(i = 0; i < nobjs; i++)
+	    paths_list = NULL;
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -1046,10 +1058,11 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		!dereference_links	/* Archive */
 	    );
 
-	    /* Check for error */
-	    error_msg = EDVObjectOPGetError();
+	    /* Check for errors */
+	    error_msg = EDVObjectOPGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
 		EDVMessageError(
 		    "Change Ownership Error",
@@ -1075,7 +1088,7 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    if(modified_path == NULL)
 			continue;
 
-		    if(!lstat(modified_path, &lstat_buf))
+		    if(!lstat((const char *)modified_path, &lstat_buf))
 			EDVObjectModifiedEmit(
 			    core,
 			    modified_path,
@@ -1100,15 +1113,18 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	else if(op == EDV_OBJ_OP_DLG_OP_CHTIME)
 	{
 	    gboolean yes_to_all = FALSE;
-	    gint i;
 	    const gchar *error_msg;
-	    GList *paths_list = NULL, *modified_paths_list;
+	    GList *glist, *paths_list, *modified_paths_list;
 	    edv_object_struct *obj;
 
 	    /* Create the paths list */
-	    for(i = 0; i < nobjs; i++)
+	    paths_list = NULL;
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		obj = objs_list[i];
+		obj = EDV_OBJECT(glist->data);
 		if(obj == NULL)
 		    continue;
 
@@ -1132,10 +1148,11 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		!dereference_links	/* Archive */
 	    );
 
-	    /* Check for error */
-	    error_msg = EDVObjectOPGetError();
+	    /* Check for errors */
+	    error_msg = EDVObjectOPGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
 		EDVMessageError(
 		    "Change Time Stamps Error",
@@ -1161,7 +1178,7 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 		    if(modified_path == NULL)
 			continue;
 
-		    if(!lstat(modified_path, &lstat_buf))
+		    if(!lstat((const char *)modified_path, &lstat_buf))
 			EDVObjectModifiedEmit(
 			    core,
 			    modified_path,
@@ -1184,10 +1201,18 @@ static void EDVObjOpDlgProcess(edv_obj_op_dlg_struct *d)
 	}
 	else
 	{
-	    g_printerr(
-"EDVObjOpDlgProcess(): Operation %i: Unsupported operation.\n",
+	    gchar *msg = g_strdup_printf(
+"EDVObjOpDlgProcess(): Operation \"%i\" unsupported.",
 		op
 	    );
+	    EDVPlaySoundError(core);
+	    EDVMessageError(
+		"Internal Error",
+		msg,
+		NULL,
+		toplevel
+	    );
+	    g_free(msg);
 	    status = -2;
 	}
 
@@ -1256,7 +1281,7 @@ static void EDVObjOpDlgCreatePermissionWidgets(
 	parent2 = w;
 
 #define NEW_PERMISSION_TOGGLE_BUTTON(_label_,_tip_,_active_,_parent_)	{ \
- if(FOPDLG_USE_PERMISSION_CHECK_BUTTON) {			\
+ if(OBJ_OP_DLG_USE_PERMISSION_CHECK_BUTTON) {			\
   /* Create permission button as a GtkCheckButton */		\
   w = gtk_check_button_new_with_label(_label_);			\
  } else {							\
@@ -1389,6 +1414,7 @@ static void EDVObjOpDlgCreatePermissionWidgets(
  *	Creates the time stamp widgets.
  */
 static void EDVObjOpDlgCreateTimeStampWidgets(
+	edv_core_struct *core,
 	GtkWidget *parent,			/* GtkVBox */
 	const gchar *pfx_label,
 	GtkWidget **year_w,
@@ -1439,6 +1465,7 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	*year_w = w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 60, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	GUISetWidgetTip(w, "Year");
 	gtk_widget_show(w);
 
@@ -1453,25 +1480,37 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	gtk_widget_show(w);
 	/* Month Popup List Box */
 	*month_pulistbox = pulistbox = PUListBoxNew(parent3, 70, -1);
-	GUISetWidgetTip(pulistbox->da, "Month");
-	PUListBoxMap(pulistbox);
 	pulist = PUListBoxGetPUList(pulistbox);
+        if(core->run_flags & EDV_RUN_SAFE_MODE)
+            PUListSetShadowStyle(pulist, PULIST_SHADOW_NONE);
 	if(pulist != NULL)
 	{
-	    gint i = PUListAddItem(pulist, "Jan");
-	    i = PUListAddItem(pulist, "Feb");
-	    i = PUListAddItem(pulist, "Mar");
-	    i = PUListAddItem(pulist, "Apr");
-	    i = PUListAddItem(pulist, "May");
-	    i = PUListAddItem(pulist, "Jun");
-	    i = PUListAddItem(pulist, "Jul");
-	    i = PUListAddItem(pulist, "Aug");
-	    i = PUListAddItem(pulist, "Sep");
-	    i = PUListAddItem(pulist, "Oct");
-	    i = PUListAddItem(pulist, "Nov");
-	    i = PUListAddItem(pulist, "Dec");
-	    PUListBoxSetLinesVisible(pulistbox, i + 1);
+	    struct tm v;
+	    gint i, n = -1;
+	    gchar buf[40];
+
+	    v.tm_sec = 0;
+	    v.tm_min = 0;
+	    v.tm_hour = 0;
+	    v.tm_mday = 1;
+	    v.tm_mon = 0;
+	    v.tm_year = 1900;
+	    v.tm_wday = 0;
+	    v.tm_yday = 0;
+	    v.tm_isdst = -1;
+
+	    for(i = 0; i < 12; i++)
+	    {
+		v.tm_mon = (int)i;
+		if(strftime(buf, sizeof(buf), "%b", &v) > 0)
+		    n = PUListAddItem(pulist, buf);
+	    }
+
+	    if(n > -1)
+                PUListBoxSetLinesVisible(pulistbox, n + 1);
 	}
+	GUISetWidgetTip(pulistbox->da, "Month");
+	PUListBoxMap(pulistbox);
 
 	/* Day GtkHBox */
 	w = gtk_hbox_new(FALSE, border_minor);
@@ -1490,6 +1529,7 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	*day_w = w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 45, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	GUISetWidgetTip(w, "Day");
 	gtk_widget_show(w);
 
@@ -1517,6 +1557,7 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	*hours_w = w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 45, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	GUISetWidgetTip(w, "Hours");
 	gtk_widget_show(w);
 
@@ -1537,6 +1578,7 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	*minutes_w = w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 45, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	GUISetWidgetTip(w, "Minutes");
 	gtk_widget_show(w);
 
@@ -1557,6 +1599,7 @@ static void EDVObjOpDlgCreateTimeStampWidgets(
 	*seconds_w = w = gtk_spin_button_new(adj, 1.0, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_set_usize(w, 45, -1);
+	GUIEditableEndowPopupMenu(w, 0);
 	GUISetWidgetTip(w, "Seconds");
 	gtk_widget_show(w);
 
@@ -1578,48 +1621,49 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	const gint	border_major = 5,
 			border_minor = 2;
 	GdkWindow *window;
-	GtkAccelGroup *accelgrp;
 	GtkStyle *style;
-	GtkWidget	*w, *parent, *parent2, *parent3, *parent4,
-			*parent5, *parent6, *parent7;
-	edv_obj_op_dlg_struct *d = EDV_OBJ_OP_DLG(
-	    g_malloc0(sizeof(edv_obj_op_dlg_struct))
-	);
-	if((d == NULL) || (core == NULL))
-	{
-	    g_free(d);
-	    return(NULL);
-	}
+	GtkAccelGroup *accelgrp;
+	GtkWidget	*w,
+			*parent, *parent2, *parent3, *parent4,
+			*parent5, *parent6, *parent7,
+			*toplevel;
+	edv_obj_op_dlg_struct *d;
 
+	if(core == NULL)
+	    return(NULL);
+
+	d = EDV_OBJ_OP_DLG(g_malloc0(sizeof(edv_obj_op_dlg_struct)));
+	if(d == NULL)
+	    return(NULL);
+
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
 	d->processing = FALSE;
-	d->busy_count = 0;
 	d->freeze_count = 0;
+	d->busy_count = 0;
 	d->core = core;
 
 	d->ref_toplevel = NULL;
 	d->op = EDV_OBJ_OP_DLG_OP_MOVE;
-	d->src_obj = NULL;
-	d->src_obj_total = 0;
+	d->objs_list = NULL;
 	d->src_dir = NULL;
 
+	d->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_realize(w);
 	window = w->window;
 	if(window != NULL)
 	{
 	    gdk_window_set_decorations(
 		window,
-		GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU |
-		GDK_DECOR_MINIMIZE
+		GDK_DECOR_BORDER | GDK_DECOR_TITLE
 	    );
 	    gdk_window_set_functions(
 		window,
-		GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE
+		GDK_FUNC_MOVE | GDK_FUNC_CLOSE
 	    );
 	}
 	gtk_widget_add_events(
@@ -1692,7 +1736,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	gtk_widget_show(w);
 	parent4 = w;
 	/* Label GtkAlignment */
-	w = gtk_alignment_new(1.0f, 0.0f, 0.0f, 0.0f);
+	w = gtk_alignment_new(1.0f, 0.5f, 0.0f, 0.0f);
 	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent5 = w;
@@ -1714,6 +1758,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	);
 	EDVEntrySetDND(core, w);
 	EDVEntrySetCompletePath(core, w);
+	GUIEditableEndowPopupMenu(w, 0);
 
 	/* Target Browse GtkButton */
 	d->target_browse_btn = w = GUIButtonPixmap(
@@ -1787,6 +1832,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	    GTK_OBJECT(w), "activate",
 	    GTK_SIGNAL_FUNC(EDVObjOpDlgProcessCB), d
 	);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	/* Owner Popup List Map Button */
 	d->target_owner_btn = w = PUListNewMapButtonArrow(
@@ -1824,6 +1870,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	    GTK_OBJECT(w), "activate",
 	    GTK_SIGNAL_FUNC(EDVObjOpDlgProcessCB), d
 	);
+	GUIEditableEndowPopupMenu(w, 0);
 	gtk_widget_show(w);
 	/* Group Popup List Map Button */
 	d->target_group_btn = w = PUListNewMapButtonArrow(
@@ -1849,6 +1896,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 
 	/* Access Time date and time widgets */
 	EDVObjOpDlgCreateTimeStampWidgets(
+	    core,
 	    parent6,
 	    "Access",
 	    &d->target_atime_year_spin,
@@ -1872,6 +1920,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 
 	/* Modify Time date and time widgets */
 	EDVObjOpDlgCreateTimeStampWidgets(
+	    core,
 	    parent6,
 	    "Modify",
 	    &d->target_mtime_year_spin,
@@ -1919,7 +1968,7 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	GTK_TOGGLE_BUTTON_SET_ACTIVE(w, FALSE);
 	GUISetWidgetTip(
 	    w,
-"Check this to operate on the links' destination instead\
+"Check this to operate on the links' target instead\
  of the links themselves"
 	);
 
@@ -2173,28 +2222,29 @@ edv_obj_op_dlg_struct *EDVObjOpDlgNew(edv_core_struct *core)
 	GUIButtonLabelUnderline(w, GDK_c);
 	gtk_widget_show(w);
 
+	d->freeze_count--;
 
 	return(d);
 }
 
 
 /*
- *	Deletes source disk object structure list and current
- *	source directory on the given dialog.
+ *	Clears the objects list and source directory.
  */
 void EDVObjOpDlgReset(edv_obj_op_dlg_struct *d)
 {
-	gint i;
-
 	if(d == NULL)
 	    return;
 
 	/* Delete the objects list */
-	for(i = 0; i < d->src_obj_total; i++)
-	    EDVObjectDelete(d->src_obj[i]);
-	g_free(d->src_obj);
-	d->src_obj = NULL;
-	d->src_obj_total = 0;
+	if(d->objs_list != NULL)
+	{
+	    g_list_foreach(
+		d->objs_list, (GFunc)EDVObjectDelete, NULL
+	    );
+	    g_list_free(d->objs_list);
+	    d->objs_list = NULL;
+	}
 
 	/* Delete the source directory */
 	g_free(d->src_dir);
@@ -2259,8 +2309,10 @@ void EDVObjOpDlgSetBusy(edv_obj_op_dlg_struct *d, const gboolean is_busy)
  */
 gboolean EDVObjOpDlgIsMapped(edv_obj_op_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(d == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(d->toplevel));
 }
 
 /*
@@ -2270,9 +2322,9 @@ gboolean EDVObjOpDlgIsMapped(edv_obj_op_dlg_struct *d)
  *
  *	The op specifies the operation.
  *
- *	The src_obj and src_obj_total specifies the list of objects
- *	to be operated on. This list of objects will be coppied and
- *	stored locally on the Object Operations Dialog.
+ *	The objs_list specifies the GList of edv_object_struct *
+ *	objects to operate on. This list will be coppied and will
+ *	not be modified by this function.
  *
  *	The src_dir specifies the initial target location.
  *
@@ -2281,18 +2333,17 @@ gboolean EDVObjOpDlgIsMapped(edv_obj_op_dlg_struct *d)
 void EDVObjOpDlgMapValues(
 	edv_obj_op_dlg_struct *d,
 	const edv_obj_op_dlg_op op,
-	edv_object_struct **src_obj,
-	const gint src_obj_total,
+	GList *objs_list,		/* List of edv_object_struct * */
 	const gchar *src_dir,
 	GtkWidget *toplevel
 )
 {
-	gint i;
 	gboolean	need_warn_chmod_link = FALSE,
 			need_warn_chtime_link = FALSE,
 			map_browse_btn = FALSE;
+	gint nobjs;
 	const gchar *title = NULL;
-	gchar *buf = NULL;
+	gchar *msg = NULL;
 	const gchar *src_obj_full_path, *src_obj_name;
 	guint8 **icon_data = NULL;
 	const gchar *icon_name = NULL;
@@ -2304,12 +2355,8 @@ void EDVObjOpDlgMapValues(
 	    return;
 
 	core = d->core;
-	if(core == NULL)
-	    return;
 
-	/* Delete the source objects list and the source directory
-	 * on the object operations dialog
-	 */
+	/* Clear the objects list and source directory */
 	EDVObjOpDlgReset(d);
 
 	/* If there is exactly one object in the source object list
@@ -2317,9 +2364,10 @@ void EDVObjOpDlgMapValues(
 	 *
 	 * Otherwise set the single object full path and name to NULL
 	 */
-	if(src_obj_total == 1)
+	nobjs = g_list_length(objs_list);
+	if(nobjs == 1)
 	{
-	    obj = src_obj[0];
+	    obj = EDV_OBJECT(objs_list->data);
 	    src_obj_name = (obj != NULL) ? obj->name : NULL;
 	    src_obj_full_path = (obj != NULL) ? obj->full_path : NULL;
 	}
@@ -2331,7 +2379,7 @@ void EDVObjOpDlgMapValues(
 	}
 
 
-	/* Set operation code */
+	/* Set the operation code */
 	d->op = op;
 
 	/* Update the widgets based on the operation */
@@ -2360,7 +2408,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_move_file_32x32_xpm;
 	    icon_name = "icon_move_file_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mueva:\n\
 \n\
@@ -2397,7 +2445,7 @@ void EDVObjOpDlgMapValues(
 		    , src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Mueva %i objetos\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -2415,7 +2463,7 @@ void EDVObjOpDlgMapValues(
 #else
 "Move %i objects\n"
 #endif
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->move_btn;
 	    op_parent = d->target_entry;
@@ -2448,7 +2496,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_copy_file_32x32_xpm;
 	    icon_name = "icon_copy_file_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La Copia:\n\
 \n\
@@ -2485,7 +2533,7 @@ void EDVObjOpDlgMapValues(
 		    , src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Copie %i objetos\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -2503,7 +2551,7 @@ void EDVObjOpDlgMapValues(
 #else
 "Copy %i objects\n"
 #endif
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->copy_btn;
 	    op_parent = d->target_entry;
@@ -2536,7 +2584,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_link2_32x32_xpm;
 	    icon_name = "icon_link2_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "El enlace:\n\
 \n\
@@ -2573,7 +2621,7 @@ void EDVObjOpDlgMapValues(
 		    , src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Trabe %i objetos\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -2591,7 +2639,7 @@ void EDVObjOpDlgMapValues(
 #else
 "Link %i objects\n"
 #endif
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->link_btn;
 	    op_parent = d->target_entry;
@@ -2624,7 +2672,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_chmod_32x32_xpm;
 	    icon_name = "icon_chmod_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Cambie los permisos de:\n\
 \n\
@@ -2661,7 +2709,7 @@ void EDVObjOpDlgMapValues(
 		    , src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Cambie los permisos de %i objetos\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -2679,7 +2727,7 @@ void EDVObjOpDlgMapValues(
 #else
 "Change permissions of %i objects\n"
 #endif
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->chmod_btn;
 	    op_parent = d->target_chmod_parent;
@@ -2711,7 +2759,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_owned_32x32_xpm;
 	    icon_name = "icon_owned_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Cambie la propiedad de:\n\
 \n\
@@ -2748,7 +2796,7 @@ void EDVObjOpDlgMapValues(
 		    , src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 #if defined(PROG_LANGUAGE_SPANISH)
 "Cambie la propiedad de %i objetos\n"
 #elif defined(PROG_LANGUAGE_FRENCH)
@@ -2766,7 +2814,7 @@ void EDVObjOpDlgMapValues(
 #else
 "Change ownership of %i objects\n"
 #endif
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->chown_btn;
 	    op_parent = d->target_chown_parent;
@@ -2780,7 +2828,7 @@ void EDVObjOpDlgMapValues(
 	    icon_data = (guint8 **)icon_time_stamp_32x32_xpm;
 	    icon_name = "icon_time_stamp_32x32_xpm";
 	    if(src_obj_name != NULL)
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 "Change time stamps of:\n\
 \n\
     %s\n\
@@ -2788,9 +2836,9 @@ void EDVObjOpDlgMapValues(
 		    src_obj_name
 		);
 	    else
-		buf = g_strdup_printf(
+		msg = g_strdup_printf(
 "Change time stamps of %i objects\n"
-		    , src_obj_total
+		    , nobjs
 		);
 	    button = d->chtime_btn;
 	    op_parent = d->target_time_parent;
@@ -2849,14 +2897,14 @@ void EDVObjOpDlgMapValues(
 	}
 
 	/* Update label */
-	if(buf != NULL)
+	if(msg != NULL)
 	{
 	    w = d->label;
 	    if(w != NULL)
-		gtk_label_set_text(GTK_LABEL(w), buf);
+		gtk_label_set_text(GTK_LABEL(w), msg);
 
-	    g_free(buf);
-	    buf = NULL;
+	    g_free(msg);
+	    msg = NULL;
 	}
 
 
@@ -2880,31 +2928,28 @@ void EDVObjOpDlgMapValues(
 		gtk_widget_grab_default(w);
 	}
 
-
-	/* Copy the specified objects list to the object operations
-	 * dialog's objects list
-	 */
-	d->src_obj_total = src_obj_total;
-	if(d->src_obj_total > 0)
+	/* Copy the objects list */
+	if(objs_list != NULL)
 	{
-	    d->src_obj = (edv_object_struct **)g_malloc(
-		d->src_obj_total * sizeof(edv_object_struct *)
-	    );
-	    if(d->src_obj != NULL)
+	    GList *glist;
+	    edv_object_struct *obj;
+
+	    for(glist = objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		for(i = 0; i < d->src_obj_total; i++)
-		    d->src_obj[i] = EDVObjectCopy(src_obj[i]);
-	    }
-	    else
-	    {
-		d->src_obj_total = 0;
+		obj = EDV_OBJECT(glist->data);
+		if(obj == NULL)
+		    continue;
+
+		obj = EDVObjectCopy(obj);
+		if(obj == NULL)
+		    continue;
+
+		d->objs_list = g_list_append(d->objs_list, obj);
 	    }
 	}
-	else
-	{
-	    d->src_obj_total = 0;
-	}
-
 
 	/* Is the operation to change permissions? */
 	w = d->target_chmod_parent;
@@ -2913,15 +2958,19 @@ void EDVObjOpDlgMapValues(
 	    /* Set the initial values for the change permission
 	     * widgets
 	     */
+	    GList *glist;
 	    edv_permission_flags permissions = 0x00000000;
 	    edv_object_struct *ref_obj;
 
 	    /* Iterate through the objects list and collect the
 	     * permission values that are common to all of them
 	     */
-	    for(i = 0; i < d->src_obj_total; i++)
+	    for(glist = d->objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		ref_obj = d->src_obj[i];
+		ref_obj = EDV_OBJECT(glist->data);
 		if(ref_obj == NULL)
 		    continue;
 
@@ -2931,7 +2980,7 @@ void EDVObjOpDlgMapValues(
 		 * If this is a symbolic link then a warning needs to
 		 * be issued about changing a symbolic link's
 		 * permissions that will actually change its
-		 * destination permissions
+		 * target permissions
 		 */
 		if(ref_obj->type == EDV_OBJECT_TYPE_LINK)
 		    need_warn_chmod_link = TRUE;
@@ -3016,15 +3065,14 @@ void EDVObjOpDlgMapValues(
 	    w = d->target_owner_entry;
 	    if(w != NULL)
 	    {
-		gtk_entry_set_text(
-		    GTK_ENTRY(w),
-		    EDVUIDGetNameFromUID(
-			core->uid, core->total_uids,
-			(ref_obj != NULL) ?
-			    ref_obj->owner_id : core->effective_user_id,
-			NULL
-		    )
-		);
+		gchar *owner = EDVUIDGetNameFromUID(
+                    core->uids_list,
+                    (ref_obj != NULL) ?
+                        ref_obj->owner_id : core->effective_user_id,
+                    NULL
+                );
+		gtk_entry_set_text(GTK_ENTRY(w), (owner != NULL) ? owner : "");
+		g_free(owner);
 		if(GTK_WIDGET_CAN_FOCUS(w))
 		    gtk_widget_grab_focus(w);
 		if(GTK_WIDGET_CAN_DEFAULT(w))
@@ -3035,15 +3083,14 @@ void EDVObjOpDlgMapValues(
 	    w = d->target_group_entry;
 	    if(w != NULL)
 	    {
-		gtk_entry_set_text(
-		    GTK_ENTRY(w),
-		    EDVGIDGetNameFromGID(
-			core->gid, core->total_gids,
-			(ref_obj != NULL) ?
-			    ref_obj->group_id : core->effective_group_id,
-			NULL
-		    )
+		gchar *group = EDVGIDGetNameFromGID(
+                    core->gids_list,
+                    (ref_obj != NULL) ?
+                        ref_obj->group_id : core->effective_group_id,
+		    NULL
 		);
+		gtk_entry_set_text(GTK_ENTRY(w), (group != NULL) ? group : "");
+		g_free(group);
 	    }
 	}
 
@@ -3054,7 +3101,7 @@ void EDVObjOpDlgMapValues(
 	    /* Set the initial time stamp widget values to the time
 	     * stamps of the specified object
 	     */
-	    gint i;
+	    GList *glist;
 	    edv_object_struct *ref_obj = obj;
 
 	    /* If there was more than one object specified then use the
@@ -3062,9 +3109,11 @@ void EDVObjOpDlgMapValues(
 	     */
 	    if(ref_obj == NULL)
 	    {
-		const gint last_obj_num = src_obj_total - 1;
+		const gint last_obj_num = nobjs - 1;
 		if(last_obj_num > -1)
-		    ref_obj = src_obj[last_obj_num];
+		    ref_obj = EDV_OBJECT(g_list_nth_data(
+			objs_list, (guint)last_obj_num
+		    ));
 	    }
 
 	    /* Got a reference object? */
@@ -3164,9 +3213,12 @@ void EDVObjOpDlgMapValues(
 	    }
 
 	    /* Check if we need to warn about links */
-	    for(i = 0; i < d->src_obj_total; i++)
+	    for(glist = d->objs_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
-		ref_obj = d->src_obj[i];
+		ref_obj = EDV_OBJECT(glist->data);
 		if(ref_obj == NULL)
 		    continue;
 
@@ -3239,13 +3291,13 @@ void EDVObjOpDlgMapValues(
 	    EDVMessageWarning(
 "Changing Permissions Warning",
 "Changing the permissions on a link will effectively\n\
-change the permissions of its destination object.",
+change the permissions of its target object.",
 "One or more of the selected object(s) are of type\n\
 link. Links do not have permissions, instead, their\n\
 permissions are determined by the permissions of the\n\
-destination object. So changing the permissions on a\n\
-link will effectively change the permissions of its\n\
-destination object.",
+target object. So changing the permissions on a link\n\
+will effectively change the permissions of the target\n\
+object.",
 		toplevel
 	    );
 	}
@@ -3255,13 +3307,13 @@ destination object.",
 	    EDVMessageWarning(
 "Changing Time Stamps Warning",
 "Changing the time stamps on a link will effectively\n\
-change the time stamps of its destination object.",
+change the time stamps of its target object.",
 "One or more of the selected object(s) are of type\n\
 link. Links do not have time stamps, instead, their\n\
 time stamps are determined by the time stamps of the\n\
-destination object. So changing the time stamps on a\n\
-link will effectively change the time stamps of its\n\
-destination object.",
+target object. So changing the time stamps on a link\n\
+will effectively change the time stamps of the target\n\
+object.",
 		toplevel
 	    );
 	}
@@ -3293,13 +3345,13 @@ destination object.",
  */
 void EDVObjOpDlgUnmap(edv_obj_op_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	GtkWidget *w;
+
+	if(d == NULL)
 	    return;
 
+	w = d->toplevel;
 	gtk_widget_hide(w);
-
-	/* Unset transient for */
 	gtk_window_set_transient_for(GTK_WINDOW(w), NULL);
 	d->ref_toplevel = NULL;
 }
@@ -3312,12 +3364,16 @@ void EDVObjOpDlgDelete(edv_obj_op_dlg_struct *d)
 	if(d == NULL)
 	    return;
 
+	EDVObjOpDlgUnmap(d);
+
 	/* Delete the object list and the source directory */
 	EDVObjOpDlgReset(d);
 
 	d->ref_toplevel = NULL;		/* Shared */
 
-	/* Begin destroying widgets */
+	d->freeze_count++;
+
+#if 0
 	GTK_WIDGET_DESTROY(d->icon_pm);
 	GTK_WIDGET_DESTROY(d->icon_fixed);
 
@@ -3345,14 +3401,18 @@ void EDVObjOpDlgDelete(edv_obj_op_dlg_struct *d)
 	GTK_WIDGET_DESTROY(d->target_chown_parent);
 
 	GTK_WIDGET_DESTROY(d->target_atime_year_spin);
+#endif
 	PUListBoxDelete(d->target_atime_month_pulistbox);
+#if 0
 	GTK_WIDGET_DESTROY(d->target_atime_day_spin);
 	GTK_WIDGET_DESTROY(d->target_atime_hours_spin);
 	GTK_WIDGET_DESTROY(d->target_atime_minutes_spin);
 	GTK_WIDGET_DESTROY(d->target_atime_seconds_spin);
 	GTK_WIDGET_DESTROY(d->target_atime_current_btn);
 	GTK_WIDGET_DESTROY(d->target_mtime_year_spin);
+#endif
 	PUListBoxDelete(d->target_mtime_month_pulistbox);
+#if 0
 	GTK_WIDGET_DESTROY(d->target_mtime_day_spin);
 	GTK_WIDGET_DESTROY(d->target_mtime_hours_spin);
 	GTK_WIDGET_DESTROY(d->target_mtime_minutes_spin);
@@ -3364,17 +3424,18 @@ void EDVObjOpDlgDelete(edv_obj_op_dlg_struct *d)
 	GTK_WIDGET_DESTROY(d->opt_dereference_links_check);
 	GTK_WIDGET_DESTROY(d->opt_parent);
 
-	GTK_WIDGET_DESTROY(d->move_btn)
-	GTK_WIDGET_DESTROY(d->copy_btn)
-	GTK_WIDGET_DESTROY(d->link_btn)
-	GTK_WIDGET_DESTROY(d->chmod_btn)
-	GTK_WIDGET_DESTROY(d->chown_btn)
-	GTK_WIDGET_DESTROY(d->chtime_btn)
-	GTK_WIDGET_DESTROY(d->cancel_btn)
+	GTK_WIDGET_DESTROY(d->move_btn);
+	GTK_WIDGET_DESTROY(d->copy_btn);
+	GTK_WIDGET_DESTROY(d->link_btn);
+	GTK_WIDGET_DESTROY(d->chmod_btn);
+	GTK_WIDGET_DESTROY(d->chown_btn);
+	GTK_WIDGET_DESTROY(d->chtime_btn);
+	GTK_WIDGET_DESTROY(d->cancel_btn);
+#endif
+	gtk_widget_destroy(d->toplevel);
+	gtk_accel_group_unref(d->accelgrp);
 
-	GTK_WIDGET_DESTROY(d->toplevel)
-
-	GTK_ACCEL_GROUP_UNREF(d->accelgrp)
+	d->freeze_count--;
 
 	g_free(d);
 }
