@@ -1,5 +1,5 @@
 /*
-                          Object Operation Dialog
+			  Object Operation Dialog
 
 	Used for the querying of copying, moving, linking, chowning,
 	and changing permissions of objects.
@@ -39,8 +39,8 @@ struct _edv_obj_op_dlg_struct {
 	GtkWidget	*toplevel;
 	GtkAccelGroup	*accelgrp;
 	gboolean	processing;
-	gint		busy_count,
-			freeze_count;
+	gint		freeze_count,
+			busy_count;
 	edv_core_struct *core;
 
 	GtkWidget	*main_vbox,
@@ -113,8 +113,7 @@ struct _edv_obj_op_dlg_struct {
 	edv_obj_op_dlg_op	op;
 
 	/* Source list of objects */
-	edv_object_struct	**src_obj;
-	gint			src_obj_total;
+	GList		*objs_list;
 
 	/* Source directory (relative/current location) */
 	gchar		*src_dir;
@@ -128,9 +127,8 @@ extern void EDVObjOpDlgSetBusy(edv_obj_op_dlg_struct *d, const gboolean is_busy)
 extern gboolean EDVObjOpDlgIsMapped(edv_obj_op_dlg_struct *d);
 extern void EDVObjOpDlgMapValues(
 	edv_obj_op_dlg_struct *d,
-	const edv_obj_op_dlg_op op,	/* One of EDV_OBJ_OP_DLG_OP_* */
-	edv_object_struct **src_obj,	/* Source list, will be coppied */
-	const gint src_obj_total,
+	const edv_obj_op_dlg_op op,
+	GList *objs_list,		/* List of edv_object_struct * */
 	const gchar *src_dir,
 	GtkWidget *toplevel
 );
