@@ -9,7 +9,6 @@
 #include "pie_chart.h"
 #include "edv_types.h"
 #include "edv_obj.h"
-#include "obj_op_dlg.h"
 
 
 typedef struct _edv_propdlg_struct      edv_propdlg_struct;
@@ -23,23 +22,32 @@ struct _edv_propdlg_struct {
 
 	GtkWidget       *toplevel;
 	GtkAccelGroup   *accelgrp;
-	gboolean        processing;
 	gint            busy_count,
 			freeze_count;
+	gboolean	has_changes;
+	gchar		*title;
 	edv_core_struct *core;
 
 	GtkWidget	*main_vbox,
-			*notebook;
-
-	GtkWidget	*ok_btn,
+			*notebook,
+			*ok_btn,
 			*apply_btn,
 			*cancel_btn,
 			*close_btn;
 
-	/* Widgets on General Page */
-	GtkWidget	*gen_icon_pm,
+	/* Information about the object being displayed */
+	gchar		*path;		/* Absolute path */
+	edv_object_type	type;
+	gchar		*mime_type;
+	GList		*ext_props_list;	/* Coppied GList of
+						 * edv_object_prop_struct * */
+
+	/* General Page */
+	GtkWidget	*gen_toplevel,
+			*gen_icon_pm,
 			*gen_name_label,
 			*gen_location_label,
+			*gen_type_label,
 			*gen_size_label,
 			*gen_timestamps_frame,
 			*gen_date_access_label,
@@ -65,35 +73,23 @@ struct _edv_propdlg_struct {
 			*gen_sgid_check,
 			*gen_sticky_check;
 
-	/* Widgets on Details Page */
-	GtkWidget 	*det_vbox,		/* Details main vbox */
+	/* Details Page */
+	GtkWidget 	*det_toplevel,
 			*det_verbose_radio,
 			*det_raw_radio,
-			*det_table,		/* Table for labels */
-			*det_device_label,
-			*det_inode_label,
-			*det_type_label,
-			*det_permissions_label,
-			*det_hard_links_label,
-			*det_uid_label,
-			*det_gid_label,
-			*det_device_type_label,
-			*det_size_label,
-			*det_io_block_size_label,
-			*det_blocks_label,
-			*det_access_time_label,
-			*det_modify_time_label,
-			*det_change_time_label;
+			*det_clist;
 
-	/* Widgets on Link Page */
-	GtkWidget	*link_connected_icon,
-			*link_broken_icon,
-			*link_dest_entry,
-			*link_dest_properties_btn;
+	/* Link Page */
+	GtkWidget	*link_toplevel,
+			*link_target_icon_pm,
+			*link_target_entry,
+			*link_target_make_relative_path_btn,
+			*link_target_properties_btn;
 
-	/* Widgets on Device Page (mount paths, not device nodes) */
+	/* Device Page (for device nodes, see Device Node Page) */
 	gint		dev_num;
-	GtkWidget	*dev_icon_pm,
+	GtkWidget	*dev_toplevel,
+			*dev_icon_pm,
 			*dev_device_path_label,
 			*dev_mount_path_label,
 			*dev_fs_type_label,
@@ -106,19 +102,11 @@ struct _edv_propdlg_struct {
 			*dev_tools_btn,
 			*dev_format_btn;
 
-	/* Widgets on Device Node Page */
-	GtkWidget	*dev_node_major_entry,	/* Major number */
+	/* Device Node Page */
+	GtkWidget	*dev_node_toplevel,
+			*dev_node_major_entry,	/* Major number */
 			*dev_node_minor_entry;	/* Minor number */
 
-	/* Reference to the object being displayed */
-	gchar		*path;		/* Absolute path */
-	edv_object_type	type;
-
-	gchar		*title;		/* Title */
-	gboolean	has_changes;	/* Has changes */
-
-	/* Object operations dialog */
-	edv_obj_op_dlg_struct	*obj_op_dlg;
 };
 
 
@@ -135,6 +123,18 @@ extern void EDVPropDlgObjectModifiedNotifyCB(
 extern void EDVPropDlgObjectRemovedNotifyCB(
 	edv_propdlg_struct *pd, const gchar *path
 );
+extern void EDVPropDlgMIMETypeAddedNotifyCB(
+        edv_propdlg_struct *pd,
+        gint mt_num, edv_mime_type_struct *m
+);
+extern void EDVPropDlgMIMETypeModifiedNotifyCB(
+        edv_propdlg_struct *pd,
+        gint mt_num, edv_mime_type_struct *m
+);
+extern void EDVPropDlgMIMETypeRemovedNotifyCB(
+        edv_propdlg_struct *pd,
+        gint mt_num
+);
 extern void EDVPropDlgMountNotifyCB(
         edv_propdlg_struct *pd, 
         gint dev_num, edv_device_struct *dev,
@@ -145,15 +145,15 @@ extern void EDVPropDlgWriteProtectChangedCB(
 );
 extern void EDVPropDlgReconfiguredNotifyCB(edv_propdlg_struct *pd);
 
-/* Front Ends */
+/* Properties Dialog */
 extern edv_propdlg_struct *EDVPropDlgNew(
 	edv_core_struct *core, edv_object_struct *obj
 );
-extern void EDVPropDlgResetHasChanges(
-	edv_propdlg_struct *pd, gboolean has_changes
+extern void EDVPropDlgSetHasChanges(
+	edv_propdlg_struct *pd, const gboolean has_changes
 );
 extern void EDVPropDlgUpdateMenus(edv_propdlg_struct *pd);
-extern void EDVPropDlgSetBusy(edv_propdlg_struct *pd, gboolean is_busy);
+extern void EDVPropDlgSetBusy(edv_propdlg_struct *pd, const gboolean busy);
 extern gboolean EDVPropDlgIsMapped(edv_propdlg_struct *pd);
 extern void EDVPropDlgMap(edv_propdlg_struct *pd);
 extern void EDVPropDlgUnmap(edv_propdlg_struct *pd);
