@@ -26,7 +26,7 @@
 #include "images/icon_save_20x20.xpm"
 #include "images/icon_trace_20x20.xpm"
 #include "images/icon_run_20x20.xpm"
-#include "images/icon_terminal_20x20.xpm"
+#include "images/icon_terminal2_20x20.xpm"
 #include "images/icon_close_20x20.xpm"
 #include "images/icon_exit_20x20.xpm"
 #include "images/icon_recover_20x20.xpm"
@@ -406,16 +406,15 @@ void EDVRecBinAccelkeysRegenerate(edv_recbin_struct *recbin)
  */
 void EDVRecBinToolBarRegenerate(edv_recbin_struct *recbin)
 {
-	gint i, total_items;
-	GList *glist;
-	toolbar_display tb_display;
-	toolbar_relief tb_relief;
-	GtkWidget *w, *parent;
+	GList *glist, *items_list;
+	GtkWidget *parent;
 	const cfg_intlist_struct *intlist;
 	const cfg_item_struct *cfg_list;
-	edv_recbin_opid_struct *opid;
+	toolbar_display tb_display;
+	toolbar_relief tb_relief;
 	toolbar_struct *toolbar;
-	toolbar_item_struct **item, *item_ptr;
+	toolbar_item_struct *item;
+	edv_recbin_opid_struct *opid;
 	edv_core_struct *core;
 
 	if(recbin == NULL)
@@ -423,12 +422,9 @@ void EDVRecBinToolBarRegenerate(edv_recbin_struct *recbin)
 
 	parent = recbin->tool_bar_handle;
 	core = recbin->core;
-	if((parent == NULL) || (core == NULL))
-	    return;
-
 	cfg_list = core->cfg_list;
 
-	/* Get Tool Bar configuration */
+	/* Get the Tool Bar configuration */
 	tb_display = (toolbar_display)EDV_GET_I(
 	    EDV_CFG_PARM_RECBIN_TOOL_BAR_DISPLAY
 	);
@@ -439,47 +435,34 @@ void EDVRecBinToolBarRegenerate(edv_recbin_struct *recbin)
 	if(intlist == NULL)
 	    return;
 
-	/* Delete existing Tool Bar if any */
+	/* Delete the existing Tool Bar */
 	ToolBarDelete(recbin->tool_bar);
 	recbin->tool_bar = NULL;
 
-	/* Create Tool Bar items list */
-	glist = intlist->list;
-	total_items = g_list_length(glist);
-	if(total_items > 0)
+	/* Create the Tool Bar items list */
+	items_list = NULL;
+	for(glist = intlist->list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
 	{
-	    item = (toolbar_item_struct **)g_malloc0(
-		total_items * sizeof(toolbar_item_struct *)
-	    );
-	    if(item == NULL)
-		total_items = 0;
-	}
-	else
-	{
-	    item = NULL;
-	}
-	for(i = 0; i < total_items; i++)
-	{
-	    /* Get Operation ID */
+	    /* Get the Operation ID */  
 	    opid = EDVRecBinMatchOPID(
-	 	recbin,
-		(edv_recbin_op)((glist != NULL) ?
-		    glist->data : NULL
-		)
+		recbin,
+		(edv_recbin_op)glist->data
 	    );
-	    glist = g_list_next(glist);
 	    if(opid == NULL)
 		continue;
 
 	    switch(opid->op)
 	    {
 	      case EDV_RECBIN_OP_NONE:
-		item[i] = item_ptr = NULL;
 		break;
 
 	      default:
-		item[i] = item_ptr = ToolBarItemNew(
+		item = ToolBarItemNew(
 		    opid->tb_item_type,
+		    NULL,
 		    opid->button_name,
 		    opid->button_icon_data,
 		    opid->tooltip,
@@ -488,26 +471,25 @@ void EDVRecBinToolBarRegenerate(edv_recbin_struct *recbin)
 		    EDVRecBinOPEnterCB, opid,
 		    EDVRecBinOPLeaveCB, opid
 		);
+		items_list = g_list_append(items_list, item);
 		break;
 	    }
 	}
 
-	/* Create new Tool Bar */
+	/* Create the Tool Bar */
 	recbin->tool_bar = toolbar = ToolBarNew(
-	    item, total_items, parent,
+	    parent,
+	    items_list,
 	    tb_display, tb_relief, FALSE
 	);
 	ToolBarMap(toolbar);
 
-	/* Delete Tool Bar items list */
-	ToolBarItemListDeleteAll(&item, &total_items);
-
-	/* Need to queue resize toplevel since the Tool Bar may have
-	 * changed in size
-	 */
-	w = recbin->toplevel;
-	if(w != NULL)
-	    gtk_widget_queue_resize(w);
+	/* Delete the Tool Bar items list */
+	if(items_list != NULL)
+	{
+	    g_list_foreach(items_list, (GFunc)ToolBarItemDelete, NULL);
+	    g_list_free(items_list);
+	}
 }
 
 /*
@@ -846,7 +828,7 @@ GList *EDVRecBinOPIDListNew(edv_recbin_struct *recbin)
 	op = EDV_RECBIN_OP_RUN_TERMINAL;
 	tb_item_type = TOOLBAR_ITEM_BUTTON;
 	flags = 0;
-	button_icon_data = (guint8 **)icon_terminal_20x20_xpm;
+	button_icon_data = (guint8 **)icon_terminal2_20x20_xpm;
 	menu_icon_data = button_icon_data;
 #ifdef PROG_LANGUAGE_ENGLISH
 	button_name = "Terminal";
@@ -2049,35 +2031,30 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 	GdkWindow *window;
 	GtkRcStyle *standard_rcstyle, *lists_rcstyle;
 	GtkStyle *style;
-	GtkWidget *w, *parent, *parent2;
+	GtkWidget	*w,
+			*parent, *parent2,
+			*toplevel;
 	GtkCList *clist;
 	const GtkTargetEntry dnd_tar_types[] = {
-{"text/plain",				0,	EDV_DND_INFO_TEXT_PLAIN},
-{"text/uri-list",			0,	EDV_DND_INFO_TEXT_URI_LIST},
-{"STRING",				0,	EDV_DND_INFO_STRING}
+{GUI_TARGET_NAME_TEXT_PLAIN,	0,	EDV_DND_INFO_TEXT_PLAIN},
+{GUI_TARGET_NAME_TEXT_URI_LIST,	0,	EDV_DND_INFO_TEXT_URI_LIST},
+{GUI_TARGET_NAME_STRING,	0,	EDV_DND_INFO_STRING}
 	};
 	const GtkTargetEntry dnd_src_types[] = {
-{EDV_DND_TARGET_RECYCLED_OBJECT,	0,	EDV_DND_INFO_RECYCLED_OBJECT}
+{EDV_DND_TARGET_RECYCLED_OBJECT, 0,	EDV_DND_INFO_RECYCLED_OBJECT}
 	};
 	const cfg_item_struct *cfg_list;
 	edv_status_bar_struct *status_bar;
-	edv_recbin_struct *recbin = EDV_RECBIN(
-	    g_malloc0(sizeof(edv_recbin_struct))
-	);
-	if((recbin == NULL) || (core == NULL))
-	{
-	    g_free(recbin);
+	edv_recbin_struct *recbin;
+
+	if(core == NULL)
 	    return(NULL);
-	}
 
 	cfg_list = core->cfg_list;
-
 	standard_rcstyle = core->standard_rcstyle;
 	lists_rcstyle = core->lists_rcstyle;
-
 	wm_name = core->wm_name;
 	wm_class = core->wm_class;
-
 	if(core->geometry_set)
 	{
 	    const GdkRectangle *rect = &core->geometry;
@@ -2098,9 +2075,16 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 	find_bar_map_state = EDV_GET_B(EDV_CFG_PARM_RECBIN_SHOW_FIND_BAR);
 	status_bar_map_state = EDV_GET_B(EDV_CFG_PARM_RECBIN_SHOW_STATUS_BAR);
 
+
+	/* Create the Recycle Bin */
+	recbin = EDV_RECBIN(g_malloc0(sizeof(edv_recbin_struct)));
+	if(recbin == NULL)
+	    return(NULL);
+
+	recbin->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	recbin->processing = FALSE;
-	recbin->busy_count = 0;
 	recbin->freeze_count = 0;
+	recbin->busy_count = 0;
 	recbin->core = core;
 
 	recbin->opid_list = EDVRecBinOPIDListNew(recbin);
@@ -2117,11 +2101,12 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 	recbin->find_bar = NULL;
 	recbin->status_bar = NULL;
 
+	recbin->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	recbin->toplevel = w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_widget_set_name(w, EDV_RECBIN_TOPLEVEL_WIDGET_NAME);
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_uposition(
 	    w, toplevel_x, toplevel_y
 	);
@@ -2134,10 +2119,6 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 	    gtk_window_set_wmclass(
 		GTK_WINDOW(w), "recyclebin", PROG_NAME
 	    );
-	gtk_window_set_policy(
-	    GTK_WINDOW(w),
-	    TRUE, TRUE, TRUE
-	);
 	gtk_widget_realize(w);
 	window = w->window;
 	if(window != NULL)
@@ -2270,6 +2251,7 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 	    EDV_RECBIN_CONTENTS_CLIST_COLUMNS_MAX
 	);
 	clist = GTK_CLIST(w);
+	gtk_widget_set_name(w, EDV_RECBIN_CLIST_WIDGET_NAME);
 	gtk_widget_add_events(
 	    w,
 	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
@@ -2397,12 +2379,13 @@ edv_recbin_struct *EDVRecBinNew(edv_core_struct *core)
 		recbin->contents_clist_menu, standard_rcstyle
 	    );
 
-
-	/* Reset values */
+	/* Set the initial values */
 	EDVRecBinSetTitle(recbin);
 	EDVRecBinAccelkeysRegenerate(recbin);
 	EDVRecBinUpdateMenus(recbin);
 
+	recbin->freeze_count--;
+
 	return(recbin);
 }
 
@@ -2443,13 +2426,11 @@ void EDVRecBinUpdateMenus(edv_recbin_struct *recbin)
 	if(recbin == NULL)
 	    return;
 
-	core = recbin->core;
-	if(core == NULL)
-	    return;
-
-	cfg_list = core->cfg_list;
+	recbin->freeze_count++;
 
 	tb = recbin->tool_bar;
+	core = recbin->core;
+	cfg_list = core->cfg_list;
 
 #define DO_SHOW			\
 { if(w != NULL) gtk_widget_show(w); }
@@ -2521,6 +2502,10 @@ void EDVRecBinUpdateMenus(edv_recbin_struct *recbin)
 	recbin->last_recbin_items = core->last_recbin_items;
 
 
+        /* Update the find bar */
+        EDVFindBarUpdateMenus(recbin->find_bar);
+
+
 	/* Begin updating menu items and tool bar items */
 
 	/* File */
@@ -2648,6 +2633,8 @@ void EDVRecBinUpdateMenus(edv_recbin_struct *recbin)
 #undef DO_SHOW
 #undef DO_HIDE
 #undef DO_SET_CHECK_STATE
+
+	recbin->freeze_count--;
 }
 
 /*
@@ -2664,8 +2651,6 @@ void EDVRecBinSetBusy(edv_recbin_struct *recbin, gboolean busy)
 
 	w = recbin->toplevel;
 	core = recbin->core;
-	if((w == NULL) || (core == NULL))
-	    return;
 
 	if(busy)
 	{
@@ -2696,8 +2681,10 @@ void EDVRecBinSetBusy(edv_recbin_struct *recbin, gboolean busy)
  */
 gboolean EDVRecBinIsMapped(edv_recbin_struct *recbin)
 {
-	GtkWidget *w = (recbin != NULL) ? recbin->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+        if(recbin == NULL)
+            return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(recbin->toplevel));
 }
 
 /*
@@ -2705,11 +2692,10 @@ gboolean EDVRecBinIsMapped(edv_recbin_struct *recbin)
  */
 void EDVRecBinMap(edv_recbin_struct *recbin)
 {
-	GtkWidget *w = (recbin != NULL) ? recbin->toplevel : NULL;
-	if(w == NULL)
+	if(recbin == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(recbin->toplevel);
 }
 
 /*
@@ -2717,11 +2703,10 @@ void EDVRecBinMap(edv_recbin_struct *recbin)
  */
 void EDVRecBinUnmap(edv_recbin_struct *recbin)
 {
-	GtkWidget *w = (recbin != NULL) ? recbin->toplevel : NULL;
-	if(w == NULL)
+	if(recbin == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(recbin->toplevel);
 }
 
 /*
@@ -2732,91 +2717,27 @@ void EDVRecBinDelete(edv_recbin_struct *recbin)
 	if(recbin == NULL)
 	    return;
 
-	/* Reset last selected item references */
+	/* Reset the last selected item references */
 	recbin->contents_clist_selected_row = -1;
 
-	/* Destroy right-click menus */
-	recbin->contents_clist_recover_mi = NULL;
-	recbin->contents_clist_purge_mi = NULL;
-	recbin->contents_clist_purge_all_mi = NULL;
-	GTK_WIDGET_DESTROY(recbin->contents_clist_menu)
-	recbin->contents_clist_menu = NULL;
 
-	/* Delete the Status Bar */
+	EDVRecBinUnmap(recbin);
+
+	recbin->freeze_count++;
+
+	GTK_WIDGET_DESTROY(recbin->contents_clist_menu);
+
 	EDVStatusBarDelete(recbin->status_bar);
-	recbin->status_bar = NULL;
-
-	/* Delete the Find Bar */
 	EDVFindBarDelete(recbin->find_bar);
-	recbin->find_bar = NULL;
-
-	/* Delete the Tool Bar */
 	ToolBarDelete(recbin->tool_bar);
-	recbin->tool_bar = NULL;
-
-
-	/* Destroy the Menus and the Menu Bar */
-	recbin->recover_mi = NULL;
-	recbin->purge_mi = NULL;
-	recbin->purge_all_mi = NULL;
-	recbin->sync_recycle_bin_mi = NULL;
-	recbin->sync_disks_mi = NULL;
-	recbin->history_mi = NULL;
-	recbin->run_mi = NULL;
-	recbin->close_mi = NULL;
-
-	recbin->edit_recover_mi = NULL;
-	recbin->edit_purge_mi = NULL;
-	recbin->edit_purge_all_mi = NULL;
-	recbin->select_all_mi = NULL;
-	recbin->unselect_all_mi = NULL;
-	recbin->invert_selection_mi = NULL;
-	recbin->find_mi = NULL;
-
-	recbin->refresh_mi = NULL;
-	recbin->refresh_all_mi = NULL;
-	recbin->show_tool_bar_micheck = NULL;
-	recbin->show_find_bar_micheck = NULL;
-	recbin->show_status_bar_micheck = NULL;
-	recbin->mime_types_mi = NULL;
-
-	recbin->write_protect_mi = NULL;
-	recbin->options_mi = NULL;
-	recbin->customize_mi = NULL;
-
-	recbin->window_new_browser_mi = NULL;
-	recbin->window_new_imbr_mi = NULL;
-	recbin->window_new_archiver_mi = NULL;
-
-	recbin->help_contents_mi = NULL;
-	recbin->help_recycle_bin_mi = NULL;
-	recbin->help_keys_list_mi = NULL;
-	recbin->help_common_operations_mi = NULL;
-	recbin->help_about_mi = NULL;
-
-	GTK_WIDGET_DESTROY(recbin->menu_bar)
-	recbin->menu_bar = NULL;
-
-
-	/* Destroy toplevel and the rest of the widgets */
-	GTK_WIDGET_DESTROY(recbin->menu_bar_handle)
-	recbin->menu_bar_handle = NULL;
-	GTK_WIDGET_DESTROY(recbin->tool_bar_handle)
-	recbin->tool_bar_handle = NULL;
-	GTK_WIDGET_DESTROY(recbin->bars_vbox)
-	recbin->bars_vbox = NULL;
-	GTK_WIDGET_DESTROY(recbin->contents_clist)
-	recbin->contents_clist = NULL;
-	GTK_WIDGET_DESTROY(recbin->main_vbox)
-	recbin->main_vbox = NULL;
-	GTK_WIDGET_DESTROY(recbin->toplevel)
-	recbin->toplevel = NULL;
-
+	GTK_WIDGET_DESTROY(recbin->toplevel);
 
 	/* Delete all Operation IDs */
 	EDVRecBinOPIDListDelete(recbin->opid_list);
 
 	g_free(recbin->contents_list_filter);
 
+	recbin->freeze_count--;
+
 	g_free(recbin);
 }
