@@ -151,8 +151,8 @@ struct _edv_recbin_struct {
 
 	GtkWidget	*toplevel;
 	gboolean	processing;
-	gint		busy_count,
-			freeze_count;
+	gint		freeze_count,
+			busy_count;
 	edv_core_struct *core;
 
 	GList		*opid_list;
