@@ -290,10 +290,10 @@ gint EDVRecBinDeleteEventCB(
 {
 	edv_recbin_struct *recbin = EDV_RECBIN(data);
 	if(recbin == NULL)
-	    return(TRUE);
+	    return(FALSE);
 
-	if(recbin->processing)
-	    return(TRUE);
+	if(recbin->processing || (recbin->freeze_count > 0))
+	    return(FALSE);
 
 	EDVRecBinOPClose(recbin);
 
@@ -322,22 +322,20 @@ gint EDVRecBinKeyEventCB(
 	if(recbin->processing)
 	    return(status);
 
-	core = recbin->core;
-	if(core == NULL)
-	    return(status);
-
 	w = recbin->toplevel;
-	focus_widget = (w != NULL) ? GTK_WINDOW(w)->focus_widget : NULL;
+	focus_widget = GTK_WINDOW(w)->focus_widget;
+	core = recbin->core;
 	cfg_list = core->cfg_list;
+
 	etype = key->type;
 	is_press = (etype == GDK_KEY_PRESS) ? TRUE : FALSE;
 	keyval = key->keyval;
 	state = key->state;
 
 /* Stop emit of signal */
-#define DO_STOP_KEY_SIGNAL_EMIT	{		\
+#define STOP_SIGNAL_EMISSION(_w_) {		\
  gtk_signal_emit_stop_by_name(			\
-  GTK_OBJECT(widget),				\
+  GTK_OBJECT(_w_),				\
   is_press ?					\
    "key_press_event" : "key_release_event"	\
  );						\
@@ -364,7 +362,7 @@ gint EDVRecBinKeyEventCB(
 		    if(opid != NULL)
 			EDVRecBinOPCB(NULL, -1, opid);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		return(status);
 	    }
@@ -406,7 +404,7 @@ gint EDVRecBinKeyEventCB(
 			gtk_clist_select_row(clist, row, 0);
 		    gtk_clist_thaw(clist);
 		}
-		DO_STOP_KEY_SIGNAL_EMIT
+		STOP_SIGNAL_EMISSION(widget);
 		status = TRUE;
 		break;
 
@@ -416,45 +414,37 @@ gint EDVRecBinKeyEventCB(
 		 * item who's name starts with the letter of the key
 		 * that was pressed
 		 */
-		if(isalnum((int)keyval) && is_press &&
-		   !(state & GDK_CONTROL_MASK) &&
-		   !(state & GDK_SHIFT_MASK)    
-		)
+		if(isalnum((int)keyval) && is_press)
 		{
-		    /* Get the column that is displaying the object
-		     * name as column_type_name
-		     */
-		    gint column_type_name = -1;
-		    const cfg_item_struct *cfg_list = core->cfg_list;
-		    const cfg_intlist_struct *intlist = EDV_GET_INTLIST(
+                    const gboolean backwards = (state & GDK_MOD1_MASK) ? TRUE : FALSE;
+                    const gint start_row = (backwards) ?
+                        (clist->rows - 1) : 0;
+
+		    /* Find the column that is set to display the name */
+		    cfg_intlist_struct *intlist = EDV_GET_INTLIST(
 			EDV_CFG_PARM_RECBIN_CONTENTS_COLUMN
 		    );
-		    if(intlist != NULL)
-		    {
-			/* Iterate through column type intlist */
-			gint i = 0;
-			GList *glist = intlist->list;
-			while(glist != NULL)
-			{
-			    if((edv_recbin_column_type)glist->data ==
-				EDV_RECBIN_COLUMN_TYPE_NAME
-			    )
-			    {
-				column_type_name = i;
-				break;
-			    }
-			    i++;
-			    glist = g_list_next(glist);
-			}
-		    }
+		    const gint column_type_name = (intlist != NULL) ?
+			g_list_index(
+			    intlist->list,
+			    (gpointer)EDV_RECBIN_COLUMN_TYPE_NAME
+			) : 0;
 
+		    /* Seek to the row who's text matches this
+		     * key event's character
+		     */
 		    gtk_clist_freeze(clist);
 		    EDVCListSeekCharacter(
-			clist, column_type_name, 0, keyval
+			clist,
+			column_type_name,	/* Column */
+			start_row,
+			backwards,
+			0,			/* Text index */
+			keyval			/* Character */
 		    );
 		    gtk_clist_thaw(clist);
 
-		    DO_STOP_KEY_SIGNAL_EMIT
+		    STOP_SIGNAL_EMISSION(widget);
 		    status = TRUE;
 		}
 		break;
@@ -462,7 +452,7 @@ gint EDVRecBinKeyEventCB(
 	}
 
 	return(status);
-#undef DO_STOP_KEY_SIGNAL_EMIT
+#undef STOP_SIGNAL_EMISSION
 }
 
 /*
@@ -845,8 +835,6 @@ void EDVRecBinSelectRowCB(
 	/* Check which clist this signal is for */
 	if(GTK_WIDGET(clist) == recbin->contents_clist)
 	{
-	    edv_core_struct *core = recbin->core;
-
 	    /* Get total number of objects selected */
 	    const gint nselected = g_list_length(clist->selection);
 
@@ -880,12 +868,9 @@ void EDVRecBinSelectRowCB(
 		      case EDV_OBJECT_TYPE_FILE:
 			type_str = "File";
 			size_str = g_strdup_printf(
-			    " (%s byte%s)",
-			    EDVGetObjectSizeStr(
-				recbin->core,
-				obj->size
-			    ),
-			    (obj->size == 1) ? "" : "s"
+			    " (%s %s)",
+			    EDVSizeStrDelim(obj->size),
+			    (obj->size == 1l) ? "byte" : "bytes"
 			);
 			break;
 		      case EDV_OBJECT_TYPE_DIRECTORY:
@@ -922,7 +907,7 @@ void EDVRecBinSelectRowCB(
 		    if(nselected > 1)
 			s = g_strdup_printf(
 			    "%s objects selected",
-			    EDVGetObjectSizeStr(core, nselected)
+			    EDVSizeStrDelim(nselected)
 			);
 		    else if(!strcmp(obj->name, ".."))
 			s = g_strdup_printf(
@@ -986,7 +971,6 @@ void EDVRecBinUnselectRowCB(
 	if(GTK_WIDGET(clist) == recbin->contents_clist)
 	{
 	    gchar *s;
-	    edv_core_struct *core = recbin->core;
 
 	    /* Get total number of objects selected */
 	    const gint nselected = g_list_length(clist->selection);
@@ -994,9 +978,9 @@ void EDVRecBinUnselectRowCB(
 	    /* Update status bar message */
 	    if(nselected > 0)
 		s = g_strdup_printf(
-		    "%s object%s selected",
-		    EDVGetObjectSizeStr(core, nselected),
-		    (nselected == 1) ? "" : "s"  
+		    "%s %s selected",
+		    EDVSizeStrDelim(nselected),
+		    (nselected == 1) ? "object" : "objects"  
 		);
 	    else
 		s = STRDUP("No objects selected");
@@ -1202,8 +1186,6 @@ void EDVRecBinRecycledObjectModifiedNotifyCB(
 	    return;
 
 	core = recbin->core;
-	if(core == NULL)
-	    return;
 
 	/* Contents List */
 	EDVRecBinContentsObjectModifiedNotify(recbin, index);
@@ -1232,8 +1214,6 @@ void EDVRecBinRecycledObjectRemovedNotifyCB(
 	    return;
 
 	core = recbin->core;
-	if(core == NULL)
-	    return;
 
 	/* Contents List */
 	EDVRecBinContentsObjectRemovedNotify(recbin, index);
@@ -1265,22 +1245,17 @@ void EDVRecBinReconfiguredNotifyCB(edv_recbin_struct *recbin)
 	    return;
 
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 	standard_rcstyle = core->standard_rcstyle;
 	lists_rcstyle = core->lists_rcstyle;
 
-
 	/* Reset last state markers so that resources get explicitly
 	 * checked due to reconfiguring
 	 */
 	recbin->last_recbin_items = -1;
 	recbin->last_write_protect_state = -1;
 
-
-	/* Update title */
+	/* Update the title */
 	EDVRecBinSetTitle(recbin);
 
 	/* Update Accelkey Labels */
