@@ -57,7 +57,9 @@ static void EDVRecBinContentsSetCellSize(
 	edv_core_struct *core, edv_recbin_struct *recbin,
 	GtkCList *clist, edv_recycled_object_struct *obj,
 	const gint row, const gint column,
-	gboolean hide_dir_size, gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 );
 static void EDVRecBinContentsSetCellType(
 	edv_core_struct *core, edv_recbin_struct *recbin,
@@ -237,6 +239,7 @@ static void EDVRecBinContentsResetColumns(
 	if(column_width_intlist == NULL)
 	    return;
 
+	gtk_clist_freeze(clist);
 
 	/* Update the clist's column settings */
 	gtk_clist_column_titles_active(clist);
@@ -348,6 +351,8 @@ static void EDVRecBinContentsResetColumns(
 	    gtk_widget_modify_style_recursive(
 		GTK_WIDGET(clist), lists_rcstyle
 	    );
+
+	gtk_clist_thaw(clist);
 }
 
 
@@ -388,6 +393,7 @@ static void EDVRecBinContentsSetCellName(
 	}
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	if(pixmap != NULL)
 	    gtk_clist_set_pixtext(
 		clist, row, column,
@@ -404,6 +410,7 @@ static void EDVRecBinContentsSetCellName(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -414,7 +421,9 @@ static void EDVRecBinContentsSetCellSize(
 	edv_core_struct *core, edv_recbin_struct *recbin,
 	GtkCList *clist, edv_recycled_object_struct *obj,
 	const gint row, const gint column,
-	gboolean hide_dir_size, gboolean hide_link_size
+	const gboolean hide_dir_size, const gboolean hide_link_size,
+        const edv_size_format size_format,
+        const gulong block_size
 )
 {
 	const gint border_minor = 2;
@@ -431,16 +440,20 @@ static void EDVRecBinContentsSetCellSize(
 	)
 	    s = "";
 	else
-	    s = EDVGetObjectSizeStr(core, obj->size);
+	    s = EDVSizeStrFormat(
+		obj->size, size_format, block_size, ',', TRUE
+	    );
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, s
+	    clist, row, column, (s != NULL) ? s : ""
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -464,6 +477,7 @@ static void EDVRecBinContentsSetCellType(
 	);
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, (type_str != NULL) ? type_str : ""
 	);
@@ -471,6 +485,7 @@ static void EDVRecBinContentsSetCellType(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -511,6 +526,7 @@ static void EDVRecBinContentsSetCellPermissions(
 	    );
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, s
 	);
@@ -518,6 +534,7 @@ static void EDVRecBinContentsSetCellPermissions(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -531,21 +548,24 @@ static void EDVRecBinContentsSetCellOwner(
 )
 {
 	/* Get owner name from object's user id */
-	const gchar *owner_name = EDVUIDGetNameFromUID(
-	    core->uid, core->total_uids,
+	gchar *owner_name = EDVUIDGetNameFromUID(
+	    core->uids_list,
 	    obj->owner_id, NULL
 	);
-	if(owner_name == NULL)
-	    owner_name = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, owner_name
+	    clist, row, column,
+	    (owner_name != NULL) ? owner_name : ""
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
+
+	g_free(owner_name);
 }
 
 /*
@@ -559,21 +579,24 @@ static void EDVRecBinContentsSetCellGroup(
 )
 {
 	/* Get group name from object's group id */
-	const gchar *group_name = EDVGIDGetNameFromGID(
-	    core->gid, core->total_gids,
+	gchar *group_name = EDVGIDGetNameFromGID(
+	    core->gids_list,
 	    obj->group_id, NULL
 	);
-	if(group_name == NULL)
-	    group_name = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, group_name
+	    clist, row, column,
+	    (group_name != NULL) ? group_name : ""
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
+
+	g_free(group_name);
 }
 
 /*
@@ -599,6 +622,7 @@ static void EDVRecBinContentsSetCellDateAccess(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -606,6 +630,7 @@ static void EDVRecBinContentsSetCellDateAccess(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -631,6 +656,7 @@ static void EDVRecBinContentsSetCellDateModified(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -638,6 +664,7 @@ static void EDVRecBinContentsSetCellDateModified(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -663,6 +690,7 @@ static void EDVRecBinContentsSetCellDateChanged(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -670,6 +698,7 @@ static void EDVRecBinContentsSetCellDateChanged(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -695,6 +724,7 @@ static void EDVRecBinContentsSetCellDateDeleted(
 	    date_str = "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, date_str
 	);
@@ -702,6 +732,7 @@ static void EDVRecBinContentsSetCellDateDeleted(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 
@@ -715,19 +746,21 @@ static void EDVRecBinContentsSetCellLinkedTo(
 	const gint row, const gint column
 )
 {
-	/* Get link destination */
-	const gchar *link_dest = ((obj->link_value != NULL) &&
+	/* Get the link's target value */
+	const gchar *target = ((obj->link_target != NULL) &&
 	    (obj->type == EDV_OBJECT_TYPE_LINK)) ?
-	    obj->link_value : "";
+	    obj->link_target : "";
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
-	    clist, row, column, link_dest
+	    clist, row, column, target
 	);
 	gtk_clist_set_cell_style(clist, row, column, NULL);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -782,6 +815,7 @@ static void EDVRecBinContentsSetCellOriginalLocation(
 	EDVObjectDelete(loc_obj);
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	if(pixmap != NULL)
 	    gtk_clist_set_pixtext(
 		clist, row, column,
@@ -796,6 +830,7 @@ static void EDVRecBinContentsSetCellOriginalLocation(
 	gtk_clist_set_shift(
 	    clist, row, column, 0, 0
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -819,12 +854,14 @@ static void EDVRecBinContentsSetCellIndex(
 	);
 
 	/* Set cell */
+	gtk_clist_freeze(clist);
 	gtk_clist_set_text(
 	    clist, row, column, num_str
 	);
 	gtk_clist_set_shift(
 	    clist, row, column, 0, -border_minor
 	);
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -850,9 +887,11 @@ static void EDVRecBinContentsSetCellCapacityUsed(
 
 	if(recbin_size_max == 0l)
 	{
+	    gtk_clist_freeze(clist);
 	    gtk_clist_set_text(
 		clist, row, column, ""
 	    );
+	    gtk_clist_thaw(clist);
 	    return;
 	}
 
@@ -873,11 +912,14 @@ static void EDVRecBinContentsSetCellCapacityUsed(
 	);
 
         /* Set the cell */
+	gtk_clist_freeze(clist);
         if(pixmap != NULL)
             gtk_clist_set_pixmap(clist, row, column, pixmap, mask);
         else 
             gtk_clist_set_text(clist, row, column, "");
         gtk_clist_set_cell_style(clist, row, column, NULL);
+	gtk_clist_thaw(clist);
+
         GDK_PIXMAP_UNREF(pixmap);
         GDK_BITMAP_UNREF(mask);
 }
@@ -901,10 +943,11 @@ static void EDVRecBinContentsSetRow(
 )
 {
 	gboolean hide_dir_size, hide_link_size, hide_link_permissions;
-	gulong recbin_size_max;
+	gulong recbin_size_max, block_size;
 	gint i;
 	const gchar *date_format;
 	GList *glist;
+	edv_size_format size_format;
 	edv_date_relativity date_relativity;
 	edv_recbin_column_type column_type;
 	cfg_intlist_struct *column_types_intlist;
@@ -918,6 +961,10 @@ static void EDVRecBinContentsSetRow(
 	    return;
 
 	/* Get additional display options */
+	size_format = (edv_size_format)EDV_GET_I(
+	    EDV_CFG_PARM_SIZE_FORMAT
+	);
+	block_size = EDV_GET_UL(EDV_CFG_PARM_BLOCK_SIZE);
 	date_relativity = (edv_date_relativity)EDV_GET_I(
 	    EDV_CFG_PARM_DATE_RELATIVITY
 	);
@@ -930,6 +977,8 @@ static void EDVRecBinContentsSetRow(
 	    EDV_CFG_PARM_RECBIN_SIZE_WARN
 	);
 
+	gtk_clist_freeze(clist);
+
 	/* Iterate through each column */
 	for(i = 0, glist = column_types_intlist->list;
 	    glist != NULL;
@@ -948,7 +997,8 @@ static void EDVRecBinContentsSetRow(
 	      case EDV_RECBIN_COLUMN_TYPE_SIZE:
 		EDVRecBinContentsSetCellSize(
 		    core, recbin, clist, obj,
-		    row, i, hide_dir_size, hide_link_size
+		    row, i, hide_dir_size, hide_link_size,
+		    size_format, block_size
 		);
 		break;
 	      case EDV_RECBIN_COLUMN_TYPE_TYPE:
@@ -1027,6 +1077,8 @@ static void EDVRecBinContentsSetRow(
 		break;
 	    }
 	}
+
+	gtk_clist_thaw(clist);
 }
 
 /*
@@ -1049,13 +1101,15 @@ static gint EDVRecBinContentsAppendObject(
 	edv_recycled_object_struct *obj
 )
 {
-	const gint columns = MAX(clist->columns, 1);
+	const gint ncolumns = MAX(clist->columns, 1);
 	gint i, new_row;
 	gchar **strv;
 
+	gtk_clist_freeze(clist);
+
 	/* Allocate the row cell values */
-	strv = (gchar **)g_malloc0(columns * sizeof(gchar *));
-	for(i = 0; i < columns; i++)
+	strv = (gchar **)g_malloc(ncolumns * sizeof(gchar *));
+	for(i = 0; i < ncolumns; i++)
 	    strv[i] = "";
 
 	/* Append the new row */
@@ -1067,6 +1121,7 @@ static gint EDVRecBinContentsAppendObject(
 	/* Failed to append row? */
 	if(new_row < 0)
 	{
+	    gtk_clist_thaw(clist);
 	    EDVRecycledObjectDelete(obj);
 	    return(-1);
 	}
@@ -1084,6 +1139,8 @@ static gint EDVRecBinContentsAppendObject(
 	    obj, EDVRecBinContentsItemDestroyCB
 	);
 
+	gtk_clist_thaw(clist);
+
 	return(new_row);
 }
 
@@ -1173,7 +1230,9 @@ static void EDVRecBinContentsAppendListing(
   }							\
  }							\
 }
+
 	/* Begin reading the recycle bin index file */
+	gtk_clist_freeze(clist);
 	while(!EDVRecBinIndexNext(rbi_ptr))
 	{
 	    obj = rbi_ptr->obj;
@@ -1243,6 +1302,8 @@ static void EDVRecBinContentsAppendListing(
 		}
 	    }
 	}
+	gtk_clist_thaw(clist);
+
 #undef UPDATE_PROGRESS
 
 	/* Close the recycle bin index file */
@@ -1258,34 +1319,33 @@ static void EDVRecBinContentsAppendListing(
 }
 
 /*
- *	Returns the row index of the contents clist that contains a
- *	recycled object structure who's index matches the given index.
+ *	Finds the row by index.
  *
- *	Can return -1 on failed match.
+ *	The index specifies the recycled object index to match
+ *	with a row's recycled object index.
+ *
+ *	Returns the matched row or negative on error.
  */
 gint EDVRecBinContentsFindRowByIndex(
 	edv_recbin_struct *recbin, const guint index
 )
 {
-	gint i;
+	gint i, n;
 	GtkCList *clist;
 	edv_recycled_object_struct *obj;
 
 	if(recbin == NULL)
-	    return(-1);
+	    return(-2);
 
-	clist = (GtkCList *)recbin->contents_clist;
-	if(clist == NULL)
-	    return(-1);
+	clist = GTK_CLIST(recbin->contents_clist);
 
-	/* Iterate through all rows */
-	for(i = 0; i < clist->rows; i++)
+	n = clist->rows;
+	for(i = 0; i < n; i++)
 	{
 	    obj = EDV_RECYCLED_OBJECT(gtk_clist_get_row_data(clist, i));
 	    if(obj == NULL)
 		continue;
 
-	    /* Full path not specified on disk object structure? */
 	    if(obj->index == index)
 		return(i);
 	}
@@ -1306,8 +1366,7 @@ gint EDVRecBinContentsFindRowByIndex(
  */
 void EDVRecBinContentsRealizeListing(edv_recbin_struct *recbin)
 {
-	gint i;
-	GtkWidget *w;
+	gint i, n;
 	GtkCList *clist;
 	edv_core_struct *core;
 	edv_recycled_object_struct *obj;
@@ -1315,12 +1374,8 @@ void EDVRecBinContentsRealizeListing(edv_recbin_struct *recbin)
 	if(recbin == NULL)
 	    return;
 
-	w = recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
-	clist = GTK_CLIST(w);
 
 	gtk_clist_freeze(clist);
 
@@ -1328,7 +1383,8 @@ void EDVRecBinContentsRealizeListing(edv_recbin_struct *recbin)
 	EDVRecBinContentsResetColumns(core, recbin, clist);
 
 	/* Update the rows */
-	for(i = 0; i < clist->rows; i++)
+	n = clist->rows;
+	for(i = 0; i < n; i++)
 	{
 	    obj = EDV_RECYCLED_OBJECT(gtk_clist_get_row_data(clist, i));
 	    if(obj == NULL)
@@ -1359,19 +1415,14 @@ void EDVRecBinContentsGetListing(
 	const gboolean update_status_bar
 )
 {
-	GtkWidget *w;
 	GtkCList *clist;
 	edv_core_struct *core;
 
 	if(recbin == NULL)
 	    return;
 
-	w = recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
-	clist = GTK_CLIST(w);
 
 	/* Report the initial progress? */
 	if(update_status_bar)
@@ -1427,10 +1478,7 @@ void EDVRecBinContentsClear(edv_recbin_struct *recbin)
 	if(recbin == NULL)
 	    return;
 
-	clist = (GtkCList *)recbin->contents_clist;
-	if(clist == NULL)
-	    return;
-
+	clist = GTK_CLIST(recbin->contents_clist);
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 	gtk_clist_thaw(clist);
@@ -1438,7 +1486,7 @@ void EDVRecBinContentsClear(edv_recbin_struct *recbin)
 
 
 /*
- *	Recycled object added notify.
+ *	Recycled object added callback.
  */
 void EDVRecBinContentsObjectAddedNotify(
 	edv_recbin_struct *recbin, guint index
@@ -1446,7 +1494,6 @@ void EDVRecBinContentsObjectAddedNotify(
 {
 	gint row;
 	const gchar *recbin_index_file;
-	GtkWidget *w;
 	GtkCList *clist;
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
@@ -1454,12 +1501,8 @@ void EDVRecBinContentsObjectAddedNotify(
 	if(recbin == NULL)
 	    return;
 
-	w = recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
-	clist = GTK_CLIST(w);
 	cfg_list = core->cfg_list;
 
 	/* Get the full path to recycle bin index file */
@@ -1471,7 +1514,7 @@ void EDVRecBinContentsObjectAddedNotify(
 	 * exists in the list
 	 */
 	row = EDVRecBinContentsFindRowByIndex(recbin, index);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
 	{
 	    /* Recycled object already in list, do nothing (no update) */
 	}
@@ -1483,21 +1526,17 @@ void EDVRecBinContentsObjectAddedNotify(
 	    );
 	    if(obj != NULL)
 	    {
-		/* Append row, transfering the recycled object structure
-		 * tar_obj to the new row. The tar_obj should not be
-		 * referenced after this point.
-		 */
-		row = EDVRecBinContentsAppendObject(
+		/* Add the recycled object to the list */
+		EDVRecBinContentsAppendObject(
 		    core, recbin, clist,
 		    obj
 		);
-		obj = NULL;
 	    }
 	}
 }
 
 /*
- *	Recycled object modified notify.
+ *	Recycled object modified callback.
  */
 void EDVRecBinContentsObjectModifiedNotify(
 	edv_recbin_struct *recbin, guint index
@@ -1505,7 +1544,6 @@ void EDVRecBinContentsObjectModifiedNotify(
 {
 	gint row;
 	const gchar *recbin_index_file;
-	GtkWidget *w;
 	GtkCList *clist;
 	const cfg_item_struct *cfg_list;
 	edv_core_struct *core;
@@ -1513,12 +1551,8 @@ void EDVRecBinContentsObjectModifiedNotify(
 	if(recbin == NULL)
 	    return;
 
-	w = recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((w == NULL) || (core == NULL))
-	    return;
-
-	clist = GTK_CLIST(w);
 	cfg_list = core->cfg_list;
 
 	/* Get the full path to recycle bin index file */
@@ -1528,16 +1562,18 @@ void EDVRecBinContentsObjectModifiedNotify(
 
 	/* Update modified recycled object in the list */
 	row = EDVRecBinContentsFindRowByIndex(recbin, index);
-	if((row >= 0) && (row < clist->rows))
+	if(row > -1)
 	{
-	    /* Update the existing recycled object in the list */
-	    edv_recycled_object_struct *obj;
-	    EDVRecycledObjectDelete(EDV_RECYCLED_OBJECT(
-		gtk_clist_get_row_data(clist, row)
-	    ));
-	    obj = EDVRecBinObjectStat(recbin_index_file, index);
+	    /* Create a new recycled object describing the modified
+	     * recycled object
+	     */
+	    edv_recycled_object_struct *obj = EDVRecBinObjectStat(
+		recbin_index_file, index
+	    );
 	    if(obj != NULL)
 	    {
+		/* Update the recycled object */
+		gtk_clist_freeze(clist);
 		gtk_clist_set_row_data_full(
 		    clist, row,
 		    obj, EDVRecBinContentsItemDestroyCB
@@ -1547,49 +1583,53 @@ void EDVRecBinContentsObjectModifiedNotify(
 		    obj,
 		    row
 		);
+		gtk_clist_thaw(clist);
 	    }
 	}
 	else
 	{
-	    /* Append this recycled object to the list */
+	    /* The modified recycled object was not in the list, so
+	     * add the modified recycled object to the list
+	     */
 	    edv_recycled_object_struct *obj = EDVRecBinObjectStat(
 		recbin_index_file, index
 	    );
 	    if(obj != NULL)
-		row = EDVRecBinContentsAppendObject(
+	    {
+		EDVRecBinContentsAppendObject(
 		    core, recbin, clist,
 		    obj
 		);
+	    }
 	}
 }
 
 /*
- *	Recycled object removed notify.
+ *	Recycled object removed callback.
  */
 void EDVRecBinContentsObjectRemovedNotify(
 	edv_recbin_struct *recbin, guint index
 )
 {
-	GtkWidget *w;
-	GtkCList *clist;
 	gint row;
-	edv_core_struct *core;
+	GtkCList *clist;
 
 	if(recbin == NULL)
 	    return;
 
-	core = recbin->core;
-	if(core == NULL)
-	    return;
+	clist = GTK_CLIST(recbin->contents_clist);
 
-	w = recbin->contents_clist;
-	if(w == NULL)
-	    return;
-	else
-	    clist = GTK_CLIST(w);
-
-	/* Remove the recycled object from the list */
+	/* Remove all the rows who's object's index matches the
+	 * removed index
+	 */
 	row = EDVRecBinContentsFindRowByIndex(recbin, index);
-	if((row >= 0) && (row < clist->rows))
-	    gtk_clist_remove(clist, row);
+	if(row > -1)
+	{
+	    gtk_clist_freeze(clist);
+	    do {
+		gtk_clist_remove(clist, row);
+		row = EDVRecBinContentsFindRowByIndex(recbin, index);
+	    } while(row > -1);
+	    gtk_clist_thaw(clist);
+	}
 }
