@@ -634,12 +634,7 @@ void EDVRecBinOPClose(edv_recbin_struct *recbin)
 	if(recbin == NULL)
 	    return;
 
-	/* Set current properties of the recycle bin to the configuration
-	 * list.
-	 */
 	EDVRecBinSyncConfiguration(recbin);
-
-	/* Unmap the recbin */
 	EDVRecBinUnmap(recbin);
 }
 
@@ -694,26 +689,24 @@ void EDVRecBinOPRecover(edv_recbin_struct *recbin)
 	    return;
 
 	toplevel = recbin->toplevel;
-	clist = (GtkCList *)recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((clist == NULL) || (core == NULL))
-	    return;
 
 	/* Check and warn if write protect is enabled */
 	if(EDVCheckWriteProtect(core, TRUE, toplevel))
 	    return;
 
-#define DO_FREE_LOCALS	{				\
+#define CLEANUP_RETURN	{				\
  if(obj_list != NULL) {					\
   g_list_foreach(					\
    obj_list, (GFunc)EDVRecycledObjectDelete, NULL	\
   );							\
   g_list_free(obj_list);				\
  }							\
+							\
+ return;						\
 }
 
-	EDVRecBinSetBusy(recbin, TRUE);
-
 	/* Get the list of selected recycled objects */
 	for(glist = clist->selection;
             glist != NULL;
@@ -734,13 +727,13 @@ void EDVRecBinOPRecover(edv_recbin_struct *recbin)
 	/* No recycled objects to recover? */
 	if(obj_list == NULL)
 	{
-	    DO_FREE_LOCALS
-	    EDVRecBinSetBusy(recbin, FALSE);
-	    return;
+	    CLEANUP_RETURN;
 	}
 
 	nobjs = g_list_length(obj_list);
 
+	EDVRecBinSetBusy(recbin, TRUE);
+
 	/* Confirm recover */
 	status = EDVConfirmRecover(
 	    core, toplevel,
@@ -755,9 +748,8 @@ void EDVRecBinOPRecover(edv_recbin_struct *recbin)
 	    break;
 
 	  default:
-	    DO_FREE_LOCALS
 	    EDVRecBinSetBusy(recbin, FALSE);
-	    return;
+	    CLEANUP_RETURN;
 	    break;
 	}
 
@@ -850,17 +842,16 @@ No longer exists, create it?",
 		TRUE, TRUE, &yes_to_all
 	    );
 
-	    /* Get error message (if any) describing the error that
-	     * might have occured in the above operation
-	     */
-	    error_msg = EDVRecBinRecoverGetError();
+	    /* Check for errors */
+	    error_msg = EDVRecBinRecoverGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
-		EDVMessageObjectOPError(
+		EDVMessageError(
 		    "Recover Recycled Object Error",
 		    error_msg,
-		    obj->name,
+		    NULL,
 		    toplevel
 		);
 	    }
@@ -897,37 +888,36 @@ No longer exists, create it?",
 	ProgressDialogBreakQuery(TRUE);
 	ProgressDialogSetTransientFor(NULL);
 
-	/* Play completed sound on success */
+	/* Play the "completed" sound on success */
 	if(status == 0)
 	    EDVPlaySoundCompleted(core);
 
 	/* Update the status bar message */
 	if(TRUE)
 	{
-	    gchar *s;
+	    gchar *msg;
 	    if(status == -4)
-		s = STRDUP(
+		msg = STRDUP(
 		    "Recover operation canceled"
 		);
 	    else if(nobjs_recovered > 0)
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Recovered %i %s",
 		    nobjs_recovered,
 		    (nobjs_recovered == 1) ? "recycled object" : "recycled objects" 		);
 	    else
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Unable to recover %s",
 		    (nobjs == 1) ? "recycled object" : "recycled objects"
 		);
-	    EDVStatusBarMessage(recbin->status_bar, s, FALSE);
-	    g_free(s);
+	    EDVStatusBarMessage(recbin->status_bar, msg, FALSE);
+	    g_free(msg);
 	}
 
-	DO_FREE_LOCALS
-
 	EDVRecBinSetBusy(recbin, FALSE);
 
-#undef DO_FREE_LOCALS
+	CLEANUP_RETURN;
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -949,21 +939,22 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 	    return;
 
 	toplevel = recbin->toplevel;
-	clist = (GtkCList *)recbin->contents_clist;
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if((clist == NULL) || (core == NULL))
-	    return;
 
 	/* Check and warn if write protect is enabled */
 	if(EDVCheckWriteProtect(core, TRUE, toplevel))
 	    return;
 
-#define DO_FREE_LOCALS	{				\
- g_list_foreach(					\
-  obj_list, (GFunc)EDVRecycledObjectDelete, NULL		\
- );							\
- g_list_free(obj_list);					\
- obj_list = NULL;					\
+#define CLEANUP_RETURN	{				\
+ if(obj_list != NULL) {					\
+  g_list_foreach(					\
+   obj_list, (GFunc)EDVRecycledObjectDelete, NULL	\
+  );							\
+  g_list_free(obj_list);				\
+ }							\
+							\
+ return;						\
 }
 
 	/* Get a list of selected recycled objects indices */
@@ -986,12 +977,13 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 	/* No recycled objects to purge? */
 	if(obj_list == NULL)
 	{
-	    DO_FREE_LOCALS
-	    return;
+	    CLEANUP_RETURN;
 	}
 
 	nobjs = g_list_length(obj_list);
 
+	EDVRecBinSetBusy(recbin, TRUE);
+
 	/* Confirm purge */
 	status = EDVConfirmPurge(
 	    core, toplevel,
@@ -1005,8 +997,8 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 	    break;
 
 	  default:
-	    DO_FREE_LOCALS
-	    return;
+	    EDVRecBinSetBusy(recbin, FALSE);
+	    CLEANUP_RETURN;
 	    break;
 	}
 
@@ -1015,8 +1007,6 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 	 */
 	obj = NULL;
 
-	EDVRecBinSetBusy(recbin, TRUE);
-
 	/* Iterate through list of recycled objects and purge each
 	 * one
 	 */
@@ -1039,17 +1029,16 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 		TRUE, TRUE, &yes_to_all
 	    );
 
-	    /* Get the error message describing the error that might
-	     * have occured in the above operation
-	     */
-	    error_msg = EDVRecBinPurgeGetError();
+	    /* Check for errors */
+	    error_msg = EDVRecBinPurgeGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
-		EDVMessageObjectOPError(
+		EDVMessageError(
 		    "Purge Recycled Object Error",
 		    error_msg,
-		    obj->name,
+		    NULL,
 		    toplevel
 		);
 	    }
@@ -1066,43 +1055,41 @@ void EDVRecBinOPPurge(edv_recbin_struct *recbin)
 		break;
 	}
 
-	/* Unmap the progress dialog, it may have been mapped during
-	 * the above operation
-	 */
+	/* Unmap the progress dialog */
 	ProgressDialogBreakQuery(TRUE);
 	ProgressDialogSetTransientFor(NULL);
 
-	/* Play completed sound on success */
+	/* Play the "completed" sound on success */
 	if(status == 0) 
 	    EDVPlaySoundCompleted(core);
 
-	EDVRecBinSetBusy(recbin, FALSE);
-
 	/* Update the status bar message */
 	if(TRUE)
 	{
-	    gchar *s;
+	    gchar *msg;
 	    if(status == -4)
-		s = STRDUP(
+		msg = STRDUP(
 		    "Purge operation canceled"
 		);
 	    else if(nobjs_purged > 0)
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Purged %i %s",
 		    nobjs_purged,
 		    (nobjs_purged == 1) ? "recycled object" : "recycled objects"
 		);
 	    else
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Unable to purge %s",
 		    (nobjs == 1) ? "recycled object" : "recycled objects"
 		);
-	    EDVStatusBarMessage(recbin->status_bar, s, FALSE);
-	    g_free(s);
+	    EDVStatusBarMessage(recbin->status_bar, msg, FALSE);
+	    g_free(msg);
 	}
 
-	DO_FREE_LOCALS
-#undef DO_FREE_LOCALS
+	EDVRecBinSetBusy(recbin, FALSE);
+
+	CLEANUP_RETURN;
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -1116,6 +1103,7 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	const gchar *error_msg, *recycled_index_file;
 	GList *glist, *index_list = NULL;
 	GtkWidget *toplevel;
+	cfg_item_struct *cfg_list;
 	edv_recbin_index_struct *rbi_ptr;
 	edv_core_struct *core;
 
@@ -1124,46 +1112,50 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 
 	toplevel = recbin->toplevel;
 	core = recbin->core;
-	if(core == NULL)
-	    return;
+	cfg_list = core->cfg_list;
 
 	/* Check and warn if write protect is enabled */
 	if(EDVCheckWriteProtect(core, TRUE, toplevel))
 	    return;
 
-	/* Get path to recycled objects index file */
-	recycled_index_file = CFGItemListGetValueS(
-	    core->cfg_list, EDV_CFG_PARM_FILE_RECYCLE_BIN_INDEX
+	/* Get the path to recycled objects index file */
+	recycled_index_file = EDV_GET_S(
+	    EDV_CFG_PARM_FILE_RECYCLE_BIN_INDEX
 	);
 	if(recycled_index_file == NULL)
 	    return;
 
-#define DO_FREE_LOCALS	{	\
+#define CLEANUP_RETURN	{	\
  g_list_free(index_list);	\
+				\
+ return;			\
 }
 
 	/* Get a list of all recycled object indices */
 	rbi_ptr = EDVRecBinIndexOpen(recycled_index_file);
-	while(!EDVRecBinIndexNext(rbi_ptr))
+	if(rbi_ptr != NULL)
 	{
-	    if(rbi_ptr->index != 0)
-		index_list = g_list_append(
-		    index_list,
-		    (gpointer)rbi_ptr->index
-		);
+	    while(!EDVRecBinIndexNext(rbi_ptr))
+	    {
+		if(rbi_ptr->index != 0)
+		    index_list = g_list_append(
+			index_list,
+			(gpointer)rbi_ptr->index
+		    );
+	    }
+	    EDVRecBinIndexClose(rbi_ptr);
 	}
-	EDVRecBinIndexClose(rbi_ptr);
-	rbi_ptr = NULL;
 
 	/* No recycled objects to purge? */
 	if(index_list == NULL)
 	{
-	    DO_FREE_LOCALS
-	    return;
+	    CLEANUP_RETURN;
 	}
 
 	nobjs = g_list_length(index_list);
 
+	EDVRecBinSetBusy(recbin, TRUE);
+
 	/* Confirm purge */
 	status = EDVConfirmPurge(
 	    core, toplevel,
@@ -1177,14 +1169,11 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	    break;
 
 	  default:
-	    DO_FREE_LOCALS
-	    return;
+	    EDVRecBinSetBusy(recbin, FALSE);
+	    CLEANUP_RETURN;
 	    break;
 	}
 
-
-	EDVRecBinSetBusy(recbin, TRUE);
-
 #ifdef EDV_RECBIN_USE_PURGE_ALL
 /* Do not define this, this has issues about progress updating,
  * read subsequent comments in this block
@@ -1197,14 +1186,13 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	    TRUE, TRUE, &yes_to_all
 	);
 
-	/* Get the error message describing the error that might have
-	 * occured in the above operation
-	 */
-	error_msg = EDVRecBinPurgeGetError();
+	/* Check for errors */
+	error_msg = EDVRecBinPurgeGetError(core);
 	if(!STRISEMPTY(error_msg))
 	{
+	    /* Report the error */
 	    EDVPlaySoundError(core);
-	    EDVMessageObjectOPError(
+	    EDVMessageError(
 		"Purge Recycled Object Error",
 		error_msg,
 		NULL,
@@ -1219,7 +1207,10 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	     * resources for each recycled object listed in our recorded
 	     * recycled objects index array
 	     */
-	    for(glist = index_list; glist != NULL; glist = g_list_next(glist))
+	    for(glist = index_list;
+		glist != NULL;
+		glist = g_list_next(glist)
+	    )
 	    {
 		index = (guint)glist->data;
 
@@ -1252,7 +1243,10 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	 * one
 	 */
 	status = 0;
-	for(glist = index_list; glist != NULL; glist = g_list_next(glist))
+	for(glist = index_list;
+	    glist != NULL;
+	    glist = g_list_next(glist)
+	)
 	{
 	    index = (guint)glist->data;
 
@@ -1266,14 +1260,13 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 		TRUE, TRUE, &yes_to_all
 	    );
 
-	    /* Get the error message describing the error that might
-	     * have occured in the above operation
-	     */
-	    error_msg = EDVRecBinPurgeGetError();
+	    /* Check for errors */
+	    error_msg = EDVRecBinPurgeGetError(core);
 	    if(!STRISEMPTY(error_msg))
 	    {
+		/* Report the error */
 		EDVPlaySoundError(core);
-		EDVMessageObjectOPError(
+		EDVMessageError(
 		    "Purge Recycled Object Error",
 		    error_msg,
 		    NULL,
@@ -1305,43 +1298,41 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
 	    );
 #endif
 
-	/* Unmap the progress dialog, it may have been mapped during
-	 * the above operation
-	 */
+	/* Unmap the progress dialog */
 	ProgressDialogBreakQuery(TRUE);
 	ProgressDialogSetTransientFor(NULL);
 
-	/* Play completed sound on success */
+	/* Play the "completed" sound on success */
 	if(status == 0)
 	    EDVPlaySoundCompleted(core);
 
-	EDVRecBinSetBusy(recbin, FALSE);
-
 	/* Update the status bar message */
 	if(TRUE)
 	{
-	    gchar *s;
+	    gchar *msg;
 	    if(status == -4)
-		s = STRDUP(
+		msg = STRDUP(
 		    "Purge operation canceled"
 		);
 	    else if(nobjs_purged > 0)
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Purged %i %s",
 		    nobjs_purged,
 		    (nobjs_purged == 1) ? "recycled object" : "recycled objects"
 		);
 	    else
-		s = g_strdup_printf(
+		msg = g_strdup_printf(
 		    "Unable to purge %s",
 		    (nobjs == 1) ? "recycled object" : "recycled objects"
 		);
-	    EDVStatusBarMessage(recbin->status_bar, s, FALSE);
-	    g_free(s);
+	    EDVStatusBarMessage(recbin->status_bar, msg, FALSE);
+	    g_free(msg);
 	}
 
-	DO_FREE_LOCALS
-#undef DO_FREE_LOCALS
+	EDVRecBinSetBusy(recbin, FALSE);
+
+	CLEANUP_RETURN;
+#undef CLEANUP_RETURN
 }
 
 /*
@@ -1349,21 +1340,14 @@ void EDVRecBinOPPurgeAll(edv_recbin_struct *recbin)
  */
 void EDVRecBinOPSelectAll(edv_recbin_struct *recbin)
 {
-	edv_core_struct *core;
 	GtkCList *clist;
-
+	edv_core_struct *core;
 
 	if(recbin == NULL)
 	    return;
 
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
-	/* Get contents clist */
-	clist = (GtkCList *)recbin->contents_clist;
-	if(clist == NULL)
-	    return;
 
 	EDVRecBinSetBusy(recbin, TRUE);
 
@@ -1387,21 +1371,14 @@ void EDVRecBinOPSelectAll(edv_recbin_struct *recbin)
  */
 void EDVRecBinOPUnselectAll(edv_recbin_struct *recbin)
 {
-	edv_core_struct *core;
 	GtkCList *clist;
-
+	edv_core_struct *core;
 
 	if(recbin == NULL)
 	    return;
 
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
-	/* Get contents clist */
-	clist = (GtkCList *)recbin->contents_clist;
-	if(clist == NULL)
-	    return;
 
 	EDVRecBinSetBusy(recbin, TRUE);
 
@@ -1425,23 +1402,16 @@ void EDVRecBinOPUnselectAll(edv_recbin_struct *recbin)
  */
 void EDVRecBinOPInvertSelection(edv_recbin_struct *recbin)
 {
-	edv_core_struct *core;
-	GtkCList *clist;
-	GList *glist, *selection;
 	gint row, total_rows;
-
+	GList *glist, *selection;
+	GtkCList *clist;
+	edv_core_struct *core;
 
 	if(recbin == NULL)
 	    return;
 
+	clist = GTK_CLIST(recbin->contents_clist);
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
-	/* Get contents clist */
-	clist = (GtkCList *)recbin->contents_clist;
-	if(clist == NULL)
-	    return;
 
 	EDVRecBinSetBusy(recbin, TRUE);
 	gtk_clist_freeze(clist);
@@ -1485,9 +1455,10 @@ void EDVRecBinOPInvertSelection(edv_recbin_struct *recbin)
  */
 void EDVRecBinOPRefresh(edv_recbin_struct *recbin)
 {
-	GtkWidget *w, *toplevel;
+	GtkWidget *toplevel;
 	GtkCList *clist;
 	const cfg_item_struct *cfg_list;
+	edv_status_bar_struct *sb;
 	edv_core_struct *core;
 
 	if(recbin == NULL)
@@ -1495,30 +1466,23 @@ void EDVRecBinOPRefresh(edv_recbin_struct *recbin)
 
 	toplevel = recbin->toplevel;
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
+	sb = recbin->status_bar;
 
 	EDVRecBinSetBusy(recbin, TRUE);
 	GUIBlockInput(toplevel, TRUE);
 
-	/* Refresh toplevel */
-	w = recbin->toplevel;
-	if(w != NULL)
-	    gtk_widget_queue_resize(w);
-
-	/* Update contents clist */
-	clist = (GtkCList *)recbin->contents_clist;
+	/* Update the Contents GtkCList */
+	clist = GTK_CLIST(recbin->contents_clist);
 	if(clist != NULL)
 	{
-	    /* Record last scroll position */
-	    gfloat last_x = GTK_ADJUSTMENT_GET_VALUE(clist->hadjustment),
-		   last_y = GTK_ADJUSTMENT_GET_VALUE(clist->vadjustment);
+	    /* Record the last scroll position */
+	    const gfloat	last_x = GTK_ADJUSTMENT_GET_VALUE(clist->hadjustment),
+				last_y = GTK_ADJUSTMENT_GET_VALUE(clist->vadjustment);
 
 	    gtk_clist_freeze(clist);
 
-	    /* Refresh list items */
+	    /* Reget the listing */
 	    EDVRecBinContentsGetListing(
 		recbin,
 		EDV_GET_B(EDV_CFG_PARM_LISTS_ANIMATE_UPDATES)
@@ -1526,14 +1490,12 @@ void EDVRecBinOPRefresh(edv_recbin_struct *recbin)
 
 	    gtk_clist_thaw(clist);
 
-	    /* Scroll back to original position */
+	    /* Scroll back to the original position */
 	    EDVScrollCListToPosition(clist, last_x, last_y);
 	}
 
 	EDVRecBinUpdateMenus(recbin);
-	EDVStatusBarMessage(
-	    recbin->status_bar, "Refreshed contents listing", FALSE
-	);
+	EDVStatusBarMessage(sb, "Refreshed contents listing", FALSE);
 
 	GUIBlockInput(toplevel, FALSE);
 	EDVRecBinSetBusy(recbin, FALSE);
@@ -1552,26 +1514,22 @@ void EDVRecBinOPRefreshAll(edv_recbin_struct *recbin)
 	    return;
 
 	core = recbin->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
-	/* Get path of recycled object index file */
+	/* Get the path to the recycled objects index file */
 	recycled_index_file = CFGItemListGetValueS(
 	    cfg_list, EDV_CFG_PARM_FILE_RECYCLE_BIN_INDEX
 	);
 
-	/* Update number of recycled objects on the core structure */
+	/* Update the number of recycled objects on the core */
 	core->last_recbin_items = EDVRecBinIndexGetTotal(
 	    recycled_index_file
 	);
 
-	/* Refresh Recycle Bin Desktop Icon */
+	/* Refresh the Recycle Bin Desktop Icon */
 	EDVRecBinDeskIconUpdate(core->recbin_deskicon);
 
-
-	/* Refresh Recycle Bin */
+	/* Refresh the Recycle Bin */
 	EDVRecBinOPRefresh(recbin);
 }
 
