@@ -33,27 +33,15 @@
 static gint EDVRunDlgDeleteEventCB(
 	GtkWidget *widget, GdkEvent *event, gpointer data
 );
-static void EDVRunDlgRunCB(
-	GtkWidget *widget, gpointer data
-);
+static void EDVRunDlgRunCB(GtkWidget *widget, gpointer data);
 static gint EDVRunDlgCommandFocusEventCB(
 	 GtkWidget *widget, GdkEventFocus *focus, gpointer data
 );
-static void EDVRunDlgBrowseCommandCB(
-	GtkWidget *widget, gpointer data
-);
-static void EDVRunDlgBrowseWorkingDirCB(
-	GtkWidget *widget, gpointer data
-);
-static void EDVRunDlgClearCB(
-	GtkWidget *widget, gpointer data
-);
-static void EDVRunDlgGetWorkingDirFromCmdCB(
-	GtkWidget *widget, gpointer data
-);
-static void EDVRunDlgOptionsCB(
-	GtkWidget *widget, gpointer data
-);
+static void EDVRunDlgBrowseCommandCB(GtkWidget *widget, gpointer data);
+static void EDVRunDlgBrowseWorkingDirCB(GtkWidget *widget, gpointer data);
+static void EDVRunDlgClearCB(GtkWidget *widget, gpointer data);
+static void EDVRunDlgGetWorkingDirFromCmdCB(GtkWidget *widget, gpointer data);
+static void EDVRunDlgOptionsCB(GtkWidget *widget, gpointer data);
 static void EDVRunDlgCloseCB(GtkWidget *widget, gpointer data);
 
 /* Utilities */
@@ -68,6 +56,12 @@ void EDVRunDlgSetWorkingDir(
 );
 void EDVRunDlgFetchValues(edv_run_dlg_struct *d);
 
+/* Operations */
+void EDVRunDlgRun(
+	edv_run_dlg_struct *d,
+	const gboolean record_cmd
+);
+
 /* Run Dialog */
 edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core);
 void EDVRunDlgSetBusy(edv_run_dlg_struct *d, gboolean is_busy);
@@ -130,233 +124,16 @@ static gint EDVRunDlgDeleteEventCB(
 /*
  *      Run Dialog run callback.
  */
-static void EDVRunDlgRunCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgRunCB(GtkWidget *widget, gpointer data)
 {
-	gint euid;
-	gchar *s, *pwd, *prog, *args, *cmd;
-	const gchar *cmd_src, *wd;
-	struct stat stat_buf;
-	const cfg_item_struct *cfg_list;
-	GtkWidget *toplevel;
-	edv_core_struct *core;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(data);
 	if(d == NULL)
 	    return;
 
-	toplevel = d->toplevel;
-	core = d->core;
-	cfg_list = core->cfg_list;
-	euid = core->effective_user_id;
-
-	EDVRunDlgSetBusy(d, TRUE);
-
-	/* Complete command as needed */
-	EDVRunDlgCompleteCommand(d);
-
-	/* Get new and previous working directories */
-	wd = gtk_entry_get_text(
-	    GTK_ENTRY(d->working_dir_entry)
-	);
-	pwd = EDVGetCWD();
-
-	/* Get command */
-	cmd_src = gtk_entry_get_text(
-	    GTK_ENTRY(GTK_COMBO(d->run_combo)->entry)
-	);
-	if(STRISEMPTY(cmd_src))
-	{
-	    g_free(pwd);
-	    EDVRunDlgSetBusy(d, FALSE);
+	if(d->freeze_count > 0)
 	    return;
-	}
 
-	/* Seek past initial spaces */
-	while(ISBLANK(*cmd_src))
-	    cmd_src++;
-
-	/* Get program from command */
-	prog = STRDUP(cmd_src);
-	s = strpbrk(prog, " \t");
-	if(s != NULL)
-	    *s = '\0';
-
-	/* Get arguments from command */
-	s = strpbrk(cmd_src, " \t");
-	if(s != NULL)
-	{
-	    while(ISBLANK(*s))
-		s++;
-	    args = STRDUP(s);
-	}
-	else
-	{
-	    args = NULL;
-	}
-
-	/* Record this command */
-	if(args != NULL)
-	    s = g_strdup_printf("%s %s", prog, args);
-	else
-	    s = g_strdup_printf("%s", prog);
-	GUIComboAddItem(d->run_combo, s);
-	g_free(s);
-
-	/* Get program stats to see if it exists and is executable */
-	if(stat(prog, &stat_buf))
-	{
-	    const gint error_code = (gint)errno;
-	    gchar *error_msg = STRDUP(g_strerror(error_code));
-	    if(error_msg != NULL)
-	    {
-		*error_msg = toupper(*error_msg);
-
-		s = g_strdup_printf(
-"%s:\n\
-\n\
-    %s",
-		    error_msg, prog
-		);
-		EDVPlaySoundWarning(core);
-		EDVMessageWarning(
-		    "Run Failed", s, NULL, toplevel
-		);
-		g_free(s);
-		g_free(error_msg);
-	    }
-	    g_free(pwd);
-	    g_free(prog);
-	    g_free(args);
-	    EDVRunDlgSetBusy(d, FALSE);
-	    return;
-	}
-	else if(((euid == stat_buf.st_uid) || (euid == 0)) ?
-		 !(stat_buf.st_mode & S_IXUSR) :
-		 !(stat_buf.st_mode & S_IXOTH)
-	       )
-	{
-	    gchar *s = g_strdup_printf(
-"Not executable:\n\
-\n\
-    %s\n\
-\n\
-The program must be set executable in order to run it.",
-		prog
-	    );
-	    EDVPlaySoundWarning(core);
-	    EDVMessageWarning(
-		"Run Failed", s, NULL, toplevel
-	    );
-	    g_free(s);
-
-	    g_free(pwd);
-	    g_free(prog);
-	    g_free(args);
-	    EDVRunDlgSetBusy(d, FALSE);
-	    return;
-	}
-
-	/* Format command */
-	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(d->run_in_terminal_check))
-	{
-	    /* Format command for running in a terminal */
-	    const gchar *cmd_terminal_run = EDV_GET_S(
-		EDV_CFG_PARM_PROG_TERMINAL_RUN
-	    );
-	    if(!STRISEMPTY(cmd_terminal_run))
-	    {
-		/* Format command for run in terminal */
-		if(args != NULL)
-		    cmd = g_strdup_printf(
-		        "%s %s %s",
-			cmd_terminal_run, prog, args
-		    );
-		else
-		    cmd = g_strdup_printf(
-			"%s %s",
-			cmd_terminal_run, prog
-		    );
-	    }
-	    else
-	    {
-		EDVPlaySoundWarning(core);
-		EDVMessageWarning(   
-		    "Run Failed",
-"The \"Run In Terminal\" command is not set.\n\
-\n\
-To set the \"Run In Terminal\" command go to\n\
-Settings->Options...->Programs.",
-		    NULL,
-		    toplevel
-		);
-		g_free(pwd);
-		g_free(prog);
-		g_free(args);
-		EDVRunDlgSetBusy(d, FALSE);
-		return;
-	    }
-	}
-	else
-	{
-	    /* Format plain command */
-	    if(args != NULL)
-		cmd = g_strdup_printf(
-		    "%s %s &",
-		    prog, args
-		);
-	    else
-		cmd = g_strdup_printf(
-		    "%s &",
-		    prog
-		);
-	}
-
-	/* Generated command successfully? */
-	if(cmd != NULL)
-	{
-	    gint p;
-	    gchar *s;
-
-	    /* Change to the new working directory */
-	    EDVSetCWD(wd);
-
-	    /* Execute command */
-	    p = EDVSystem(cmd);
-
-	    /* Change back to the previous working directory */
-	    EDVSetCWD(pwd);
-
-	    /* Error executing the command? */
-	    if(p <= 0)
-	    {
-		s = g_strdup_printf(
-"Unable to execute command:\n\
-\n\
-    %s",
-		    cmd
-		);
-		EDVPlaySoundError(core);
-		EDVMessageError(
-"Run Failed",
-		    s,
-"Please verify that the program exists, is executable, and any\n\
-specified arguments are valid.",
-		    toplevel
-		);
-		g_free(s);
-	    }
-	}
-
-	/* Do not keep dialog after running? */
-	if(!GTK_TOGGLE_BUTTON_GET_ACTIVE(d->keep_dialog_check))
-	    EDVRunDlgCloseCB(d->close_btn, d);
-
-	g_free(pwd);
-	g_free(cmd);
-	g_free(prog);
-	g_free(args);
-	EDVRunDlgSetBusy(d, FALSE);
+	EDVRunDlgRun(d, TRUE);
 }
 
 
@@ -391,70 +168,121 @@ static gint EDVRunDlgCommandFocusEventCB(
 /*
  *	Run Dialog browse command callback.
  */
-static void EDVRunDlgBrowseCommandCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgBrowseCommandCB(GtkWidget *widget, gpointer data)
 {
-	gboolean status;
-	gchar *s, *cur_prog;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
-	GtkWidget *toplevel, *w;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	const gchar *s;
+	gchar *prog, *args, **paths_list = NULL;
+	GtkWidget *toplevel;
 	GtkEntry *entry;
+	GtkCombo *combo;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(data);
 	if((d == NULL) || FileBrowserIsQuery())
 	    return;
 
-	toplevel = d->toplevel;
-	w = d->run_combo;
-	if(w == NULL)
+	if(d->freeze_count > 0)
 	    return;
-	entry = GTK_ENTRY(GTK_COMBO(w)->entry);
 
-	cur_prog = STRDUP(gtk_entry_get_text(entry));
-	s = strpbrk(cur_prog, " \t");
+	toplevel = d->toplevel;
+	combo = GTK_COMBO(d->run_combo);
+	entry = GTK_ENTRY(combo->entry);
+
+	/* Get the program from the current command */
+	s = gtk_entry_get_text(entry);
 	if(s != NULL)
-	    *s = '\0';
-
+	{
+	    const gchar *s_end = (const gchar *)strpbrk(
+		(const char *)s, " \t"
+	    );
+	    if(s_end != NULL)
+	    {
+		/* Get the program */
+		const gint len = s_end - s;
+		prog = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+		if(prog != NULL)
+		{
+		    if(len > 0)
+			memcpy(prog, s, len * sizeof(gchar));
+		    prog[len] = '\0';
+		    prog = g_strstrip(prog);
+		}
+		/* Get the arguments */
+		args = STRDUP(s_end);
+		args = g_strstrip(args);
+	    }
+	    else
+	    {
+		prog = STRDUP(s);
+		prog = g_strstrip(prog);
+		args = NULL;
+	    }
+	}
+	else
+	{
+	    prog = NULL;
+	    args = NULL;
+	}
 
 	EDVRunDlgSetBusy(d, TRUE);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
-	/* Query user for program */
+	/* Query the user for a program */
+	d->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 	    "Select Program",
 	    "Select", "Cancel",
-	    cur_prog,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    prog,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	d->freeze_count--;
 
-	/* Got user response? */
-	if(status)
+	/* Got response? */
+	if(response)
 	{
-	    const gchar *new_prog = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *new_prog = (npaths > 0) ?
+		paths_list[0] : NULL;
 	    if(!STRISEMPTY(new_prog))
 	    {
-		gtk_entry_set_text(entry, new_prog);
-		gtk_entry_set_position(entry, -1);
+		gchar *cmd_new;
+
+		if(args != NULL)
+		    cmd_new = g_strconcat(
+			new_prog,
+			" ",
+			args,
+			NULL
+		    );
+		else
+		    cmd_new = STRDUP(new_prog);
+
+		if(cmd_new != NULL)
+		{
+		    cmd_new = g_strstrip(cmd_new);
+
+		    gtk_entry_set_text(entry, cmd_new);
+		    gtk_entry_set_position(entry, -1);
+
+		    g_free(cmd_new);
+		}
 	    }
 	}
 
 	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
-	g_free(cur_prog);
+	g_free(prog);
+	g_free(args);
 
 	EDVRunDlgSetBusy(d, FALSE);
 }
@@ -462,56 +290,52 @@ static void EDVRunDlgBrowseCommandCB(
 /*
  *	Run Dialog browse working directory callback.
  */
-static void EDVRunDlgBrowseWorkingDirCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgBrowseWorkingDirCB(GtkWidget *widget, gpointer data)
 {
-	gboolean status;
-	gchar *cur_dir;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
-	GtkWidget *toplevel, *w;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	gchar *cur_dir, **paths_list = NULL;
+	GtkWidget *toplevel;
 	GtkEntry *entry;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(data);
 	if((d == NULL) || FileBrowserIsQuery())
 	    return;
 
-	toplevel = d->toplevel;
-	w = d->working_dir_entry;
-	if(w == NULL)
+	if(d->freeze_count > 0)
 	    return;
-	entry = GTK_ENTRY(w);
 
+	toplevel = d->toplevel;
+	entry = GTK_ENTRY(d->working_dir_entry);
 	cur_dir = STRDUP(gtk_entry_get_text(entry));
 
-
 	EDVRunDlgSetBusy(d, TRUE);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
-	/* Query user for directory */
+	/* Query the user for a directory */
+	d->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 	    "Select Directory",
 	    "Select", "Cancel",
 	    cur_dir,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	d->freeze_count--;
 
-	/* Got user response? */
-	if(status)
+	/* Got response? */
+	if(response)
 	{
-	    const gchar *new_dir = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *new_dir = (npaths > 0) ?
+		paths_list[0] : NULL;
 	    if(!STRISEMPTY(new_dir))
 	    {
 		gtk_entry_set_text(entry, new_dir);
@@ -519,8 +343,8 @@ static void EDVRunDlgBrowseWorkingDirCB(
 	    }
 	}
 
-	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	/* Delete the file types list */
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	g_free(cur_dir);
 
@@ -532,9 +356,7 @@ static void EDVRunDlgBrowseWorkingDirCB(
  *
  *      The data must be a GtkEntry.
  */
-static void EDVRunDlgClearCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgClearCB(GtkWidget *widget, gpointer data)
 {
 	GtkWidget *w = (GtkWidget *)data;
 	GtkEntry *entry;
@@ -556,32 +378,24 @@ static void EDVRunDlgClearCB(
  *	Run dialog get working directory from current run command value
  *	callback.
  */
-static void EDVRunDlgGetWorkingDirFromCmdCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgGetWorkingDirFromCmdCB(GtkWidget *widget, gpointer data)
 {
 	const gchar *cmd;
-	GtkWidget *w;
 	GtkEntry *entry;
 	GtkCombo *combo;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(data);
 	if(d == NULL)
 	    return;
 
-	w = d->run_combo;
-	if(w != NULL)
-	    combo = GTK_COMBO(w);
-	else
-	    return;
-
-	w = d->working_dir_entry;
-	if(w != NULL)
-	    entry = GTK_ENTRY(w);
-	else
+	if(d->freeze_count > 0)
 	    return;
 
 	EDVRunDlgSetBusy(d, TRUE);
 
+	combo = GTK_COMBO(d->run_combo);
+	entry = GTK_ENTRY(d->working_dir_entry);
+
+	/* Get the current command */
 	cmd = gtk_entry_get_text(GTK_ENTRY(combo->entry));
 	if(cmd != NULL)
 	{
@@ -619,19 +433,20 @@ static void EDVRunDlgGetWorkingDirFromCmdCB(
 /*
  *	Options button callback.
  */
-static void EDVRunDlgOptionsCB(
-	GtkWidget *widget, gpointer data
-)
+static void EDVRunDlgOptionsCB(GtkWidget *widget, gpointer data)
 {
 	edv_core_struct *core;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(data);
 	if(d == NULL)
 	    return;
 
-	core = d->core;
+	if(d->freeze_count > 0)
+	    return;
 
 	EDVRunDlgSetBusy(d, TRUE);
 
+	core = d->core;
+
 	EDVMapOptionsWinPage(
 	    core,
 	    "Programs",		/* Page Name */
@@ -655,8 +470,11 @@ static void EDVRunDlgCloseCB(
 	if(d == NULL)
 	    return;
 
+	if(d->freeze_count > 0)
+	    return;
+
 	core = d->core;
-	cfg_list = (core != NULL) ? core->cfg_list : NULL;
+	cfg_list = core->cfg_list;
 
 	/* Begin recording current widget values to global
 	 * configuration
@@ -665,12 +483,10 @@ static void EDVRunDlgCloseCB(
 	/* Run history */
 	w = d->run_combo;
 	if(w != NULL)
-	{
 	    EDVSaveFileGList(
 		EDV_GET_S(EDV_CFG_PARM_FILE_RUNDLG_HISTORY),
 		GUIComboGetList(w)
 	    );
-	}
 
 	/* Toplevel geometry */
 	w = d->toplevel;
@@ -716,7 +532,7 @@ static void EDVRunDlgCloseCB(
 	    );
 
 
-	/* Now close dialog */
+	/* Close the dialog */
 	EDVRunDlgUnmap(d);
 }
 
@@ -732,54 +548,74 @@ static void EDVRunDlgCompleteCommand(edv_run_dlg_struct *d)
 	if(d == NULL)
 	    return;
 
-	/* Get current command */
+	/* Get the current command */
 	cmd_src = gtk_entry_get_text(
 	    GTK_ENTRY(GTK_COMBO(d->run_combo)->entry)
 	);   
 	if(!STRISEMPTY(cmd_src))
 	{
-	    gchar *s, *prog, *args;
+	    gchar *prog, *args;
+	    const gchar *s;
 
 	    /* Seek past spaces in command */
 	    while(ISBLANK(*cmd_src))
 		cmd_src++;
 
-	    /* Get program from command */
-	    prog = STRDUP(cmd_src);
-	    s = strpbrk(prog, " \t");
-	    if(s != NULL)
-		*s = '\0';   
-
-	    /* Get arguments from command */
-	    s = strpbrk(cmd_src, " \t");
+	    /* Get the program and arguments from the command */
+	    s = (const gchar *)strpbrk((const char *)cmd_src, " \t");
 	    if(s != NULL)
 	    {
-		while(ISBLANK(*s))
-		    s++;
+		const gint len = s - cmd_src;
+		prog = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+		if(prog != NULL)
+		{
+		    if(len > 0)
+			memcpy(prog, cmd_src, len * sizeof(gchar));
+		    prog[len] = '\0';
+		}
+
 		args = STRDUP(s);
+		args = g_strstrip(args);
 	    }
 	    else
 	    {
+		prog = STRDUP(cmd_src);
+		prog = g_strstrip(prog);
 		args = NULL;
 	    }
 
 	    /* If the program location is not an absolute path then
 	     * we need to complete it
 	     */
-	    s = g_path_is_absolute(prog) ? NULL : EDVWhich(prog);
-	    if(s != NULL)
+	    if(!g_path_is_absolute(prog))
 	    {
-		GtkCombo *combo = GTK_COMBO(d->run_combo);
+		gchar *prog_full_path = EDVWhich(prog);
+		if(prog_full_path != NULL)
+		{
+		    gchar *cmd_new;
+		    GtkCombo *combo = GTK_COMBO(d->run_combo);
 
-		/* Set new program with completed path */
-		g_free(prog);
-		prog = s;
+		    /* Set the new program with the completed path */
+		    g_free(prog);
+		    prog = prog_full_path;
 
-		/* Format and set new command */
-		s = (args != NULL) ?
-		    g_strconcat(prog, " ", args, NULL) : g_strdup(prog);
-		gtk_entry_set_text(GTK_ENTRY(combo->entry), s);
-		g_free(s);
+		    /* Format and set the new command */
+		    if(args != NULL)
+			cmd_new = g_strconcat(
+			    prog,
+			    " ",
+			    args,
+			    NULL
+			);
+		    else
+			cmd_new = STRDUP(prog);
+
+		    if(cmd_new != NULL)
+		    {
+			gtk_entry_set_text(GTK_ENTRY(combo->entry), cmd_new);
+			g_free(cmd_new);
+		    }
+		}
 	    }
 
 	    g_free(prog);
@@ -801,10 +637,7 @@ void EDVRunDlgSetCommand(
 	if(d == NULL)
 	    return;
 
-	combo = (GtkCombo *)d->run_combo;
-	if(combo == NULL)
-	    return;
-
+	combo = GTK_COMBO(d->run_combo);
 	entry = GTK_ENTRY(combo->entry);
 	gtk_entry_set_text(entry, (command != NULL) ? command : "");
 	gtk_entry_set_position(entry, -1);
@@ -822,10 +655,7 @@ void EDVRunDlgSetWorkingDir(
 	if(d == NULL)
 	    return;
 
-	entry = (GtkEntry *)d->working_dir_entry;
-	if(entry == NULL)
-	    return;
-
+	entry = GTK_ENTRY(d->working_dir_entry);
 	gtk_entry_set_text(entry, (working_dir != NULL) ? working_dir : "");
 	gtk_entry_set_position(entry, -1);
 }
@@ -844,9 +674,6 @@ void EDVRunDlgFetchValues(edv_run_dlg_struct *d)
 	    return;
 
 	core = d->core;
-	if(core == NULL)
-	    return;
-
 	cfg_list = core->cfg_list;
 
 	/* Begin fetching values from global configuration and
@@ -854,7 +681,7 @@ void EDVRunDlgFetchValues(edv_run_dlg_struct *d)
 	 */
 
 	/* Run History */
-	combo = (GtkCombo *)d->run_combo;
+	combo = GTK_COMBO(d->run_combo);
 	if(combo != NULL)
 	{
 	    const gchar *path = EDV_GET_S(EDV_CFG_PARM_FILE_RUNDLG_HISTORY);
@@ -884,6 +711,333 @@ void EDVRunDlgFetchValues(edv_run_dlg_struct *d)
 }
 
 
+/*
+ *	Run.
+ *
+ *	If record_cmd is TRUE then the command will be added to the
+ *	run GtkCombo's list.
+ */
+void EDVRunDlgRun(
+	edv_run_dlg_struct *d,
+	const gboolean record_cmd
+)
+{
+	gint euid;
+	gchar *cmd, *cmd_src, *wd;
+	const cfg_item_struct *cfg_list;
+	GtkWidget *toplevel;
+	edv_core_struct *core;
+
+	if(d == NULL)
+	    return;
+
+	toplevel = d->toplevel;
+	core = d->core;
+	cfg_list = core->cfg_list;
+	euid = core->effective_user_id;
+
+	EDVRunDlgSetBusy(d, TRUE);
+
+	/* Complete the command as needed */
+	EDVRunDlgCompleteCommand(d);
+
+	/* Get the working directory */
+	wd = STRDUP(gtk_entry_get_text(GTK_ENTRY(d->working_dir_entry)));
+
+	/* Get the command */
+	cmd_src = STRDUP(gtk_entry_get_text(
+	    GTK_ENTRY(GTK_COMBO(d->run_combo)->entry)
+	));
+	if(STRISEMPTY(cmd_src))
+	{
+	    g_free(wd);
+	    g_free(cmd_src);
+	    EDVRunDlgSetBusy(d, FALSE);
+	    return;
+	}
+
+	/* Record this command in the command GtkCombo's list */
+	if(record_cmd)
+	{
+	    GUIComboAddItem(d->run_combo, cmd_src);
+	}
+
+	/* Check if the program is executable */
+	if(cmd_src != NULL)
+	{
+	    /* Get the program path from the command */
+	    gchar *prog_path;
+	    const gchar	*s = cmd_src,
+			*s_end = (const gchar *)strpbrk(
+		(const char *)s, " \t"
+	    );
+	    if(s_end != NULL)
+	    {
+		const gint len = s_end - s;
+		prog_path = (gchar *)g_malloc((len + 1) * sizeof(gchar));
+		if(prog_path != NULL)
+		{
+		    if(len > 0)
+			memcpy(prog_path, s, len * sizeof(gchar));
+		    prog_path[len] = '\0';
+		    prog_path = g_strstrip(prog_path);
+		}
+	    }
+	    else
+	    {
+		prog_path = STRDUP(s);
+		prog_path = g_strstrip(prog_path);
+	    }
+
+	    /* Got the program? */
+	    if(prog_path != NULL)
+	    {
+		mode_t m;
+		struct stat stat_buf;
+
+		/* Not an absolute path? */
+		if(!g_path_is_absolute(prog_path))
+		{
+		    EDVPlaySoundWarning(core);
+		    EDVMessageWarning(
+			"Run Failed",
+"The path to the program must be an absolute path.",
+			NULL,
+			toplevel
+		    );
+		    g_free(prog_path);
+		    g_free(wd);
+		    g_free(cmd_src);
+		    EDVRunDlgSetBusy(d, FALSE);
+		    return;
+		}
+		/* Does not exist? */
+		if(stat((const char *)prog_path, &stat_buf))
+		{
+		    const gint error_code = (gint)errno;
+		    gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+			g_strerror(error_code),
+			prog_path
+		    );
+		    EDVPlaySoundWarning(core);
+		    EDVMessageWarning(
+			"Run Failed",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
+		    g_free(prog_path);
+		    g_free(wd);
+		    g_free(cmd_src);
+		    EDVRunDlgSetBusy(d, FALSE);
+		    return;
+		}
+		/* Not executable? */
+		m = stat_buf.st_mode;
+#if !defined(_WIN32)
+		if(!(m & S_IXUSR) && !(m & S_IXGRP) && !(m & S_IXOTH))
+		{
+		    gchar *msg = g_strdup_printf(
+"Not executable:\n\
+\n\
+    %s",
+			prog_path
+		    );
+		    EDVPlaySoundWarning(core);
+		    EDVMessageWarning(
+			"Run Failed",
+			msg,
+			NULL,
+			toplevel
+		    );
+		    g_free(msg);
+		    g_free(prog_path);
+		    g_free(wd);
+		    g_free(cmd_src);
+		    EDVRunDlgSetBusy(d, FALSE);
+		    return;
+		}
+#endif	/* !_WIN32 */
+		g_free(prog_path);
+	    }
+	}
+
+	/* Check if the working directory is valid */
+	if(wd != NULL)
+	{
+	    mode_t m;
+	    struct stat stat_buf;
+
+	    /* Not an absolute path? */
+	    if(!g_path_is_absolute(wd))
+	    {
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(
+		    "Run Failed",
+"The working directory path must be an absolute path.",
+		    NULL,
+		    toplevel
+		);
+		g_free(wd);
+		g_free(cmd_src);
+		EDVRunDlgSetBusy(d, FALSE);
+		return;
+	    }
+	    /* Does not exist? */
+	    if(stat((const char *)wd, &stat_buf))
+	    {
+		const gint error_code = (gint)errno;
+		gchar *msg = g_strdup_printf(
+"%s:\n\
+\n\
+    %s",
+		    g_strerror(error_code),
+		    wd
+		);
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(
+		    "Run Failed",
+		    msg,
+		    NULL,
+		    toplevel
+		);
+		g_free(msg);
+		g_free(wd);
+		g_free(cmd_src);
+		EDVRunDlgSetBusy(d, FALSE);
+		return;
+	    }
+	    /* Not executable? */
+	    m = stat_buf.st_mode;
+#if !defined(_WIN32)
+	    if(!(m & S_IXUSR) && !(m & S_IXGRP) && !(m & S_IXOTH))
+	    {
+		gchar *msg = g_strdup_printf(
+"Not executable:\n\
+\n\
+    %s",
+		    wd
+		);
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(
+		    "Run Failed",
+		    msg,
+		    NULL,
+		    toplevel
+		);
+		g_free(msg);
+		g_free(wd);
+		g_free(cmd_src);
+		EDVRunDlgSetBusy(d, FALSE);
+		return;
+	    }
+#endif	/* !_WIN32 */
+	}
+
+	/* Format the target command */
+	if(GTK_TOGGLE_BUTTON_GET_ACTIVE(d->run_in_terminal_check))
+	{
+	    /* Format the command for running in a terminal */
+	    const gchar *cmd_terminal_run = EDV_GET_S(
+		EDV_CFG_PARM_PROG_TERMINAL_RUN
+	    );
+	    if(!STRISEMPTY(cmd_terminal_run))
+	    {
+		/* Format the command for run in terminal */
+		cmd = g_strconcat(
+		    cmd_terminal_run,
+		    " ",
+		    cmd_src,
+		    NULL
+		);
+	    }
+	    else
+	    {
+		EDVPlaySoundWarning(core);
+		EDVMessageWarning(   
+		    "Run Failed",
+"The \"Run In Terminal\" command is not set.\n\
+\n\
+To set the \"Run In Terminal\" command go to\n\
+Settings->Options...->Programs.",
+		    NULL,
+		    toplevel
+		);
+		g_free(wd);
+		g_free(cmd_src);
+		EDVRunDlgSetBusy(d, FALSE);
+		return;
+	    }
+	}
+	else
+	{
+	    /* Format a plain command */
+	    cmd = g_strconcat(
+		cmd_src,
+		NULL
+	    );
+	}
+
+	/* Generated the target command successfully? */
+	if(cmd != NULL)
+	{
+	    gint p;
+
+	    /* Get the current working directory */
+	    gchar *cwd = EDVGetCWD();
+
+	    /* Change to the new working directory */
+	    EDVSetCWD(wd);
+
+	    /* Execute the command */
+	    p = EDVSystem(cmd);
+	    if(p < 0)
+	    {
+		const gint error_code = (gint)errno;
+		gchar *msg = g_strdup_printf(
+"Unable to execute the command:\n\
+\n\
+    %s\n\
+\n\
+%s.",
+		    cmd, g_strerror(error_code)
+		);
+		EDVPlaySoundError(core);
+		EDVMessageError(
+"Run Failed",
+		    msg,
+"Please check to make sure that the path to the program\n\
+is correct and that the program is set executable.",
+		    toplevel
+		);
+		g_free(msg);
+	    }
+
+	    /* Change back to the previous working directory */
+	    if(cwd != NULL)
+	    {
+		EDVSetCWD(cwd);
+		g_free(cwd);
+	    }
+
+	    g_free(cmd);
+	}
+
+	/* Do not keep dialog after running? */
+	if(!GTK_TOGGLE_BUTTON_GET_ACTIVE(d->keep_dialog_check))
+	    EDVRunDlgCloseCB(d->close_btn, d);
+
+	g_free(wd);
+	g_free(cmd_src);
+
+	EDVRunDlgSetBusy(d, FALSE);
+}
+
+
 /*
  *	Creates a new Run Dialog.
  */
@@ -892,11 +1046,13 @@ edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core)
 	const gint	border_major = 5,
 			border_minor = 2;
 	gint x = 0, y = 0;
-	GdkWindow *window;
-	GtkAccelGroup *accelgrp;
-	GtkStyle *style;
 	gpointer combo_rtn;
-	GtkWidget *w, *parent, *parent2;
+	GdkWindow *window;
+	GtkStyle *style;
+	GtkAccelGroup *accelgrp;
+	GtkWidget	*w,
+			*parent, *parent2,
+			*toplevel;
 	GtkCombo *combo;
 	const cfg_item_struct *cfg_list;
 	edv_run_dlg_struct *d = EDV_RUN_DLG(
@@ -909,22 +1065,21 @@ edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core)
 	}
 
 	cfg_list = core->cfg_list;
-	if(cfg_list != NULL)
-	{
-	    x = EDV_GET_I(EDV_CFG_PARM_RUNDLG_X);
-	    y = EDV_GET_I(EDV_CFG_PARM_RUNDLG_Y);
-	}
+	x = EDV_GET_I(EDV_CFG_PARM_RUNDLG_X);
+	y = EDV_GET_I(EDV_CFG_PARM_RUNDLG_Y);
 
+	d->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	d->accelgrp = accelgrp = gtk_accel_group_new();
 	d->processing = FALSE;
 	d->busy_count = 0;
+	d->freeze_count = 0;
 	d->core = core;
 
+	d->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	d->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	/* Toplevel GtkWindow */
+	w = toplevel;
+	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, TRUE);
 	gtk_widget_set_usize(w, RUNDLG_WIDTH, RUNDLG_HEIGHT);
 	gtk_widget_set_uposition(w, x, y);
 	gtk_window_set_title(GTK_WINDOW(w), RUNDLG_TITLE);
@@ -937,12 +1092,11 @@ edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core)
 	{
 	    gdk_window_set_decorations(
 		window,
-		GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU |
-		GDK_DECOR_MINIMIZE
+		GDK_DECOR_BORDER | GDK_DECOR_TITLE
 	    );
 	    gdk_window_set_functions(
 		window,
-		GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE
+		GDK_FUNC_MOVE | GDK_FUNC_CLOSE
 	    );
 	    GUISetWMIcon(window, (guint8 **)icon_executable_48x48_xpm);
 	}
@@ -955,7 +1109,7 @@ edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core)
 	parent = w;
 
 
-	/* Main vbox */
+	/* Main GtkVBox */
 	d->main_vbox = w = gtk_vbox_new(FALSE, border_major);
 	gtk_container_border_width(GTK_CONTAINER(w), border_major);
 	gtk_container_add(GTK_CONTAINER(parent), w);
@@ -1505,6 +1659,7 @@ edv_run_dlg_struct *EDVRunDlgNew(edv_core_struct *core)
 	gtk_widget_show(w);
 
 
+	d->freeze_count--;
 
 	return(d);
 }
@@ -1518,13 +1673,10 @@ void EDVRunDlgSetBusy(edv_run_dlg_struct *d, gboolean is_busy)
 	GtkWidget *w;
 	edv_core_struct *core;
 
-
 	if(d == NULL)
 	    return;
 
 	core = d->core;
-	if(core == NULL)
-	    return;
 
 	w = d->toplevel;
 	if(w != NULL)
@@ -1568,8 +1720,10 @@ void EDVRunDlgSetBusy(edv_run_dlg_struct *d, gboolean is_busy)
  */
 gboolean EDVRunDlgIsMapped(edv_run_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(d == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(d->toplevel));
 }
 
 /*
@@ -1577,20 +1731,16 @@ gboolean EDVRunDlgIsMapped(edv_run_dlg_struct *d)
  */
 void EDVRunDlgMap(edv_run_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	GtkCombo *combo;
+
+	if(d == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(d->toplevel);
 
-	/* Set the Run Combo as the default widget */
-	w = d->run_combo;
-	if(w != NULL)
-	{
-	    GtkCombo *combo = GTK_COMBO(w);
-	    gtk_widget_grab_focus(combo->entry);
-/*	    gtk_entry_set_position(GTK_ENTRY(combo->entry), 0); */
-	}
+	combo = GTK_COMBO(d->run_combo);
+	gtk_widget_grab_focus(combo->entry);
+/*	gtk_entry_set_position(GTK_ENTRY(combo->entry), 0); */
 }
 
 /*
@@ -1598,11 +1748,10 @@ void EDVRunDlgMap(edv_run_dlg_struct *d)
  */
 void EDVRunDlgUnmap(edv_run_dlg_struct *d)
 {
-	GtkWidget *w = (d != NULL) ? d->toplevel : NULL;
-	if(w == NULL)
+	if(d == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(d->toplevel);
 }
 
 /*
@@ -1613,40 +1762,14 @@ void EDVRunDlgDelete(edv_run_dlg_struct *d)
 	if(d == NULL)
 	    return;
 
-	/* Begin destroying widgets */
-	GTK_WIDGET_DESTROY(d->run_combo)
-	d->run_combo = NULL;
-	GTK_WIDGET_DESTROY(d->browse_btn)
-	d->browse_btn = NULL;
-	GTK_WIDGET_DESTROY(d->clear_btn)
-	d->clear_btn = NULL;
+	EDVRunDlgUnmap(d);
 
-	GTK_WIDGET_DESTROY(d->working_dir_entry)
-	d->working_dir_entry = NULL;
-	GTK_WIDGET_DESTROY(d->working_dir_browse_btn)
-	d->working_dir_browse_btn = NULL;
-	GTK_WIDGET_DESTROY(d->working_dir_clear_btn)
-	d->working_dir_clear_btn = NULL;
-	GTK_WIDGET_DESTROY(d->working_dir_get_from_cmd_btn)
-	d->working_dir_get_from_cmd_btn = NULL;
+	d->freeze_count++;
 
-	GTK_WIDGET_DESTROY(d->run_in_terminal_check)
-	d->run_in_terminal_check = NULL;
-	GTK_WIDGET_DESTROY(d->keep_dialog_check)
-	d->keep_dialog_check = NULL;
-	GTK_WIDGET_DESTROY(d->options_btn)
-	d->options_btn = NULL;
+	gtk_widget_destroy(d->toplevel);
+	gtk_accel_group_unref(d->accelgrp);
 
-	GTK_WIDGET_DESTROY(d->run_btn)
-	d->run_btn = NULL;
-	GTK_WIDGET_DESTROY(d->close_btn)
-	d->close_btn = NULL;
-
-	GTK_WIDGET_DESTROY(d->toplevel)
-	d->toplevel = NULL;
-
-	GTK_ACCEL_GROUP_UNREF(d->accelgrp)
-	d->accelgrp = NULL;
+	d->freeze_count--;
 
 	g_free(d);
 }
