@@ -4,6 +4,12 @@
 #include <gtk/gtk.h>
 
 
+/*
+ *	Widget Name:
+ */
+#define STATIC_TIP_WIDGET_NAME	"gtk-tooltips"
+
+
 /*
  *	Aligns:
  */
