@@ -5,6 +5,7 @@
 #include "../include/disk.h"
 
 #include "guiutils.h"
+#include "pulist.h"
 #include "cdialog.h"
 #include "fb.h"
 #include "csd.h"
@@ -24,42 +25,28 @@ static void StyleEditGdkColorToCSDColor(
 	csd_color_struct *tar_c
 );
 
-static void StyleEditAnyValueChangedCB(
-	GtkWidget *widget, gpointer data
-);
+static void StyleEditAnyValueChangedCB(GtkWidget *widget, gpointer data);
 static void StyleEditStateChangeCB(
-	GtkAdjustment *adj, gpointer data
-);
-static void StyleEditPreviousStateCB(
-	GtkWidget *widget, gpointer data
-);
-static void StyleEditNextStateCB(
-	GtkWidget *widget, gpointer data
-);
-static void StyleEditResetCB(
-	gpointer widget, gpointer data
-);
-static void StyleEditBGPixmapPathCB(
-	GtkWidget *widget, gpointer data
-);
-static gint StyleEditPreviewEventCB(
-	gpointer widget, GdkEvent *event, gpointer data
+	pulistbox_struct *pulistbox, gint i, gpointer data
 );
+static void StyleEditResetCB(gpointer widget, gpointer data);
+static void StyleEditBGPixmapPathCB(GtkWidget *widget, gpointer data);
+static gint StyleEditPreviewEventCB(gpointer widget, GdkEvent *event, gpointer data);
 
 static void StyleEditPreviewUpdate(style_edit_struct *se);
 static void StyleEditPreviewDraw(style_edit_struct *se);
 
 GtkStateType StyleEditCurrentState(style_edit_struct *se);
+void StyleEditSetState(style_edit_struct *se, const GtkStateType state);
 
 style_edit_struct *StyleEditNew(
 	GtkWidget *parent, const gchar *name
 );
-void StyleEditDelete(style_edit_struct *se);
 static void StyleEditSetWidgetsToStyle(
-	style_edit_struct *se, GtkStateType state
+	style_edit_struct *se, const GtkStateType state
 );
 static void StyleEditSetStyleToWidgets(
-	style_edit_struct *se, GtkStateType state
+	style_edit_struct *se, const GtkStateType state
 );
 void StyleEditSetStyle(style_edit_struct *se, const GtkRcStyle *style);
 GtkRcStyle *StyleEditGetStyle(style_edit_struct *se);
@@ -75,18 +62,19 @@ void StyleEditSetChangedCB(
 	),
 	gpointer changed_client_data
 );
+void StyleEditDelete(style_edit_struct *se);
 
 
-#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
-#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
-#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
-#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)
+#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
+#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
+#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
+#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)
 
-#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
-#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
-#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
-#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
-#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)
+#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
+#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
+#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
+#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
+#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)
 
 
 /*
@@ -97,7 +85,7 @@ static void StyleEditCSDColorToGdkColor(
 	GdkColor *tar_c
 )
 {
-	GDK_COLOR_SET_COEFF(tar_c, src_c->r, src_c->g, src_c->b)
+	GDK_COLOR_SET_COEFF(tar_c, src_c->r, src_c->g, src_c->b);
 }
 
 /*
@@ -130,16 +118,16 @@ static void StyleEditAnyValueChangedCB(
 
 	se->freeze_count++;
 
-	/* Update widget values to style */
+	/* Update the widget values to the current style */
 	StyleEditSetWidgetsToStyle(
 	    se,
 	    StyleEditCurrentState(se)
 	);
 
-	/* Update preview style */
+	/* Update the preview style */
 	StyleEditPreviewUpdate(se);
 
-	/* Call changed callback */
+	/* Call the changed callback */
 	if(se->changed_cb != NULL)
 	    se->changed_cb(
 		se,			/* Style Edit */
@@ -150,20 +138,28 @@ static void StyleEditAnyValueChangedCB(
 }
 
 /*
- *      Style Edit state adjustment "value_changed" signal callback.
+ *	State popup list box change callback.
  */
 static void StyleEditStateChangeCB(
-	GtkAdjustment *adj, gpointer data
+	pulistbox_struct *pulistbox, gint i, gpointer data
 )
 {
 	GtkStateType state, prev_state;
-	const gchar *s = "";
-	GtkWidget *w;
+	GtkAdjustment *adj;
 	GtkRcStyle *style;
 	style_edit_struct *se = STYLE_EDIT(data);
 	if(se == NULL)
 	    return;
 
+	if(se->freeze_count > 0)
+	    return;
+
+	se->freeze_count++;
+
+	/* Set the new state */
+	adj = se->state_adj;
+	adj->value = (gfloat)((GtkStateType)i);
+
 	style = se->style;
 	prev_state = se->prev_state;
 	state = StyleEditCurrentState(se);
@@ -177,84 +173,7 @@ static void StyleEditStateChangeCB(
 	    se->prev_state = state;
 	}
 
-	/* Update state entry */
-	w = se->state_entry;
-	switch(state)
-	{
-	  case GTK_STATE_NORMAL:
-	    s = "Normal";
-	    break;
-	  case GTK_STATE_ACTIVE:
-	    s = "Active";
-	    break;
-	  case GTK_STATE_PRELIGHT:
-	    s = "Prelight";
-	    break;
-	  case GTK_STATE_SELECTED:
-	    s = "Selected";
-	    break;
-	  case GTK_STATE_INSENSITIVE:
-	    s = "Insensitive";
-	    break;
-	}
-	gtk_entry_set_text(GTK_ENTRY(w), s);
-	gtk_entry_set_position(GTK_ENTRY(w), -1);
-
-#if 0
-	/* Update button sensitivity */
-	GTK_WIDGET_SET_SENSITIVE(
-	    se->state_previous_btn,
-	    (state > 0) ? TRUE : FALSE
-	)
-	GTK_WIDGET_SET_SENSITIVE(
-	    se->state_next_btn,
-	    (state < (STYLE_EDIT_STATES - 1)) ? TRUE : FALSE
-	)
-#endif
-}
-
-/*
- *      Style Edit previous state callback.
- */
-static void StyleEditPreviousStateCB(
-	GtkWidget *widget, gpointer data
-)
-{
-	GtkAdjustment *adj;
-	style_edit_struct *se = STYLE_EDIT(data);
-	if(se == NULL)
-	    return;
-
-	adj = se->state_adj;
-	gtk_adjustment_set_value(
-	    adj,
-	    MAX(
-		adj->value - adj->step_increment,
-		adj->lower
-	    )
-	);
-}
-
-/*
- *      Style Edit next state callback.
- */
-static void StyleEditNextStateCB(
-	GtkWidget *widget, gpointer data
-)
-{
-	GtkAdjustment *adj;
-	style_edit_struct *se = STYLE_EDIT(data);
-	if(se == NULL)
-	    return;
-
-	adj = se->state_adj;
-	gtk_adjustment_set_value(
-	    adj,
-	    MIN(
-		adj->value + adj->step_increment,
-		adj->upper - adj->page_size
-	    )
-	);
+	se->freeze_count--;
 }
 
 /*
@@ -270,8 +189,12 @@ static void StyleEditResetCB(
 	if(se == NULL)
 	    return;
 
+	if(se->freeze_count > 0)
+	    return;
+
 	toplevel = gtk_widget_get_toplevel(se->toplevel);
 
+	se->freeze_count++;
 	CDialogSetTransientFor(toplevel);
 	response = CDialogGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
@@ -316,58 +239,62 @@ the GTK default style values?\n",
 	if(response != CDIALOG_RESPONSE_YES)
 	    return;
 
+	se->freeze_count--;
+
 	/* Set style values to GTK default style values */
 	StyleEditRestoreDefaultStyle(se);
 
 #if 0
-	/* Set style value to widgets */
+	/* Set the style values to the widgets */
 	StyleEditSetStyleToWidgets(
 	    se,
 	    StyleEditCurrentState(se)
 	);
 #endif
 
-	/* Notify about change */
-	StyleEditAnyValueChangedCB(NULL, se);
+	/* Notify about the change */
+	StyleEditAnyValueChangedCB(widget, data);
 }
 
 /*
- *      Style Edit bg pixmap path browse callback.
+ *	Style Edit bg pixmap path browse callback.
  */
 static void StyleEditBGPixmapPathCB(
 	GtkWidget *widget, gpointer data
 )
 {
-	gboolean status;
-	gchar *startup_path;
-	fb_type_struct **ftype = NULL, *ftype_rtn = NULL;
-	gint total_ftypes = 0;
-	gchar **path_rtn = NULL;
-	gint total_path_rtns = 0;
+	gboolean response;
+	gint nftypes = 0, npaths = 0;
+	gchar *startup_path, **paths_list = NULL;
 	GtkWidget *toplevel;
+	fb_type_struct **ftypes_list = NULL, *ftype_rtn = NULL;
 	style_edit_struct *se = STYLE_EDIT(data);
 	if((se == NULL) || FileBrowserIsQuery())
 	    return;
 
+	if(se->freeze_count > 0)
+	    return;
+
 	toplevel = gtk_widget_get_toplevel(se->toplevel);
 
 	startup_path = g_dirname(
 	    gtk_entry_get_text(GTK_ENTRY(se->bg_pixmap_name_entry))
 	);
 
-	/* Create file types list */
+	/* Create the file types list */
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    ".xpm", "XPM Image"
+	    &ftypes_list, &nftypes,
+	    ".xpm", "X Pixmap"
 	);
 	FileBrowserTypeListNew(
-	    &ftype, &total_ftypes,
-	    "*.*", "All files"
+	    &ftypes_list, &nftypes,
+	    "*.*", "All Files"
 	);
 
-	/* Query user for background image */
+	/* Query the user for an image */
+	se->freeze_count++;
 	FileBrowserSetTransientFor(toplevel);
-	status = FileBrowserGetResponse(
+	response = FileBrowserGetResponse(
 #if defined(PROG_LANGUAGE_SPANISH)
 "La Imagen Selecta",
 "Selecto",
@@ -402,17 +329,18 @@ static void StyleEditBGPixmapPathCB(
 "Cancel",
 #endif
 	    STRISEMPTY(startup_path) ? NULL : startup_path,
-	    ftype, total_ftypes,
-	    &path_rtn, &total_path_rtns,
+	    ftypes_list, nftypes,
+	    &paths_list, &npaths,
 	    &ftype_rtn
 	);
 	FileBrowserSetTransientFor(NULL);
+	se->freeze_count--;
 
 	/* Got user response? */
-	if(status)
+	if(response)
 	{
-	    const gchar *path = (total_path_rtns > 0) ?
-		path_rtn[0] : NULL;
+	    const gchar *path = (npaths > 0) ?
+		paths_list[0] : NULL;
 	    if(!STRISEMPTY(path))
 	    {
 		GtkEntry *entry = GTK_ENTRY(se->bg_pixmap_name_entry);
@@ -428,7 +356,7 @@ static void StyleEditBGPixmapPathCB(
 	}
 
 	/* Delete file types list */
-	FileBrowserDeleteTypeList(ftype, total_ftypes);
+	FileBrowserDeleteTypeList(ftypes_list, nftypes);
 
 	g_free(startup_path);
 }
@@ -450,6 +378,9 @@ static gint StyleEditPreviewEventCB(
 	if((widget == NULL) || (event == NULL) || (se == NULL))
 	    return(status);
 
+	if(se->freeze_count > 0)
+	    return(status);
+
 	switch((gint)event->type)
 	{
 	  case GDK_CONFIGURE:
@@ -522,10 +453,8 @@ static void StyleEditPreviewUpdate(style_edit_struct *se)
 	    return;
 
 	style = gtk_rc_style_copy(se->style);
-	gtk_widget_modify_style(
-	    se->preview_da, style
-	);
-	GTK_RC_STYLE_UNREF(style)
+	gtk_widget_modify_style(se->preview_da, style);
+	GTK_RC_STYLE_UNREF(style);
 
 	StyleEditPreviewDraw(se);
 }
@@ -667,32 +596,60 @@ static void StyleEditPreviewDraw(style_edit_struct *se)
 
 
 /*
- *      Returns the current Style Edit's state.
+ *	Returns the current state.
  */
 GtkStateType StyleEditCurrentState(style_edit_struct *se)
 {
-	GtkAdjustment *adj = se->state_adj;
+	GtkAdjustment *adj;
+
+	if(se == NULL)
+	    return(GTK_STATE_NORMAL);
+
+	adj = se->state_adj;
+
 	return((GtkStateType)CLIP(
 	    adj->value, adj->lower, adj->upper - adj->page_size
 	));
 }
 
+/*
+ *	Switches to the state.
+ */
+void StyleEditSetState(style_edit_struct *se, const GtkStateType state)
+{
+	if(se == NULL)
+	    return;
+
+	se->freeze_count++;
+	PUListBoxSelect(se->state_pulistbox, (gint)state);
+	se->freeze_count--;
+
+	StyleEditStateChangeCB(se->state_pulistbox, (gint)state, se);
+}
+
 
 /*
- *      Creates a new Style Edit.
+ *	Creates a new Style Edit.
  */
 style_edit_struct *StyleEditNew(
 	GtkWidget *parent, const gchar *name
 )
 {
 	const gint border_major = 5;
+	gint i;
 	gpointer entry_rtn, browse_rtn;
 	GtkAdjustment *adj;
-	GtkWidget *w, *parent2, *parent3, *parent4;
+	GtkWidget	*w, *parent2, *parent3, *parent4,
+			*toplevel;
+	pulist_struct *pulist;
+	pulistbox_struct *pulistbox;
 	style_edit_struct *se = STYLE_EDIT(
 	    g_malloc0(sizeof(style_edit_struct))
 	);
+	if(se == NULL)
+	    return(NULL);
 
+	se->toplevel = toplevel = gtk_vbox_new(FALSE, border_major);
 	se->freeze_count = 0;
 	se->prev_state = GTK_STATE_NORMAL;
 	se->state_adj = adj = (GtkAdjustment *)gtk_adjustment_new(
@@ -700,18 +657,12 @@ style_edit_struct *StyleEditNew(
 	    0.0f, (gfloat)(STYLE_EDIT_STATES - 1),
 	    1.0f, 1.0f, 0.0f
 	);
-	gtk_signal_connect(
-	    GTK_OBJECT(adj), "value_changed",
-	    GTK_SIGNAL_FUNC(StyleEditStateChangeCB), se
-	);
-
 	se->style = gtk_rc_style_new();
 
+	se->freeze_count++;
 
-	/* Begin creating widgets */
-
-	/* Toplevel */
-	se->toplevel = w = gtk_vbox_new(FALSE, border_major);
+	/* Toplevel GtkVBox */
+	w = toplevel;
 	if(GTK_IS_BOX(parent))
 	    gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	else if(GTK_IS_CONTAINER(parent))
@@ -719,8 +670,7 @@ style_edit_struct *StyleEditNew(
 	gtk_widget_show(w);
 	parent = w;
 
-
-	/* Hbox for label and state widgets */
+	/* Name & State GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -735,48 +685,28 @@ style_edit_struct *StyleEditNew(
 	    gtk_widget_show(w);
 	}
 
-	/* Hbox for state widgets */
+	/* State GtkHBox */
 	w = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent3 = w;
-	/* Previous Button */
-	se->state_previous_btn = w = gtk_button_new();
-	gtk_signal_connect(
-	    GTK_OBJECT(w), "clicked",
-	    GTK_SIGNAL_FUNC(StyleEditPreviousStateCB), se
+
+	/* State popup list box */
+	se->state_pulistbox = pulistbox = PUListBoxNew(
+	    parent3, 100, -1
 	);
-	gtk_widget_set_usize(w, 20, 20);
-	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);
-	parent4 = w;
-	/* Arrow */
-	w = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(parent4), w);
-	gtk_widget_show(w);
-
-	/* Entry */
-	se->state_entry = w = gtk_entry_new();
-	gtk_widget_set_usize(w, 80, -1);
-	gtk_entry_set_editable(GTK_ENTRY(w), FALSE);
-	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);
-
-	/* Next Button */
-	se->state_next_btn = w = gtk_button_new();
-	gtk_signal_connect(
-	    GTK_OBJECT(w), "clicked",
-	    GTK_SIGNAL_FUNC(StyleEditNextStateCB), se
+	PUListBoxSetChangedCB(
+	    pulistbox,
+	    StyleEditStateChangeCB, se
 	);
-	gtk_widget_set_usize(w, 20, 20);
-	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
-	gtk_widget_show(w);
-	parent4 = w;
-	/* Arrow */
-	w = gtk_arrow_new(GTK_ARROW_RIGHT, GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(parent4), w);
-	gtk_widget_show(w);
-
+	PUListBoxMap(pulistbox);
+	pulist = PUListBoxGetPUList(pulistbox);
+	i = PUListAddItem(pulist, "Normal");
+	i = PUListAddItem(pulist, "Active");
+	i = PUListAddItem(pulist, "Prelight");
+	i = PUListAddItem(pulist, "Selected");
+	i = PUListAddItem(pulist, "Insensitive");
+	PUListBoxSetLinesVisible(pulistbox, i + 1);
 
 	/* Font Name */
 	se->font_name_prompt = w = FSDPromptNewSimple(
@@ -792,19 +722,19 @@ style_edit_struct *StyleEditNew(
 	);
 
 
-	/* Hbox for multiple columns */
+	/* Multiple columns GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	/* Vbox for Column 1 */
+	/* Column 1 GtkVBox */
 	w = gtk_vbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
 	gtk_widget_show(w);
 	parent3 = w;
 
-	/* Hbox for color buttons */
+	/* Color buttons GtkHBox */
 	w = gtk_hbox_new(TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -891,7 +821,7 @@ style_edit_struct *StyleEditNew(
 
 
 
-	/* Hbox for BG Pixmap Path */
+	/* BG Pixmap Path GtkHBox */
 	w = gtk_hbox_new(FALSE, border_major);
 	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
@@ -910,6 +840,7 @@ style_edit_struct *StyleEditNew(
 	    GTK_OBJECT(w), "changed",
 	    GTK_SIGNAL_FUNC(StyleEditAnyValueChangedCB), se
 	);
+	GUIEditableEndowPopupMenu(w, 0);
 
 
 	/* Preview frame for column 2 */
@@ -989,10 +920,10 @@ style_edit_struct *StyleEditNew(
 	gtk_widget_show(w);
 
 
+	se->freeze_count--;
 
 
-
-	/* Update current state */
+	/* Update the current state */
 	gtk_signal_emit_by_name(
 	    GTK_OBJECT(adj), "value_changed"
 	);
@@ -1002,38 +933,11 @@ style_edit_struct *StyleEditNew(
 }
 
 /*
- *      Deletes the Style Edit.
- */
-void StyleEditDelete(style_edit_struct *se)
-{
-	if(se == NULL)
-	    return;
-
-	gtk_widget_destroy(se->state_previous_btn);
-	gtk_widget_destroy(se->state_entry);
-	gtk_widget_destroy(se->state_next_btn);
-	gtk_widget_destroy(se->font_name_prompt);
-	gtk_widget_destroy(se->fg_color_btn);
-	gtk_widget_destroy(se->bg_color_btn);
-	gtk_widget_destroy(se->text_color_btn);
-	gtk_widget_destroy(se->base_color_btn);
-	gtk_widget_destroy(se->bg_pixmap_name_entry);
-	gtk_widget_destroy(se->preview_da);
-
-	gtk_widget_destroy(se->toplevel);
-	gtk_object_unref(GTK_OBJECT(se->state_adj));
-
-	GTK_RC_STYLE_UNREF(se->style)
-
-	g_free(se);
-}
-
-/*
- *      Sets values on the Style Edit's widgets to its style
- *      structure.
+ *	Sets values on the Style Edit's widgets to its style
+ *	structure.
  */
 static void StyleEditSetWidgetsToStyle(
-	style_edit_struct *se, GtkStateType state
+	style_edit_struct *se, const GtkStateType state
 )
 {
 	csd_color_struct csd_c;
@@ -1081,7 +985,7 @@ static void StyleEditSetWidgetsToStyle(
  *      it to the widgets.
  */
 static void StyleEditSetStyleToWidgets(
-	style_edit_struct *se, GtkStateType state
+	style_edit_struct *se, const GtkStateType state
 )
 {
 	const gchar *s;
@@ -1286,3 +1190,23 @@ void StyleEditSetChangedCB(
 	se->changed_client_data = changed_client_data;
 }
 
+/*
+ *	Deletes the Style Edit.
+ */
+void StyleEditDelete(style_edit_struct *se)
+{
+	if(se == NULL)
+	    return;
+
+	se->freeze_count++;
+
+	PUListBoxDelete(se->state_pulistbox);
+	gtk_widget_destroy(se->toplevel);
+	gtk_object_unref(GTK_OBJECT(se->state_adj));
+
+	GTK_RC_STYLE_UNREF(se->style);
+
+	se->freeze_count--;
+
+	g_free(se);
+}
