@@ -437,9 +437,11 @@ int main(int argc, char *argv[])
 
 	/* Thumbs list */
 	tlist = TListNew(
-	    FALSE, 100, 90, 2,
-	    NULL,	/* Client data */
-	    select_cb, unselect_cb
+	    GTK_ORIENTATION_VERTICAL,
+	    100, 90,			/* Width, Height */
+	    2,				/* Border */
+	    select_cb, NULL,
+	    unselect_cb, NULL
 	);
 	tlist->selection_mode = GTK_SELECTION_EXTENDED;
 	w = tlist->toplevel;
