@@ -10,7 +10,6 @@
 #include "../toolbar.h"
 
 #include "../edv_types.h"
-#include "../edv_id.h"
 #include "../edv_utils.h"
 #include "../lib/endeavour2.h"
 
@@ -560,7 +559,7 @@ void ZipToolRefreshDevice(
 		g_free(ss_mb);
 
 		zt->capacity_piechart = pc = PieChartNew(
-		    adj, &c, 110, 70,
+		    110, 70, adj, &c,
 		    NULL, buf,
 #ifdef PROG_LANGUAGE_ENGLISH
 "Free"
@@ -760,8 +759,8 @@ void ZipToolRefreshDevice(
 		prefix_label_width = 70;
 
 		/* Ownership Banner */
-		GDK_COLOR_SET_COEFF(&color_fg, 1.0f, 1.0f, 1.0f)   
-		GDK_COLOR_SET_COEFF(&color_bg, 0.0f, 0.0f, 0.0f)
+		GDK_COLOR_SET_COEFF(&color_fg, 1.0f, 1.0f, 1.0f);
+		GDK_COLOR_SET_COEFF(&color_bg, 0.0f, 0.0f, 0.0f);
 		w = GUIBannerCreate(
 		    "Ownership",
 "-adobe-helvetica-bold-r-normal-*-14-*-*-*-*-*-iso8859-1",
@@ -773,25 +772,25 @@ void ZipToolRefreshDevice(
 		gtk_widget_show(w);
 
 		/* Owner Label */
-		buf = STRDUP(
-		    stat_status ? "" :
-		    EDVUIDGetNameFromUID(
-			zt->uid, zt->total_uids,
-			stat_buf.st_uid, NULL
-		    )
-		);
-		ADD_LABEL("Owner:", buf)
+		if(stat_status)
+		    buf = STRDUP("");
+		else
+		    buf = EDVUIDGetNameFromUID(
+			zt->ctx,
+			(gint)stat_buf.st_uid
+		    );
+		ADD_LABEL("Owner:", buf);
 		g_free(buf);
 
 		/* Group Label */
-		buf = STRDUP(
-		    stat_status ? "" :
-		    EDVGIDGetNameFromGID(
-			zt->gid, zt->total_gids,
-			stat_buf.st_gid, NULL
-		    )
-		);
-		ADD_LABEL("Group:", buf)
+		if(stat_status)
+		    buf = STRDUP("");
+		else
+		    buf = EDVGIDGetNameFromGID(
+			zt->ctx,
+			(gint)stat_buf.st_gid
+		    );
+		ADD_LABEL("Group:", buf);
 		g_free(buf);
 
 		/* Permissions Label */
@@ -892,36 +891,33 @@ zip_tool_struct *ZipToolNew(edv_context_struct *ctx)
 {
 	gint border_minor = 2;
 	GdkWindow *window;
+	GList *tb_items_list;
 	GtkWidget	*w, *parent, *parent2, *parent3,
-			*main_vbox;
+			*toplevel, *main_vbox;
 	GtkAccelGroup *accelgrp;
-	toolbar_struct *toolbar;
-	toolbar_item_struct **tb_item;
-	gint total_tb_items;
+	toolbar_struct *tb;
+	toolbar_item_struct *tb_item;
 	zip_tool_struct *zt = (zip_tool_struct *)g_malloc0(
 	    sizeof(zip_tool_struct)
 	);
 	if(zt == NULL)
 	    return(NULL);
 
-
-	/* Reset values */
+	zt->toplevel = toplevel = gtk_window_new(GTK_WINDOW_DIALOG);
 	zt->accelgrp = accelgrp = gtk_accel_group_new();
-	zt->ctx = ctx;
-	zt->busy_count = 0;
 	zt->freeze_count = 0;
+	zt->busy_count = 0;
+	zt->ctx = ctx;
 	zt->opid = ZipToolOPIDListNew(zt);
-	zt->uid = EDVUIDGetSystem(&zt->total_uids);
-	zt->gid = EDVGIDGetSystem(&zt->total_gids);
 	zt->device = NULL;
 	zt->lock_state = ZIP_TOOL_LOCK_STATE_UNLOCKED;
-
-	/* Load cursors */
 	zt->busy_cur = gdk_cursor_new(GDK_WATCH);
 	zt->text_cur = gdk_cursor_new(GDK_XTERM);
 
-	/* Toplevel */
-	zt->toplevel = w = gtk_window_new(GTK_WINDOW_DIALOG);
+	zt->freeze_count++;
+
+	/* Toplevel GtkWindow */
+	w = toplevel;
 	gtk_widget_set_usize(w, ZIP_TOOL_WIDTH, ZIP_TOOL_HEIGHT);
 	gtk_window_set_title(GTK_WINDOW(w), PROG_NAME);
 	gtk_window_set_wmclass(                        
@@ -961,68 +957,71 @@ zip_tool_struct *ZipToolNew(edv_context_struct *ctx)
 	parent = w;
 
 
-	/* Create Tool Bar Items list */
-	total_tb_items = 0;
-	tb_item = NULL;
+	/* Create the Tool Bar Items list */
+	tb_items_list = NULL;
 #define DO_APPEND_BUTTON(_id_)	{			\
  zip_tool_opid_struct *opid = ZipToolOPIDListFind(	\
   zt->opid, (_id_)					\
  );							\
  if(opid != NULL) {					\
-  ToolBarItemListAppend(				\
-   &tb_item, &total_tb_items,				\
-   ToolBarItemNew(					\
-    TOOLBAR_ITEM_BUTTON, opid->button_name,		\
-    opid->icon_data, opid->tooltip, opid->id,		\
-    ZipToolOPIDCB, opid,				\
-    ZipToolOPIDEnterCB, opid,				\
-    ZipToolOPIDLeaveCB, opid				\
-   )							\
+  tb_item = ToolBarItemNew(				\
+   TOOLBAR_ITEM_BUTTON,					\
+   NULL,						\
+   opid->button_name,					\
+   opid->icon_data, opid->tooltip, opid->id,		\
+   ZipToolOPIDCB, opid,					\
+   ZipToolOPIDEnterCB, opid,				\
+   ZipToolOPIDLeaveCB, opid				\
   );							\
+  tb_items_list = g_list_append(tb_items_list, tb_item);\
 } }
 #define DO_APPEND_SEPARATOR	{			\
-  ToolBarItemListAppend(				\
-   &tb_item, &total_tb_items,				\
-   ToolBarItemNew(					\
-    TOOLBAR_ITEM_SEPARATOR, NULL,			\
-    NULL, NULL,						\
-    0,							\
-    NULL, NULL, NULL, NULL, NULL, NULL			\
-   )							\
-  );							\
+ tb_item = ToolBarItemNew(				\
+  TOOLBAR_ITEM_SEPARATOR,				\
+  NULL,							\
+  NULL,							\
+  NULL, NULL,						\
+  0,							\
+  NULL, NULL, NULL, NULL, NULL, NULL			\
+ );							\
+ tb_items_list = g_list_append(tb_items_list, tb_item);	\
 }
-
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_MOUNT)
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_UNMOUNT)
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_MOUNT);
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_UNMOUNT);
 	DO_APPEND_SEPARATOR
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_SPIN_DOWN)
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_EJECT)
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_PASSWORD)
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_SPIN_DOWN);
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_EJECT);
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_PASSWORD);
 	DO_APPEND_SEPARATOR
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_FSCK)
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_FSCK);
 	DO_APPEND_SEPARATOR
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_REFRESH)
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_REFRESH);
 	DO_APPEND_SEPARATOR
-	DO_APPEND_BUTTON(ZIP_TOOL_OPID_CLOSE)
+	DO_APPEND_BUTTON(ZIP_TOOL_OPID_CLOSE);
 
 #undef DO_APPEND_BUTTON
 #undef DO_APPEND_SEPARATOR
 
-	/* Create tool bar */
+	/* Create the Tool Bar */
 	zt->tool_bar_handle = w = gtk_handle_box_new();
 	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
 	gtk_widget_show(w);
 	parent2 = w;
 
-	zt->tool_bar = toolbar = ToolBarNew(
-	    tb_item, total_tb_items, parent2,
+	zt->tool_bar = tb = ToolBarNew(
+	    parent2,
+	    tb_items_list,
 	    TOOLBAR_DISPLAY_PICTURES_AND_TEXT,
 	    TOOLBAR_RELIEF_NONE, FALSE
 	);
-	ToolBarMap(toolbar);
+	ToolBarMap(tb);
 
-	/* Delete Tool Bar Items list */
-	ToolBarItemListDeleteAll(&tb_item, &total_tb_items);
+	/* Delete the Tool Bar Items list */
+	if(tb_items_list != NULL)
+	{
+	    g_list_foreach(tb_items_list, (GFunc)ToolBarItemDelete, NULL);
+	    g_list_free(tb_items_list);
+	}
 
 
 	/* Event box to hold display parent */
@@ -1178,6 +1177,8 @@ widget \"*ziptool-base\" style \"ziptool-base-style\"\n\
 
 	ZipToolUpdate(zt);
 
+	zt->freeze_count--;
+
 	return(zt);
 }
 
@@ -1318,8 +1319,10 @@ void ZipToolStatusMessage(
  */
 gboolean ZipToolIsMapped(zip_tool_struct *zt)
 {
-	GtkWidget *w = (zt != NULL) ? zt->toplevel : NULL;
-	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
+	if(zt == NULL)
+	    return(FALSE);
+
+	return(GTK_WIDGET_MAPPED(zt->toplevel));
 }
 
 /*
@@ -1327,11 +1330,10 @@ gboolean ZipToolIsMapped(zip_tool_struct *zt)
  */
 void ZipToolMap(zip_tool_struct *zt)
 {
-	GtkWidget *w = (zt != NULL) ? zt->toplevel : NULL;
-	if(w == NULL)
+	if(zt == NULL)
 	    return;
 
-	gtk_widget_show_raise(w);
+	gtk_widget_show_raise(zt->toplevel);
 }
 
 /*
@@ -1339,11 +1341,10 @@ void ZipToolMap(zip_tool_struct *zt)
  */
 void ZipToolUnmap(zip_tool_struct *zt)
 {
-	GtkWidget *w = (zt != NULL) ? zt->toplevel : NULL;
-	if(w == NULL)
+	if(zt == NULL)
 	    return;
 
-	gtk_widget_hide(w);
+	gtk_widget_hide(zt->toplevel);
 }
 
 /*
@@ -1351,12 +1352,14 @@ void ZipToolUnmap(zip_tool_struct *zt)
  */
 void ZipToolDelete(zip_tool_struct *zt)
 {
-	gint i;
-
 	if(zt == NULL)
 	    return;
 
-	GTK_WIDGET_DESTROY(zt->menu)
+	ZipToolUnmap(zt);
+
+	zt->freeze_count++;
+
+	GTK_WIDGET_DESTROY(zt->menu);
 
 	ToolBarDelete(zt->tool_bar);
 	GTK_WIDGET_DESTROY(zt->tool_bar_handle)
@@ -1364,32 +1367,27 @@ void ZipToolDelete(zip_tool_struct *zt)
 	GTK_WIDGET_DESTROY(zt->status_label)
 
 	PieChartDelete(zt->capacity_piechart);
-	GTK_WIDGET_DESTROY(zt->display_mount_path_btn)
-	GTK_WIDGET_DESTROY(zt->display_mount_btn)
-	GTK_WIDGET_DESTROY(zt->display_lock_btn)
+	GTK_WIDGET_DESTROY(zt->display_mount_path_btn);
+	GTK_WIDGET_DESTROY(zt->display_mount_btn);
+	GTK_WIDGET_DESTROY(zt->display_lock_btn);
 
-	GTK_WIDGET_DESTROY(zt->display_child)
-	GTK_WIDGET_DESTROY(zt->display_parent)
-	GTK_WIDGET_DESTROY(zt->display_event_box)
+	GTK_WIDGET_DESTROY(zt->display_child);
+	GTK_WIDGET_DESTROY(zt->display_parent);
+	GTK_WIDGET_DESTROY(zt->display_event_box);
 
-	GTK_WIDGET_DESTROY(zt->main_vbox)
-	GTK_WIDGET_DESTROY(zt->toplevel)
+	GTK_WIDGET_DESTROY(zt->main_vbox);
+	GTK_WIDGET_DESTROY(zt->toplevel);
 
-	GTK_ACCEL_GROUP_UNREF(zt->accelgrp)
+	GTK_ACCEL_GROUP_UNREF(zt->accelgrp);
 
 	ZipToolOPIDListDelete(zt->opid);
 
-	GDK_CURSOR_DESTROY(zt->busy_cur)
-	GDK_CURSOR_DESTROY(zt->text_cur)
+	GDK_CURSOR_DESTROY(zt->busy_cur);
+	GDK_CURSOR_DESTROY(zt->text_cur);
 
 	EDVDeviceDelete(zt->device);
 
-	for(i = 0; i < zt->total_uids; i++)
-	    EDVUIDDelete(zt->uid[i]);
-	g_free(zt->uid);
-	for(i = 0; i < zt->total_gids; i++)
-	    EDVGIDDelete(zt->gid[i]);
-	g_free(zt->gid);
+	zt->freeze_count--;
 
 	g_free(zt);
 }
